package org.modelio.modelfragment.command;

import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.ui.PlatformUI;
import org.modelio.modelfragment.api.IModelFragmentStereotypes;
import org.modelio.modelfragment.command.ihm.EditFragmentDataModel;
import org.modelio.modelfragment.command.ihm.EditFragmentDialog;
import org.modelio.modelfragment.model.DeployedFragment;
import org.modelio.modelfragment.model.ModelUtils;

import com.modeliosoft.modelio.api.mdac.IMdac;
import com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.ITransaction;
import com.modeliosoft.modelio.api.model.InvalidTransactionException;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;

public class EditModelFragmentCommand extends DefaultMdacContextualCommand {

	public boolean accept(ObList<IElement> selectedElements, IMdac mdac) {
		// Check that there is only one selected element
		return (selectedElements.size() == 1) 
				&& (selectedElements.get(0) instanceof IArtifact)
				&& ((IArtifact)selectedElements.get(0)).isStereotyped(IModelFragmentStereotypes.MODELFRAGMENT);
	}


	public void actionPerformed(ObList<IElement> selectedElements, IMdac mdac) {

		IModelingSession session =  Modelio.getInstance().getModelingSession();
		ITransaction transaction = session.createTransaction("Edit Model fragment definition");

		try{
//			List<DeployedFragment> fragments = ModelUtils.getDeployedFragment();

			//			DefinitionDialog dialog = new DefinitionDialog(new Shell(), 
			//					new FragmentDefinition((IArtifact) selectedElements.get(0)), fragments);

			EditFragmentDataModel model = new EditFragmentDataModel((IArtifact) selectedElements.get(0));
			EditFragmentDialog dialog = new EditFragmentDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), model);
			int code = dialog.open();

			if (code == IDialogConstants.OK_ID) {
				model.save();
			} 


			session.commit(transaction);
			transaction = null;
		} catch (InvalidTransactionException e) {
			// Transaction already rollbacked
			transaction = null;
		} finally {
			if (transaction != null)
				session.rollback(transaction);
		}

	}


}
