package org.modelio.modelfragment.command;

import java.io.File;
import java.io.IOException;
import java.util.UUID;
import java.util.zip.ZipException;

import org.galaxy.client.api.GalaxyService;
import org.galaxy.client.service.fragmentmanagement.ModelFragmentData;
import org.galaxy.client.service.fragmentmanagement.ressource.ModelFragmentRessource;
import org.modelio.modelfragment.api.IModelFragmentStereotypes;
import org.modelio.modelfragment.model.FragmentDefinition;

import com.modeliosoft.modelio.api.exchange.XmiException;
import com.modeliosoft.modelio.api.exchange.XmiExportConfiguration;
import com.modeliosoft.modelio.api.exchange.XmiExportConfiguration.VersionExport;
import com.modeliosoft.modelio.api.mdac.IMdac;
import com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;

/**
 * Implementation of the IMdacContextualCommand interface. <br>
 * The MDAC contextual commands are displayed in the contextual menu and in the specific toolbar of each mdac property
 * page. <br>
 * The developer may inherit the DefaultMdacContextualCommand class which contains a default standard contextual command
 * implementation.
 * 
 */
public class PublishModelFragmentCommand extends DefaultMdacContextualCommand {

	/**
	 * Constructor.
	 */
	public PublishModelFragmentCommand() {
		super();
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand#accept(com.modeliosoft.modelio.api.utils.ObList,
	 *      com.modeliosoft.modelio.api.mdac.IMdac)
	 */
	public boolean accept(ObList<IElement> selectedElements, IMdac mdac) {
		// Check that there is only one selected element
		return (selectedElements.size() == 1) 
				&& (selectedElements.get(0) instanceof IArtifact)
				&& ((IArtifact)selectedElements.get(0)).isStereotyped(IModelFragmentStereotypes.MODELFRAGMENT);
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand#actionPerformed(com.modeliosoft.modelio.api.utils.ObList,
	 *      com.modeliosoft.modelio.api.mdac.IMdac)
	 */
	public void actionPerformed(ObList<IElement> selectedElements, IMdac mdac) {


		ModelFragmentRessource resource = new ModelFragmentRessource();
		String url = mdac.getConfiguration().getParameterValue("ServerURL");
		GalaxyService service = new GalaxyService(url);
		FragmentDefinition def = new FragmentDefinition((IArtifact) selectedElements.get(0));
		resource.setName(def.getName());
		resource.setVersion(def.getVersion());
		resource.setDescription(def.getDescription());
		

		try {
			
//			ModelFragmentData[] fragments = service.getModelFragmentManagement().findAllModelFragments();

			resource.setGalaxyId(UUID.randomUUID().toString());
			File xmiFile = new File(mdac.getConfiguration()
					.getProjectSpacePath() + "/model.xmi");

			XmiExportConfiguration export = new XmiExportConfiguration();
			export.setXmiFile(xmiFile);
			export.setEntryPoint(def.getManifested().get(0));
			export.setVersionExport(VersionExport.EMF300);
	
			Modelio.getInstance().getExchangeService().exportXmiFile(export, null);
			resource.setXmiFile(xmiFile);
			
			File fragment = service.getModelFragmentManagement()
					.packageFragment(resource,
							new File(mdac.getConfiguration()
									.getProjectSpacePath() + "/fragments"));

			service.getModelFragmentManagement().publish(fragment);


		} catch (ZipException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (XmiException e) {
			e.printStackTrace();
		}

	}

	private void generateXMIFile(IElement iElement, File xmiFile) {
		XmiExportConfiguration exportConfig = new XmiExportConfiguration();
		exportConfig.setXmiFile(xmiFile);
		exportConfig.setVersionExport(VersionExport.EMF300);

		try {
			Modelio.getInstance().getExchangeService().exportXmiFile(exportConfig, null);
		} catch (XmiException e) {
			e.printStackTrace();
		}

	}

}
