package org.modelio.modelfragment.command;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.eclipse.swt.widgets.Shell;
import org.galaxy.client.api.GalaxyService;
import org.galaxy.client.service.fragmentmanagement.ModelFragmentData;
import org.modelio.modelfragment.command.ihm.RetrieveDialog;
import org.modelio.modelfragment.utils.ReadOnlyMetamodelVisitor;
import org.modelio.modelfragment.utils.ReadWriteMetamodelVisitor;

import com.modeliosoft.modelio.api.exchange.XmiException;
import com.modeliosoft.modelio.api.exchange.XmiImportConfiguration;
import com.modeliosoft.modelio.api.mdac.IMdac;
import com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.ITransaction;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;

/**
 * Implementation of the IMdacContextualCommand interface. <br>
 * The MDAC contextual commands are displayed in the contextual menu and in the specific toolbar of each mdac property
 * page. <br>
 * The developer may inherit the DefaultMdacContextualCommand class which contains a default standard contextual command
 * implementation.
 * 
 */
public class RetrieveModelFragmentCommand extends DefaultMdacContextualCommand {

	static final int BUFFER = 2048;

	/**
	 * Constructor.
	 */
	public RetrieveModelFragmentCommand() {
		super();
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand#accept(com.modeliosoft.modelio.api.utils.ObList,
	 *      com.modeliosoft.modelio.api.mdac.IMdac)
	 */
	public boolean accept(ObList<IElement> selectedElements, IMdac mdac) {
		return selectedElements.size() == 1 
				&& selectedElements.get(0).equals(Modelio.getInstance().getModelingSession().getModel().getRoot());
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.commands.DefaultMdacContextualCommand#actionPerformed(com.modeliosoft.modelio.api.utils.ObList,
	 *      com.modeliosoft.modelio.api.mdac.IMdac)
	 */
	public void actionPerformed(ObList<IElement> selectedElements, IMdac mdac) {
		String url = mdac.getConfiguration().getParameterValue("ServerURL");
		GalaxyService service = new GalaxyService(url);

		try {
			ModelFragmentData[] fragments = service.getModelFragmentManagement().findAllModelFragments();

			RetrieveDialog dialog = new RetrieveDialog(new Shell(), fragments);

			if (dialog.open() == 0) {
				ModelFragmentData fragment = dialog.getSelectedFragment();

				// create root
				IModelingSession session = Modelio.getInstance().getModelingSession();
				IUmlModel model = session.getModel();
				IPackage pack = null;
				IPackage root = model.getRoot();
				boolean exist = false;

				for (IModelElement existingPack : root.getOwnedElement().select(IPackage.class)){
					if (existingPack.getName().equals(fragment.getName())){
						pack = (IPackage) existingPack;
						exist = true;
						break;
					}
				}

				if (!exist){
					ITransaction tr = session.createTransaction("create Owner ");
					pack = model.createPackage();
					pack.setName(fragment.getName());
					pack.setOwner(model.getRoot());
					session.commit(tr);
				}


				setReadWrite(pack);
				List<File> files = service.getModelFragmentManagement()
						.retrieveAll(fragment.getName(),
								fragment.getVersion(),
								new File(mdac.getConfiguration().getProjectSpacePath() +
										"/fragments"));
				importFragments(files, pack);

				setReadOnly(pack);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void setReadOnly(IPackage pack) {
		pack.accept(new ReadOnlyMetamodelVisitor());
		
	}

	private void importFragments(List<File> files, IPackage owner) {



		for (File file : files){
			unzip(file);

			XmiImportConfiguration importConfig = new XmiImportConfiguration(new File(file.getParentFile().getAbsolutePath() 
					+ File.separator + "model.xmi"));

			importConfig.setOwner(owner);

			try {
				Modelio.getInstance().getExchangeService().importXmiFile(importConfig, null);
			} catch (XmiException e) {
				e.printStackTrace();
			}
		}


	}

	private void setReadWrite(IPackage owner) {
		owner.accept(new ReadWriteMetamodelVisitor());
	}


	private void unzip(File file) {

		try {
			ZipFile zipFile = new ZipFile(file.getAbsolutePath());
			Enumeration entries = zipFile.entries();
			while(entries.hasMoreElements()) {
				ZipEntry entry = ((ZipEntry) entries.nextElement());
				if(entry.isDirectory()) {
					// Assume directories are stored parents first then children.
					// This is not robust, just for demonstration purposes.
					(new File(entry.getName())).mkdir();
					continue;
				}
				System.err.println("Extracting file: " + entry.getName());
				copyInputStream(zipFile.getInputStream(entry),
						new BufferedOutputStream(new FileOutputStream(file.getParent() + File.separator + entry.getName())));
			}
			zipFile.close();
		} catch (IOException ioe) {
			System.err.println("Unhandled exception:");
			ioe.printStackTrace();
			return;
		}

	}

	public static final void copyInputStream(InputStream in, OutputStream out)
			throws IOException
			{

		byte[] buffer = new byte[1024];
		int len;
		while((len = in.read(buffer)) >= 0)
			out.write(buffer, 0, len);
		in.close();
		out.close();
			}
}
