package org.modelio.modelfragment.command.ihm;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.PluginTransfer;
import org.modelio.modelfragment.model.DeployedFragment;
import org.modelio.modelfragment.model.FragmentDefinition;
import org.modelio.modelfragment.model.ModelUtils;
import org.modelio.modelfragment.utils.I18nMessageService;
import org.modelio.modelfragment.utils.SWTResourceManager;

import com.modeliosoft.modelio.api.ui.ModelioDialog;
import com.modeliosoft.modelio.api.ui.UIColor;
import com.modeliosoft.modelio.api.ui.dnd.ModelElementTransfer;

public class DefinitionDialog extends ModelioDialog {

    private Button ok;
    private Text nameText;
    private Text versionText;
    private Text descriptionText;
    private TableViewer manifestationList = null;
    private TableViewer dependencyList = null;
    private java.util.List<DeployedFragment>  datas;
    
    
    private FragmentDefinition resource;
    
    
    public DefinitionDialog(Shell shell, FragmentDefinition resource, java.util.List<DeployedFragment> fragments) {
        super(shell);
        setShellStyle(SWT.DIALOG_TRIM | SWT.ON_TOP | getDefaultOrientation());
        this.resource = resource;
    }
    
    public FragmentDefinition getResource(){
        return resource;
    }
    
    @Override
    protected void okPressed() {
        this.resource.setName(nameText.getText());
        this.resource.setVersion(versionText.getText());
        this.resource.setDescription(descriptionText.getText());
        
        for(DeployedFragment deplFrag : ModelUtils.getDeployedFragment()) 
//        for(int idx : this.dependencyList.getSelectionIndices()){
//            this.resource.addDependency(this.datas[idx].getName(), this.datas[idx].getVersion());
//        }
        
        super.okPressed();
    }

    @Override
    public void addButtonsInButtonBar(Composite parent) {
    	this.ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL, false);
    }

    @Override
    public Control createContentArea(Composite parent) {
        
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        
        Label nameLabel = new Label(composite, SWT.NONE);
        GridData gd_nameLabel = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
        gd_nameLabel.widthHint = 72;
        nameLabel.setLayoutData(gd_nameLabel);
        nameLabel.setText("Name");
        
        this.nameText = new Text(composite, SWT.BORDER);
        this.nameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        this.nameText.setBackground(SWTResourceManager.getColor(225, 225, 225));
        Label versionLabel = new Label(composite, SWT.NONE);
        GridData gd_versionLabel = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
        gd_versionLabel.widthHint = 72;
        gd_versionLabel.heightHint = 15;
        versionLabel.setLayoutData(gd_versionLabel);
        versionLabel.setText("Version");
        
        this.versionText = new Text(composite, SWT.BORDER);
        this.versionText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        this.versionText.setBackground(SWTResourceManager.getColor(225, 225, 225));
        Label descriptionLabel = new Label(composite, SWT.NONE);
        GridData gd_descriptionLabel = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
        gd_descriptionLabel.heightHint = 131;
        gd_descriptionLabel.widthHint = 73;
        descriptionLabel.setLayoutData(gd_descriptionLabel);
        descriptionLabel.setText("Description");
        
        this.descriptionText = new Text(composite, SWT.BORDER | SWT.MULTI);
        GridData gd_descriptionText = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
        gd_descriptionText.heightHint = 130;
        gd_descriptionText.widthHint = 350;
        this.descriptionText.setBackground(SWTResourceManager.getColor(225, 225, 225));
        this.descriptionText.setLayoutData(gd_descriptionText);
        
        
        Label lblDependencys = new Label(composite, SWT.NONE);
        GridData gd_lblDependencys = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_lblDependencys.heightHint = 95;
        lblDependencys.setLayoutData(gd_lblDependencys);
        lblDependencys.setText("Dependency");
        
//        this.dependencyList = new List(composite, SWT.BORDER | SWT.V_SCROLL| SWT.MULTI);
        GridData gd_dependencyList = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
        gd_dependencyList.heightHint = 94;
        gd_dependencyList.widthHint = 316;
//        this.dependencyList.setLayoutData(gd_dependencyList);
        
        if(this.datas != null){
            for(DeployedFragment data : this.datas){
            	this.dependencyList.add(data.getName() + "-" + data.getVersion()); 
            }
        }
              
        return null;
    }
    
    
    private void initManifestationsListViewer(Composite area, boolean isModelComponentEditable) {
        this.manifestationList = new TableViewer(area, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
        
        this.manifestationList.setContentProvider(new EditManifestationsContentProvider());
        this.manifestationList.setLabelProvider(new EditManifestationsLabelProvider());
        
        if (isModelComponentEditable) {
            int operations = DND.DROP_MOVE;
            Transfer[] types = new Transfer[] { ModelElementTransfer.getInstance(),
                    PluginTransfer.getInstance() };
            DropTarget target = new DropTarget(this.manifestationList.getTable(), operations);
            target.setTransfer(types);
//            EditManifestationsDropListener dropListener = new EditManifestationsDropListener(this,
//                                                                                             this.controller);
//            target.addDropListener(dropListener);
        
            this.manifestationList.getTable().addKeyListener(new KeyListener() {
        
                @Override
                public void keyPressed(KeyEvent e) {
                }
        
                @Override
                public void keyReleased(KeyEvent e) {
                    if (e.keyCode == SWT.DEL) {
                        Table table = (Table) e.getSource();
                        TableItem[] items = table.getSelection();
                        for (TableItem item : items) {
                            EditManifestedAdapter adapter = (EditManifestedAdapter) item.getData();
//                            EditRamcDialog.this.controller.removeExportedAdapter(adapter);
                        }
                    }
//                    EditRamcDialog.this.refresh();
                }
        
            });
        } else {
            this.manifestationList.getTable().setForeground(UIColor.EDITOR_ROTEXT_FG);
            this.manifestationList.getTable().setBackground(UIColor.TEXT_READONLY_BG);
        }
        
        final GridData gd_ramcManifestationsList = new GridData(SWT.FILL, SWT.FILL, true, false);
        gd_ramcManifestationsList.heightHint = 100;
        this.manifestationList.getTable().setLayoutData(gd_ramcManifestationsList);
    }

    private void createManifestationsField(Composite area, boolean isModelComponentEditable) {
        // Ramc manifestations field:
        //---------------------------
        Label manifestationsLabel = new Label(area, SWT.NONE);
        manifestationsLabel.setText(I18nMessageService.getString("EditRamcDialog.RamcManifestationLabel"));
        
        this.initManifestationsListViewer(area, isModelComponentEditable);
       
        final GridData gd_ramcManifestationsLabel = new GridData(SWT.LEFT, SWT.UP, false, false, 1, 1);
        manifestationsLabel.setLayoutData(gd_ramcManifestationsLabel);
        
        final GridData gd_targetLabel = new GridData(SWT.LEFT, SWT.UP, false, false, 1, 1);
        gd_targetLabel.heightHint = 21;
        gd_targetLabel.widthHint = 21;
//        targetLabel.setLayoutData(gd_targetLabel);
    }
    
    @Override
    public void init() {
        Shell shell = getShell();
        shell.setSize(500, 450);
        int x = (this.getShell().getSize().x / 2) - 250;
        int y = (this.getShell().getSize().y / 2) - 100;
        shell.setLocation(x, y);
        shell.setMinimumSize(400, 450);
        shell.setText("Modelio");
        setTitle("Model Fragment Edition");
        setMessage("Edit a Model Fragment");
       
    }

}
