/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package org.modelio.modelfragment.command.ihm;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.modelio.modelfragment.api.IModelFragmentStereotypes;
import org.modelio.modelfragment.utils.I18nMessageService;

import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IElementImport;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.ui.dnd.ModelElementTransfer;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.ObRef;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

/**
 * Edit dependencies drop listener.
 */
@objid ("32c59bb8-fea8-11dd-8b31-0014222a9f79")
public class EditDependenciesDropListener implements DropTargetListener {
    @objid ("bcc886ae-ff45-11dd-95ab-0014222a9f79")
    private EditFragmentDialog dialog;

    @objid ("bcc886b6-ff45-11dd-95ab-0014222a9f79")
    private EditFragmentController controler = null;

    @objid ("b1cc5480-0a3b-11de-8d83-0014222a9f79")
    private IArtifact currentRamc = null;


    @objid ("bc1353f4-ff45-11dd-95ab-0014222a9f79")
    public EditDependenciesDropListener(EditFragmentDialog dialog, EditFragmentController controler, IArtifact currentRamc) {
        this.controler = controler;
        this.dialog = dialog;
        this.currentRamc = currentRamc;
    }

    @objid ("ca4eeacb-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void dragEnter(DropTargetEvent event) {
        ModelElementTransfer elementTransfer = ModelElementTransfer.getInstance();
        
        // We take ModelElementTransfert data
        for (int i = 0; i < event.dataTypes.length; i++) {
            if (elementTransfer.isSupportedType(event.dataTypes[i])) {
                event.currentDataType = event.dataTypes[i];
                break;
            }
        }
        
        if (elementTransfer.isSupportedType(event.currentDataType)) {
            boolean accept = true;
            // NOTE: on unsupported platforms this will return null
            ObRef[] refs = (ObRef[]) elementTransfer.nativeToJava(event.currentDataType);
            if (refs != null) {
                IElement[] elements = new IElement[refs.length];
                for (int i = 0; i < elements.length ; i++) {
                    elements[i] = Modelio.getInstance().getModelingSession().findByRef(refs[i]);
                }
        
                // Supported platform, check the type of elements
                for (IElement element : elements) {
                    if (element instanceof IArtifact) {
                        IArtifact artifact = (IArtifact) element;
                        if (artifact.equals(this.currentRamc)) {
                            accept = false;
                        }
                        if (!artifact.isStereotyped("ModelComponentArchive")) {
                            accept = false;
                        }
                    } else {
                        accept = false;
                    }
                }
            } else {
                // On linux, event.currentDataType is not filled, we have to validate the drag and test the condition in the drop
                accept = true;
            }
        
            if (!accept) {
                event.detail = DND.DROP_NONE;
            }
        }
    }

    @objid ("ca514d25-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void dragLeave(DropTargetEvent event) {
    }

    @objid ("ca53af7e-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void dragOperationChanged(DropTargetEvent event) {
    }

    @objid ("ca53af8b-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void dragOver(DropTargetEvent event) {
    }

    @objid ("ca58743b-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void dropAccept(DropTargetEvent event) {
    }

    @objid ("ca5611e4-feb2-11dd-8b31-0014222a9f79")
    @Override
    public void drop(DropTargetEvent event) {
        ModelElementTransfer elementTransfer = ModelElementTransfer.getInstance();
        
        // We take ModelElementTransfert data
        for (int i = 0; i < event.dataTypes.length; i++) {
            if (elementTransfer.isSupportedType(event.dataTypes[i])) {
                event.currentDataType = event.dataTypes[i];
                break;
            }
        }
        
        if (elementTransfer.isSupportedType(event.currentDataType)) {
            ObRef[] refs = (ObRef[]) elementTransfer.nativeToJava(event.currentDataType);
            if (refs != null) {
                IModelingSession modelingSession = Modelio.getInstance().getModelingSession();
                List<IElement> elements = new ArrayList<IElement>(refs.length);
                for (int i = 0; i < refs.length ; i++) {
                    elements.add(modelingSession.findByRef(refs[i]));
                }
                
                // Under linux, we must check the elements are valid before validating the drop...
                if (System.getProperty("os.name").equals("Linux")) {
                    for (IElement element : elements) {
                        if (element instanceof IArtifact) {
                            IArtifact artifact = (IArtifact) element;
                            if (artifact.equals(this.currentRamc)) {
                                return;
                            }
                            if (!artifact.isStereotyped("ModelComponentArchive")) {
                                return;
                            }
                        } else {
                            return;
                        }
                    }
                }
        
                final List<IArtifact> parentRamcs = new ArrayList<IArtifact>();
                final List<IArtifact> cycleArtifacts = new ArrayList<IArtifact>();
        
                for (IElement element : elements) {
                    final IArtifact artifact = (IArtifact) element;
                    if (artifact.isStereotyped("ModelComponentArchive")) {
                        if (!causeCycle(artifact)) {
                            parentRamcs.add(artifact);
                        } else {
                            cycleArtifacts.add(artifact);
                        }
                    }
                }
        
                if (cycleArtifacts.size() > 0) {
                    final StringBuffer cycleArtifactNames = new StringBuffer();
                    int i = 0;
                    for (IArtifact cycleArtifact : cycleArtifacts) {
                        if (i > 0) {
                            cycleArtifactNames.append(", ");
                        }
                        cycleArtifactNames.append(cycleArtifact.getName());
                        i++;
                    }
        
                    if (cycleArtifacts.size() > 1) {
                        MessageDialog.openInformation(this.dialog.getShell(), I18nMessageService
                                .getString("EditRamcDialog.CycleArtifacts.Title"), I18nMessageService
                                .getString("EditRamcDialog.CycleArtifacts.ModelComponentsMessage",
                                            cycleArtifactNames.toString(),
                                            this.currentRamc.getName()));
                    } else {
                        MessageDialog.openInformation(this.dialog.getShell(), I18nMessageService
                                .getString("EditRamcDialog.CycleArtifacts.Title"), I18nMessageService
                                .getString("EditRamcDialog.CycleArtifacts.ModelComponentMessage",
                                            cycleArtifactNames.toString(),
                                            this.currentRamc.getName()));
                    }
                }
        
                this.controler.addRequiredFragments(parentRamcs);
                this.dialog.refresh();
            }
        }
    }

    @objid ("b1bba40b-0a3b-11de-8d83-0014222a9f79")
    private boolean causeCycle(IArtifact artifact) {
        ObList<IElementImport> elementImports = artifact.getOwnedImport();
        INameSpace importedNamespace = null;
        
        for (IElementImport elementImport : elementImports) {
            importedNamespace = elementImport.getImportedElement();
            if (importedNamespace instanceof IArtifact &&
                importedNamespace.isStereotyped(IModelFragmentStereotypes.MODELFRAGMENTDEPLOYED))
            {
                if (importedNamespace.equals(this.currentRamc)) {
                    return true;
                } else {
                    return causeCycle((IArtifact) importedNamespace);
                }
            }
        }
        return false;
    }

}
