/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package org.modelio.modelfragment.command.ihm;

import org.modelio.modelfragment.api.IModelFragmentTagTypes;

import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.modelio.Version;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

@objid ("be401dfd-fe92-11dd-8b31-0014222a9f79")
public class EditFragmentAdapter {
    @objid ("be49a764-fe92-11dd-8b31-0014222a9f79")
    private String fragmentName = "";

    @objid ("be49a767-fe92-11dd-8b31-0014222a9f79")
    private String fragmentDescription = "";

    @objid ("be474530-fe92-11dd-8b31-0014222a9f79")
    private IArtifact fragment = null;

    @objid ("be44e2b9-fe92-11dd-8b31-0014222a9f79")
    private Version fragmentVersion = null;


    @objid ("be49a76e-fe92-11dd-8b31-0014222a9f79")
    public EditFragmentAdapter(IArtifact fragment) {
        this.fragment = fragment;
        
        this.fragmentName = fragment.getName();
        this.fragmentVersion = this.getVersion();
        this.fragmentDescription = this.getDescription();
    }

    @objid ("be49a769-fe92-11dd-8b31-0014222a9f79")
    private String getDescription() {
        ObList<INote> notes = this.fragment.getDescriptor();
        
        for (INote note : notes) {
            INoteType type = note.getModel();
            if (type != null && type.getName().equals("description")) {
                return note.getContent();
            }
        }
        return "";
    }

    @objid ("be49a785-fe92-11dd-8b31-0014222a9f79")
    private Version getVersion() {
    	
		for(ITaggedValue tag : this.fragment.getTag()){
			if (tag.getName().equals(IModelFragmentTagTypes.MODELFRAGMENTDEPLOYED_VERSION)){
				return new Version(tag.getQualifier().getValue());
			}
		}
		return null;


    }

    @objid ("be4280da-fe92-11dd-8b31-0014222a9f79")
    public String getFragmentName() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragmentName;
    }

    @objid ("be44e2dd-fe92-11dd-8b31-0014222a9f79")
    public String getFragmentDescription() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragmentDescription;
    }

    @objid ("be49a768-fe92-11dd-8b31-0014222a9f79")
    public Version getFragmentVersion() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragmentVersion;
    }

    @objid ("be44e2b8-fe92-11dd-8b31-0014222a9f79")
    public IArtifact getFragment() {
        return this.fragment;
    }

    @objid ("bd1996a0-ff45-11dd-95ab-0014222a9f79")
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof EditFragmentAdapter) {
            return this.fragment.equals(((EditFragmentAdapter) obj).getFragment());
        }
        return false;
    }

}
