/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package org.modelio.modelfragment.command.ihm;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.model.change.IModelChangeEvent;
import com.modeliosoft.modelio.core.model.change.IModelChangeListener;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Display;

@objid ("be401e02-fe92-11dd-8b31-0014222a9f79")
public class EditFragmentController implements ModifyListener, ICheckStateListener, IModelChangeListener {
    @objid ("be474516-fe92-11dd-8b31-0014222a9f79")
    private EditFragmentDataModel dataModel = null;

    @objid ("be47455c-fe92-11dd-8b31-0014222a9f79")
    private EditFragmentDialog dialog = null;

    /**
     * This regular expression is used to match allowed ramc name. '\p{L}' = any unicode letter (allowing for é, à,
     * ... and so on) '\p{N}' = any unicode number '.', ' ' and '_' are also allowed
     */
    @objid ("ca34b0ec-feb2-11dd-8b31-0014222a9f79")
    private static final Pattern FragmentNamePattern = Pattern.compile("[\\p{L}\\p{N}\\._ ]+");


    @objid ("be474533-fe92-11dd-8b31-0014222a9f79")
    public EditFragmentController(EditFragmentDataModel dataModel, EditFragmentDialog dialog) {
        this.dataModel = dataModel;
        this.dialog = dialog;
    }

    @objid ("be44e2e8-fe92-11dd-8b31-0014222a9f79")
    @Override
    public void modifyText(ModifyEvent event) {
        boolean isValidRamcName = true;
        boolean isValidRamcVersion = true;
        boolean isValidRamcPath = true;
        
        if (event.widget == this.dialog.fragNameText) {
            isValidRamcName = validateFragmentName();
            if (isValidRamcName) {
                this.dataModel.setFragmentName(this.dialog.getFragmentName());
            }
            this.dialog.invalidateFragmentName(!isValidRamcName);
        }
        
//        if (event.widget == this.dialog.fragPathText) {
//            isValidRamcPath = validateFragmentPath();
//            if (isValidRamcPath) {
//                this.dataModel.setModelComponentPath(new File(this.dialog.getFragmentPath()));
//            }
//            this.dialog.invalidateFragmentPath(!isValidRamcPath);
//        }
        
        if (event.widget == this.dialog.fragVersionText) {
//            isValidRamcVersion = validateFragmentVersion();
//            if (isValidRamcVersion) {
                this.dataModel.setFragmentVersion(this.dialog.getVersion());
//            }
//            this.dialog.invalidateFragmentVersion(!isValidRamcVersion);
        }
        
        String description = this.dialog.getDescription();
        description = description.replaceAll("\r\n", "\n");
        this.dataModel.setFragmentDescription(description);
        
        this.dialog.updateButtons(isValidRamcVersion && isValidRamcPath);
    }

    @objid ("be44e30d-fe92-11dd-8b31-0014222a9f79")
    @Override
    public void checkStateChanged(CheckStateChangedEvent event) {
        EditFragmentContributorAdapter adapter = (EditFragmentContributorAdapter) event.getElement();
        
        adapter.setSelected(event.getChecked());
    }

    @objid ("be49a76a-fe92-11dd-8b31-0014222a9f79")
    private boolean validateFragmentVersion() {
        try {
            // Test version validity
            this.dialog.getVersion();
            return true;
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

//    @objid ("be4280de-fe92-11dd-8b31-0014222a9f79")
//    private boolean validateFragmentPath() {
//        String ramcPath = this.dialog.getFragmentPath();
//        File ramcFile = new File(ramcPath);
//        
//        if (ramcPath == null || ramcPath.equals("")) {
//            return false;
//        }
//        
//        try {
//            // Test ramcFile validity
//            ramcFile.getCanonicalFile();
//        } catch (IOException e) {
//            return false;
//        }
//        
//        if (ramcPath.endsWith(".ramc")) {
//            return true;
//        }
//        return false;
//    }

    @objid ("be428094-fe92-11dd-8b31-0014222a9f79")
    public boolean validateData() {
        boolean isRamcNameValid = validateFragmentName();
        boolean isRamcVersionValid = validateFragmentVersion();
//        boolean isRamcPathValid = validateFragmentPath();
        
        this.dialog.invalidateFragmentName(!isRamcNameValid);
        this.dialog.invalidateFragmentVersion(!isRamcVersionValid);
//        this.dialog.invalidateFragmentPath(!isRamcPathValid);
//        return isRamcNameValid && isRamcVersionValid && isRamcPathValid;
        return isRamcNameValid && isRamcVersionValid;
    }

    @objid ("ca2b2784-feb2-11dd-8b31-0014222a9f79")
    private boolean validateFragmentName() {
        String fragmentName = this.dialog.getFragmentName();
        return FragmentNamePattern.matcher(fragmentName).matches();
    }

    @objid ("bb37fba2-ff45-11dd-95ab-0014222a9f79")
    public void addRequiredFragments(List<IArtifact> ramcs) {
        EditFragmentAdapter adapter = null;
        List<EditFragmentAdapter> requiredRamcs = this.dataModel.getParentFragments();
        
        for (IArtifact ramc : ramcs) {
            adapter = new EditFragmentAdapter(ramc);
            if (!requiredRamcs.contains(adapter)) {
                requiredRamcs.add(adapter);
            }
        }
    }

    @objid ("bb3a5df9-ff45-11dd-95ab-0014222a9f79")
    public void removeRequiredFragment(EditFragmentAdapter adapter) {
        List<EditFragmentAdapter> requiredRamcs = this.dataModel.getParentFragments();
        requiredRamcs.remove(adapter);
    }

    @objid ("22f932fc-ff6f-11dd-95ab-0014222a9f79")
    public void addManifestedElements(List<IElement> elements) {
        EditManifestedAdapter adapter = null;
        List<EditManifestedAdapter> manifestedElements = this.dataModel.getManifestedElements();
        
        for (IElement element : elements) {
            adapter = new EditManifestedAdapter(element);
            if (!manifestedElements.contains(adapter)) {
                manifestedElements.add(adapter);
            }
        }
    }

    @objid ("22fdf7a8-ff6f-11dd-95ab-0014222a9f79")
    public void removeExportedAdapter(EditManifestedAdapter adapter) {
        List<EditManifestedAdapter> manifestedElements = this.dataModel.getManifestedElements();
        manifestedElements.remove(adapter);
    }

    @objid ("72a88547-027f-11de-af8a-0014222a9f79")
    @Override
    public void modelChanged(IModelingSession session, IModelChangeEvent event) {
        if (!this.dataModel.getModelComponent().isValid()) {
            this.dialog.close(Window.CANCEL);
            return;
        }
        
        List<EditManifestedAdapter> manifestedAdapters = this.dataModel.getManifestedElements();
        List<EditManifestedAdapter> manifestedAdaptersToRemove = new ArrayList<EditManifestedAdapter>();
        
        for (EditManifestedAdapter manifestedAdapter : manifestedAdapters) {
            if (!manifestedAdapter.getElement().isValid()) {
                manifestedAdaptersToRemove.add(manifestedAdapter);
            }
        }
        
        for (EditManifestedAdapter manifestedAdapterToRemove : manifestedAdaptersToRemove) {
            removeExportedAdapter(manifestedAdapterToRemove);
        }
        
        List<EditFragmentAdapter> requiredAdapters = this.dataModel.getParentFragments();
        List<EditFragmentAdapter> requiredAdaptersToRemove = new ArrayList<EditFragmentAdapter>();
        for (EditFragmentAdapter requiredAdapter : requiredAdapters) {
            if (!requiredAdapter.getFragment().isValid()) {
                requiredAdaptersToRemove.add(requiredAdapter);
            }
        }
        
        for (EditFragmentAdapter requiredAdapterToRemove : requiredAdaptersToRemove) {
            removeRequiredFragment(requiredAdapterToRemove);
        }
        
        // Re enter the UI thread
        Display display = Display.getDefault();
        if (display != null) {
            display.asyncExec(new Runnable() {
                @Override
                public void run()
                {
                    EditFragmentController.this.dialog.refresh();
                }
            });
        }
    }

}
