/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package org.modelio.modelfragment.command.ihm;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.modelio.modelfragment.api.IModelFragmentStereotypes;
import org.modelio.modelfragment.model.FragmentDefinition;

import com.modeliosoft.modelio.api.model.analyst.IDictionary;
import com.modeliosoft.modelio.api.model.analyst.IRequirementContainer;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IElementImport;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

@objid ("be401dfc-fe92-11dd-8b31-0014222a9f79")
public class EditFragmentDataModel {
    @objid ("be49a78e-fe92-11dd-8b31-0014222a9f79")
    private String fragName = "";

    @objid ("be49a78d-fe92-11dd-8b31-0014222a9f79")
    private String fragDescription = "";

    @objid ("be4280dc-fe92-11dd-8b31-0014222a9f79")
    private FragmentDefinition frag = null;

    @objid ("be42809a-fe92-11dd-8b31-0014222a9f79")
    private String fragVersion = null;

    @objid ("be4280e1-fe92-11dd-8b31-0014222a9f79")
    private List<EditFragmentAdapter> parentfrags = null;

    @objid ("be44e2b1-fe92-11dd-8b31-0014222a9f79")
    private File modelComponentPath = null;

    @objid ("2414ead3-ff6f-11dd-95ab-0014222a9f79")
    private List<EditManifestedAdapter> manifestedElements = null;

    @objid ("2419af85-ff6f-11dd-95ab-0014222a9f79")
    private Set<File> exportedFiles = null;


    @objid ("be47451e-fe92-11dd-8b31-0014222a9f79")
    public EditFragmentDataModel(IArtifact frag) {
        this.frag = new FragmentDefinition(frag);
        
        this.fragName = frag.getName();
        this.fragVersion = this.frag.getVersion();
        this.fragDescription = this.frag.getDescription();
        
        this.initManifested();
        this.initDependencies();
//        this.initExportedFiles();
    }

    @objid ("be47454e-fe92-11dd-8b31-0014222a9f79")
    private String getDescription() {
        return this.frag.getDescription();
//        
//        for (INote note : notes) {
//            INoteType type = note.getModel();
//            if (type != null && type.getName().equals("description")) {
//                return note.getContent();
//            }
//        }
//        return "";
    }

    @objid ("be474545-fe92-11dd-8b31-0014222a9f79")
    private String getVersion() {
//        ObList<ITaggedValue> taggedValues = this.frag.getVersion();
//        
//        for (ITaggedValue taggedValue : taggedValues) {
//            ITagType type = taggedValue.getDefinition();
//            if (type != null && type.getName().equals("ModelComponentVersion")) {
//                ObList<ITagParameter> parameters = taggedValue.getActual();
//                if (parameters.size() == 1) {
//                    return new Version(parameters.get(0).getValue());
//                }
//            }
//        }
//        return new Version(0, 0, 0, 0);
    	return this.frag.getVersion();
    }

    @objid ("be474562-fe92-11dd-8b31-0014222a9f79")
    private void initDependencies() {
        this.parentfrags = new ArrayList<EditFragmentAdapter>();
        
        ObList<IElementImport> elementImports = this.frag.getDefinition().getOwnedImport();
       
        for (IElementImport elementImport : elementImports) {
            INameSpace importedNamespace = elementImport.getImportedElement();
        
            if (importedNamespace instanceof IArtifact &&
                importedNamespace.isStereotyped(IModelFragmentStereotypes.MODELFRAGMENT)) {
                IArtifact fragArtifact = (IArtifact) importedNamespace;
                this.parentfrags.add(new EditFragmentAdapter(fragArtifact));
            }
        }
    }

    @objid ("be44e2c5-fe92-11dd-8b31-0014222a9f79")
    public String getFragmentName() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragName;
    }

    @objid ("be47450c-fe92-11dd-8b31-0014222a9f79")
    public String getFragmentDescription() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragDescription;
    }

    @objid ("be47452b-fe92-11dd-8b31-0014222a9f79")
    public void setFragmentDescription(String value) {
        // Automatically generated method. Please delete this comment before entering specific code.
        this.fragDescription = value;
    }

    @objid ("be44e2ea-fe92-11dd-8b31-0014222a9f79")
    public String getFragmentVersion() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.fragVersion;
    }

    @objid ("be44e2da-fe92-11dd-8b31-0014222a9f79")
    public void setFragmentVersion(String value) {
        // Automatically generated method. Please delete this comment before entering specific code.
        this.fragVersion = value;
    }

    @objid ("be44e315-fe92-11dd-8b31-0014222a9f79")
    public List<EditFragmentAdapter> getParentFragments() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.parentfrags;
    }
   

    @objid ("9852d350-fe95-11dd-8b31-0014222a9f79")
    public File getModelComponentPath() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.modelComponentPath;
    }

    @objid ("a5178b4e-fe95-11dd-8b31-0014222a9f79")
    public void setModelComponentPath(File value) {
        // Automatically generated method. Please delete this comment before entering specific code.
        this.modelComponentPath = value;
    }

    @objid ("cb316a30-feb2-11dd-8b31-0014222a9f79")
    public void setFragmentName(String name) {
        this.fragName = name;
    }

    @objid ("23d48b4c-ff6f-11dd-95ab-0014222a9f79")
    public Set<File> getExportedFiles() {
        return this.exportedFiles;
    }

//    @objid ("23d95003-ff6f-11dd-95ab-0014222a9f79")
//    private void initExportedFiles() {
//        this.exportedFiles = new HashSet<File>();
//        
//        ObList<ITaggedValue> taggedvalues = this.frag.getTag();
//        
//        for (ITaggedValue taggedvalue : taggedvalues) {
//            ITagType type = taggedvalue.getDefinition();
//            if (type != null && type.getName().equals("ModelComponentFiles")) {
//                for (ITagParameter parameter : taggedvalue.getActual()) {
//                    this.exportedFiles.add(new File(parameter.getValue()));
//                }
//            }
//        }
//    }
    
   public void save(){
	   this.frag.setName(this.fragName);
	   this.frag.setDescription(this.fragDescription);
	   this.frag.setVersion(this.fragVersion);
	   for (EditManifestedAdapter manifested : this.manifestedElements){
		   this.frag.addManifested((IPackage)manifested.getElement());
	   }
   }

    @objid ("23d95008-ff6f-11dd-95ab-0014222a9f79")
    private void initManifested() {
        this.manifestedElements = new ArrayList<EditManifestedAdapter>();
        
        ObList<IManifestation> manifestations = this.frag.getDefinition().getUtilized();
        IModelElement me = null;
        
        for (IManifestation manifestation : manifestations) {
            me = manifestation.getUtilizedElement();
            if (me instanceof INameSpace) {
                this.manifestedElements.add(new EditManifestedAdapter(me));
            } else if (me instanceof IDictionary) {
                this.manifestedElements.add(new EditManifestedAdapter(me));
            } else if (me instanceof IRequirementContainer) {
                this.manifestedElements.add(new EditManifestedAdapter(me));
            }
        }
    }

    @objid ("24043a54-ff6f-11dd-95ab-0014222a9f79")
    public List<EditManifestedAdapter> getManifestedElements() {
        return this.manifestedElements;
    }

    @objid ("24069caf-ff6f-11dd-95ab-0014222a9f79")
    public void addExportedFile(File file) {
        this.exportedFiles.add(file);
    }

    @objid ("b2aa69e1-ff73-11dd-95ab-0014222a9f79")
    public void removeExportedFile(File file) {
        this.exportedFiles.remove(file);
    }

    @objid ("80cdfcea-027a-11de-af8a-0014222a9f79")
    public IArtifact getModelComponent() {
        return this.frag.getDefinition();
    }

}
