/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package org.modelio.modelfragment.command.ihm;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;

import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.analyst.IDictionary;
import com.modeliosoft.modelio.api.model.analyst.IPropertySet;
import com.modeliosoft.modelio.api.model.analyst.IPropertyType;
import com.modeliosoft.modelio.api.model.analyst.IRequirementContainer;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElementStatus;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.ui.dnd.ModelElementTransfer;
import com.modeliosoft.modelio.api.utils.ObRef;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

/**
 * Edit manifestations drop listener.
 */
@objid ("070aa32e-ff48-11dd-95ab-0014222a9f79")
public class EditManifestationsDropListener implements DropTargetListener {
    @objid ("070d05a4-ff48-11dd-95ab-0014222a9f79")
    private EditFragmentDialog dialog;

    @objid ("070d05a7-ff48-11dd-95ab-0014222a9f79")
    private EditFragmentController controler = null;


    /**
     * Default constructor.
     * @param dialog the ramc edition dialog.
     * @param controler the edition controller.
     */
    @objid ("070d058a-ff48-11dd-95ab-0014222a9f79")
    public EditManifestationsDropListener(EditFragmentDialog dialog, EditFragmentController controler) {
        this.controler = controler;
        this.dialog = dialog;
    }

    @objid ("070aa330-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void dragEnter(DropTargetEvent event) {
        ModelElementTransfer elementTransfer = ModelElementTransfer.getInstance();
        
        // We take ModelElementTransfert data
        for (int i = 0; i < event.dataTypes.length; i++) {
            if (elementTransfer.isSupportedType(event.dataTypes[i])) {
                event.currentDataType = event.dataTypes[i];
                break;
            }
        }
        
        if (elementTransfer.isSupportedType(event.currentDataType)) {
            boolean accept;
            // NOTE: on unsupported platforms this will return null
            ObRef[] refs = (ObRef[]) elementTransfer.nativeToJava(event.currentDataType);
            if (refs != null) {
                IModelingSession modelingSession = Modelio.getInstance().getModelingSession();
                List<IElement> elements = new ArrayList<IElement>(refs.length);
                for (int i = 0; i < refs.length ; i++) {
                    elements.add(modelingSession.findByRef(refs[i]));
                }
        
                accept = checkDragged(elements);
            } else {
                // On linux, event.currentDataType is not filled, we have to validate the drag and test the condition in the drop
                accept = true;
            }
        
            if (!accept) {
                event.detail = DND.DROP_NONE;
            }
        }
    }

    @objid ("070aa331-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void dragLeave(DropTargetEvent event) {
    }

    @objid ("070aa332-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void dragOperationChanged(DropTargetEvent event) {
    }

    @objid ("070aa333-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void dragOver(DropTargetEvent event) {
    }

    @objid ("070aa334-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void drop(DropTargetEvent event) {
        ModelElementTransfer elementTransfer = ModelElementTransfer.getInstance();
        
        // We take ModelElementTransfert data
        for (int i = 0; i < event.dataTypes.length; i++) {
            if (elementTransfer.isSupportedType(event.dataTypes[i])) {
                event.currentDataType = event.dataTypes[i];
                break;
            }
        }
        
        if (elementTransfer.isSupportedType(event.currentDataType)) {
            ObRef[] refs = (ObRef[]) event.data;
            if (refs != null) {
                // Resolve refs
                List<IElement> elements = new ArrayList<IElement>(refs.length);
                IModelingSession modelingSession = Modelio.getInstance().getModelingSession();
                for (ObRef ref : refs) {
                    IElement elt = modelingSession.findByRef(ref);
                    if (elt != null) {
                        elements.add(elt);
                    }
                }
        
                // Check the elements
                if (checkDragged(elements)) {
                    this.controler.addManifestedElements(elements);
                    this.dialog.refresh();
                }
            }
        }
    }

    @objid ("070aa335-ff48-11dd-95ab-0014222a9f79")
    @Override
    public void dropAccept(DropTargetEvent event) {
    }

    @objid ("2484f950-ff6f-11dd-95ab-0014222a9f79")
    private boolean canBeExported(IElement element) {
        if (element instanceof IPackage) {
            return !element.equals(Modelio.getInstance().getModelingSession().getModel().getRoot());
        } else if (element instanceof INameSpace) {
            return true;
        } else if (element instanceof IDictionary) {
            return true;
        } else if (element instanceof IRequirementContainer) {
            return true;
        } else if (element instanceof IPropertySet) {
            return true;
        } else if (element instanceof IPropertyType) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Check the given elements can be dragged.
     * @param elements The dragged elements
     * @return true if the given elements can be dragged.
     */
    @objid ("86dd305b-303c-11df-95d7-001ec947ccaf")
    private boolean checkDragged(List<IElement> elements) {
        for (IElement element : elements) {
            final IElementStatus elementStatus = element.getElementStatus();
            if (!canBeExported(element) || elementStatus.isRamcObject()) {
                return false;
            }
        }
        return true;
    }

}
