package org.modelio.modelfragment.command.ihm;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.galaxy.client.service.fragmentmanagement.ModelFragmentData;
import org.modelio.modelfragment.utils.SWTResourceManager;

import com.modeliosoft.modelio.api.ui.ModelioDialog;

public class RetrieveDialog extends ModelioDialog{

	private Button ok;
	private Text descriptionText;
	private Group descriptionGroup;
	private List dependencyList;
	private List fragmentList;

	private ModelFragmentData[] fragments;
	private ModelFragmentData selectedFragment;

	public RetrieveDialog(Shell shell, ModelFragmentData[] fragments) {
		super(shell);
		setShellStyle(SWT.DIALOG_TRIM | SWT.ON_TOP | getDefaultOrientation());
		this.fragments = fragments;
	}


	public ModelFragmentData getSelectedFragment(){
		return selectedFragment;
	}

	@Override
	protected void okPressed() {
		super.okPressed();
	}

	@Override
	public void addButtonsInButtonBar(Composite parent) {
		ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL, false);
		ok.setEnabled(false);
	}

	@Override
	public Control createContentArea(Composite composite) {
		Composite parent  = new Composite(composite, composite.getStyle());
		parent.setLayout(new GridLayout(4, false));

		Label fragmentLabel = new Label(parent, SWT.NONE);
		GridData gd_fragmentLabel = new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1);
		gd_fragmentLabel.heightHint = 285;
		fragmentLabel.setLayoutData(gd_fragmentLabel);
		fragmentLabel.setText("Fragment");

		fragmentList = new List(parent, SWT.BORDER | SWT.H_SCROLL);
		GridData gd_fragmentList = new GridData(SWT.CENTER, SWT.FILL, false, false, 1, 1);
		gd_fragmentList.heightHint = 161;
		gd_fragmentList.widthHint = 181;
		fragmentList.setLayoutData(gd_fragmentList);

		Group descriptionGroup = new Group(parent, SWT.NONE);
		descriptionGroup.setText("Model Fragment");
		descriptionGroup.setLayout(new GridLayout(1, false));
		GridData gd_descriptionGroup = new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1);
		gd_descriptionGroup.widthHint = 338;
		descriptionGroup.setLayoutData(gd_descriptionGroup);

		Label descriptionLabel = new Label(descriptionGroup, SWT.NONE);
		GridData gd_descriptionLabel = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
		gd_descriptionLabel.widthHint = 327;
		gd_descriptionLabel.heightHint = 15;
		descriptionLabel.setLayoutData(gd_descriptionLabel);
		descriptionLabel.setText("Description");

		descriptionText = new Text(descriptionGroup, SWT.BORDER | SWT.READ_ONLY | SWT.MULTI);
		descriptionText.setBackground(SWTResourceManager.getColor(255, 228, 181));
		GridData gd_descriptionText = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
		gd_descriptionText.heightHint = 130;
		descriptionText.setLayoutData(gd_descriptionText);

		Label dependencyLabel = new Label(descriptionGroup, SWT.NONE);
		GridData gd_dependencyLabel = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
		gd_dependencyLabel.widthHint = 328;
		gd_dependencyLabel.heightHint = 15;
		dependencyLabel.setLayoutData(gd_dependencyLabel);
		dependencyLabel.setText("Dependency");

		dependencyList = new List(descriptionGroup, SWT.BORDER | SWT.H_SCROLL);
		dependencyList.setBackground(SWTResourceManager.getColor(255, 222, 173));
		GridData gd_dependencyList = new GridData(SWT.FILL, SWT.TOP, false, false, 1, 1);
		gd_dependencyList.heightHint = 53;
		dependencyList.setLayoutData(gd_dependencyList);


		for(ModelFragmentData data  : fragments){
			fragmentList.add(data.getName() + "-" +data.getVersion());
		}

		return null;
	}

	@Override
	public void init() {
		Shell shell = getShell();
		shell.setSize(620, 450);
		int x = (this.getShell().getSize().x / 2) - 250;
		int y = (this.getShell().getSize().y / 2) - 100;
		shell.setLocation(x, y);
		shell.setMinimumSize(620, 450);
		shell.setText("Modelio");
		setTitle("Retrieve Model Fragment");
		setMessage("Retrieve Model Fragment");
		addListeners();
	}

	private void addListeners() {
		this.fragmentList.addListener(SWT.DefaultSelection, new Listener(){
			public void handleEvent(Event event) {
				if (event.widget.equals(fragmentList)) {
					if(fragmentList.getSelectionIndex() >= 0){  
						ok.setEnabled(true);
						selectedFragment = fragments[fragmentList.getSelectionIndex()];

						if(selectedFragment.getDescription() != null)
							descriptionText.setText(selectedFragment.getDescription()); 

						//        	                descriptionGroup.setText(selectedFragment.getName() + "-" + selectedFragment.getVersion());

						dependencyList.removeAll();

						//        	                for(ModelFragmentData data : selectedFragment.getDependencies()){
						//        	                    dependencyList.add(data.getName() + "-" +data.getVersion());
						//        	                }


					} else{
						descriptionText.setText("");
						descriptionGroup.setText("");
						dependencyList.removeAll();
						ok.setEnabled(false);
					}
				}
			}	
		});
	}

	//    @Override
	//    public void handleEvent(Event event) {
	//        if (event.widget.equals(this.fragmentList)) {
	//            if(this.fragmentList.getSelectionIndex() >= 0){  
	//            	
	//                selectedFragment = fragments[this.fragmentList.getSelectionIndex()];
	//                
	//                if(selectedFragment.getDescription() != null)
	//                    this.descriptionText.setText(selectedFragment.getDescription()); 
	//   
	//                this.descriptionGroup.setText(selectedFragment.getName() + "-" +selectedFragment.getVersion());
	//          
	//                this.dependencyList.removeAll();
	//                
	//                for(ModelFragmentData data : selectedFragment.getDependencies()){
	//                    this.dependencyList.add(data.getName() + "-" +data.getVersion());
	//                }
	//                ok.setEnabled(true);
	//                
	//            } else{
	//                this.descriptionText.setText("");
	//                this.descriptionGroup.setText("");
	//                this.dependencyList.removeAll();
	//                ok.setEnabled(false);
	//            }
	//        }
	//    }

}
