package org.modelio.modelfragment.impl;

import org.modelio.modelfragment.utils.ResourcesManager;

import com.modeliosoft.modelio.api.mdac.AbstractJavaMdac;
import com.modeliosoft.modelio.api.mdac.IMdacSession;
import com.modeliosoft.modelio.api.mdac.IParameterEditionModel;
import com.modeliosoft.modelio.api.mdac.IParameterModel;
import com.modeliosoft.modelio.api.mdac.paramEdition.ParameterGroupModel;
import com.modeliosoft.modelio.api.mdac.paramEdition.ParametersEditionModel;
import com.modeliosoft.modelio.api.mdac.paramEdition.StringParameterModel;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.ITransaction;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.module.IMdacConfiguration;

/**
 * Implementation of the IMdac interface. <br>
 * All Modelio java modules should inherit from this class.
 * 
 */
public class ModelFragmentMdac extends AbstractJavaMdac {

    private ModelFragmentPeerMdac peerMdac = null;

    private ModelFragmentSession session = null;

    public ModelFragmentPeerMdac getPeerMdac() {
        return this.peerMdac;
    }

    /**
     * Return the session attached to the current module.
     * <p>
     * <p>
     * This session is used to manage the module lifecycle by declaring the desired implementation on start, select...
     * methods.
     */
    public IMdacSession getSession() {
        return this.session;
    }

    /**
     * Method automatically called just after the creation of the module.
     * <p>
     * <p>
     * The module is automatically instanciated at the beginning of the MDA lifecycle and constructor implementation is
     * not accessible to the module developer.
     * <p>
     * <p>
     * The <code>init</code> method allows the developer to execute the desired initialization.
     * 
     * 
     * @see com.modeliosoft.modelio.api.mdac.AbstractJavaMdac#init()
     */
    @Override
    public void init() {
        // Add the module initialization code
    	ResourcesManager.getInstance().setJMDAC(this);
    }

    /**
     * Builds a new module.
     * <p>
     * <p>
     * This constructor must not be called by the user. It is automatically invoked by Modelio when the module is
     * installed, selected or started.
     */
    public ModelFragmentMdac(IModelingSession modelingSession, IModule model, IMdacConfiguration configuration) {
        super(modelingSession, model, configuration);
        this.session = new ModelFragmentSession(this);
        this.peerMdac = new ModelFragmentPeerMdac(this);
        this.peerMdac.init();
    }

    /**
     * @see com.modeliosoft.modelio.api.mdac.AbstractJavaMdac#getParametersEditionModel()
     */
    @Override
    public IParameterEditionModel getParametersEditionModel() {

        IMdacConfiguration configuration = this.getConfiguration();
        ParametersEditionModel parameters = new ParametersEditionModel(this);
        IParameterModel parameter;

        this.parameterEditionModel = parameters;

        ITransaction tr = Modelio.getInstance().getModelingSession().createTransaction("Parameter");
        try {
            ParameterGroupModel group = new ParameterGroupModel("group", "Galaxy Repository");

            parameters.addGroup(group);

            parameter = new StringParameterModel(configuration,
                                                 "ServerURL",
                                                 "Galaxy Server URL",
                                                 "URL of galaxy Repository",
                                                 "");

            group.addParameter(parameter);
            Modelio.getInstance().getModelingSession().commit(tr);
        } catch (Exception e) {
            Modelio.getInstance().getModelingSession().rollback(tr);
        }

        return this.parameterEditionModel;
    }

    public String getMdacImage() {
        return "/res/icons/fragment16.png";
    }

}
