package org.modelio.modelfragment.impl;

import java.util.Map;

import com.modeliosoft.modelio.api.log.ILogService;
import com.modeliosoft.modelio.api.mdac.DefaultMdacSession;
import com.modeliosoft.modelio.api.mdac.MdacException;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.modelio.Version;

/**
 * Implementation of the IMdacSession interface.
 * <br>This default implementation may be inherited by the mdac developers in order to simplify the code writing of the mdac session.
 */
public class ModelFragmentSession extends DefaultMdacSession {

	/**
	 * Constructor.
	 */
	public ModelFragmentSession(ModelFragmentMdac module) {
		super(module);
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.DefaultMdacSession#start()
	 */
	@Override
	public boolean start() throws MdacException {
		// get the version of the module
		Version moduleVersion = this.mdac.getVersion();

		// get the Modelio log service
		ILogService logService = Modelio.getInstance().getLogService();

		String message = "Start of " + this.mdac.getName() + " " + moduleVersion;
		logService.info(this.mdac, message);
		return super.start();
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.DefaultMdacSession#stop()
	 */
	@Override
	public void stop() throws MdacException {
		super.stop();
	}

	public static boolean install(String modelioPath, String mdaPath) throws MdacException {
		return DefaultMdacSession.install(modelioPath, mdaPath);
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.DefaultMdacSession#select()
	 */
	@Override
	public boolean select() throws MdacException {
		return super.select();
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.DefaultMdacSession#unselect()
	 */
	@Override
	public void unselect() throws MdacException {
		super.unselect();
	}

	/**
	 * @see com.modeliosoft.modelio.api.mdac.DefaultMdacSession#upgrade(com.modeliosoft.modelio.api.modelio.Version, java.util.Map)
	 */
	@Override
	public void upgrade(Version oldVersion, Map<String, String> oldParameters) throws MdacException {
		super.upgrade(oldVersion, oldParameters);
	}
}
