package org.modelio.modelfragment.model;

import java.util.ArrayList;
import java.util.List;

import org.modelio.modelfragment.api.IModelFragmentStereotypes;
import org.modelio.modelfragment.api.IModelFragmentTagTypes;

import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.ITransaction;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.InvalidTransactionException;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IUsage;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.NoteTypeNotFoundException;
import com.modeliosoft.modelio.api.utils.TagTypeNotFoundException;

public class FragmentDefinition {

	private IArtifact _definition = null;

	private String _descName = "description";

	public FragmentDefinition(IArtifact component){
		this._definition = component;
	}


	public void  setName(String name){
		this._definition.setName(name);
	}

	public String getName(){
		return this._definition.getName();
	}


	public void setVersion(String version){
		IModelingSession session = Modelio.getInstance().getModelingSession();

		ITagParameter tagVersion = null;
		ITaggedValue taggedVersion = null;
		for (ITaggedValue currentTag : this._definition.getTag()){
			if (currentTag.getDefinition().getName().equals(IModelFragmentTagTypes.MODELFRAGMENT_VERSION)){
				taggedVersion = currentTag;
				tagVersion = currentTag.getQualifier();
				break;
			}

		}

		if (tagVersion == null){
			IUmlModel model = session.getModel();
			try {
				if (taggedVersion == null){


					taggedVersion = model.createTaggedValue(IModelFragmentTagTypes.MODELFRAGMENT_VERSION, this._definition);
				}
				taggedVersion.setAnnoted(this._definition);
				tagVersion = model.createTagParameter();
				tagVersion.setAnnoted(taggedVersion);
				taggedVersion.setQualifier(tagVersion);
			} catch (TagTypeNotFoundException e) {
				e.printStackTrace();
			}				

		}

		tagVersion.setValue(version);

	}


	public String getVersion(){
		String result = "";
		for(ITaggedValue tag : this._definition.getTag()){
			if (tag.getName().equals(IModelFragmentTagTypes.MODELFRAGMENT_VERSION)){
				return tag.getQualifier().getValue();
			}
		}
		return result;
	}


	public void setDescription(String description){
		IModelingSession session = Modelio.getInstance().getModelingSession();
		ITransaction tr = session.createTransaction("change version");
		try{
			INote descriptionNote = this._definition.getNote(_descName);

			if (descriptionNote == null){
				IUmlModel model = session.getModel();
				descriptionNote = model.createNote(_descName, this._definition, description);
			}else
				descriptionNote.setContent(description);

			session.commit(tr);
			tr = null;
		} catch (InvalidTransactionException e) {
			// Transaction already rollbacked
			tr = null;
		} catch (NoteTypeNotFoundException e) {
			e.printStackTrace();
		} finally {
			if (tr != null)
				session.rollback(tr);
		}
	}


	public String getDescription(){
		String result = "";
		for (INote note : this._definition.getDescriptor()){
			if (note.getModel().getName().equals("description")){
				return note.getContent();
			}
		}

		return result;
	}


	public void addDependency(String compName, String compVersion){
		for(DeployedFragment def : ModelUtils.getDeployedFragment()){
			if (def.getName().equals(compName) && def.getVersion().equals(compVersion))
				addDependency(def);
		}
	}




	public List<IPackage> getManifested(){

		List<IPackage> result = new ArrayList<IPackage>();

		for(IManifestation manisfest : this._definition.getUtilized()){
			IModelElement utilized = manisfest.getUtilizedElement(); 
			if (utilized instanceof IPackage)
				result.add((IPackage) utilized); 
		}

		return result;
	}

	public void addManifested(IPackage manifest){

		for(IManifestation manisfest : this._definition.getUtilized()){
			IModelElement utilized = manisfest.getUtilizedElement(); 
			if (utilized.equals(manifest)){
				return;
			}
		}

		addManifestation(manifest);

	}

	public List<DeployedFragment> getDependencies(){
		List<DeployedFragment> parents = new ArrayList<DeployedFragment>();
		for (IUsage usage : this._definition.getDependsOnDependency(IUsage.class)){
			IModelElement component = usage.getDependsOn();
			if (component instanceof IComponent){
				parents.add( new DeployedFragment((IComponent)component));				
			}
		}

		return parents;

	}

	private void addManifestation(IPackage manifested) {
		IModelingSession session = Modelio.getInstance().getModelingSession();
		//		ITransaction tr = session.createTransaction("change version");
		//		try{
		IManifestation manifest = session.getModel().createManifestation();
		manifest.setOwner(this._definition);
		manifest.setUtilizedElement(manifested);
		//			session.commit(tr);
		//		} catch (InvalidTransactionException e) {
		//			// Transaction already rollbacked
		//			tr = null;
		//		} finally {
		//			if (tr != null)
		//				session.rollback(tr);
		//		}

	}


	public void removeDependency(String compName, String compVersion){
		IModelingSession session = Modelio.getInstance().getModelingSession();
		ITransaction tr = session.createTransaction("change version");
		try{
			for(IUsage usage : this._definition.getDependsOnDependency(IUsage.class)){
				IModelElement dependElt = usage.getDependsOn();
				if ((dependElt instanceof IComponent) 
						&& (dependElt.isStereotyped(IModelFragmentStereotypes.MODELFRAGMENT))){

					FragmentDefinition def = new FragmentDefinition((IArtifact) dependElt);
					if (def.getName().equals(compName) && def.getVersion().equals(compVersion))
						usage.delete();
				}
			}
		} catch (InvalidTransactionException e) {
			// Transaction already rollbacked
			tr = null;
		} finally {
			if (tr != null)
				session.rollback(tr);
		}
	}


	public IArtifact getDefinition(){
		return this._definition;
	}

	private void addDependency(DeployedFragment dependFrag){
		IModelingSession session = Modelio.getInstance().getModelingSession();
		ITransaction tr = session.createTransaction("change version");
		try{
			session.getModel().createUsage(this._definition, dependFrag.getDeployedFragment());
			session.commit(tr);
			tr = null;
		} catch (InvalidTransactionException e) {
			// Transaction already rollbacked
			tr = null;
		} finally {
			if (tr != null)
				session.rollback(tr);
		}

	}

}
