/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.generation;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.model.objing.IOElement;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ScopeChecker;
import org.eclipse.uml2.uml.Element;

/**
 * This class manages the export sequence of each element.
 * i.e. creation if necessary, attachement, and properties setting
 * @author ebrosse
 */
@objid ("0388b2c4-4583-11e0-b54c-00137279a832")
public class ExportModel {
    @objid ("0388b2c5-4583-11e0-b54c-00137279a832")
    private ProgressBarComposite progressBar = null;

    @objid ("0388b2c6-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = new CompositionOwnerVisitor();

    @objid ("0388b2c7-4583-11e0-b54c-00137279a832")
    private ScopeChecker scopeChecker = new ScopeChecker(GenerationProperties
                                                         .getInstance().getSelectedPackage(), this.ownerVisitor);


    /**
     * The export method is in charge of the export sequence for a given IOElement
     * @param ioelem : the IOElement to export
     */
    @objid ("0388b2c8-4583-11e0-b54c-00137279a832")
    public void export(IOElement ioelem) {
        try {
            if ( this.scopeChecker.contains(ioelem.getObjingElement())) {
        
                Element ecoreElt = TotalExportMap.getInstance().get(ioelem.getObjingID());
        
                if (ecoreElt == null || isSuperCall(ioelem)) {
        
                    if (ecoreElt == null) {
                        ecoreElt =  PartialExportMap.getInstance().remove(ioelem.getObjingID());
                    } else { // Case of the mapping for the super class:
                        PartialExportMap.getInstance().remove(ioelem.getObjingID());
                        TotalExportMap.getInstance().remove(ioelem.getObjingID());
                    }
        
                    if (ecoreElt == null) {
                        // Creation:
                        ecoreElt = createElement(ioelem);
                    }
        
                    if (ecoreElt != null) {
                        PartialExportMap.getInstance().put(ioelem.getObjingID(), ecoreElt);
        
                        // Finish creation:
                        finishElementCreation(ecoreElt, ioelem);
        
                        PartialExportMap.getInstance().remove(ioelem.getObjingID());
                        TotalExportMap.getInstance().put(ioelem.getObjingID(), ecoreElt);
                    } else {
                        throw new NotFoundException("Element of type "
                                                    + ioelem.getObjingElement().getClass()
                                                    .getSimpleName()
                                                    + " has no mapping to EMF-UML2.");
                    }
                }
            }
        } catch (NotFoundException e) {
          LOG.warning(XMI.PLUGIN_ID, e.getLocalizedMessage());
        }
    }

    @objid ("038b1516-4583-11e0-b54c-00137279a832")
    private boolean isSuperCall(IOElement ioelem) {
        IElement objingElt = ioelem.getObjingElement();
        
        String mapperName = ioelem.getClass().getSimpleName();
        String eltClassName = objingElt.getClass().getSimpleName();
        
        mapperName = mapperName.substring(1);
        eltClassName = eltClassName.substring(2);
        
        if (mapperName.equals(eltClassName))
            return false;
        else
            return true;
    }

    /**
     * A constructor with a given progress bar
     * @param progressBar : the given progress bar
     */
    @objid ("038b151a-4583-11e0-b54c-00137279a832")
    public ExportModel(ProgressBarComposite progressBar) {
        super();
        this.progressBar = progressBar;
        GenerationProperties.getInstance().setScopeChecker(this.scopeChecker);
    }

    /**
     * The default constructor
     */
    @objid ("038b151d-4583-11e0-b54c-00137279a832")
    public ExportModel() {
        super();
        this.progressBar = null;
    }

    @objid ("038b151f-4583-11e0-b54c-00137279a832")
    private Element createElement(IOElement ioelem) {
        return ioelem.createEcoreElt();
    }

    @objid ("038fd9cf-4583-11e0-b54c-00137279a832")
    private void finishElementCreation(Element ecoreElt, IOElement ioelem) {
        if ( GenerationProperties.getInstance().isRoundtripEnabled() && 
                 (ioelem.getObjingElement().getElementStatus().isRamcObject())
                 ){
             ObjingEAnnotation.addObjingID(ecoreElt, ioelem.getObjingElement().getIdentifier());
         }
        
         ioelem.attach(ecoreElt, this.ownerVisitor);
         ioelem.setProperties(ecoreElt);
        
         if (this.progressBar != null){
             this.progressBar.addElement();
             if (ioelem.getObjingElement() instanceof INameSpace){
                 this. progressBar.addValue();
             }
         }
    }

}
