/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.generation;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.api.model.analyst.IRequirement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.api.XMIExtension;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportProfile;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ModelerModuleStereotypes;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.ScopeChecker;
import com.modeliosoft.modelio.xmi.util.TaggedValueUtils;
import com.modeliosoft.modelio.xmi.util.XMIFileUtils;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * This class provides the XMI export services.
 * @author ebrosse
 */
@objid ("038fd9d3-4583-11e0-b54c-00137279a832")
public class ExportServices {
    @objid ("fab1a2d2-2b21-11e1-8e26-0027103f347d")
    private static final String version = "2.2";

    @objid ("0834ceb5-a997-11e1-9a8c-0027103f347d")
    private final Shell _shell;


    /**
     * Create an Ecore Resource in the resourcePath location
     * @param resourcePath The desired location of the Ecore Resource
     * @return
     * The Ecore Resource located in resourcePath
     */
    @objid ("038fd9d6-4583-11e0-b54c-00137279a832")
    public Resource createResource(final String resourcePath) {
        File file = new File(resourcePath);
        
        if (!file.exists()) {
            file.getParentFile().mkdirs();
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        // Register the default resource factory -- only needed for
        // stand-alone!
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI,
                UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap()
        .put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        URI uri = URI.createURI(ResourceLoader.getInstance().getUMLRessource());
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri
                .appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri
                .appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri
                .appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap()
        .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        
        URI fileURI2 = URI.createFileURI(file.getAbsolutePath());
        Resource resource2 = resourceSet2.createResource(fileURI2);
        return resource2;
    }

    @objid ("03923c28-4583-11e0-b54c-00137279a832")
    private void applyProfile() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        Model model = genProp.getEcoreModel();
        
        for(Profile profile : genProp.getExportedProfiles()){
            try{
                model.applyProfile(profile);
            }catch(Exception e){
                String errorMsg = e.getMessage();
                LOG.error(XMI.PLUGIN_ID, errorMsg);
            }
        }
        
        if (genProp.isSysMLApplied()){
            try{
                model.applyProfile(genProp.getSysMLProfile());
            }catch(Exception e){
                String errorMsg = e.getMessage();
                LOG.error(XMI.PLUGIN_ID, errorMsg);
            }
            applySysML();
        }
        
        
        for (IModelElement element :  genProp.getStereotypesExported()){
        
            Element ecoreElement = genProp.getMappedElement(element);
        
            if  (ecoreElement != null){
        
                exportStereotype(element, ecoreElement);
        
                // Apply TaggedValues
                if (genProp.isRoundtripEnabled()){
                    TaggedValueUtils.setValueEAnnotation(element, ecoreElement);
                }else{
                    TaggedValueUtils.setValue(element, ecoreElement);
                }
            }
        }
        
        genProp.cleanStereotypesExported();
    }

    /**
     * @param theShell the current shell
     */
    @objid ("03923c2a-4583-11e0-b54c-00137279a832")
    public ExportServices(Shell theShell) {
        this._shell = theShell;
    }

    /**
     * Save the Modelio model in the Ecore resource
     * @param resource The Ecore resource
     * @param progressBar The progress bar of the XMI dialog
     * @return
     * false if an error occurs during the export
     */
    @objid ("03923c2d-4583-11e0-b54c-00137279a832")
    public boolean createEcoreModel(Resource resource, ProgressBarComposite progressBar) {
        boolean error = false;
        if (resource != null) {
        
            GenerationProperties genProp = GenerationProperties.getInstance();
        
            PartialExportMap partialMap = PartialExportMap.getInstance();
            partialMap.clear();
            TotalExportMap totalMap = TotalExportMap.getInstance();
            totalMap.clear();
        
            IPackage selectedPkg = genProp.getSelectedPackage();
        
            Model ecoreModel = UMLFactory.eINSTANCE.createModel();
            ObjingEAnnotation.setExporterVersion(ecoreModel, version);
            ObjingEAnnotation.setRoundTrip(ecoreModel, genProp.isRoundtripEnabled());
            ecoreModel.setName(selectedPkg.getName());
            genProp.setEcoreModel(ecoreModel);
        
            List<IModelTree> exportScopeElts = new ArrayList<IModelTree>();
        
            exportScopeElts.add(selectedPkg);
        
        
            partialMap.put(selectedPkg.getIdentifier(), ecoreModel);
        
            genProp.setExportScopeElts(exportScopeElts);
        
            XMIExportBehavior exportBehavior = new XMIExportBehavior(progressBar);
            GenericMetamodelVisitor visitObjingModel = new GenericMetamodelVisitor(exportBehavior);
            genProp.setObjingVisitor(visitObjingModel);
        
            boolean ecoreRootNull = true;
            for (IModelTree rootElt : exportScopeElts) {
                rootElt.accept(visitObjingModel);
        
                Namespace ecoreRoot = (Namespace) genProp
                        .getMappedElement(rootElt);
        
                if (ecoreRoot != null) {
                    ecoreRootNull = false;
                    if (!ecoreRoot.equals(ecoreModel)){
                        ecoreModel.getPackagedElements().add((PackageableElement)ecoreRoot);
                    }
        
                    for (PrimitiveType primitive : PrimitiveTypeMapper.getPredifinedTypeList()){
                        ecoreModel.getPackagedElements().add(primitive);
                    }
                }
            }
        
            if (!ecoreRootNull) {
                if(progressBar != null) 
                    progressBar.setLabel(XMI.I18N.getString("progressBar.content.export.XMIFileSave"));
        
                resource.getContents().add(ecoreModel);
        
                //define the profile structure
                profileDefinition();
        
                //save stereotype
                profileExport(progressBar);
        
                applyProfile(); 
        
                partialMap.clear();
                totalMap.clear();
        
                if(progressBar != null) progressBar.setLabel(XMI.I18N.getString("progressBar.content.export.XMIFileSave"));
        
                error = saveRessource(resource);
        
                if (error) {
                    Display.getDefault().asyncExec(new Runnable() {
                        public void run() {
                            MessageBox messageBox = new MessageBox(_shell, SWT.ICON_ERROR);
                            messageBox.setMessage(XMI.I18N.getString("info.export.result_failed.inSave"));
                            messageBox.setText(XMI.I18N.getString("info.export.result_failed"));
                            messageBox.open();
                            _shell.dispose();
                        }
                    });
        
        
                    LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("info.export.result_failed"));
        
                }
        
            } else {
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        MessageBox messageBox = new MessageBox(_shell, SWT.ICON_ERROR);
                        messageBox.setMessage(XMI.I18N.getString("info.export.result_failed"));
                        messageBox.setText(XMI.I18N.getString("info.export.result_failed.root_null"));
                        messageBox.open();
                        _shell.dispose();
                    }
                });
        
        
                LOG.error(XMI.PLUGIN_ID, XMI.I18N
                        .getString("info.export.result_failed.root_null"));
        
            }
        
            genProp.supressProfile();
            if (genProp.isSysMLApplied())
                genProp.supressSysMLProfile();
        }
        return error;
    }

    @objid ("03949e80-4583-11e0-b54c-00137279a832")
    private void profileExport(ProgressBarComposite progressBar) {
        for(Profile profile : GenerationProperties.getInstance().getExportedProfiles()){
            if (profile.getOwner() == null) { 
        
                if(progressBar != null) progressBar.setLabel(XMI.I18N.getMessage("progressBar.content.export.XMIProfileSave", profile.getName()));
        
                String extension = null;
        
                if (GenerationProperties.getInstance().getFileExtension().equals(XMIExtension.XMI))
                    extension = ".xmi";
                else
                    extension = ".uml";
        
                String profilePath = GenerationProperties.getInstance().getFileDirectory() + java.io.File.separator
                        +  profile.getName() +  ".profile" + extension;
        
                Resource resourceProfile = createResource(profilePath);
        
                resourceProfile.getContents().add(profile);
        
                for (Iterator<?> allContents = UMLUtil.getAllContents(profile, true, false); allContents.hasNext();) {
                    EObject eObject = (EObject) allContents.next();
                    if (eObject instanceof Element) {
                        resourceProfile.getContents().addAll(((Element) eObject).getStereotypeApplications());
                    }
                }
        
                try {
                    resourceProfile.save(null);
        
                    if (!GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300))
                        XMIFileUtils.changeToUML(resourceProfile.getURI().toFileString());
        
                } catch (IOException ioe) {
        
                    String errorMsg = ioe.getMessage();
                    if (errorMsg != null)
                        LOG.error(XMI.PLUGIN_ID, "Profile " + profile.getName() + " export failed. " + errorMsg);
                    else 
                        LOG.error(XMI.PLUGIN_ID, "Profile " + profile.getName() + " export failed. " );
                }
            }
        }
        
        
        if (GenerationProperties.getInstance().isSysMLApplied()){
        
            Profile profile = GenerationProperties.getInstance().getSysMLProfile();
        
            String profileName = profile.getName();
        
            if(progressBar != null) progressBar.setLabel(XMI.I18N.getMessage("progressBar.content.export.XMIProfileSave", profileName));
        
            String extension = null;
        
            if (GenerationProperties.getInstance().getFileExtension().equals(XMIExtension.XMI))
                extension = ".xmi";
            else
                extension = ".uml";
        
            String profilePath = GenerationProperties.getInstance().getFileDirectory() + java.io.File.separator
                    + profileName +  ".profile" + extension;
        
            Resource resourceProfile = createResource(profilePath);
        
            resourceProfile.getContents().add(profile);
        
            for (Iterator<?> allContents = UMLUtil.getAllContents(profile, true, false); allContents.hasNext();) {
                EObject eObject = (EObject) allContents.next();
                if (eObject instanceof Element) {
                    resourceProfile.getContents().addAll(((Element) eObject).getStereotypeApplications());
                }
            }
        
            try {
                resourceProfile.save(null);
        
                if (!GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300))
                    XMIFileUtils.changeToUML(resourceProfile.getURI().toFileString());
        
            } catch (IOException ioe) {
                String errorMsg = ioe.getMessage();
                if (errorMsg != null)
                    LOG.error(XMI.PLUGIN_ID, "Profile " + profile.getName() + " export failed. " + errorMsg);
                else 
                    LOG.error(XMI.PLUGIN_ID, "Profile " + profile.getName() + " export failed. " );
            }
        }
    }

    /**
     * Export the Modelio model into Ecore model
     * @param selectedPkg The root of the Modelio model
     * @param progressBar The progress bar of the XMI dialog
     * @return The Ecore model
     */
    @objid ("03949e88-4583-11e0-b54c-00137279a832")
    public Model createEcoreModel(IPackage selectedPkg, ProgressBarComposite progressBar) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        PartialExportMap partialMap = PartialExportMap.getInstance();
        TotalExportMap totalMap = TotalExportMap.getInstance();
        
        partialMap.clear();
        totalMap.clear();
        
        Model ecoreModel = UMLFactory.eINSTANCE.createModel();
        ObjingEAnnotation.setExporterVersion(ecoreModel, version);     
        ObjingEAnnotation.setRoundTrip(ecoreModel, genProp.isRoundtripEnabled());
        
        List<IModelTree> exportScopeElts = new ArrayList<IModelTree>();
        exportScopeElts.add(selectedPkg);
        genProp.setExportScopeElts(exportScopeElts);
        
        ecoreModel.setName(selectedPkg.getName());
        partialMap.put(selectedPkg.getIdentifier(), ecoreModel);
        
        genProp.setEcoreModel(ecoreModel);
        
        XMIExportBehavior exportBehavior = new XMIExportBehavior(progressBar);
        GenericMetamodelVisitor visitObjingModel = new GenericMetamodelVisitor(exportBehavior);
        genProp.setObjingVisitor(visitObjingModel);
        
        boolean ecoreRootNull = true;
        for (IModelTree rootElt : exportScopeElts) {
            rootElt.accept(visitObjingModel);
        
            Namespace ecoreRoot = (Namespace) genProp.getMappedElement(rootElt);
        
            if (ecoreRoot != null) {
                ecoreRootNull = false;
                if (!ecoreRoot.equals(ecoreModel))
                    ecoreModel.getPackagedElements().add((PackageableElement)ecoreRoot);
            }
        }
        
        if (ecoreRootNull) {
        
            LOG.error(XMI.PLUGIN_ID, XMI.I18N
                    .getString("info.export.result_failed.root_null"));
        
        }
        return ecoreModel;
    }

    /**
     * This method counts the number of ModelTree available inside a given one's
     * @param theElement The root ModelTree
     * @return
     * The number of sub ModelTree
     */
    @objid ("039700d8-4583-11e0-b54c-00137279a832")
    public int countModelTrees(IModelTree theElement) {
        int sum = 1;
        
        ObList<IModelTree> ownedElts = theElement.getOwnedElement();
        for (IModelTree elt : ownedElts) {
            sum += countModelTrees(elt);
        }
        return sum;
    }

    /**
     * Export a Profile in EMF format
     * @param resource The EMF resource which will be the profile
     * @param progressBar The progress bar of the XMI dialog
     * @return
     * true if the profile has been exported
     */
    @objid ("039700e5-4583-11e0-b54c-00137279a832")
    public boolean createEcoreProfile(Resource resource, ProgressBarComposite progressBar) {
        boolean error = false;
        
        if (resource != null) {
            GenerationProperties genProp = GenerationProperties.getInstance();
        
            IProfile rootProfile = (IProfile) genProp.getSelectedPackage();
        
            PartialExportMap partialMap = PartialExportMap.getInstance();
            TotalExportMap totalMap = TotalExportMap.getInstance();
            partialMap.clear();
            totalMap.clear();
        
            Profile ecoreProfile = ProfileUtils.createEcoreProfile(rootProfile);
        
            TotalExportMap.getInstance().put(rootProfile.getIdentifier(), ecoreProfile);
        
            CompositionOwnerVisitor ownerVisitor = new CompositionOwnerVisitor();
        
            ScopeChecker scopeChecker = new ScopeChecker(GenerationProperties
                    .getInstance().getSelectedPackage(), ownerVisitor);
            GenerationProperties.getInstance().setScopeChecker(scopeChecker);
        
            ProfileExportVisitorImpl profileVisitor = new ProfileExportVisitorImpl(progressBar);
            PExportProfile pprofile = new PExportProfile(rootProfile);
            profileVisitor.visit(pprofile);
        
            if(progressBar!= null) 
                progressBar.setLabel(XMI.I18N.getString("progressBar.content.export.XMIFileSave"));
            
            ObjingEAnnotation.setExporterVersion(ecoreProfile, version);  
            resource.getContents().add(ecoreProfile);
        
            try {
                resource.save(null);
            } catch (Exception ioe) {
                error = true;
                String errorMsg = ioe.getMessage();
                if (errorMsg != null)
                    LOG.error(XMI.PLUGIN_ID, errorMsg);
            }
        
        
            if (error) {
                LOG.error(XMI.PLUGIN_ID, XMI.I18N
                        .getString("info.export.result_failed"));
        
            } else {
                LOG.error(XMI.PLUGIN_ID, XMI.I18N
                        .getString("info.export.result_done"));
        
                if(progressBar!= null) progressBar.addFinalValue();
        
            }
        }
        return error;
    }

    @objid ("03996333-4583-11e0-b54c-00137279a832")
    private boolean saveRessource(Resource resource) {
        File file = new File(resource.getURI().toFileString());
        
        if (file.exists())
            file.delete();
        
        try {
            resource.save(null);
        } catch (IOException e) {
            String errorMsg = e.getMessage();
            if (errorMsg != null)
                LOG.error(XMI.PLUGIN_ID, errorMsg);
            return true;
        }
        return false;
    }

    @objid ("03996338-4583-11e0-b54c-00137279a832")
    private void exportStereotype(IModelElement element, Element ecoreElement) {
        // apply existing stereotypes 
        
        GenerationProperties genProp  = GenerationProperties.getInstance();
        
        for (IStereotype stereotype : element.getExtension()){
        
            Stereotype ecoreStereotype = (Stereotype) genProp.getMappedElement(stereotype);
        
            if ((ecoreStereotype != null) &&  (ecoreElement.isStereotypeApplicable(ecoreStereotype))
                    && (!(ecoreElement.getAppliedStereotypes().contains(ecoreStereotype)))){
        
                try{
                    ecoreElement.applyStereotype(ecoreStereotype);
        
                }catch(IllegalArgumentException e){
        
                    String errorMsg = e.getMessage();
                    if (errorMsg != null)
                        LOG.error(XMI.PLUGIN_ID, errorMsg);
                }
        
            }
        }
        
        // apply exiting metaclass references
        for(ITaggedValue taggedValue : element.getTag()){
        
            IMetaclassReference metaclassReference = taggedValue.getDefinition().getOwnerReference();
        
            if (metaclassReference != null){
        
                Stereotype ecoreStereotype = (Stereotype) genProp.getMappedElement(metaclassReference);
        
                if ((ecoreStereotype != null) 
                        &&  ecoreElement.isStereotypeApplicable(ecoreStereotype)
                        && (!ecoreElement.getAppliedStereotypes().contains(ecoreStereotype)))
                    ecoreElement.applyStereotype(ecoreStereotype);
            }
        }
    }

    @objid ("7f95f21a-1a74-11e1-8103-0027103f347d")
    private void applySysML() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        Profile sysMLProfile =  genProp.getSysMLProfile();
        
        for (IModelElement element :  genProp.getSysMLExported()){
        
            Element ecoreElement = genProp.getMappedElement(element);
        
            if  (ecoreElement != null){
                if (element instanceof IRequirement){
                    IRequirement requirement = (IRequirement) element;
                    Stereotype requirementSter = sysMLProfile.getOwnedStereotype("Requirement");
                    try{
                        ecoreElement.applyStereotype(requirementSter);
                        ecoreElement.setValue(requirementSter,     "Text", requirement.getText());
                        ecoreElement.setValue(requirementSter,     "Id", requirement.getName());
                    }catch(Exception e){
        
                        String errorMsg = e.getMessage();
                        if (errorMsg != null)
                            LOG.error(XMI.PLUGIN_ID, errorMsg);
                    }
                }else if (element instanceof IDependency){
                    IDependency dependency  = (IDependency) element;
                    try{
                        if (element.isStereotyped(ModelerModuleStereotypes.SATISFY)){
                            Stereotype satisfyStereotype = sysMLProfile.getOwnedStereotype("Satisfy");
                            genProp.getMappedElement(dependency).applyStereotype(satisfyStereotype);
                            setRequirementsRelated(dependency);
                        }else if (element.isStereotyped(ModelerModuleStereotypes.VERIFY)){ 
                            Stereotype verifyStereotype = sysMLProfile.getOwnedStereotype("Verify");
                            genProp.getMappedElement(dependency).applyStereotype(verifyStereotype);
                            setRequirementsRelated(dependency);
                        }else if (element.isStereotyped(ModelerModuleStereotypes.DERIVE)){ 
                            Stereotype deriveStereotype = sysMLProfile.getOwnedStereotype("DeriveReqt");
                            genProp.getMappedElement(dependency).applyStereotype(deriveStereotype);
                            setRequirementsRelated(dependency);
                        }
                    }catch(IllegalArgumentException e){
        
                        String errorMsg = e.getMessage();
                        if (errorMsg != null)
                            LOG.error(XMI.PLUGIN_ID, errorMsg);
                    }
                }else if (element instanceof INote){
                    INote note = (INote) element;
        
                    if (note.getModel().getName().equals("Rationale")){
                        Stereotype ratStereotype = sysMLProfile.getOwnedStereotype("Rationale");
                        genProp.getMappedElement(note).applyStereotype(ratStereotype);
                    }else if (note.getModel().getName().equals("Problem")){
                        Stereotype ratStereotype = sysMLProfile.getOwnedStereotype("Problem");
                        genProp.getMappedElement(note).applyStereotype(ratStereotype);
                    }
                }
            }
        }
        
        genProp.cleanSysMLElementExported();
    }

    @objid ("7f96192c-1a74-11e1-8103-0027103f347d")
    private void setRequirementsRelated(final IDependency element) {
        if (!(element.getDependsOn() instanceof IRequirement))
            setRequirementRelated( GenerationProperties.getInstance().getMappedElement(element.getDependsOn()));
        
        if (!(element.getImpacted() instanceof IRequirement))
            setRequirementRelated( GenerationProperties.getInstance().getMappedElement(element.getImpacted()));
    }

    @objid ("7f966750-1a74-11e1-8103-0027103f347d")
    private void setRequirementRelated(final Element mappedElement) {
        mappedElement.applyStereotype(GenerationProperties.getInstance().getSysMLProfile()
                .getOwnedStereotype("RequirementRelated"));
    }

    @objid ("7f96b574-1a74-11e1-8103-0027103f347d")
    private void profileDefinition() {
        for(Profile profile : GenerationProperties.getInstance().getExportedProfiles()){
            profile.define();
        }
        
        if (GenerationProperties.getInstance().isSysMLApplied())
            GenerationProperties.getInstance().getSysMLProfile().define();
    }

}
