/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.generation;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportAttribut;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportGeneralization;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportNoteType;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportProfile;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportReference;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportStereotype;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;

@objid ("05b8fb5a-4583-11e0-b54c-00137279a832")
public class ProfileExportVisitorImpl implements IProfileExportVisitor {
    @objid ("05b8fb5b-4583-11e0-b54c-00137279a832")
    private ProgressBarComposite theProgressBar = null;


    @objid ("05b8fb5c-4583-11e0-b54c-00137279a832")
    public ProfileExportVisitorImpl(ProgressBarComposite progressBar) {
        this.theProgressBar = progressBar;
    }

    @objid ("05b8fb5f-4583-11e0-b54c-00137279a832")
    public ProfileExportVisitorImpl() {
    }

    @objid ("05b8fb61-4583-11e0-b54c-00137279a832")
    public void visit(PExportProfile profile) {
        profile.visit();
                
        for (PExportProfile subProfile : profile.getSubProfiles()){
            subProfile.accept(this);
            if (this.theProgressBar != null){
                this.theProgressBar.addElement();    
            }
        }
        
        for (PExportProfile subProfile : ProfileUtils.getSubProfiles(profile)){
            subProfile.accept(this);
            if (this.theProgressBar != null){
                this.theProgressBar.addElement();    
            }
        }
                
        for (PExportStereotype subStereotype : profile.getStereotypes()){
            
            subStereotype.accept(this);
            if (this.theProgressBar != null){
                this.theProgressBar.addElement();    
                this.theProgressBar.addValue();
            }
        }
                
                
        for (PExportReference subreference : profile.getReferences()){
            subreference.accept(this);
            if (this.theProgressBar != null){
                this.theProgressBar.addElement();    
            }
        }
    }

    @objid ("05b8fb64-4583-11e0-b54c-00137279a832")
    public void visit(PExportStereotype stereotype) {
        stereotype.visit();
                
        IStereotype elt = ((IStereotype)stereotype.getElt());
                
        PExportGeneralization generalization = new PExportGeneralization(
                elt.getBaseClassName(), elt);
        
        generalization.accept(this);
                
        for (PExportAttribut attribut : stereotype.getAttribute())
            attribut.accept(this);
        
        for (PExportNoteType noteType : stereotype.getNoteTypes())
            noteType.accept(this);
    }

    @objid ("05bb5db4-4583-11e0-b54c-00137279a832")
    public void visit(PExportAttribut attribut) {
        attribut.visit();
    }

    @objid ("05bb5db7-4583-11e0-b54c-00137279a832")
    public void visit(PExportGeneralization generalization) {
        generalization.visit();
    }

    @objid ("05bb5dba-4583-11e0-b54c-00137279a832")
    public void visit(PExportReference reference) {
        reference.visit();
                
        for (PExportAttribut attribut : reference.getAttribute())
            attribut.accept(this);
        
        for (PExportNoteType noteType : reference.getNoteTypes())
            noteType.accept(this);
    }

    @objid ("05bb5dbd-4583-11e0-b54c-00137279a832")
    public void visit(PExportNoteType noteType) {
        noteType.visit();
    }

}
