/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.generation.ExportServices;
import com.modeliosoft.modelio.xmi.generation.PartialExportMap;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.gui.report.ReportManager;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.AbortProcessException;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.XMIFileUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Shell;

/**
 * The thread of XMI model export
 * @author ebrosse
 */
@objid ("0645a61b-4583-11e0-b54c-00137279a832")
public class ExportThread extends XMIThread implements IRunnableWithProgress {
    @objid ("0645a61c-4583-11e0-b54c-00137279a832")
    private ExportServices service = null;


    @Override
    @objid ("0645a61e-4583-11e0-b54c-00137279a832")
    public void run(IProgressMonitor localMonitor) {
        boolean error = false;
        String errorMessage = "";
        Resource resource = null;
        try {
        
            GenerationProperties genProp = GenerationProperties.getInstance();
        
            genProp.setReportModel(ReportManager.getNewReport());
        
            this.progressBar.setNumberElement(this.service
                    .countModelTrees(genProp.getSelectedPackage()) * 2);
            this.progressBar.setLabel(XMI.I18N
                    .getString("progressBar.content.export.XMIFileInit"));
        
            resource = this.service.createResource(genProp.getFilePath());
        
        
            error = this.service.createEcoreModel(resource, this.progressBar);
        
            if (!error){
                VersionExport versionExport = GenerationProperties.getInstance().getExportVersion();
                if (!versionExport.equals(VersionExport.EMF300))
                    XMIFileUtils.changeToUML(genProp.getFilePath());
        
                this.progressBar.addFinalValue();
                if (GenerationProperties.getInstance().getReportModel().isEmpty()){
                    completeBox();
                }
        
            }else{
                incompleteBox(errorMessage);
                this.progressBar.addFinalValue();
            }
        
        } catch (AbortProcessException e) {
            cancelation();
        } catch (Exception e) {
            errorMessage = e.getMessage();
            if (errorMessage == null){
                errorMessage = XMI.I18N.getString("fileChooser.dialog.export.errorMessage");
            }
            LOG.error(XMI.PLUGIN_ID, errorMessage);           
            incompleteBox(errorMessage);
            this.progressBar.addFinalValue();
        }finally {
            if (resource != null)
                resource.unload();
            TotalExportMap.getInstance().clear();
            PartialExportMap.getInstance().clear();
        
        }
    }

    /**
     * @param shell the current shell
     * @param progressBar the progress bar of the XMI dialog
     */
    @objid ("0645a622-4583-11e0-b54c-00137279a832")
    public ExportThread(Shell shell, ProgressBarComposite progressBar) {
        super();
        this.service = new ExportServices(shell);
        this.progressBar = progressBar;
        this.shell = shell;
    }

}
