/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.ImportServices;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.AbortProcessException;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

/**
 * This class is the Thread of Model import
 * @author ebrosse
 */
@objid ("065191e5-4583-11e0-b54c-00137279a832")
public class ImportThread extends XMIThread implements IRunnableWithProgress {
    @objid ("065191e6-4583-11e0-b54c-00137279a832")
    private ImportServices service = null;

    @objid ("0653f435-4583-11e0-b54c-00137279a832")
    private boolean error;


    @Override
    @objid ("0653f436-4583-11e0-b54c-00137279a832")
    public void run(IProgressMonitor localMonitor) {
        Resource resource = null;
        // loading resource
        try {
            this.progressBar.setLabel(XMI.I18N
                    .getString("progressBar.content.import.XMIFileLoading"));
        
            resource = this.service.getResource(new File(ReverseProperties.getInstance().getFilePath()));
        
            if (resource != null){
                this.error =  this.service.importEcoreModel(resource, this.progressBar, this.shell); 
            }else{
                this.error = true;
            }
        
            this.progressBar.addFinalValue();
        
            if (this.error){
                incompleteBox("");
            }else{
                completeBox();
            }
        
        } catch (AbortProcessException e) {
            cancelation();
            ReverseProperties.getInstance().setRollback(true);
        } catch (Exception e) {
            String  messageException = e.getMessage();  
            if (messageException != null)
                LOG.error(XMI.PLUGIN_ID, messageException);     
            
            this.error = true;
        
            catchException(e.getMessage(), XMI.I18N.getString("error.import.invalidModel"));
        
            if ((resource != null) && (resource.getURI().toFileString().endsWith(".emf"))){ 
                File file = new File(resource.getURI().toFileString());
                if (file.exists())
                    file.delete();
        
            }
        
            this.progressBar.addFinalValue();
        
        } finally {
            if (resource != null)
                resource.unload();
            TotalImportMap.getInstance().clear();
            PartialImportMap.getInstance().clear();
        }
    }

    /**
     * @param shell The curernt shell
     * @param progressBar The progress bar of the XMI dialog
     */
    @objid ("0653f439-4583-11e0-b54c-00137279a832")
    public ImportThread(Shell shell, ProgressBarComposite progressBar) {
        super();
        this.service = new ImportServices();
        this.progressBar = progressBar;
        this.shell = shell;
    }

    @objid ("0656568e-4583-11e0-b54c-00137279a832")
    private void catchException(final String messageException, final String titleException) {
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                MessageBox messageBox = new MessageBox(Display.getDefault().getActiveShell(), SWT.ICON_ERROR);
        
                if (messageException != null)
                    messageBox.setMessage(messageException);
        
                messageBox.setText(titleException);
                messageBox.open();
        
            }
        });
        
        if (messageException != null)
            LOG.error(XMI.PLUGIN_ID, messageException);     
        
        ReverseProperties.getInstance().setRollback(true);
    }

}
