/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.api.XMIExtension;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/**
 * This class creates central composite of the XMI export dialogs (model and profile exports).
 * It is composed of several SWT composites, each of them is dedicated to a specific export option
 * i.e. the file path, the extention of the file, the UML version, etc.
 * 
 * This is a SWT composite
 * @author ebrosse
 */
@objid ("0662425b-4583-11e0-b54c-00137279a832")
public class OptionComposite extends Composite {
    @objid ("aa2859d8-6a7c-11e0-957f-00137279a832")
    private Button compatibility = null;

    @objid ("aa2859d9-6a7c-11e0-957f-00137279a832")
    private Combo combo = null;

    @objid ("aa2859da-6a7c-11e0-957f-00137279a832")
    private Label label = null;

    @objid ("aa2859db-6a7c-11e0-957f-00137279a832")
    private Group groupOwner = null;

    @objid ("aa2859dc-6a7c-11e0-957f-00137279a832")
    private Group groupExtension = null;

    @objid ("aa2859dd-6a7c-11e0-957f-00137279a832")
    private Button xmi = null;

    @objid ("aa2859de-6a7c-11e0-957f-00137279a832")
    private Button uml = null;

    @objid ("aa2859df-6a7c-11e0-957f-00137279a832")
    private Group groupCompatibility = null;

    @objid ("9c249110-1a76-11e1-8103-0027103f347d")
    private VersionExport[] versions = VersionExport.values();


    /**
     * The constructor of the OptionComposite
     * @param parent : the parent composite
     * @param style : the SWT style of the returned composite
     */
    @objid ("0664a4b2-4583-11e0-b54c-00137279a832")
    public OptionComposite(final Composite parent, final int style) {
        super(parent, style);
        setLayout(new FormLayout());
        
        this.groupOwner = new Group(this, SWT.NONE);
        final FormData fd_group = new FormData();
        fd_group.bottom = new FormAttachment(100, 0);
        fd_group.top = new FormAttachment(0, 0);
        fd_group.right = new FormAttachment(100, 0);
        fd_group.left = new FormAttachment(0, 0);
        
        this.groupOwner.setLayoutData(fd_group);
        
        this.groupOwner.setText(XMI.I18N.getString("fileChooser.options.export.group.option.name"));
        
        this.groupCompatibility = new Group(this.groupOwner, SWT.NONE);
        this.groupCompatibility.setBounds(35, 20, 350, 80);
        this.groupCompatibility.setText(XMI.I18N.getString("fileChooser.options.export.group.compatibility.name"));
        
        this.compatibility = new Button(this.groupCompatibility, SWT.CHECK);
        this.compatibility.setBounds(30, 20, 220, 16);
        
        this.compatibility.setText(XMI.I18N.getString("fileChooser.options.export.enableRoundtrip.name"));
        
        boolean compatibilityMdacParameterValue = Boolean.valueOf(XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.CompatibilityActivated"));
        
        this.compatibility.setSelection(compatibilityMdacParameterValue);
        
        this.combo = new Combo(this.groupCompatibility, SWT.READ_ONLY);
        
        String currentExport = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.VersionExport");
        for (int i = 0; i < this.versions.length; i++) {
            this.combo.add(XMI.I18N.getString("Ui.Parameter.VersionExport." + this.versions[i].name()));
            if (this.versions[i].name().equals(currentExport))
                this.combo.select(i);
        
        }
        
        this.combo.setBounds(130, 50, 150, 13);
        
        this.label = new Label(this.groupCompatibility, SWT.WRAP);
        this.label.setText(XMI.I18N.getString("fileChooser.options.export.version.name") + " : ");
        this.label.setBounds(40, 50, 90, 13);
               
        this.groupExtension = new Group(this.groupOwner, SWT.NONE);
               
        this.groupExtension.setText(XMI.I18N.getString("fileChooser.options.export.group.extension.name"));
        
        this.groupExtension.setBounds(410, 20, 140, 80);
        this.xmi = new Button(this.groupExtension, SWT.RADIO);
        this.xmi.setBounds(40, 20, 100, 20);
        this.xmi.setText(".xmi");
        
        this.uml = new Button(this.groupExtension, SWT.RADIO);
        this.uml.setBounds(40, 50, 100, 20);
        this.uml.setText(".uml");
        
        String extension = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.FileExtension");
        
        if (extension.equals(XMIExtension.XMI.toString())) {
            this.xmi.setSelection(true);
        } else {
            this.uml.setSelection(true);
        }
        
        this.groupOwner.pack();
    }

    /**
     * This method returns the UML version specified
     * @return a UML version of the export
     */
    @objid ("0664a4b6-4583-11e0-b54c-00137279a832")
    public VersionExport getVersion() {
        return this.versions[this.combo.getSelectionIndex()];
    }

    /**
     * This method returns the UML radio button
     * @return SWT Button
     */
    @objid ("06670705-4583-11e0-b54c-00137279a832")
    public Button getUMLButton() {
        return this.uml;
    }

    /**
     * This method returns the XMI radio button
     * @return SWT Button
     */
    @objid ("06670709-4583-11e0-b54c-00137279a832")
    public Button getXMIButton() {
        return this.xmi;
    }

    /**
     * This method returns the specified file extension
     * @return the file extension .xmi or .uml
     */
    @objid ("0667070d-4583-11e0-b54c-00137279a832")
    public XMIExtension getExtension() {
        if (this.uml.getSelection())
            return XMIExtension.UML;
        else
            return XMIExtension.XMI;
    }

    /**
     * This method returns the compatibility checkbox
     * @return SWT CheckBox
     */
    @objid ("0669695e-4583-11e0-b54c-00137279a832")
    public Button getCompatibilityButton() {
        return this.compatibility;
    }

    /**
     * this method returns the label of the composite
     * @return SWT Label
     */
    @objid ("06696962-4583-11e0-b54c-00137279a832")
    public Label getLabelButton() {
        return this.label;
    }

    /**
     * This method returns the UML format ComboButton
     * @return the SWT Combo
     */
    @objid ("06696966-4583-11e0-b54c-00137279a832")
    public Combo getComboButton() {
        return this.combo;
    }

    /**
     * This methods updates the selection of .uml file extension
     */
    @objid ("0669696a-4583-11e0-b54c-00137279a832")
    public void setUMLSelected() {
        this.uml.setSelection(true);
        this.xmi.setSelection(false);
    }

    /**
     * This methods updates the selection of .xmi file extension
     */
    @objid ("066bcbb8-4583-11e0-b54c-00137279a832")
    public void setXMISelected() {
        this.xmi.setSelection(true);
        this.uml.setSelection(false);
    }

    /**
     * This method returns the availability of the annotation export
     * @return true if the annotation must be exported
     */
    @objid ("066bcbba-4583-11e0-b54c-00137279a832")
    public boolean isAnnoted() {
        return this.compatibility.getSelection();
    }

}
