/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.AbortProcessException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;

/**
 * SWT composite containing a SWT progress bar
 * @author ebrosse
 */
@objid ("066bcbbe-4583-11e0-b54c-00137279a832")
public class ProgressBarComposite extends Composite {
    @objid ("066bcbbf-4583-11e0-b54c-00137279a832")
    private int selection = 0;

    @objid ("066bcbc0-4583-11e0-b54c-00137279a832")
    private int total = 0;

    @objid ("066bcbc1-4583-11e0-b54c-00137279a832")
    private int max = 0;

    @objid ("066bcbc2-4583-11e0-b54c-00137279a832")
    private String title = XMI.I18N.getString("progressBar.title");

    @objid ("066bcbc3-4583-11e0-b54c-00137279a832")
    private int numberElement = 0;

    @objid ("066bcbc4-4583-11e0-b54c-00137279a832")
    private int incrementation = 0;

    @objid ("066bcbc5-4583-11e0-b54c-00137279a832")
    private int totalElement = 0;

    @objid ("066e2e14-4583-11e0-b54c-00137279a832")
    private String valuePlural = XMI.I18N.getString("progressBar.setValuePlural");

    @objid ("aa2f80e7-6a7c-11e0-957f-00137279a832")
    private ProgressBar progressBar;

    @objid ("aa2f80e8-6a7c-11e0-957f-00137279a832")
    private Label chargeInProgress;

    @objid ("aa2f80e9-6a7c-11e0-957f-00137279a832")
    private Group group;


    /**
     * @return the progress bar
     */
    @objid ("066e2e15-4583-11e0-b54c-00137279a832")
    public ProgressBar getProgressBar() {
        return this.progressBar;
    }

    /**
     * @return the progress bar label
     */
    @objid ("066e2e19-4583-11e0-b54c-00137279a832")
    public Label getLabelProgessBar() {
        return this.chargeInProgress;
    }

    /**
     * add one to the current number of treated element
     */
    @objid ("066e2e1d-4583-11e0-b54c-00137279a832")
    public void addValue() {
        if (SwtWizardWindow.isCancelation()) {
            abortProcess();
        }
        if (Display.getDefault().isDisposed())
            return;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (ProgressBarComposite.this.progressBar.isDisposed())
                    return;
                ProgressBarComposite.this.total++;
                if (ProgressBarComposite.this.incrementation == 0 || ProgressBarComposite.this.total % ProgressBarComposite.this.incrementation == 0) {
                    if (ProgressBarComposite.this.selection < ProgressBarComposite.this.max) {
                        ProgressBarComposite.this.selection++;
                        ProgressBarComposite.this.progressBar.setSelection(ProgressBarComposite.this.selection);
                    }
                }
            }
        });
    }

    /**
     * fullfill the progress bar
     */
    @objid ("066e2e1f-4583-11e0-b54c-00137279a832")
    public void addFinalValue() {
        if (Display.getDefault().isDisposed())
            return;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (ProgressBarComposite.this.progressBar.isDisposed())
                    return;
                ProgressBarComposite.this.progressBar.setSelection(ProgressBarComposite.this.max);
            }
        });
    }

    /**
     * @param value set the Progress bar label
     */
    @objid ("0670906c-4583-11e0-b54c-00137279a832")
    public void setLabel(final String value) {
        final String v = value;
        if (Display.getDefault().isDisposed())
            return;
        
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (ProgressBarComposite.this.progressBar.isDisposed())
                    return;
                setTextGroup(v, true);
            }
        });
    }

    /**
     * add one element
     */
    @objid ("0670906f-4583-11e0-b54c-00137279a832")
    public void addElement() {
        if (SwtWizardWindow.isCancelation()) {
            abortProcess();
        }
        this.totalElement++;
        if (Display.getDefault().isDisposed())
            return;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (ProgressBarComposite.this.progressBar.isDisposed())
                    return;
                if (ProgressBarComposite.this.totalElement > 1)
                    setTextGroup(String.valueOf(ProgressBarComposite.this.totalElement) + " " + ProgressBarComposite.this.valuePlural, true);
                else
                    setTextGroup(XMI.I18N.getMessage("progressBar.setValueSingular", String.valueOf(ProgressBarComposite.this.totalElement)), true);
            }
        });
    }

    /**
     * @return the current number of treated element
     */
    @objid ("06709071-4583-11e0-b54c-00137279a832")
    public int getNumberElement() {
        return this.numberElement;
    }

    /**
     * @param numberElement the number of treated element
     */
    @objid ("06709075-4583-11e0-b54c-00137279a832")
    public void setNumberElement(int numberElement) {
        this.numberElement = numberElement;
        this.incrementation = (numberElement) / this.max;
    }

    @objid ("06709078-4583-11e0-b54c-00137279a832")
    private void abortProcess() throws AbortProcessException {
        SwtWizardWindow.setCancellation(true);
        throw new AbortProcessException();
    }

    @objid ("0672f2c7-4583-11e0-b54c-00137279a832")
    void setTextGroup(String text, boolean inProgression) {
        if (!inProgression) {
            this.group.setText(" " + text + " ");
        } else {
            this.group.setText(" " + this.title + " : " + text + " ");
        }
    }

    @objid ("0672f2cb-4583-11e0-b54c-00137279a832")
    private void setTextGroup(String text) {
        setTextGroup(text, false);
    }

    /**
     * @param parent The parent Composite
     * @param style The SWT style
     */
    @objid ("0672f2ce-4583-11e0-b54c-00137279a832")
    public ProgressBarComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new FormLayout());
        this.group = new Group(this, SWT.NONE);
        this.group.setLayout(new FormLayout());
        final FormData fd_group = new FormData();
        fd_group.bottom = new FormAttachment(100, -5);
        fd_group.top = new FormAttachment(0, 0);
        fd_group.right = new FormAttachment(100, 0);
        fd_group.left = new FormAttachment(0, 0);
        
        this.group.setLayoutData(fd_group);
        setTextGroup(this.title);
        this.progressBar = new ProgressBar(this.group, SWT.NONE);
        final FormData fd_progressBar = new FormData();
        fd_progressBar.right = new FormAttachment(100, -5);
        fd_progressBar.left = new FormAttachment(0, 5);
        fd_progressBar.bottom = new FormAttachment(0, 25);
        fd_progressBar.top = new FormAttachment(0, 5);
        
        this.progressBar.setLayoutData(fd_progressBar);
        this.max = this.progressBar.getMaximum();
    }

    /**
     * Initialize value
     */
    @objid ("0672f2d2-4583-11e0-b54c-00137279a832")
    public void setInitialValue() {
        if (Display.getDefault().isDisposed())
            return;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (ProgressBarComposite.this.progressBar.isDisposed())
                    return;
                ProgressBarComposite.this.progressBar.setSelection(0);
            }
        });
    }

}
