/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.XMIExtension;
import com.modeliosoft.modelio.xmi.gui.report.ReportManager;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

/**
 * This class provides the XMI export dialog
 * @author ebrosse
 */
@objid ("068f8efe-4583-11e0-b54c-00137279a832")
public class SwtWizardExport extends SwtWizardWindow {

    @Override
    @objid ("068f8eff-4583-11e0-b54c-00137279a832")
    public void setLabels() {
        setTitle(XMI.I18N.getString("fileChooser.banner.export.title"));
        setDescription(XMI.I18N.getString("fileChooser.banner.export.description"));
        setFrametitle(XMI.I18N.getString("fileChooser.frame.export.title"));
        setCancelButton(XMI.I18N.getString("fileChooser.buttons.export.cancel.name"));
        setValidateButton(XMI.I18N.getString("fileChooser.buttons.export.export.name"));
    }

    @Override
    @objid ("068f8f02-4583-11e0-b54c-00137279a832")
    public void validationAction() {
        File theFile = getFileChooserComposite().getCurrentFile();
        GenerationProperties.getInstance().setFilePath(theFile);
        path = theFile.getParent();
        if (testExportFilePath(theFile)) {
        
            GenerationProperties.getInstance().setFileExtension((this.getOptionComposite()).getExtension());
        
            GenerationProperties.getInstance().setVersionExport((this.getOptionComposite()).getVersion());
        
            GenerationProperties.getInstance().setRoundtripEnabled(this.getOptionComposite().getCompatibilityButton().getSelection());
            try {
                XMILogs.getInstance().setLogFile(theFile.getAbsolutePath());
                IProgressService service = PlatformUI.getWorkbench().getProgressService();
                service.busyCursorWhile (new ExportThread(this.shell,
                                                          getTheProgressBar()));
        
                if (!GenerationProperties.getInstance().getReportModel().isEmpty()){
                    Display.getDefault().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            ReportManager.showGenerationReport(GenerationProperties.getInstance().getReportModel());
        
                        }
                    });
                    this.shell.dispose();
                }
        
        
            } catch (InterruptedException e) {
                LOG.error(XMI.PLUGIN_ID, e);
            } catch (InvocationTargetException e) {
                LOG.error(XMI.PLUGIN_ID, e);
            }
        }
    }

    @Override
    @objid ("068f8f05-4583-11e0-b54c-00137279a832")
    public void setPath() {
        if (path.equals(""))
            path = ResourceLoader.getInstance().getProjectRoot() + java.io.File.separator + "XMI";
        
        String extension = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.FileExtension");
        
        if (extension.equals(XMIExtension.UML.toString()))
            extension = ".uml";
        else
            extension = ".xmi";
        
        this.fileChooserComposite.getDialog().setFilterPath(path);
        this.fileChooserComposite.getDialog().setFileName(((IPackage) this.selectedElt).getName() + extension);
        path = checkAndReplaceEndPath(path);
        this.fileChooserComposite.setText(path + java.io.File.separator + ((IPackage) this.selectedElt).getName() + extension);
    }

    /**
     * @param parent : the parent shell
     * @param style : the SWT style
     */
    @objid ("0691f159-4583-11e0-b54c-00137279a832")
    public SwtWizardExport(final Shell parent, final int style) {
        super(parent, style);
        setSelectedType(SWT.SAVE);
    }

    /**
     * @param parent : the parent shell
     */
    @objid ("0691f15d-4583-11e0-b54c-00137279a832")
    public SwtWizardExport(final Shell parent) {
        this(parent, SWT.NONE);
        this.exportWindows = true;
    }

    @Override
    @objid ("0691f160-4583-11e0-b54c-00137279a832")
    public void setDefaultDialog() {
        this.fileChooserComposite.getDialog().setFilterNames(new String[] { "XMI Files", "UML Files" });
        this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.xmi", "*.uml" });
        
        String extension = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.FileExtension");
        
        if (extension.equals(XMIExtension.UML.toString()))
            this.fileChooserComposite.getDialog().setFilterIndex(1);
        
        // cards
        setPath();
    }

    @Override
    @objid ("0691f162-4583-11e0-b54c-00137279a832")
    public void setOptionComposite(Shell shell) {
        this.optionComposite = new OptionComposite(shell, SWT.NONE);
    }

}
