/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.XMIExtension;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

/**
 * This class provides the XMI export profile dialog
 * @author ebrosse
 */
@objid ("069453ba-4583-11e0-b54c-00137279a832")
public class SwtWizardExportProfile extends SwtWizardWindow {

    /**
     * @param parent : shell parent
     * @param style : SWT style
     */
    @objid ("069453bb-4583-11e0-b54c-00137279a832")
    public SwtWizardExportProfile(final Shell parent, final int style) {
        super(parent, style);
        setSelectedType(SWT.NONE);
        this.exportWindows = true;
    }

    /**
     * @param parent : shell parent
     */
    @objid ("0696b60e-4583-11e0-b54c-00137279a832")
    public SwtWizardExportProfile(final Shell parent) {
        this(parent, SWT.NONE);
    }

    @Override
    @objid ("0696b611-4583-11e0-b54c-00137279a832")
    public void setLabels() {
        setTitle(XMI.I18N.getString("fileChooser.banner.exportProfile.title"));
        setDescription(XMI.I18N.getString("fileChooser.banner.export.description"));
        setFrametitle(XMI.I18N.getString("fileChooser.frame.exportProfile.title"));
        setCancelButton(XMI.I18N.getString("fileChooser.buttons.export.cancel.name"));
        setValidateButton(XMI.I18N.getString("fileChooser.buttons.export.export.name"));
    }

    @Override
    @objid ("0696b614-4583-11e0-b54c-00137279a832")
    public void validationAction() {
        final File theFile = getFileChooserComposite().getCurrentFile();
        GenerationProperties.getInstance().setFilePath(theFile);
        path = theFile.getParent();
        if (testExportFilePath(theFile)) {
        
            GenerationProperties.getInstance().setFileExtension((this.getOptionComposite()).getExtension());
        
            GenerationProperties.getInstance().setVersionExport((this.getOptionComposite()).getVersion());
        
            GenerationProperties.getInstance().setRoundtripEnabled(this.getOptionComposite().getCompatibilityButton().getSelection());
            try {  
                XMILogs.getInstance().setLogFile(theFile.getAbsolutePath());
                final IProgressService service = PlatformUI.getWorkbench().getProgressService();
                service.busyCursorWhile (new ExportProfileThread(this.shell,
                                                                 getTheProgressBar()));
            } catch (final InterruptedException e) {
                LOG.error(XMI.PLUGIN_ID, e);
            } catch (final InvocationTargetException e) {
                LOG.error(XMI.PLUGIN_ID, e);
            }
        
        }
    }

    @Override
    @objid ("0696b617-4583-11e0-b54c-00137279a832")
    public void setOptionComposite(final Shell shell) {
        this.optionComposite = new OptionComposite(shell, SWT.NONE);
    }

    @Override
    @objid ("06991869-4583-11e0-b54c-00137279a832")
    public void setPath() {
        if (path.equals(""))
            path = ResourceLoader.getInstance().getProjectRoot() + java.io.File.separator + "XMI";
        
        String extension = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.FileExtension");
        
        if (extension.equals(XMIExtension.UML))
            extension = ".uml";
        else
            extension = ".xmi";
        
        this.fileChooserComposite.getDialog().setFilterPath(path);
        this.fileChooserComposite.getDialog().setFileName(((IProfile) this.selectedElt).getName() + extension);
        path = checkAndReplaceEndPath(path);
        this.fileChooserComposite.setText(path + java.io.File.separator + ((IProfile) this.selectedElt).getName() + extension);
    }

    @Override
    @objid ("0699186c-4583-11e0-b54c-00137279a832")
    public void setDefaultDialog() {
        this.fileChooserComposite.getDialog().setFilterNames(new String[] { "XMI Files", "UML Files" });
        this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.xmi", "*.uml" });
        setPath();
    }

}
