/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import com.modeliosoft.modelio.api.model.InvalidTransactionException;
import com.modeliosoft.modelio.core.jni.ITransaction;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.core.transaction.ITransactionManager;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

/**
 * This class provides the XMI import dialog
 * @author ebrosse
 */
@objid ("069b7ac0-4583-11e0-b54c-00137279a832")
public class SwtWizardImport extends SwtWizardWindow {

    @Override
    @objid ("069b7ac1-4583-11e0-b54c-00137279a832")
    public void validationAction() {
        final File theFile = getFileChooserComposite().getCurrentFile();
        ReverseProperties.getInstance().setFilePath(theFile);
        path = theFile.getParent();
        if (theFile.exists() && theFile.isFile()) {
            String extension = theFile.getName();
            extension = extension.substring(extension.lastIndexOf("."));
            if (extension.equals(".uml") || extension.equals(".xmi") || extension.equals(".xml")) {
                initTransaction();
                try {
                    XMILogs.getInstance().setLogFile(theFile.getAbsolutePath());
                    IProgressService service = PlatformUI.getWorkbench().getProgressService();
                    service.busyCursorWhile (new ImportThread(this.shell,
                            getTheProgressBar() ));
                } catch (final Exception e) {
                    this.error = true;
                    catchException(e, XMI.I18N.getString("error.import.uncatchedException"));
                } finally {
                    commitTransaction();
                }
            } else {
                Display.getDefault().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        final MessageBox messageBox = new MessageBox(SwtWizardImport.this.shell, SWT.ICON_INFORMATION);
                        messageBox.setText(XMI.I18N.getString("fileChooser.dialog.wrongEcoreFormat.title"));
                        messageBox.setMessage(XMI.I18N.getString("fileChooser.dialog.wrongEcoreFormat.description"));
                        messageBox.open();
                    }
                });
            }
        } else {
            fileDontExist();
        }
        enableComposites();
    }

    @Override
    @objid ("069b7ac4-4583-11e0-b54c-00137279a832")
    public void setLabels() {
        setTitle(XMI.I18N.getString("fileChooser.banner.import.title"));
        setDescription(XMI.I18N.getString("fileChooser.banner.import.description"));
        setFrametitle(XMI.I18N.getString("fileChooser.frame.import.title"));
        setCancelButton(XMI.I18N.getString("fileChooser.buttons.import.cancel.name"));
        setValidateButton(XMI.I18N.getString("fileChooser.buttons.import.import.name"));
    }

    @Override
    @objid ("069b7ac7-4583-11e0-b54c-00137279a832")
    public void setPath() {
        try {         
            if (path.equals(""))
                path = ResourceLoader.getInstance().getProjectRoot() + java.io.File.separator + "XMI";
        
            this.fileChooserComposite.getDialog().setFilterPath(path);       
            this.fileChooserComposite.getDialog().setFileName("");      
            this.fileChooserComposite.setText(path);
        } catch (final NoClassDefFoundError e) {       
            this.fileChooserComposite.setText("");
        }
    }

    @Override
    @objid ("069b7aca-4583-11e0-b54c-00137279a832")
    public void setDefaultDialog() {
        this.fileChooserComposite.getDialog().setFilterNames(new String[] {"All Files (*.xmi; *.uml; *.xml)", "XMI Files (*.xmi)", "UML Files (*.uml)", "XML Files (*.xml)" });
        this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.xmi; *.uml; *.xml", "*.xmi", "*.uml", "*.xml" }); 
        
        setPath();
    }

    /**
     * @param parent : the parent shell
     * @param style : the SWT style
     */
    @objid ("069ddd1b-4583-11e0-b54c-00137279a832")
    public SwtWizardImport(final Shell parent, final int style) {
        super(parent, style);
        setSelectedType(SWT.OPEN);
        this.exportWindows = false;
    }

    /**
     * @param parent : the parent shell
     */
    @objid ("069ddd1f-4583-11e0-b54c-00137279a832")
    public SwtWizardImport(final Shell parent) {
        this(parent, SWT.NONE);
    }

    @Override
    @objid ("06a03f75-4583-11e0-b54c-00137279a832")
    public void setOptionComposite(final Shell shell) {
        this.optionComposite = null;
    }

    @objid ("06a03f79-4583-11e0-b54c-00137279a832")
    private void initTransaction() {
        final ITransaction t = XMI.getInstance().getModelingSession().getTransactionManager().createTransaction("Create-update objing model from XMI");
        
        ReverseProperties.getInstance().setOpenedTransaction(t);
    }

    @objid ("06a03f7b-4583-11e0-b54c-00137279a832")
    private void commitTransaction() {
        ITransaction t = ReverseProperties.getInstance().getOpenedTransaction();
        ITransactionManager tm = XMI.getInstance().getModelingSession().getTransactionManager();
        
        if ((this.error) ){
            tm.rollback(t);
            t = null;
        }else{
        
            if (ReverseProperties.getInstance().isRollback()){
                tm.rollback(t);
                t = null;
                ReverseProperties.getInstance().setRollback(false);
        
            }else{
                try{
                    tm.commit(t);
                }catch(InvalidTransactionException e){
                    tm.rollback(t);
                    t = null;
                }
            }
        }
    }

    @objid ("06a03f7d-4583-11e0-b54c-00137279a832")
    private void catchException(final Exception e, final String title) {
        LOG.error(XMI.PLUGIN_ID, e);
        
        final String text = e.getStackTrace()[0].toString();
        final String msgTitle = title;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
        
                final MessageBox messageBox = new MessageBox(SwtWizardImport.this.shell, SWT.ICON_ERROR);
        
                if (text != null)
                    messageBox.setMessage(text);
        
                messageBox.setText(msgTitle);
                messageBox.open();        
            }
        });
    }

}
