/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import com.modeliosoft.modelio.api.model.InvalidTransactionException;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.jni.ITransaction;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.core.transaction.ITransactionManager;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

/**
 * This class provides the XMI import profile dialog
 * @author ebrosse
 */
@objid ("06a2a1ce-4583-11e0-b54c-00137279a832")
public class SwtWizardImportProfile extends SwtWizardWindow {

    /**
     * @param parent : the parent shell
     * @param style : the SWT style
     */
    @objid ("06a2a1cf-4583-11e0-b54c-00137279a832")
    public SwtWizardImportProfile(final Shell parent, final int style) {
        super(parent, style);
        setSelectedType(SWT.OPEN);
        this.exportWindows = false;
    }

    /**
     * @param parent : the parent shell
     */
    @objid ("06a2a1d4-4583-11e0-b54c-00137279a832")
    public SwtWizardImportProfile(final Shell parent) {
        this(parent, SWT.NONE);
    }

    @Override
    @objid ("06a2a1d8-4583-11e0-b54c-00137279a832")
    public void validationAction() {
        final File theFile = getFileChooserComposite().getCurrentFile();
        ReverseProperties.getInstance().setFilePath(theFile);
        path = theFile.getParent();
        ITransactionManager tm = null;
        ITransaction t = null;
        IModelingSession session = null;
        if (theFile.exists() && theFile.isFile()) {
            session = XMI.getInstance().getModelingSession();
            tm = session.getTransactionManager();
            t = tm.createTransaction("Create-update Profile");
            final ReverseProperties revProp = ReverseProperties.getInstance();
            try {
        
                revProp.setOpenedTransaction(t);
                XMILogs.getInstance().setLogFile(theFile.getAbsolutePath());
                IProgressService service = PlatformUI.getWorkbench().getProgressService();
                service.busyCursorWhile (new ImportProfileThread(this.shell,
                                                                 getTheProgressBar(), getSelectedPkg()));
            } catch (final InvalidTransactionException e) {
                this.error = true;
                LOG.error(XMI.PLUGIN_ID, "InvalidTransactionException : ");
                LOG.error(XMI.PLUGIN_ID, e);
        
            } catch (final Exception e) {
                this.error = true;
                LOG.error(XMI.PLUGIN_ID, "Exception : ");
                LOG.error(XMI.PLUGIN_ID, e);
            } finally {
        
                t = revProp.getOpenedTransaction();
        
                if (this.error) {
                    tm.rollback(t);
                    t = null;
                } else {
                    if (revProp.isRollback()) {
                        tm.rollback(t);
                        t = null;
                        revProp.setRollback(false);
                    } else {
                        tm.commit(t);
                    }
                }
            }
        
        } else {
            fileDontExist();
        }
        enableComposites();
    }

    @Override
    @objid ("06a50429-4583-11e0-b54c-00137279a832")
    public void setLabels() {
        setTitle(XMI.I18N.getString("fileChooser.banner.importProfile.title"));
        setDescription(XMI.I18N.getString("fileChooser.banner.import.description"));
        setFrametitle(XMI.I18N.getString("fileChooser.frame.importProfile.title"));
        setCancelButton(XMI.I18N.getString("fileChooser.buttons.import.cancel.name"));
        setValidateButton(XMI.I18N.getString("fileChooser.buttons.import.import.name"));
    }

    @Override
    @objid ("06a5042c-4583-11e0-b54c-00137279a832")
    public void setOptionComposite(final Shell shell) {
        this.optionComposite = null;
    }

    @Override
    @objid ("06a50430-4583-11e0-b54c-00137279a832")
    public void setDefaultDialog() {
        this.fileChooserComposite.getDialog().setFilterNames(new String[] {"All Files (*.xmi; *.uml; *.xml)", "XMI Files (*.xmi)", "UML Files (*.uml)", "XML Files (*.xml)" });
        this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.xmi; *.uml; *.xml", "*.xmi", "*.uml", "*.xml" }); 
        
        setPath();
    }

    @Override
    @objid ("06a50432-4583-11e0-b54c-00137279a832")
    public void setPath() {
        String importPath = XMI.getInstance().getModelingSession().getProjectPropertiesService().readProperty("Modelio", "Ui.Parameter.ImportPath");
        if ((importPath == null) || (importPath.equals("")))
            importPath = ResourceLoader.getInstance().getProjectRoot() + java.io.File.separator + "XMI";
        
        this.fileChooserComposite.getDialog().setFilterPath(importPath);
        this.fileChooserComposite.getDialog().setFileName("");
        this.fileChooserComposite.setText(importPath);
    }

}
