/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import java.io.File;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.BareBonesBrowserLaunch;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

/**
 * @author ebrosse
 */
@objid ("06a76683-4583-11e0-b54c-00137279a832")
public abstract class SwtWizardWindow extends Dialog {
    @objid ("06a76684-4583-11e0-b54c-00137279a832")
    private String title = "";

    @objid ("06a76685-4583-11e0-b54c-00137279a832")
    private String description = "";

    @objid ("06a76686-4583-11e0-b54c-00137279a832")
    private String frametitle = "";

    @objid ("06a76687-4583-11e0-b54c-00137279a832")
    private String cancelButton = "";

    @objid ("06a76688-4583-11e0-b54c-00137279a832")
    private String validateButton = "";

    @objid ("06a76689-4583-11e0-b54c-00137279a832")
    private int selectedType = SWT.OPEN;

    @objid ("06a7668a-4583-11e0-b54c-00137279a832")
    private static boolean cancelation = false;

    @objid ("06a7668d-4583-11e0-b54c-00137279a832")
    protected FileChooserComposite fileChooserComposite = null;

    @objid ("06a7668e-4583-11e0-b54c-00137279a832")
    protected OptionComposite optionComposite = null;

    @objid ("06a7668f-4583-11e0-b54c-00137279a832")
    private ValidationBoutonComposite validateComposite = null;

    @objid ("06a76690-4583-11e0-b54c-00137279a832")
    protected ProgressBarComposite theProgressBar = null;

    @objid ("06a76691-4583-11e0-b54c-00137279a832")
    protected XMIThread theThread = null;

    @objid ("06a76692-4583-11e0-b54c-00137279a832")
    protected boolean exportWindows = false;

    @objid ("06a76693-4583-11e0-b54c-00137279a832")
    protected boolean error = false;

    @objid ("6c401e94-1a76-11e1-8103-0027103f347d")
    protected static String path = "";

    @objid ("0036bce4-e405-1fbc-8d66-000bdb5cdcea")
    protected IElement selectedElt = null;

    @objid ("808766d0-a9a8-11e1-9a8c-0027103f347d")
    protected Shell shell = null;


    /**
     * @return nothing
     */
    @objid ("06a9c8dd-4583-11e0-b54c-00137279a832")
    public Object open() {
        setCancellation(false);
        createContents();
        Display display = getParent().getDisplay();
        centerOnPrimaryScreen(display);
        this.shell.open();
        this.shell.layout();
        
        while (!this.shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        return null;
    }

    @objid ("06a9c8e1-4583-11e0-b54c-00137279a832")
    protected void createContents() {
        setLabels();
        
        this.shell = new Shell(getParent(), 67696 | SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.TITLE);
        this.shell.setLayout(new FormLayout());
        this.shell.setText(this.frametitle);
        
        this.shell.setImage(ResourceLoader.getInstance().getImage("wobjing16.bmp"));
        
        // Header composite
        final Composite headerComposite = new BannerComposite(this.shell, SWT.NONE, this.title, this.description);
        headerComposite.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW));
        
        // File chooser composite
        this.fileChooserComposite = new FileChooserComposite(this.shell, SWT.NONE, this.selectedType);
        
        // Option Composite
        setOptionComposite(this.shell);
        
        // Progress Bar Composite
        this.theProgressBar = new ProgressBarComposite(this.shell, SWT.NONE);
        
        // Validation Composite
        this. validateComposite = new ValidationBoutonComposite(this.shell, SWT.NONE, this.cancelButton, this.validateButton);
        
        this.validateComposite.getValidationButton().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableComposites();
        
                if (getFileChooserComposite().getCurrentFile() != null) {
                    validationAction();
                } else {
                    selectAFile();  
                }
                enableComposites();
            }
        });
        
        this.validateComposite.getCancelButton().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                setCancellation(true);
                cancelAction();
            }
        });
        
        final FormData fd_headerComposite = new FormData();
        fd_headerComposite.right = new FormAttachment(100, 0);
        fd_headerComposite.bottom = new FormAttachment(0, 50);
        fd_headerComposite.top = new FormAttachment(0, 0);
        fd_headerComposite.left = new FormAttachment(0, 0);
        headerComposite.setLayoutData(fd_headerComposite);
        
        final FormData fd_fileChooserComposite = new FormData();
        fd_fileChooserComposite.top = new FormAttachment(headerComposite, 5);
        fd_fileChooserComposite.right = new FormAttachment(headerComposite, 0, SWT.RIGHT);
        fd_fileChooserComposite.left = new FormAttachment(headerComposite, 0, SWT.LEFT);
        this.fileChooserComposite.setLayoutData(fd_fileChooserComposite);
        
        if (this.exportWindows) {
            final FormData fd_optionComposite = new FormData();
            fd_optionComposite.top = new FormAttachment(this.fileChooserComposite, 0);
            fd_optionComposite.left = new FormAttachment(this.fileChooserComposite, 5, SWT.LEFT);
            fd_optionComposite.right = new FormAttachment(this.fileChooserComposite, -5, SWT.RIGHT);
            this.optionComposite.setLayoutData(fd_optionComposite);
        
            this.optionComposite.getUMLButton().addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    if (SwtWizardWindow.this.optionComposite.getUMLButton().getSelection()) {
                        setUMLExtension();
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFilterNames(new String[] { "UML Files", "XMI Files" });
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.uml", "*.xmi" });
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFileName(((IPackage) SwtWizardWindow.this.selectedElt).getName() + ".uml");
                    }
                }
            });
        
            this.optionComposite.getUMLButton().setToolTipText(XMI.I18N.getString("fileChooser.options.export.description.uml"));
        
            this.optionComposite.getCompatibilityButton().setToolTipText(XMI.I18N.getString("fileChooser.options.export.description.annotation"));
        
            this.optionComposite.getComboButton().setToolTipText(XMI.I18N.getString("fileChooser.options.export.description.compatibility"));
        
            this.optionComposite.getLabelButton().setToolTipText(XMI.I18N.getString("fileChooser.options.export.description.compatibility"));
        
        
            this.optionComposite.getXMIButton().addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    if (SwtWizardWindow.this.optionComposite.getXMIButton().getSelection()) {
                        setXMIExtension();
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFilterNames(new String[] { "XMI Files", "UML Files" });
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFilterExtensions(new String[] { "*.xmi", "*.uml" });
                        SwtWizardWindow.this.fileChooserComposite.getDialog().setFileName(((IPackage) SwtWizardWindow.this.selectedElt).getName() + ".xmi");
                    }
                }
        
            });
        
            this.optionComposite.getXMIButton().setToolTipText(XMI.I18N.getString("fileChooser.options.export.description.xmi"));
        
        
        }
        
        this.fileChooserComposite.getSearch().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                checkPath(SwtWizardWindow.this.fileChooserComposite.searchFile());
            }
        });
        
        this.fileChooserComposite.getTextButton().addMouseTrackListener(new MouseTrackAdapter() {
            @Override
            public void mouseExit(MouseEvent e) {      
                checkPath(SwtWizardWindow.this.fileChooserComposite.getText());
            }
        });
        
        this.fileChooserComposite.getTextButton().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                checkPath(SwtWizardWindow.this.fileChooserComposite.getText());
            }
        });
        
        final FormData fd_laProgressBar = new FormData();
        
        if (this.exportWindows) {
            fd_laProgressBar.top = new FormAttachment(this.optionComposite, 0);
        } else {
            fd_laProgressBar.top = new FormAttachment(this.fileChooserComposite, 0);
        }
        fd_laProgressBar.left = new FormAttachment(this.fileChooserComposite, 5, SWT.LEFT);
        fd_laProgressBar.right = new FormAttachment(this.fileChooserComposite, -5, SWT.RIGHT);
        this.theProgressBar.setLayoutData(fd_laProgressBar);
        
        final FormData fd_validateComposite = new FormData();
        fd_validateComposite.top = new FormAttachment(this.theProgressBar, 5);
        fd_validateComposite.bottom = new FormAttachment(100, -5);
        fd_validateComposite.left = new FormAttachment(this.theProgressBar, 0, SWT.LEFT);
        fd_validateComposite.right = new FormAttachment(this.theProgressBar, 0, SWT.RIGHT);
        this.validateComposite.setLayoutData(fd_validateComposite);
        
        setDefaultDialog();
        this.shell.pack();
        this.shell.setMinimumSize(new Point(600, this.shell.getBounds().height));
        
        this.validateComposite.getValidationButton().setFocus();
    }

    /**
     * set default configuration of the dialog box
     */
    @objid ("06a9c8e3-4583-11e0-b54c-00137279a832")
    public abstract void setDefaultDialog();

    /**
     * @return the selected element i.e. IPackage or IModule
     */
    @objid ("06a9c8e4-4583-11e0-b54c-00137279a832")
    public IElement getSelectedPkg() {
        return this.selectedElt;
    }

    @objid ("06a9c8e8-4583-11e0-b54c-00137279a832")
    void cancelAction() {
        if (this.theThread != null && this.theThread.isAlive()) {
            this.theThread.interrupt();
        }
        
        
        if ((this.shell != null) && (!this.shell.isDisposed()))
                this.shell.dispose();
    }

    @objid ("06a9c8ea-4583-11e0-b54c-00137279a832")
    void disableComposites() {
        enableOrDisableCompistes(false);
    }

    /**
     * enable all composites
     */
    @objid ("06ac2b36-4583-11e0-b54c-00137279a832")
    public void enableComposites() {
        enableOrDisableCompistes(true);
    }

    @objid ("06ac2b38-4583-11e0-b54c-00137279a832")
    private void enableOrDisableCompistes(boolean isEnable) {
        if (!this.shell.isDisposed()) {
            if (this.fileChooserComposite != null)
                this.fileChooserComposite.setEnabled(isEnable);
            if (this.optionComposite != null) {
                this.optionComposite.setEnabled(isEnable);
            }
            this.validateComposite.getValidationButton().setEnabled(isEnable);
        
            //                        this.validateComposite.getCancelButton().setEnabled(isEnable);
        }
    }

    /**
     * @param cancelButton : the text of the cancel button
     */
    @objid ("06ac2b3b-4583-11e0-b54c-00137279a832")
    public void setCancelButton(final String cancelButton) {
        this.cancelButton = cancelButton;
    }

    /**
     * @param description : the description of the windows
     */
    @objid ("06ac2b3e-4583-11e0-b54c-00137279a832")
    public void setDescription(final String description) {
        this.description = description;
    }

    /**
     * @param frametitle : the title of the windows frame
     */
    @objid ("06ac2b41-4583-11e0-b54c-00137279a832")
    public void setFrametitle(final String frametitle) {
        this.frametitle = frametitle;
    }

    /**
     * @param title : the title of the windows
     */
    @objid ("06ac2b44-4583-11e0-b54c-00137279a832")
    public void setTitle(final String title) {
        this.title = title;
    }

    /**
     * @param validateButton : the button of validation
     */
    @objid ("06ae8d91-4583-11e0-b54c-00137279a832")
    public void setValidateButton(final String validateButton) {
        this.validateButton = validateButton;
    }

    @objid ("06ae8d94-4583-11e0-b54c-00137279a832")
    protected FileChooserComposite getFileChooserComposite() {
        return this.fileChooserComposite;
    }

    @objid ("06ae8d98-4583-11e0-b54c-00137279a832")
    protected ProgressBarComposite getTheProgressBar() {
        return this.theProgressBar;
    }

    @objid ("06ae8d9c-4583-11e0-b54c-00137279a832")
    protected void setSelectedType(int selectedType) {
        this.selectedType = selectedType;
    }

    /**
     * @return the optionComposite
     */
    @objid ("06b0efea-4583-11e0-b54c-00137279a832")
    public OptionComposite getOptionComposite() {
        return this.optionComposite;
    }

    /**
     * Warning user that he does not select a file
     */
    @objid ("06b0efee-4583-11e0-b54c-00137279a832")
    public void selectAFile() {
        MessageBox messageBox = new MessageBox(this.shell, SWT.ICON_WARNING);
        messageBox.setMessage(this.description);
        messageBox.open();
    }

    /**
     * @return true if the process is cancelled by the user
     */
    @objid ("06b0eff0-4583-11e0-b54c-00137279a832")
    public static boolean isCancelation() {
        return SwtWizardWindow.cancelation;
    }

    /**
     * @param cancelation : set the cancellation of the process
     */
    @objid ("06b0eff4-4583-11e0-b54c-00137279a832")
    public static void setCancellation(final boolean cancelation) {
        SwtWizardWindow.cancelation = cancelation;
    }

    /**
     * the action i.e. import or export
     */
    @objid ("06b35244-4583-11e0-b54c-00137279a832")
    public abstract void validationAction();

    /**
     * set button labels
     */
    @objid ("06b35245-4583-11e0-b54c-00137279a832")
    public abstract void setLabels();

    /**
     * initialize file path
     */
    @objid ("06b35246-4583-11e0-b54c-00137279a832")
    public abstract void setPath();

    /**
     * launch a dialog box for wrong path
     */
    @objid ("06b35247-4583-11e0-b54c-00137279a832")
    public void fileDontExist() {
        MessageBox messageBox = new MessageBox(this.shell, SWT.ICON_WARNING);
        messageBox.setMessage(XMI.I18N.getString("fileChooser.banner.import.dontexit"));
        messageBox.open();
    }

    @objid ("06b35249-4583-11e0-b54c-00137279a832")
    void helpPressed() {
        String urlPath = "http://127.0.0.1:1697/help/index.jsp";
        BareBonesBrowserLaunch.openURL(urlPath, this.shell);
    }

    /**
     * @param parent : the shell parent
     * @param style : the swt style
     */
    @objid ("06b3524b-4583-11e0-b54c-00137279a832")
    public SwtWizardWindow(final Shell parent, final int style) {
        super(parent != null ? parent : new Shell(Display.getDefault()), style);
    }

    /**
     * @param parent : the shell parent
     */
    @objid ("06b5b49e-4583-11e0-b54c-00137279a832")
    public SwtWizardWindow(final Shell parent) {
        this(parent, SWT.NONE);
    }

    /**
     * @param selectedElt : set the selected element
     */
    @objid ("06b5b4a1-4583-11e0-b54c-00137279a832")
    public void setSelectedElt(final IElement selectedElt) {
        this.selectedElt = selectedElt;
    }

    /**
     * @param shell : the current shell
     */
    @objid ("06b5b4a4-4583-11e0-b54c-00137279a832")
    public abstract void setOptionComposite(Shell shell);

    @objid ("06b5b4a6-4583-11e0-b54c-00137279a832")
    private void centerOnPrimaryScreen(Display display) {
        Monitor primary = display.getPrimaryMonitor();
        Rectangle bounds = primary.getBounds();
        Rectangle rect = this.shell.getBounds();
        int x = bounds.x + (bounds.width - rect.width) / 2;
        int y = bounds.y + (bounds.height - rect.height) / 2;
        this.shell.setLocation(x, y);
        this.shell.open();
    }

    @objid ("06b5b4aa-4583-11e0-b54c-00137279a832")
    void checkPath(String filePath) {
        if (this.exportWindows) {
            if ((filePath != null) && (!filePath.equals(""))) {
                int length = filePath.length();
                if ((length - 4) == filePath.lastIndexOf(".uml")) {
                    this.optionComposite.setUMLSelected();
                } else if (filePath.lastIndexOf(".xmi") == (length - 4)) {
                    this.optionComposite.setXMISelected();
                } else {
                    if (this.optionComposite.getXMIButton().getSelection()) {
                        setXMIExtension();
                    } else {
                        setUMLExtension();
                    }
                }
            }
        }
    }

    @objid ("06b816f8-4583-11e0-b54c-00137279a832")
    void setUMLExtension() {
        String filePath = this.fileChooserComposite.getText();
        int length = filePath.length();
        if ((length - 4) != filePath.lastIndexOf(".uml")) {
            if (filePath.lastIndexOf(".xmi") == (length - 4)) {
                filePath = filePath.substring(0, length - 4) + ".uml";
            } else {
                filePath = filePath + ".uml";
            }
            this.fileChooserComposite.setText(filePath);
        }
    }

    @objid ("06b816fa-4583-11e0-b54c-00137279a832")
    void setXMIExtension() {
        String filePath = this.fileChooserComposite.getText();
        int length = filePath.length();
        if ((length - 4) != filePath.lastIndexOf(".xmi")) {
            if (filePath.lastIndexOf(".uml") == (length - 4)) {
                filePath = filePath.substring(0, length - 4) + ".xmi";
            } else {
                filePath = filePath + ".xmi";
            }
            this.fileChooserComposite.setText(filePath);
        }
    }

    @objid ("06b816fc-4583-11e0-b54c-00137279a832")
    protected void fileCanBeCreated(File file) {
        MessageBox messageBox = new MessageBox(this.shell, SWT.OK);
        messageBox.setMessage(XMI.I18N.getMessage("fileChooser.dialog.confirm.canNotCreateFile.label", file.getName()));
        messageBox.setText( XMI.I18N.getString("fileChooser.dialog.confirm.canNotCreateFile.title"));
        messageBox.open();
    }

    @objid ("06b816ff-4583-11e0-b54c-00137279a832")
    protected boolean confirmOverwrite(File file) {
        MessageBox messageBox = new MessageBox(this.shell, SWT.YES | SWT.NO);
        messageBox.setMessage(XMI.I18N.getMessage("fileChooser.dialog.confirm.overwriteFile.label", file.getName()));
        messageBox.setText(XMI.I18N.getString("fileChooser.dialog.confirm.overwriteFile.title"));
        return (messageBox.open() == SWT.YES);
    }

    @objid ("06ba7953-4583-11e0-b54c-00137279a832")
    protected boolean testExportFilePath(File theFile) {
        if (theFile.exists()) {
            return confirmOverwrite(theFile);      
        } 
        
        if (theFile.getParentFile().exists() || theFile.getParentFile().mkdirs()) {
            return true;       
        } 
        
        fileCanBeCreated(theFile);
        return false;
    }

    /**
     * @param inpath : the initial path
     * @return the same path in a correct form
     */
    @objid ("0699186e-4583-11e0-b54c-00137279a832")
    public static String checkAndReplaceEndPath(final String inpath) {
        if (inpath.endsWith("\\")) {
            return inpath.substring(0, inpath.lastIndexOf("\\"));
        } else if (inpath.endsWith("/")) {
            return inpath.substring(0, inpath.lastIndexOf("/"));
        }
        return inpath;
    }

}
