/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui;

import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

/**
 * XMI thread.
 * This abstract class define the common structure of four XMI threads
 * @author ebrosse
 */
@objid ("06c8c773-4583-11e0-b54c-00137279a832")
public abstract class XMIThread extends Thread {
    @objid ("06c8c774-4583-11e0-b54c-00137279a832")
    protected ProgressBarComposite progressBar = null;

    @objid ("aa7e2e81-6a7c-11e0-957f-00137279a832")
    protected Shell shell = null;

    @objid ("6c68b748-1a76-11e1-8103-0027103f347d")
    private String description = "";

    @objid ("6c68b749-1a76-11e1-8103-0027103f347d")
    private String title = "";


    @objid ("06c8c778-4583-11e0-b54c-00137279a832")
    protected void cancelation() {
        SwtWizardWindow.setCancellation(false);
    }

    @objid ("06cb29c8-4583-11e0-b54c-00137279a832")
    protected void completeBox() {
        if (this.shell != null){
            this.title = XMI.I18N.getString("fileChooser.dialog.endBox");
        
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
            if (!logs.isEmpty()){
                this.description = XMI.I18N.getMessage("warning.log.check", logs.getLogFile() + ".log");
                logs.setEmpty(true);
            }else{
                this.description = XMI.I18N.getString("fileChooser.dialog.endBox");
            }
        
            Display.getDefault().asyncExec(new Runnable() {
                @Override
                public void run() {
                    customMessageBox(SWT.ICON_INFORMATION);
                }
            });
        }
    }

    @objid ("06cb29ca-4583-11e0-b54c-00137279a832")
    void customMessageBox(int icon) {
        MessageBox messageBox = new MessageBox(this.shell, icon);
        messageBox.setMessage(this.description);
        messageBox.setText(this.title);
        if (messageBox.open() == SWT.OK) {            
            this.shell.dispose();             
        }
    }

    @objid ("6c69056c-1a76-11e1-8103-0027103f347d")
    protected void incompleteBox(final String errorMessage) {
        if (this.shell != null){
            this.title = XMI.I18N.getString("fileChooser.dialog.errorBox");
        
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
            if (!logs.isEmpty()){
                logs.setEmpty(true);
            }
        
            this.description = errorMessage;
        
            Display.getDefault().asyncExec(new Runnable() {
                @Override
                public void run() {
                    customMessageBox(SWT.ICON_ERROR);
                }
            });
        }
    }

    @Override
    @objid ("13a8d7bb-26f7-11e1-921d-0027103f347d")
    public void interrupt() {
        super.interrupt();
        ReverseProperties.getInstance().setRollback(true);
    }

}
