/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui.report;

import com.modeliosoft.modelio.api.app.navigation.INavigationService;
import com.modeliosoft.modelio.api.ui.ModelioDialog;
import com.modeliosoft.modelio.api.ui.UIColor;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.report.ReportModel.ElementMessage;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog box used for the generation report.
 */
@objid ("0677b77d-4583-11e0-b54c-00137279a832")
class ReportDialog extends ModelioDialog {
    @objid ("0677b77e-4583-11e0-b54c-00137279a832")
    private ReportModel model;

    @objid ("54e7bf56-3b70-11e1-8577-0027103f347d")
     INavigationService navigationService;

    @objid ("c176b49c-3b70-11e1-8577-0027103f347d")
    private Image warningImage;

    @objid ("c176b49d-3b70-11e1-8577-0027103f347d")
    private Image errorImage;

    @objid ("c176b49e-3b70-11e1-8577-0027103f347d")
     Table table;

    @objid ("c176b49f-3b70-11e1-8577-0027103f347d")
    private Image infoImage;

    @objid ("c176dbb0-3b70-11e1-8577-0027103f347d")
     Text descriptionText;


    @objid ("0677b784-4583-11e0-b54c-00137279a832")
    public ReportDialog(Shell parentShell, final INavigationService navigationService) {
        super (parentShell);
        setShellStyle (SWT.DIALOG_TRIM | getDefaultOrientation ());
        this.navigationService = navigationService;
    }

    @Override
    @objid ("0677b787-4583-11e0-b54c-00137279a832")
    public void addButtonsInButtonBar(Composite parent) {
        createButton (parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    }

    @Override
    @objid ("067a19d4-4583-11e0-b54c-00137279a832")
    public Control createContentArea(Composite parent) {
        this.warningImage = JFaceResources.getImage ("dialog_messasge_warning_image"); //$NON-NLS-1$
        this.errorImage = JFaceResources.getImage ("dialog_message_error_image"); //$NON-NLS-1$
        this.infoImage = JFaceResources.getImage ("dialog_messasge_info_image"); //$NON-NLS-1$
        
        Point s = getInitialSize ();
        
        
        this.table = new Table (parent, SWT.MULTI | SWT.BORDER |
                SWT.FULL_SELECTION);
        GridData data = new GridData (SWT.FILL, SWT.FILL, true, true);
        this.table.setLayoutData (data);
        this.table.setLinesVisible (true);
        
        TableColumn column = new TableColumn (this.table, SWT.NONE);
        column.setText ("Message");
        
        updateViewFromModel ();
        
        column.setWidth (s.x);
        
        this.table.addMouseListener (new MouseListener () {
            @Override
            public void mouseDoubleClick (MouseEvent e) {
                TableItem item = ReportDialog.this.table.getItem (new Point (e.x, e.y));
                if (item != null) {
                    if (item.getData () instanceof ElementMessage) {
                        ElementMessage theElement = (ElementMessage) item.getData ();
        
                        String desc = theElement.description;
                        ReportDialog.this.descriptionText.setText (desc);
        
                        // On a double clic, select the element
                        if (theElement.element != null && theElement.element.isValid ()) {
                            ReportDialog.this.navigationService.fireNavigate (theElement.element);
                        }
                    }
                }
            }
        
            @Override
            public void mouseDown (MouseEvent e) {
                // Nothing to do
            }
        
            @Override
            public void mouseUp (MouseEvent e) {
                TableItem item = ReportDialog.this.table.getItem (new Point (e.x, e.y));
                if (item != null) {
                    if (item.getData () instanceof ElementMessage) {
                        ElementMessage theElement = (ElementMessage) item.getData ();
        
                        String desc = theElement.description;
                        ReportDialog.this.descriptionText.setText (desc);
        
                        // On a CTRL + clic, select the element
                        if ((e.stateMask & SWT.CTRL) != 0) {
                            if (theElement.element != null && theElement.element.isValid ()) {
                                ReportDialog.this.navigationService.fireNavigate (theElement.element);
                            }
                        }
                    }
                }
            }
        });
        
        this.descriptionText = new Text (parent, SWT.BORDER | SWT.MULTI | SWT.READ_ONLY | SWT.WRAP);
        GridData data_description = new GridData (SWT.FILL, SWT.FILL, true, false);
        data_description.heightHint = 50;
        this.descriptionText.setLayoutData (data_description);
        this.descriptionText.setBackground (UIColor.TEXT_READONLY_BG);
        return parent;
    }

    @objid ("067a19da-4583-11e0-b54c-00137279a832")
    private void updateViewFromModel() {
        if (this.table != null) {
            this.table.removeAll ();
        
            if (this.model != null) {
                for (ElementMessage error : this.model.getErrors ()) {
                    TableItem item = new TableItem (this.table, SWT.NONE);
                    item.setImage (0, this.errorImage);
                    item.setText (0, error.message);
                    item.setData (error);
                }
        
                for (ElementMessage warning : this.model.getWarnings ()) {
                    TableItem item = new TableItem (this.table, SWT.NONE);
                    item.setImage (0, this.warningImage);
                    item.setText (0, warning.message);
                    item.setData (warning);
                }
        
                for (ElementMessage info : this.model.getInfos ()) {
                    TableItem item = new TableItem (this.table, SWT.NONE);
                    item.setImage (0, this.infoImage);
                    item.setText (0, info.message);
                    item.setData (info);
                }
            }
        
            this.table.getColumn (0).pack ();
        }
    }

    @objid ("067a19dc-4583-11e0-b54c-00137279a832")
    public void setModel(ReportModel model) {
        this.model = model;
        updateViewFromModel ();
    }

    @objid ("067a19df-4583-11e0-b54c-00137279a832")
    public boolean isDisposed() {
        Shell s = getShell ();
        return s == null || s.isDisposed ();
    }

    @Override
    @objid ("067c7c2f-4583-11e0-b54c-00137279a832")
    public void init() {
        Shell shell = getShell ();
        
        // Put the messages in the banner area
        setLogoImage (null);
        shell.setText (XMI.I18N.getString ("Gui.Export.ReportDialogTitle")); 
        setTitle (XMI.I18N.getString ("Gui.Export.ReportDialogTitle")); 
        setMessage (XMI.I18N.getString ("Gui.Export.ReportDialogMessage"));
    }

    @Override
    @objid ("3254b4de-6de7-11e1-87ca-0027103f347d")
    protected Point getInitialSize() {
        Point p = super.getInitialSize();
        p.x = (int) Math.floor(p.x*1.35);
        return p;
    }

}
