/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui.report;

import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.report.ReportModel.ElementMessage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

/**
 * This class is the controller of the XMI report windows.
 * 
 * It provides all needed services and manages the relations between the XMI report model and the XMI report dialog.
 * @author ebrosse
 */
@objid ("067c7c32-4583-11e0-b54c-00137279a832")
public class ReportManager {
    @objid ("067c7c33-4583-11e0-b54c-00137279a832")
    private static ReportDialog dialog;


    /**
     * This method opens the XMI report dialog
     * @param report : the report model exposed in report dialog
     */
    @objid ("54e96d30-3b70-11e1-8577-0027103f347d")
    public static void showGenerationReport(final ReportModel report) {
        if (report == null || report.isEmpty ()) {
            if (ReportManager.dialog != null &&
                    !ReportManager.dialog.isDisposed ()) {
                ReportManager.dialog.close ();
            }
        } else {
            
            // Get the current display
            Display display = Display.getCurrent ();
            if (display == null) {
                display = Display.getDefault ();
            }  
            
            if (ReportManager.dialog == null ||
                    ReportManager.dialog.isDisposed ()) {
                ReportManager.dialog = new ReportDialog (display.getActiveShell(), Modelio.getInstance().getNavigationService());
            }
        
            ReportManager.dialog.setModel(report);
            
            if (ReportManager.dialog.open () == SWT.OK) {            
                display.getActiveShell().dispose();
            }
        }
    }

    /**
     * This method creates a new Report Model
     * @return the created Report Model
     */
    @objid ("54e96d33-3b70-11e1-8577-0027103f347d")
    public static ReportModel getNewReport() {
        return new ReportModel ();
    }

    /**
     * This method prints the given report model into Modelio.err and Modelio.out
     * @param report : the report model which will be printed.
     */
    @objid ("54e99445-3b70-11e1-8577-0027103f347d")
    public static void printGenerationReport(final ReportModel report) {
        if (report != null && !report.isEmpty ()) {
            for (ElementMessage errorMsg : report.getErrors ()) {
                Modelio.err.println (errorMsg.message);
            }
        
            for (ElementMessage errorMsg : report.getWarnings ()) {
                Modelio.out.println (errorMsg.message);
            }
        
            for (ElementMessage errorMsg : report.getInfos ()) {
                Modelio.out.println (errorMsg.message);
            }
        }
    }

}
