/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.gui.report;

import java.util.Set;
import java.util.TreeSet;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

/**
 * This class represents the report model of XMI exportq
 * @author ebrosse
 */
@objid ("067ede8c-4583-11e0-b54c-00137279a832")
public class ReportModel implements IReportWriter {
    @objid ("067ede8d-4583-11e0-b54c-00137279a832")
    private Set<ElementMessage> errors;

    @objid ("067ede90-4583-11e0-b54c-00137279a832")
    private Set<ElementMessage> warnings;

    @objid ("067ede93-4583-11e0-b54c-00137279a832")
    private Set<ElementMessage> infos;


    /**
     * This default constructor initializes the lists of message (errors, warning and infos)
     */
    @objid ("068140e3-4583-11e0-b54c-00137279a832")
    public ReportModel() {
        this.errors = new TreeSet<ElementMessage> ();
        this.warnings = new TreeSet<ElementMessage> ();
        this.infos = new TreeSet<ElementMessage> ();
    }

    @Override
    @objid ("068140e5-4583-11e0-b54c-00137279a832")
    public void addWarning(final String initialMessage, IElement element, String description) {
        String message = initialMessage;
        if (message == null) {
            message = "";
        }
        
        this.warnings.add (new ElementMessage (message, element, description));
    }

    @Override
    @objid ("068140ea-4583-11e0-b54c-00137279a832")
    public void addError(final String initialMessage, IElement element, String description) {
        String message = initialMessage;
        if (message == null) {
            message = "";
        }
        
        this.errors.add (new ElementMessage (message, element, description));
    }

    /**
     * This method returns the list of error message
     * @return set of error message
     */
    @objid ("068140ef-4583-11e0-b54c-00137279a832")
    public Set<ElementMessage> getErrors() {
        return this.errors;
    }

    /**
     * This method returns the list of warning message
     * @return set of warning message
     */
    @objid ("0683a33d-4583-11e0-b54c-00137279a832")
    public Set<ElementMessage> getWarnings() {
        return this.warnings;
    }

    @Override
    @objid ("0683a343-4583-11e0-b54c-00137279a832")
    public boolean isEmpty() {
        return !(hasErrors () || hasWarnings () || hasInfos ());
    }

    /**
     * Adds a warning message at the existing list
     * @param message : the text of the message
     * @param element : the IElement
     */
    @objid ("0683a347-4583-11e0-b54c-00137279a832")
    public void addWarning(final String message, final IElement element) {
        this.warnings.add (new ElementMessage (message, element, ""));
    }

    /**
     * Adds an error message at the existing list
     * @param message : the text of the message
     * @param element : the IElement
     */
    @objid ("06860596-4583-11e0-b54c-00137279a832")
    public void addError(String message, IElement element) {
        this.errors.add (new ElementMessage (message, element, ""));
    }

    @Override
    @objid ("0686059a-4583-11e0-b54c-00137279a832")
    public void addInfo(String message, IElement element, String description) {
        this.infos.add (new ElementMessage (message, element, description));
    }

    /**
     * Adds an info message at the existing list
     * @param message : the text of the message
     * @param element : the IElement
     */
    @objid ("068605a0-4583-11e0-b54c-00137279a832")
    public void addInfo(String message, IElement element) {
        this.infos.add (new ElementMessage (message, element, ""));
    }

    /**
     * This method returns the list of info message
     * @return set of info message
     */
    @objid ("068605a4-4583-11e0-b54c-00137279a832")
    public Set<ElementMessage> getInfos() {
        return this.infos;
    }

    @Override
    @objid ("068867f4-4583-11e0-b54c-00137279a832")
    public boolean hasErrors() {
        return !this.errors.isEmpty ();
    }

    @Override
    @objid ("068867f9-4583-11e0-b54c-00137279a832")
    public boolean hasInfos() {
        return !this.infos.isEmpty ();
    }

    @Override
    @objid ("068aca4b-4583-11e0-b54c-00137279a832")
    public boolean hasWarnings() {
        return !this.warnings.isEmpty ();
    }

    @objid ("068aca50-4583-11e0-b54c-00137279a832")
    class ElementMessage implements Comparable<ElementMessage> {
        @objid ("068aca51-4583-11e0-b54c-00137279a832")
        public String message;

        @objid ("068aca52-4583-11e0-b54c-00137279a832")
        public IElement element;

        @objid ("068aca53-4583-11e0-b54c-00137279a832")
        public String description;


        @objid ("068aca54-4583-11e0-b54c-00137279a832")
        ElementMessage(final String message, final IElement element, final String description) {
            this.message = message;
            this.element = element;
            this.description = description;
        }

        @Override
        @objid ("068d2ca4-4583-11e0-b54c-00137279a832")
        public boolean equals(final Object obj) {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            ElementMessage other = (ElementMessage) obj;
            if (!getOuterType().equals(other.getOuterType()))
                return false;
            if (this.description == null) {
                if (other.description != null)
                    return false;
            } else if (!this.description.equals(other.description))
                return false;
            if (this.element == null) {
                if (other.element != null)
                    return false;
            } else if (!this.element.equals(other.element))
                return false;
            if (this.message == null) {
                if (other.message != null)
                    return false;
            } else if (!this.message.equals(other.message))
                return false;
            return true;
        }

        @Override
        @objid ("068d2caa-4583-11e0-b54c-00137279a832")
        public int compareTo(final ElementMessage anotherMessage) {
            int result = this.element.compareTo(anotherMessage.element);
            if (result == 0)
                return this.message.compareTo (anotherMessage.message) ;
            else
                return result;
        }

        @Override
        @objid ("54e65fa0-3b70-11e1-8577-0027103f347d")
        public int hashCode() {
            final int prime = 31;
            int result = 1;
            result = prime * result + getOuterType().hashCode();
            result = prime * result + ((this.description == null) ? 0 : this.description.hashCode());
            result = prime * result + ((this.element == null) ? 0 : this.element.hashCode());
            result = prime * result + ((this.message == null) ? 0 : this.message.hashCode());
            return result;
        }

        @objid ("54e6d4dd-3b70-11e1-8577-0027103f347d")
        private ReportModel getOuterType() {
            return ReportModel.this;
        }

    }

}
