/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.impl;

import java.io.File;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.jni.ITransaction;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.core.transaction.ITransactionManager;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.ExportConfiguration;
import com.modeliosoft.modelio.xmi.api.IXMI;
import com.modeliosoft.modelio.xmi.api.ImportConfiguration;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.generation.ExportServices;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.ImportServices;
import com.modeliosoft.modelio.xmi.util.AbortProcessException;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMIFileUtils;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

/**
 * This class implements the services provided by XMI
 * @author ebrosse
 */
@objid ("06eeed15-4583-11e0-b54c-00137279a832")
public class XMIImpl implements IXMI {
    @objid ("06eeed16-4583-11e0-b54c-00137279a832")
    private ImportServices importService = null;

    @objid ("06eeed17-4583-11e0-b54c-00137279a832")
    private ExportServices exportService = null;


    @objid ("06eeed18-4583-11e0-b54c-00137279a832")
    private void cancelProcess(Shell shell) {
        if ((shell != null) && (!System.getProperty("os.name").equals("Linux"))) {
            shell.dispose();
        }
        
        LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("info.import.result_failed"));
        ITransaction t = ReverseProperties.getInstance().getOpenedTransaction();
        if (t != null) {
            XMI.getInstance().getModelingSession().getTransactionManager().rollback(t);
            t = null;
        }
    }

    @objid ("06f14f68-4583-11e0-b54c-00137279a832")
    private void commitTransaction(boolean error) {
        ITransaction t = ReverseProperties.getInstance().getOpenedTransaction();
        ITransactionManager tm = XMI.getInstance().getModelingSession().getTransactionManager();
        
        if ((error)) {
            tm.rollback(t);
            t = null;
        } else {
        
            if (ReverseProperties.getInstance().isRollback()) {
                tm.rollback(t);
                t = null;
                ReverseProperties.getInstance().setRollback(false);
            } else {
                tm.commit(t);
            }
        }
    }

    @objid ("06f14f77-4583-11e0-b54c-00137279a832")
    private void exportModel(final ExportConfiguration configuration) {
        Shell shell = null;
        try {
         
            String xmiFilePath = configuration.getXmiFile().getAbsolutePath() ;
            GenerationProperties.getInstance().setSelectedPackage(configuration.getEntryPoint());
            Resource resource = this.exportService.createResource(xmiFilePath);
            GenerationProperties.getInstance().setRoundtripEnabled(configuration.exportedAnotation());
            this.exportService.createEcoreModel(resource, null);
        
            if (!configuration.getVersionExport().equals(VersionExport.EMF300))
                XMIFileUtils.changeToUML(xmiFilePath);
        
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("info.export.result_done"));
        } catch (AbortProcessException e) {
            cancelProcess(shell);
        } catch (Exception e) {
            e.printStackTrace();
            cancelProcess(shell);
            LOG.error(XMI.PLUGIN_ID, "Erreur.");
            LOG.error(XMI.PLUGIN_ID, e);
        } finally {
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
        }
    }

    @Override
    @objid ("06f3b1c8-4583-11e0-b54c-00137279a832")
    public void exportXMIFile(final ExportConfiguration configuration) throws Exception {
        if (configuration.getEntryPoint() == null) {
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.nullGivenParameter"));
        } else {
            if (configuration.getXmiFile() != null) {
                initExportService(configuration.getEntryPoint());
                GenerationProperties.getInstance().setFilePath(configuration.getXmiFile());
                exportModel(configuration);
            } else {
                LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.invalidFilePath"));
            }
        }
    }

    @objid ("06f61427-4583-11e0-b54c-00137279a832")
    private boolean importModel(File xmiFile) {
        boolean error = false;
        IModelingSession session = XMI.getInstance().getModelingSession();
        ITransaction t = session.getTransactionManager().createTransaction("Create-update objing model from XMI");
        ReverseProperties.getInstance().setOpenedTransaction(t);
        
        
        Shell shell = Display.getDefault().getActiveShell();
        try {
            Resource resource = this.importService.getResource(xmiFile);
        
            if ((resource != null)) {
                error = this.importService.importEcoreModel(resource, null, shell);
            }
        } catch (AbortProcessException e) {
            // cancelProcess(shell);
        } catch (Exception e) {
            error = true;
            // cancelProcess(shell);
            LOG.error(XMI.PLUGIN_ID, "Erreur.");
            LOG.error(XMI.PLUGIN_ID, e);
        } finally {
            commitTransaction(error);
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
        }
        return error;
    }

    @objid ("06f8767f-4583-11e0-b54c-00137279a832")
    private void initExportService(IPackage entryPoint) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        genProp.initialize();
        genProp.setTimeDisplayerActivated(false);
        genProp.setSelectedPackage(entryPoint);
        
        if (this.exportService == null)
            this.exportService = new ExportServices(null);
    }

    @objid ("06f87682-4583-11e0-b54c-00137279a832")
    private void initImportService() {
        ReverseProperties.getInstance().initialize();
        if (this.importService == null)
            this.importService = new ImportServices();
    }

    @Override
    @objid ("6164f1e1-5ed0-11e1-9159-0027103f347d")
    public void importXMIModel(final ImportConfiguration configuration) {
        if (configuration.getXmiFile() != null && configuration.getXmiFile().isFile()) {
            initImportService();
            ReverseProperties.getInstance().setUMLRoot((IPackage)configuration.getOwner());
            importModel(configuration.getXmiFile());
        } else {
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.invalidFilePath"));
        }
    }

    @Override
    @objid ("61654009-5ed0-11e1-9159-0027103f347d")
    public void importXMIProfile(final ImportConfiguration configuration) {
        if(configuration.getOwner() != null && configuration.getOwner() instanceof IModule){
            if (configuration.getXmiFile() != null && configuration.getXmiFile().isFile()) {
                initImportService();
                importProfile(configuration.getXmiFile(),(IModule)configuration.getOwner() );
            } else {
                LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.invalidFilePath"));
            }
        }else{
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.invalidOwner"));
        
        }
    }

    @Override
    @objid ("2cd6bcd3-94ff-11e1-bc83-bc305ba4815c")
    public void exportXMIProfile(final ExportConfiguration configuration) throws Exception {
        if (configuration.getEntryPoint() == null || !(configuration.getEntryPoint() instanceof IProfile)) {
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.nullGivenParameter"));
        } else {
            if (configuration.getXmiFile() != null) {
                initExportService((IProfile)configuration.getEntryPoint());
                GenerationProperties.getInstance().setFilePath(configuration.getXmiFile());
                exportProfile(configuration);        
            } else {
                LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("error.invalidFilePath"));
            }
        }
    }

    @objid ("2cd6e3e6-94ff-11e1-bc83-bc305ba4815c")
    private void exportProfile(final ExportConfiguration configuration) {
        Shell shell = null;
        try {      
            String xmiFilePath = configuration.getXmiFile().getAbsolutePath() ;
            GenerationProperties.getInstance().setSelectedPackage((IProfile)configuration.getEntryPoint());
            Resource resource = this.exportService.createResource(xmiFilePath);
            GenerationProperties.getInstance().setRoundtripEnabled(configuration.exportedAnotation());
            this.exportService.createEcoreProfile(resource, null);
        
            if (!configuration.getVersionExport().equals(VersionExport.EMF300))
                XMIFileUtils.changeToUML(xmiFilePath);
        
            LOG.error(XMI.PLUGIN_ID, XMI.I18N.getString("info.export.result_done"));
        } catch (AbortProcessException e) {
            cancelProcess(shell);
        } catch (Exception e) {
            e.printStackTrace();
            cancelProcess(shell);
            LOG.error(XMI.PLUGIN_ID, "Erreur.");
            LOG.error(XMI.PLUGIN_ID, e);
        } finally {
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
        }
    }

    @objid ("2cd75918-94ff-11e1-bc83-bc305ba4815c")
    private boolean importProfile(final File xmiFile, final IModule module) {
        boolean error = false;
        IModelingSession session = XMI.getInstance().getModelingSession();
        ITransaction t = session.getTransactionManager().createTransaction("Create-update objing model from XMI");
        ReverseProperties.getInstance().setOpenedTransaction(t);
        
        
        Shell shell = Display.getDefault().getActiveShell();
        try {
            Resource resource = this.importService.getResource(xmiFile);
        
            if ((resource != null)) {
                error = this.importService.importEcoreProfile(resource,module, null, shell);
            }
        } catch (AbortProcessException e) {
            // cancelProcess(shell);
        } catch (Exception e) {
            error = true;
            // cancelProcess(shell);
            LOG.error(XMI.PLUGIN_ID, "Erreur.");
            LOG.error(XMI.PLUGIN_ID, e);
        } finally {
            commitTransaction(error);
            XMILogs logs = XMILogs.getInstance();
            logs.saveLog();
        }
        return error;
    }

}
