/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IAcceptCallEventAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IAcceptChangeEventAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IAcceptSignalAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IAcceptTimeEventAction;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ReceiveOperationEvent;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.TimeEvent;
import org.eclipse.uml2.uml.Trigger;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0706c492-4583-11e0-b54c-00137279a832")
public class EAcceptEventAction extends EActivityNode implements IEElement {
    @objid ("0706c493-4583-11e0-b54c-00137279a832")
    private boolean hasReceiveOperationEvent = false;

    @objid ("0706c494-4583-11e0-b54c-00137279a832")
    private boolean hasSignalEvent = false;

    @objid ("0706c495-4583-11e0-b54c-00137279a832")
    private boolean hasChangeEvent = false;

    @objid ("0706c496-4583-11e0-b54c-00137279a832")
    private boolean hasTimeEvent = false;


    @objid ("0706c497-4583-11e0-b54c-00137279a832")
    public EAcceptEventAction() {
    }

    @objid ("0706c499-4583-11e0-b54c-00137279a832")
    public EAcceptEventAction(AcceptEventAction element) {
        super(element);
        if (EcoreModelNavigation.hasReceiveOperationEvent(element)) {
            hasReceiveOperationEvent = true;
            
        } else if (EcoreModelNavigation.hasSignalEvent(element)) {
            hasSignalEvent = true;
        
        } else if (EcoreModelNavigation.hasChangeEvent(element)) {
            hasChangeEvent = true;
            
        } else if (EcoreModelNavigation.hasTimeEvent(element)) {
            hasTimeEvent = true;
        }
    }

    @objid ("0706c49c-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        AcceptEventAction ecoreElement = (AcceptEventAction) getEcoreElement();
        if (hasReceiveOperationEvent) {
            return Modelio.getInstance().getModelingSession().getModel()
                    .createAcceptCallEventAction();
        } else if (hasSignalEvent) {
            return Modelio.getInstance().getModelingSession().getModel()
                    .createAcceptSignalAction();
        } else if (hasChangeEvent) {
            return Modelio.getInstance().getModelingSession().getModel()
                    .createAcceptChangeEventAction();
        } else if (hasTimeEvent) {
            return Modelio.getInstance().getModelingSession().getModel()
                    .createAcceptTimeEventAction();
        } else
            if (ReverseProperties.getInstance().isRoundtripEnabled()){
                String signal = ObjingEAnnotation.getSignal(ecoreElement);
                if ((signal != null) && (signal.equals("signal")))
                    return Modelio.getInstance().getModelingSession().getModel()
                    .createAcceptSignalAction();
            }
        return Modelio.getInstance().getModelingSession().getModel()
                .createAcceptCallEventAction();
    }

    @objid ("070926f0-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("070926f3-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("070926f8-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (hasReceiveOperationEvent)
            setCalled((IAcceptCallEventAction) objingElt);
        else if (hasSignalEvent)
            setAccepted((IAcceptSignalAction) objingElt);
        else if (hasChangeEvent)
            setChangeExpression((IAcceptChangeEventAction) objingElt);
        else if (hasTimeEvent)
            setTimeExpression((IAcceptTimeEventAction) objingElt);
    }

    @objid ("070deba0-4583-11e0-b54c-00137279a832")
    private void setCalled(IAcceptCallEventAction action) {
        for (Object trigger :  ((AcceptEventAction) getEcoreElement()).getTriggers()) {
            Event event = ((Trigger) trigger).getEvent();
            if (event instanceof ReceiveOperationEvent) {
                Operation ecoreOperation = ((ReceiveOperationEvent) event)
                        .getOperation();
                if (ecoreOperation != null) {
                    Object objingOperation =  ReverseProperties.getInstance()
                            .getMappedElement(ecoreOperation);
                    if (objingOperation instanceof IOperation) {
                        action.setCalled((IOperation) objingOperation);
                        break;
                    }
                }
            }
        }
    }

    @objid ("070deba3-4583-11e0-b54c-00137279a832")
    private void setAccepted(IAcceptSignalAction action) {
        for (Object trigger :  ((AcceptEventAction) getEcoreElement()).getTriggers()) {
            Event event = ((Trigger) trigger).getEvent();
            if (event instanceof SignalEvent) {
                Signal ecoreSignal = ((SignalEvent) event).getSignal();
                if (ecoreSignal != null) {
                    Object objingSignal =  ReverseProperties.getInstance().getMappedElement(ecoreSignal);
                    if (objingSignal instanceof ISignal)
                        action.addAccepted((ISignal) objingSignal);
                }
            }
        }
    }

    @objid ("070deba6-4583-11e0-b54c-00137279a832")
    private void setChangeExpression(IAcceptChangeEventAction action) {
        for (Object trigger :  ((AcceptEventAction) getEcoreElement()).getTriggers()) {
            Event event = ((Trigger) trigger).getEvent();
            if (event instanceof ChangeEvent) {
                ValueSpecification value = ((ChangeEvent) event)
                        .getChangeExpression();
                if (value != null) {
                    String stringValue = value.stringValue();
                    if (stringValue != null) {
                        action.setchangeExpresion(stringValue);
                        break;
                    }
                }
            }
        }
    }

    @objid ("070deba9-4583-11e0-b54c-00137279a832")
    private void setTimeExpression(IAcceptTimeEventAction action) {
        for (Object trigger :  ((AcceptEventAction) getEcoreElement()).getTriggers()) {
            Event event = ((Trigger) trigger).getEvent();
            if (event instanceof TimeEvent) {
                ValueSpecification value = ((TimeEvent) event).getWhen();
                if (value != null) {
                    String stringValue = value.stringValue();
                    if (stringValue != null) {
                        action.settimeExpresion(stringValue);
                        break;
                    }
                }
            }
        }
    }

}
