/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityPartition;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IStructuredActivityNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.StructuredActivityNode;

@objid ("6a111406-1a76-11e1-8103-0027103f347d")
public class EActivityNode extends ENamedElement implements IEElement {

    @objid ("6a113b18-1a76-11e1-8103-0027103f347d")
    public IElement createObjingElt() {
        // TODO Auto-generated method stub
        return null;
    }

    @objid ("6a11622b-1a76-11e1-8103-0027103f347d")
    public EActivityNode(final ActivityNode element) {
        super(element);
    }

    @objid ("6a11b04f-1a76-11e1-8103-0027103f347d")
    public void attach(final List<Object> objingElt) {
    }

    @objid ("6a122584-1a76-11e1-8103-0027103f347d")
    public void attach(final IElement objingElt) {
        ActivityNode ecoreElement = (ActivityNode) getEcoreElement();
        if (ecoreElement.getOwner() != null) {
            if (EcoreModelNavigation.isInClause(ecoreElement)) {
                attachToClause(objingElt);
            } else if (EcoreModelNavigation.isInPartition(ecoreElement)) {
                attachToPartition(objingElt);
            } else if (EcoreModelNavigation
                    .isInStructuredActivityNode(ecoreElement)) {
                attachToStructuredActivityNode(objingElt);
            } else if (EcoreModelNavigation.isInActivity(ecoreElement)) {
                attachToActivity(objingElt);
            } else {
                ReverseProperties.getInstance().deleteElement(ecoreElement, objingElt);
            }
        }
    }

    @objid ("6a124c98-1a76-11e1-8103-0027103f347d")
    private void attachToPartition(final IElement objingElt) {
        List<ActivityPartition> ecorePartitions = EcoreModelNavigation
        .getOwnerPartitions((ActivityNode)getEcoreElement());
        
        if (ecorePartitions.size() > 0) {
            ActivityPartition ecorePartition = null;
            if (ecorePartitions.size() > 1)
                ecorePartition = EcoreModelNavigation
                .getLeafPartition(ecorePartitions);
            else
                ecorePartition = ecorePartitions.get(0);
        
            if (ecorePartition != null) {
                IActivityPartition objingPartition = (IActivityPartition)  ReverseProperties.getInstance()
                .getMappedElement(ecorePartition);
                ((IActivityNode) objingElt).setOwnerPartition(objingPartition);
            }
        }
    }

    @objid ("6a129aba-1a76-11e1-8103-0027103f347d")
    private void attachToStructuredActivityNode(final IElement objingElt) {
        StructuredActivityNode ecoreNode = EcoreModelNavigation
        .getOwnerStructuredActivityNode((ActivityNode)getEcoreElement());
        if (ecoreNode != null) {
            IStructuredActivityNode objingNode = (IStructuredActivityNode)  ReverseProperties.getInstance()
            .getMappedElement(ecoreNode);
            if (objingNode != null)
                ((IActivityNode) objingElt).setOwnerNode(objingNode);
        }
    }

    @objid ("6a12c1cc-1a76-11e1-8103-0027103f347d")
    private void attachToClause(final IElement objingElt) {
        Clause ecoreClause = EcoreModelNavigation.getOwnerClause((ActivityNode)getEcoreElement());
        if (ecoreClause != null) {
            IClause objingClause = (IClause)  ReverseProperties.getInstance()
            .getMappedElement(ecoreClause);
            if (objingClause != null)
                ((IActivityNode) objingElt)
                .setOwnerClause(objingClause);
        }
    }

    @objid ("6a12e8e0-1a76-11e1-8103-0027103f347d")
    private void attachToActivity(final IElement objingElt) {
        Activity ecoreActivity = EcoreModelNavigation
        .getOwnerActivity((ActivityNode)getEcoreElement());
        if (ecoreActivity != null) {
            IActivity objingActivity = (IActivity)  ReverseProperties.getInstance()
            .getMappedElement(ecoreActivity);
            if (objingActivity != null)
                ((IActivityNode) objingElt).setOwner(objingActivity);
        }
    }

    @objid ("6a133702-1a76-11e1-8103-0027103f347d")
    public void setProperties(final IElement objingElt) {
        super.setProperties(objingElt);
    }

    @objid ("6a135e16-1a76-11e1-8103-0027103f347d")
    public EActivityNode() {
    }

}
