/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOpaqueAction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.AddStructuralFeatureValueAction;
import org.eclipse.uml2.uml.StructuralFeature;

@objid ("0749866c-4583-11e0-b54c-00137279a832")
public class EAddStructuralFeatureValueAction extends EActivityNode implements IEElement {

    @objid ("0749866d-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IOpaqueAction element = Modelio.getInstance().getModelingSession().getModel()
        .createOpaqueAction();
        
        try{
            IStereotype stereo = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IOpaqueAction.class, XMIStereotypes.UML2ADDSTRUCTURALFEATUREVALUEACTION);
            element.addExtension(stereo);
        }catch(StereotypeNotFoundException e){
        
            Modelio.err.println("StereotypeNotFoundException : ");
            e.printStackTrace(Modelio.err);
        }
        return element;
    }

    @objid ("07498671-4583-11e0-b54c-00137279a832")
    public EAddStructuralFeatureValueAction(AddStructuralFeatureValueAction element) {
        super(element);
    }

    @objid ("074be8cb-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("074be8ce-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("074e4b1e-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setFeature((IOpaqueAction) objingElt);
    }

    @objid ("0750ad7f-4583-11e0-b54c-00137279a832")
    public EAddStructuralFeatureValueAction() {
    }

    @objid ("0750ad81-4583-11e0-b54c-00137279a832")
    private void setFeature(IOpaqueAction objingElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        StructuralFeature feature = ((AddStructuralFeatureValueAction) getEcoreElement()).getStructuralFeature();
        IDependency dependency = session.getModel().createDependency();
        
        try{
            dependency.addExtension(session.getMetamodelExtensions()
                    .getStereotype(IDependency.class, XMIStereotypes.UML2STRUCTURALFEATUREREFERENCE));
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
        
        Object behavior = ReverseProperties.getInstance().getMappedElement(feature);
        
        IModelElement obBehavior = null;
        
        if (behavior instanceof IModelElement){
            obBehavior = (IModelElement) behavior;
        }else if (behavior instanceof List<?>){
            obBehavior = ((List<IModelElement>) behavior).get(0);
        }
        
        
        if (obBehavior != null){
            dependency.setDependsOn(obBehavior);
            dependency.setImpacted(objingElt);
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(dependency);
        }
    }

}
