/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IClassAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.INode;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Element;

@objid ("076ae75e-4583-11e0-b54c-00137279a832")
public class EAssociationClass extends ENamedElement implements IEElement {
    @objid ("076ae75f-4583-11e0-b54c-00137279a832")
    private boolean isDeleted = false;

    @objid ("076ae760-4583-11e0-b54c-00137279a832")
    private IClassAssociation objingClassAssociation = null;

    @objid ("076ae761-4583-11e0-b54c-00137279a832")
    private IAssociation objingAssociation = null;

    @objid ("076ae762-4583-11e0-b54c-00137279a832")
    private IClass objingClass = null;


    @objid ("076ae763-4583-11e0-b54c-00137279a832")
    public IClass createObjingElt() {
        // Creation of the association class
        objingClassAssociation = Modelio.getInstance()
                .getModelingSession().getModel().createClassAssociation();
        // Creation of the Class
        objingClass = Modelio.getInstance().getModelingSession()
                .getModel().createClass();
        // Creation of the Association
        objingAssociation = Modelio.getInstance().getModelingSession()
                .getModel().createAssociation();
        // Set of the class and Association
        objingClassAssociation.setClassPart(objingClass);
        objingClassAssociation.setAssociationPart(objingAssociation);
        return objingClass;
    }

    @objid ("076d49b1-4583-11e0-b54c-00137279a832")
    private void attachClass() {
        if (!isDeleted) {
            ReverseProperties revProp = ReverseProperties.getInstance();
                
            Element ecoreOwner = getEcoreElement().getOwner();
                
            if (ecoreOwner != null) {
               
                Object objingOwner =  revProp.getMappedElement(ecoreOwner);
                
                if (objingOwner instanceof IProfile) {
                    objingClass.setOwner(revProp.getExternalPackage());
                }else if (objingOwner instanceof IModelTree) {
                    objingClass.setOwner((IModelTree) objingOwner);
                } else if (objingOwner instanceof IClass) {
                    objingClass.setOwner((IClass) objingOwner);
                } else if (objingOwner instanceof INode) {
                    objingClass.setOwner((INode) objingOwner);
                } else {
                     objingClass.setOwner(revProp.getExternalPackage());
                }
            } else {
                 objingClass.setOwner(revProp.getExternalPackage());
            }
        }
    }

    @objid ("076d49b3-4583-11e0-b54c-00137279a832")
    private void attachAssociation() {
        if (!isDeleted) {
            ReverseProperties revProp = ReverseProperties.getInstance();
               int nbEnds = 0; 
            for (Object memberEnd : ((AssociationClass)getEcoreElement()).getMemberEnds()) {
                
                Object ends = revProp.getMappedElement((Element) memberEnd);
                IAssociationEnd objingAssocEnd = null;
                for (IModelElement end : (ArrayList<IModelElement>) ends ){
                    if (end instanceof IAssociationEnd){
                        objingAssocEnd = (IAssociationEnd) end;
                        break;
                    }
                }
        //                IAssociationEnd objingAssocEnd = (IAssociationEnd) revProp
        //                        .getMappedElement((Element) memberEnd);
                
                if (objingAssocEnd != null) {
                    // Links the AssociationEnd to the Association:
                    objingAssocEnd.setRelated(objingAssociation);
                    nbEnds++;
                }
            }
            if(nbEnds !=2){
                
                Modelio.getInstance().getModelingSession().getModel().deleteElement(objingAssociation);
                objingAssociation = null;
            
            }
            
        }
    }

    @objid ("076d49b5-4583-11e0-b54c-00137279a832")
    private void setClassProperties() {
        if (!isDeleted) {
            ReverseProperties revProp = ReverseProperties.getInstance();
            setClassName();
        //            setVisibility(objingClass);
            if (objingClass instanceof IClass) {
                setClassAbstract();
                setClassLeaf();
                setClassActive();
                
                if (revProp.isRoundtripEnabled()) {
                    setClassPrimitiveEAnnotation();
                    setClassMainEAnnotation();
                }
            }
        }
    }

    @objid ("076d49b7-4583-11e0-b54c-00137279a832")
    private void setClassName() {
        String name = ((AssociationClass)getEcoreElement()).getName();
        if (EcoreModelNavigation.isNotNull(name))
            objingClass.setName(name);
        else 
            objingClass.setName("");
    }

    @objid ("076d49b9-4583-11e0-b54c-00137279a832")
    private void setClassAbstract() {
        objingClass.setAbstract(((AssociationClass)getEcoreElement()).isAbstract());
    }

    @objid ("076fac0a-4583-11e0-b54c-00137279a832")
    private void setClassLeaf() {
        objingClass.setLeaf(((AssociationClass)getEcoreElement()).isLeaf());
    }

    @objid ("076fac0c-4583-11e0-b54c-00137279a832")
    private void setClassActive() {
        objingClass.setActive(((AssociationClass)getEcoreElement()).isActive());
    }

    @objid ("076fac0e-4583-11e0-b54c-00137279a832")
    private void setClassPrimitiveEAnnotation() {
        objingClass.setElementary(ObjingEAnnotation.isPrimitive(((AssociationClass)getEcoreElement())));
    }

    @objid ("076fac10-4583-11e0-b54c-00137279a832")
    private void setClassMainEAnnotation() {
        objingClass.setMain(ObjingEAnnotation.isMain(((AssociationClass)getEcoreElement())));
    }

    @objid ("076fac12-4583-11e0-b54c-00137279a832")
    private void setAssociationProperties() {
        setAssocName();
    }

    @objid ("076fac14-4583-11e0-b54c-00137279a832")
    private void setAssocName() {
        String name = ((AssociationClass)getEcoreElement()).getName();
        if (EcoreModelNavigation.isNotNull(name))
            objingAssociation.setName(name);
        else
            objingAssociation.setName("");
    }

    @objid ("076fac16-4583-11e0-b54c-00137279a832")
    private void deleteElements() {
        PartialImportMap.getInstance().remove((getEcoreElement()));
        TotalImportMap.getInstance().remove((getEcoreElement()));
        IUmlModel model = Modelio.getInstance().getModelingSession()
                .getModel();
        if (objingClass != null)
            model.deleteElement(objingClass);
        if (objingAssociation != null)
            model.deleteElement(objingAssociation);
        if (objingClassAssociation != null)
            model.deleteElement(objingClassAssociation);
        isDeleted = true;
    }

    @objid ("07720e64-4583-11e0-b54c-00137279a832")
    public EAssociationClass(AssociationClass element) {
        super(element);
    }

    @objid ("07720e6d-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        initialize((IClass) objingElt);
        attachClass();
        attachAssociation();
    }

    @objid ("07720e70-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("077470c1-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setClassProperties();
        if (objingAssociation != null)
        setAssociationProperties();
    }

    @objid ("077470c4-4583-11e0-b54c-00137279a832")
    private void initialize(IClass partialClass) {
        try {
            objingClass = partialClass;
            objingClassAssociation = objingClass.getLinkToAssociation().get(0);
            objingAssociation = objingClassAssociation.getAssociationPart();
        } catch (Exception e) {
            deleteElements();
        }
    }

    @objid ("077470c7-4583-11e0-b54c-00137279a832")
    public EAssociationClass() {
    }

}
