/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IObjectNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ObObjectNodeOrderingKindEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.CentralBufferNode;
import org.eclipse.uml2.uml.ObjectNodeOrderingKind;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("07a68220-4583-11e0-b54c-00137279a832")
public class ECentralBufferNode extends EActivityNode implements IEElement {
    @objid ("07a68221-4583-11e0-b54c-00137279a832")
    private CentralBufferNode ecoreElement;


    @objid ("07a68222-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if ( ReverseProperties.getInstance().isRoundtripEnabled()){
            String type = ObjingEAnnotation.getType(ecoreElement);
            if ((type != null) && (type.equals("InstanceNode"))){
                    return Modelio.getInstance().getModelingSession().getModel()
                    .createInstanceNode();
            }
            return Modelio.getInstance().getModelingSession().getModel()
            .createCentralBufferNode();
        }
        return Modelio.getInstance().getModelingSession().getModel()
        .createCentralBufferNode();
    }

    @objid ("07a68226-4583-11e0-b54c-00137279a832")
    public ECentralBufferNode(CentralBufferNode element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("07a8e47a-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("07a8e47d-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("07a8e482-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (objingElt instanceof IObjectNode){
            setControlType((IObjectNode) objingElt);
            setOrdering((IObjectNode) objingElt);
            setSelectionBehavior((IObjectNode) objingElt);
            setUpperBound((IObjectNode) objingElt);
            setType((IObjectNode) objingElt);
            // setRepresented((ICentralBufferNode)objingElt);
            // setRepresentedAttribute((ICentralBufferNode)objingElt);
            // setRepresentedRole((ICentralBufferNode)objingElt);
            // setRepresentedRealParameter((ICentralBufferNode)objingElt);
            setState((IObjectNode) objingElt);
        }
    }

    @objid ("07ab46dd-4583-11e0-b54c-00137279a832")
    private void setControlType(IObjectNode node) {
        node.setControlType(ecoreElement.isControlType());
    }

    @objid ("07ab46e0-4583-11e0-b54c-00137279a832")
    private void setOrdering(IObjectNode node) {
        switch (ecoreElement.getOrdering().getValue()) {
        case ObjectNodeOrderingKind.FIFO:
            node.setOrdering(ObObjectNodeOrderingKindEnum.F_I_F_O);
            break;
        case ObjectNodeOrderingKind.LIFO:
            node.setOrdering(ObObjectNodeOrderingKindEnum.L_I_F_O);
            break;
        case ObjectNodeOrderingKind.ORDERED:
            node.setOrdering(ObObjectNodeOrderingKindEnum.ORDERED);
            break;
        case ObjectNodeOrderingKind.UNORDERED:
            node.setOrdering(ObObjectNodeOrderingKindEnum.UNORDERED);
            break;
        default:
            node.setOrdering(ObObjectNodeOrderingKindEnum.F_I_F_O);
        }
    }

    @objid ("07ab46e3-4583-11e0-b54c-00137279a832")
    private void setSelectionBehavior(IObjectNode node) {
        Behavior ecoreBehavior = ecoreElement.getSelection();
        if (ecoreBehavior instanceof OpaqueBehavior) {
            String objingBehavior = "";
            for (Object body : ((OpaqueBehavior) ecoreBehavior).getBodies()) {
                objingBehavior = objingBehavior.concat((String) body);
            }
            node.setSelectionBehavior(objingBehavior);
        } else if (ecoreBehavior != null) {
            String behaviorName = ecoreBehavior.getName();
            if (behaviorName != null)
                node.setSelectionBehavior(behaviorName);
        }
    }

    @objid ("07ada930-4583-11e0-b54c-00137279a832")
    private void setUpperBound(IObjectNode node) {
        ValueSpecification ecoreUpperBound = ecoreElement.getUpperBound();
        if (ecoreUpperBound != null) {
            String stringValue = ecoreUpperBound.stringValue();
            if (stringValue != null)
                node.setUpperBound(stringValue);
        }
    }

    @objid ("07ada933-4583-11e0-b54c-00137279a832")
    private void setType(IObjectNode node) {
        Type ecoreType = ecoreElement.getType();
        if (ecoreType != null) {
            Object objingType =  ReverseProperties.getInstance().getMappedElement(ecoreType);
            if (objingType instanceof IGeneralClass)
                node.setType((IGeneralClass) objingType);
        }
    }

    @objid ("07ada936-4583-11e0-b54c-00137279a832")
    private void setState(IObjectNode node) {
        EList<?> ecoreStates = ecoreElement.getInStates();
        if (ecoreStates != null && ecoreStates.size() > 0) {
            State ecoreState = (State) ecoreStates.get(0);
            if (ecoreState != null) {
                Object objingState =  ReverseProperties.getInstance().getMappedElement(ecoreState);
                if (objingState instanceof IState)
                    node.setInState((IState) objingState);
            }
        }
    }

    @objid ("07ada939-4583-11e0-b54c-00137279a832")
    public ECentralBufferNode() {
    }

}
