/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.INode;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;

@objid ("07b4d040-4583-11e0-b54c-00137279a832")
public class EClass extends ENamedElement implements IEElement {
    @objid ("07b4d041-4583-11e0-b54c-00137279a832")
    private boolean isDeleted = false;


    @objid ("07b4d042-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession()
        .getModel().createClass();
    }

    @objid ("07b4d046-4583-11e0-b54c-00137279a832")
    public EClass(Class element) {
        super(element);
    }

    @objid ("07b7329c-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (!isDeleted) {
            ReverseProperties revProp = ReverseProperties.getInstance();
        
            Element ecoreOwner = getEcoreElement().getOwner();
        
            IClassifier objingClass = (IClassifier) objingElt;
        
            if (ecoreOwner != null) {
        
                Object objingOwner =  revProp.getMappedElement(ecoreOwner);
                if (objingOwner instanceof IProfile) {
                        objingClass.setOwner(ReverseProperties.getInstance().getExternalPackage());  
                } else if (objingOwner instanceof INode) {
                    //rule R44
                    objingClass.setOwner(ReverseProperties.getInstance().getExternalPackage()); 
                } else if (objingOwner instanceof IModelTree) {
                    objingClass.setOwner((IModelTree) objingOwner);                
                } else {
                    objingClass.setOwner(ReverseProperties.getInstance().getExternalPackage());   
                }
            } else {
                objingClass.setOwner(ReverseProperties.getInstance().getExternalPackage());   
            }
        }
    }

    @objid ("07b7329f-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("07b732a4-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (ReverseProperties.getInstance().isRoundtripEnabled() && ObjingEAnnotation.isDestroy(((Class)getEcoreElement()))){
            deleteElement(objingElt);
        }else{
        
        
            if (!isDeleted && objingElt instanceof IClassifier) {
                ReverseProperties revProp = ReverseProperties.getInstance();
                IClassifier objingClass = (IClassifier) objingElt;
        
                if (objingClass instanceof IClass) {
                    setAbstract((IClass) objingClass);
                    setLeaf((IClass) objingClass);
                    setActive((IClass) objingClass);
        
                    if (revProp.isRoundtripEnabled()) {
                        setPrimitiveEAnnotation((IClass) objingClass);
                        setMainEAnnotation((IClass) objingClass);
                        setRootEAnnotation((IClass) objingClass);
                    }
                }
            }
        }
    }

    @objid ("07b994f2-4583-11e0-b54c-00137279a832")
    private void setAbstract(IClass objingElt) {
        objingElt.setAbstract(((Class)getEcoreElement()).isAbstract());
    }

    @objid ("07b994f5-4583-11e0-b54c-00137279a832")
    private void setActive(IClass objingElt) {
        objingElt.setActive(((Class)getEcoreElement()).isActive());
    }

    @objid ("07b994fb-4583-11e0-b54c-00137279a832")
    private void setLeaf(IClass objingElt) {
        objingElt.setLeaf(((Class)getEcoreElement()).isLeaf());
    }

    @objid ("07b994fe-4583-11e0-b54c-00137279a832")
    private void setMainEAnnotation(IClass objingElt) {
        objingElt.setMain(ObjingEAnnotation.isMain((Class)getEcoreElement()));
    }

    @objid ("07bbf74d-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(IClass objingElt) {
        objingElt.setElementary(ObjingEAnnotation.isPrimitive((Class)getEcoreElement()));
    }

    @objid ("07bbf750-4583-11e0-b54c-00137279a832")
    private void deleteElement(IElement objingElt) {
        PartialImportMap.getInstance().remove((Class)getEcoreElement());
        TotalImportMap.getInstance().remove((Class)getEcoreElement());
        Modelio.getInstance().getModelingSession().getModel()
        .deleteElement(objingElt);
        isDeleted = true;
    }

    @objid ("07bbf753-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(IClass objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot(((Class)getEcoreElement())));
    }

    @objid ("07bbf756-4583-11e0-b54c-00137279a832")
    public EClass() {
    }

}
