/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IConditionalNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.ValueSpecificationAction;

@objid ("07ca4566-4583-11e0-b54c-00137279a832")
public class EClause extends EElement implements IEElement {
    @objid ("07ca4567-4583-11e0-b54c-00137279a832")
    private Clause ecoreElement;


    @objid ("07ca4568-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createClause();
    }

    @objid ("07ca4570-4583-11e0-b54c-00137279a832")
    public EClause(Clause element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("07cca7c6-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Element ecoreOwner = ecoreElement.getOwner();
                
        if (ecoreOwner != null) {
            Object objingOwner =  ReverseProperties.getInstance().getMappedElement(ecoreOwner);
            if (objingOwner instanceof IConditionalNode)
                ((IClause) objingElt)
                        .setOwner((IConditionalNode) objingOwner);
        }
    }

    @objid ("07cca7c9-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("07cca7ce-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {

    	super.setProperties(objingElt);
    	
        setBody((IClause) objingElt);
        setTest((IClause) objingElt);
        setName((IClause) objingElt);
    }

    @objid ("07cf0a1a-4583-11e0-b54c-00137279a832")
    private void setBody(IClause clause) {
        for (Object ecoreBody : ecoreElement.getBodies()) {
            Object objingBody =  ReverseProperties.getInstance()
                    .getMappedElement((ExecutableNode) ecoreBody);
            if (objingBody instanceof IActivityNode)
                ((IActivityNode) objingBody).setOwnerClause(clause);
        }
    }

    @objid ("07cf0a1d-4583-11e0-b54c-00137279a832")
    private void setTest(IClause clause) {
        String objingTest = "";
        for (Object ecoreTest : ecoreElement.getTests()) {
            if (ecoreTest instanceof ValueSpecificationAction) {
                ValueSpecification ecoreValue = ((ValueSpecificationAction) ecoreTest)
                        .getValue();
                if (ecoreValue != null) {
                    String stringValue = ecoreValue.stringValue();
                    if (stringValue != null)
                        objingTest = objingTest.concat(stringValue);
                }
            } else if (ecoreTest != null) {
                String ecoreName = ((ExecutableNode) ecoreTest).getName();
                if (ecoreName != null)
                    objingTest = objingTest.concat(ecoreName);
            }
        }
        clause.setTest(objingTest);
    }

    @objid ("07cf0a20-4583-11e0-b54c-00137279a832")
    private void setName(IClause clause) {
        if ( ReverseProperties.getInstance().isRoundtripEnabled())
            clause.setName(ObjingEAnnotation.getName(ecoreElement));
    }

    @objid ("07cf0a23-4583-11e0-b54c-00137279a832")
    public EClause() {
    }

}
