/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOpaqueAction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.ClearAssociationAction;

@objid ("07d16c74-4583-11e0-b54c-00137279a832")
public class EClearAssociationAction extends EActivityNode implements IEElement {
    @objid ("07d16c75-4583-11e0-b54c-00137279a832")
    private ClearAssociationAction ecoreElement;


    @objid ("07d16c76-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IOpaqueAction element = Modelio.getInstance().getModelingSession().getModel()
        .createOpaqueAction();
                
        try{
            IStereotype stereo = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IOpaqueAction.class, XMIStereotypes.UML2CLEARASSOCIATIONACTION);
            element.addExtension(stereo);
        }catch(StereotypeNotFoundException e){
            Modelio.err.println("StereotypeNotFoundException : ");
            e.printStackTrace(Modelio.err);
        }
        return element;
    }

    @objid ("07d16c7a-4583-11e0-b54c-00137279a832")
    public EClearAssociationAction(ClearAssociationAction element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("07d3cece-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("07d3ced1-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("07d3ced6-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setAssociation((IOpaqueAction) objingElt);
    }

    @objid ("07d63131-4583-11e0-b54c-00137279a832")
    public EClearAssociationAction() {
    }

    @objid ("07d63133-4583-11e0-b54c-00137279a832")
    private void setAssociation(IOpaqueAction objingElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        Association association = ecoreElement.getAssociation();
        if (association != null){
            IModelElement obBehavior = (IModelElement)ReverseProperties.getInstance().getMappedElement(association);
                
            IDependency dependency = session.getModel().createDependency();
            try{
                dependency.addExtension(session.getMetamodelExtensions()
                        .getStereotype(IDependency.class, XMIStereotypes.UML2ASSOCIATIONREFERENCE));
            }catch(StereotypeNotFoundException e){
                
            }
                
            dependency.setDependsOn(obBehavior);
            dependency.setImpacted(objingElt);
        }
    }

}
