/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.ICollaboration;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Element;

@objid ("07e6e19f-4583-11e0-b54c-00137279a832")
public class ECollaboration extends ENamedElement implements IEElement {
    @objid ("07e6e1a0-4583-11e0-b54c-00137279a832")
    private Collaboration ecoreElement;


    @objid ("07e6e1a1-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createCollaboration();
    }

    @objid ("07e6e1a5-4583-11e0-b54c-00137279a832")
    public ECollaboration(Collaboration element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("07e943fc-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Element ecoreOwner = ecoreElement.getOwner();
        
        IModelElement objingOwner = (IModelElement) revProp
        .getMappedElement(ecoreOwner);
        ICollaboration objingCollab = (ICollaboration) objingElt;
        
        if (objingOwner instanceof IBehavior) {                
            objingCollab.setBRepresented((IBehavior) objingOwner);
        } else if (objingOwner instanceof IProfile) {
            objingCollab.setOwner(revProp.getExternalPackage());
        } else if (objingOwner instanceof IModelTree) {
            objingCollab.setOwner((IModelTree) objingOwner);
        } else {
            objingCollab.setOwner(revProp.getExternalPackage());
        }
    }

    @objid ("07e943ff-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("07e94404-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setAbstract((ICollaboration) objingElt);
        setLeaf((ICollaboration) objingElt);
        //        setVisibility(objingElt);
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setConcurrentEAnnotation((ICollaboration) objingElt);
            setRootEAnnotation((ICollaboration)objingElt);
        }
    }

    @objid ("07eba655-4583-11e0-b54c-00137279a832")
    private void setAbstract(ICollaboration objingElt) {
        objingElt.setAbstract(ecoreElement.isAbstract());
    }

    @objid ("07eba658-4583-11e0-b54c-00137279a832")
    private void setLeaf(ICollaboration objingElt) {
        objingElt.setLeaf(ecoreElement.isLeaf());
    }

    @objid ("07eba65b-4583-11e0-b54c-00137279a832")
    private void setConcurrentEAnnotation(ICollaboration objingElt) {
        objingElt.setConcurrent(ObjingEAnnotation.isConcurrent(ecoreElement));
    }

    @objid ("07eba65e-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(ICollaboration objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot(ecoreElement));
    }

    @objid ("07ee08ac-4583-11e0-b54c-00137279a832")
    public ECollaboration() {
    }

}
