/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.NoteTypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Element;

@objid ("07f9f46e-4583-11e0-b54c-00137279a832")
public class EComment extends EElement implements IEElement {
    @objid ("07f9f474-4583-11e0-b54c-00137279a832")
    private Comment ecoreElement;


    @objid ("07f9f475-4583-11e0-b54c-00137279a832")
    public ArrayList<INote> createObjingElt() {
        return new ArrayList<INote>();
    }

    @objid ("07fc56cc-4583-11e0-b54c-00137279a832")
    private INote createNote() {
        INote result = Modelio.getInstance().getModelingSession().getModel()
                .createNote();
        try{
            result.setModel(Modelio.getInstance().getModelingSession().getMetamodelExtensions().getNoteType(IModelElement.class, "comment"));
        
        }catch(NoteTypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
        return result;
    }

    @objid ("07fc56d0-4583-11e0-b54c-00137279a832")
    public EComment(Comment element) {
        super(element);
        this.ecoreElement = element;
    }

    @objid ("07feb923-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("07feb926-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        List<Element> annotatedElementList = this.ecoreElement.getAnnotatedElements();
        if (annotatedElementList == null || annotatedElementList.size() == 0) {
            annotatedElementList = new ArrayList<Element>();
            annotatedElementList.add(this.ecoreElement.getOwner());
        }
        
        
        for (Object annotatedElement : annotatedElementList) {
            Element ecoreAnnotatedElt = (Element) annotatedElement;
            Object objingAnnotatedElt = revProp.getMappedElement(ecoreAnnotatedElt);
        
            if ((objingAnnotatedElt != null) && (objingAnnotatedElt instanceof IElement)) {
                if (ecoreAnnotatedElt instanceof AssociationClass){
                    if (ObjingEAnnotation.isOwnedByAssociation(this.ecoreElement)){
                        createNote(((IClass) objingAnnotatedElt).getLinkToAssociation().get(0).getAssociationPart(), objingElt,
                                   ecoreAnnotatedElt, revProp);
                    }else if (ObjingEAnnotation.isOwnedByAssociationClass(this.ecoreElement)){
                        createNote(((IClass) objingAnnotatedElt).getLinkToAssociation().get(0), objingElt,
                                   ecoreAnnotatedElt, revProp);
                    }else{
                        createNote((IElement) objingAnnotatedElt, objingElt,
                                   ecoreAnnotatedElt, revProp);
                    }
                }else{
        
                    createNote((IElement) objingAnnotatedElt, objingElt,
                               ecoreAnnotatedElt, revProp);
                }
            } else if (objingAnnotatedElt instanceof ArrayList) {
                Iterator<Object> it = ((ArrayList<Object>) objingAnnotatedElt).iterator();
                while (it.hasNext()) {
                    IElement objingAnnotatedElt2 = (IElement) it.next();
                    if ((objingAnnotatedElt2 != null) && (objingAnnotatedElt2 instanceof IElement)) {
                        createNote(objingAnnotatedElt2, objingElt,
                                   ecoreAnnotatedElt, revProp);
                    }
                }
            }
        }
    }

    @objid ("07feb92b-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt != null) {
            ReverseProperties revProp = ReverseProperties.getInstance();
        
            setBody((INote) objingElt);
        
            if (revProp.isRoundtripEnabled()) {
                setModel((INote) objingElt);// Sets the NoteType.
            } 
        
        }
        

    	super.setProperties(objingElt);
    }

    @objid ("07feb92e-4583-11e0-b54c-00137279a832")
    public void createNote(IElement objingAnnotatedElt, List<Object> objingElt, Element ecoreAnnotatedElt, ReverseProperties revProp) {
        if (objingAnnotatedElt != null && objingAnnotatedElt instanceof IModelElement) {
            INote objingNote = createNote();
            objingNote.setSubject((IModelElement) objingAnnotatedElt);
            ((IModelElement) objingAnnotatedElt).addDescriptor(objingNote);
            objingElt.add(objingNote);
        }
    }

    @objid ("08011b81-4583-11e0-b54c-00137279a832")
    private void setBody(INote objingElt) {
        String body = this.ecoreElement.getBody();
        if (body != null)
            objingElt.setContent(body);
    }

    @objid ("08011b84-4583-11e0-b54c-00137279a832")
    private void setModel(INote objingElt) {
        String baseClass = objingElt.getSubject().getClass().getCanonicalName();
        String noteTypeName = ObjingEAnnotation.getNoteTypeName(this.ecoreElement);
        
        INoteType objingNoteType = null;
        if (noteTypeName != null)
            objingNoteType = ObjingModelNavigation.getNoteType(baseClass,
                                                               noteTypeName);
        
        if (objingNoteType != null) {
            objingElt.setModel(objingNoteType);
        }
    }

    @objid ("08011b87-4583-11e0-b54c-00137279a832")
    public EComment() {
    }

}
