/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ComponentRealization;
import org.eclipse.uml2.uml.Element;

@objid ("0811cbfa-4583-11e0-b54c-00137279a832")
public class EComponentRealization extends ERealization implements IEElement {

    @objid ("0811cbfc-4583-11e0-b54c-00137279a832")
    public List<IElement> createObjingElt() {
        List<IElement> result = new ArrayList<IElement>();
        ComponentRealization ecoreElt = (ComponentRealization) getEcoreElement();
        
        if (ecoreElt.getRealizingClassifiers() != null){
        
            int size = ecoreElt.getRealizingClassifiers().size();
            for (int i =0; i < size; i++){
                IDependency dep = Modelio.getInstance().getModelingSession().getModel().createDependency();
                try{
                    dep.addExtension(Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotype(
                            IDependency.class, XMIStereotypes.UML2COMPONENTREALIZATION));
                }catch(StereotypeNotFoundException e){
                    e.printStackTrace(Modelio.err);
                }
                result.add(dep);
            }
        }
        return result;
    }

    @objid ("08142e4d-4583-11e0-b54c-00137279a832")
    public EComponentRealization(ComponentRealization element) {
        super(element);
    }

    @objid ("08142e56-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("081690a6-4583-11e0-b54c-00137279a832")
    public void attach(final List<Object> objingElts) {
        ComponentRealization ecoreElt = (ComponentRealization) getEcoreElement();
        Element client = ecoreElt.getAbstraction();
        
        if (client != null){
            Object objClient = ReverseProperties.getInstance().getMappedElement(client);
            if (objClient instanceof IModelElement){
                int i = 0; 
                for (Object objingElt : objingElts){
                    Classifier classifier = ecoreElt.getRealizingClassifiers().get(i);
                    Object objClassifier = ReverseProperties.getInstance().getMappedElement(classifier);
                    if ((objingElt instanceof IDependency) && (objClassifier instanceof IModelElement)){
                        IDependency dep = (IDependency)objingElt;
                        dep.setDependsOn((IModelElement)objClient);
                        dep.setImpacted((IModelElement) objClassifier);
                    }
                    i++;
                }
            }
        }
    }

    @objid ("081690ab-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
    }

    @objid ("081690ae-4583-11e0-b54c-00137279a832")
    public EComponentRealization() {
    }

}
