/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IAbstraction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IConstraint;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Abstraction;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0848a209-4583-11e0-b54c-00137279a832")
public class EConstraint extends ENamedElement implements IEElement {
    @objid ("0848a20a-4583-11e0-b54c-00137279a832")
    private boolean isDeleted = false;

    @objid ("0848a20b-4583-11e0-b54c-00137279a832")
    private boolean isCondition = false;

    @objid ("0848a20c-4583-11e0-b54c-00137279a832")
    private Constraint ecoreElement;

    @objid ("6a5b1920-1a76-11e1-8103-0027103f347d")
    private boolean isLinked = false;


    @objid ("0848a20d-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if (!isCondition && isLinked){
            return Modelio.getInstance().getModelingSession().getModel()
            .createConstraint();
        }
        return null;
    }

    @objid ("0848a211-4583-11e0-b54c-00137279a832")
    public EConstraint(Constraint element) {
        super(element);
        ecoreElement = element;
        isCondition = EcoreModelNavigation.isCondition(element);
        isLinked = EcoreModelNavigation.isLinked(element);
    }

    @objid ("084b0468-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        // The current objingElt may be a StateInvariant -> repport to
        // StateInvariant's "setBody()" method for
        // more explanations.
        /*if (objingElt instanceof ICondition)
            attachCondition((ICondition) objingElt);
        else */
        if (objingElt instanceof IConstraint)
            attachConstraint((IConstraint) objingElt);
    }

    @objid ("084b046b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("084fc917-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (!isDeleted) {
            /*if (isCondition)
                setName((ICondition) objingElt);
            else */
            if (objingElt instanceof IConstraint) {
                setBody((IConstraint) objingElt);
            }
        }
    }

    @objid ("08522b73-4583-11e0-b54c-00137279a832")
    private void setBody(IConstraint objingElt) {
        ValueSpecification specification = ecoreElement.getSpecification();
        if (specification != null) {
            String body = specification.stringValue();
            if (body != null)
                objingElt.setBody(body);
        }
    }

    @objid ("08522b76-4583-11e0-b54c-00137279a832")
    private void attachConstraint(IConstraint constraint) {
        EList<?> ecoreConstrainedElts = ecoreElement.getConstrainedElements();
        boolean attached = false;
        if (ecoreConstrainedElts.size() > 0) {
            for (Object elt : ecoreConstrainedElts) {
                if ( (elt instanceof Element) && (elt != null)){
                    Element ecoreConstrainedElt = (Element) elt;
                    Object object =  ReverseProperties.getInstance().getMappedElement(ecoreConstrainedElt);
                
                    if (object instanceof IModelElement){
                        IModelElement objingConstrainedElt = (IModelElement) object;
                        constraint.addConstrainedElement(objingConstrainedElt);
                        attached = true;
                    }else if (ecoreConstrainedElt instanceof Abstraction){
                        List<IAbstraction> list = (ArrayList<IAbstraction>) object;
                        for (IAbstraction objingConstrainedElt : list){
                            constraint.addConstrainedElement(objingConstrainedElt);
                            attached = true;
                        }
                    }else if (ecoreConstrainedElt instanceof Comment){
                        List<INote> list = (ArrayList<INote>) object;
                        for (INote objingConstrainedElt : list){
                            constraint.addConstrainedElement(objingConstrainedElt);
                            attached = true;
                        }
                    }else if(ecoreConstrainedElt instanceof Usage){
                
                        List<IModelElement> list = (ArrayList<IModelElement>) object;
                        for (IModelElement objingConstrainedElt : list){
                            constraint.addConstrainedElement(objingConstrainedElt);
                            attached = true;
                        }
                
                    }else if (ecoreConstrainedElt instanceof Dependency){
                        List<IDependency> list = (ArrayList<IDependency>) object;
                        for (IDependency objingConstrainedElt : list){
                            constraint.addConstrainedElement(objingConstrainedElt);
                            attached = true;
                        }
                    }
                }
            }
            
            if (!attached){
                PartialImportMap.getInstance().remove(ecoreElement);
                TotalImportMap.getInstance().remove(ecoreElement);
                Modelio.getInstance().getModelingSession().getModel()
                .deleteElement(constraint);
                isDeleted = true;
            }
        } else {
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(constraint);
            isDeleted = true;
        }
    }

    @objid ("08522b7c-4583-11e0-b54c-00137279a832")
    public EConstraint() {
    }

}
