/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IControlFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInterruptibleActivityRegion;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ControlFlow;
import org.eclipse.uml2.uml.InterruptibleActivityRegion;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("08595280-4583-11e0-b54c-00137279a832")
public class EControlFlow extends ENamedElement implements IEElement {
    @objid ("08595281-4583-11e0-b54c-00137279a832")
    private ReverseProperties revProp = ReverseProperties.getInstance();

    @objid ("08595282-4583-11e0-b54c-00137279a832")
    private ControlFlow ecoreElement;


    @objid ("08595283-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ActivityNode ecoreSource = ecoreElement.getSource();
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if ((ecoreSource != null) && (ecoreTarget != null)) {
            Object objingSource = revProp.getMappedElement(ecoreSource);
            if (objingSource != null){
                if (objingSource instanceof IActivityNode)
                    return Modelio.getInstance().getModelingSession().getModel()
                    .createControlFlow();
            }
        }
        return null;
    }

    @objid ("08595287-4583-11e0-b54c-00137279a832")
    public EControlFlow(ControlFlow element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("085bb4de-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        // Attachment is done when setting the Source of the Edge.
    }

    @objid ("085bb4e1-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("085e1732-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setInterrupts((IControlFlow) objingElt);
        setGuard((IControlFlow) objingElt);
        setWeight((IControlFlow) objingElt);
        
        setTarget((IControlFlow) objingElt);
        setSource((IControlFlow) objingElt);
    }

    @objid ("085e1738-4583-11e0-b54c-00137279a832")
    private void setSource(IControlFlow flow) {
        ActivityNode ecoreSource = ecoreElement.getSource();
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if ((ecoreSource != null) && (ecoreTarget != null)) {
            Object objingSource = revProp.getMappedElement(ecoreSource);
            if (objingSource != null){
                if (objingSource instanceof IActivityNode)
                    ((IActivityEdge) flow)
                    .setSource((IActivityNode) objingSource);
            }else{
                Modelio.getInstance().getModelingSession().getModel().deleteElement(flow);
            }
        
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(flow);
        }
    }

    @objid ("085e173b-4583-11e0-b54c-00137279a832")
    private void setTarget(IControlFlow flow) {
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if (ecoreTarget != null) {
            Object objingTarget = revProp.getMappedElement(ecoreTarget);
            if (objingTarget instanceof IActivityNode)
                ((IActivityEdge) flow)
                .setTarget((IActivityNode) objingTarget);
        }
    }

    @objid ("085e173e-4583-11e0-b54c-00137279a832")
    private void setInterrupts(IControlFlow flow) {
        InterruptibleActivityRegion ecoreRegion = ecoreElement.getInterrupts();
        if (ecoreRegion != null) {
            Object objingRegion = revProp.getMappedElement(ecoreRegion);
            if (objingRegion instanceof IInterruptibleActivityRegion)
                ((IActivityEdge) flow)
                .setInterrupts((IInterruptibleActivityRegion) objingRegion);
        }
    }

    @objid ("085e1741-4583-11e0-b54c-00137279a832")
    private void setGuard(IControlFlow flow) {
        ValueSpecification ecoreGuard = ecoreElement.getGuard();
        if (ecoreGuard != null) {
            String stringValue = ecoreGuard.stringValue();
            if (stringValue != null)
                flow.setGuard(stringValue);
            else
                flow.setGuard("");
        
        }else
            flow.setGuard("");
    }

    @objid ("0860798c-4583-11e0-b54c-00137279a832")
    private void setWeight(IControlFlow flow) {
        boolean isSet = false;
        ValueSpecification ecoreWeight = ecoreElement.getWeight();
        if (ecoreWeight != null) {
            String stringValue = ecoreWeight.stringValue();
            if (stringValue != null) {
                flow.setWeight(stringValue);
                isSet = true;
            }
        }
        
        if (!isSet)
            flow.setWeight("1");
    }

    @objid ("0860798f-4583-11e0-b54c-00137279a832")
    public EControlFlow() {
    }

}
