/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * This class manages the import of  Ecore DataType
 * @author ebrosse
 */
@objid ("0881da80-4583-11e0-b54c-00137279a832")
public class EDataType extends ENamedElement implements IEElement {
    @objid ("0881da81-4583-11e0-b54c-00137279a832")
    private DataType ecoreElement;


    @Override
    @objid ("0881da82-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if (!PrimitiveTypeMapper.isPredefinedType(this.ecoreElement)) {
            return Modelio.getInstance().getModelingSession().getModel().createDataType();
        }
        return null;
    }

    /**
     * Constructor with the imported Ecore DataType
     * @param element : the imported Ecore DataType
     */
    @objid ("08843cd2-4583-11e0-b54c-00137279a832")
    public EDataType(DataType element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("08843cdb-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (!((this.ecoreElement instanceof PrimitiveType) || (this.ecoreElement instanceof Enumeration))) {
        
            IModelTree objingDTImport = (IModelTree) objingElt;
        
            IModelTree objingOwner = EcoreModelNavigation.getNearestDataTypeOwner(this.ecoreElement);
        
            if (objingOwner != null){
                if (objingOwner instanceof IProfile){
                    objingDTImport.setOwner(ReverseProperties.getInstance().getExternalPackage());
                }else {
                    objingDTImport.setOwner(objingOwner);
                }
            }
        }
    }

    @Override
    @objid ("08843cde-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("08869f2f-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (!(PrimitiveTypeMapper.isPredefinedType(this.ecoreElement))) {
            super.setProperties(objingElt);
            if (!(this.ecoreElement instanceof PrimitiveType)) {
                setVisibility((IGeneralClass) objingElt);
                if (objingElt instanceof IDataType) {
                    setAbstract((IDataType) objingElt);
                    setLeaf((IDataType) objingElt);
        
                    if (ReverseProperties.getInstance().isRoundtripEnabled()) {
                        setPrimitiveEAnnotation((IDataType) objingElt);
                        setRootEAnnotation((IDataType) objingElt);
                    }
                }
            }
        }
    }

    @objid ("08869f32-4583-11e0-b54c-00137279a832")
    private void setVisibility(IGeneralClass objingElt) {
        switch (this.ecoreElement.getVisibility().getValue()) {
        case VisibilityKind.PUBLIC:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
            break;
        case VisibilityKind.PRIVATE:
            objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
            break;
        case VisibilityKind.PACKAGE:
            objingElt.setVisibility(ObVisibilityModeEnum.PACKAGE_VISIBILITY);
            break;
        case VisibilityKind.PROTECTED:
            objingElt.setVisibility(ObVisibilityModeEnum.PROTECTED);
            break;
        default:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
        }
        
        if (ObjingEAnnotation.isUndefined(this.ecoreElement))
            objingElt
            .setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
    }

    @objid ("08869f38-4583-11e0-b54c-00137279a832")
    private void setAbstract(IDataType objingElt) {
        objingElt.setAbstract(this.ecoreElement.isAbstract());
    }

    @objid ("08869f3b-4583-11e0-b54c-00137279a832")
    private void setLeaf(IDataType objingElt) {
        objingElt.setLeaf(this.ecoreElement.isLeaf());
    }

    @objid ("08890188-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(IDataType objingElt) {
        objingElt.setElementary(ObjingEAnnotation.isPrimitive(this.ecoreElement));
    }

    @objid ("0889018b-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(IDataType objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot(this.ecoreElement));
    }

    /**
     * The default constructor
     */
    @objid ("0889018e-4583-11e0-b54c-00137279a832")
    public EDataType() {
    }

}
