/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IBinding;
import com.modeliosoft.modelio.api.model.uml.statik.ICollaborationUse;
import com.modeliosoft.modelio.api.model.uml.statik.IConnector;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.statik.IProvidedInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IRequiredInterface;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Abstraction;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Manifestation;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Usage;

@objid ("08928aee-4583-11e0-b54c-00137279a832")
public class EDependency extends ENamedElement implements IEElement {
    @objid ("08928aef-4583-11e0-b54c-00137279a832")
    private Dependency ecoreElement;


    @objid ("08928af0-4583-11e0-b54c-00137279a832")
    public List<IDependency> createObjingElt() {
        return new ArrayList<IDependency>();
    }

    @objid ("08928af6-4583-11e0-b54c-00137279a832")
    public EDependency(Dependency element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0894ed4c-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0894ed4f-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (ObjingEAnnotation.isDestroy(ecoreElement) || ObjingEAnnotation.isDeleted(ecoreElement)){
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        }else{
            setFlow((IModelElement) objingElt);
        }
    }

    @objid ("0894ed52-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
        if (ReverseProperties.getInstance().isRoundtripEnabled() && ObjingEAnnotation.isProvidedInterface(ecoreElement)){ 
            attachProvidedInterface(objingElt);
        }else if (ReverseProperties.getInstance().isRoundtripEnabled() && ObjingEAnnotation.isRequiredInterface(ecoreElement)){
            attachRequiredInterface(objingElt);    
        }else{
            attachDependency(objingElt);
        }
    }

    @objid ("0894ed57-4583-11e0-b54c-00137279a832")
    private void attachDependency(List<Object> objingElt) {
        EList<?> clientList = ecoreElement.getClients();
        EList<?> supplierList = ecoreElement.getSuppliers();
        
        for (Object eClient : clientList) {
            if (eClient instanceof NamedElement) {
                NamedElement ecoreClient = (NamedElement) eClient;
        
                IModelElement objingClient = getObjingEltFromMap(ecoreClient);
        
                if (objingClient != null) {
                    for (Object eSupplier : supplierList) {
                        if (eSupplier instanceof NamedElement) {
                            NamedElement ecoreSupplier = (NamedElement) eSupplier;
        
                            IModelElement objingSupplier = getObjingEltFromMap(ecoreSupplier);
        
                            if (objingSupplier != null) {
                                // Warning : unlike in UML2, in Ijing,
                                // Manifestation does no inherit from Dependency
                                IElement objingTypeOfDependency = createTypeOfDependency(
                                        objingClient, objingSupplier);
                                if (objingTypeOfDependency != null
                                        && !objingElt
                                        .contains(objingTypeOfDependency))
                                    objingElt.add(objingTypeOfDependency);
                            }
                        }
                    }
                }
            }
        }
    }

    @objid ("08974fa6-4583-11e0-b54c-00137279a832")
    private void attachProvidedInterface(List<Object> objingElt) {
        EList<?> clientList = ecoreElement.getClients();
        EList<?> supplierList = ecoreElement.getSuppliers();
        
        for (Object eClient : clientList) {
            if (eClient instanceof NamedElement) {
                NamedElement ecoreClient = (NamedElement) eClient;
        
                IModelElement objingClient = getObjingEltFromMap(ecoreClient);
        
                if (objingClient != null) {
        
                    List<IModelElement> objingSuppliers = new ArrayList<IModelElement>();
                    for (Object eSupplier : supplierList) {
        
                        if (eSupplier instanceof NamedElement) {
                            NamedElement ecoreSupplier = (NamedElement) eSupplier;
        
                            IModelElement objingSupplier = getObjingEltFromMap(ecoreSupplier);
        
                            if (objingSupplier != null) {
                                objingSuppliers.add(objingSupplier);
                            }
                        }
                    }
        
                    IProvidedInterface provided = createProvidedInterface(objingClient, objingSuppliers);
        
                    objingElt.add(provided);
                }
            }
        }
    }

    @objid ("08974fab-4583-11e0-b54c-00137279a832")
    private void attachRequiredInterface(List<Object> objingElt) {
        EList<?> clientList = ecoreElement.getClients();
        EList<?> supplierList = ecoreElement.getSuppliers();
        
        for (Object eSupplier : supplierList) {
            if (eSupplier instanceof NamedElement) {
                NamedElement ecoreSupplier = (NamedElement) eSupplier;
        
                IModelElement objingSupplier = getObjingEltFromMap(ecoreSupplier);
        
                if (objingSupplier != null) {
        
                    List<IModelElement> objingClients = new ArrayList<IModelElement>();
                    for (Object eClient : clientList) {
        
                        if (eClient instanceof NamedElement) {
                            NamedElement ecoreClient = (NamedElement) eClient;
        
                            IModelElement objingClient = getObjingEltFromMap(ecoreClient);
        
                            if (objingClient != null) {
                                objingClients.add(objingClient);
                            }
                        }
                    }
        
                    IRequiredInterface required = createRequiredInterface(objingClients, objingSupplier);
        
                    objingElt.add(required);
                }
            }
        }
    }

    @objid ("0899b1fc-4583-11e0-b54c-00137279a832")
    private IElement createTypeOfDependency(IModelElement objingClient, IModelElement objingSupplier) {
        IElement objingTypeOfDependency = null;
        
        if (ecoreElement instanceof Manifestation) {
            objingTypeOfDependency = createManifestation(objingClient,
                    objingSupplier);
            //            } else if (revProp.isRoundtripEnabled() && ObjingEAnnotation.isProvidedInterface(ecoreElement)){
            //            objingTypeOfDependency = createProvidedInterface(objingClient,
            //            objingSupplier);
            //            }else if (revProp.isRoundtripEnabled() && ObjingEAnnotation.isRequiredInterface(ecoreElement)){
            //            objingTypeOfDependency = createRequiredInterface(objingClient,
            //            objingSupplier);
        }else if ((ecoreElement.getOwner() instanceof CollaborationUse) 
                && (((CollaborationUse)ecoreElement.getOwner()).getRoleBindings().contains(ecoreElement) )){
        
            objingTypeOfDependency = createBinding(objingClient,
                    objingSupplier, (CollaborationUse) ecoreElement.getOwner());
        }else{
        
            objingTypeOfDependency = createDependency(objingClient,
                    objingSupplier);
        }
        return objingTypeOfDependency;
    }

    @objid ("0899b202-4583-11e0-b54c-00137279a832")
    private IBinding createBinding(IModelElement objingClient, IModelElement objingSupplier, CollaborationUse ecoreOwner) {
        IBinding objElement = Modelio.getInstance().getModelingSession().getModel().createBinding();
        
        ICollaborationUse objOwner = (ICollaborationUse) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        objElement.setOwner(objOwner);
        
        for (Object client : ecoreElement.getClients()){
            IElement objClient = (IElement) ReverseProperties.getInstance().getMappedElement((Element)client);
        
            if (objClient instanceof IConnectorEnd){
                objElement.setConnectorEndRole((IConnectorEnd) objClient);
            }else if (objClient instanceof IConnector){
                objElement.setConnectorRole((IConnector) objClient);
            }else if (objClient instanceof IBindableInstance){
                objElement.setRole((IBindableInstance) objClient);
            }
        }
        
        for(Object supplier : ecoreElement.getSuppliers()){
            IElement objSupplier = (IElement) ReverseProperties.getInstance().getMappedElement((Element)supplier);
        
            if (objSupplier instanceof IModelElement)
                objElement.setRepresentedFeature((IModelElement) objingSupplier);
        }
        return objElement;
    }

    @objid ("0899b209-4583-11e0-b54c-00137279a832")
    private IManifestation createManifestation(IModelElement objingClient, IModelElement objingSupplier) {
        if (objingClient instanceof IArtifact) {
            IManifestation manif = Modelio.getInstance()
            .getModelingSession().getModel().createManifestation();
        
            String name = ecoreElement.getName();
            if (EcoreModelNavigation.isNotNull(name))
                manif.setName(name);
            else 
                manif.setName("");
        
            manif.setOwner((IArtifact) objingClient);
            manif.setUtilizedElement(objingSupplier);
            return manif;
        }
        return null;
    }

    @objid ("089c1457-4583-11e0-b54c-00137279a832")
    private IRequiredInterface createRequiredInterface(List<IModelElement> objingClients, IModelElement objingSupplier) {
        IRequiredInterface required = Modelio.getInstance()
        .getModelingSession().getModel().createRequiredInterface();
        
        required.setRequiring((IPort)objingSupplier);
        for (IModelElement objingClient :  objingClients)
            required.addRequiredElement((IInterface)objingClient);
        return required;
    }

    @objid ("089c145f-4583-11e0-b54c-00137279a832")
    private IProvidedInterface createProvidedInterface(IModelElement objingClient, List<IModelElement> objingSuppliers) {
        IProvidedInterface provided = Modelio.getInstance()
        .getModelingSession().getModel().createProvidedInterface();
        
        provided.setProviding((IPort)objingClient);
        for (IModelElement objingSupplier :  objingSuppliers)
            provided.addProvidedElement((IInterface) objingSupplier);
        return provided;
    }

    @objid ("089e76b1-4583-11e0-b54c-00137279a832")
    private IDependency createDependency(IModelElement objingClient, IModelElement objingSupplier) {
        IDependency objingDependency = null;
        if (ecoreElement instanceof Abstraction) {
            objingDependency = Modelio.getInstance().getModelingSession().getModel().createDependency();
            try{
                objingDependency.addExtension(Modelio.getInstance().getModelingSession().getMetamodelExtensions()
                        .getStereotype(IDependency.class, XMIStereotypes.UML2ABSTRACTION));
            }catch(StereotypeNotFoundException e){
                e.printStackTrace(Modelio.err);
            }
        
        } else if (ecoreElement instanceof Usage) {
            objingDependency = Modelio.getInstance().getModelingSession()
            .getModel().createUsage();
        } else if (ecoreElement instanceof Dependency) {
            objingDependency = Modelio.getInstance().getModelingSession()
            .getModel().createDependency();
        }
        
        if (objingDependency != null){
            String name = ecoreElement.getName();
            if (EcoreModelNavigation.isNotNull(name))
                objingDependency.setName(name);
            else 
                objingDependency.setName("");
            objingDependency.setImpacted(objingClient);
            objingDependency.setDependsOn(objingSupplier);
        }
        return objingDependency;
    }

    @objid ("089e76b7-4583-11e0-b54c-00137279a832")
    private IModelElement getObjingEltFromMap(final NamedElement ecoreClient) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        IModelElement objingClient = null;
        
        Object objClient = revProp.getMappedElement(ecoreClient);
        
        // Case of the client is itself a dependency => list of objing elts
        if (objClient instanceof ArrayList
                && ((ArrayList<?>) objClient).size() > 0) {
            objClient = ((ArrayList<Object>) objClient).get(0);
        }
        
        // not else -> objClient may have changed
        if (objClient instanceof IModelElement) {
            objingClient = (IModelElement) objClient;
        }
        return objingClient;
    }

    @objid ("089e76bc-4583-11e0-b54c-00137279a832")
    private void setFlow(IModelElement objingElt) {
        if (ObjingEAnnotation.isFlow(ecoreElement)){
            try {
                IStereotype ster = Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotype(IDependency.class,"flow");
        
                if (!objingElt.isStereotyped(ster.getName()))
                    objingElt.addExtension(ster);
            } catch (StereotypeNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    @objid ("08a0d90a-4583-11e0-b54c-00137279a832")
    public EDependency() {
    }

}
