/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  

package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;
import org.eclipse.uml2.uml.Abstraction;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Manifestation;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.internal.impl.EnumerationLiteralImpl;
import org.eclipse.uml2.uml.internal.impl.ExtensionImpl;
import org.eclipse.uml2.uml.internal.impl.PrimitiveTypeImpl;

import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.exceptions.ObElementNotFoundException;
import com.modeliosoft.modelio.core.app.O;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;

@objid ("08da117a-4583-11e0-b54c-00137279a832")
public class EElement implements IEElement{
	@objid ("08da117b-4583-11e0-b54c-00137279a832")
	private Element ecoreElt = null;


	@objid ("08da117f-4583-11e0-b54c-00137279a832")
	public Element getEcoreElement() {
		return this.ecoreElt;
	}

	@objid ("08dc73d8-4583-11e0-b54c-00137279a832")
	public void setStereotypes() {
		try{
			Object object = ReverseProperties.getInstance().getMappedElement(this.ecoreElt);

			if (object != null){
				if (object instanceof IModelElement){
					IModelElement modelElement = (IModelElement) object;
					if ((modelElement.getTag() != null) && (modelElement.getTag().size() == 0) 
							&& (modelElement.getExtension().size() == 0))
						for ( Object stereo : this.ecoreElt.getAppliedStereotypes()){
							setStereotype(modelElement, (Stereotype) stereo);
						}                    setStereotypesAnnotation(modelElement);
				}else if (object instanceof List<?>){
					for (Object createdElt : (List<?>) object){
						if (createdElt instanceof IModelElement){
							IModelElement modelElement = (IModelElement) createdElt;
							if ((modelElement.getTag() != null) && (modelElement.getTag().size() == 0) 
									&& (modelElement.getExtension().size() == 0))
								for (Object stereo : this.ecoreElt.getAppliedStereotypes()){
									setStereotype(modelElement, (Stereotype) stereo);
								}
							setStereotypesAnnotation(modelElement);
						}
					}
				}else if (this.ecoreElt instanceof Manifestation){
					List<IManifestation> list = (ArrayList<IManifestation>) object;
					for (IManifestation manifestation : list)
						if ( (manifestation.getTag() != null) && (manifestation.getTag().size() == 0) && (manifestation.getExtension().size() == 0))
							for ( Object stereo : this.ecoreElt.getAppliedStereotypes())
							{
								setStereotype(manifestation, (Stereotype) stereo);
							}
				}else if (this.ecoreElt instanceof Abstraction){
					List<IDependency> list = (ArrayList<IDependency>) object;
					for (IDependency abstraction : list)
						if ( (abstraction.getTag() != null) && (abstraction.getTag().size() == 0) && (abstraction.getExtension().size() == 1))
							for ( Object stereo : this.ecoreElt.getAppliedStereotypes())
							{
								setStereotype(abstraction, (Stereotype) stereo);
							}
				}else if (this.ecoreElt instanceof Comment){
					List<INote> list = (ArrayList<INote>) object;
					for (INote note : list)
						if ( (note.getTag() != null) && (note.getTag().size() == 0) && (note.getExtension().size() == 0))
							for ( Object stereo : this.ecoreElt.getAppliedStereotypes())
							{
								setStereotype(note, (Stereotype) stereo);
							}

				}else if (this.ecoreElt instanceof Dependency){
					List<IDependency> list = (ArrayList<IDependency>) object;
					for (IModelElement dependency : list)
						if ( (dependency.getTag() != null) && (dependency.getTag().size() == 0) && (dependency.getExtension().size() == 0))
							for ( Object stereo : this.ecoreElt.getAppliedStereotypes())
							{
								setStereotype(dependency, (Stereotype) stereo);
							}

				}else if(this.ecoreElt instanceof Usage){
					List<IModelElement> list = (ArrayList<IModelElement>) object;
					for (IModelElement usage : list)
						if ((usage.getTag() != null) &&  (usage.getTag().size() == 0) && (usage.getExtension().size() == 0))
							for ( Object stereo : this.ecoreElt.getAppliedStereotypes())
							{
								setStereotype(usage, (Stereotype) stereo);
							}
				}
			}
		}catch(ObElementNotFoundException e){
			String errorMsg = e.getMessage();
			LOG.warning(XMI.PLUGIN_ID, errorMsg);
		}
	}

	@objid ("08dc73db-4583-11e0-b54c-00137279a832")
	public EElement() {
	}

	@objid ("08dc73dd-4583-11e0-b54c-00137279a832")
	public EElement(Element element) {
		this.ecoreElt = element;
	}


	public void setProperties(final IElement objingElement) {
		if (objingElement instanceof IModelElement){
			setEcoreId((IModelElement) objingElement);
			setXMIImported((IModelElement) objingElement);
		}
	}
	
	private void setEcoreId(IModelElement objingElement) {
		IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
		try{
			ITaggedValue tag = model.createTaggedValue(XMITagTypes.MODELELEMENT_ECOREID, objingElement);
			ITagParameter param = model.createTagParameter();
			param.setValue(ObjingEAnnotation.getEcoreId(getEcoreElement()));
			tag.setQualifier(param);
		}catch(Exception e){
			e.printStackTrace();
		}

	}
	
	private void setXMIImported(IModelElement objingElement) {
		IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
		try{
			ITaggedValue tag = model.createTaggedValue(XMITagTypes.MODELELEMENT_XMIIMPORTED, objingElement);
			 model.createTagParameter("true", tag);
		}catch(Exception e){
			e.printStackTrace();
		}

	}

	@objid ("08ded634-4583-11e0-b54c-00137279a832")
	private void setStereotype(IModelElement objElement, Stereotype stereotype) {
		ProfileUtils.createObjProfile(stereotype.getProfile());
		ProfileUtils.addExtension(objElement, stereotype);

		List<Property> listStereotypeProperties = new ArrayList<Property>();

		for ( Object attribute : stereotype.getOwnedAttributes()){
			if ((attribute instanceof Property ) && (!(((Property)attribute).getAssociation() instanceof ExtensionImpl)))
				listStereotypeProperties.add((Property)attribute);
		}

		for( Property property : listStereotypeProperties){
			setProperties(objElement, stereotype, property, listStereotypeProperties);
		}
	}

	@objid ("08ded63a-4583-11e0-b54c-00137279a832")
	private void setProperties(IModelElement objModelElement, Stereotype stereotype, Property property, List<Property> listStereotypeProperties) {
		String propertyName = property.getName();
		ProfileUtils.visitProperty(property);
		IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
		Type ecoreType = property.getType();
		ITagType tagType = null;
		String taggedValueId = ObjingEAnnotation.getTaggedValue(property);

		List<IModelElement> tagTypes = (List<IModelElement>) TotalImportMap.getInstance().get(property);
		for (IModelElement tagTypeTemp : tagTypes){
			if ((tagTypeTemp instanceof ITagType)
					&& (objModelElement.getExtension().contains(((ITagType)tagTypeTemp).getOwnerStereotype()))){
				tagType = (ITagType) tagTypeTemp;
			}
		}

		if (tagType != null){

			if ((taggedValueId != null ) && (!taggedValueId.equals(""))){

				Object propertyValue  = this.ecoreElt.getValue(stereotype, propertyName);
				if ( propertyValue != null  ){            
					if (propertyValue.equals(true)){
						createTaggedValue(tagType, objModelElement);
					}else if (!(propertyValue.equals(false))){
						setStringProperty(objModelElement,  stereotype, tagType,  taggedValueId, listStereotypeProperties);
					}
				}

			}else{
				Object value = this.ecoreElt.getValue(stereotype, propertyName);

				if ((value != null) && (!value.equals(false))){

					ITaggedValue taggedValue = createTaggedValue(tagType, objModelElement);

					if (!value.equals(true) &&  (property.getType() != null)){ 

						String currentEcoreTypeName = ecoreType.getName();

						if (currentEcoreTypeName == null) {
							if (ecoreType instanceof PrimitiveTypeImpl){
								currentEcoreTypeName = ((PrimitiveTypeImpl) ecoreType).eProxyURI().fragment();
							}else
								currentEcoreTypeName = ecoreType.eResource().getURI().fragment();
						}

						if (currentEcoreTypeName.equals("String")){
							if (value instanceof String ){
								model.createTagParameter((String)value, taggedValue);
							}else if (value instanceof EDataTypeUniqueEList<?>){
								EDataTypeUniqueEList<String> tabString = (EDataTypeUniqueEList<String>) value;
								for (String singleValue  : tabString){
									model.createTagParameter(singleValue, taggedValue);
								}
							}

						}else if ((ecoreType instanceof Class) 
								&& (EcoreModelNavigation.isMetaclass((Class)ecoreType)) 
								&&(value instanceof Element)){

							if(value instanceof NamedElement) {   
								model.createTagParameter(((NamedElement)value).getName(), taggedValue); 
							}else{
								model.createTagParameter(value.toString(), taggedValue);
							}

						}else if (value instanceof EnumerationLiteralImpl){
							model.createTagParameter(((EnumerationLiteralImpl)value).getName(), taggedValue);
						}else{
							model.createTagParameter(value.toString(), taggedValue);
						}   
					} 

				}

			}   
		}
	}

	@objid ("08e13888-4583-11e0-b54c-00137279a832")
	private void setStringProperty(IModelElement objModelElement, Stereotype stereotype, ITagType tagType, String taggedValueId, final List<Property> listStereotypeProperties) {
		IUmlModel model = Modelio.getInstance().getModelingSession().getModel();

		ITaggedValue taggedValue = createTaggedValue(tagType, objModelElement);
		ArrayList<Object> toBeRemoved = new ArrayList<Object>();

		//Qualifier

		for (Property attribute : listStereotypeProperties){
			String ecoreTaggedValueId = ObjingEAnnotation.getTaggedValue(attribute);
			if (!ObjingEAnnotation.isTagParameter(attribute) 
					&& (ecoreTaggedValueId != null)
					&& (ecoreTaggedValueId.equals(taggedValueId))
					&&(tagType.getName().equals(attribute.getName()))){

				Object value = this.ecoreElt.getValue(stereotype, attribute.getName());

				if ((value != null) && (!(value instanceof Boolean ))){
					ITagParameter tagParameter = model.createTagParameter();
					taggedValue.setQualifier(tagParameter);
					tagParameter.setValue(value.toString());
				}
				toBeRemoved.add(attribute);
			}
		}

		listStereotypeProperties.removeAll(toBeRemoved);
		toBeRemoved.clear();

		//TagParameter 

		for (Property attribute :  listStereotypeProperties){
			if (ObjingEAnnotation.isTagParameter(attribute) 
					&& (ObjingEAnnotation.getTaggedValue(attribute).equals(taggedValueId))
					&&(tagType.getName().equals(attribute.getName()))){

				Object value = this.ecoreElt.getValue(stereotype, attribute.getName());

				if ((value != null) && (!(value instanceof Boolean ))){
					if (value instanceof EDataTypeUniqueEList){
						EDataTypeUniqueEList<String> tabString = (EDataTypeUniqueEList<String>) value;
						if (tabString.size() != 0){
							for (String valeur : tabString){
								model.createTagParameter(valeur, taggedValue);
							}
						}else{
							model.createTagParameter("", taggedValue);
						}
					}else if (value instanceof String){
						model.createTagParameter((String)value, taggedValue);
					}
					toBeRemoved.add(attribute);
				}
			}
		}

		listStereotypeProperties.removeAll(toBeRemoved);
		toBeRemoved.clear();
	}

	@objid ("08e39ae5-4583-11e0-b54c-00137279a832")
	private void setStereotypesAnnotation(IModelElement modelElement) {
		String completeSter = ObjingEAnnotation.getStereotype(this.ecoreElt);
		if ((completeSter != null ) && (!completeSter.equals(""))){
			int index = completeSter.lastIndexOf(":");
			String profile = completeSter.substring(0, index);
			String stereotype = completeSter.substring(index+1, completeSter.length());
			ArrayList<IStereotype> objStereotypes = new ArrayList<IStereotype>();


			for (IModule currentModule : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
				if (currentModule.getName().equals("LocalModule")){
					for(IProfile currentProfile : currentModule.getOwnedProfile()){
						if (currentProfile.getName().equals(profile)){
							for (IStereotype currentStereotype : currentProfile.getDefinedStereotype()){
								if (currentStereotype.getName().equals(stereotype)){
									objStereotypes.add(currentStereotype);
									break;
								}
							}
						}
					}
				}
			}

			if (objStereotypes.size() != 0){

				ObList<IStereotype> objStereotypesClass = Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotypes(modelElement.getClass());

				for(IStereotype objStereotype : objStereotypes){

					if (objStereotypesClass.contains(objStereotype)){
						IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
						modelElement.addExtension(objStereotype);
						for (ITagType tagType : objStereotype.getDefinedTagType()){
							String value = ObjingEAnnotation.getObjingEAnnotation(this.ecoreElt, tagType.getName());

							if (!value.equals("")){

								ITaggedValue taggedValue = createTaggedValue(tagType, modelElement);
								if ((!tagType.getParamNumber().equals('0')) || (
										tagType.getParamNumber().equals('0') && (value.equalsIgnoreCase("true")))){
									model.createTagParameter(value, taggedValue);
								}

							}
						}
					}
				}
			}

		}
	}

	@objid ("a89b2cca-b176-11e1-b699-0027103f347d")
	private ITaggedValue createTaggedValue(ITagType tagType, IModelElement modelElt) {
		ITaggedValue taggedValue = Modelio.getInstance().getModelingSession().getModel().createTaggedValue();
		taggedValue.setDefinition(tagType);
		taggedValue.setAnnoted(modelElt);
		return taggedValue;
	}

	@Override
	public Object createObjingElt() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void attach(IElement objingElt) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void attach(List<Object> objingElts) {
		// TODO Auto-generated method stub
		
	}

}
