/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IElementImport;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.ElementImport;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * This class handles the import of Ecore ElementImport
 * @author ebrosse
 */
@objid ("08e39ae8-4583-11e0-b54c-00137279a832")
public class EElementImport extends EElement implements IEElement {
    @objid ("08e39ae9-4583-11e0-b54c-00137279a832")
    private ElementImport ecoreElement;


    @Override
    @objid ("08e39aea-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        // I take the ecore Imported and Importing
        PackageableElement ecoreImported = this.ecoreElement.getImportedElement();
        Namespace ecoreImporting = this.ecoreElement.getImportingNamespace();
        
        // with the ecore I take de IjingElement
        
        if ((ecoreImported != null) && (ecoreImporting != null)){
            Object objingImported = revProp.getMappedElement(ecoreImported);
            // I set to the objingElt Imported Importing previousely find
            if ((objingImported != null) && (objingImported instanceof IElement) && !(objingImported instanceof IProfile)){
                return Modelio.getInstance()
                .getModelingSession().getModel().createElementImport();
            }
        }
        return null;
    }

    /**
     * The constructor with the imported Ecore ElementImport
     * @param element : the imported Ecore ElementImport
     */
    @objid ("08e5fd3d-4583-11e0-b54c-00137279a832")
    public EElementImport(final ElementImport element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("08e5fd46-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        // I take the model map
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        // I take the ecore Imported and Importing
        PackageableElement ecoreImported = this.ecoreElement.getImportedElement();
        Namespace ecoreImporting = this.ecoreElement.getImportingNamespace();
        
        // with the ecore I take de IjingElement
        INameSpace objingImported = (INameSpace) revProp
        .getMappedElement(ecoreImported);
        IModelElement objingImporting = (IModelElement) revProp
        .getMappedElement(ecoreImporting);
        
        // I set to the objingElt Imported Importing previousely find
        if ((objingImported != null)  && !(objingImported instanceof IProfile)){
            IElementImport objingEIImport = (IElementImport) objingElt;
            objingEIImport.setImportedElement(objingImported);
            if (objingImporting != null) {
                if (objingImporting instanceof INameSpace) {
                    objingEIImport
                    .setImportingNameSpace((INameSpace) objingImporting);
                } else {
                    objingEIImport
                    .setImportingOperation((IOperation) objingImporting);
                }
        
            }
        }else{
            String ownerName = "";
            if (ecoreImported != null)
                ownerName = ecoreImported.getName();
        
            XMILogs xmilogs = XMILogs.getInstance();
            xmilogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.unimportedElement.elementImport.ownerNull"
                    , this.ecoreElement.getName(), ownerName));
        
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        }
    }

    @Override
    @objid ("08e85f96-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("08e85f9b-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        setVisibility((IElementImport) objingElt);
        setName((IElementImport) objingElt);

    	super.setProperties(objingElt);
    }

    @objid ("08e85f9e-4583-11e0-b54c-00137279a832")
    private void setVisibility(final IElementImport objingElt) {
        switch (this.ecoreElement.getVisibility().getValue()) {
        case VisibilityKind.PUBLIC:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
            break;
        case VisibilityKind.PRIVATE:
            objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
            break;
        default:
            // Visibility of a PackageImport should be Public or Private.
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
        
        }
        
        if (ObjingEAnnotation.isUndefined(this.ecoreElement))
            objingElt
            .setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
    }

    @objid ("08e85fa1-4583-11e0-b54c-00137279a832")
    private void setName(IElementImport objingElt) {
        String name = this.ecoreElement.getAlias();
        if (!(name == null || "".equals(name)))
            objingElt.setName(name);
    }

}
