/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IMetamodelExtensions;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.usecase.IExtensionPoint;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCaseDependency;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Extend;
import org.eclipse.uml2.uml.ExtensionPoint;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class manages the import of Ecore Named Element
 * @author ebrosse
 */
@objid ("0923fa65-4583-11e0-b54c-00137279a832")
public class EExtend extends ENamedElement implements IEElement {
    @objid ("0923fa66-4583-11e0-b54c-00137279a832")
    private Extend ecoreElement;


    @Override
    @objid ("0923fa67-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IUseCaseDependency objingUseCaseDependency = Modelio
                .getInstance().getModelingSession().getModel()
                .createUseCaseDependency();
        return objingUseCaseDependency;
    }

    /**
     * Constructor with the imported Ecore Extend
     * @param element : the imported Ecore Extend
     */
    @objid ("09265cba-4583-11e0-b54c-00137279a832")
    public EExtend(final Extend element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("09265cc3-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        UseCase ecoreTarget = this.ecoreElement.getExtendedCase();
        UseCase ecoreOrigin = this.ecoreElement.getExtension();
        IUseCase objingTarget = (IUseCase) revProp
                .getMappedElement(ecoreTarget);
        IUseCase objingOrigin = (IUseCase) revProp
                .getMappedElement(ecoreOrigin);
                
        if (objingTarget != null && objingOrigin != null) {
            IUseCaseDependency objingUseCaseDependency = (IUseCaseDependency) objingElt;
            objingUseCaseDependency.setOrigin(objingOrigin);
            objingUseCaseDependency.setTarget(objingTarget);
                
            IMetamodelExtensions extensions = Modelio.getInstance()
                    .getModelingSession().getMetamodelExtensions();
            try {
                IStereotype extendStereo = extensions.getStereotype(
                        IUseCaseDependency.class, XMI.I18N
                                .getString("objing.java.stereotype.extend"));
                objingUseCaseDependency.addExtension(extendStereo);
            } catch (StereotypeNotFoundException e) {
                e.printStackTrace(Modelio.err);
            }
        }
    }

    @Override
    @objid ("09265cc6-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("0928bf17-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setExtensionPoint((IUseCaseDependency) objingElt);
    }

    @objid ("0928bf1a-4583-11e0-b54c-00137279a832")
    private void setExtensionPoint(IUseCaseDependency dependency) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        for (Object extPoint : this.ecoreElement.getExtensionLocations()) {
            ExtensionPoint ecoreExtensionPoint = (ExtensionPoint) extPoint;
            IExtensionPoint objingExtensionPoint = (IExtensionPoint) revProp
                    .getMappedElement(ecoreExtensionPoint);
                
            if (objingExtensionPoint != null) {
                dependency.addExtensionLocation(objingExtensionPoint);
            }
        }
    }

}
