/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.model.uml.usecase.IExtensionPoint;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ExtensionPoint;
import org.eclipse.uml2.uml.UseCase;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("0934aade-4583-11e0-b54c-00137279a832")
public class EExtensionPoint extends ENamedElement implements IEElement {
    @objid ("0934aadf-4583-11e0-b54c-00137279a832")
    private ExtensionPoint ecoreElement;


    @objid ("0934aae0-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IExtensionPoint objingExtensionPoint = Modelio.getInstance()
                .getModelingSession().getModel().createExtensionPoint();
        return objingExtensionPoint;
    }

    @objid ("09370d33-4583-11e0-b54c-00137279a832")
    public EExtensionPoint(ExtensionPoint element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("09370d3c-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        IExtensionPoint currentObjingElt = (IExtensionPoint) objingElt;
                
        UseCase ecoreOwner = (UseCase) ecoreElement.getOwner();
                
        IUseCase objingOwner = (IUseCase) revProp
                .getMappedElement(ecoreOwner);
                
        if (objingOwner != null) {
            currentObjingElt.setOwner(objingOwner);
        }
    }

    @objid ("09396f8a-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("09396f8f-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setVisibility((IExtensionPoint) objingElt);
    }

    @objid ("09396f92-4583-11e0-b54c-00137279a832")
    public void setVisibility(IExtensionPoint objingElt) {
        switch (ecoreElement.getVisibility().getValue()) {
        case VisibilityKind.PUBLIC:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
            break;
        case VisibilityKind.PRIVATE:
            objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
            break;
        case VisibilityKind.PACKAGE:
            objingElt.setVisibility(ObVisibilityModeEnum.PACKAGE_VISIBILITY);
            break;
        case VisibilityKind.PROTECTED:
            objingElt.setVisibility(ObVisibilityModeEnum.PROTECTED);
            break;
        default:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
        }
        
        if (ObjingEAnnotation.isUndefined(ecoreElement))
            objingElt
            .setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
    }

    @objid ("09396f98-4583-11e0-b54c-00137279a832")
    public EExtensionPoint() {
    }

}
