/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralization;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Stereotype;

/**
 * This class manages the import of Ecore Generalization
 * @author ebrosse
 */
@objid ("095f952a-4583-11e0-b54c-00137279a832")
public class EGeneralization extends EElement implements IEElement {
    @objid ("095f952b-4583-11e0-b54c-00137279a832")
    private Generalization ecoreElement;


    @Override
    @objid ("095f952c-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Classifier ecoreGeneral = this.ecoreElement.getGeneral();
        Classifier ecoreSpecific = this.ecoreElement.getSpecific();
        if ((ecoreGeneral != null) && (ecoreSpecific != null)){
        
            if  ((ecoreGeneral instanceof Stereotype) && (ecoreSpecific instanceof Stereotype)){
                
                List<IStereotype> objingGenerals = new ArrayList<IStereotype>();
                List<IStereotype> objingSpecifics = new ArrayList<IStereotype>();
                
                Object objingGeneral = ReverseProperties.getInstance().getMappedElement(ecoreGeneral);
                if (objingGeneral instanceof IStereotype)
                    objingGenerals.add((IStereotype) objingGeneral);
                else 
                    objingGenerals = (ArrayList<IStereotype>) objingGeneral;
                
                Object objingSpecific = ReverseProperties.getInstance().getMappedElement(ecoreSpecific);
                if (objingSpecific instanceof IStereotype)
                    objingSpecifics.add((IStereotype) objingSpecific);
                else 
                    objingSpecifics = (ArrayList<IStereotype>) objingSpecific;
                
                for (IStereotype currentSpecific : objingSpecifics){
                    for (IStereotype currentGeneral : objingGenerals){
                        if (currentSpecific.getBaseClassName().equals(currentGeneral.getBaseClassName())
                                && currentSpecific.getElementStatus().isModifiable()){
                            currentSpecific.setParent(currentGeneral);
                            break;
                        }
                    }
                }
                
                return null;
            }else if (!(ecoreGeneral instanceof Stereotype) && !(ecoreSpecific instanceof Stereotype)){
        
                IElement objingGeneral = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreGeneral);
                IElement objingSpecific = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreSpecific);
                if (objingGeneral instanceof INameSpace && objingSpecific instanceof INameSpace 
                        && (!PrimitiveTypeMapper.isPredefinedType(ecoreSpecific))) {
                    IGeneralization objingGeneralization = Modelio.getInstance().getModelingSession().getModel().createGeneralization();
                    return objingGeneralization;
                }else{
                    
                    XMILogs xmilogs = XMILogs.getInstance();
                    String objingGeneralName = " ";
                    String objingSpecificName = " ";
                        
                    if (objingGeneral != null) 
                            objingGeneralName = objingGeneral.getClass().getSimpleName();
                    
                    if (objingSpecific != null)
                        objingSpecificName = objingSpecific.getClass().getSimpleName();
                    
                        xmilogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.unsupportedEnds", " "
                            , "IGeneralization", objingGeneralName, objingSpecificName));
                   
        
                    return null;
                }
            }else
                return null;
        }else
            return null;
    }

    /**
     * The EGeneralization constructor with the imported Ecore Generalization as parameter
     * @param element : the imported Ecore Generalization
     */
    @objid ("095f9533-4583-11e0-b54c-00137279a832")
    public EGeneralization(final Generalization element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("0961f787-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        
        Classifier ecoreGeneral = this.ecoreElement.getGeneral();
        Classifier ecoreSpecific = this.ecoreElement.getSpecific();
        
        IElement objingGeneral = (IElement) revProp
        .getMappedElement(ecoreGeneral);
        IElement objingSpecific = (IElement) revProp
        .getMappedElement(ecoreSpecific);
        
        if (objingGeneral instanceof INameSpace
                && objingSpecific instanceof INameSpace
                && objingElt instanceof IGeneralization) {
            IGeneralization objingGen = (IGeneralization) objingElt;
            objingGen.setSuperType((INameSpace) objingGeneral);
            objingGen.setSubType((INameSpace) objingSpecific);
        } else {
            PartialImportMap.getInstance().remove(this.ecoreElement);
            TotalImportMap.getInstance().remove(this.ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(objingElt);
        }
    }

    @Override
    @objid ("0961f78a-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("096459de-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        if (revProp.isRoundtripEnabled())
            setDiscriminator((IGeneralization) objingElt);
        

    	super.setProperties(objingElt);
    }

    @objid ("096459e1-4583-11e0-b54c-00137279a832")
    private void setDiscriminator(IGeneralization objingElt) {
        String discriminator = ObjingEAnnotation.getDiscriminator(this.ecoreElement);
        if (discriminator != null)
            objingElt.setDiscriminator(discriminator);
    }

}
