/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IMetamodelExtensions;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCaseDependency;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Include;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class mandes the import of Ecore include elements
 * @author ebrosse
 */
@objid ("0972a810-4583-11e0-b54c-00137279a832")
public class EInclude extends ENamedElement implements IEElement {
    @objid ("09750a54-4583-11e0-b54c-00137279a832")
    private Include ecoreElement;


    @Override
    @objid ("09750a55-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                 .createUseCaseDependency();
    }

    /**
     * EInclude constructor with the imported Ecore Include as parameter
     * @param element : the imported Ecore include
     */
    @objid ("09750a59-4583-11e0-b54c-00137279a832")
    public EInclude(final Include element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("09776cae-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        UseCase ecoreTarget = this.ecoreElement.getAddition();
        UseCase ecoreOrigin = this.ecoreElement.getIncludingCase();
        IUseCase objingTarget = (IUseCase) revProp
                .getMappedElement(ecoreTarget);
        IUseCase objingOrigin = (IUseCase) revProp
                .getMappedElement(ecoreOrigin);
                
        if (objingTarget != null && objingOrigin != null) {
            IUseCaseDependency objingUseCaseDependency = (IUseCaseDependency) objingElt;
            objingUseCaseDependency.setOrigin(objingOrigin);
            objingUseCaseDependency.setTarget(objingTarget);
            IMetamodelExtensions include = Modelio.getInstance()
                    .getModelingSession().getMetamodelExtensions();
            try {
                IStereotype extendStereo = include.getStereotype(
                        IUseCaseDependency.class, XMI.I18N
                                .getString("objing.java.stereotype.include"));
                objingUseCaseDependency.addExtension(extendStereo);
            } catch (StereotypeNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    @objid ("09776cb1-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("09776cb6-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
    }

}
