/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.information.IInformationFlow;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InformationFlow;

@objid ("0979cf08-4583-11e0-b54c-00137279a832")
public class EInformationFlow extends ENamedElement implements IEElement {
    @objid ("0979cf09-4583-11e0-b54c-00137279a832")
    private InformationFlow ecoreElement;


    @objid ("0979cf0a-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession()
                .getModel().createInformationFlow();
    }

    @objid ("0979cf0e-4583-11e0-b54c-00137279a832")
    public EInformationFlow(InformationFlow element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("097e93bd-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        /*
         * // I take the model map EOModelMap modelMap =
         * EOModelMap.getInstance(); ReverseProperties revProp =
         * ReverseProperties.getInstance(); // I take the ecore Imported and
         * Importing ecoreElement.getRelationships();
         * ecoreElement.getConveyeds(); ecoreElement.getRealizingConnectors();
         * ecoreElement.getRealizingActivityEdges();
         * ecoreElement.getRealizingMessages();
         * 
         * EList targetsEcore = ecoreElement.getTargets(); EList sourcesEcore =
         * ecoreElement.getSources();
         */
        List<Element> sourceList = ecoreElement.getSources();
        
        boolean attached = false;
        
        if (sourceList.size() > 0){
            for (Object ecoreSource : sourceList){
                IElement objingSource = (IElement) ReverseProperties.getInstance()
                        .getMappedElement((Element)ecoreSource);
                if (objingSource instanceof INameSpace) {
                    IInformationFlow informationFlow = (IInformationFlow) objingElt;
                    informationFlow.setOwner((INameSpace) objingSource);
                    attached = true;
                }
            }
        
        }
        
        if (!attached){
            Element ecoreOwner = ecoreElement.getOwner();
        
            IElement objingOwner = (IElement) ReverseProperties.getInstance()
                    .getMappedElement(ecoreOwner);
        
            if (objingOwner instanceof INameSpace) {
                IInformationFlow informationFlow = (IInformationFlow) objingElt;
                informationFlow.setOwner((INameSpace) objingOwner);
        
            }else{
                PartialImportMap.getInstance().remove(ecoreElement);
                TotalImportMap.getInstance().remove(ecoreElement);
                Modelio.getInstance().getModelingSession().getModel()
                .deleteElement(objingElt);
            }
        }
    }

    @objid ("097e93c0-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0980f619-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setInformationSource((IInformationFlow)objingElt);
        setInformationTarget((IInformationFlow)objingElt);
        setRealizingMessage((IInformationFlow)objingElt);
        setRealizingActivityEdge((IInformationFlow)objingElt);
        setConveyed((IInformationFlow)objingElt);
        //        setRealizingConnector((InformationFlow)ecoreElt);
        setRealizingAssociation((IInformationFlow)objingElt);
        checkSourceAndTarget((IInformationFlow)objingElt);
    }

    @objid ("0980f61f-4583-11e0-b54c-00137279a832")
    private void setInformationSource(IInformationFlow objingElt) {
        for (Object ecoreSource :  ecoreElement.getInformationSources()) {
            Object objingSource = ReverseProperties.getInstance().getMappedElement((Element)ecoreSource);
            if (objingSource instanceof IModelElement) {
                objingElt.addInformationSource((IModelElement)objingSource);
            }
        }
    }

    @objid ("0980f622-4583-11e0-b54c-00137279a832")
    private void setInformationTarget(IInformationFlow objingElt) {
        for (Object ecoreTarget : ecoreElement.getInformationTargets()) {
            Object objingTarget = ReverseProperties.getInstance().getMappedElement((Element)ecoreTarget);
            if (objingTarget instanceof IModelElement) {
                objingElt.addInformationTarget((IModelElement)objingTarget);
            }
        }
    }

    @objid ("0980f625-4583-11e0-b54c-00137279a832")
    private void setRealizingMessage(IInformationFlow objingElt) {
        for (Object ecoreMsg : ecoreElement.getRealizingMessages()) {
            Object objingMsg = ReverseProperties.getInstance().getMappedElement((Element)ecoreMsg);
            if (objingMsg instanceof IMessage)
                objingElt.addRealizingMessage((IMessage) objingMsg);
        }
    }

    @objid ("09835870-4583-11e0-b54c-00137279a832")
    private void setRealizingActivityEdge(IInformationFlow objingElt) {
        for (Object ecoreEdge : ecoreElement.getRealizingActivityEdges()) {
            Object objingEdge = ReverseProperties.getInstance().getMappedElement((Element)ecoreEdge);
            if (objingEdge instanceof IActivityEdge)
                objingElt.addRealizingActivityEdge((IActivityEdge)objingEdge);
        }
    }

    @objid ("09835873-4583-11e0-b54c-00137279a832")
    private void setConveyed(IInformationFlow objingElt) {
        for (Object ecoreConveyed : ecoreElement.getConveyeds()) {
            Object objingConveyed = ReverseProperties.getInstance().getMappedElement((Element)ecoreConveyed);
            if (objingConveyed instanceof IClassifier)
                objingElt.addConveyed((IClassifier)objingConveyed);
        }
    }

    @objid ("09835876-4583-11e0-b54c-00137279a832")
    private void setRealizingAssociation(IInformationFlow objingElt) {
        for (Object ecoreRealizingAssoc : ecoreElement.getRealizations()) {
            Object realization = ReverseProperties.getInstance().getMappedElement((Element)ecoreRealizingAssoc);
            if (realization instanceof IAssociation)
                objingElt.addRealizingAssociation((IAssociation)realization);
        }
    }

    @objid ("09835879-4583-11e0-b54c-00137279a832")
    public EInformationFlow() {
    }

    @objid ("0983587b-4583-11e0-b54c-00137279a832")
    private void checkSourceAndTarget(IInformationFlow objingElt) {
        ObList<IModelElement> sources = objingElt.getInformationSource();
        ObList<IModelElement> targets = objingElt.getInformationSource();
        if ((sources.size() == 0) || (targets.size() == 0))
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
    }

}
