/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.information.IInformationItem;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InformationItem;

@objid ("0985baca-4583-11e0-b54c-00137279a832")
public class EInformationItem extends ENamedElement implements IEElement {
    @objid ("0985bacb-4583-11e0-b54c-00137279a832")
    private InformationItem ecoreElement;


    @objid ("0985bacc-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel().createInformationItem();
    }

    @objid ("0985bad0-4583-11e0-b54c-00137279a832")
    public EInformationItem(InformationItem element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("09881d29-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Element ecoreOwner = ecoreElement.getOwner();
        Object objOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        if (objOwner instanceof IProfile){
             ((IInformationItem) objingElt).setOwner(ReverseProperties.getInstance().getExternalPackage());
        }else if (objOwner instanceof IModelTree){
            ((IInformationItem) objingElt).setOwner((IModelTree) objOwner);
        } else if (objingElt instanceof IModelElement){
            EcoreModelNavigation.unsupportedOwnedWithIj((IElement)objOwner, (IModelElement) objingElt);
            ((IInformationItem) objingElt).setOwner(ReverseProperties.getInstance().getExternalPackage());
        }else
             ((IInformationItem) objingElt).setOwner(ReverseProperties.getInstance().getExternalPackage());
    }

    @objid ("09881d2c-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("09881d31-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setRepresented((IInformationItem) objingElt);
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setAbstract((IInformationItem) objingElt);
            setLeaf((IInformationItem) objingElt);
            setRoot((IInformationItem) objingElt);
            setEAVisibility((IInformationItem) objingElt);
        }
    }

    @objid ("098a7f7e-4583-11e0-b54c-00137279a832")
    private void setEAVisibility(IInformationItem objingElt) {
        int literal = ObjingEAnnotation.getVisibility(ecoreElement);
        objingElt.setVisibility(ObVisibilityModeEnum.values()[literal]);
    }

    @objid ("098a7f81-4583-11e0-b54c-00137279a832")
    private void setRoot(IInformationItem objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot(ecoreElement));
    }

    @objid ("098a7f84-4583-11e0-b54c-00137279a832")
    private void setLeaf(IInformationItem objingElt) {
        objingElt.setLeaf(ObjingEAnnotation.isLeaf(ecoreElement));
    }

    @objid ("098a7f87-4583-11e0-b54c-00137279a832")
    private void setAbstract(IInformationItem objingElt) {
        objingElt.setAbstract(ObjingEAnnotation.isAbstract(ecoreElement));
    }

    @objid ("098a7f8a-4583-11e0-b54c-00137279a832")
    private void setRepresented(IInformationItem objingElt) {
        for(Object represented : ecoreElement.getRepresenteds()){
            Object objRepresented = ReverseProperties.getInstance().getMappedElement((Element)represented);
            if (objRepresented instanceof IClassifier)
                objingElt.addRepresented((IClassifier) objRepresented);
        }
    }

    @objid ("098ce1d9-4583-11e0-b54c-00137279a832")
    public EInformationItem() {
    }

}
