/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumerationLiteral;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;

@objid ("09a25702-4583-11e0-b54c-00137279a832")
public class EInstanceSpecification extends ENamedElement implements IEElement {
    @objid ("09a25705-4583-11e0-b54c-00137279a832")
    private InstanceSpecification ecoreElement;

    @objid ("68ea382e-1a76-11e1-8103-0027103f347d")
    private Boolean isConnector = false;

    @objid ("68ea5f41-1a76-11e1-8103-0027103f347d")
    private Boolean isLink = false;


    @objid ("09a25706-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        if (revProp.isRoundtripEnabled()){
            if (ObjingEAnnotation.isConnector(ecoreElement)){
                isConnector = true;
                return Modelio.getInstance().getModelingSession().getModel().createConnector();
            }else if (ObjingEAnnotation.isLink(ecoreElement)){
                isLink = true;
                return Modelio.getInstance().getModelingSession().getModel().createLink();
            }
        }
        
        
        if (EcoreModelNavigation.isAssocInstance(ecoreElement)){
            if (EcoreModelNavigation.isConnector(ecoreElement)){
        
                isConnector = true;
                return Modelio.getInstance().getModelingSession().getModel().createConnector();
            }else  if (EcoreModelNavigation.isLink(ecoreElement)){
                isLink = true;
                return Modelio.getInstance().getModelingSession().getModel().createLink();
            }
        }
        return Modelio.getInstance().getModelingSession().getModel().createInstance();
    }

    @objid ("09a2570a-4583-11e0-b54c-00137279a832")
    public EInstanceSpecification(InstanceSpecification element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("09a4b95d-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (!(isConnector) && !(isLink))
            attachInstance(objingElt);
    }

    @objid ("09a4b960-4583-11e0-b54c-00137279a832")
    private void attachInstance(IElement objingElt) {
        if (!(objingElt instanceof IEnumerationLiteral)){
            ReverseProperties revProp = ReverseProperties.getInstance();
        
            Element ecoreOwner = ecoreElement.getOwner();
        
            IElement owner =  (IElement) revProp.getMappedElement(ecoreOwner);
        
            if ((owner != null) &&  (owner instanceof INameSpace))
                ((IInstance) objingElt).setOwner((INameSpace)owner);
        
        }
    }

    @objid ("09a4b963-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("09a4b968-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if ((isConnector) ||(isLink)){
            setConnectorProperties((ILink) objingElt);
        }else if (objingElt instanceof IInstance){
            setInstanceProperties((IInstance) objingElt);
        
        
        }
    }

    @objid ("09a71bb6-4583-11e0-b54c-00137279a832")
    private void setType(IInstance objingElt) {
        String typeName = ObjingEAnnotation.getType(ecoreElement);
        if ((typeName != null) && ( !typeName.equals(""))){
            INameSpace objType = ObjingModelNavigation.getPrimitiveType(typeName);
            objingElt.setBase(objType);
        }
    }

    @objid ("09a71bb9-4583-11e0-b54c-00137279a832")
    private void setConnectorProperties(ILink objingElt) {
    }

    @objid ("09a71bbc-4583-11e0-b54c-00137279a832")
    private void setInstanceProperties(IInstance objingElt) {
        //        if (!(objingElt instanceof IEnumerationLiteral)){
        setClassifier( objingElt);
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setIsConstant( objingElt);
            setValue( objingElt);
            setMultiMax( objingElt);
            setMultiMin( objingElt);
            setType((IInstance) objingElt);
        }
        
        //        }
    }

    @objid ("09a71bc2-4583-11e0-b54c-00137279a832")
    private void setClassifier(IInstance objingElt) {
        if (ecoreElement.getClassifiers().size() > 0){
        
            Element ecoreClassifier = (Element) ecoreElement.getClassifiers().get(0);
        
            IElement objClassifier = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreClassifier);
        
            if ((objClassifier != null) && (objClassifier instanceof INameSpace)){
                objingElt.setBase((INameSpace) objClassifier);
        
            }
        
        }
    }

    @objid ("09a71bc5-4583-11e0-b54c-00137279a832")
    private void setIsConstant(IInstance objingElt) {
        objingElt.setConstant(ObjingEAnnotation.isConstant(ecoreElement));
    }

    @objid ("09a97e11-4583-11e0-b54c-00137279a832")
    private void setMultiMax(IInstance objingElt) {
        String max =  ObjingEAnnotation.getMultiMax(ecoreElement);
        if (max != null)
            objingElt.setMultiplicityMax(max);
    }

    @objid ("09a97e14-4583-11e0-b54c-00137279a832")
    private void setValue(IInstance objingElt) {
        String value =  ObjingEAnnotation.getValue(ecoreElement);
        if (value != null)
            objingElt.setValue(value);
    }

    @objid ("09a97e17-4583-11e0-b54c-00137279a832")
    private void setMultiMin(IInstance objingElt) {
        String min =  ObjingEAnnotation.getMultiMin(ecoreElement);
        if (min != null)
            objingElt.setMultiplicityMin(min);
    }

    @objid ("09a97e1a-4583-11e0-b54c-00137279a832")
    public EInstanceSpecification() {
    }

}
