/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.diagrams.ISequenceDiagram;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessageEnd;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.sequence.LifelineImportMapper;
import com.modeliosoft.modelio.xmi.model.ecore.sequence.PartDecompositionImportMapper;
import com.modeliosoft.modelio.xmi.model.ecore.sequence.SequenceElementImportMapper;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.SequenceElementVisitor;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.PartDecomposition;

/**
 * This class handles the import of Ecore Interaction
 * @author ebrosse
 */
@objid ("09b0a51f-4583-11e0-b54c-00137279a832")
public class EInteraction extends ENamedElement implements IEElement {
    @objid ("09b0a520-4583-11e0-b54c-00137279a832")
    private PartialImportMap partialCreationMap = PartialImportMap
    .getInstance();

    @objid ("09b0a521-4583-11e0-b54c-00137279a832")
    private Interaction ecoreElement;


    @Override
    @objid ("09b0a522-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createInteraction();
    }

    @objid ("09b0a526-4583-11e0-b54c-00137279a832")
    public EInteraction(Interaction element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("09b3077e-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Element ecoreOwner = this.ecoreElement.getOwner();
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        
        Object objingOwner =  revProp.getMappedElement(ecoreOwner);
        
        
        if (objingOwner != null) {
        
            INameSpace finalOwner = null;
            if (objingOwner instanceof ITransition){
                finalOwner = ObjingModelNavigation.getNearestPackage((ITransition)objingOwner);
            }else if (objingOwner instanceof INameSpace){
                finalOwner = (INameSpace) objingOwner;
            }
        
            if (finalOwner != null){
                IInteraction objingIImport = (IInteraction) objingElt;
                objingIImport.setOwner(finalOwner);
            }else{
                objingElt.delete();
            }
        }
    }

    @Override
    @objid ("09b30781-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("09b30786-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setReentrant((IInteraction) objingElt);
        createDiagramContext((IInteraction) objingElt);
        importLifelines();
        importPartDecomposition();
        importUnconnectedGates((IInteraction) objingElt);
        importSequencedElements(this.ecoreElement, ReverseProperties.getInstance()
                                .getSequenceElementVisitor());
    }

    @objid ("09b569d2-4583-11e0-b54c-00137279a832")
    private void setReentrant(IInteraction interaction) {
        interaction.setReentrant(this.ecoreElement.isReentrant());
    }

    @objid ("09b569d8-4583-11e0-b54c-00137279a832")
    private void createDiagramContext(IInteraction interaction) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        
        ISequenceDiagram seqDiag = session.getModel().createSequenceDiagram();
        
        
        String name = interaction.getName();
        if (EcoreModelNavigation.isNotNull(name))
            seqDiag.setName(name);
        
        seqDiag.setOrigin(interaction);
        seqDiag.setOwner(session.getModel().getUmlProject().getDiagramRoot());
        
        ReverseProperties.getInstance().addInteraction(this.ecoreElement);
    }

    @objid ("09b569db-4583-11e0-b54c-00137279a832")
    private void importLifelines() {
        for (Lifeline ecoreLifeline : this.ecoreElement.getLifelines()) {
            LifelineImportMapper lifelineMapper = new LifelineImportMapper(ecoreLifeline);
        
            if (TotalImportMap.getInstance().get(ecoreLifeline) == null) {
                ILifeline objingLifeline = null;
        
                if (this.partialCreationMap.get(ecoreLifeline) == null) {
                    objingLifeline = (ILifeline) lifelineMapper.createObjingElt();
                    this.partialCreationMap.put(ecoreLifeline, objingLifeline);
                }else{
                    objingLifeline = (ILifeline) this.partialCreationMap.get(ecoreLifeline);
                    this.partialCreationMap.put(ecoreLifeline, objingLifeline);
                }
        
                lifelineMapper.attach(objingLifeline);
                lifelineMapper.setProperties(objingLifeline);
                lifelineMapper.setStereotypes();
        
                TotalImportMap.getInstance().put(ecoreLifeline, this.partialCreationMap
                                                 .remove(ecoreLifeline));
            }
        }
    }

    @objid ("09b569de-4583-11e0-b54c-00137279a832")
    private void importPartDecomposition() {
        for (Element ecoreIF : this.ecoreElement.getFragments()) {
            if (ecoreIF instanceof PartDecomposition) {
                PartDecompositionImportMapper partDecompMapper = new PartDecompositionImportMapper(
                                                                                                   (PartDecomposition) ecoreIF);
        
                if (TotalImportMap.getInstance().get(ecoreIF) == null) {
                    IPartDecomposition objingPD = null;
        
                    if (this.partialCreationMap.get(ecoreIF) == null) {
                        objingPD = (IPartDecomposition) partDecompMapper
                                .createObjingElt();
                        this.partialCreationMap.put(ecoreIF, objingPD);
                    }else{
                        objingPD = (IPartDecomposition) this.partialCreationMap.get(ecoreIF);
                        this.partialCreationMap.put(ecoreIF, objingPD);
                    }
        
                    partDecompMapper.attach(objingPD);
                    partDecompMapper.setProperties(objingPD);
                    partDecompMapper.setStereotypes();
        
                    TotalImportMap.getInstance().put(ecoreIF, this.partialCreationMap
                                                     .remove(ecoreIF));
                }
            }
        }
    }

    @objid ("09b7cc2c-4583-11e0-b54c-00137279a832")
    private void importUnconnectedGates(IInteraction interaction) {
        for (Object ecoreGate : this.ecoreElement.getFormalGates()) {
            createUnconnectedGate((Gate) ecoreGate, interaction);
        }
    }

    @objid ("09b7cc2f-4583-11e0-b54c-00137279a832")
    public void createUnconnectedGate(Gate ecoreGate, IInteraction interaction) {
        IGate objingGate = (IGate) ReverseProperties.getInstance().getMappedElement(ecoreGate);
        
        if (objingGate == null)
            objingGate = Modelio.getInstance().getModelingSession().getModel().createGate();
        
        String name = ecoreGate.getName();
        if (EcoreModelNavigation.isNotNull(name))
            objingGate.setName(name);
        
        if (this.ecoreElement.getFormalGates().contains(ecoreGate)){
            interaction.addFormalGate(objingGate);
        }
        
        try {
            interaction.insert(objingGate, null, ObRelativeTimeEnum.AFTER);
            TotalImportMap.getInstance().put(ecoreGate, objingGate);
        } catch (InvalidInteractionManipulationException e) {
            e.printStackTrace();
            this.partialCreationMap.remove(ecoreGate);
            TotalImportMap.getInstance().remove(ecoreGate);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(objingGate);
        }
    }

    @objid ("09b7cc33-4583-11e0-b54c-00137279a832")
    private void importSequencedElements(InteractionFragment ecoreIFContainer, SequenceElementVisitor sequenceVisitor) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        List<Element> insertableEltsList = new ArrayList<Element>();
        List<InteractionFragment> fragmentContainerList = new ArrayList<InteractionFragment>();
        List<SequenceElementImportMapper> mapperList = new ArrayList<SequenceElementImportMapper>();
        
        for (InteractionFragment ecoreIF : getFragments(ecoreIFContainer)) {
            for (Element insertableElt : revProp.getInsertableIFVisitor()
                    .getInsertableElements(ecoreIF)) {
                addInsertableElement(insertableElt, sequenceVisitor, mapperList, insertableEltsList);
            }
        
            if (ecoreIF instanceof CombinedFragment) {
                fragmentContainerList.addAll(((CombinedFragment) ecoreIF)
                                             .getOperands());
            }
        }
        
        // Inserts elements in diagram and adds to the total creation map:
        insertElementsInDiagram(insertableEltsList, mapperList);
        
        // Recursive call for each InteractionFragment (InteractionOperands)
        // that
        // contains sequence diagram elements.
        for (InteractionFragment containerOfIF : fragmentContainerList) {
            importSequencedElements(containerOfIF, sequenceVisitor);
        }
    }

    @objid ("09b7cc37-4583-11e0-b54c-00137279a832")
    private void addInsertableElement(Element insertableElt, SequenceElementVisitor sequenceVisitor, List<SequenceElementImportMapper> mapperList, List<Element> insertableEltsList) {
        Object objingInserted = TotalImportMap.getInstance().get(insertableElt);
        if (objingInserted == null) {
            objingInserted = this.partialCreationMap.get(insertableElt);
        
            if (objingInserted == null) {
                // Create and add the element in the partial creation
                // map:
                SequenceElementImportMapper mapper = sequenceVisitor
                        .getSequenceImportMapper(insertableElt);
                Object objingElt = mapper.createObjingElt();
                this.partialCreationMap.put(insertableElt, objingElt);            
                if (insertableElt instanceof Message) {                            
                    storeMappedEnds((Message) insertableElt, objingElt);            
                }                                                                
        
                // Add to the list of insertable elements:
                mapperList.add(mapper);
                insertableEltsList.add(insertableElt);
            }
        }
    }

    @objid ("09ba2e89-4583-11e0-b54c-00137279a832")
    private void storeMappedEnds(Message message, Object mappedElement) {
        MessageEnd sendEvent = message.getSendEvent();
        if ((sendEvent != null) && !(sendEvent instanceof Gate))
            TotalImportMap.getInstance().put(sendEvent, mappedElement);
        
        MessageEnd receiveEvent = message.getReceiveEvent();
        if ((receiveEvent != null) && !(receiveEvent instanceof Gate))
            TotalImportMap.getInstance().put(receiveEvent, mappedElement);
        
        if (!(sendEvent instanceof Gate)){
            Message replyMsg = EcoreModelNavigation.getReplyMessage(message);
            if (replyMsg != null) {
                this.partialCreationMap.put(replyMsg, mappedElement);
                this.partialCreationMap.put(replyMsg.getSendEvent(), mappedElement);
                this.partialCreationMap.put(replyMsg.getReceiveEvent(), mappedElement);
            }
        }
    }

    @objid ("09ba2e8d-4583-11e0-b54c-00137279a832")
    private List<InteractionFragment> getFragments(InteractionFragment ecoreIFContainer) {
        List<InteractionFragment> fragments = new ArrayList<InteractionFragment>();
        
        EList<?> fragmentList = null;
        
        if (ecoreIFContainer instanceof Interaction)
            fragmentList = ((Interaction) ecoreIFContainer).getFragments();
        else if (ecoreIFContainer instanceof InteractionOperand)
            fragmentList = ((InteractionOperand) ecoreIFContainer)
            .getFragments();
        
        if (fragmentList != null) {
            for (Object f : fragmentList) {
                fragments.add((InteractionFragment) f);
            }
        }
        return fragments;
    }

    @objid ("09ba2e94-4583-11e0-b54c-00137279a832")
    private void insertElementsInDiagram(List<Element> insertableEltsList, List<SequenceElementImportMapper> mapperList) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        for (int i = 0; i < insertableEltsList.size(); i++) {
        
            Element ecoreInsertable = insertableEltsList.get(i);
            SequenceElementImportMapper mapper = mapperList.get(i);
        
            IElement objingElt = (IElement) revProp
                    .getMappedElement(ecoreInsertable);
            mapper.attach(objingElt);
            mapper.setProperties(objingElt);
            mapper.setStereotypes();
        
            TotalImportMap.getInstance().put(ecoreInsertable, this.partialCreationMap
                                             .remove(ecoreInsertable));
        
            if (ecoreInsertable instanceof Message) {
                mapEcoreEndsToObjingEnds((Message) ecoreInsertable,
                                         (IMessage) objingElt);
            }
        }
    }

    @objid ("09bc90e6-4583-11e0-b54c-00137279a832")
    private void mapEcoreEndsToObjingEnds(final Message ecoreMsg, final IMessage objingMsg) {
        MessageEnd eSendEvent = ecoreMsg.getSendEvent();
        if (eSendEvent != null) {
            this.partialCreationMap.remove(eSendEvent);
            IMessageEnd oSendEvent = objingMsg.getSendEvent();
            TotalImportMap.getInstance().put(eSendEvent, oSendEvent);
        }
        
        MessageEnd eReceiveEvent = ecoreMsg.getReceiveEvent();
        if (eReceiveEvent != null) {
            this.partialCreationMap.remove(eReceiveEvent);
            IMessageEnd oReceiveEvent = objingMsg.getReceiveEvent();
            TotalImportMap.getInstance().put(eReceiveEvent, oReceiveEvent);
        }
        
        Message eReplyMsg = EcoreModelNavigation.getReplyMessage(ecoreMsg);
        if (eReplyMsg != null) {
            IMessage oReplyMsg = ObjingModelNavigation
                    .getReplyMessage(objingMsg);
            if (oReplyMsg != null) {
                this.partialCreationMap.remove(eReplyMsg);
                TotalImportMap.getInstance().put(eReplyMsg, oReplyMsg);
        
                MessageEnd eSendEventFromReply = eReplyMsg.getSendEvent();
                if (eSendEventFromReply != null) {
                    this.partialCreationMap.remove(eSendEventFromReply);
                    IMessageEnd oSendEventFromReply = oReplyMsg.getSendEvent();
                    TotalImportMap.getInstance().put(eSendEventFromReply,
                                                     oSendEventFromReply);
                }
        
                MessageEnd eReceiveEventFromReply = eReplyMsg.getReceiveEvent();
                if (eReceiveEventFromReply != null) {
                    this.partialCreationMap.remove(eReceiveEventFromReply);
                    IMessageEnd oReceiveEventFromReply = oReplyMsg
                            .getReceiveEvent();
                    TotalImportMap.getInstance().put(eReceiveEventFromReply,
                                                     oReceiveEventFromReply);
                }
            }
        }
    }

}
