/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;

/**
 * This class handles the import of Ecore Interfaces.
 * @author ebrosse
 */
@objid ("09cfa3b0-4583-11e0-b54c-00137279a832")
public class EInterface extends ENamedElement implements IEElement {
    @objid ("09cfa3b1-4583-11e0-b54c-00137279a832")
    private Interface ecoreElement;


    @Override
    @objid ("09cfa3b2-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession()
        .getModel().createInterface();
    }

    /**
     * @param element : the imported Ecore Interface
     */
    @objid ("09cfa3b6-4583-11e0-b54c-00137279a832")
    public EInterface(final Interface element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("09d20612-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Element ecoreOwner = this.ecoreElement.getOwner();
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        IElement objingOwner = (IElement) revProp
        .getMappedElement(ecoreOwner);
        
         if ((objingOwner instanceof IModelTree) && !(objingOwner instanceof IProfile)){
            ((IInterface) objingElt).setOwner((IModelTree) objingOwner);
        }else
            ((IInterface) objingElt).setOwner(ReverseProperties.getInstance().getExternalPackage());
    }

    @Override
    @objid ("09d20615-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("09d46865-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        
        setAbstract((IInterface) objingElt);
        setLeaf((IInterface) objingElt);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()) {
            setPrimitiveEAnnotation((IInterface) objingElt);
        }
    }

    @objid ("09d46868-4583-11e0-b54c-00137279a832")
    private void setAbstract(IInterface objingElt) {
        objingElt.setAbstract(this.ecoreElement.isAbstract());
    }

    @objid ("09d4686b-4583-11e0-b54c-00137279a832")
    private void setLeaf(IInterface objingElt) {
        objingElt.setLeaf(this.ecoreElement.isLeaf());
    }

    @objid ("09d46871-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(IInterface objingElt) {
        objingElt.setElementary(ObjingEAnnotation.isPrimitive(this.ecoreElement));
    }

}
