/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ILoopNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.LoopNode;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.ValueSpecificationAction;

@objid ("0a31641c-4583-11e0-b54c-00137279a832")
public class ELoopNode extends EStructuredActivityNode implements IEElement {
    @objid ("0a31641d-4583-11e0-b54c-00137279a832")
    private ReverseProperties revProp = ReverseProperties.getInstance();

    @objid ("0a31641e-4583-11e0-b54c-00137279a832")
    private LoopNode ecoreElement;


    @objid ("0a31641f-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createLoopNode();
    }

    @objid ("0a33c674-4583-11e0-b54c-00137279a832")
    public ELoopNode(LoopNode element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0a33c67d-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("0a33c680-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0a3628d0-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        
        // Part of the Properties are setted when mapping the super type
        // (StructuredActivityNode)
        
        // Properties specific to LoopNodes:
        setIsTestedFirst((ILoopNode) objingElt);
        setBody((ILoopNode) objingElt);
        setSetup((ILoopNode) objingElt);
        setTest((ILoopNode) objingElt);
    }

    @objid ("0a3628d3-4583-11e0-b54c-00137279a832")
    private void setIsTestedFirst(ILoopNode node) {
        node.setTestedFirst(ecoreElement.isTestedFirst());
    }

    @objid ("0a3628d6-4583-11e0-b54c-00137279a832")
    private void setBody(ILoopNode node) {
        for (Object ecoreBody : ecoreElement.getBodyParts()) {
            Object objingBody = revProp.getMappedElement((Element) ecoreBody);
            if (objingBody instanceof IActivityNode)
                node.addBody((IActivityNode) objingBody);
        }
    }

    @objid ("0a3628d9-4583-11e0-b54c-00137279a832")
    private void setSetup(ILoopNode node) {
        String objingSetup = "";
        for (Object ecoreSetupPart : ecoreElement.getSetupParts()) {
            if (ecoreSetupPart instanceof ValueSpecificationAction) {
                ValueSpecification ecoreSetup = ((ValueSpecificationAction) ecoreSetupPart)
                .getValue();
                if (ecoreSetup != null) {
                    String stringValue = ecoreSetup.stringValue();
                    if (stringValue != null)
                        objingSetup = objingSetup.concat(stringValue);
                }
            } else if (ecoreSetupPart != null) {
                String ecoreName = ((ExecutableNode) ecoreSetupPart).getName();
                if (ecoreName != null)
                    objingSetup = objingSetup.concat(ecoreName);
            }
        }
        node.setSetup(objingSetup);
    }

    @objid ("0a3628dc-4583-11e0-b54c-00137279a832")
    private void setTest(ILoopNode node) {
        String objingTest = "";
        for (Object ecoreTestPart : ecoreElement.getTests()) {
            if (ecoreTestPart instanceof ValueSpecificationAction) {
        
                ValueSpecification ecoreTest = ((ValueSpecificationAction) ecoreTestPart)
                .getValue();
                if (ecoreTest != null) {
                    String stringValue = ecoreTest.stringValue();
                    if (stringValue != null)
                        objingTest = objingTest.concat(stringValue);
                }
            } else if (ecoreTestPart != null) {
                String ecoreName = ((ExecutableNode) ecoreTestPart).getName();
                if (ecoreName != null)
                    objingTest = objingTest.concat(ecoreName);
            }
        }
        node.setTest(objingTest);
    }

    @objid ("0a388b28-4583-11e0-b54c-00137279a832")
    public ELoopNode() {
    }

}
