/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;

@objid ("0a52c506-4583-11e0-b54c-00137279a832")
public class EModel extends ENamedElement implements IEElement {
    @objid ("0a52c507-4583-11e0-b54c-00137279a832")
    private boolean isRoot = false;

    @objid ("0a52c508-4583-11e0-b54c-00137279a832")
    private Model ecoreElement;


    @objid ("0a52c509-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        for (Package ecoremodel : ReverseProperties.getInstance().getEcoreModel()){
             if (ecoreElement.equals(ecoremodel)){
                isRoot = true;
                return Modelio.getInstance().getModelingSession().getModel().getRoot();
               }
        }
        IPackage objingElt = Modelio.getInstance().getModelingSession().getModel().createPackage();
        ((IPackage)objingElt).setName(ecoreElement.getName());
        return objingElt;
    }

    @objid ("0a52c50d-4583-11e0-b54c-00137279a832")
    public EModel(Model element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0a552764-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (!isRoot){
            ReverseProperties revProp = ReverseProperties.getInstance();
            Element ecoreOwner = ecoreElement.getOwner();
                
            IElement objingOwner = (IElement) revProp
            .getMappedElement(ecoreOwner);
                
            if (ecoreOwner instanceof Model) {
                IPackage objingPkg = (IPackage) objingElt;
                objingPkg.setOwner((IPackage) objingOwner);
            }else if (ecoreOwner instanceof Package) {
                IPackage objingPkg = (IPackage) objingElt;
                objingPkg.setOwner((IPackage) objingOwner);
            }else if (ecoreOwner instanceof Component){
                ((IPackage) objingElt).setOwner((IComponent) objingOwner);
            }
        }
    }

    @objid ("0a552767-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0a55276c-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (!isRoot){
            super.setProperties(objingElt);
                
        //            setVisibility(objingElt);
                
            if (ReverseProperties.getInstance().isRoundtripEnabled()) {
                setAbstractEAnnotation((IPackage) objingElt);
                setInstantiableEAnnotation((IPackage) objingElt);
                setLeafEAnnotation((IPackage) objingElt);
                setRootEAnnotation((IPackage) objingElt);
            }
        }
    }

    @objid ("0a55276f-4583-11e0-b54c-00137279a832")
    public EModel() {
    }

    @objid ("0a5789be-4583-11e0-b54c-00137279a832")
    private void setAbstractEAnnotation(IPackage objingElt) {
        objingElt.setAbstract(ObjingEAnnotation.isAbstract((ecoreElement)));
    }

    @objid ("0a5789c1-4583-11e0-b54c-00137279a832")
    private void setInstantiableEAnnotation(IPackage objingElt) {
        objingElt.setInstantiable(ObjingEAnnotation.isInstantiable((ecoreElement)));
    }

    @objid ("0a5789c4-4583-11e0-b54c-00137279a832")
    private void setLeafEAnnotation(IPackage objingElt) {
        objingElt.setLeaf(ObjingEAnnotation.isLeaf((ecoreElement)));
    }

    @objid ("0a5789c7-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(IPackage objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot((ecoreElement)));
    }

}
