/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  

package com.modeliosoft.modelio.xmi.model.ecore;

import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("0a59ec14-4583-11e0-b54c-00137279a832")
public class ENamedElement extends EElement {

	@objid ("0a59ec1c-4583-11e0-b54c-00137279a832")
	public ENamedElement(NamedElement element) {
		super(element);
	}

	@objid ("0a5c4e70-4583-11e0-b54c-00137279a832")
	private void setVisibility(INameSpace objingElt) {
		NamedElement ecoreNamedElt = (NamedElement) getEcoreElement();
		if (objingElt.getElementStatus().isModifiable()){
			switch (ecoreNamedElt.getVisibility().getValue()) {
			case VisibilityKind.PACKAGE:
				objingElt
				.setVisibility(ObVisibilityModeEnum.PACKAGE_VISIBILITY);
				break;
			case VisibilityKind.PRIVATE:
				objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
				break;
			case VisibilityKind.PROTECTED:
				objingElt.setVisibility(ObVisibilityModeEnum.PROTECTED);
				break;
			case VisibilityKind.PUBLIC:
				objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
				break;
			default:
				objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
			}

			if (ObjingEAnnotation.isUndefined(ecoreNamedElt))
				objingElt.setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
		}
	}

	@objid ("0a5c4e73-4583-11e0-b54c-00137279a832")
	public ENamedElement() {
	}

	@objid ("695cd5fc-1a76-11e1-8103-0027103f347d")
	protected void setName(final IModelElement objingElt) {
		String name = ((NamedElement)getEcoreElement()).getName();
		if (objingElt.getElementStatus().isModifiable()){
			if (EcoreModelNavigation.isNotNull(name))
				objingElt.setName(name);
			else 
				objingElt.setName("");
		}
	}

	@objid ("695cfd0e-1a76-11e1-8103-0027103f347d")
	public void setProperties(final IElement objingElement) {
		
		
		if (objingElement instanceof IModelElement)
			setName((IModelElement) objingElement);

		if (objingElement instanceof INameSpace)
			setVisibility((INameSpace)objingElement);

		super.setProperties(objingElement);
	}

	

}
