/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInterruptibleActivityRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IObjectFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ObObjectFlowEffectKindEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.InterruptibleActivityRegion;
import org.eclipse.uml2.uml.ObjectFlow;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0a63757c-4583-11e0-b54c-00137279a832")
public class EObjectFlow extends ENamedElement implements IEElement {
    @objid ("0a63757d-4583-11e0-b54c-00137279a832")
    private ObjectFlow ecoreElement;


    @objid ("0a63757e-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ActivityNode ecoreSource = ecoreElement.getSource();
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if ((ecoreSource != null) && (ecoreTarget != null)) {
            Object objingSource = ReverseProperties.getInstance().getMappedElement(ecoreSource);
            if ((objingSource != null)&& (objingSource instanceof IActivityNode)){
                      return Modelio.getInstance().getModelingSession().getModel().createObjectFlow();
            }else{
                return null;
            }
        }else{
           return null;
        }
    }

    @objid ("0a637582-4583-11e0-b54c-00137279a832")
    public EObjectFlow(ObjectFlow element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0a65d7d7-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        // Attachment is done when setting the Source of the Edge.
    }

    @objid ("0a6a9c8b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0a6a9c90-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        
        // Properties defined on super classes:
        setTarget((IObjectFlow) objingElt);
        setInterrupts((IObjectFlow) objingElt);
        setGuard((IObjectFlow) objingElt);
        setWeight((IObjectFlow) objingElt);
                
        // Properties of IObjectFlows:
        setEffectEAnnotation((IObjectFlow) objingElt);
        setMultiCast((IObjectFlow) objingElt);
        setMultiReceive((IObjectFlow) objingElt);
        setSelectionBehavior((IObjectFlow) objingElt);
        setTransformationBehavior((IObjectFlow) objingElt);
                
        setSource((IObjectFlow) objingElt);
    }

    @objid ("0a6cfee4-4583-11e0-b54c-00137279a832")
    private void setSource(IObjectFlow flow) {
        ActivityNode ecoreSource = ecoreElement.getSource();
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if ((ecoreSource != null) && (ecoreTarget != null)) {
            Object objingSource = ReverseProperties.getInstance().getMappedElement(ecoreSource);
            if (objingSource != null){
                if (objingSource instanceof IActivityNode)
                    ((IActivityEdge) flow)
                    .setSource((IActivityNode) objingSource);
            }else{
                Modelio.getInstance().getModelingSession().getModel().deleteElement(flow);
            }
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(flow);
        }
    }

    @objid ("0a6cfee7-4583-11e0-b54c-00137279a832")
    private void setTarget(IObjectFlow flow) {
        ActivityNode ecoreTarget = ecoreElement.getTarget();
        if (ecoreTarget != null) {
            Object objingTarget = ReverseProperties.getInstance().getMappedElement(ecoreTarget);
            if (objingTarget instanceof IActivityNode)
                ((IActivityEdge) flow).setTarget((IActivityNode) objingTarget);
        }
    }

    @objid ("0a6cfeea-4583-11e0-b54c-00137279a832")
    private void setInterrupts(IObjectFlow flow) {
        InterruptibleActivityRegion ecoreRegion = ecoreElement.getInterrupts();
        if (ecoreRegion != null) {
            Object objingRegion = ReverseProperties.getInstance().getMappedElement(ecoreRegion);
            if (objingRegion instanceof IInterruptibleActivityRegion)
                ((IActivityEdge) flow)
                .setInterrupts((IInterruptibleActivityRegion) objingRegion);
        }
    }

    @objid ("0a6cfeed-4583-11e0-b54c-00137279a832")
    private void setGuard(IObjectFlow flow) {
        ValueSpecification ecoreGuard = ecoreElement.getGuard();
        if (ecoreGuard != null) {
            String stringValue = ecoreGuard.stringValue();
            if (stringValue != null)
                flow.setGuard(stringValue);
            else
                flow.setGuard("");
                
        }else
            flow.setGuard("");
    }

    @objid ("0a6cfef0-4583-11e0-b54c-00137279a832")
    private void setWeight(IObjectFlow flow) {
        boolean isSet = false;
        ValueSpecification ecoreWeight = ecoreElement.getWeight();
        if (ecoreWeight != null) {
            String stringValue = ecoreWeight.stringValue();
            if (stringValue != null) {
                flow.setWeight(stringValue);
                isSet = true;
            }
        }
                
        if (!isSet)
            flow.setWeight("1");
    }

    @objid ("0a6cfef3-4583-11e0-b54c-00137279a832")
    private void setEffectEAnnotation(IObjectFlow flow) {
        String effect = ObjingEAnnotation.getEffect(ecoreElement);
        if ("CREATE_FLOW".equals(effect))
            flow.setEffect(ObObjectFlowEffectKindEnum.CREATE_FLOW);
        else if ("DELETE_FLOW".equals(effect))
            flow.setEffect(ObObjectFlowEffectKindEnum.DELETE_FLOW);
        else if ("EXCEPTION_FLOW".equals(effect))
            flow.setEffect(ObObjectFlowEffectKindEnum.EXCEPTION_FLOW);
        else if ("READ_FLOW".equals(effect))
            flow.setEffect(ObObjectFlowEffectKindEnum.READ_FLOW);
        else if ("UPDATEFLOW".equals(effect))
            flow.setEffect(ObObjectFlowEffectKindEnum.UPDATEFLOW);
        else
            flow.setEffect(ObObjectFlowEffectKindEnum.READ_FLOW);
    }

    @objid ("0a6f613e-4583-11e0-b54c-00137279a832")
    private void setMultiCast(IObjectFlow flow) {
        flow.setMultiCast(ecoreElement.isMulticast());
    }

    @objid ("0a6f6141-4583-11e0-b54c-00137279a832")
    private void setMultiReceive(IObjectFlow flow) {
        flow.setMultiReceive(ecoreElement.isMultireceive());
    }

    @objid ("0a6f6144-4583-11e0-b54c-00137279a832")
    private void setSelectionBehavior(IObjectFlow flow) {
        Behavior ecoreBehavior = ecoreElement.getSelection();
        if (ecoreBehavior instanceof OpaqueBehavior) {
            String objingBehavior = "";
            for (Object body : ((OpaqueBehavior) ecoreBehavior).getBodies()) {
                objingBehavior = objingBehavior.concat((String) body);
            }
            flow.setSelectionBehavior(objingBehavior);
        } else if (ecoreBehavior != null) {
            String behaviorName = ecoreBehavior.getName();
            if (behaviorName != null)
                flow.setSelectionBehavior(behaviorName);
        }
    }

    @objid ("0a6f6147-4583-11e0-b54c-00137279a832")
    private void setTransformationBehavior(IObjectFlow flow) {
        Behavior ecoreBehavior = ecoreElement.getTransformation();
        if (ecoreBehavior instanceof OpaqueBehavior) {
            String objingBehavior = "";
            for (Object body : ((OpaqueBehavior) ecoreBehavior).getBodies()) {
                objingBehavior = objingBehavior.concat((String) body);
            }
            flow.setTransformationBehavior(objingBehavior);
        } else if (ecoreBehavior != null) {
            String behaviorName = ecoreBehavior.getName();
            if (behaviorName != null)
                flow.setTransformationBehavior(behaviorName);
        }
    }

    @objid ("0a6f614a-4583-11e0-b54c-00137279a832")
    public EObjectFlow() {
    }

}
