/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IOpaqueBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IInternalTransition;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.State;

/**
 * This class handles the import of Ecore Opaque Behavior
 * @author ebrosse
 */
@objid ("0a84d668-4583-11e0-b54c-00137279a832")
public class EOpaqueBehavior extends ENamedElement implements IEElement {
    @objid ("0a84d66c-4583-11e0-b54c-00137279a832")
    private OpaqueBehavior ecoreElement;


    @Override
    @objid ("0a84d66d-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Element ecoreOwner = this.ecoreElement.getOwner();
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Object objOwner = revProp.getMappedElement(ecoreOwner);
        if (objOwner instanceof ITransition){
            String content = "";
            for (String temp : this.ecoreElement.getBodies()){
                content = content + temp;
            }
            ((ITransition) objOwner).setEffect(content);
            
            for (Behavior beh : this.ecoreElement.getRedefinedBehaviors()){
                Object objBeh = revProp.getMappedElement(beh);
                if(objBeh instanceof IBehavior){
                    ((ITransition) objOwner).setBehaviorEffect((IBehavior) objBeh);
                    break;
                }
                
            }
        
            return null;
        }else if (revProp.isRoundtripEnabled() && ObjingEAnnotation.isDeleted(this.ecoreElement)){
            return null;
        }else
            return Modelio.getInstance().getModelingSession().getModel().createOpaqueBehavior();
    }

    @objid ("0a84d671-4583-11e0-b54c-00137279a832")
    public EOpaqueBehavior(OpaqueBehavior element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("0a8738ca-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Element ecoreOwner = this.ecoreElement.getOwner();
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        
        INameSpace objingOwner = EcoreModelNavigation.getNearestOpaqueBehaviorOwner(this.ecoreElement);
        
        if (objingOwner != null){
            IOpaqueBehavior objingDTImport = (IOpaqueBehavior) objingElt;
            objingDTImport.setOwner(objingOwner);
        }
        
        Object objOwner = revProp.getMappedElement(ecoreOwner);
        if (objOwner instanceof ITransition){
            String content = "";
            for (String temp : this.ecoreElement.getBodies()){
                content = content + temp;
            }
            ((ITransition) objOwner).setEffect(content);
        
            for (Behavior beh : this.ecoreElement.getRedefinedBehaviors()){
                Object objBeh = revProp.getMappedElement(beh);
                if(objBeh instanceof IBehavior){
                    ((ITransition) objOwner).setBehaviorEffect((IBehavior) objBeh);
                    break;
                }
                
            }
        
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        
        }else if (objOwner instanceof IState){
        
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
            IInternalTransition internaltransion = Modelio.getInstance().getModelingSession().getModel().createInternalTransition();
            internaltransion.setSComposed((IState)objOwner);
            String content = "";
            for (String temp : this.ecoreElement.getBodies()){
                content = content + temp;
            }
            internaltransion.setEffect(content);
            if ((((State)ecoreOwner).getExit() != null) 
                    && (((State)ecoreOwner).getExit().equals(this.ecoreElement))){
                internaltransion.setReceivedEvents("Exit");
            }else if ((((State)ecoreOwner).getEntry() != null) 
                    && (((State)ecoreOwner).getEntry().equals(this.ecoreElement))){
                internaltransion.setReceivedEvents("Entry");
        
            }else if ((((State)ecoreOwner).getDoActivity() != null) 
                    && (((State)ecoreOwner).getDoActivity().equals(this.ecoreElement))){
                internaltransion.setReceivedEvents("Do");
        
            }
        
        }
    }

    @Override
    @objid ("0a8738cd-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("0a899b1c-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (objingElt != null){
            setBehaviorEffect((IOpaqueBehavior) objingElt);
            setBody((IOpaqueBehavior) objingElt);
        }
    }

    @objid ("0a899b27-4583-11e0-b54c-00137279a832")
    private void setBody(IOpaqueBehavior objingElt) {
        String body = "";
        for (String ecoreBody : this.ecoreElement.getBodies()){
            body += ecoreBody;
        }
        objingElt.setBody(body);
    }

    @objid ("0a8bfd76-4583-11e0-b54c-00137279a832")
    private void setBehaviorEffect(IOpaqueBehavior objingElt) {
        Element ecoreOwner = this.ecoreElement.getOwner();
        
        if (ecoreOwner != null) {
            Object objingOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
            if (objingOwner instanceof ITransition) {
                ((ITransition) objingOwner).setBehaviorEffect(objingElt);
            }
        }
    }

}
