/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IRaisedException;
import com.modeliosoft.modelio.api.model.uml.statik.ObMethodPassingModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.CallConcurrencyKind;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Type;

@objid ("0a932484-4583-11e0-b54c-00137279a832")
public class EOperation extends EFeature implements IEElement {

    @objid ("0a932485-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Element ecoreOwner = getEcoreElement().getOwner();
        Object objingOwner =  ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        if ((objingOwner instanceof IClassifier)  && !(objingOwner instanceof IEnumeration)){
            return Modelio.getInstance().getModelingSession().getModel().createOperation();
        }else{
            return null;
        }
    }

    @objid ("0a93248d-4583-11e0-b54c-00137279a832")
    public EOperation(Operation element) {
        super(element);
    }

    @objid ("0a9586e2-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (objingElt != null) {
            ReverseProperties revProp = ReverseProperties.getInstance();
            Operation ecoreElement = (Operation) getEcoreElement();
            Element ecoreOwner = ecoreElement.getOwner();
            IElement objingOwner = (IElement) revProp
            .getMappedElement(ecoreOwner);
        
            if ((objingOwner instanceof IClassifier)  && !(objingOwner instanceof IEnumeration))
                ((IOperation) objingElt).setOwner((IClassifier) objingOwner);
        
            else {
                PartialImportMap.getInstance().remove(ecoreElement);
                TotalImportMap.getInstance().remove(ecoreElement);
                Modelio.getInstance().getModelingSession().getModel()
                .deleteElement(objingElt);
            }
        }
    }

    @objid ("0a9586e5-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0a9586ea-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (objingElt != null) {
            //                    setCreateEAnnotation((IOperation) objingElt);
            //                    setDestroyEAnnotation((IOperation) objingElt);
            setVisibility((IOperation) objingElt);
            setAbstract((IOperation) objingElt);
            setFinal((IOperation) objingElt);
            setIsClass((IOperation) objingElt);
            setPassingMode((IOperation) objingElt);
            setConcurrency((IOperation) objingElt);
            setRaisedException((IOperation) objingElt);
            setRedefines((IOperation) objingElt);
        }
        
        setMethod((IOperation) objingElt);
    }

    @objid ("0a9586ed-4583-11e0-b54c-00137279a832")
    private boolean isExceptionAlreadyThrown(IOperation objingElt, IElement objingThrownTypeFromMap) {
        for (IRaisedException objingExcep : objingElt.getThrown()) {
            IClassifier objingThrownType = objingExcep.getThrownType();
            if (objingThrownTypeFromMap.equals(objingThrownType))
                return true;
        }
        return false;
    }

    @objid ("0a97e940-4583-11e0-b54c-00137279a832")
    private void setRaisedException(IOperation objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        // Test if the number of raised exceptions in the Ecore model is the
        // same in the Ijing model:
        EList<?> ecoreRaisedExceptions = ((Operation)getEcoreElement()).getRaisedExceptions();
        ObList<IRaisedException> objingRaisedExceptions = objingElt
        .getThrown();
        int objingExcepNumber = objingRaisedExceptions.size();
        
        // Warning : no update in case of same number of thrown exceptions.
        if (ecoreRaisedExceptions.size() != objingExcepNumber) {
            boolean isNewExeptionLink;
            for (Object raisedException : ecoreRaisedExceptions) {
                isNewExeptionLink = false;
                Type ecoreThrownType = (Type) raisedException;
                IElement objingThrownType = (IElement) revProp
                .getMappedElement(ecoreThrownType);
        
                if (objingThrownType instanceof IClassifier) {
                    if (isNewExeptionLink
                            || !isExceptionAlreadyThrown(objingElt,
                                    objingThrownType)) {
                        IRaisedException objingExceptionLink = Modelio
                        .getInstance().getModelingSession().getModel()
                        .createRaisedException();
        
                        objingExceptionLink.setThrower(objingElt);
                        objingExceptionLink
                        .setThrownType((IClassifier) objingThrownType);
                    }
                }
            }
        }
    }

    @objid ("0a97e943-4583-11e0-b54c-00137279a832")
    private void setAbstract(IOperation objingElt) {
        objingElt.setAbstract(((Operation)getEcoreElement()).isAbstract());
    }

    @objid ("0a97e946-4583-11e0-b54c-00137279a832")
    private void setFinal(IOperation objingElt) {
        objingElt.setFinal(((Operation)getEcoreElement()).isLeaf());
    }

    @objid ("0a9a4b92-4583-11e0-b54c-00137279a832")
    private void setIsClass(IOperation objingElt) {
        objingElt.setClass(((Operation)getEcoreElement()).isStatic());
    }

    @objid ("0a9a4b95-4583-11e0-b54c-00137279a832")
    private void setPassingMode(IOperation objingElt) {
        if (((Operation)getEcoreElement()).isQuery())
            objingElt.setPassing(ObMethodPassingModeEnum.METHOD_IN);
        else
            objingElt.setPassing(ObMethodPassingModeEnum.METHOD_OUT);
    }

    @objid ("0a9a4b98-4583-11e0-b54c-00137279a832")
    private void setConcurrency(IOperation objingElt) {
        // Since concurrency is represented as a boolean in Ijing, the mapping
        // is true for the
        // CONCURRENT value, and false else (SEQUENTIAL by default):
        Operation ecoreElement = ((Operation)getEcoreElement());
        switch (ecoreElement.getConcurrency().getValue()) {
        case CallConcurrencyKind.CONCURRENT:
            objingElt.setConcurrency(true);
        default:
            objingElt.setConcurrency(false);
        }
    }

    @objid ("0a9a4b9b-4583-11e0-b54c-00137279a832")
    public EOperation() {
    }

    @objid ("0a9a4b9d-4583-11e0-b54c-00137279a832")
    private void setMethod(IOperation objingElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        Operation ecoreElement = ((Operation)getEcoreElement());
        for (Behavior behavior : ecoreElement.getMethods()){
            if (behavior.getOwner() instanceof Transition){
                Object temp = ReverseProperties.getInstance().getMappedElement(behavior.getOwner());
                if (temp instanceof ITransition){
                    ((ITransition) temp).setProcessed(objingElt);
                }
            }else{
                IDependency dependency = session.getModel().createDependency();
                try{
                    dependency.addExtension(session.getMetamodelExtensions()
                            .getStereotype(IDependency.class, XMIStereotypes.UML2METHODREFERENCE));
                }catch(StereotypeNotFoundException e){
        
                }
        
                IModelElement obBehavior = (IModelElement)ReverseProperties.getInstance().getMappedElement(behavior);
                dependency.setDependsOn(obBehavior);
                dependency.setImpacted(objingElt);
            }
        }
    }

    @objid ("0a9cadec-4583-11e0-b54c-00137279a832")
    private void setRedefines(IOperation objingElt) {
        for (Operation ecoreOperation : ((Operation)getEcoreElement()).getRedefinedOperations()){
            Object objOperation = ReverseProperties.getInstance().getMappedElement(ecoreOperation);
            if (objOperation instanceof IOperation){
                objingElt.addRedefinition((IOperation) objOperation);
            }
        }
    }

}
