/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ICallAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOutputPin;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ObObjectNodeOrderingKindEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.CallAction;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ObjectNodeOrderingKind;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.OutputPin;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.Type;

@objid ("0aa3d4fa-4583-11e0-b54c-00137279a832")
public class EOutputPin extends EPin implements IEElement {
    @objid ("0aa3d4fc-4583-11e0-b54c-00137279a832")
    private OutputPin ecoreElement;


    @objid ("0aa3d4fe-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return  Modelio.getInstance().getModelingSession().getModel()
        .createOutputPin();
    }

    @objid ("0aa3d502-4583-11e0-b54c-00137279a832")
    public EOutputPin(OutputPin element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0aa6375c-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        attachToAction(objingElt);
    }

    @objid ("0aa6375f-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0aa899ae-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        // Properties defined on ObjectNode
        setControlType((IOutputPin) objingElt);
        setOrdering((IOutputPin) objingElt);
        setSelectionBehavior((IOutputPin) objingElt);
        setType((IOutputPin) objingElt);
        // setRepresented((IOutputPin)objingElt);
        // setRepresentedAttribute((IOutputPin)objingElt);
        // setRepresentedRole((IOutputPin)objingElt);
        // setRepresentedRealParameter((IOutputPin)objingElt);
        setState((IOutputPin) objingElt);
        
        // Properties defined on Pin
        setControl((IOutputPin) objingElt);
        setExpansionEAnnotation((IOutputPin) objingElt);
        setMatched((IOutputPin) objingElt);
    }

    @objid ("0aa899b1-4583-11e0-b54c-00137279a832")
    private void attachToAction(IElement objingElt) {
        Action ecoreAction = (Action) getEcoreElement().getOwner();
        
        Object objingAction = ReverseProperties.getInstance().getMappedElement(ecoreAction);
        if (objingAction instanceof IActivityAction) 
            ((IOutputPin) objingElt).setOutputing((IActivityAction) objingAction);
        else{
            XMILogs.getInstance().writelnInLog("owner of outputpin was " + objingAction.getClass().getSimpleName());
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        }
    }

    @objid ("0aaafc0c-4583-11e0-b54c-00137279a832")
    private void setControlType(IOutputPin pin) {
        pin.setControlType(ecoreElement.isControlType());
    }

    @objid ("0aaafc0f-4583-11e0-b54c-00137279a832")
    private void setOrdering(IOutputPin pin) {
        switch (ecoreElement.getOrdering().getValue()) {
        case ObjectNodeOrderingKind.FIFO:
            pin.setOrdering(ObObjectNodeOrderingKindEnum.F_I_F_O);
            break;
        case ObjectNodeOrderingKind.LIFO:
            pin.setOrdering(ObObjectNodeOrderingKindEnum.L_I_F_O);
            break;
        case ObjectNodeOrderingKind.ORDERED:
            pin.setOrdering(ObObjectNodeOrderingKindEnum.ORDERED);
            break;
        case ObjectNodeOrderingKind.UNORDERED:
            pin.setOrdering(ObObjectNodeOrderingKindEnum.UNORDERED);
            break;
        default:
            pin.setOrdering(ObObjectNodeOrderingKindEnum.F_I_F_O);
        }
    }

    @objid ("0aaafc12-4583-11e0-b54c-00137279a832")
    private void setSelectionBehavior(IOutputPin pin) {
        Behavior ecoreBehavior = ecoreElement.getSelection();
        if (ecoreBehavior instanceof OpaqueBehavior) {
            String objingBehavior = "";
            for (Object body : ((OpaqueBehavior) ecoreBehavior).getBodies()) {
                objingBehavior = objingBehavior.concat((String) body);
            }
            pin.setSelectionBehavior(objingBehavior);
        } else if (ecoreBehavior != null) {
            String behaviorName = ecoreBehavior.getName();
            if (behaviorName != null)
                pin.setSelectionBehavior(behaviorName);
        }
    }

    @objid ("0aad5e62-4583-11e0-b54c-00137279a832")
    private void setType(IOutputPin pin) {
        Type ecoreType = ecoreElement.getType();
        if (ecoreType != null) {
            Object objingType = ReverseProperties.getInstance().getMappedElement(ecoreType);
            if (objingType instanceof IGeneralClass)
                pin.setType((IGeneralClass) objingType);
        }
    }

    @objid ("0aad5e65-4583-11e0-b54c-00137279a832")
    private void setState(IOutputPin pin) {
        EList<?> ecoreStates = ecoreElement.getInStates();
        if (ecoreStates != null && ecoreStates.size() > 0) {
            State ecoreState = (State) ecoreStates.get(0);
            if (ecoreState != null) {
                Object objingState = ReverseProperties.getInstance().getMappedElement(ecoreState);
                if (objingState instanceof IState)
                    pin.setInState((IState) objingState);
            }
        }
    }

    @objid ("0aad5e68-4583-11e0-b54c-00137279a832")
    private void setControl(IOutputPin pin) {
        pin.setControl(ecoreElement.isControl());
    }

    @objid ("0aad5e6b-4583-11e0-b54c-00137279a832")
    private void setExpansionEAnnotation(IOutputPin pin) {
        pin.setExpansion(ObjingEAnnotation.isExpansion(ecoreElement));
    }

    @objid ("0aad5e6e-4583-11e0-b54c-00137279a832")
    private void setMatched(IOutputPin pin) {
        Element ecoreOwnerElt = ecoreElement.getOwner();
        Object obOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwnerElt);
        
        if ((ecoreOwnerElt instanceof CallAction) && (obOwner instanceof ICallAction)){
        
            List<Parameter> ecoreParamList = EcoreModelNavigation
            .getMatchedParameters(ecoreElement);
        
            List<OutputPin> outputPinList = ((CallAction) ecoreOwnerElt).getResults();
            
            for (Parameter ecoreParam : ecoreParamList) {
        
                if (((ecoreElement.getType() != null)
                        && (ecoreParam.getType() != null)
                        && (ecoreElement.getType().equals(ecoreParam.getType())))
                        && (EcoreModelNavigation.getMultiplicityMax(ecoreElement).equals(EcoreModelNavigation.getMultiplicityMax(ecoreParam)))
                        && (EcoreModelNavigation.getMultiplicityMin(ecoreElement).equals(EcoreModelNavigation.getMultiplicityMin(ecoreParam)))
                        && (ecoreParamList.indexOf(ecoreParam) == outputPinList.indexOf(ecoreElement))
        //                        && (EcoreModelNavigation.getOrdering(ecoreParam) == EcoreModelNavigation.getOrdering(ecoreElement))
                        ){
        
                    Object obParam = ReverseProperties.getInstance()
                    .getMappedElement(ecoreParam);
        
                    if (obParam instanceof IParameter) {
                        pin.setMatched((IParameter) obParam);
                    }
                }
            }
        }
    }

    @objid ("0aafc0c1-4583-11e0-b54c-00137279a832")
    public EOutputPin() {
    }

}
