/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.model.uml.statik.IPackageImport;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.ObElementNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageImport;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * This class manages the import of Ecoe PackageImports
 * @author ebrosse
 */
@objid ("0ab4858a-4583-11e0-b54c-00137279a832")
public class EPackageImport extends EElement implements IEElement {
    @objid ("0ab6e7ca-4583-11e0-b54c-00137279a832")
    private PackageImport ecoreElement;


    @Override
    @objid ("0ab6e7cb-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Package ecoreImported = this.ecoreElement.getImportedPackage();
        
        Package model = EcoreModelNavigation.getRoot(ecoreImported);
        
        if  ((model != null)  && revProp.getEcoreModel().contains(model) ){
            return Modelio.getInstance()
            .getModelingSession().getModel().createPackageImport();
        }
        return null;
    }

    /**
     * The EPackageImport constructor.
     * It takes the imported Ecore PackageImport as parameter
     * @param element : the imported Ecore PackageImport
     */
    @objid ("0ab6e7d3-4583-11e0-b54c-00137279a832")
    public EPackageImport(final PackageImport element) {
        super(element);
        this.ecoreElement = element;
    }

    @Override
    @objid ("0ab94a28-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Package ecoreImported = this.ecoreElement.getImportedPackage();
        Namespace ecoreImporting = this.ecoreElement.getImportingNamespace();
        
        IPackage objingImported = (IPackage) revProp
        .getMappedElement(ecoreImported);
        IModelElement objingImporting = (IModelElement) revProp
        .getMappedElement(ecoreImporting);
        
        Package model = EcoreModelNavigation.getRoot(ecoreImported);
        
        if  ((model != null)  && revProp.getEcoreModel().contains(model) ){
        
            if (objingImported != null && objingImporting != null) {
                IPackageImport objingPkgImport = (IPackageImport) objingElt;
        
                objingPkgImport.setImportedPackage(objingImported);
                if (objingImporting instanceof IOperation)
                    objingPkgImport
                    .setImportingOperation((IOperation) objingImporting);
                else
                    objingPkgImport
                    .setImportingNameSpace((INameSpace) objingImporting);
            }
        
            // I set to the objingElt Imported Importing previousely find
            if (objingImported != null){
                IPackageImport objingPkgImport = (IPackageImport) objingElt;
                objingPkgImport.setImportedPackage(objingImported);
                if (objingImporting != null) {
                    if (objingImporting instanceof INameSpace) {
                        objingPkgImport
                        .setImportingNameSpace((INameSpace) objingImporting);
                    } else {
                        objingPkgImport
                        .setImportingOperation((IOperation) objingImporting);
                    }
        
                }
            }else{
                String ownerName = "";
                if (ecoreImported != null)
                    ownerName = ecoreImported.getName();
        
                XMILogs xmilogs = XMILogs.getInstance();
                xmilogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.unimportedElement.importPackage.ownerNull"
                        , ownerName));
        
                Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
            }
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingImported);
        }
    }

    @Override
    @objid ("0ab94a2b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("0ab94a30-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        try{
            ReverseProperties revProp = ReverseProperties.getInstance();
            setVisibility((IPackageImport) objingElt);
        
            if (revProp.isRoundtripEnabled())
                setNameFromEAnnotation((IPackageImport) objingElt);
        }catch(ObElementNotFoundException e){
            e.printStackTrace(Modelio.err);
        }

    	super.setProperties(objingElt);
    }

    @objid ("0ab94a33-4583-11e0-b54c-00137279a832")
    private void setNameFromEAnnotation(IPackageImport objingElt) {
        if (ObjingEAnnotation.getName(this.ecoreElement) != null) {
            objingElt.setName(ObjingEAnnotation.getName(this.ecoreElement));
        }
    }

    @objid ("0abbac7e-4583-11e0-b54c-00137279a832")
    private void setVisibility(IPackageImport objingElt) {
        switch (this.ecoreElement.getVisibility().getValue()) {
        case VisibilityKind.PUBLIC:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
            break;
        case VisibilityKind.PRIVATE:
            objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
            break;
        default:
            // Visibility of a PackageImport should be Public or Private.
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
        // TODO : inform the user of the illegal value of the visibility in
        // a report
        // window.
        }
        
        if (ObjingEAnnotation.isUndefined(this.ecoreElement))
            objingElt
            .setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
    }

}
