/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehaviorParameter;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ObParameterEffectKindEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.model.uml.statik.ObPassingModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.BehavioralFeature;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.ValueSpecification;

/**
 * This class manages the import of Ecore Parameter
 * @author ebrosse
 */
@objid ("0ac79841-4583-11e0-b54c-00137279a832")
public class EParameter extends ENamedElement implements IEElement {
    @objid ("0ac79842-4583-11e0-b54c-00137279a832")
    private boolean isBehavior = false;


    @Override
    @objid ("0ac79843-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IElement objingParameter;
        
        if (this.isBehavior)
            objingParameter = Modelio.getInstance().getModelingSession().getModel().createBehaviorParameter();
        else 
            objingParameter = Modelio.getInstance().getModelingSession().getModel().createParameter();
        return objingParameter;
    }

    /**
     * Constructor of EParameter.
     * It takes the imported Ecore Parameter as parameter
     * @param element : the imported Ecore Parameter
     */
    @objid ("0ac9fa9f-4583-11e0-b54c-00137279a832")
    public EParameter(final Parameter element) {
        super(element);
        
        Element owner = element.getOwner();
        if (owner != null){
            this.isBehavior = !(ReverseProperties.getInstance().getMappedElement(owner) instanceof IOperation);
        }
    }

    @Override
    @objid ("0acc5cf4-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Parameter ecoreElement = ((Parameter)getEcoreElement());
        if (objingElt != null) {
            if (this.isBehavior){
                Object owner =  ReverseProperties.getInstance().getMappedElement(ecoreElement.getOwner());
                if ((owner != null) && (owner instanceof IBehavior))
                    ((IBehavior) owner).addParameter((IBehaviorParameter)objingElt);
                else{ 
                    PartialImportMap.getInstance().remove(ecoreElement);
                    TotalImportMap.getInstance().remove(ecoreElement);
                    Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
                }
        
            }else{
        
                Element ecoreOwner = ecoreElement.getOwner();
                IElement objingOperation = null;
                try{
                    objingOperation = (IElement) ReverseProperties.getInstance()
                    .getMappedElement(ecoreOwner);
                }catch (RuntimeException e){
                    Modelio.err.println("RuntimeException : ");
                    e.printStackTrace(Modelio.err);
                }
        
                if ((objingOperation instanceof IOperation) && (objingOperation != null)){
                    ParameterDirectionKind direction = ecoreElement.getDirection();
        
                    // The same processing is applied for In, Out and InOut
                    // parameters. Only the return
                    // value is a specific case:
                    switch (direction.getValue()) {
                    case ParameterDirectionKind.IN:
                        ((IOperation) objingOperation)
                        .addIO((IParameter) objingElt);
                        ((IParameter) objingElt).setParameterPassing(ObPassingModeEnum.IN);
                        break;
                    case ParameterDirectionKind.OUT:
                        ((IOperation) objingOperation)
                        .addIO((IParameter) objingElt);
                        ((IParameter) objingElt).setParameterPassing(ObPassingModeEnum.OUT);
                        break;
                    case ParameterDirectionKind.INOUT:
                        ((IOperation) objingOperation)
                        .addIO((IParameter) objingElt);
                        ((IParameter) objingElt).setParameterPassing(ObPassingModeEnum.INOUT);
                        break;
                    case ParameterDirectionKind.RETURN:
                        IParameter temp = ((IOperation) objingOperation).getReturn();
                        if (temp != null){
                            Modelio.getInstance().getModelingSession().getModel().deleteElement(temp);
                            XMILogs xmilogs = XMILogs.getInstance();
                            xmilogs.writelnInLog(XMI.I18N
                                    .getMessage("logFile.warning.multipleReturnParameter", ((BehavioralFeature) ecoreOwner).getName()));
                        }
                        ((IOperation) objingOperation)
                        .setReturn((IParameter) objingElt);
                        break;
                    }
                } else {
                    PartialImportMap.getInstance().remove(ecoreElement);
                    TotalImportMap.getInstance().remove(ecoreElement);
                    Modelio.getInstance().getModelingSession().getModel()
                    .deleteElement(objingElt);
                }
            }
        }
    }

    @Override
    @objid ("0acc5cf7-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @Override
    @objid ("0acc5cfc-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (objingElt != null) {
            ReverseProperties revProp = ReverseProperties.getInstance();
        
            setType((IParameter) objingElt);
            setDefaultValue((IParameter) objingElt);
            setDirection((IParameter) objingElt);
            setMultiplicity((IParameter) objingElt);
            setUnique((IParameter) objingElt);
            setOrdered((IParameter) objingElt);
            
            if (revProp.isRoundtripEnabled()) {
                setTypeConstraint((IParameter) objingElt);
            }
        
            if (this.isBehavior){
                setEffect((IBehaviorParameter) objingElt);
                setIsException((IBehaviorParameter) objingElt);
                setIsStream((IBehaviorParameter) objingElt);
            }
        }
    }

    @objid ("0acc5cff-4583-11e0-b54c-00137279a832")
    private void setDefaultValue(IParameter objingElt) {
        Parameter ecoreElement = ((Parameter)getEcoreElement());
        ValueSpecification defaultValue = ecoreElement.getDefaultValue();
        if (defaultValue != null) {
            String defaultValueStr = defaultValue.stringValue();
            if (defaultValueStr != null)
                objingElt.setDefaultValue(defaultValueStr);
        }
        
        String value =ecoreElement.getDefault();
        if (value != null)
            objingElt.setDefaultValue(value);
    }

    @objid ("0acc5d02-4583-11e0-b54c-00137279a832")
    private void setDirection(IParameter objingElt) {
        Parameter ecoreElement = ((Parameter)getEcoreElement());
        ParameterDirectionKind direction = ecoreElement.getDirection();
        
        switch (direction.getValue()) {
        case ParameterDirectionKind.IN:
            objingElt.setParameterPassing(ObPassingModeEnum.IN);
            break;
        case ParameterDirectionKind.OUT:
            objingElt.setParameterPassing(ObPassingModeEnum.OUT);
            break;
        case ParameterDirectionKind.INOUT:
            objingElt.setParameterPassing(ObPassingModeEnum.INOUT);
            break;
        case ParameterDirectionKind.RETURN:
            break;
        }
    }

    @objid ("0acebf4e-4583-11e0-b54c-00137279a832")
    private void setMultiplicity(IParameter objingElt) {
        objingElt.setMultiplicityMin(EcoreModelNavigation.getMultiplicityMin((Parameter) this.getEcoreElement()));
        
        
        objingElt.setMultiplicityMax(EcoreModelNavigation.getMultiplicityMax((Parameter)this.getEcoreElement()));
    }

    @objid ("0acebf54-4583-11e0-b54c-00137279a832")
    private void setType(IParameter objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Parameter ecoreElement = ((Parameter)getEcoreElement());
        
        Type ecoreType = ecoreElement.getType();
        
        if (ecoreType != null) {
            IElement objingType = (IElement) revProp
            .getMappedElement(ecoreType);
        
            if (objingType != null) {
                if (objingType instanceof IGeneralClass) {
                    objingElt.setType((IGeneralClass) objingType);
                }else{
                    XMILogs xmiLogs = XMILogs.getInstance();
                    xmiLogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.unsupportedRelationBetweenTypeImported", 
                            "Type", ecoreElement.getName(), "EParameter",ecoreType.getName(), ecoreType.getClass().getSimpleName()));
                }
            } 
        }
        else {
            if (!(( ReverseProperties.getInstance().isRoundtripEnabled()) 
                    && (ObjingEAnnotation.isNoType(ecoreElement))))
        
                objingElt.setType(Modelio.getInstance().getModelingSession()
                        .getModel().getUmlTypes().getUNDEFINED());
        
        }
    }

    @objid ("0acebf57-4583-11e0-b54c-00137279a832")
    private void setTypeConstraint(IParameter objingElt) {
        String typeConstraint = ObjingEAnnotation
        .getTypeConstraint((getEcoreElement()));
        if (typeConstraint != null)
            objingElt.setTypeConstraint(typeConstraint);
    }

    @objid ("0ad121a8-4583-11e0-b54c-00137279a832")
    private void setIsStream(IBehaviorParameter objingElt) {
        objingElt.setStream(((Parameter)getEcoreElement()).isStream());
    }

    @objid ("0ad121ab-4583-11e0-b54c-00137279a832")
    private void setIsException(IBehaviorParameter objingElt) {
        objingElt.setException(((Parameter)getEcoreElement()).isException());
    }

    @objid ("0ad121ae-4583-11e0-b54c-00137279a832")
    private void setEffect(IBehaviorParameter objingElt) {
        Parameter ecoreElement = ((Parameter)getEcoreElement());
        switch (ecoreElement.getEffect()){
        case CREATE_LITERAL : 
            objingElt.setEffect(ObParameterEffectKindEnum.CREATE);
            break;
        case DELETE_LITERAL : 
            objingElt.setEffect(ObParameterEffectKindEnum.DELETE);
            break;
        case READ_LITERAL : 
            objingElt.setEffect(ObParameterEffectKindEnum.READ);
            break;
        case UPDATE_LITERAL : 
            objingElt.setEffect(ObParameterEffectKindEnum.UPDATE);
            break;
        
        }
    }

    @objid ("7aa79a6a-63b7-11e1-904f-0027103f347d")
    private void setOrdered(final IParameter objingElt) {
        objingElt.setOrdered(((Parameter)getEcoreElement()).isOrdered());
    }

    @objid ("7aa7c17d-63b7-11e1-904f-0027103f347d")
    private void setUnique(final IParameter objingElt) {
        objingElt.setUnique(((Parameter)getEcoreElement()).isUnique());
    }

}
