/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.statik.IProvidedInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IRequiredInterface;
import com.modeliosoft.modelio.api.model.uml.statik.ObPortOrientationEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.Usage;

@objid ("0ae43478-4583-11e0-b54c-00137279a832")
public class EPort extends EFeature implements IEElement {
    @objid ("0ae43479-4583-11e0-b54c-00137279a832")
    private Port ecoreElement;


    @objid ("0ae4347a-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel().createPort();
    }

    @objid ("0ae43482-4583-11e0-b54c-00137279a832")
    public EPort(Port element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0ae696d7-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ObjingModelNavigation.attachPort(objingElt, ecoreElement);
    }

    @objid ("0ae696da-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0ae696df-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        
        setBehavior((IPort) objingElt);
        setService((IPort) objingElt);
        setRequiredInterfaces((IPort) objingElt);
        setProvidedInterfaces((IPort) objingElt);
        setType((IPort) objingElt);
        setEnd((IPort) objingElt);
        setRedefinedPort((IPort)objingElt);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setInterfaces((IPort) objingElt);
            setPortDirection((IPort) objingElt);
        }
    }

    @objid ("0ae8f92c-4583-11e0-b54c-00137279a832")
    private void setType(IPort objingElt) {
        Type type = this.ecoreElement.getType();
        if (type != null) {
            IElement objType = (IElement) ReverseProperties.getInstance().getMappedElement(type);
            if (objType instanceof INameSpace)
                objingElt.setBase((INameSpace) objType);
        
        
            if (type instanceof IInterface){
                boolean found = false; 
                for (IProvidedInterface provided : objingElt.getProvided()){
                    if ((provided.getProvidedElement().size() == 1) && (provided.getProvidedElement().get(0).equals(objType))){
                        found = true;
                    }
                }
        
                if (!found) {
                    IProvidedInterface provider = Modelio.getInstance()
                    .getModelingSession().getModel().createProvidedInterface();
        
                    objingElt.addProvided(provider);
                    provider.setProviding(objingElt);
                    provider.addProvidedElement((IInterface)type);
        
                }
            }
        }
    }

    @objid ("0ae8f92f-4583-11e0-b54c-00137279a832")
    private void setInterfaces(IPort objingElt) {
        Integer temp = ObjingEAnnotation.getNumberProvidedInterface(this.ecoreElement);
        for (int i = 0; i < temp; i++){
            IProvidedInterface providedInterface = Modelio.getInstance().getModelingSession().getModel()
            .createProvidedInterface();
            providedInterface.setProviding(objingElt);
            objingElt.addProvided(providedInterface);
        }
        
        temp = ObjingEAnnotation.getNumberRequiredInterface(this.ecoreElement);
        for (int i = 0; i < temp; i++){
            IRequiredInterface requiredInterface = Modelio.getInstance().getModelingSession().getModel()
            .createRequiredInterface();
            requiredInterface.setRequiring(objingElt);
            objingElt.addRequired(requiredInterface);
        }
    }

    @objid ("0ae8f932-4583-11e0-b54c-00137279a832")
    private void setProvidedInterfaces(IPort objingElt) {
        Type type = this.ecoreElement.getType();
        if (type != null) {
            if (type instanceof BehavioredClassifier){
                BehavioredClassifier behaClass = (BehavioredClassifier)type;
                for (Object object : behaClass.getInterfaceRealizations()){
                    if (object instanceof InterfaceRealization){
                        InterfaceRealization interfaceRealization = (InterfaceRealization) object;
                        IProvidedInterface providedInterface =  null;
                        Object temp = ReverseProperties.getInstance().getMappedElement((interfaceRealization).getContract());
                        if (temp instanceof IInterface){
                            providedInterface = Modelio.getInstance().getModelingSession().getModel()
                            .createProvidedInterface();
                            providedInterface.setProviding(objingElt);
                            providedInterface.addProvidedElement((IInterface)temp);
                            objingElt.addProvided(providedInterface);
                        }
                    }
                }
            }
        
        }
    }

    @objid ("0ae8f935-4583-11e0-b54c-00137279a832")
    private void setRequiredInterfaces(IPort objingElt) {
        Type type = this.ecoreElement.getType();
        if (type != null) {
            setRequiredInterface(objingElt, type);
            if (type instanceof Classifier){
                for (Object superType : ((Classifier)type).getGenerals()){
                    setRequiredInterface(objingElt, (Type) superType);
                }
            }
        }
    }

    @objid ("0ae8f938-4583-11e0-b54c-00137279a832")
    private void setRequiredInterface(IPort objingElt, Type type) {
        for (Object object : type.getSourceDirectedRelationships()){
            if (object instanceof Usage){
                Usage usage = (Usage) object;
                List<IModelElement> usages = new ArrayList<IModelElement>();
        
                for (Object supplier : usage.getSuppliers()){
                    Object objSupplier =  ReverseProperties.getInstance().getMappedElement((Element)supplier);
                    if (objSupplier instanceof IInterface){
                        IRequiredInterface requiredInterface = Modelio.getInstance().getModelingSession().getModel().createRequiredInterface();
                        requiredInterface.setRequiring(objingElt);
                        requiredInterface.addRequiredElement((IInterface) objSupplier);
                        objingElt.addRequired(requiredInterface);
                        usages.add(requiredInterface);
                    }
                }
        
                TotalImportMap.getInstance().put(usage, usages);
            }
        
        }
    }

    @objid ("0aeb5b8a-4583-11e0-b54c-00137279a832")
    private void setEnd(IPort objingElt) {
        for (Object end : this.ecoreElement.getEnds()){
            if (end instanceof ConnectorEnd){
                ConnectorEnd connectorEnd  = (ConnectorEnd) end;
                Property prop = connectorEnd.getPartWithPort();
                if ( prop != null ){
        
                    IPort portClone = clonePort(objingElt);
                    Object objOwner =  ReverseProperties.getInstance().getMappedElement(prop);
        
                    if (objOwner instanceof IInstance) {       
                        ((IInstance) objOwner).addPart(portClone);
                    }else if (objOwner instanceof ArrayList<?>){
                        for (IModelElement objElt : (ArrayList<IModelElement>) objOwner){
                            if (objElt instanceof IInstance)
                                ((IInstance) objElt).addPart(portClone);
                        }
                    }
        
                    Object objConnectorEnd =  ReverseProperties.getInstance().getMappedElement(connectorEnd);
                    if (objConnectorEnd instanceof IConnectorEnd)
                        portClone.addConnection((IConnectorEnd)objConnectorEnd);
                }
            }
        }
    }

    @objid ("0aeb5b8d-4583-11e0-b54c-00137279a832")
    public EPort() {
    }

    @objid ("6878abde-1a76-11e1-8103-0027103f347d")
    private IPort clonePort(final IPort objingElt) {
        IPort result = Modelio.getInstance().getModelingSession().getModel().createPort();
        result.setRepresentedFeature(objingElt);
        for(IProvidedInterface provided : objingElt.getProvided()){
            result.addProvided((IProvidedInterface) Modelio.getInstance().getModelManipulationService().clone(provided) );
        }
        
        for(IRequiredInterface required : objingElt.getRequired()){
            result.addRequired((IRequiredInterface)   Modelio.getInstance().getModelManipulationService().clone(required) );
        }
        
        result.setBehavior(objingElt.isBehavior());
        result.setService(objingElt.isService());
        result.setName(objingElt.getName());
        result.setBase(objingElt.getBase());
        // set Instance properties
        result.setValue(objingElt.getValue());
        result.setMultiplicityMax(objingElt.getMultiplicityMax());
        result.setMultiplicityMin(objingElt.getMultiplicityMin());
        
        result.setConstant(objingElt.isConstant());
        return result;
    }

    @objid ("6878fa03-1a76-11e1-8103-0027103f347d")
    private void setRedefinedPort(final IPort objingElt) {
        EList<Port> redefinedPorts = this.ecoreElement.getRedefinedPorts();
        
        if ((redefinedPorts != null) && (redefinedPorts.size() > 0)){
            Object redefinedPort = ReverseProperties.getInstance().getMappedElement(redefinedPorts.get(0));
            if (redefinedPort instanceof IModelElement){
                objingElt.setRepresentedFeature((IModelElement) redefinedPort);
            }
        }
    }

    @objid ("68794826-1a76-11e1-8103-0027103f347d")
    private void setBehavior(final IPort objingElt) {
        objingElt.setBehavior(this.ecoreElement.isBehavior());
    }

    @objid ("6879964a-1a76-11e1-8103-0027103f347d")
    private void setService(final IPort objingElt) {
        objingElt.setService(this.ecoreElement.isService());
    }

    @objid ("e51db956-abf0-11e1-9002-0027103f347d")
    private void setPortDirection(final IPort objingElt) {
        String direction = ObjingEAnnotation.getPortDirection(this.ecoreElement);
        objingElt.setDirection(ObPortOrientationEnum.valueOf(direction));
    }

}
