/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.ObVisibilityModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("0aedbde0-4583-11e0-b54c-00137279a832")
public class EPrimitiveType extends ENamedElement implements IEElement {
    @objid ("0aedbde2-4583-11e0-b54c-00137279a832")
    private PrimitiveType ecoreElement;


    @objid ("0aedbde3-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if (!(PrimitiveTypeMapper.isPredefinedType(ecoreElement))) {
            return Modelio.getInstance().getModelingSession().getModel().createDataType();
        }
        return null;
    }

    @objid ("0aedbde7-4583-11e0-b54c-00137279a832")
    public EPrimitiveType(PrimitiveType element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0af02040-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (!PrimitiveTypeMapper.isPredefinedType(ecoreElement)){
            ReverseProperties revProp = ReverseProperties.getInstance();
        
            // we get the owner
            Element ecoreOwner = ecoreElement.getOwner();
        
            Object objingOwner =  revProp.getMappedElement(ecoreOwner);
        
            if ((objingOwner != null) 
                    &&  (objingOwner instanceof IModelTree) 
                    && ! (objingOwner instanceof IProfile)){
                ((IGeneralClass)objingElt).setOwner((IModelTree)objingOwner);
            }else
                ((IGeneralClass)objingElt).setOwner(ReverseProperties.getInstance().getExternalPackage());      
                
        }
    }

    @objid ("0af02043-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0af02048-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt.getElementStatus().isModifiable()){
            super.setProperties(objingElt);
            if (!PrimitiveTypeMapper.isPredefinedType(ecoreElement)){
                if (objingElt instanceof IGeneralClass){
                    setVisibility((IGeneralClass)objingElt);
                }
        
                if (objingElt instanceof IDataType) {
                    setAbstract((IDataType) objingElt);
                    setLeaf((IDataType) objingElt);
        
                    if (ReverseProperties.getInstance().isRoundtripEnabled()) {
                        setPrimitiveEAnnotation((IDataType) objingElt);
                        setRootEAnnotation((IDataType) objingElt);
                    }
        
                }
            }
        }
    }

    @objid ("0af28294-4583-11e0-b54c-00137279a832")
    public EPrimitiveType() {
    }

    @objid ("0af7474b-4583-11e0-b54c-00137279a832")
    private void setAbstract(IDataType objingElt) {
        objingElt.setAbstract(ecoreElement.isAbstract());
    }

    @objid ("0af7474e-4583-11e0-b54c-00137279a832")
    private void setLeaf(IDataType objingElt) {
        objingElt.setLeaf(ecoreElement.isLeaf());
    }

    @objid ("0af74751-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(IDataType objingElt) {
        objingElt.setElementary(ObjingEAnnotation.isPrimitive(ecoreElement));
    }

    @objid ("0af74754-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(IDataType objingElt) {
        objingElt.setRoot(ObjingEAnnotation.isRoot(ecoreElement));
    }

    @objid ("0af9a9a3-4583-11e0-b54c-00137279a832")
    public void setVisibility(IGeneralClass objingElt) {
        switch (ecoreElement.getVisibility().getValue()) {
        case VisibilityKind.PUBLIC:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
            break;
        case VisibilityKind.PRIVATE:
            objingElt.setVisibility(ObVisibilityModeEnum.PRIVATE);
            break;
        case VisibilityKind.PACKAGE:
            objingElt.setVisibility(ObVisibilityModeEnum.PACKAGE_VISIBILITY);
            break;
        case VisibilityKind.PROTECTED:
            objingElt.setVisibility(ObVisibilityModeEnum.PROTECTED);
            break;
        default:
            objingElt.setVisibility(ObVisibilityModeEnum.PUBLIC);
        }
        
        if (ObjingEAnnotation.isUndefined(ecoreElement))
            objingElt
            .setVisibility(ObVisibilityModeEnum.VISIBILITY_UNDEFINED);
    }

}
