/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IAttribute;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.ICollaboration;
import com.modeliosoft.modelio.api.model.uml.statik.IFeature;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.statik.ObAggregationKindEnum;
import com.modeliosoft.modelio.api.model.uml.statik.ObKindOfAccessEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0b059566-4583-11e0-b54c-00137279a832")
public class EProperty extends EFeature implements IEElement {

    @objid ("0b05956a-4583-11e0-b54c-00137279a832")
    public List<IModelElement> createObjingElt() {
        Property ecoreElement = (Property) getEcoreElement();
        
        if (ecoreElement.getOwner() instanceof Stereotype){       
            ProfileUtils.visitProperty(ecoreElement);      
        }else{
        
            List<IModelElement> result = new ArrayList<IModelElement>();
        
            IUmlModel objingModel = Modelio.getInstance().getModelingSession().getModel();
            // association
            if (EcoreModelNavigation.isAssocEnd(ecoreElement)) {
                if (EcoreModelNavigation.hasTwoValidEnds(ecoreElement.getAssociation())
                        && (EcoreModelNavigation.isValid(ecoreElement))){
                    result.add(objingModel.createAssociationEnd());
                }
        
            } 
        
            if (EcoreModelNavigation.isPart(ecoreElement)){
                IBindableInstance inst = objingModel.createBindableInstance();
                if (EcoreModelNavigation.isAssocEnd(ecoreElement)){
                    inst.setRepresentedFeature(result.get(0));
                }
                result.add(inst);
            } else if (EcoreModelNavigation.isPort(ecoreElement)){
                result.add( objingModel.createPort());
            }
        
            if (result.size() == 0 ){ 
                Element ecoreOwner = getEcoreElement().getOwner();
                Object objingOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
                if ((objingOwner instanceof IClassifier) ||  
                        ((ecoreOwner instanceof Property) && (EcoreModelNavigation.isAssocEnd((Property)ecoreOwner))))
                    result.add( objingModel.createAttribute());
            }
            return result;
        }
        return null;
    }

    @objid ("0b118129-4583-11e0-b54c-00137279a832")
    public EProperty(Property element) {
        super(element);
    }

    @objid ("0b11812c-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
        result.add("AssociationEnd");
        result.add("BindableInstance");
        result.add("Attribute");
        return result;
    }

    @objid ("0b118132-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Property ecoreElement = ((Property) getEcoreElement());
        if (!(ecoreElement.getOwner() instanceof Stereotype)){
            if (objingElt instanceof IAttribute){
                attachAttribute((IAttribute) objingElt);
            } else if ((EcoreModelNavigation.isAssocEnd(ecoreElement)) && !(objingElt instanceof IAttribute)) {
                if (EcoreModelNavigation.isPart(ecoreElement)){
                    if (objingElt instanceof IBindableInstance){
                        attachBindableInstance(objingElt);
                        IElement represented = ((IBindableInstance)objingElt).getRepresentedFeature();
                        if (represented instanceof IAssociationEnd)
                            ObjingModelNavigation.attachAssociationEnd(
                                                                       (IAssociationEnd)((IBindableInstance)objingElt).getRepresentedFeature(), (Property) getEcoreElement());
        
                    }else{
                        ObjingModelNavigation.attachAssociationEnd((IAssociationEnd)objingElt, (Property) getEcoreElement());
                    }
                }else{
                    ObjingModelNavigation.attachAssociationEnd((IAssociationEnd)objingElt, (Property) getEcoreElement());
                }
            } else if ((EcoreModelNavigation.isPart(ecoreElement)) && !(EcoreModelNavigation.isPort(ecoreElement))){
                attachBindableInstance(objingElt);
            }
        }
    }

    @objid ("0b13e382-4583-11e0-b54c-00137279a832")
    public void attach(final List<Object> objingElts) {
        for (Object objingElt : objingElts){
            if (objingElt instanceof IElement)
                attach((IElement) objingElt);
        }
    }

    @objid ("0b13e387-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (!(((Property) getEcoreElement()).getOwner() instanceof Stereotype)){
        
            super.setProperties(objingElt);
            if (objingElt instanceof IFeature)
                setFeature( (IFeature) objingElt);
            else if (objingElt instanceof IInstance){
                ObjingModelNavigation.setInstance((IInstance)objingElt, (Property) getEcoreElement() );
                if (objingElt instanceof IBindableInstance){
                    IModelElement represented = ((IBindableInstance)objingElt).getRepresentedFeature();
                    if ((represented != null) && (represented instanceof IAssociationEnd)){
                        setFeature( (IFeature) represented);
                    }
                }
            }
        }
    }

    @objid ("0b13e38a-4583-11e0-b54c-00137279a832")
    private void setFeature(IFeature objingElt) {
        setVisibility((IFeature) objingElt);
        if (objingElt instanceof IAssociationEnd) {
            setAssociation((IAssociationEnd) objingElt);
            setNavigation((IAssociationEnd) objingElt);
            setChangeable((IAssociationEnd) objingElt);
            setAssociationKind((IAssociationEnd) objingElt);
            setOrdered((IAssociationEnd) objingElt);
            setUnique((IAssociationEnd) objingElt);
            setAccessMode((IAssociationEnd) objingElt);
            if (ReverseProperties.getInstance().isRoundtripEnabled()) {
                setAccessModeEAnnotation((IAssociationEnd) objingElt);
            }
        } else if (objingElt instanceof IAttribute) {
            setType((IAttribute) objingElt);
            setOrdered((IAttribute) objingElt);
            setUnique((IAttribute) objingElt);
            setValue((IAttribute) objingElt);
            setAccessMode((IAttribute) objingElt);
        
            if (ReverseProperties.getInstance().isRoundtripEnabled()) {
                setChangeableEAnnotation((IAttribute) objingElt);
                setDerivedEAnnotation((IAttribute) objingElt);
                setTargetIsClassEAnnotation((IAttribute) objingElt);
                setTypeConstraint((IAttribute) objingElt);
                setValueEAnnotation((IAttribute) objingElt);
            }
        }
        if (ReverseProperties.getInstance().isRoundtripEnabled()) {
            setAbstractEAnnotation((IFeature) objingElt);
        }
        setMultiplicity((IFeature) objingElt);
        setClass((IFeature) objingElt);
    }

    @objid ("0b1645db-4583-11e0-b54c-00137279a832")
    private void setNavigation(IAssociationEnd objingElt) {
        Element ecoreOwner = getEcoreElement().getOwner();
        if (ecoreOwner instanceof Association){
            if (((Association)ecoreOwner).getNavigableOwnedEnds().contains(getEcoreElement()))
                objingElt.setNavigable(true);
            else
                objingElt.setNavigable(false);
        }else if (ecoreOwner instanceof Classifier)
            objingElt.setNavigable(true);
    }

    @objid ("0b1645de-4583-11e0-b54c-00137279a832")
    private void setType(IAttribute objingElt) {
        Type ecoreType = ((Property) getEcoreElement()).getType();
        
        if (ecoreType != null) {
            // If the Attribute is typed by another element, we get this
            // Classifier:
            Object objingType = ReverseProperties.getInstance()
                    .getMappedElement(ecoreType);
            if ((objingType != null) && (objingType instanceof IGeneralClass))
                objingElt.setType((IGeneralClass) objingType);
        }
    }

    @objid ("0b1645e4-4583-11e0-b54c-00137279a832")
    private void setAssociationKind(IAssociationEnd end) {
        AggregationKind ecoreAggType = ((Property) getEcoreElement()).getAggregation();
        if (ecoreAggType != null) {
            switch (ecoreAggType.getValue()) {
                case AggregationKind.NONE:
                    end.setAggregation(ObAggregationKindEnum.KIND_IS_ASSOCIATION);
                    break;
                case AggregationKind.SHARED:
                    end.setAggregation(ObAggregationKindEnum.KIND_IS_AGGREGATION);
                    break;
                case AggregationKind.COMPOSITE:
                    end.setAggregation(ObAggregationKindEnum.KIND_IS_COMPOSITION);
                    break;
            }
        }
    }

    @objid ("0b1645e7-4583-11e0-b54c-00137279a832")
    private void setChangeable(IAssociationEnd objingElt) {
        objingElt.setChangeable(((Property) getEcoreElement()).isReadOnly());
    }

    @objid ("0b18a835-4583-11e0-b54c-00137279a832")
    private void setMultiplicity(IFeature feature) {
        String multMin = EcoreModelNavigation.getMultiplicityMin(((Property) getEcoreElement()));
        String multMax = EcoreModelNavigation.getMultiplicityMax(((Property) getEcoreElement()));
        
        if (feature instanceof IAttribute) {
            if (!("".equals(multMin))) {
                ((IAttribute) feature).setMultiplicityMin(multMin);
            }
            if (!("".equals(multMax))) {
                ((IAttribute) feature).setMultiplicityMax(multMax);
            }
        } else if (feature instanceof IAssociationEnd) {
            if (!("".equals(multMin))) {
                ((IAssociationEnd) feature).setMultiplicityMin(multMin);
            }
            if (!("".equals(multMax))) {
                ((IAssociationEnd) feature).setMultiplicityMax(multMax);
            }
        
            if(("".equals(multMax))&&("".equals(multMin))){
                ((IAssociationEnd) feature).setMultiplicityMin("undefined");
                ((IAssociationEnd) feature).setMultiplicityMax("undefined");
            }
        
        } else {
            throw new NotFoundException("Feature element ("
                    + feature.getClass().getSimpleName() + ") Not Found");
        }
        if (feature instanceof IAttribute) 
            testMutiplicity((IAttribute)feature);
        else if (feature instanceof IAssociationEnd) 
            testMutiplicity((IAssociationEnd)feature);
    }

    @objid ("0b18a838-4583-11e0-b54c-00137279a832")
    private void setChangeableEAnnotation(IAttribute attribute) {
        String accessMode = ObjingEAnnotation.getAccessMode(getEcoreElement());
        
        if (ObjingEAnnotation.READ_VALUE.equals(accessMode)) {
            attribute.setChangeable(ObKindOfAccessEnum.READ);
        } else if (ObjingEAnnotation.WRITE_VALUE.equals(accessMode)) {
            attribute.setChangeable(ObKindOfAccessEnum.WRITE);
        } else if (ObjingEAnnotation.READ_WRITE_VALUE.equals(accessMode)) {
            attribute.setChangeable(ObKindOfAccessEnum.READ_WRITE);
        } else if (ObjingEAnnotation.ACCESS_NONE_VALUE.equals(accessMode)) {
            attribute.setChangeable(ObKindOfAccessEnum.ACCES_NONE);
        }
    }

    @objid ("0b18a83b-4583-11e0-b54c-00137279a832")
    private void setDerivedEAnnotation(IAttribute attribute) {
        attribute.setDerived(ObjingEAnnotation
                             .isDynamicDependency(getEcoreElement()));
    }

    @objid ("0b18a83e-4583-11e0-b54c-00137279a832")
    private void setValueEAnnotation(IAttribute attribute) {
        String value = ObjingEAnnotation.getValue(getEcoreElement());
        if ((value != null) && (!value.equals("")))
            attribute.setValue(value);
    }

    @objid ("0b1b0a8e-4583-11e0-b54c-00137279a832")
    private void setOrdered(IAttribute attribute) {
        attribute.setOrdered(((Property) getEcoreElement()).isOrdered());
    }

    @objid ("0b1b0a91-4583-11e0-b54c-00137279a832")
    private void setOrdered(IAssociationEnd attribute) {
        attribute.setOrdered(((Property) getEcoreElement()).isOrdered());
    }

    @objid ("0b1b0a94-4583-11e0-b54c-00137279a832")
    private void setAbstractEAnnotation(IFeature objingElt) {
        objingElt.setAbstract(ObjingEAnnotation.isAbstract(getEcoreElement()));
    }

    @objid ("0b1b0a97-4583-11e0-b54c-00137279a832")
    private void setAccessModeEAnnotation(IAssociationEnd objingElt) {
        String access = ObjingEAnnotation.getAccessMode(getEcoreElement());
        
        if (access != null){
            if (access.equals(ObjingEAnnotation.READ_VALUE))
                objingElt.setChangeable(ObKindOfAccessEnum.READ);
            else if (access.equals(ObjingEAnnotation.WRITE_VALUE))     
                objingElt.setChangeable(ObKindOfAccessEnum.WRITE);
        
            else if (access.equals(ObjingEAnnotation.READ_WRITE_VALUE))
                objingElt.setChangeable(ObKindOfAccessEnum.READ_WRITE); 
            else if (!((Property) getEcoreElement()).isReadOnly())
                objingElt.setChangeable(ObKindOfAccessEnum.ACCES_NONE);
        }
    }

    @objid ("0b1d6ce8-4583-11e0-b54c-00137279a832")
    private void setUnique(IAttribute attribute) {
        attribute.setUnique(((Property) getEcoreElement()).isUnique());
    }

    @objid ("0b1d6ceb-4583-11e0-b54c-00137279a832")
    private void setUnique(IAssociationEnd attribute) {
        attribute.setUnique(((Property) getEcoreElement()).isUnique());
    }

    @objid ("0b1d6cee-4583-11e0-b54c-00137279a832")
    private void setTargetIsClassEAnnotation(IAttribute attribute) {
        attribute.setTargetIsClass(ObjingEAnnotation
                                   .isTargetIsClass(getEcoreElement()));
    }

    @objid ("0b1d6cf1-4583-11e0-b54c-00137279a832")
    private void setTypeConstraint(IAttribute attribute) {
        String typeConstraint = ObjingEAnnotation.getTypeConstraint(getEcoreElement());
        if (typeConstraint != null)
            attribute.setTypeConstraint(typeConstraint);
    }

    @objid ("0b1d6cf4-4583-11e0-b54c-00137279a832")
    private void setValue(IAttribute attribute) {
        String defaultValue = ((Property) getEcoreElement()).getDefault();
        if (defaultValue != null) {
            attribute.setValue(defaultValue);
        }else{ 
            ValueSpecification value = ((Property) getEcoreElement()).getDefaultValue();
            if (value instanceof Expression){
                Expression expr = (Expression) value;
                defaultValue = expr.getSymbol();
                 if (defaultValue != null) 
                        attribute.setValue(defaultValue);
            }
        }
    }

    @objid ("0b1fcf42-4583-11e0-b54c-00137279a832")
    private void setClass(IFeature feature) {
        feature.setClass(((Property) getEcoreElement()).isStatic());
    }

    @objid ("0b1fcf45-4583-11e0-b54c-00137279a832")
    private void attachAttribute(final IAttribute objingAttribute) {
        Element ecoreOwner = getEcoreElement().getOwner();
        Object objingOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        if (objingOwner instanceof IClassifier)
            objingAttribute.setOwner((IClassifier) objingOwner);
        else if ((ecoreOwner instanceof Property) && (EcoreModelNavigation.isAssocEnd((Property) ecoreOwner))) {
            for (IElement objElt : (ArrayList<IModelElement>) objingOwner){
                if (objElt instanceof IAssociationEnd)
                    objingAttribute.setQualified((IAssociationEnd) objElt);
            }
        
        }
    }

    @objid ("0b1fcf48-4583-11e0-b54c-00137279a832")
    private void attachBindableInstance(IElement objingElt) {
        if (!(objingElt instanceof IPort)){
            // The objing element is an BindableInstance
        
            Property ecoreElement = ((Property) getEcoreElement());
            IBindableInstance objingAttribute = (IBindableInstance) objingElt;
        
            Element ecoreOwner = ecoreElement.getOwner();
        
            IElement objingOwner = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
            Boolean attached = false ;
        
            if (ReverseProperties.getInstance().isRoundtripEnabled()){
        
                if (ObjingEAnnotation.isDeleted(ecoreElement)){
                    attached = true;
                    Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        
                }else {
                    String ownerID = ObjingEAnnotation.getOwner(ecoreElement);  
        
                    if ((ownerID != null) && ( !ownerID.equals(""))){
        
                        if (ecoreOwner instanceof StructuredClassifier)
                            for (Object attribute : ((StructuredClassifier) ecoreOwner).getOwnedAttributes()){
                                if (attribute instanceof Property){
                                    Property prop = (Property) attribute;
                                    List<String> ids = ObjingEAnnotation.getObjingIDs(prop);
                                    if (ids.size() > 0){
                                        String id = ids.get(0);
                                        if (id.equals(ownerID)){
                                            if ((((IBindableInstance) objingElt).getInternalOwner()== null) && (((IBindableInstance) objingElt).getOwner() == null))
                                                objingAttribute.setCluster(((IBindableInstance) ReverseProperties.getInstance().getMappedElement(prop)));
                                            attached = true;
                                        }
                                    }
                                }
                            }
                    }
                }
            }
        
        
            if (!attached){ 
                if (objingOwner instanceof IClassifier){
                    if (objingAttribute.getCluster() == null)
                        objingAttribute.setInternalOwner((IClassifier) objingOwner);
        
                    IModelElement represented = objingAttribute.getRepresentedFeature();
                    if ((represented != null) && (represented instanceof IAssociationEnd)){
                        ObjingModelNavigation.attachAssociationEnd((IAssociationEnd)represented, (Property) getEcoreElement());
                    }
                }else if ((objingOwner instanceof ICollaboration))
                    objingAttribute.setOwner((ICollaboration) objingOwner);
                else if ((objingOwner instanceof IInteraction)){
                    ObList<?> collabList =  ((IInteraction) objingOwner).getOwnedCollaboration();
                    ICollaboration collab = null;
                    if (!collabList.isEmpty()){
                        collab = ((ICollaboration)collabList.get(0));
                    }else{
                        collab = Modelio.getInstance().getModelingSession().getModel().createCollaboration();
                        collab.setName("locals");
                        ((IInteraction) objingOwner).addOwnedCollaboration(collab);
                    }
        
                    if (collab.getCompositionOwner() instanceof IBehavior)
                        collab.addDeclared(objingAttribute);
                    else
                        collab.addRepresentingInstance(objingAttribute);
        
                }
            }
        }
    }

    @objid ("0b22319c-4583-11e0-b54c-00137279a832")
    private void setAssociation(IAssociationEnd end) {
        // Links to the Association:
        Association ecoreAssoc =  ((Property) getEcoreElement()).getAssociation();
        
        Object objingAssoc = ReverseProperties.getInstance().getMappedElement(ecoreAssoc);
        
        if ((objingAssoc instanceof IAssociation) && (end.getOwner() != null)) {
            objingAssoc = (IAssociation) objingAssoc;
            end.setRelated((IAssociation) objingAssoc);
        }
    }

    @objid ("0b22319f-4583-11e0-b54c-00137279a832")
    public EProperty() {
    }

    @objid ("0b2231a1-4583-11e0-b54c-00137279a832")
    private void testMutiplicity(IAttribute attribute) {
        String multMin = attribute.getMultiplicityMin();
        String multMax = attribute.getMultiplicityMax();
        
        if (multMin.equals("*")){
            if (multMax.equals("*")){
                attribute.setMultiplicityMin("undefined");
            }else{
                attribute.setMultiplicityMin(multMax);
                attribute.setMultiplicityMax("*");
            }
        }
    }

    @objid ("0b2231a4-4583-11e0-b54c-00137279a832")
    private void testMutiplicity(IAssociationEnd assocEnd) {
        String multMin = assocEnd.getMultiplicityMin();
        String multMax = assocEnd.getMultiplicityMax();
        
        if (multMin.equals("*")){
            if (multMax.equals("*")){
                assocEnd.setMultiplicityMin("undefined");
            }else{
                assocEnd.setMultiplicityMin(multMax);
                assocEnd.setMultiplicityMax("*");
            }
        }
    }

    @objid ("0b2493f6-4583-11e0-b54c-00137279a832")
    private void setAccessMode(IAttribute objingElt) {
        if (((Property) getEcoreElement()).isReadOnly()){
            objingElt.setChangeable(ObKindOfAccessEnum.READ);
        }
    }

    @objid ("0b2493f9-4583-11e0-b54c-00137279a832")
    private void setAccessMode(IAssociationEnd objingElt) {
        if (((Property) getEcoreElement()).isReadOnly()){
            objingElt.setChangeable(ObKindOfAccessEnum.READ);
        }
    }

}
