/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IAbstractPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEntryPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IExitPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateVertex;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;

@objid ("0b37a6c8-4583-11e0-b54c-00137279a832")
public class EPseudostate extends ENamedElement implements IEElement {
    @objid ("0b37a6c9-4583-11e0-b54c-00137279a832")
    private Pseudostate ecoreElement;


    @objid ("0b37a6ca-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IAbstractPseudoState state = null;
        
        switch (ecoreElement.getKind().getValue()) {
        case PseudostateKind.INITIAL:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createInitialPseudoState();
            
            break;
        case PseudostateKind.DEEP_HISTORY:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createDeepHistoryPseudoState();
            
            break;
        case PseudostateKind.SHALLOW_HISTORY:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createShallowHistoryPseudoState();
            
            break;
        case PseudostateKind.JOIN:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createJoinPseudoState();
            
            break;
        case PseudostateKind.FORK:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createForkPseudoState();
            
            break;
        case PseudostateKind.JUNCTION:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createJunctionPseudoState();
            
            break;
        case PseudostateKind.CHOICE:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createChoicePseudoState();
            
            break;
        case PseudostateKind.ENTRY_POINT:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createEntryPointPseudoState();
            
            break;
        case PseudostateKind.EXIT_POINT:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createExitPointPseudoState();
            
            break;
        case PseudostateKind.TERMINATE:
            state = Modelio.getInstance().getModelingSession().getModel()
            .createTerminatePseudoState();
            
            break;
        }
        return state;
    }

    @objid ("0b37a6ce-4583-11e0-b54c-00137279a832")
    public EPseudostate(Pseudostate element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0b3a0928-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Region ecoreContainer = ecoreElement.getContainer();
                
        if (ecoreContainer != null) {
            Object objingContainer = ReverseProperties.getInstance().getMappedElement(ecoreContainer);
                
            if ((objingContainer instanceof IRegion) && (objingElt instanceof IStateVertex))
                    ((IRegion) objingContainer).addSub((IStateVertex) objingElt);
            
        }else{
            StateMachine ecoreStateMachine = ecoreElement.getStateMachine();
            if (ecoreStateMachine != null){
                Object objingStateMachine = ReverseProperties.getInstance().getMappedElement(ecoreStateMachine);
                
                if ((objingStateMachine instanceof IStateMachine) && (objingElt instanceof IStateVertex)) {
                
                        ((IStateMachine) objingStateMachine).getTop().addSub((IStateVertex) objingElt);
                    
                }
            }else{
                Element ecoreOwner = ecoreElement.getOwner();
                if (ecoreOwner instanceof State){
                    IState objOwner = (IState) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
                    
                    if (objingElt instanceof IEntryPointPseudoState)
                        objOwner.addEntryPoint((IEntryPointPseudoState)objingElt);
                    else if (objingElt instanceof IExitPointPseudoState)
                        objOwner.addExitPoint((IExitPointPseudoState)objingElt);
                    
                }
            }
            
        }
    }

    @objid ("0b3a092b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0b3c6b7a-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        //        setPseudoParent(objingElt);
        //        setKind((IPseudoState)objingElt);
    }

    @objid ("0b3c6b86-4583-11e0-b54c-00137279a832")
    public EPseudostate() {
    }

}
