/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOpaqueAction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ReadStructuralFeatureAction;
import org.eclipse.uml2.uml.StructuralFeature;

@objid ("0b7f2d52-4583-11e0-b54c-00137279a832")
public class EReadStructuralFeatureAction extends EActivityNode implements IEElement {
    @objid ("0b7f2d53-4583-11e0-b54c-00137279a832")
    private ReadStructuralFeatureAction ecoreElement;


    @objid ("0b7f2d54-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        IOpaqueAction element = Modelio.getInstance().getModelingSession().getModel()
        .createOpaqueAction();
                
        try{
            IStereotype stereo = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IOpaqueAction.class, XMIStereotypes.UML2READSTRUCTURALFEATUREACTION);
            element.addExtension(stereo);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
        return element;
    }

    @objid ("0b7f2d58-4583-11e0-b54c-00137279a832")
    public EReadStructuralFeatureAction(ReadStructuralFeatureAction element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0b818fac-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("0b818faf-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0b818fb4-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setFeature((IOpaqueAction) objingElt);
    }

    @objid ("0b83f20f-4583-11e0-b54c-00137279a832")
    public EReadStructuralFeatureAction() {
    }

    @objid ("0b83f211-4583-11e0-b54c-00137279a832")
    private void setFeature(IOpaqueAction objingElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        StructuralFeature feature = ecoreElement.getStructuralFeature();
        if (feature != null) {
            Object behavior = ReverseProperties.getInstance().getMappedElement(feature);
            
            IModelElement obBehavior = null;
            
            if (behavior instanceof IModelElement){
                obBehavior = (IModelElement) behavior;
            }else if (behavior instanceof List){
                obBehavior = ((List<IModelElement>) behavior).get(0);
            }
            
            IDependency dependency = session.getModel().createDependency();
            try{
                dependency.addExtension(session.getMetamodelExtensions()
                        .getStereotype(IDependency.class, XMIStereotypes.UML2STRUCTURALFEATUREREFERENCE));
            }catch(StereotypeNotFoundException e){
                e.printStackTrace(Modelio.err);
            }
                
            dependency.setDependsOn(obBehavior);
            dependency.setImpacted(objingElt);
        }
    }

}
