/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IRaisedException;
import com.modeliosoft.modelio.api.model.uml.statik.ObMethodPassingModeEnum;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.CallConcurrencyKind;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Reception;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.Type;

@objid ("0b9e2be9-4583-11e0-b54c-00137279a832")
public class EReception extends EFeature implements IEElement {

    @objid ("0b9e2bea-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Element ecoreOwner = getEcoreElement().getOwner();
        Object objingOwner =  ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        if ((objingOwner instanceof IClassifier)  && !(objingOwner instanceof IEnumeration)){
            IOperation operation = Modelio.getInstance().getModelingSession().getModel().createOperation();
            try{
                IStereotype ster = Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotype(IOperation.class, XMIStereotypes.UML2RECEPTION);
                operation.addExtension(ster);
            }catch(StereotypeNotFoundException e){
                e.printStackTrace(Modelio.err);
            }
            return operation;
        }
        return null;
    }

    @objid ("0ba08e44-4583-11e0-b54c-00137279a832")
    public EReception(Reception element) {
        super(element);
    }

    @objid ("0ba08e47-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
        result.add("");
        return result;
    }

    @objid ("0ba2f098-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (objingElt != null) {
            ReverseProperties revProp = ReverseProperties.getInstance();
            Reception ecoreElement = (Reception) getEcoreElement();
            Element ecoreOwner = ecoreElement.getOwner();
            IElement objingOwner = (IElement) revProp
            .getMappedElement(ecoreOwner);
        
            if ((objingOwner instanceof IClassifier)  && !(objingOwner instanceof IEnumeration))
                ((IOperation) objingElt).setOwner((IClassifier) objingOwner);
        
            else {
                PartialImportMap.getInstance().remove(ecoreElement);
                TotalImportMap.getInstance().remove(ecoreElement);
                Modelio.getInstance().getModelingSession().getModel()
                .deleteElement(objingElt);
            }
        }
    }

    @objid ("0ba2f09b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0ba2f0a0-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt != null) {
            super.setProperties(objingElt);
            //                    setCreateEAnnotation((IOperation) objingElt);
            //                    setDestroyEAnnotation((IOperation) objingElt);
            setVisibility((IOperation) objingElt);
            setAbstract((IOperation) objingElt);
            setFinal((IOperation) objingElt);
            setIsClass((IOperation) objingElt);
            setPassingMode((IOperation) objingElt);
            setConcurrency((IOperation) objingElt);
            setRaisedException((IOperation) objingElt);
            setSignal((IOperation) objingElt);
        }
        
        setMethod((IOperation) objingElt);
    }

    @objid ("0ba2f0a3-4583-11e0-b54c-00137279a832")
    public EReception() {
    }

    @objid ("0ba552f2-4583-11e0-b54c-00137279a832")
    private void setSignal(IOperation objingElt) {
        Signal ecoreSignal = ((Reception) getEcoreElement()).getSignal();
        if (ecoreSignal != null){
            IElement objSignal = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreSignal);
            if ((objSignal != null) && (objSignal instanceof ISignal)){
                objingElt.addSRepresentation((ISignal)objSignal);
            }
        }
    }

    @objid ("0ba552f5-4583-11e0-b54c-00137279a832")
    private boolean isExceptionAlreadyThrown(IOperation objingElt, IElement objingThrownTypeFromMap) {
        for (IRaisedException objingExcep : objingElt.getThrown()) {
            IClassifier objingThrownType = objingExcep.getThrownType();
            if (objingThrownTypeFromMap.equals(objingThrownType))
                return true;
        }
        return false;
    }

    @objid ("0ba552fe-4583-11e0-b54c-00137279a832")
    private void setRaisedException(IOperation objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        // Test if the number of raised exceptions in the Ecore model is the
        // same in the Ijing model:
        EList<?> ecoreRaisedExceptions = ((Reception)getEcoreElement()).getRaisedExceptions();
        ObList<IRaisedException> objingRaisedExceptions = objingElt
        .getThrown();
        int objingExcepNumber = objingRaisedExceptions.size();
        
        // Warning : no update in case of same number of thrown exceptions.
        if (ecoreRaisedExceptions.size() != objingExcepNumber) {
            boolean isNewExeptionLink;
            for (Object raisedException : ecoreRaisedExceptions) {
                isNewExeptionLink = false;
                Type ecoreThrownType = (Type) raisedException;
                IElement objingThrownType = (IElement) revProp
                .getMappedElement(ecoreThrownType);
        
                if (objingThrownType instanceof IClassifier) {
                    if (isNewExeptionLink
                            || !isExceptionAlreadyThrown(objingElt,
                                    objingThrownType)) {
                        IRaisedException objingExceptionLink = Modelio
                        .getInstance().getModelingSession().getModel()
                        .createRaisedException();
        
                        objingExceptionLink.setThrower(objingElt);
                        objingExceptionLink
                        .setThrownType((IClassifier) objingThrownType);
                    }
                }
            }
        }
    }

    @objid ("0ba55301-4583-11e0-b54c-00137279a832")
    private void setAbstract(IOperation objingElt) {
        objingElt.setAbstract(((Reception)getEcoreElement()).isAbstract());
    }

    @objid ("0ba7b54c-4583-11e0-b54c-00137279a832")
    private void setFinal(IOperation objingElt) {
        objingElt.setFinal(((Reception)getEcoreElement()).isLeaf());
    }

    @objid ("0ba7b54f-4583-11e0-b54c-00137279a832")
    private void setIsClass(IOperation objingElt) {
        objingElt.setClass(((Reception)getEcoreElement()).isStatic());
    }

    @objid ("0ba7b552-4583-11e0-b54c-00137279a832")
    private void setPassingMode(IOperation objingElt) {
        Element ecoreElt = getEcoreElement();
        if (ecoreElt instanceof Operation){
            if (((Operation)ecoreElt).isQuery())
                objingElt.setPassing(ObMethodPassingModeEnum.METHOD_IN);
            else
                objingElt.setPassing(ObMethodPassingModeEnum.METHOD_OUT);
        }else
            objingElt.setPassing(ObMethodPassingModeEnum.METHOD_OUT);
    }

    @objid ("0ba7b555-4583-11e0-b54c-00137279a832")
    private void setConcurrency(IOperation objingElt) {
        // Since concurrency is represented as a boolean in Ijing, the mapping
        // is true for the
        // CONCURRENT value, and false else (SEQUENTIAL by default):
        Reception ecoreElement = ((Reception)getEcoreElement());
        switch (ecoreElement.getConcurrency().getValue()) {
        case CallConcurrencyKind.CONCURRENT:
            objingElt.setConcurrency(true);
        default:
            objingElt.setConcurrency(false);
        }
    }

    @objid ("0ba7b558-4583-11e0-b54c-00137279a832")
    private void setMethod(IOperation objingElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        Reception ecoreElement = ((Reception)getEcoreElement());
        for (Behavior behavior : ecoreElement.getMethods()){
            IDependency dependency = session.getModel().createDependency();
            try{
                dependency.addExtension(session.getMetamodelExtensions()
                        .getStereotype(IDependency.class, XMIStereotypes.UML2METHODREFERENCE));
            }catch(StereotypeNotFoundException e){
        
            }
        
            IModelElement obBehavior = (IModelElement)ReverseProperties.getInstance().getMappedElement(behavior);
            dependency.setDependsOn(obBehavior);
            dependency.setImpacted(objingElt);
        }
    }

}
