/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.Type;

@objid ("0bfb279c-4583-11e0-b54c-00137279a832")
public class ESignal extends ENamedElement implements IEElement {
    @objid ("0bfb279d-4583-11e0-b54c-00137279a832")
    private Signal ecoreElement;


    @objid ("0bfb279e-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createSignal();
    }

    @objid ("0bfd89f5-4583-11e0-b54c-00137279a832")
    public ESignal(Signal element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0bfd89fe-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Element ecoreOwner = ecoreElement.getOwner();
        
        Object objingOwner = revProp.getMappedElement(ecoreOwner);
        
        if ((objingOwner != null) 
                && (objingOwner instanceof IModelTree) 
                && !((objingOwner instanceof IProfile))) {
            ((ISignal) objingElt).setOwner((IModelTree)objingOwner);
        
        }else if (objingOwner instanceof IModelTree){
            ((ISignal) objingElt).setOwner(revProp.getExternalPackage());
        }
    }

    @objid ("0bffec4e-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0bffec53-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (ReverseProperties.getInstance().isRoundtripEnabled()) {
            super.setProperties(objingElt);
            setBaseEAnnotation((ISignal) objingElt);
            setOBaseEAnnotation((ISignal) objingElt);
            setPBaseEAnnotation((ISignal) objingElt);
            setIsEventEAnnotation((ISignal) objingElt);
            setIsExceptionEAnnotation((ISignal) objingElt);
        
        }
    }

    @objid ("0bffec59-4583-11e0-b54c-00137279a832")
    private void setBaseEAnnotation(ISignal objingElt) {
        for (Object attribut : ecoreElement.getOwnedAttributes()){
            if (attribut instanceof Property){
                Property property = (Property) attribut;
                if ((property.getName().equals(ObjingEAnnotation.IS_BASE))&&(ObjingEAnnotation.isBase(property))){
                    //                    objingElt.setBase((IGeneralClass) ReverseProperties.getInstance().getMappedElement(property.getType()));
        
                    ReverseProperties revProp = ReverseProperties.getInstance();
        
                    Type ecoreType = property.getType();
        
                    Modelio.getInstance().getModelingSession()
                    .getModel().getUmlTypes();
        
                    if (ecoreType != null) {
        
                        // If the Attribute is typed by another element, we get this
                        // Ijing
                        // Classifier:
                        IElement objingType = (IElement) revProp
                        .getMappedElement(ecoreType);
                        objingElt.setBase((IGeneralClass) objingType);
        
                    }
                }
            }
        }
    }

    @objid ("0bffec5c-4583-11e0-b54c-00137279a832")
    private void setOBaseEAnnotation(ISignal objingElt) {
        for (Object attribut : ecoreElement.getOwnedAttributes()){
            if (attribut instanceof Property){
                Property property = (Property) attribut;
                if ((property.getName().equals(ObjingEAnnotation.IS_OBASE))&&(ObjingEAnnotation.isOBase(property))){
                    objingElt.setOBase((IOperation) ReverseProperties.getInstance().getMappedElement(property.getType()));
        
                }
            }
        }
    }

    @objid ("0bffec5f-4583-11e0-b54c-00137279a832")
    private void setPBaseEAnnotation(ISignal objingElt) {
        for (Object attribut : ecoreElement.getOwnedAttributes()){
            if (attribut instanceof Property){
                Property property = (Property) attribut;
                if ((property.getName().equals(ObjingEAnnotation.IS_PBASE))&&(ObjingEAnnotation.isPBase(property))){
                    objingElt.setPBase((IParameter) ReverseProperties.getInstance().getMappedElement(property.getType()));
        
        
                }
            }
        }
    }

    @objid ("0c024ea8-4583-11e0-b54c-00137279a832")
    private void setIsEventEAnnotation(ISignal objingElt) {
        objingElt.setEvent(ObjingEAnnotation.isEvent((ecoreElement)));
    }

    @objid ("0c024eab-4583-11e0-b54c-00137279a832")
    private void setIsExceptionEAnnotation(ISignal objingElt) {
        objingElt.setException(ObjingEAnnotation.isException((ecoreElement)));
    }

    @objid ("0c024eae-4583-11e0-b54c-00137279a832")
    public ESignal() {
    }

}
