/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ObEventTypeEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEvent;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.ObElementNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.Transition;

@objid ("0c04b102-4583-11e0-b54c-00137279a832")
public class ESignalEvent extends ENamedElement implements IEElement {
    @objid ("0c04b103-4583-11e0-b54c-00137279a832")
    private SignalEvent ecoreElement;


    @objid ("0c04b104-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Element ecoreOwner = ecoreElement.getOwner();
        if (ecoreOwner instanceof Transition){
            IEvent result = Modelio.getInstance().getModelingSession().getModel().createEvent();
            result.setKind(ObEventTypeEnum.SIGNAL_EVENT);
            return result;
        }
        return null;
    }

    @objid ("0c04b108-4583-11e0-b54c-00137279a832")
    public ESignalEvent(SignalEvent element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0c07135c-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        try{
            if (((IEvent) objingElt).getComposed() == null){
                Element ecoreOwner = ecoreElement.getOwner();
                if (ecoreOwner instanceof Transition){
                    IStateMachine sm = (IStateMachine) ReverseProperties.getInstance().getMappedElement(((Transition) ecoreOwner).getContainer().getStateMachine());
                    sm.addEComponent((IEvent)objingElt);
                }else{
                    Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
                }
            }
        }catch(ObElementNotFoundException e){
            //Signal Event was an extra element. It is not imported in Modelio
        }
    }

    @objid ("0c07135f-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0c071364-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setSignal((IEvent)objingElt);
    }

    @objid ("0c071367-4583-11e0-b54c-00137279a832")
    public ESignalEvent() {
    }

    @objid ("0c0975b9-4583-11e0-b54c-00137279a832")
    private void setSignal(IEvent objingElt) {
        Signal signal =    ecoreElement.getSignal();
        if (signal != null){
            IElement objSignal = (IElement) ReverseProperties.getInstance().getMappedElement(signal);
            if (objSignal instanceof ISignal){
                objingElt.setModel((ISignal) objSignal);
            }
        }
    }

}
