/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAttribute;
import com.modeliosoft.modelio.api.model.uml.statik.IAttributeLink;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.ILinkEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0c0975bc-4583-11e0-b54c-00137279a832")
public class ESlot extends EElement implements IEElement {
    @objid ("0c0975bd-4583-11e0-b54c-00137279a832")
    private Slot ecoreElement;


    @objid ("0c0975be-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        InstanceSpecification ecoreOwner = ecoreElement.getOwningInstance();
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            if (ObjingEAnnotation.isPort(ecoreElement))
                return Modelio.getInstance().getModelingSession().getModel().createPort();
            else if (ObjingEAnnotation.isBindableInstance(ecoreElement))
                return Modelio.getInstance().getModelingSession().getModel().createBindableInstance();
            else if (ObjingEAnnotation.isConnector(ecoreOwner))
                return Modelio.getInstance().getModelingSession().getModel().createConnectorEnd();
            else if (ObjingEAnnotation.isLink(ecoreOwner))
                return Modelio.getInstance().getModelingSession().getModel().createLinkEnd();
            else if (ObjingEAnnotation.isAttributeLink(ecoreElement))
                return Modelio.getInstance().getModelingSession().getModel().createAttributeLink();
        }
        
        Element feature = ecoreElement.getDefiningFeature();
        
        if (feature != null){
            Object objFeature = ReverseProperties.getInstance().getMappedElement(feature);
            if  (feature instanceof Port){
                return Modelio.getInstance().getModelingSession().getModel().createPort();
            }else if (objFeature  instanceof IAttribute){
                return Modelio.getInstance().getModelingSession().getModel().createAttributeLink();
            }else if (EcoreModelNavigation.isConnector(ecoreOwner) ){
                return Modelio.getInstance().getModelingSession().getModel().createConnectorEnd();
            }else if (EcoreModelNavigation.isLink(ecoreOwner) ){
                return Modelio.getInstance().getModelingSession().getModel().createLinkEnd();
            }else if ((feature instanceof Property ) && ((Property)feature).isComposite()){
                return Modelio.getInstance().getModelingSession().getModel().createBindableInstance();
            }
        }
        return Modelio.getInstance().getModelingSession().getModel().createBindableInstance();
    }

    @objid ("0c109cc4-4583-11e0-b54c-00137279a832")
    private String getValueSpecification() {
        Type type = PrimitiveTypeMapper.getString();
        ValueSpecification result = ecoreElement.getValue("", type);
        if (result != null){
            return result.stringValue();
        }
        
        type = PrimitiveTypeMapper.getBoolean();
        result = ecoreElement.getValue("", type);
        if (result != null){
            return result.stringValue();
        }
        
        type = PrimitiveTypeMapper.getUnlimited();
        result = ecoreElement.getValue("", type);
        if (result != null){
            return result.stringValue();
        }
        
        type = PrimitiveTypeMapper.getInteger();
        result = ecoreElement.getValue("", type);
        if (result != null){
            return result.stringValue();
        }
        
        EList<ValueSpecification> values = ecoreElement.getValues();
        String stringResult = "";
        for (ValueSpecification value : values){
            stringResult += value.stringValue();
        }
        return stringResult;
    }

    @objid ("0c109cc8-4583-11e0-b54c-00137279a832")
    public ESlot(Slot element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0c109ccb-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        return new ArrayList<String>();
    }

    @objid ("0c109cd1-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (objingElt instanceof IConnectorEnd){
            attachConnectorEnd((IConnectorEnd)objingElt);
        }else if (objingElt instanceof ILinkEnd){
            attachLinkEnd((ILinkEnd)objingElt);
        }else if (objingElt instanceof IAttributeLink){
            attachAttributeLink((IAttributeLink)objingElt);
        }else{
            attachBindableInstance((IBindableInstance)objingElt);
        }
    }

    @objid ("0c12ff1f-4583-11e0-b54c-00137279a832")
    private void attachAttributeLink(IAttributeLink objingElt) {
        InstanceSpecification ecoreOwner = (InstanceSpecification) ecoreElement.getOwner();
        IInstance objRoot = (IInstance) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        
        if (ReverseProperties.getInstance().isRoundtripEnabled() 
                && (ObjingEAnnotation.getOwner(ecoreElement) != null)&& (!ObjingEAnnotation.getOwner(ecoreElement).equals(""))){
        
            String ownerId = ObjingEAnnotation.getOwner(ecoreElement);
            for (Object object : ecoreOwner.getSlots()){
                Slot slot = (Slot) object;
                List<String> ids = ObjingEAnnotation.getObjingIDs(slot);
                if (ids.size() > 0){
                    String id =  ids.get(0);
                    if (id.equals(ownerId)){
                        Object objOwner = ReverseProperties.getInstance().getMappedElement(slot);
                        if (objOwner instanceof IBindableInstance){
                            objingElt.setAttributed((IInstance)objOwner);
                            return;
                        }
        
                    }
                }
            }
        
            List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreOwner);
            if (ids.size() > 0){
                String id =  ids.get(0);
                if (id.equals(ownerId)){
                    Object objOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
                    if (objOwner instanceof IInstance){
                        objingElt.setAttributed((IInstance)objOwner);    
                        return;
                    }
                }
            }
        }
        
        objingElt.setAttributed(objRoot);
    }

    @objid ("0c12ff22-4583-11e0-b54c-00137279a832")
    private void attachConnectorEnd(IConnectorEnd objingElt) {
        ValueSpecification inst = ecoreElement.getValue("", null);
        if (inst != null){
            InstanceSpecification ecoreOwner = ((InstanceValue)inst).getInstance();
            if ((ecoreOwner != null) && (ReverseProperties.getInstance().isRoundtripEnabled()) && (ObjingEAnnotation.getOwner(ecoreElement) != null)){
                String ownerId = ObjingEAnnotation.getOwner(ecoreElement);
                for (Object object : ecoreOwner.getSlots()){
                    Slot slot = (Slot) object;
                    List<String> ids  = ObjingEAnnotation.getObjingIDs(slot);
                    if (ids.size() > 0){
                        String id =  ids.get(0);
                        if (id.equals(ownerId)){
                            Object objOwner = ReverseProperties.getInstance().getMappedElement(slot);
                            if (objOwner instanceof IBindableInstance){
                                objingElt.setLinked((IBindableInstance) objOwner);
                                break;
                            }
                        }
                    }
                }
        
            }
        
        }
    }

    @objid ("0c12ff25-4583-11e0-b54c-00137279a832")
    private void attachLinkEnd(ILinkEnd objingElt) {
        InstanceSpecification ecoreOwner = (InstanceSpecification) ecoreElement.getOwner();
        
        Object obj =  ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        if (obj instanceof ILink){
        
            ILink link = (ILink) obj;
        
            objingElt.setLinkNode(link);
        
            for ( ValueSpecification inst : ecoreElement.getValues()) {
                if (inst instanceof InstanceValue){
                    InstanceSpecification instSpe = ((InstanceValue)inst).getInstance();
                    if (instSpe != null){
                        IElement objOwner = (IElement) ReverseProperties.getInstance().getMappedElement(instSpe);
                        if (objOwner instanceof IInstance)
                            objingElt.setLinked((IInstance)objOwner);
                    }
                }
            }
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
        }
    }

    @objid ("0c12ff28-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0c12ff2d-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof IAttributeLink){
            setAttributeProperties((IAttributeLink)objingElt);
        }else if (objingElt instanceof IPort){
            setPortProperties((IPort) objingElt);
            setBIProperties((IPort) objingElt);
            //        }else if ((ReverseProperties.getInstance().isRoundtripEnabled())
            //                &&(ObjingEAnnotation.isLink(ecoreElement.getOwner()))){
        }else if (objingElt instanceof ILinkEnd){
            setLinkEndProperties((ILinkEnd) objingElt);
        }else{
            setBIProperties((IBindableInstance) objingElt);
        }
        

    	super.setProperties(objingElt);
    }

    @objid ("0c156179-4583-11e0-b54c-00137279a832")
    private void setLinkEndProperties(ILinkEnd objingElt) {
        objingElt.setUnique(ObjingEAnnotation.isUnique(ecoreElement));
        objingElt.setOrdered(ObjingEAnnotation.isOrdered(ecoreElement));
        objingElt.setNavigable(ObjingEAnnotation.isNavigable(ecoreElement));
        setMultiMax(objingElt);
        setMultiMin(objingElt);
        setName(objingElt);
        setLink(objingElt);
    }

    @objid ("0c15617c-4583-11e0-b54c-00137279a832")
    private void setLink(ILinkEnd objingElt) {
        Object link = ReverseProperties.getInstance().getMappedElement(ecoreElement.getOwningInstance());
        if (link instanceof ILink)
            objingElt.setLinkNode((ILink) link);
    }

    @objid ("0c15617f-4583-11e0-b54c-00137279a832")
    private void setMultiMax(ILinkEnd objingElt) {
        String temp = ObjingEAnnotation.getMultiMax(ecoreElement);
        if (temp != null)
            objingElt.setMultiplicityMax(temp);
    }

    @objid ("0c156182-4583-11e0-b54c-00137279a832")
    private void setMultiMin(ILinkEnd objingElt) {
        String temp = ObjingEAnnotation.getMultiMin(ecoreElement);
        if (temp != null)
            objingElt.setMultiplicityMin(temp);
    }

    @objid ("0c156185-4583-11e0-b54c-00137279a832")
    private void setName(ILinkEnd objingElt) {
        String temp = ObjingEAnnotation.getName(ecoreElement);
        if (temp != null)
            objingElt.setName(temp);
    }

    @objid ("0c17c3d2-4583-11e0-b54c-00137279a832")
    private void setAttributeProperties(IAttributeLink objingElt) {
        setAttributed((IAttributeLink)objingElt);
        
        ((IAttributeLink)objingElt).setName("AttributeLink");
        
        String value = getValueSpecification();
        if (value != null)
            objingElt.setValue(value);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setEAnnotationValue((IAttributeLink)objingElt);
            setEAnnotationName((IAttributeLink)objingElt);
        }
    }

    @objid ("0c17c3d5-4583-11e0-b54c-00137279a832")
    private void setBIProperties(IBindableInstance objingElt) {
        setName(objingElt);
        setValue(objingElt);
        setMultiplicityMax(objingElt);
        setMultiplicityMin(objingElt);
        
        objingElt.setConstant(ObjingEAnnotation.isConstant(ecoreElement));
    }

    @objid ("0c17c3d8-4583-11e0-b54c-00137279a832")
    private void setName(IBindableInstance objingElt) {
        String temp = ObjingEAnnotation.getName(ecoreElement);
        if (temp != null)
            objingElt.setName(temp);
    }

    @objid ("0c17c3db-4583-11e0-b54c-00137279a832")
    private void setValue(IBindableInstance objingElt) {
        String temp = getValueSpecification();
        if (temp != null)
            objingElt.setValue(temp);
    }

    @objid ("0c17c3de-4583-11e0-b54c-00137279a832")
    private void setMultiplicityMax(IBindableInstance objingElt) {
        String temp = ObjingEAnnotation.getMultiMax(ecoreElement);
        if (temp != null)
            objingElt.setMultiplicityMax(temp);
    }

    @objid ("0c1a262c-4583-11e0-b54c-00137279a832")
    private void setMultiplicityMin(IBindableInstance objingElt) {
        String temp = ObjingEAnnotation.getMultiMin(ecoreElement);
        if (temp != null)
            objingElt.setMultiplicityMin(temp);
    }

    @objid ("0c1a262f-4583-11e0-b54c-00137279a832")
    private void setPortProperties(IPort objPort) {
        objPort.setBehavior(ObjingEAnnotation.isBehavior(ecoreElement));
        objPort.setService(ObjingEAnnotation.isService(ecoreElement));
    }

    @objid ("0c1a2632-4583-11e0-b54c-00137279a832")
    private void setAttributed(IAttributeLink objingElt) {
        StructuralFeature ecoreFeature = ecoreElement.getDefiningFeature();
        
        if (ecoreFeature != null ) {
        
            Object objingFeature =  ReverseProperties.getInstance().getMappedElement(ecoreFeature);
        
            if (objingFeature instanceof IAttribute) {
                objingElt.setBase((IAttribute) objingFeature);
            }else if (objingFeature instanceof List<?>){               
                for (Object elt : ((List<IElement>) objingFeature)){
                    if (elt instanceof IAttribute) {
                        objingElt.setBase((IAttribute) elt);
                        break;
                    }
                }
            }
        }
    }

    @objid ("0c1a2635-4583-11e0-b54c-00137279a832")
    private void setEAnnotationValue(IAttributeLink objingElt) {
        String value = ObjingEAnnotation.getValue(ecoreElement);
        
        if (value != null)
            objingElt.setValue(value);
    }

    @objid ("0c1a2638-4583-11e0-b54c-00137279a832")
    private void setEAnnotationName(IAttributeLink objingElt) {
        String value = ObjingEAnnotation.getName(ecoreElement);
        
        if (value != null)
            objingElt.setName(value);
    }

    @objid ("0c1c8888-4583-11e0-b54c-00137279a832")
    public ESlot() {
    }

    @objid ("4c48bb1b-ab2f-11e1-83be-0027103f347d")
    private void attachBindableInstance(final IBindableInstance objingElt) {
        InstanceSpecification ecoreOwner = (InstanceSpecification) ecoreElement.getOwner();
        IInstance objRoot = (IInstance) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled() 
                && (ObjingEAnnotation.getOwner(ecoreElement) != null)
                && (!ObjingEAnnotation.getOwner(ecoreElement).equals(""))){
        
            String ownerId = ObjingEAnnotation.getOwner(ecoreElement);
            for (Object object : ecoreOwner.getSlots()){
                Slot slot = (Slot) object;
                List<String> ids = ObjingEAnnotation.getObjingIDs(slot);
                if (ids.size() > 0){
                    String id = ids.get(0);
                    if (id.equals(ownerId)){
                        Object objOwner = ReverseProperties.getInstance().getMappedElement(slot);
                        if (objOwner instanceof IInstance){
                            objingElt.setCluster((IInstance)objOwner);
                            return;
                        }
                    }
                }
            }
            List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreOwner);
            if (ids.size() >0){
                String id =  ids.get(0);
        
                if (id.equals(ownerId)){
                    Object objOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
                    if (objOwner instanceof IInstance){
                        objingElt.setCluster((IInstance)objOwner);
                        return;
                    }
                }
            }
        
        }
        
        objingElt.setCluster(objRoot);
        return;
    }

}
