/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;

@objid ("0c2611ee-4583-11e0-b54c-00137279a832")
public class EState extends ENamedElement implements IEElement {
    @objid ("0c2611ef-4583-11e0-b54c-00137279a832")
    private State ecoreElement;


    @objid ("0c2611f0-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createState();
    }

    @objid ("0c2611f4-4583-11e0-b54c-00137279a832")
    public EState(State element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0c287448-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        Region ecoreContainer = ecoreElement.getContainer();
        
        if (ecoreContainer != null) {
            Object objingContainer = ReverseProperties.getInstance().getMappedElement(ecoreContainer);
        
            if ((objingContainer instanceof IRegion) && (objingElt instanceof IState)){
                ((IRegion) objingContainer).addSub((IState) objingElt);
            }else{
                Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
                XMILogs.getInstance().writelnInLog("owner of state was " + ecoreContainer.getClass().getSimpleName());             
            }
        
        }else{
            XMILogs.getInstance().writelnInLog("the state named " + ecoreElement.getName() + " has not owner");
        }
    }

    @objid ("0c28744b-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0c287450-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (objingElt instanceof IState) {            
            setSubMachine((IState) objingElt);
            setPseudoParent((IState) objingElt);
        }
    }

    @objid ("0c287456-4583-11e0-b54c-00137279a832")
    private void setSubMachine(IState state) {
        StateMachine ecoreSubMachine = ecoreElement.getSubmachine();
        if (ecoreSubMachine != null) {
            Object objingSubMachine = ReverseProperties.getInstance().getMappedElement(ecoreSubMachine);
            if (objingSubMachine instanceof IStateMachine) {
                state.setSubMachine((IStateMachine) objingSubMachine);
            }
        }
    }

    @objid ("0c2ad6a2-4583-11e0-b54c-00137279a832")
    private void setPseudoParent(IState state) {
    }

    @objid ("0c2ad6a8-4583-11e0-b54c-00137279a832")
    public EState() {
    }

}
