/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ObEventTypeEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEvent;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.TimeEvent;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0c87d258-4583-11e0-b54c-00137279a832")
public class ETimeEvent extends ENamedElement implements IEElement {
    @objid ("0c87d259-4583-11e0-b54c-00137279a832")
    private TimeEvent ecoreElement;


    @objid ("0c87d25a-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        Element ecoreOwner = getEcoreElement().getOwner();
        IElement objingOwner = (IElement) ReverseProperties.getInstance().getMappedElement(ecoreOwner);
        if (objingOwner instanceof IBehavior){
            IEvent result = Modelio.getInstance().getModelingSession().getModel().createEvent();
            result.setKind(ObEventTypeEnum.TIME_EVENT);
            return result;
        }else
            return null;
    }

    @objid ("0c87d25e-4583-11e0-b54c-00137279a832")
    public ETimeEvent(TimeEvent element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0c8a34b3-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        if (((IEvent) objingElt).getComposed() == null){
            Element ecoreOwner = ecoreElement.getOwner();
            if (ecoreOwner instanceof Transition){
                IStateMachine sm = (IStateMachine) ReverseProperties.getInstance().getMappedElement(((Transition) ecoreOwner).getContainer().getStateMachine());
                sm.addEComponent((IEvent)objingElt);
            }else{
                Modelio.getInstance().getModelingSession().getModel().deleteElement(objingElt);
            }
        }
    }

    @objid ("0c8a34b6-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0c8a34bb-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setExpression((IEvent)objingElt);
    }

    @objid ("0c8a34be-4583-11e0-b54c-00137279a832")
    public ETimeEvent() {
    }

    @objid ("0c8c970f-4583-11e0-b54c-00137279a832")
    private void setExpression(IEvent objingElt) {
        ValueSpecification value =    ecoreElement.getWhen();
        
        if (value != null){
            String expr = value.stringValue();
            if (expr != null){
                objingElt.setExpression(expr);
            }
        }
    }

}
