/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEvent;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IInternalTransition;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateVertex;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ProtocolTransition;
import org.eclipse.uml2.uml.SendOperationEvent;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Trigger;
import org.eclipse.uml2.uml.Vertex;

@objid ("0c9fa9dc-4583-11e0-b54c-00137279a832")
public class ETransition extends ENamedElement implements IEElement {
    @objid ("0c9fa9dd-4583-11e0-b54c-00137279a832")
    private boolean isInternalTransition;

    @objid ("0c9fa9de-4583-11e0-b54c-00137279a832")
    private Transition ecoreElement;


    @objid ("0c9fa9df-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if (isInternalTransition)
            return Modelio.getInstance().getModelingSession().getModel().createInternalTransition();
        return Modelio.getInstance().getModelingSession().getModel()
        .createTransition();
    }

    @objid ("0c9fa9e3-4583-11e0-b54c-00137279a832")
    public ETransition(Transition element) {
        super(element);
        ecoreElement = element;
        isInternalTransition = EcoreModelNavigation.isInternalTransition(element);
    }

    @objid ("0ca20c37-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0ca20c3a-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0ca20c3f-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        if (!isInternalTransition)
            setTarget((ITransition) objingElt);
        
        setSource((ITransition) objingElt);
        
        setEffects((ITransition) objingElt);
        setTrigger((ITransition) objingElt);
        setPost((ITransition) objingElt);
        setGuard((ITransition) objingElt);
        
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setReceivedEvent((ITransition) objingElt);
            setSentEvents((ITransition) objingElt);
            setEffect((ITransition) objingElt);
        
        }
    }

    @objid ("0ca20c42-4583-11e0-b54c-00137279a832")
    private void setEffect(ITransition objingElt) {
        String effect = ObjingEAnnotation.getEffect(ecoreElement);
        if (effect != null)
            objingElt.setEffect(effect);
    }

    @objid ("0ca46e90-4583-11e0-b54c-00137279a832")
    private void setSentEvents(ITransition objingElt) {
        String signalString = ObjingEAnnotation.getSignal(ecoreElement);
        if (signalString != null)
            objingElt.setSentEvents(signalString);
    }

    @objid ("0ca46e93-4583-11e0-b54c-00137279a832")
    private void setReceivedEvent(ITransition objingElt) {
        String receivedEvent = ObjingEAnnotation.getReceivedEvent(ecoreElement);
        if (receivedEvent != null){
            if (objingElt instanceof IInternalTransition){
                objingElt.setReceivedEvents(receivedEvent);
            }else{
                objingElt.setReceivedEvents(receivedEvent);
            }
        }
    }

    @objid ("0ca46e96-4583-11e0-b54c-00137279a832")
    private void setGuard(ITransition objingElt) {
        Constraint guardConstraint = ecoreElement.getGuard();
        String guard = null;
        if (guardConstraint != null)
            guard = guardConstraint.getName();
        
        if (guard != null)
            objingElt.setGuard(guard);
    }

    @objid ("0ca46e99-4583-11e0-b54c-00137279a832")
    private void setPost(ITransition objingElt) {
        String post = null;
        
        if (ecoreElement instanceof ProtocolTransition){
            Constraint postConstraint = ((ProtocolTransition) ecoreElement).getPostCondition();
            if (postConstraint != null)
                post = postConstraint.getName();
        }else if (ReverseProperties.getInstance().isRoundtripEnabled()){
            post = ObjingEAnnotation.getPostCondition(ecoreElement);
        }
        if (post != null)
            objingElt.setPostCondition(post);
    }

    @objid ("0ca46e9c-4583-11e0-b54c-00137279a832")
    private void setSource(ITransition transition) {
        Vertex ecoreSource = ecoreElement.getSource();
        if (ecoreSource != null) {
            Object objingSource = ReverseProperties.getInstance().getMappedElement(ecoreSource);
        
            if (objingSource instanceof IStateVertex) {
                if (isInternalTransition && objingSource instanceof IState)
                    ((IInternalTransition) transition)
                    .setSComposed((IState) objingSource);
                else
                    transition.setSource((IStateVertex) objingSource);
            }
        }
    }

    @objid ("0ca46e9f-4583-11e0-b54c-00137279a832")
    private void setTarget(ITransition transition) {
        Vertex ecoreTarget = ecoreElement.getTarget();
        if (ecoreTarget != null) {
            Object objingTarget = ReverseProperties.getInstance().getMappedElement(ecoreTarget);
        
            if (objingTarget instanceof IStateVertex)
                transition.setTarget((IStateVertex) objingTarget);
        }
    }

    @objid ("0ca46ea2-4583-11e0-b54c-00137279a832")
    private void setEffects(ITransition transition) {
        Behavior behavior = ecoreElement.getEffect();
        
        if (behavior instanceof OpaqueBehavior) {
            for (Object clientDependency : behavior.getClientDependencies()) {
                for (Object ecoreSupplier : ((Dependency) clientDependency)
                        .getSuppliers()) {
                    if (ecoreSupplier instanceof Signal) {
                        Object objingSupplier = ReverseProperties.getInstance()
                        .getMappedElement((Signal) ecoreSupplier);
                        if (objingSupplier instanceof ISignal) {
                            transition.setEffects((ISignal) objingSupplier);
                            return;
                        }
                    }
                }
            }
        }
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            String signal = ObjingEAnnotation.getSignal(ecoreElement);
            if (signal != null){
                transition.setEffect(signal);
            }
        }
    }

    @objid ("0ca46ea5-4583-11e0-b54c-00137279a832")
    private void setTrigger(ITransition transition) {
        EList<?> ecoreTriggerList = ecoreElement.getTriggers();
        if (ecoreTriggerList.size() > 0) {
            Trigger ecoreTrigger = (Trigger) ecoreTriggerList.get(0);
            Event ecoreEvent = ecoreTrigger.getEvent();
            if (ecoreEvent != null){    
                if (ecoreEvent instanceof SendOperationEvent) {
                    setTriggerEvent(transition, (SendOperationEvent) ecoreEvent);
        
                } else if (ecoreEvent instanceof ChangeEvent) {
                    setReceivedEvent(transition, (ChangeEvent) ecoreEvent);
                } else {
                    setEvent(transition,  ecoreEvent);
                }
            }
        }
    }

    @objid ("0ca46ea8-4583-11e0-b54c-00137279a832")
    private void setTriggerEvent(ITransition transition, SendOperationEvent ecoreEvent) {
        Operation ecoreOp = ecoreEvent.getOperation();
        if (ecoreOp != null) {
            StateMachine ecoreSM = EcoreModelNavigation
            .getMostEnclosingStateMachine(ecoreElement);
            Object objingSM = ReverseProperties.getInstance().getMappedElement(ecoreSM);
            Object objingOp = ReverseProperties.getInstance().getMappedElement(ecoreOp);
        
            if (objingSM instanceof IStateMachine
                    && objingOp instanceof IOperation) {
                IEvent objingEvent = Modelio.getInstance()
                .getModelingSession().getModel().createEvent();
                objingEvent.setComposed((IStateMachine) objingSM);
                transition.setTrigger(objingEvent);
        
                String name = ecoreEvent.getName();
                if (EcoreModelNavigation.isNotNull(name))
                    objingEvent.setName(name);
                else 
                    objingEvent.setName("");
        
        
                objingEvent.setCalled((IOperation) objingOp);
            }
        }
    }

    @objid ("0ca46eac-4583-11e0-b54c-00137279a832")
    private void setReceivedEvent(ITransition transition, ChangeEvent event) {
        IEvent objEvent = (IEvent) ReverseProperties.getInstance().getMappedElement(event);
        transition.setTrigger(objEvent);
    }

    @objid ("0ca46eb0-4583-11e0-b54c-00137279a832")
    public ETransition() {
    }

    @objid ("693d6580-1a76-11e1-8103-0027103f347d")
    private void setEvent(final ITransition transition, final Event ecoreEvent) {
        IEvent objEvent = (IEvent) ReverseProperties.getInstance().getMappedElement(ecoreEvent);
        transition.setTrigger(objEvent);
    }

}
