/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore;

import java.util.List;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOpaqueAction;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ValueSpecificationAction;

@objid ("0ccf58e4-4583-11e0-b54c-00137279a832")
public class EValueSpecificationAction extends EActivityNode implements IEElement {
    @objid ("0ccf58e5-4583-11e0-b54c-00137279a832")
    private ValueSpecificationAction ecoreElement;


    @objid ("0ccf58e6-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            if(ObjingEAnnotation.isDeleted(ecoreElement))
                return null;
        }
                
        IOpaqueAction element = Modelio.getInstance().getModelingSession().getModel()
        .createOpaqueAction();
                
        try{
            IStereotype stereo = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IOpaqueAction.class, XMIStereotypes.UML2VALUESPECIFICATIONACTION);
            element.addExtension(stereo);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
        return element;
    }

    @objid ("0ccf58ea-4583-11e0-b54c-00137279a832")
    public EValueSpecificationAction(ValueSpecificationAction element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0cd1bb3e-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        super.attach(objingElt);
    }

    @objid ("0cd1bb41-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0cd1bb46-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        super.setProperties(objingElt);
        setValue((IOpaqueAction) objingElt);
    }

    @objid ("0cd41da1-4583-11e0-b54c-00137279a832")
    public EValueSpecificationAction() {
    }

    @objid ("0cd67ff2-4583-11e0-b54c-00137279a832")
    private void setValue(IOpaqueAction objingElt) {
        ITagType value =  null;
                
        IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
        try{
                
            IStereotype stereo = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IOpaqueAction.class, XMIStereotypes.UML2VALUESPECIFICATIONACTION);
                
            for (ITagType tagType : stereo.getDefinedTagType()){
                if (tagType.getName().equals("Value"))
                    value = tagType;
            }
                
            ITaggedValue taggedValue = model.createTaggedValue();
                
            taggedValue.setDefinition(value);
            taggedValue.setAnnoted(objingElt);
                
            ITagParameter actual = model.createTagParameter();
            actual.setValue(ecoreElement.getValue().stringValue());
            taggedValue.addActual(actual);
                
                
        }catch(StereotypeNotFoundException e){
            //normally always found ...
        }
    }

}
