/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ActionExecutionSpecification;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.OccurrenceSpecification;

@objid ("0cf57e88-4583-11e0-b54c-00137279a832")
public class ActionExecutionSpecificationImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0cf57e89-4583-11e0-b54c-00137279a832")
    private ReverseProperties revProp;

    @objid ("0cf57e8a-4583-11e0-b54c-00137279a832")
    private ActionExecutionSpecification ecoreElement;

    @objid ("0cf57e8b-4583-11e0-b54c-00137279a832")
    private OccurrenceSpecification ecoreStart;

    @objid ("0cf57e8c-4583-11e0-b54c-00137279a832")
    private OccurrenceSpecification ecoreFinish;


    @objid ("0cf57e8d-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createExecutionSpecification();
    }

    @objid ("0cf57e91-4583-11e0-b54c-00137279a832")
    public ActionExecutionSpecificationImportMapper(ActionExecutionSpecification element) {
        super(element);
        ecoreElement = element;
        revProp = ReverseProperties.getInstance();
        ecoreStart = ecoreElement.getStart();
        ecoreFinish = ecoreElement.getFinish();
    }

    @objid ("0cf7e0e0-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("ExecutionSpecification");
        return result;
    }

    @objid ("0cf7e0e6-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0cf7e0e9-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0cfa4338-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        setName((IExecutionSpecification) objingElt);
        insertInDiagram((IExecutionSpecification) objingElt);
    }

    @objid ("0cfa433b-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(IExecutionSpecification specification) {
        Lifeline eCoveredLL = (Lifeline) ecoreElement.getCovereds().get(0);
                
        if (eCoveredLL != null) {
            ILifeline oCoveredLL = (ILifeline) revProp
                    .getMappedElement(eCoveredLL);
            if (oCoveredLL != null) {
                InteractionFragment ecoreEnclosing = EcoreModelNavigation
                        .getEnclosing(ecoreElement);
                
                InteractionFragment ecoreBeforeStart = null;
                if (ecoreStart != null)
                    ecoreBeforeStart = getEndBeforeStart(ecoreEnclosing);
                
                InteractionFragment ecoreBeforeFinish = null;
                if (ecoreStart != null)
                    ecoreBeforeFinish = getEndBeforeFinish(ecoreEnclosing);
                
                IInteractionFragment objingBeforeStart = null;
                IInteractionFragment objingBeforeFinish = null;
                if (ecoreBeforeStart != null) {
                    IModelElement oBeforeStart = (IModelElement) revProp
                            .getMappedElement(ecoreBeforeStart);
                    if (!(oBeforeStart instanceof IInteractionFragment))
                        objingBeforeStart = ObjingModelNavigation
                                .getConnectedFragment(oBeforeStart);
                    else
                        objingBeforeStart = (IInteractionFragment) oBeforeStart;
                }
                
                if (ecoreBeforeFinish != null && ecoreStart != null
                        && ecoreFinish != null
                        && !ecoreBeforeFinish.equals(ecoreStart)
                        && !ecoreBeforeFinish.equals(ecoreElement)) {
                    IModelElement oBeforeFinish = (IModelElement) revProp
                            .getMappedElement(ecoreBeforeFinish);
                    if (!(oBeforeFinish instanceof IInteractionFragment))
                        objingBeforeFinish = ObjingModelNavigation
                                .getConnectedFragment(oBeforeFinish);
                    else
                        objingBeforeFinish = (IInteractionFragment) oBeforeFinish;
                }
                
                try {
                    if (endsAreInserted(ecoreBeforeStart, ecoreBeforeFinish)) {
                        addSpecification(ecoreEnclosing, specification,
                                oCoveredLL, objingBeforeStart,
                                objingBeforeFinish);
                    } else {
                        insertSpecification(ecoreEnclosing, specification,
                                oCoveredLL, objingBeforeStart);
                    }
                } catch (Exception e) {
                   e.printStackTrace(Modelio.err);
                    PartialImportMap.getInstance().remove(ecoreElement);
                    TotalImportMap.getInstance().remove(ecoreElement);
                    Modelio.getInstance().getModelingSession().getModel()
                            .deleteElement(specification);
                }
            }
        }
    }

    @objid ("0cfa433e-4583-11e0-b54c-00137279a832")
    private boolean endsAreInserted(InteractionFragment beforeStart, InteractionFragment beforeFinish) {
        if (beforeStart != null && beforeFinish != null) {
            TotalImportMap totalCreationMap = TotalImportMap.getInstance();
            if (totalCreationMap.get(beforeStart) != null
                    && totalCreationMap.get(beforeFinish) != null) {
                return true;
            }
        }
        return false;
    }

    @objid ("0cfa4344-4583-11e0-b54c-00137279a832")
    private InteractionFragment getEndBeforeStart(InteractionFragment ecoreEnclosing) {
        InteractionFragment ecoreBeforeStart = null;
                
        if (ecoreEnclosing instanceof Interaction)
            ecoreBeforeStart = EcoreModelNavigation
                    .getPreviousFragmentInInteraction(ecoreStart);
        else if (ecoreEnclosing instanceof InteractionOperand)
            ecoreBeforeStart = EcoreModelNavigation
                    .getPreviousFragmentInOperand(ecoreStart);
        return ecoreBeforeStart;
    }

    @objid ("0cfca592-4583-11e0-b54c-00137279a832")
    private InteractionFragment getEndBeforeFinish(InteractionFragment ecoreEnclosing) {
        InteractionFragment ecoreBeforeEnd = null;
                
        if (ecoreEnclosing instanceof Interaction)
            ecoreBeforeEnd = EcoreModelNavigation
                    .getPreviousFragmentInInteraction(ecoreFinish);
        else if (ecoreEnclosing instanceof InteractionOperand)
            ecoreBeforeEnd = EcoreModelNavigation
                    .getPreviousFragmentInOperand(ecoreFinish);
        return ecoreBeforeEnd;
    }

    @objid ("0cfca597-4583-11e0-b54c-00137279a832")
    private void addSpecification(InteractionFragment ecoreEnclosing, IExecutionSpecification specification, ILifeline coveredLL, IInteractionFragment objingBeforeStart, IInteractionFragment objingBeforeFinish) throws InvalidInteractionManipulationException {
        IModelElement objingEnclosing = (IModelElement) revProp
                .getMappedElement(ecoreEnclosing);
                
        if (objingEnclosing instanceof IInteraction) {
            ((IInteraction) objingEnclosing).add(specification, coveredLL,
                    objingBeforeStart, ObRelativeTimeEnum.AFTER,
                    objingBeforeFinish, ObRelativeTimeEnum.AFTER);
        } else if (objingEnclosing instanceof IInteractionOperand) {
            ((IInteractionOperand) objingEnclosing).add(specification,
                    coveredLL, objingBeforeStart, ObRelativeTimeEnum.AFTER,
                    objingBeforeFinish, ObRelativeTimeEnum.AFTER);
        }
    }

    @objid ("0cfca59e-4583-11e0-b54c-00137279a832")
    private void insertSpecification(InteractionFragment ecoreEnclosing, IExecutionSpecification specification, ILifeline coveredLL, IInteractionFragment objingPrevious) throws InvalidInteractionManipulationException {
        IModelElement objingEnclosing = (IModelElement) revProp
                .getMappedElement(ecoreEnclosing);
                
        if (objingEnclosing instanceof IInteraction) {
            ((IInteraction) objingEnclosing).insert(specification, coveredLL,
                    objingPrevious, ObRelativeTimeEnum.AFTER);
        } else if (objingEnclosing instanceof IInteractionOperand) {
            if (objingPrevious != null)
                ((IInteractionOperand) objingEnclosing).insert(specification,
                        coveredLL, objingPrevious, ObRelativeTimeEnum.AFTER);
            else
                ((IInteractionOperand) objingEnclosing).insert(specification,
                        coveredLL, (IInteractionOperand) objingEnclosing,
                        ObRelativeTimeEnum.DURING);
        }
    }

}
