/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ICombinedFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObInteractionOperatorEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.InteractionOperatorKind;
import org.eclipse.uml2.uml.Lifeline;

@objid ("0d062f07-4583-11e0-b54c-00137279a832")
public class CombinedFragmentImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d062f08-4583-11e0-b54c-00137279a832")
    private CombinedFragment ecoreElement;


    @objid ("0d062f09-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createCombinedFragment();
    }

    @objid ("0d089155-4583-11e0-b54c-00137279a832")
    private List<ILifeline> getCoveredLifelines() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        List<ILifeline> oCoveredList = new ArrayList<ILifeline>();
                
        for (Object eCoveredLL : ecoreElement.getCovereds()) {
            ILifeline oCoveredLL = (ILifeline) revProp
                    .getMappedElement((Lifeline) eCoveredLL);
                
            if (oCoveredLL != null)
                oCoveredList.add(oCoveredLL);
        }
        return oCoveredList;
    }

    @objid ("0d08915b-4583-11e0-b54c-00137279a832")
    public CombinedFragmentImportMapper(CombinedFragment element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d08915e-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("CombinedFragment");
        return result;
    }

    @objid ("0d0af3b2-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0d0af3b5-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0d0af3ba-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof ICombinedFragment) {
            setName((ICombinedFragment) objingElt);
            setOperand((ICombinedFragment) objingElt);
            insertInDiagram((ICombinedFragment) objingElt);
        }
    }

    @objid ("0d0af3bd-4583-11e0-b54c-00137279a832")
    private void setOperand(ICombinedFragment fragment) {
        InteractionOperatorKind kindOfOperator = ecoreElement
                .getInteractionOperator();
        switch (kindOfOperator.getValue()) {
        case InteractionOperatorKind.ALT:
            fragment.setOperator(ObInteractionOperatorEnum.ALT_OP);
            break;
        case InteractionOperatorKind.ASSERT:
            fragment.setOperator(ObInteractionOperatorEnum.ASSERT_OP);
            break;
        case InteractionOperatorKind.BREAK:
            fragment.setOperator(ObInteractionOperatorEnum.BREAK_OP);
            break;
        case InteractionOperatorKind.CONSIDER:
            fragment.setOperator(ObInteractionOperatorEnum.CONSIDER_OP);
            break;
        case InteractionOperatorKind.CRITICAL:
            fragment.setOperator(ObInteractionOperatorEnum.CRITICAL_OP);
            break;
        case InteractionOperatorKind.IGNORE:
            fragment.setOperator(ObInteractionOperatorEnum.IGNORE_OP);
            break;
        case InteractionOperatorKind.LOOP:
            fragment.setOperator(ObInteractionOperatorEnum.LOOP_OP);
            break;
        case InteractionOperatorKind.NEG:
            fragment.setOperator(ObInteractionOperatorEnum.NEG_OP);
            break;
        case InteractionOperatorKind.OPT:
            fragment.setOperator(ObInteractionOperatorEnum.OPT_OP);
            break;
        case InteractionOperatorKind.PAR:
            fragment.setOperator(ObInteractionOperatorEnum.PAR_OP);
            break;
        case InteractionOperatorKind.SEQ:
            fragment.setOperator(ObInteractionOperatorEnum.SEQ_OP);
            break;
        case InteractionOperatorKind.STRICT:
            fragment.setOperator(ObInteractionOperatorEnum.STRICT_OP);
            break;
        }
    }

    @objid ("0d0d5609-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(ICombinedFragment fragment) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        List<ILifeline> oCoveredLLs = getCoveredLifelines();
                
        if (oCoveredLLs.size() > 0) {
                
            Interaction ecoreInt = EcoreModelNavigation
                    .getMostEnclosingInteraction(ecoreElement);
            IInteraction objingInt = (IInteraction) revProp
                    .getMappedElement(ecoreInt);
                
            InteractionFragment ecoreEnclosing = EcoreModelNavigation
                    .getEnclosing(ecoreElement);
                
            if (ecoreEnclosing instanceof Interaction) {
                InteractionFragment ecorePrevious = EcoreModelNavigation
                        .getPreviousFragmentInInteraction(ecoreElement);
                IInteractionFragment objingPrevious = null;
                if (ecorePrevious != null)
                    objingPrevious = (IInteractionFragment) revProp
                            .getMappedElement(ecorePrevious);
                
                insertCFInInteraction(objingInt, fragment, oCoveredLLs,
                        objingPrevious, ObRelativeTimeEnum.AFTER);
            } else if (ecoreEnclosing instanceof InteractionOperand) {
                IInteractionOperand objingEnclosing = (IInteractionOperand) revProp
                        .getMappedElement(ecoreEnclosing);
                InteractionFragment ecorePrevious = EcoreModelNavigation
                        .getPreviousFragmentInOperand(ecoreElement);
                IInteractionFragment objingPrevious = null;
                if (ecorePrevious != null)
                    objingPrevious = (IInteractionFragment) revProp
                            .getMappedElement(ecorePrevious);
                
                if (objingPrevious == null)
                    insertCFInIO(objingEnclosing, fragment, oCoveredLLs,
                            objingEnclosing, ObRelativeTimeEnum.DURING);
                else
                    insertCFInIO(objingEnclosing, fragment, oCoveredLLs,
                            objingPrevious, ObRelativeTimeEnum.AFTER);
            } else
                throw new NotFoundException("CombinedFragments enclosed in "
                        + ecoreEnclosing.getClass().getSimpleName()
                        + " are not supported.");
        }else{
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel().deleteElement(fragment);
        }
    }

    @objid ("0d0d560c-4583-11e0-b54c-00137279a832")
    private void insertCFInInteraction(IInteraction objingInt, ICombinedFragment fragment, List<ILifeline> coveredLLs, IInteractionFragment previousIF, ObRelativeTimeEnum relativeTime) {
        try {
            objingInt.insert(fragment, coveredLLs, previousIF, relativeTime);
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
                    .deleteElement(fragment);
        } catch (Exception e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
                    .deleteElement(fragment);
        }
    }

    @objid ("0d0d5615-4583-11e0-b54c-00137279a832")
    private void insertCFInIO(IInteractionOperand intOperand, ICombinedFragment fragment, List<ILifeline> coveredLLs, IInteractionFragment previousIF, ObRelativeTimeEnum relativeTime) {
        try {
            intOperand.insert(fragment, coveredLLs, previousIF, relativeTime);
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
                    .deleteElement(fragment);
        } catch (Exception e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
                    .deleteElement(fragment);
        }
    }

}
