/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ICombinedFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InteractionConstraint;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0d252d97-4583-11e0-b54c-00137279a832")
public class InteractionOperandImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d252d98-4583-11e0-b54c-00137279a832")
    private InteractionOperand ecoreElement;


    @objid ("0d252d99-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createInteractionOperand();
    }

    @objid ("0d278fe7-4583-11e0-b54c-00137279a832")
    public InteractionOperandImportMapper(InteractionOperand element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d278fea-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("InteractionOperand");
        return result;
    }

    @objid ("0d278ff0-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0d29f240-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0d29f245-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if ((objingElt != null) &&( objingElt instanceof IInteractionOperand)) {
            setName((IInteractionOperand) objingElt);
            setGuard((IInteractionOperand) objingElt);
            insertInDiagram((IInteractionOperand) objingElt);
        }
    }

    @objid ("0d29f248-4583-11e0-b54c-00137279a832")
    private void setGuard(IInteractionOperand objingOperand) {
        InteractionConstraint guard = ecoreElement.getGuard();
        if (guard != null) {
            ValueSpecification specification = guard.getSpecification();
            if (specification != null) {
                String value = specification.stringValue();
                if (EcoreModelNavigation.isNotNull(value))
                    objingOperand.setGuard(value);
                else 
                    objingOperand.setName("");
            }
        }
    }

    @objid ("0d29f24b-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(IInteractionOperand objingOperand) {
        Element ecoreOwner = ecoreElement.getOwner();
                
        if (ecoreOwner instanceof CombinedFragment) {
            ReverseProperties revProp = ReverseProperties.getInstance();
            ICombinedFragment objingOwner = (ICombinedFragment) revProp
                    .getMappedElement(ecoreOwner);
                
            if (objingOwner != null) {
                InteractionOperand ecorePreviousOperand = EcoreModelNavigation
                        .getPreviousOperand(ecoreElement);
                IInteractionOperand objingPreviousOperand = null;
                if (ecorePreviousOperand != null)
                    objingPreviousOperand = (IInteractionOperand) revProp
                            .getMappedElement(ecorePreviousOperand);
                
                try {
                    if (objingPreviousOperand == null)
                        objingOwner.insert(objingOperand, null,
                                ObRelativeTimeEnum.BEFORE);
                    else
                        objingOwner
                                .insert(objingOperand, objingPreviousOperand,
                                        ObRelativeTimeEnum.AFTER);
                } catch (Exception e) {
                    Modelio.err
                            .println("InvalidInteractionManipulationException: attachment issue when setting \""
                                    + objingOwner.getName()
                                    + "\" CombinedFragment as owner of the \""
                                    + objingOperand.getName()
                                    + "\" InteractionOperand.");
                   e.printStackTrace(Modelio.err);
                    PartialImportMap.getInstance().remove(ecoreElement);
                    TotalImportMap.getInstance().remove(ecoreElement);
                    Modelio.getInstance().getModelingSession().getModel()
                            .deleteElement(objingOperand);
                }
            }else{
                 PartialImportMap.getInstance().remove(ecoreElement);
                 TotalImportMap.getInstance().remove(ecoreElement);
                 Modelio.getInstance().getModelingSession().getModel()
                         .deleteElement(objingOperand);
            }
        }
    }

}
