/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionUse;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.EInteraction;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.InteractionUse;
import org.eclipse.uml2.uml.Lifeline;

@objid ("0d29f24e-4583-11e0-b54c-00137279a832")
public class InteractionUseImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d2c549a-4583-11e0-b54c-00137279a832")
     InteractionUse ecoreElement;


    @objid ("0d2c549b-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        List<ILifeline> oCoveredLLs = getCoveredLifelines();
        
        if (oCoveredLLs.size() > 0) {
            return Modelio.getInstance().getModelingSession().getModel()
            .createInteractionUse();}
        return null;
    }

    @objid ("0d2c549f-4583-11e0-b54c-00137279a832")
    private List<ILifeline> getCoveredLifelines() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        List<ILifeline> oCoveredList = new ArrayList<ILifeline>();
        
        for (Object eCoveredLL : ecoreElement.getCovereds()) {
            ILifeline oCoveredLL = (ILifeline) revProp.getMappedElement((Lifeline) eCoveredLL);
        
            if (oCoveredLL != null)
                oCoveredList.add(oCoveredLL);
        }
        return oCoveredList;
    }

    @objid ("0d2c54a5-4583-11e0-b54c-00137279a832")
    public InteractionUseImportMapper(InteractionUse element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d2c54a8-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("InteractionUse");
        return result;
    }

    @objid ("0d2eb6f9-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0d2eb6fc-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0d2eb701-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof IInteractionUse) {
            setName((IInteractionUse) objingElt);
            setRefersTo((IInteractionUse) objingElt);
            insertInDiagram((IInteractionUse) objingElt);
            // The InteractionUse must be inserted before inserting a Gate
            // inside.
            addActualGates((IInteractionUse) objingElt);
        }
    }

    @objid ("0d31194e-4583-11e0-b54c-00137279a832")
    private void setRefersTo(IInteractionUse use) {
        Interaction ecoreRefered = ecoreElement.getRefersTo();
        if (ecoreRefered != null) {
            ReverseProperties revProp = ReverseProperties.getInstance();
            Object objingRefered = revProp.getMappedElement(ecoreRefered);
        
            if (objingRefered instanceof IInteraction) {
                use.setRefersTo((IInteraction) objingRefered);
                EInteraction ecoreInteraction = new EInteraction(ecoreRefered);
                for (Object ecoreGate : ecoreRefered.getFormalGates()) {
                    ecoreInteraction.createUnconnectedGate((Gate)ecoreGate, (IInteraction) objingRefered);
                }
        
        
            }
        }
    }

    @objid ("0d311951-4583-11e0-b54c-00137279a832")
    private void addActualGates(IInteractionUse use) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        for (Object g : ecoreElement.getActualGates()) {
            Gate actualGate = (Gate) g;
            if (!revProp.isMappedElement(actualGate)) {
                IGate objingGate = (IGate) ReverseProperties.getInstance().getMappedElement(actualGate); 
                if (objingGate == null)
                    objingGate = Modelio.getInstance().getModelingSession().getModel().createGate();
        
                String actualName = actualGate.getName();
                if (EcoreModelNavigation.isNotNull(actualName))
                    objingGate.setName(actualName);
                else 
                    objingGate.setName("");
        
                Gate referedGate = EcoreModelNavigation.getFormalGate(
                        ecoreElement.getRefersTo(), actualGate);
                if (referedGate != null) {
                    Object gate = revProp.getMappedElement(referedGate);
                    IGate objingRefered = null;
                    if (gate != null) {
                        objingRefered = (IGate) gate;
        
                    }else{
                        objingRefered = Modelio.getInstance().getModelingSession().getModel().createGate();
                        objingRefered.setName(actualName);
                        IInteraction interaction =  (IInteraction)revProp.getMappedElement(ecoreElement.getRefersTo());
                        try {
                            interaction.insert(objingRefered, null,
                                    ObRelativeTimeEnum.AFTER);
                            interaction.addFormalGate(objingRefered);
                            TotalImportMap.getInstance().put(referedGate,
                                    objingRefered);
                        } catch (InvalidInteractionManipulationException e) {
                            e.printStackTrace(Modelio.err);
                            TotalImportMap.getInstance().remove(actualGate);
                            TotalImportMap.getInstance().remove(actualGate);
                            Modelio.getInstance().getModelingSession()
                            .getModel().deleteElement(objingGate);
                        }
        
                    }
        
        
                    objingGate.setFormal(objingRefered);
                }
        
                if (EcoreModelNavigation.isNotNull(actualName))
                    objingGate.setName(actualName);
        
                try {
                    ObList<IGate> gateList = use.getActualGate();
                    IInteractionFragment temp = null;
                    if (gateList.size() > 0 ){
                        temp = gateList.get(gateList.size() -1);
                        use.insert(objingGate,temp , ObRelativeTimeEnum.AFTER);
                    }else{
                        use.insert(objingGate,use , ObRelativeTimeEnum.ON);
                    }
                    TotalImportMap.getInstance().put(actualGate,
                            objingGate);
                } catch (InvalidInteractionManipulationException e) {
                    e.printStackTrace(Modelio.err);
                    TotalImportMap.getInstance().remove(actualGate);
                    TotalImportMap.getInstance().remove(actualGate);
                    Modelio.getInstance().getModelingSession()
                    .getModel().deleteElement(objingGate);
                }
                //                } else {
                    //    // The objing formal gate G1 of the current actual gate
                    //                // G2 has not yet been mapped.
                    //                // A solution would be to do the treatment when mapping
                    //                // G1.
                    //                // Unfortunately, the relation between InteractionUse
                    //                // and Interaction is only navigable
                    //                // in this direction: we can't get the list of
                    //                // InteractionUses that reference an
                    //                // Interaction.
                    //                // As a consequence, the actual gate can't be mapped
                    //                // correctly.
        
                    //                // However, when inserting an InteractionUse referencing
                    //                // an Interaction in a diagram,
                    //                // Joni unmasks actual Gates.
                //                // Thus, only the name won't fit with the one defined in
                //                // the Ecore model.
        
                //                // Deletion of the created Gate:
                //                Objecteering.getInstance().getModelingSession()
                //                .getModel().deleteElement(objingGate);
        
                //                String formalGateName = referedGate.getName();
                //                Object mappedGate = revProp
                //                .getMappedElement(referedGate);
                //                if (formalGateName != null
                //                && mappedGate instanceof IGate) {
                //                // Getting the actual Gate automatically unmasked by
                //                // Joni:
                //                for (IGate objingActualGate : use.getActualGate()) {
                //                IGate formalGate = objingActualGate
                //                .getFormal();
                //                if (formalGate.equals(mappedGate)) {
                //                objingActualGate.setName(formalGate
                //                .getName());
                //                break;
                //                }
                //                }
                //                }
                //                }
        
            }
        }
    }

    @objid ("0d311954-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(IInteractionUse intUse) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        List<ILifeline> oCoveredLLs = getCoveredLifelines();
        
        if (oCoveredLLs.size() > 0) {
            Interaction ecoreInt = EcoreModelNavigation
            .getMostEnclosingInteraction(ecoreElement);
            IInteraction objingInt = (IInteraction) revProp
            .getMappedElement(ecoreInt);
        
            InteractionFragment ecoreEnclosing = EcoreModelNavigation
            .getEnclosing(ecoreElement);
        
            if (ecoreEnclosing instanceof Interaction) {
                insertIUInInteraction(objingInt, intUse, oCoveredLLs);
            } else if (ecoreEnclosing instanceof InteractionOperand) {
                insertIUInOperand(objingInt, ecoreEnclosing, intUse,
                        oCoveredLLs);
            } else
                throw new NotFoundException("CombinedFragments enclosed in "
                        + ecoreEnclosing.getClass().getSimpleName()
                        + " are not supported.");
        }else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(intUse);
        }
    }

    @objid ("0d311957-4583-11e0-b54c-00137279a832")
    private void insertIUInInteraction(IInteraction objingInt, IInteractionUse intUse, List<ILifeline> coveredLLs) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        InteractionFragment ecorePrevious = EcoreModelNavigation
        .getPreviousFragmentInInteraction(ecoreElement);
        IInteractionFragment objingPrevious = null;
        if (ecorePrevious != null)
            objingPrevious = (IInteractionFragment) revProp
            .getMappedElement(ecorePrevious);
        
        try {
            objingInt.insert(intUse, coveredLLs, objingPrevious,
                    ObRelativeTimeEnum.AFTER);
        } catch (InvalidInteractionManipulationException e) {
            e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(intUse);
        } catch (Exception e) {
            e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(intUse);
        }
    }

    @objid ("0d337ba8-4583-11e0-b54c-00137279a832")
    private void insertIUInOperand(IInteraction objingInt, InteractionFragment ecoreEnclosing, IInteractionUse intUse, List<ILifeline> coveredLLs) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        IInteractionOperand objingEnclosing = (IInteractionOperand) revProp
        .getMappedElement(ecoreEnclosing);
        InteractionFragment ecorePrevious = EcoreModelNavigation
        .getPreviousFragmentInOperand(ecoreElement);
        IInteractionFragment objingPrevious = null;
        if (ecorePrevious != null)
            objingPrevious = (IInteractionFragment) revProp
            .getMappedElement(ecorePrevious);
        
        if (objingPrevious == null)
            insertIUInIO(objingEnclosing, intUse, coveredLLs, objingEnclosing,
                    ObRelativeTimeEnum.DURING);
        else
            insertIUInIO(objingEnclosing, intUse, coveredLLs, objingPrevious,
                    ObRelativeTimeEnum.AFTER);
    }

    @objid ("0d337bb0-4583-11e0-b54c-00137279a832")
    private void insertIUInIO(IInteractionOperand intOperand, IInteractionUse intUse, List<ILifeline> coveredLLs, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        try {
            intOperand.insert(intUse, coveredLLs, objingPrevious, relativeTime);
        } catch (InvalidInteractionManipulationException e) {
            e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(intUse);
        } catch (Exception e) {
            e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(intUse);
        }
    }

}
