/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.ENamedElement;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;

@objid ("0d337bb9-4583-11e0-b54c-00137279a832")
public class LifelineImportMapper extends ENamedElement implements IEElement {
    @objid ("0d35de02-4583-11e0-b54c-00137279a832")
    private Lifeline ecoreElement;


    @objid ("0d35de03-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Interaction ecoreInt = ecoreElement.getInteraction();
                
        // objingInt shall not be null as we already created it (at least)
        // in the "attach()" method.
        IInteraction objingInt = (IInteraction) revProp
                .getMappedElement(ecoreInt);
        return Modelio.getInstance().getModelingSession().getModel().createLifeline(ecoreElement.getName(),objingInt);
    }

    @objid ("0d35de07-4583-11e0-b54c-00137279a832")
    public LifelineImportMapper(Lifeline element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d35de0a-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("Lifeline");
        return result;
    }

    @objid ("0d35de10-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        Interaction ecoreInt = ecoreElement.getInteraction();
                
        IInteraction objingInt = (IInteraction) revProp
                .getMappedElement(ecoreInt);
                
        if (objingInt instanceof IInteraction) {
            objingInt.addOwnedLine((ILifeline) objingElt);
        }
    }

    @objid ("0d38405c-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0d384061-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        setName((ILifeline) objingElt);
        setRepresented((ILifeline) objingElt);
        //        insertInDiagram((ILifeline) objingElt);
    }

    @objid ("0d384064-4583-11e0-b54c-00137279a832")
    private void setName(ILifeline objingElt) {
        String name = ecoreElement.getName();
        if (EcoreModelNavigation.isNotNull(name))
            objingElt.setName(name);
        else 
            objingElt.setName("");
    }

    @objid ("0d38406a-4583-11e0-b54c-00137279a832")
    private void setRepresented(ILifeline lifeline) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        if (revProp.isRoundtripEnabled()) {
            boolean isRepresented = false;
            for (Object dependency : ecoreElement.getClientDependencies()){
                if (ObjingEAnnotation.isRepresentation((Element)dependency)){
                    IElement representation = (IElement)revProp.getMappedElement((Element)((Dependency)dependency).getSuppliers().get(0));
                    if (representation instanceof IInstance){
                        lifeline.setRepresented((IInstance)representation);
                        isRepresented = true;
                    }
                }
            }
            if (!isRepresented){
                setNormalRepresented(revProp,lifeline);
            }
                
        }else {
            setNormalRepresented(revProp,lifeline);
            
                
        }
    }

    @objid ("0d3aa2b8-4583-11e0-b54c-00137279a832")
    private void setNormalRepresented(ReverseProperties revProp, ILifeline lifeline) {
        ConnectableElement ecoreRepresented = ecoreElement.getRepresents();
        if (ecoreRepresented != null) {
            
            Object objingRepresented = revProp.getMappedElement(ecoreRepresented);
            if (objingRepresented instanceof IInstance)
                lifeline.setRepresented((IInstance)objingRepresented);
            else if (objingRepresented instanceof List){
                for (IModelElement elt : (ArrayList<IModelElement>) objingRepresented){
                    if (elt instanceof IInstance){
                        lifeline.setRepresented((IInstance) elt);
                    }
                }
            }
        }
    }

}
