/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionOccurenceSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionUse;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessageEnd;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObMessageKindEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObMessageSortEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.InteractionUse;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.MessageKind;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OccurrenceSpecification;

@objid ("0d3aa2bc-4583-11e0-b54c-00137279a832")
public class MessageImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d3aa2bd-4583-11e0-b54c-00137279a832")
    private Message ecoreElement;

    @objid ("0d3aa2be-4583-11e0-b54c-00137279a832")
    private MessageEnd ecoreReceived = null;

    @objid ("0d3aa2bf-4583-11e0-b54c-00137279a832")
    private MessageEnd ecoreSent = null;

    @objid ("99842144-1a76-11e1-8103-0027103f347d")
    private ObMessageKindEnum kindOfMessage = null;


    @objid ("0d3aa2c1-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createMessage();
    }

    @objid ("0d3aa2c5-4583-11e0-b54c-00137279a832")
    public MessageImportMapper(Message element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d3d0512-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0d3d0515-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0d3d051a-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof IMessage) {
            setName((IMessage) objingElt, ecoreElement);
            setKindOfMessage((IMessage) objingElt, ecoreElement);
            setSortOfMessage((IMessage) objingElt, ecoreElement);
            setSignature((IMessage) objingElt, ecoreElement);
            setStartAndEndElement((IMessage)objingElt);
            insertInDiagram((IMessage) objingElt);
        }
    }

    @objid ("0d3d051d-4583-11e0-b54c-00137279a832")
    private void setStartAndEndElement(IMessage objingElt) {
        TotalImportMap totalCreationMap = TotalImportMap.getInstance();
                
        if (ecoreSent != null){
            objingElt.getSendEvent().setName(ecoreSent.getName());
            totalCreationMap.put(ecoreSent, objingElt.getSendEvent());
        }
                
        if (ecoreReceived != null){
            objingElt.getReceiveEvent().setName(ecoreReceived.getName());
            totalCreationMap.put(ecoreReceived, objingElt.getReceiveEvent());
        }
    }

    @objid ("0d3f676a-4583-11e0-b54c-00137279a832")
    private void setKindOfMessage(IMessage message, Message ecoreMessage) {
        MessageKind ecoreKind = ecoreMessage.getMessageKind();
        switch (ecoreKind.getValue()) {
        case MessageKind.COMPLETE:
            message.setKindOfMessage(ObMessageKindEnum.COMPLETE_KIND);
            kindOfMessage = ObMessageKindEnum.COMPLETE_KIND;
            break;
        case MessageKind.FOUND:
            message.setKindOfMessage(ObMessageKindEnum.FOUND_KIND);
            kindOfMessage = ObMessageKindEnum.FOUND_KIND;
            break;
        case MessageKind.LOST:
            message.setKindOfMessage(ObMessageKindEnum.LOST_KIND);
            kindOfMessage = ObMessageKindEnum.LOST_KIND;
            break;
        case MessageKind.UNKNOWN:
            message.setKindOfMessage(ObMessageKindEnum.UNKNOWN_KIND);
            kindOfMessage = ObMessageKindEnum.UNKNOWN_KIND;
            break;
        default:
            message.setKindOfMessage(ObMessageKindEnum.UNKNOWN_KIND);
        kindOfMessage = ObMessageKindEnum.UNKNOWN_KIND;
        break;
        }
                
        // The getMessageKind() method is not reliable for LOST and FOUND
        // messages ...
        if (EcoreModelNavigation.isFoundMessage(ecoreMessage)) {
            message.setKindOfMessage(ObMessageKindEnum.FOUND_KIND);
            kindOfMessage = ObMessageKindEnum.FOUND_KIND;
        } else if (EcoreModelNavigation.isLostMessage(ecoreMessage)) {
            message.setKindOfMessage(ObMessageKindEnum.LOST_KIND);
            kindOfMessage = ObMessageKindEnum.LOST_KIND;
        }
    }

    @objid ("0d3f676e-4583-11e0-b54c-00137279a832")
    private void setSortOfMessage(IMessage message, Message ecoreMessage) {
        MessageSort ecoreSort = ecoreMessage.getMessageSort();
        switch (ecoreSort.getValue()) {
        case MessageSort.ASYNCH_CALL:
            message.setSortOfMessage(ObMessageSortEnum.A_SYNC_CALL);
            break;
        case MessageSort.ASYNCH_SIGNAL:
            message.setSortOfMessage(ObMessageSortEnum.A_SYNC_SIGNAL);
            break;
        case MessageSort.CREATE_MESSAGE:
            message.setSortOfMessage(ObMessageSortEnum.CREATE_MESSAGE);
            break;
        case MessageSort.DELETE_MESSAGE:
            message.setSortOfMessage(ObMessageSortEnum.DESTROY_MESSAGE);
            break;
        case MessageSort.REPLY:
            message.setSortOfMessage(ObMessageSortEnum.RETURN_MESSAGE);
            break;
        case MessageSort.SYNCH_CALL:
            message.setSortOfMessage(ObMessageSortEnum.SYNC_CALL);
            break;
        }
    }

    @objid ("0d3f6772-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(IMessage message) {
        ecoreReceived = ecoreElement.getReceiveEvent();
        ecoreSent = ecoreElement.getSendEvent();
                
        if (ecoreReceived instanceof Gate || ecoreSent instanceof Gate)
            insertMessageConnectedToGate(message);
        else
            insertMessageConnectedToLifelines(message);
    }

    @objid ("0d3f6775-4583-11e0-b54c-00137279a832")
    private void insertMessageConnectedToGate(IMessage message) {
        if (ecoreReceived instanceof Gate)
            insertMsgReceivedByGate(message, (Gate) ecoreReceived,
                    (MessageOccurrenceSpecification) ecoreSent);
        else
            insertMessageSentByGate(message,
                    (MessageOccurrenceSpecification) ecoreReceived,
                    (Gate) ecoreSent);
    }

    @objid ("0d3f6778-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("Message");
        return result;
    }

    @objid ("0d41c9c5-4583-11e0-b54c-00137279a832")
    private void insertMsgReceivedByGate(IMessage message, Gate receivedGate, MessageOccurrenceSpecification sentMOS) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        // Gets ecore and objing lifelines:
        EList<?> coveredLL = null;
        if (sentMOS != null)
            coveredLL = sentMOS.getCovereds();
                
        ILifeline objingSendingLL = null;
        if (coveredLL != null && coveredLL.size() > 0) {
            Lifeline ecoreSendingLL = (Lifeline) coveredLL.get(0);
            objingSendingLL = (ILifeline) revProp
            .getMappedElement(ecoreSendingLL);
        }
                
        // Gets the enclosing fragment (the Interaction or an
        // InteractionOperand):
        InteractionFragment ecoreEnclosing = EcoreModelNavigation
        .getEnclosing(ecoreElement);
        Object objingEnclosing = revProp.getMappedElement(ecoreEnclosing);
                
        // Gets the previous fragment in the diagram (before the sending
        // fragment of the message):
        InteractionFragment ecorePrevious = null;
        if (ecoreEnclosing instanceof Interaction)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInInteraction(ecoreElement);
        else if (ecoreEnclosing instanceof InteractionOperand)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInOperand(ecoreElement);
                
        IInteractionFragment objingPrevious = null;
        ObRelativeTimeEnum relativeTime = ObRelativeTimeEnum.AFTER;
        if (ecorePrevious != null)
            objingPrevious = (IInteractionFragment) revProp
            .getMappedElement(ecorePrevious);
                
        // If previous is the owner InteractionUse:
        if (ecorePrevious instanceof InteractionUse
                && ((InteractionUse) ecorePrevious).getActualGates().contains(
                        receivedGate))
            relativeTime = ObRelativeTimeEnum.DURING;
                
        // As we are in case of a message received by a gate, it can't be a
        // "Lost" message.
        if (kindOfMessage == ObMessageKindEnum.FOUND_KIND)
            insertFoundMsgAndGate(objingEnclosing, message, receivedGate,
                    objingPrevious, relativeTime);
        else{
            insertCompleteMsgReceivedByGate(objingEnclosing, sentMOS,
                    objingSendingLL, objingPrevious, relativeTime, message,
                    receivedGate);
                
                
        }
    }

    @objid ("0d41c9ca-4583-11e0-b54c-00137279a832")
    private void insertFoundMsgAndGate(Object objingEnclosing, IMessage message, Gate ecoreGate, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        IGate objingGate = (IGate) revProp.getMappedElement(ecoreGate);
        try {
            if (objingGate == null)
                objingGate = Modelio.getInstance().getModelingSession().getModel().createGate();
                
            String name = ecoreGate.getName();
            if (EcoreModelNavigation.isNotNull(name))
                objingGate.setName(name);
                
            Gate referedGate = EcoreModelNavigation.getReferedGate(ecoreGate);
            if (referedGate != null
                    && !EcoreModelNavigation.isFormalGate(ecoreGate)
                    && revProp.isMappedElement(referedGate)) {
                IGate objingFormalGate = (IGate) revProp
                .getMappedElement(referedGate);
                objingGate.setFormal(objingFormalGate);
            }
                
            TotalImportMap.getInstance().put(ecoreGate, objingGate);
                
            Element owner = ecoreGate.getOwner();
            IModelElement objingOwner = (IModelElement) revProp
            .getMappedElement(owner);
                
            // Insertion of the Gate:
            if (objingOwner instanceof IPartDecomposition) {
                ((IPartDecomposition) objingOwner).insert(objingGate,
                        objingPrevious, relativeTime);
            } else if (objingOwner instanceof IInteractionUse) {
                ((IInteractionUse) objingOwner).insert(objingGate,
                        objingPrevious, relativeTime);
            } else if (objingEnclosing instanceof IInteraction) {
                ((IInteraction) objingEnclosing).insert(objingGate,
                        objingPrevious, relativeTime);
            }
                
            // Insertion of the Message:
            if (objingEnclosing instanceof IInteraction) {
                ((IInteraction) objingEnclosing).insert(message, null,
                        objingGate);
            } else if (objingEnclosing instanceof IInteractionOperand) {
                ((IInteractionOperand) objingEnclosing).insert(message, null,
                        objingGate);
            }
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreGate);
            TotalImportMap.getInstance().remove(ecoreGate);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(objingGate);
        }
    }

    @objid ("0d41c9d1-4583-11e0-b54c-00137279a832")
    private void insertLostMsgAndGate(Object objingEnclosing, IMessage message, Gate ecoreGate, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        IGate objingGate = (IGate) revProp.getMappedElement(ecoreGate);
                
        try {
                
            if (objingGate == null)
                objingGate = Modelio.getInstance().getModelingSession().getModel().createGate();
                
            String name = ecoreGate.getName();
            if (EcoreModelNavigation.isNotNull(name))
                objingGate.setName(name);
                
            Gate referedGate = EcoreModelNavigation.getReferedGate(ecoreGate);
            if (referedGate != null
                    && !EcoreModelNavigation.isFormalGate(ecoreGate)
                    && revProp.isMappedElement(referedGate)) {
                IGate objingFormalGate = (IGate) revProp
                .getMappedElement(referedGate);
                objingGate.setFormal(objingFormalGate);
            }
                
            TotalImportMap.getInstance().put(ecoreGate, objingGate);
                
            Element owner = ecoreGate.getOwner();
            IModelElement objingOwner = (IModelElement) revProp
            .getMappedElement(owner);
                
            // Insertion of the Gate:
            if (objingOwner instanceof IPartDecomposition) {
                ((IPartDecomposition) objingOwner).insert(objingGate,
                        objingPrevious, relativeTime);
            } else if (objingOwner instanceof IInteractionUse) {
                ((IInteractionUse) objingOwner).insert(objingGate,
                        objingPrevious, relativeTime);
            } else if (objingEnclosing instanceof IInteraction) {
                ((IInteraction) objingEnclosing).insert(objingGate,
                        objingPrevious, relativeTime);
            }
                
            // Insertion of the Message:
            if (objingEnclosing instanceof IInteraction) {
                ((IInteraction) objingEnclosing).insert(message, objingGate,
                        null);
            } else if (objingEnclosing instanceof IInteractionOperand) {
                ((IInteractionOperand) objingEnclosing).insert(message,
                        objingGate, null);
            }
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreGate);
            TotalImportMap.getInstance().remove(ecoreGate);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(objingGate);
        }
    }

    @objid ("0d442c1e-4583-11e0-b54c-00137279a832")
    private void insertCompleteMsgReceivedByGate(Object objingEnclosing, MessageOccurrenceSpecification sentMOS, ILifeline objingSendingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime, IMessage message, Gate receivedGate) {
        // Creates and inserts the sending end of the message:
        IMessageEnd objingSendingEnd = createAndInsertMOS(objingEnclosing,
                sentMOS, objingSendingLL, objingPrevious, relativeTime);
                
        // Creates and inserts the receiving end of the message (the Gate):
                
        IGate objingReceivingGate = null;
        if (objingSendingEnd != null) {
            objingReceivingGate = createAndInsertGate(objingEnclosing,
                    receivedGate, objingSendingEnd, ObRelativeTimeEnum.ON);
        }
                
                
                
        // Inserts the message in the diagram:
        if (objingSendingEnd != null && objingReceivingGate != null) {
            if (objingEnclosing instanceof IInteraction) {
                insertMessage((IInteraction) objingEnclosing, message,
                        objingSendingEnd, objingReceivingGate);
            } else if (objingEnclosing instanceof IInteractionOperand) {
                insertMessage((IInteractionOperand) objingEnclosing, message,
                        objingSendingEnd, objingReceivingGate);
                
                
            }
        }
    }

    @objid ("0d442c27-4583-11e0-b54c-00137279a832")
    private IMessageEnd createAndInsertMOS(Object objingEnclosing, MessageOccurrenceSpecification sentMOS, ILifeline objingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum after) {
        IExecutionOccurenceSpecification objingEnd = Modelio
        .getInstance().getModelingSession().getModel()
        .createExecutionOccurenceSpecification();
        if (sentMOS != null) {
            String name = sentMOS.getName();
            if (EcoreModelNavigation.isNotNull(name))
                objingEnd.setName(name);
                
            TotalImportMap.getInstance().put(sentMOS, objingEnd);
        }
                
        try {
            if (objingEnclosing instanceof IInteraction) {
                ((IInteraction) objingEnclosing).insert(objingEnd, objingLL,
                        objingPrevious, after);
            } else if (objingEnclosing instanceof IInteractionOperand) {
                if (objingPrevious == null)
                    ((IInteraction) objingEnclosing).insert(objingEnd,
                            objingLL, (IInteractionOperand) objingEnclosing,
                            ObRelativeTimeEnum.DURING);
                else
                    ((IInteraction) objingEnclosing).insert(objingEnd,
                            objingLL, objingPrevious, after);
            }
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            if (sentMOS != null) {
                PartialImportMap.getInstance().remove(sentMOS);
                TotalImportMap.getInstance().remove(sentMOS);
            }
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(objingEnd);
            objingEnd = null;
        }
        return objingEnd;
    }

    @objid ("0d442c30-4583-11e0-b54c-00137279a832")
    private IGate createAndInsertGate(Object objingEnclosing, Gate receivedGate, IInteractionFragment objingSendingFragment, ObRelativeTimeEnum on) {
        Object temp = ReverseProperties.getInstance().getMappedElement(receivedGate);
                
        IGate objingGate = null;
                
        if  (!(temp != null) && (temp instanceof IGate)){
                
            objingGate = Modelio.getInstance().getModelingSession().getModel().createGate();
                
            String name = receivedGate.getName();
            if (EcoreModelNavigation.isNotNull(name))
                objingGate.setName(name);
                
            TotalImportMap.getInstance().put(receivedGate, objingGate);
                
            ReverseProperties revProp = ReverseProperties.getInstance();
            Gate referedGate = EcoreModelNavigation.getReferedGate(receivedGate);
            if (referedGate != null
                    && !EcoreModelNavigation.isFormalGate(receivedGate)
                    && revProp.isMappedElement(referedGate)) {
                IGate objingFormalGate = (IGate) revProp
                .getMappedElement(referedGate);
                objingGate.setFormal(objingFormalGate);
            }
                
            Element owner = receivedGate.getOwner();
            IModelElement objingOwner = (IModelElement) revProp
            .getMappedElement(owner);
                
                
                
            try {
                if (objingOwner instanceof IPartDecomposition) {
                    ((IPartDecomposition) objingOwner).insert(objingGate,
                            objingSendingFragment, on);
                } else if (objingOwner instanceof IInteractionUse) {
                    ((IInteractionUse) objingOwner).insert(objingGate,
                            objingSendingFragment, on);
                    //                    revProp.addInteractionUse((IInteractionUse)objingOwner);
                
                } else if (objingEnclosing instanceof IInteraction) {
                    ((IInteraction) objingEnclosing).insert(objingGate,
                            objingSendingFragment, on);
                } else if (objingEnclosing instanceof IInteractionOperand) {
                    ((IInteraction) objingEnclosing).insert(objingGate,
                            objingSendingFragment, on);
                }
            } catch (InvalidInteractionManipulationException e) {
               e.printStackTrace(Modelio.err);
                PartialImportMap.getInstance().remove(receivedGate);
                TotalImportMap.getInstance().remove(receivedGate);
                Modelio.getInstance().getModelingSession().getModel()
                .deleteElement(objingGate);
                objingGate = null;
            }
        }else{
            objingGate = (IGate) temp;
        }
        return objingGate;
    }

    @objid ("0d468e7b-4583-11e0-b54c-00137279a832")
    private void insertMessage(IModelElement enclosing, IMessage message, IMessageEnd objingSendingEnd, IMessageEnd objingReceivingEnd) {
        try {
            if (enclosing instanceof IInteraction)
                ((IInteraction) enclosing).insert(message, objingSendingEnd,
                        objingReceivingEnd);
            else if (enclosing instanceof IInteractionOperand)
                ((IInteractionOperand) enclosing).insert(message,
                        objingSendingEnd, objingReceivingEnd);
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(message);
        }
    }

    @objid ("0d468e81-4583-11e0-b54c-00137279a832")
    private void insertMessageSentByGate(IMessage message, MessageOccurrenceSpecification receivedMOS, Gate sentGate) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        // Gets ecore and objing lifelines:
        EList<?> coveredLL = null;
        if (receivedMOS != null)
            coveredLL = receivedMOS.getCovereds();
                
        ILifeline objingSendingLL = null;
                
        if (coveredLL != null && coveredLL.size() > 0) {
            Lifeline ecoreSendingLL = (Lifeline) coveredLL.get(0);
            objingSendingLL = (ILifeline) revProp
            .getMappedElement(ecoreSendingLL);
        }
                
        // Gets the enclosing fragment (the Interaction or an
        // InteractionOperand):
        InteractionFragment ecoreEnclosing = EcoreModelNavigation
        .getEnclosing(ecoreElement);
        Object objingEnclosing = revProp.getMappedElement(ecoreEnclosing);
                
        // Gets the previous fragment in the diagram (before the sending
        // fragment of the message):
        InteractionFragment ecorePrevious = null;
        if (ecoreEnclosing instanceof Interaction)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInInteraction(ecoreElement);
        else if (ecoreEnclosing instanceof InteractionOperand)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInOperand(ecoreElement);
                
        IInteractionFragment objingPrevious = null;
        ObRelativeTimeEnum relativeTime = ObRelativeTimeEnum.AFTER;
        if (ecorePrevious != null)
            objingPrevious = (IInteractionFragment) revProp
            .getMappedElement(ecorePrevious);
                
        // If previous is the owner InteractionUse:
        if (ecorePrevious instanceof InteractionUse
                && ((InteractionUse) ecorePrevious).getActualGates().contains(
                        sentGate))
            relativeTime = ObRelativeTimeEnum.DURING;
                
        // As we are in case of a message sent by a gate, it can't be a
        // "Found" message.
        if (kindOfMessage == ObMessageKindEnum.LOST_KIND)
            insertLostMsgAndGate(objingEnclosing, message, sentGate,
                    objingPrevious, relativeTime);
        else
            insertCompleteMsgSentByGate(objingEnclosing, receivedMOS,
                    objingSendingLL, objingPrevious, relativeTime, message,
                    sentGate);
    }

    @objid ("0d468e86-4583-11e0-b54c-00137279a832")
    private void insertCompleteMsgSentByGate(Object objingEnclosing, MessageOccurrenceSpecification receivedMOS, ILifeline objingSendingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime, IMessage message, Gate sentGate) {
        // Creates and inserts the sending end of the message (the Gate):
        IGate objingSendingGate = createAndInsertGate(objingEnclosing,
                sentGate, objingPrevious, relativeTime);
                
        // Creates and inserts the receiving end of the message:
        IMessageEnd objingReceivingEnd = null;
        if (objingSendingGate != null) {
            objingReceivingEnd = createAndInsertMOS(objingEnclosing,
                    receivedMOS, objingSendingLL, objingSendingGate,
                    ObRelativeTimeEnum.ON);
        }
                
        // Inserts the message in the diagram:
        if (objingSendingGate != null && objingReceivingEnd != null) {
            if (objingEnclosing instanceof IInteraction) {
                insertMessage((IInteraction) objingEnclosing, message,
                        objingSendingGate, objingReceivingEnd);
            } else if (objingEnclosing instanceof IInteractionOperand) {
                insertMessage((IInteractionOperand) objingEnclosing, message,
                        objingSendingGate, objingReceivingEnd);
            }
        }
    }

    @objid ("0d48f0d6-4583-11e0-b54c-00137279a832")
    private void insertMessageConnectedToLifelines(IMessage message) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        MessageOccurrenceSpecification ecoreReceivedMOS = (MessageOccurrenceSpecification) ecoreReceived;
        MessageOccurrenceSpecification ecoreSentMOS = (MessageOccurrenceSpecification) ecoreSent;
                
        // Gets ecore and objing lifelines:
        EList<?> receivingLL = null;
        if (ecoreReceivedMOS != null)
            receivingLL = ecoreReceivedMOS.getCovereds();
                
        EList<?> sendingLL = null;
        if (ecoreSentMOS != null)
            sendingLL = ecoreSentMOS.getCovereds();
                
        ILifeline objingReceivingLL = null;
        ILifeline objingSendingLL = null;
                
        if (receivingLL != null && receivingLL.size() > 0) {// else: case of a
            // "found" message
            Lifeline ecoreReceivingLL = (Lifeline) receivingLL.get(0);
            objingReceivingLL = (ILifeline) revProp
            .getMappedElement(ecoreReceivingLL);
        }
                
        if (sendingLL != null && sendingLL.size() > 0) {// else: case of a
            // "lost" message
            Lifeline ecoreSendingLL = (Lifeline) sendingLL.get(0);
            objingSendingLL = (ILifeline) revProp
            .getMappedElement(ecoreSendingLL);
        }
                
        // Gets the previous fragment in the diagram (before the sending
        // fragment of the message):
        InteractionFragment ecorePrevious = null;
        MessageOccurrenceSpecification where = null;
        if (ecoreSentMOS != null)
            where = ecoreSentMOS;
        else if (ecoreReceivedMOS != null)
            where = ecoreReceivedMOS;
                
        // Gets the enclosing fragment (the Interaction or an
        // InteractionOperand):
        InteractionFragment ecoreEnclosing = EcoreModelNavigation
        .getEnclosing(where);
        Object objingEnclosing = revProp.getMappedElement(ecoreEnclosing);
                
        if (ecoreEnclosing instanceof Interaction)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInInteraction(where);
        else if (ecoreEnclosing instanceof InteractionOperand)
            ecorePrevious = EcoreModelNavigation
            .getPreviousFragmentInOperand(where);
                
        IInteractionFragment objingPrevious = null;
        if (ecorePrevious != null){
            IElement temp = (IElement) revProp.getMappedElement(ecorePrevious);
            if (temp instanceof IInteractionFragment){
                objingPrevious = (IInteractionFragment) temp;
            }
                
        }
                
        BehaviorExecutionSpecification behavior = EcoreModelNavigation.getFinalExecutionSpecification((MessageOccurrenceSpecification)ecoreSent);
                
        // Inserts the message in the diagram:
        if (objingEnclosing instanceof IInteraction) {
                
            if (behavior != null){
                objingPrevious = ((IExecutionSpecification) revProp.getMappedElement(behavior)).getFinish();
                insertMsgOnLLFromInteraction((IInteraction) objingEnclosing,
                        message, objingSendingLL, objingReceivingLL,
                        objingPrevious, ObRelativeTimeEnum.DURING);
                
            }else{
                insertMsgOnLLFromInteraction((IInteraction) objingEnclosing,
                        message, objingSendingLL, objingReceivingLL,
                        objingPrevious, ObRelativeTimeEnum.AFTER);
            }
                
        } else if (objingEnclosing instanceof IInteractionOperand) {
            insertMsgOnLLFromInteractionOperand(
                    (IInteractionOperand) objingEnclosing, message,
                    objingSendingLL, objingReceivingLL, objingPrevious,
                    ObRelativeTimeEnum.AFTER);
        }
    }

    @objid ("0d48f0d9-4583-11e0-b54c-00137279a832")
    private void insertMsgOnLLFromInteraction(IInteraction interaction, IMessage message, ILifeline objingSendingLL, ILifeline objingReceivingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        try {
            String name = null;
            if (kindOfMessage == ObMessageKindEnum.FOUND_KIND) {
                
                MessageEnd ecoreMsgEnd = ecoreElement.getReceiveEvent();
                
                if (ecoreMsgEnd != null)
                    name = ecoreMsgEnd.getName();
                
                IMessageEnd msgEnd = createAndInsertMessageEndInInteraction(
                        interaction, name, objingReceivingLL, objingPrevious, relativeTime);
                
                interaction.insert(message, null,msgEnd);
                
            } else if (kindOfMessage == ObMessageKindEnum.LOST_KIND) {
                
                MessageEnd ecoreMsgEnd = ecoreElement.getSendEvent();
                
                if (ecoreMsgEnd != null)
                    name = ecoreMsgEnd.getName();
                
                IMessageEnd msgEnd = createAndInsertMessageEndInInteraction(
                        interaction, ecoreElement.getSendEvent().getName(),
                        objingSendingLL, objingPrevious, relativeTime);
                
                interaction.insert(message, msgEnd, null);
            } else {
                
                interaction.insert(message, objingSendingLL, objingReceivingLL,
                        objingPrevious, relativeTime);
                
                if (ecoreElement.getMessageSort().equals(MessageSort.SYNCH_CALL_LITERAL)) {
                    setPropertiesForSynchCall(message);
                }else if (ecoreElement.getMessageSort().equals(MessageSort.ASYNCH_CALL_LITERAL)){
                    setPropertiesForASynchCall(message);
                }else if (ecoreElement.getMessageSort().equals(MessageSort.CREATE_MESSAGE_LITERAL)){
                    setPropertiesForACreateMessage(message);
                }
                
                
            }
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(message);
                
        }
    }

    @objid ("0d48f0e1-4583-11e0-b54c-00137279a832")
    private void setPropertiesForASynchCall(IMessage message) {
        ILifeline lifeline = message.getReceiveEvent().getCovered().get(0);
        ObList<IInteractionFragment> fragmentList = lifeline.getCoveredBy();
                
        for (IInteractionFragment fragment : fragmentList){
            if (fragment instanceof IExecutionSpecification){
                IExecutionSpecification execution = (IExecutionSpecification) fragment;
                IExecutionOccurenceSpecification start = execution.getStart();
                IMessageEnd receive = message.getReceiveEvent();
                if ((start != null) && (receive != null) && start.equals(receive)){
                
                    BehaviorExecutionSpecification ecorebehavior = null;
                
                    Lifeline ecoreLifeline =  (Lifeline)((MessageOccurrenceSpecification)ecoreElement.getReceiveEvent()).getCovereds().get(0);
                
                    for (Object ecoreFragment : ecoreLifeline.getCoveredBys()){
                        if (ecoreFragment instanceof BehaviorExecutionSpecification){
                            BehaviorExecutionSpecification temp = (BehaviorExecutionSpecification) ecoreFragment;
                            if (temp.getStart().equals(ecoreElement.getReceiveEvent()))
                                ecorebehavior = temp;
                        }
                    }
                    if (ecorebehavior != null){
                        BehaviorExecutionSpecificationImportMapper importmapper = new BehaviorExecutionSpecificationImportMapper(ecorebehavior);
                        importmapper.setName(execution);
                        importmapper.setStartAndEndElement(execution);
                
                        ecorebehavior.getFinish();
                
                    }
                
                }
            }
        }
    }

    @objid ("0d4b532c-4583-11e0-b54c-00137279a832")
    private void setPropertiesForSynchCall(IMessage message) {
        ILifeline lifeline = message.getReceiveEvent().getCovered().get(0);
        ObList<IInteractionFragment> fragmentList = lifeline.getCoveredBy();
                
        for (IInteractionFragment fragment : fragmentList){
            if (fragment instanceof IExecutionSpecification){
                IExecutionSpecification execution = (IExecutionSpecification) fragment;
                IExecutionOccurenceSpecification start = execution.getStart();
                IMessageEnd receive = message.getReceiveEvent();
                if ((start != null) && (receive != null) && start.equals(receive)){
                
                    BehaviorExecutionSpecification ecorebehavior = null;
                
                    Lifeline ecoreLifeline =  (Lifeline)((MessageOccurrenceSpecification)ecoreElement.getReceiveEvent()).getCovereds().get(0);
                
                    for (Object ecoreFragment : ecoreLifeline.getCoveredBys()){
                        if (ecoreFragment instanceof BehaviorExecutionSpecification){
                            BehaviorExecutionSpecification temp = (BehaviorExecutionSpecification) ecoreFragment;
                            if (temp.getStart().equals(ecoreElement.getReceiveEvent()))
                                ecorebehavior = temp;
                        }
                    }
                    if (ecorebehavior != null){
                        BehaviorExecutionSpecificationImportMapper importmapper = new BehaviorExecutionSpecificationImportMapper(ecorebehavior);
                        importmapper.setName(execution);
                        importmapper.setStartAndEndElement(execution);
                
                        OccurrenceSpecification finish = ecorebehavior.getFinish();
                        if (finish != null && finish instanceof MessageOccurrenceSpecification){
                            Message ecoreReply = ((MessageOccurrenceSpecification)finish).getMessage(); 
                            IMessage reply = execution.getFinish().getSentMessage();
                            MessageImportMapper messImportMapper = new MessageImportMapper(ecoreReply);
                            messImportMapper.setName( reply, ecoreReply);
                            messImportMapper.setSignature( reply, ecoreReply);
                
                        }
                
                    }
                
                }
            }
        }
    }

    @objid ("0d4b532f-4583-11e0-b54c-00137279a832")
    private void setName(IMessage message, Message ecoreMessage) {
        String name = ecoreMessage.getName();
        if (EcoreModelNavigation.isNotNull(name))
            message.setName(name);
        else 
            message.setName("");
    }

    @objid ("0d4b5333-4583-11e0-b54c-00137279a832")
    private void setSignature(IMessage message, Message ecoreMesasge) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        NamedElement signature = ecoreMesasge.getSignature();
                
        if (signature != null) {
            IModelElement objingSignature = (IModelElement) revProp
            .getMappedElement(signature);
            if (objingSignature instanceof ISignal)
                message.setSignalSignature((ISignal) objingSignature);
            else if ((objingSignature instanceof IOperation) &&
                    message.getSortOfMessage().equals(ObMessageSortEnum.SYNC_CALL)){
                message.setInvoked((IOperation) objingSignature);
            }
        }
    }

    @objid ("0d4b5337-4583-11e0-b54c-00137279a832")
    private IMessageEnd createAndInsertMessageEndInInteraction(IInteraction interaction, String name, ILifeline objingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) throws InvalidInteractionManipulationException {
        IExecutionOccurenceSpecification end = Modelio.getInstance()
        .getModelingSession().getModel()
        .createExecutionOccurenceSpecification();
        if (name != null)
            end.setName(name);
        interaction.insert(end, objingLL, objingPrevious, relativeTime);
        return end;
    }

    @objid ("0d4db587-4583-11e0-b54c-00137279a832")
    private void insertMsgOnLLFromInteractionOperand(IInteractionOperand operand, IMessage message, ILifeline objingSendingLL, ILifeline objingReceivingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        try {
            if (objingPrevious == null) {
                objingPrevious = operand;
                relativeTime = ObRelativeTimeEnum.DURING;
            }
                
            if (kindOfMessage == ObMessageKindEnum.FOUND_KIND) {
                
                MessageEnd ecoreMsgEnd = ecoreElement.getReceiveEvent();
                
                if (ecoreMsgEnd != null)
                    ecoreMsgEnd.getName();
                
                IMessageEnd msgEnd = createAndInsertMessageEndInOperand(
                        operand, ecoreElement.getReceiveEvent().getName(),
                        objingReceivingLL, objingPrevious, relativeTime);
                operand.insert(message, null, msgEnd);
            } else if (kindOfMessage == ObMessageKindEnum.LOST_KIND) {
                IMessageEnd msgEnd = createAndInsertMessageEndInOperand(
                        operand, ecoreElement.getSendEvent().getName(),
                        objingSendingLL, objingPrevious, relativeTime);
                operand.insert(message, msgEnd, null);
            } else {
                operand.insert(message, objingSendingLL, objingReceivingLL,
                        objingPrevious, relativeTime);
                
                if (ecoreElement.getMessageSort() == MessageSort.SYNCH_CALL_LITERAL) {
                    setPropertiesForSynchCall(message);
                }else if (ecoreElement.getMessageSort() == MessageSort.ASYNCH_CALL_LITERAL){
                    setPropertiesForASynchCall(message);
                }else if (ecoreElement.getMessageSort().equals(MessageSort.CREATE_MESSAGE_LITERAL)){
                    setPropertiesForACreateMessage(message);
                }
                
            }
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(message);
        }
    }

    @objid ("0d4db58f-4583-11e0-b54c-00137279a832")
    private IMessageEnd createAndInsertMessageEndInOperand(IInteractionOperand operand, String name, ILifeline objingLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) throws InvalidInteractionManipulationException {
        IExecutionOccurenceSpecification end = Modelio.getInstance()
        .getModelingSession().getModel()
        .createExecutionOccurenceSpecification();
        if (name != null)
            end.setName(name);
        operand.insert(end, objingLL, objingPrevious, relativeTime);
        return end;
    }

    @objid ("0d5017e3-4583-11e0-b54c-00137279a832")
    private void setPropertiesForACreateMessage(IMessage message) {
        ILifeline lifeline = message.getReceiveEvent().getCovered().get(0);
        ObList<IInteractionFragment> fragmentList = lifeline.getCoveredBy();
                
        for (IInteractionFragment fragment : fragmentList){
            if (fragment instanceof IExecutionSpecification){
                IExecutionSpecification execution = (IExecutionSpecification) fragment;
                IExecutionOccurenceSpecification start = execution.getStart();
                IMessageEnd receive = message.getReceiveEvent();
                if ((start != null) && (receive != null) && start.equals(receive)){
                
                    BehaviorExecutionSpecification ecorebehavior = null;
                
                    Lifeline ecoreLifeline =  (Lifeline)((MessageOccurrenceSpecification)ecoreElement.getReceiveEvent()).getCovereds().get(0);
                
                    for (Object ecoreFragment : ecoreLifeline.getCoveredBys()){
                        if (ecoreFragment instanceof BehaviorExecutionSpecification){
                            BehaviorExecutionSpecification temp = (BehaviorExecutionSpecification) ecoreFragment;
                            if (temp.getStart().equals(ecoreElement.getReceiveEvent()))
                                ecorebehavior = temp;
                        }
                    }
                    if (ecorebehavior != null){
                        BehaviorExecutionSpecificationImportMapper importmapper = new BehaviorExecutionSpecificationImportMapper(ecorebehavior);
                        importmapper.setName(execution);
                        importmapper.setStartAndEndElement(execution);
                    }
                
                }
            }
        }
    }

}
