/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.PartDecomposition;

@objid ("0d5c03a8-4583-11e0-b54c-00137279a832")
public class PartDecompositionImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d5c03a9-4583-11e0-b54c-00137279a832")
    private PartDecomposition ecoreElement;

    @objid ("0d5c03aa-4583-11e0-b54c-00137279a832")
    private Interaction ecoreInteraction;


    @objid ("0d5c03ab-4583-11e0-b54c-00137279a832")
    public Object createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
                .createPartDecomposition();
    }

    @objid ("0d5c03af-4583-11e0-b54c-00137279a832")
    public PartDecompositionImportMapper(PartDecomposition element) {
        super(element);
        ecoreElement = element;
        ecoreInteraction = ecoreElement.getEnclosingInteraction();
    }

    @objid ("0d5e65fd-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
         result.add("PartDecomposition");
        return result;
    }

    @objid ("0d5e6603-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        // Sets its decomposed Lifeline:
        for (Object ecoreLifeline : ecoreInteraction.getLifelines()) {
            Lifeline ecoreLL = (Lifeline) ecoreLifeline;
            if (ecoreElement.equals(ecoreLL.getDecomposedAs())) {
                Object objingLL = revProp.getMappedElement(ecoreLL);
                if ((objingLL instanceof ILifeline ) && (objingLL != null))
                    ((IPartDecomposition) objingElt)
                            .setDecomposed((ILifeline) objingLL);
                break;
            }
        }
    }

    @objid ("0d5e6606-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElts) {
    }

    @objid ("0d60c856-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof IPartDecomposition) {
            setName((IPartDecomposition) objingElt);
            setReferedInteraction((IPartDecomposition) objingElt);
            // Insertion in diagram done when inserting the decomposed Lifeline.
        }
    }

    @objid ("0d60c859-4583-11e0-b54c-00137279a832")
    private void setReferedInteraction(IPartDecomposition objingElt) {
        ReverseProperties revProp = ReverseProperties.getInstance();
                
        // Sets the refered Interaction:
        Interaction ecoreReferedInteraction = ecoreElement.getRefersTo();
        if (ecoreReferedInteraction != null) {
            IInteraction objingReferedInteraction = (IInteraction) revProp
                    .getMappedElement(ecoreReferedInteraction);
            if (objingReferedInteraction != null) {
                ((IPartDecomposition) objingElt)
                        .setRefersTo(objingReferedInteraction);
            }
        }
    }

}
