/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.ecore.sequence;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObRelativeTimeEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateInvariant;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IConstraint;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidInteractionManipulationException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.StateInvariant;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0d632ab0-4583-11e0-b54c-00137279a832")
public class StateInvariantImportMapper extends SequenceElementImportMapper implements IEElement {
    @objid ("0d632ab1-4583-11e0-b54c-00137279a832")
    private StateInvariant ecoreElement;


    @objid ("0d632ab2-4583-11e0-b54c-00137279a832")
    public IElement createObjingElt() {
        return Modelio.getInstance().getModelingSession().getModel()
        .createStateInvariant();
    }

    @objid ("0d632ab6-4583-11e0-b54c-00137279a832")
    public StateInvariantImportMapper(StateInvariant element) {
        super(element);
        ecoreElement = element;
    }

    @objid ("0d632ab9-4583-11e0-b54c-00137279a832")
    public List<String> getObjingClassName() {
        List<String> result = new ArrayList<String>();
        result.add("StateInvariant");
        return result;
    }

    @objid ("0d658d0e-4583-11e0-b54c-00137279a832")
    public void attach(IElement objingElt) {
    }

    @objid ("0d658d11-4583-11e0-b54c-00137279a832")
    public void attach(List<Object> objingElt) {
    }

    @objid ("0d658d16-4583-11e0-b54c-00137279a832")
    public void setProperties(IElement objingElt) {
        if (objingElt instanceof IStateInvariant) {
            setName((IStateInvariant) objingElt);
            setBody((IStateInvariant) objingElt);
            insertInDiagram((IStateInvariant) objingElt);
        }
    }

    @objid ("0d67ef64-4583-11e0-b54c-00137279a832")
    private void setBody(IStateInvariant invariant) {
        // In UML2, the body of the StateInvariant (SI) is stored in a
        // constraint.
        // In Modelio, in import, we won't create this constraint.
        // => If the Constraint in objing has already been created, we delete
        // it, and, in
        // the HashMap, we associate the ecore constraint (key) with the objing
        // SI (value).
        ReverseProperties revProp = ReverseProperties.getInstance();
        Constraint ecoreConstraint = ecoreElement.getInvariant();
        Object objConstraint = revProp.getMappedElement(ecoreConstraint);
        if (objConstraint instanceof IConstraint) {
            PartialImportMap.getInstance().remove(ecoreConstraint);
            TotalImportMap.getInstance().remove(ecoreConstraint);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement((IConstraint) objConstraint);
        }
        TotalImportMap.getInstance().put(ecoreConstraint, invariant);
        
        ValueSpecification valueSpecif = ecoreConstraint.getSpecification();
        if (valueSpecif != null) {
            String body = valueSpecif.stringValue();
            if (body != null)
                invariant.setBody(body);
        }
    }

    @objid ("0d67ef67-4583-11e0-b54c-00137279a832")
    private void insertInDiagram(IStateInvariant invariant) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Lifeline eCoveredLL = (Lifeline) ecoreElement.getCovereds().get(0);
        
        if (eCoveredLL != null) {
            ILifeline oCoveredLL = (ILifeline) revProp
            .getMappedElement(eCoveredLL);
            if (oCoveredLL != null) {
                Interaction ecoreInt = eCoveredLL.getInteraction();
                IInteraction objingInt = (IInteraction) revProp
                .getMappedElement(ecoreInt);
        
                InteractionFragment ecoreEnclosing = EcoreModelNavigation
                .getEnclosing(ecoreElement);
        
                if (ecoreEnclosing instanceof Interaction) {
                    InteractionFragment ecorePrevious = EcoreModelNavigation
                    .getPreviousFragmentInInteraction(ecoreElement);
                    IInteractionFragment objingPrevious = null;
                    if (ecorePrevious != null)
                        objingPrevious = (IInteractionFragment) revProp
                        .getMappedElement(ecorePrevious);
        
                    insertSIInInteraction(objingInt, invariant, oCoveredLL,
                                          objingPrevious, ObRelativeTimeEnum.AFTER);
                } else if (ecoreEnclosing instanceof InteractionOperand) {
                    IInteractionOperand objingEnclosing = (IInteractionOperand) revProp
                    .getMappedElement(ecoreEnclosing);
                    InteractionFragment ecorePrevious = EcoreModelNavigation
                    .getPreviousFragmentInOperand(ecoreElement);
                    IInteractionFragment objingPrevious = null;
                    if (ecorePrevious != null)
                        objingPrevious = (IInteractionFragment) revProp
                        .getMappedElement(ecorePrevious);
        
                    if (objingPrevious == null)
                        insertSIInIO(objingEnclosing, invariant, oCoveredLL,
                                     objingEnclosing, ObRelativeTimeEnum.DURING);
                    else
                        insertSIInIO(objingEnclosing, invariant, oCoveredLL,
                                     objingPrevious, ObRelativeTimeEnum.AFTER);
                } else
                    throw new NotFoundException("StateInvariants enclosed in "
                                                + ecoreEnclosing.getClass().getSimpleName()
                                                + " are not supported.");
            }
        }
    }

    @objid ("0d67ef6a-4583-11e0-b54c-00137279a832")
    private void insertSIInInteraction(IInteraction objingInt, IStateInvariant invariant, ILifeline coveredLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        try {
            objingInt
            .insert(invariant, coveredLL, objingPrevious, relativeTime);
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(invariant);
        } catch (Exception e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(invariant);
        }
    }

    @objid ("0d67ef71-4583-11e0-b54c-00137279a832")
    private void insertSIInIO(IInteractionOperand enclosingOperand, IStateInvariant invariant, ILifeline coveredLL, IInteractionFragment objingPrevious, ObRelativeTimeEnum relativeTime) {
        try {
            enclosingOperand.insert(invariant, coveredLL, objingPrevious,
                                    relativeTime);
        } catch (InvalidInteractionManipulationException e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(invariant);
        } catch (Exception e) {
           e.printStackTrace(Modelio.err);
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().remove(ecoreElement);
            Modelio.getInstance().getModelingSession().getModel()
            .deleteElement(invariant);
        }
    }

}
