/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IAbstraction;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import org.eclipse.uml2.uml.Abstraction;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class is in charge of the Abstraction export
 * @author ebrosse
 */
@objid ("0d6a51ce-4583-11e0-b54c-00137279a832")
public class OAbstraction extends OElement implements IOElement {
    @objid ("0d6a51cf-4583-11e0-b54c-00137279a832")
    private IAbstraction objingElement;


    @Override
    @objid ("0d6a51d0-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createAbstraction();
    }

    /**
     * Default constructor
     */
    @objid ("0d6cb419-4583-11e0-b54c-00137279a832")
    public OAbstraction() {
    }

    /**
     * A constructor with the exported Abstraction
     * @param abstraction The exported Abstraction
     */
    @objid ("0d6cb41b-4583-11e0-b54c-00137279a832")
    public OAbstraction(final IAbstraction abstraction) {
        super(abstraction);
        this.objingElement = abstraction;
    }

    @Override
    @objid ("0d6cb41e-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // Done on Dependency when calling the super() in the visitor.
    }

    @Override
    @objid ("0d6cb422-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setAbstractionMapping((Abstraction) ecoreElt);
    }

    @objid ("0d6f1673-4583-11e0-b54c-00137279a832")
    private void setAbstractionMapping(Abstraction ecoreAbstraction) {
        OpaqueExpression expr = UMLFactory.eINSTANCE.createOpaqueExpression();
        String abstractionName = this.objingElement.getName();
        String mappingName = "";
        
        if (!"".equals(abstractionName))
            mappingName = abstractionName + "_";
        
        mappingName += "Mapping";
        
        expr.setName(mappingName);
        expr.getBodies().add(this.objingElement.getMapping());
        ecoreAbstraction.setMapping(expr);
    }

}
