/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IAcceptSignalAction;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.Trigger;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0d7fc6e8-4583-11e0-b54c-00137279a832")
public class OAcceptSignalAction extends OActivityNode implements IOElement {
    @objid ("0d7fc6e9-4583-11e0-b54c-00137279a832")
    private IAcceptSignalAction objingElement;

    @objid ("0d7fc6eb-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0d7fc6ec-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createAcceptEventAction();
    }

    @objid ("0d7fc6f0-4583-11e0-b54c-00137279a832")
    public OAcceptSignalAction() {
    }

    @objid ("0d7fc6f2-4583-11e0-b54c-00137279a832")
    public OAcceptSignalAction(IAcceptSignalAction element) {
        super(element);
        objingElement = element;
    }

    @objid ("0d822946-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        super.attach(ecoreElt, ownerVisitor);
    }

    @objid ("0d82294a-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setAccepted((AcceptEventAction) ecoreElt);
        setSignal((AcceptEventAction) ecoreElt);
    }

    @objid ("0d848baa-4583-11e0-b54c-00137279a832")
    private void setAccepted(AcceptEventAction action) {
        for (ISignal objingSignal : objingElement.getAccepted()) {
            Element ecoreSignal = genProp.getMappedElement(objingSignal);
            if (ecoreSignal instanceof Signal) {
                Trigger trigger = UMLFactory.eINSTANCE.createTrigger();
                SignalEvent signalEvent = UMLFactory.eINSTANCE
                .createSignalEvent();
                action.getTriggers().add(trigger);
                trigger.setEvent(signalEvent);
                signalEvent.setSignal((Signal) ecoreSignal);
        
                // Attach the Event to the model via composition relation:
                    Package nearestPkg = action.getNearestPackage();
                    nearestPkg.getPackagedElements().add(signalEvent);
            }
        }
    }

    @objid ("66490508-1a76-11e1-8103-0027103f347d")
    private void setSignal(final AcceptEventAction ecoreElt) {
        if (genProp.isRoundtripEnabled()){
            ObjingEAnnotation.setSignal(ecoreElt, "signal");
        }
    }

}
