/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IActivty
 * @author ebrosse
 */
@objid ("0d8e1504-4583-11e0-b54c-00137279a832")
public class OActivity extends OModelElement {
    @objid ("0d8e1505-4583-11e0-b54c-00137279a832")
    private IActivity objingElement;


    @Override
    @objid ("0d92d9b8-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createActivity();
    }

    /**
     * Contructor of the OActivity.
     * It takes the exported Modelio Activity as parameter
     * @param element : the exported Modelio Activity
     */
    @objid ("0d92d9be-4583-11e0-b54c-00137279a832")
    public OActivity(final IActivity element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0d953c17-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IElement objingOwner = ownerVisitor.getCompositionOwner(this.objingElement);
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
               
        if (ecoreOwner != null) {
            if (objingOwner instanceof INameSpace
                    && ecoreOwner instanceof Namespace)
                 attachToNameSpace(ecoreElt, (Namespace) ecoreOwner);
            else if (objingOwner instanceof IOperation)
                attachToOperation(ecoreElt, (IOperation) objingOwner);         
        }
    }

    @Override
    @objid ("0d953c1b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setReadOnly((Activity) ecoreElt);
        setSingleExecution((Activity) ecoreElt);
        setReentrant((Activity) ecoreElt);
    }

    @objid ("0d953c1e-4583-11e0-b54c-00137279a832")
    private void attachToNameSpace(Element ecoreElt, Namespace ecoreOwner) {
        String ownerId = ObjingEAnnotation.getOwner(ecoreElt);
        if ((ownerId == null) || (ownerId.equals(""))){
            
        if (ecoreOwner instanceof Package) {
            Package ownerIsPkg = (Package) ecoreOwner;
            ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
        } else if (ecoreOwner instanceof Component) {
            Component ownerIsCmpnt = (Component) ecoreOwner;
            ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
        } else if (ecoreOwner instanceof Class) {
            Class ownerIsClass = (Class) ecoreOwner;
            ownerIsClass.getOwnedBehaviors().add((Activity)ecoreElt);
        } else if (ecoreOwner instanceof Interface) {
            Interface ownerIsItf = (Interface) ecoreOwner;
            ownerIsItf.getNestedClassifiers().add((Classifier)ecoreElt);
        } else if (ecoreOwner instanceof BehavioredClassifier) {
            BehavioredClassifier ownerIsBehavioredClassifier = (BehavioredClassifier) ecoreOwner;
            ownerIsBehavioredClassifier.getOwnedBehaviors().add((Behavior)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
        }else{
            ObjingEAnnotation.deleteOwner(ecoreElt);
        }
    }

    @objid ("0d953c22-4583-11e0-b54c-00137279a832")
    private void attachToOperation(Element ecoreElt, IOperation objingOwner) {
        IClassifier objingOpOwner = objingOwner.getOwner();
        String errorMessage =   XMI.I18N.getMessage("logFile.warning.moving.activity",
                                                    this.objingElement.getName(), objingOwner.getName(),
                                                    objingOpOwner.getName());
        XMILogs.getInstance().writelnInLog(errorMessage
              );
        
                
        GenerationProperties genProp = GenerationProperties.getInstance();
        genProp.getReportModel().addWarning( errorMessage, this.objingElement);
        
        Element ecoreOpOwner = genProp.getMappedElement(objingOpOwner);
                
        if (ecoreOpOwner instanceof Namespace) {
            attachToNameSpace(ecoreElt, (Namespace) ecoreOpOwner);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOpOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0d979e71-4583-11e0-b54c-00137279a832")
    private void setReadOnly(Activity activity) {
        activity.setIsReadOnly(this.objingElement.isReadOnly());
    }

    @objid ("0d979e74-4583-11e0-b54c-00137279a832")
    private void setSingleExecution(Activity activity) {
        activity.setIsSingleExecution(this.objingElement.isSingleExecution());
    }

    @objid ("0d979e77-4583-11e0-b54c-00137279a832")
    private void setReentrant(Activity activity) {
        activity.setIsReentrant(this.objingElement.isReentrant());
    }

}
