/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityGroup;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityPartition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityGroup;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.Element;

@objid ("66ad2328-1a76-11e1-8103-0027103f347d")
public class OActivityGroup extends OModelElement implements IOElement {
    @objid ("66ad4a3a-1a76-11e1-8103-0027103f347d")
    private IActivityGroup objingElement;


    @objid ("66ad714c-1a76-11e1-8103-0027103f347d")
    public Element createEcoreElt() {
        return null;
    }

    @objid ("66ad985f-1a76-11e1-8103-0027103f347d")
    public OActivityGroup() {
    }

    @objid ("66adbf71-1a76-11e1-8103-0027103f347d")
    public OActivityGroup(final IActivityPartition element) {
        super(element);
        objingElement = element;
    }

    @objid ("66ae0d96-1a76-11e1-8103-0027103f347d")
    public void attach(final Element ecoreElt, final CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (objingOwner instanceof IActivityPartition)
                attachToActivityPartition(ecoreElt, ecoreOwner);
            else if (objingOwner instanceof IActivity)
                attachToActivity(ecoreElt, ecoreOwner);
        }
    }

    @objid ("66ae82ca-1a76-11e1-8103-0027103f347d")
    public void setProperties(final Element ecoreElt) {
        super.setProperties(ecoreElt);
    }

    @objid ("66aed0ee-1a76-11e1-8103-0027103f347d")
    private void attachToActivityPartition(final Element ecoreElt, final Element ecoreOwner) {
        if (ecoreOwner instanceof ActivityPartition) {
            ActivityPartition owner = (ActivityPartition) ecoreOwner;
            owner.getSubpartitions().add((ActivityPartition)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("66af1f14-1a76-11e1-8103-0027103f347d")
    private void attachToActivity(final Element ecoreElt, final Element ecoreOwner) {
        if (ecoreOwner instanceof Activity) {
            Activity owner = (Activity) ecoreOwner;
            owner.getGroups().add((ActivityGroup)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

}
