/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityPartition;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IConditionalNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IStructuredActivityNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.StructuredActivityNode;

@objid ("66b0097e-1a76-11e1-8103-0027103f347d")
public class OActivityNode extends OModelElement implements IOElement {
    @objid ("66b0097f-1a76-11e1-8103-0027103f347d")
     CompositionOwnerVisitor ownerVisitor = null;


    @objid ("66b03090-1a76-11e1-8103-0027103f347d")
    public OActivityNode(final IActivityNode element) {
        super(element);
    }

    @objid ("66b07eb4-1a76-11e1-8103-0027103f347d")
    public OActivityNode() {
    }

    @Override
    @objid ("66b0ccd8-1a76-11e1-8103-0027103f347d")
    public void attach(final Element ecoreElt, final CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
        
        IElement objingOwner = ownerVisitor.getCompositionOwner(getObjingElement());
        Element ecoreOwner =  GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (objingOwner instanceof IStructuredActivityNode)
                attachToStructuredActivityNode(ecoreElt, ecoreOwner);
            else if (objingOwner instanceof IActivityPartition)
                attachToActivityPartition(ecoreElt, ecoreOwner);
            else if (objingOwner instanceof IActivity)
                attachToActivity(ecoreElt, ecoreOwner);
            else if (objingOwner instanceof IClause)
                attachToClause(ecoreElt, ecoreOwner, (IClause) objingOwner);
        }
    }

    @objid ("66b0f3ec-1a76-11e1-8103-0027103f347d")
    private void attachToStructuredActivityNode(final Element ecoreElt, final Element ecoreOwner) {
        if (ecoreOwner instanceof StructuredActivityNode) {
            StructuredActivityNode owner = (StructuredActivityNode) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("66b11afe-1a76-11e1-8103-0027103f347d")
    private void attachToActivityPartition(final Element ecoreElt, final Element ecoreOwner) {
        if (ecoreOwner instanceof ActivityPartition) {
            ActivityPartition owner = (ActivityPartition) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(getObjingElement(), IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity =  GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("66b14210-1a76-11e1-8103-0027103f347d")
    private void attachToActivity(final Element ecoreElt, final Element ecoreOwner) {
        if (ecoreOwner instanceof Activity) {
            Activity owner = (Activity) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("66b16922-1a76-11e1-8103-0027103f347d")
    private void attachToClause(final Element ecoreElt, final Element ecoreOwner, final IClause objingOwnerClause) {
        boolean isAttached = false;
        if (ecoreOwner instanceof Clause) {
            Clause owner = (Clause) ecoreOwner;
        
            // A CentralBufferNode can't have any Pin in Ijing => use of the
            // "body" role:
            if (ecoreElt instanceof ExecutableNode)
                owner.getBodies().add((ExecutableNode)ecoreElt);
        
            // Setting composition relation (in ConditionalNode):
            IConditionalNode objingConditional = objingOwnerClause.getOwner();
        
            if (objingConditional != null) {
                Element ecoreConditional =  GenerationProperties.getInstance()
                        .getMappedElement(objingConditional);
                if (ecoreConditional instanceof StructuredActivityNode) {
                    ((StructuredActivityNode) ecoreConditional).getNodes().add(
                                                                               (ActivityNode)ecoreElt);
                    isAttached = true;
                }
            }
        }
        
        if (!isAttached) {
            // Setting composition relation (in Activity):
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(getObjingElement(), IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity =  GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        }
    }

    @Override
    @objid ("66b19036-1a76-11e1-8103-0027103f347d")
    public Element createEcoreElt() {
        return null;
    }

    @Override
    @objid ("66b1b747-1a76-11e1-8103-0027103f347d")
    public void setProperties(final Element ecoreElt) {
        super.setProperties(ecoreElt);
    }

}
