/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.uml2.uml.Artifact;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Manifestation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0db8ff63-4583-11e0-b54c-00137279a832")
public class OArtifact extends ONameSpace implements IOElement {
    @objid ("0db8ff64-4583-11e0-b54c-00137279a832")
    private IArtifact objingElement;


    @objid ("0db8ff65-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (objingElement.isStereotyped(XMIStereotypes.UML2DEPLOYMENTSPECIFICATION)){
            return UMLFactory.eINSTANCE.createDeploymentSpecification();
        }else{
            return UMLFactory.eINSTANCE.createArtifact();
        }
    }

    @objid ("0dbb61b3-4583-11e0-b54c-00137279a832")
    public OArtifact() {
    }

    @objid ("0dbb61b5-4583-11e0-b54c-00137279a832")
    public OArtifact(IArtifact param) {
        super(param);
        objingElement = param;
    }

    @objid ("0dbb61be-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ((Classifier)ecoreElt).setOwningTemplateParameter(ownerIsTemplateParameter);
            } else if (ecoreOwner instanceof Artifact) {
                Artifact ownerIsArtifact = (Artifact) ecoreOwner;
                ownerIsArtifact.getNestedArtifacts().add((Artifact)ecoreElt);
               
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0dbdc40d-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setFileName((Artifact) ecoreElt);
        setUtilized((Artifact) ecoreElt);
    }

    @objid ("0dbdc416-4583-11e0-b54c-00137279a832")
    private void setFileName(Artifact artifact) {
        String fileName = objingElement.getFileName();
        if (!"".equals(fileName))
            artifact.setFileName(fileName);
    }

    @objid ("0dbdc419-4583-11e0-b54c-00137279a832")
    private void setUtilized(Artifact artifact) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        List<Manifestation> ecoreManifList = new ArrayList<Manifestation>();
        
        for (IManifestation objingManif : objingElement.getUtilized()) {
            Manifestation ecoreManif = (Manifestation) genProp
            .getMappedElement(objingManif);
        
            if (ecoreManif != null && !ecoreManifList.contains(ecoreManif)) {
                ecoreManifList.add(ecoreManif);
            }
        }
        
        if (ecoreManifList.size() > 0)
            artifact.getManifestations().addAll(ecoreManifList);
    }

}
