/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class manages the IAssociation elements
 * @author ebrosse
 */
@objid ("0dc02666-4583-11e0-b54c-00137279a832")
public class OAssociation extends OModelElement {
    @objid ("0dc02667-4583-11e0-b54c-00137279a832")
    private IAssociation objingElement = null;

    @objid ("0dc02668-4583-11e0-b54c-00137279a832")
    private boolean isClassAssociation = true;


    @Override
    @objid ("0dc02669-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        Element element = null;
        
        if (!this.isClassAssociation){
            if (this.objingElement.isStereotyped(XMIStereotypes.UML2COMMUNICATIONPATH)){
                element = UMLFactory.eINSTANCE.createCommunicationPath();
            }else if (this.objingElement.isStereotyped(XMIStereotypes.UML2EXTENSION)){
                element = UMLFactory.eINSTANCE.createExtension();
            }else{
                element = createEcoreAssociation();
            }
        }else
            // The Ecore AssociationClass can be created when parsing the
            // related Ijing ClassAssociation, Class
            // or current Association.
            element = getOrCreateEcoreAssociationClass();
        return element;
    }

    @objid ("0dc0266d-4583-11e0-b54c-00137279a832")
    private AssociationClass getOrCreateEcoreAssociationClass() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        // Gets or creates the Ecore AssociationClass:
        return  (AssociationClass) genProp
         .getMappedElement(this.objingElement);
    }

    @objid ("0dc02671-4583-11e0-b54c-00137279a832")
    private Element createEcoreAssociation() {
        return UMLFactory.eINSTANCE.createAssociation();
    }

    /**
     * Constructor of OAssociation.
     * it takes the exported Modelio Association as parameter
     * @param element : the exported Modelio Association
     */
    @objid ("0dc288c2-4583-11e0-b54c-00137279a832")
    public OAssociation(final IAssociation element) {
        super(element);
        this.objingElement = element;
        this.isClassAssociation = ObjingModelNavigation.isClassAssociation(this.objingElement);
    }

    @Override
    @objid ("0dc288cb-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ObjingModelNavigation.isNotOwnedByActor(this.objingElement)) {
        
            if (!this.isClassAssociation)
                linkEcoreAssociation((Association) ecoreElt);
        }else{
            XMILogs xmiLog = XMILogs.getInstance();
            String message = XMI.I18N.getMessage("logFile.warning.unexportedAssociationCauseOwnerTypeActor",
                                                 this.objingElement.getName());
            xmiLog.writelnInLog(message);
            GenerationProperties.getInstance().getReportModel().addError(message,  this.objingElement);
            
            ecoreElt.destroy();
        }
    }

    @Override
    @objid ("0dc4eb1c-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
    }

    @objid ("0dc4eb1f-4583-11e0-b54c-00137279a832")
    private void linkEcoreAssociation(Association ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        Classifier ecoreOwner = null;
        for (IAssociationEnd assocEnd : this.objingElement.getConnection()) {
            // Links the Association to the OwnerClassifier of one of the
            // Properties
            // (sets the owner of the Association):
            if (ecoreOwner == null) {
                ecoreOwner = (Classifier) genProp.getMappedElement(assocEnd
                        .getOwner());
                if (ecoreOwner != null) {
                    // Components and Nodes are handled as Class
                    // (they inherit from Class)
        
                    // Class, Component, Node
                    if (ecoreOwner instanceof Class) {
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else if (ecoreOwner instanceof Interface) {// Interface
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else if (ecoreOwner instanceof UseCase) {// UseCase
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else if (ecoreOwner instanceof Actor) {// Actor
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else if (ecoreOwner instanceof DataType) {// DataType
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else if (ecoreOwner instanceof Signal) {// Signal
                        EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                                .getOwner(), ecoreElt);
                    } else {
                        ecoreElt.destroy();
                        throw new NotFoundException("Owner Class ("
                                + ecoreOwner.getClass().getSimpleName()
                                + ") Not Found");
                    }
                }
            }
        }
    }

}
