/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.List;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IFeature;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ModelUtils;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class manages the export of IAssociationEnd elements
 * @author ebrosse
 */
@objid ("0dc74d74-4583-11e0-b54c-00137279a832")
public class OAssociationEnd extends OFeature {
    @objid ("0dc74d75-4583-11e0-b54c-00137279a832")
    private IAssociationEnd objingElement;

    @objid ("0dc74d76-4583-11e0-b54c-00137279a832")
    private boolean isLinkedToClassAssociation;


    @Override
    @objid ("0dc74d78-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IAssociation assoc = this.objingElement.getRelated();
        
        if (ObjingModelNavigation.isNotOwnedByActor(assoc)) {
            if (this.objingElement.isStereotyped(XMIStereotypes.UML2EXTENSIONEND)){
                return  UMLFactory.eINSTANCE.createExtensionEnd();
            }
            return UMLFactory.eINSTANCE.createProperty();
            
        }else{ 
            String message = XMI.I18N.getMessage("error.export.assocBetweenActors",
                                                assoc.getName());
            XMILogs.getInstance().writelnInLog(message);
        
            GenerationProperties genProp = GenerationProperties.getInstance();
            genProp.getReportModel().addError(message, assoc);
        }
        return null;
    }

    /**
     * Constructor of OAssociationEnd.
     * It takes the exported Modelio IAssociationEnd as parameter
     * @param element : the exported IAssociationEnd
     */
    @objid ("0dc74d7e-4583-11e0-b54c-00137279a832")
    public OAssociationEnd(final IAssociationEnd element) {
        super(element);
        this.objingElement = element;
        if (ObjingModelNavigation.isClassAssociation(this.objingElement))
            this.isLinkedToClassAssociation = true;
        else
            this. isLinkedToClassAssociation = false;
    }

    @Override
    @objid ("0dc9afd2-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (!this.isLinkedToClassAssociation)
            linkEcoreProperty((Property) ecoreElt);
        // If the Class representing the current ClassAssociation represents
        // other ClassAssociations, the ClassAssociation is not valid for export:
        else if (ObjingModelNavigation.isRepresentedByAUniqueClass(this.objingElement.getRelated()
                                                                   .getLinkToClass()))
            linkEcorePropertyLinkedToAC((Property) ecoreElt);
        else
            ecoreElt.destroy();
    }

    @Override
    @objid ("0dc9afd6-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        Property ecoreProp = (Property) ecoreElt;
        // setAssociationName(); -> done in OAssociation class.
        
        setType(ecoreProp);
        setQuantity(ecoreProp);
        setAssociationKind(ecoreProp);
        setReadOnly(ecoreProp);
        setNavigeable(ecoreProp);
        // setNavigable(); -> the navigation is particular in UML2: When a
        // property is owned by
        // an association, it represents a non-navigable end of the association.
        // In this case, the
        // property does not appear in the namespace of any of the associated
        // classifiers. When
        // a property at an end of an association is owned by one of the
        // associated classifiers,
        // it represents a navigable end of the association. In this case, the
        // property is also an
        // attribute of the associated classifier. Only binary associations may
        // have navigable ends.
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setAccessModeEAnnotation(ecoreProp); 
            setAbstractEAnnotation(ecoreElt);
        }
        
        setStatic(ecoreProp);
        setOrdered(ecoreProp);
        setUnique(ecoreProp);
    }

    @objid ("0dc9afdc-4583-11e0-b54c-00137279a832")
    private void setQuantity(Property ecoreElt) {
        setMin(ecoreElt);
        setMax(ecoreElt);
    }

    @objid ("0dcc1228-4583-11e0-b54c-00137279a832")
    private void setMin(Property ecoreProp) {
        String objingMultMin = this.objingElement.getMultiplicityMin();
        
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreProp.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null){ 
                    if (min != 1)
                        ecoreProp.setLower(min);
                }else {
                    XMILogs logs = XMILogs.getInstance();
                    
                    String contextualMsg = XMI.I18N.getMessage(
                                       "logFile.exception.stringConverter.multiplicityMinWithName",
                                       "IAssociationEnd", ecoreProp.getName());
                    
                    String message = XMI.I18N.getMessage(
                                                        "logFile.exception.stringConverter.defaultMsg",
                                                        "String", "\"" + objingMultMin + "\"", "Integer",
                                                        contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0dcc122b-4583-11e0-b54c-00137279a832")
    private void setMax(Property ecoreProp) {
        String objingMultMax = this.objingElement.getMultiplicityMax();
        
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreProp.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null){ 
                    if (max != 1)
                        ecoreProp.setUpper(max);
                }else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                                       "logFile.exception.stringConverter.multiplicityMaxWithName",
                                       "IAssociationEnd",ecoreProp.getName());
                    
                    String message = XMI.I18N.getMessage(
                                                        "logFile.exception.stringConverter.defaultMsg",
                                                        "String", "\"" + objingMultMax + "\"", "Integer",
                                                        contextualMsg);
                    logs.writelnInLog(message);
        
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0dcc122e-4583-11e0-b54c-00137279a832")
    private void setAssociationKind(Property ecoreProp) {
        switch (this.objingElement.getAggregation()) {
            case KIND_IS_ASSOCIATION:
                ecoreProp.setAggregation(AggregationKind.NONE_LITERAL);
                break;
            case KIND_IS_AGGREGATION:
                ecoreProp.setAggregation(AggregationKind.SHARED_LITERAL);
                break;
            case KIND_IS_COMPOSITION:
                ecoreProp.setAggregation(AggregationKind.COMPOSITE_LITERAL);
                break;
        }
    }

    @objid ("0dcc1234-4583-11e0-b54c-00137279a832")
    private void setAccessModeEAnnotation(Property ecoreProp) {
        switch (this.objingElement.getChangeable()) {
            case READ:
                ObjingEAnnotation.setAccessMode(ecoreProp,
                                                ObjingEAnnotation.READ_VALUE);
                break;
            case WRITE:
                ObjingEAnnotation.setAccessMode(ecoreProp,
                                                ObjingEAnnotation.WRITE_VALUE);
                break;
            case READ_WRITE:
                ObjingEAnnotation.setAccessMode(ecoreProp,
                                                ObjingEAnnotation.READ_WRITE_VALUE);
                break;
            case ACCES_NONE:
                ObjingEAnnotation.setAccessMode(ecoreProp,
                                                ObjingEAnnotation.ACCESS_NONE_VALUE);
                break;
                // default : do nothing.
        }
    }

    @objid ("0dcc1237-4583-11e0-b54c-00137279a832")
    private void setAbstractEAnnotation(final Element ecoreElt) {
        ObjingEAnnotation.setAbstract(ecoreElt, ((IFeature)getObjingElement()).isAbstract());
    }

    @objid ("0dce7482-4583-11e0-b54c-00137279a832")
    private void setStatic(Property ecoreProp) {
        ecoreProp.setIsStatic(this.objingElement.isClass());
    }

    @objid ("0dce7485-4583-11e0-b54c-00137279a832")
    private void setOrdered(Property ecoreProp) {
        ecoreProp.setIsOrdered(this.objingElement.isOrdered());
    }

    @objid ("0dce7488-4583-11e0-b54c-00137279a832")
    private void setUnique(Property ecoreProp) {
        if (this.objingElement.isUnique())
            ecoreProp.setIsUnique(true);
    }

    @objid ("0dce748b-4583-11e0-b54c-00137279a832")
    private void linkEcoreProperty(Property ecoreProperty) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        
        // Link to the Ecore Association:
        IAssociation objingAssoc =this. objingElement.getRelated();
        Association ecoreAssoc = (Association) genProp.getMappedElement(objingAssoc);
        
        if (ecoreAssoc != null) {
            // Associates the ecore Association to the Property:
            ecoreProperty.setAssociation(ecoreAssoc);
        
            // If the role is navigable: the property is owned by the classifier
            // (the ecore classifier must have already been created => if not:
            // call on the visitor on it).
        
            if (ModelUtils.isOwnedByClassifier(this.objingElement)){
                attachClassifierEnd(ecoreProperty, ecoreAssoc);
            }else{
                if (this.objingElement.isNavigable()) {   
                    attachNavigableEnd(ecoreProperty, ecoreAssoc, this.objingElement); 
                } else {   
        
                    // If the role is not navigable: the property is owned by the
                    // association (if the ecore association has not been created
                    // yet, the ownership relation will be handled during the visit
                    // of the association).
                    //                        ecoreProperty.setIsNavigable(false);  
                    ecoreProperty.setIsNavigable(this.objingElement.isNavigable()); 
                    // Sets the Association as owner of the Property:
                    if (!ecoreAssoc.getOwnedEnds().contains(ecoreProperty)){
                        ecoreAssoc.getOwnedEnds().add(ecoreProperty);                    
                    }
                }
            }
        }
    }

    @objid ("0dce748e-4583-11e0-b54c-00137279a832")
    private void linkEcorePropertyLinkedToAC(Property ecoreProperty) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        // Link to the Ecore AssociationClass if it has already been created:
        IAssociation objingAssoc = this.objingElement.getRelated();
        AssociationClass ecoreAssocClass = (AssociationClass) genProp
                .getMappedElement(objingAssoc);
        
        if (ecoreAssocClass != null) {
            // Associates the ecore AssociationClass to the Property:
            ecoreProperty.setAssociation(ecoreAssocClass);
        
            // If the role is navigable: the property is owned by the classifier
            // (the ecore classifier must
            // have already been created => if not: call on the visitor on it).
            if (this.objingElement.isNavigable()) {
                attachNavigableEndLinkedToAC(ecoreProperty);
            } else {
                // If the role is not navigable: the property is owned by the
                // AssociationClass (if the ecore AssociationClass has not been
                // created yet, the ownership relation will be handled during
                // the visit of the AssociationClass).
                ecoreProperty.setIsNavigable(this.objingElement.isNavigable());
                //                        ecoreProperty.setIsNavigable(false);
                // Sets the Association as owner of the Property:
                if (!ecoreAssocClass.getOwnedEnds().contains(ecoreProperty))
                    ecoreAssocClass.getOwnedEnds().add(ecoreProperty);
        
            }
        }
    }

    @objid ("0dd0d6dc-4583-11e0-b54c-00137279a832")
    private void setType(Property ecoreProperty) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        List<IAssociationEnd> oppositeEnds = ObjingModelNavigation
                .getOppositeAssociationEnds(this.objingElement);
        
        int nbEnds = oppositeEnds.size();
        
        if (nbEnds == 1) {
            // Case of a binary association: the type of the current Property
            // is given by the opposite Classifier
            IAssociationEnd oppositeEnd = oppositeEnds.get(0);
            IClassifier owner = oppositeEnd.getOwner();
            Type ecorePropertyType = (Type) genProp.getMappedElement(owner);
        
            // Types the Property with the Classifier:
            if (ecorePropertyType != null)
                ecoreProperty.setType(ecorePropertyType);
        
        
        } else if (nbEnds > 1) {
            // Case of a N ary association: the type of the current Property
            // is given by its related Classifier (its owner in Ijing)
            IClassifier objingOwner = this.objingElement.getOwner();
            Type ecoreType = (Type) genProp.getMappedElement(objingOwner);
        
            if (ecoreType != null) {
                ecoreProperty.setType(ecoreType);
            }
        }
    }

    @objid ("0dd0d6e2-4583-11e0-b54c-00137279a832")
    private void attachNavigableEnd(Property ecoreProperty, Association ecoreAssoc, IAssociationEnd objAssocEnd) {
        ecoreProperty.setIsNavigable(objAssocEnd.isNavigable());
        
        ecoreAssoc.getMemberEnds().add(ecoreProperty);
        
        IClassifier objingOwner = objAssocEnd.getOwner();
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            // Link to its owner Classifier:
            // Class, Component, Node
            if (ecoreOwner instanceof StructuredClassifier) {
                StructuredClassifier ownerIsSC = (StructuredClassifier) ecoreOwner;
                ownerIsSC.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Interface) {// Interface
                Interface ownerIsInterface = (Interface) ecoreOwner;
                ownerIsInterface.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Signal) {// Signal
                Signal ownerIsSignal = (Signal) ecoreOwner;
                ownerIsSignal.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof UseCase) {// UseCase
                UseCase ownerIsUseCase = (UseCase) ecoreOwner;
                EList<Property> attributes = ownerIsUseCase.getAttributes();
                try {
                    if (!attributes.contains(ecoreProperty))
                        attributes.add(ecoreProperty);
                } catch (UnsupportedOperationException e) {
                    ecoreAssoc.getOwnedEnds().add(ecoreProperty);
                }
            } else if (ecoreOwner instanceof Actor) {// Actor
                ecoreAssoc.getOwnedEnds().add(ecoreProperty);
        
            } else if (ecoreOwner instanceof DataType) {// DataType
                DataType ownerIsDataType = (DataType) ecoreOwner;
                EList<Property> attributes = ownerIsDataType.getOwnedAttributes();
                if (!attributes.contains(ecoreProperty))
                    attributes.add(ecoreProperty);
        
            } else if (ecoreOwner instanceof Classifier) {
                Classifier ownerIsClassifier = (Classifier) ecoreOwner;
                try {
                    ownerIsClassifier.getAttributes().add(ecoreProperty);
                } catch (UnsupportedOperationException e) {
                    e.printStackTrace(Modelio.err);
                }
            } else {
                ecoreProperty.destroy();
                throw new NotFoundException("Owner Class (" + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0dd0d6e7-4583-11e0-b54c-00137279a832")
    private void attachNavigableEndLinkedToAC(Property ecoreProperty) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        ecoreProperty.setIsNavigable(this.objingElement.isNavigable());
        
        IClassifier objingOwner = this.objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            // Link to its owner Classifier:
            if (ecoreOwner instanceof StructuredClassifier) {
                StructuredClassifier ownerIsSC = (StructuredClassifier) ecoreOwner;
                ownerIsSC.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsInterface = (Interface) ecoreOwner;
                ownerIsInterface.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof DataType) {
                DataType ownerIsDataType = (DataType) ecoreOwner;
                ownerIsDataType.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Signal) {
                Signal ownerIsSignal = (Signal) ecoreOwner;
                ownerIsSignal.getOwnedAttributes().add(ecoreProperty);
            }
            else {
                ecoreProperty.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0dd33936-4583-11e0-b54c-00137279a832")
    private void setReadOnly(Property ecoreProp) {
        switch (this.objingElement.getChangeable()) {
            case READ:
                if (this.objingElement.isNavigable())
                    ecoreProp.setIsReadOnly(true);
        }
    }

    @objid ("65fc67bc-1a76-11e1-8103-0027103f347d")
    private void setNavigeable(final Property ecoreProp) {
        try {
            ecoreProp.setIsNavigable(this.objingElement.isNavigable());
        }catch (IllegalStateException e){
            e.printStackTrace(Modelio.err);
        }
        Association association = ecoreProp.getAssociation();
        if ((association != null) && (ecoreProp.getOwner().equals(association)) && (this.objingElement.isNavigable())){
            association.getNavigableOwnedEnds().add(ecoreProp);
        }
    }

    @objid ("65fcdcf2-1a76-11e1-8103-0027103f347d")
    private void attachClassifierEnd(final Property ecoreProperty, final Association ecoreAssoc) {
        ecoreAssoc.getMemberEnds().add(ecoreProperty);
        
        IClassifier objingOwner = this.objingElement.getOwner();
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            // Link to its owner Classifier:
            // Class, Component, Node
            if (ecoreOwner instanceof StructuredClassifier) {
                StructuredClassifier ownerIsSC = (StructuredClassifier) ecoreOwner;
                ownerIsSC.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Interface) {// Interface
                Interface ownerIsInterface = (Interface) ecoreOwner;
                ownerIsInterface.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof Signal) {// Signal
                Signal ownerIsSignal = (Signal) ecoreOwner;
                ownerIsSignal.getOwnedAttributes().add(ecoreProperty);
            } else if (ecoreOwner instanceof UseCase)  {// UseCase
                UseCase ownerIsUseCase = (UseCase) ecoreOwner;
                EList<Property> attributes=  ownerIsUseCase.getAttributes();
                try{
                    if (!attributes.contains(ecoreProperty))
                        attributes.add(ecoreProperty);
                }catch(UnsupportedOperationException e){
                    ecoreAssoc.getOwnedEnds().add(ecoreProperty);
                }
            } else if (ecoreOwner instanceof Actor) {// Actor
                ecoreAssoc.getOwnedEnds().add(ecoreProperty);
        
            } else if (ecoreOwner instanceof DataType) {// DataType
                DataType ownerIsDataType = (DataType) ecoreOwner;
                EList<Property> attributes=  ownerIsDataType.getOwnedAttributes();
                if (!attributes.contains(ecoreProperty))
                    attributes.add(ecoreProperty);
        
            } else if (ecoreOwner instanceof Classifier) {
                Classifier ownerIsClassifier = (Classifier) ecoreOwner;
                try  {
                    ownerIsClassifier.getAttributes().add(ecoreProperty);
                }catch (UnsupportedOperationException e) {
                    e.printStackTrace(Modelio.err);
                }
            } else {
                ecoreProperty.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

}
