/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IAttribute;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IFeature;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Artifact;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Node;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class manages the export of IAttribute elements
 * @author ebrosse
 */
@objid ("0dd33939-4583-11e0-b54c-00137279a832")
public class OAttribute extends OFeature {
    @objid ("0dd3393a-4583-11e0-b54c-00137279a832")
    private IAttribute objingElement;


    @Override
    @objid ("0dd3393b-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createProperty();
    }

    /**
     * Constructor of OAttribute.
     * @param element : the exported IAttribute
     */
    @objid ("0dd33941-4583-11e0-b54c-00137279a832")
    public OAttribute(final IAttribute element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0dd59b98-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        Element ecoreOwner = null;
        
        // In case of a Qualified Association, the Attribute has no Owner but a
        // Qualified AssociationEnd:
        IAssociationEnd qualifiedAssocEnd = this.objingElement.getQualified();
        
        if (qualifiedAssocEnd != null) {
            ecoreOwner = genProp.getMappedElement(qualifiedAssocEnd);
        } else {
            IClassifier objingOwner = this.objingElement.getOwner();
            ecoreOwner = genProp.getMappedElement(objingOwner);
        }
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Component) {
                Component ownerIsComponent = (Component) ecoreOwner;
                ownerIsComponent.getOwnedAttributes().add((Property)ecoreElt);
            } else if (ecoreOwner instanceof Node) {
                Node ownerIsNode = (Node) ecoreOwner;
                ownerIsNode.getOwnedAttributes().add((Property)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getOwnedAttributes().add((Property)ecoreElt);
            }else if (ecoreOwner instanceof Interface) {
                Interface ownerIsInterface = (Interface) ecoreOwner;
                ownerIsInterface.getOwnedAttributes().add((Property)ecoreElt);
            } else if (ecoreOwner instanceof DataType) {
                DataType ownerIsDataType = (DataType) ecoreOwner;
                ownerIsDataType.getOwnedAttributes().add((Property)ecoreElt);
            }else if (ecoreOwner instanceof Property) {
                Property ownerIsProperty = (Property) ecoreOwner;
                ownerIsProperty.getQualifiers().add((Property)ecoreElt);
            } else if (ecoreOwner instanceof Artifact) {
                Artifact ownerIsArtifact = (Artifact) ecoreOwner;
                ownerIsArtifact.getOwnedAttributes().add((Property)ecoreElt);
            } else if (ecoreOwner instanceof Signal) {
                Signal ownerIsSignal = (Signal) ecoreOwner;
                ownerIsSignal.getOwnedAttributes().add((Property)ecoreElt);
            } else
                // Attributes defined on Actor and UseCase are not exported in UML2.
                // They are
                // added into an EAnnotation:
                if (ecoreOwner instanceof Actor || ecoreOwner instanceof UseCase) {
        
                    ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                            this.objingElement.getOwner(), this.objingElement, ecoreElt);
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreOwner.getClass().getSimpleName() + ") Not Found");
                }
        
        }else
            ecoreElt.destroy();
    }

    @Override
    @objid ("0dd59b9c-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setType((Property) ecoreElt);
        setStatic((Property) ecoreElt);       
        setReadOnly((Property) ecoreElt);
        setExpressionOfValue((Property) ecoreElt);
        setMin((Property) ecoreElt);
        setMax((Property) ecoreElt);
        
        setOrdered((Property) ecoreElt);
        setUnique((Property) ecoreElt);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setTypeConstraintEAnnotation((Property) ecoreElt);
            setAccessModeEAnnotation((Property) ecoreElt);
            setTargetIsClassEAnnotation((Property) ecoreElt);
            setDynamicDependencyEAnnotation((Property) ecoreElt);
            setAbstractEAnnotation(ecoreElt);
        }
    }

    @objid ("0dd7fdee-4583-11e0-b54c-00137279a832")
    private void setStatic(Property ecoreProp) {
        ecoreProp.setIsStatic(this.objingElement.isClass());
    }

    @objid ("0dd7fdf1-4583-11e0-b54c-00137279a832")
    private void setTypeConstraintEAnnotation(Property ecoreProp) {
        ObjingEAnnotation.setTypeConstraint(ecoreProp, this.objingElement
                .getTypeConstraint());
    }

    @objid ("0dd7fdf7-4583-11e0-b54c-00137279a832")
    private void setAbstractEAnnotation(final Element ecoreElt) {
        ObjingEAnnotation.setAbstract(ecoreElt, ((IFeature)getObjingElement()).isAbstract());
    }

    @objid ("0dda6044-4583-11e0-b54c-00137279a832")
    private void setExpressionOfValue(Property ecoreProp) {
        String objingDefaultValue = this.objingElement.getValue();
        IGeneralClass objingType = this.objingElement.getType();
        
        // If objingValue is "" then we don't set a default value for the UML2
        
        if (!"".equals(objingDefaultValue) ) {
        
            if (objingType != null) {
                
                boolean isDynamicDependency = this.objingElement.isDerived();
        
                IUMLTypes umlTypes = Modelio.getInstance()
                        .getModelingSession().getModel().getUmlTypes();
        
                if ((ObjingModelNavigation.OBJING_NULL_VALUE != null) 
                        && (ObjingModelNavigation.OBJING_NULL_VALUE.equals(objingDefaultValue.toLowerCase())))
        
                    ecoreProp.setNullDefaultValue();
        
                else if ((umlTypes.getBOOLEAN() != null) && (umlTypes.getBOOLEAN().equals(objingType))) { //Boolean Case
                    // If the attribute is a Dynamic Dependency, we don't enable
                    StringConverter.setFilterEnabled(!isDynamicDependency);
                    Boolean bool = StringConverter
                            .getBoolean(objingDefaultValue);
                    if (bool != null)
                        ecoreProp.setBooleanDefaultValue(bool);
                    else {
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultValue",
                                this.objingElement.getName(),
                                "Attribute" );
        
                        String message = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultMsg",
                                "String", "\"" + objingDefaultValue + "\"",
                                "Boolean");
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                        Expression value = UMLFactory.eINSTANCE.createExpression();
                        value.setSymbol(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
                }else if ((umlTypes.getINTEGER() != null) && (umlTypes.getINTEGER().equals(objingType))) {  //INTEGER case
                    // If the attribute is a Dynamic Dependency, we don't enable
                    // string filters:
                    StringConverter.setFilterEnabled(!isDynamicDependency);
                    Integer objingIntValue = StringConverter
                            .getInteger(objingDefaultValue);
                    
                    if (objingIntValue != null) {
                        if (objingIntValue >= 0)
                            ecoreProp
                            .setUnlimitedNaturalDefaultValue(objingIntValue);
                        else
                            ecoreProp.setIntegerDefaultValue(objingIntValue);
                    } else {
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultValue",
                                this.objingElement.getName(),
                                "Attribute" );
        
                        String message = XMI.I18N.getMessage("logFile.exception.stringConverter.defaultMsg",
                                "String", "\"" + objingDefaultValue + "\"",
                                "Integer");
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                        Expression value = UMLFactory.eINSTANCE.createExpression();
                        value.setSymbol(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
                } else if (((umlTypes.getCHAR() != null) && (umlTypes.getCHAR().equals(objingType)))
                        || ((umlTypes.getSTRING() != null) && (umlTypes.getSTRING().equals(objingType)))){//CHAR and STRING case
                    
                    ecoreProp.setStringDefaultValue(objingDefaultValue);
        
                } else if (objingType instanceof IEnumeration){ //Enumeration case
                    
                    if  (ObjingModelNavigation.isEnumerationliteral((IEnumeration) objingType, objingDefaultValue)){
        
                        InstanceValue value = UMLFactory.eINSTANCE.createInstanceValue();
        
                        Object ecoreType = GenerationProperties.getInstance().getMappedElement(objingType);
                        if (ecoreType instanceof Type){
                            value.setType((Type) ecoreType);
                        }
        
                        Object ecoreInstance = GenerationProperties.getInstance().getMappedElement(ObjingModelNavigation.getEnumerationliteral((IEnumeration)objingType, objingDefaultValue));
                        if (ecoreInstance instanceof InstanceSpecification){
                            value.setInstance((InstanceSpecification) ecoreInstance);
                        }
        
                        ecoreProp.setDefaultValue(value);
                        
                    }else{
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.warning.wrongLiteral", objingDefaultValue,  objingType.getName());                                
                        
                        logs.writelnInLog(contextualMsg);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement);
        
                        OpaqueExpression value = UMLFactory.eINSTANCE.createOpaqueExpression();
                        value.getBodies().add(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
        
                } else{ // No possible mapping.
        
                    Expression value = UMLFactory.eINSTANCE.createExpression();
                    value.setSymbol(objingDefaultValue);
                    ecoreProp.setDefaultValue(value);
                }
        
            } else{ // No type
                OpaqueExpression value = UMLFactory.eINSTANCE.createOpaqueExpression();
                value.getBodies().add(objingDefaultValue);
                ecoreProp.setDefaultValue(value);
            }
        }
    }

    @objid ("0dda6047-4583-11e0-b54c-00137279a832")
    private void setMin(Property ecoreProp) {
        String objingMultMin = this.objingElement.getMultiplicityMin();
        
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreProp.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null){ 
                    if (min != 1)
                        ecoreProp.setLower(min);
                    //else it is the default value
                }else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg =XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMinWithName",
                            "Attribute", ecoreProp.getName());
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + objingMultMin + "\"", "Integer",
                            contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0dda604a-4583-11e0-b54c-00137279a832")
    private void setDynamicDependencyEAnnotation(Property ecoreProp) {
        ObjingEAnnotation.setDynamicDependency(ecoreProp, this.objingElement
                .isDerived());
    }

    @objid ("0dda604d-4583-11e0-b54c-00137279a832")
    private void setOrdered(Property ecoreProp) {
        ecoreProp.setIsOrdered(this.objingElement.isOrdered());
    }

    @objid ("0dda6050-4583-11e0-b54c-00137279a832")
    private void setUnique(Property ecoreProp) {
        if (this.objingElement.isUnique()){
            ecoreProp.setIsUnique(true);
        }else 
            ecoreProp.setIsUnique(false);
    }

    @objid ("0dda6053-4583-11e0-b54c-00137279a832")
    private void setAccessModeEAnnotation(Property ecoreProp) {
        switch (this.objingElement.getChangeable()) {
        case READ:
            ObjingEAnnotation.setAccessMode(ecoreProp,
                    ObjingEAnnotation.READ_VALUE);
            break;
        case WRITE:
            ObjingEAnnotation.setAccessMode(ecoreProp,
                    ObjingEAnnotation.WRITE_VALUE);
            break;
        case READ_WRITE:
            ObjingEAnnotation.setAccessMode(ecoreProp,
                    ObjingEAnnotation.READ_WRITE_VALUE);
            break;
        case ACCES_NONE:
            ObjingEAnnotation.setAccessMode(ecoreProp,
                    ObjingEAnnotation.ACCESS_NONE_VALUE);
            break;
            // default : do nothing.
        }
    }

    @objid ("0ddcc29f-4583-11e0-b54c-00137279a832")
    private void setType(Property ecoreProp) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        // Getting type of the Attribute:
        IGeneralClass objingType = this.objingElement.getType();
        if (objingType != null) {
            if (PrimitiveTypeMapper.isPredefinedType(objingType)) {
                PrimitiveTypeMapper.setEcorePredefinedType(ecoreProp, (IDataType) objingType);
            } else {
                Element ecoreElt = genProp.getMappedElement(objingType);
        
                if ((ecoreElt != null) && (ecoreElt instanceof Type)){
                    ecoreProp.setType((Type) ecoreElt);
                }else{
                    XMILogs xmiLogs = XMILogs.getInstance();
                    String message = XMI.I18N.getMessage("logFile.warning.unsupportedTypeExport", 
                            this.objingElement.getName(), this.objingElement.getClass().getName());
                    xmiLogs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
                }
            }
        }else {
            ObjingEAnnotation.setNoType(ecoreProp, true);
        }
    }

    @objid ("0ddcc2a2-4583-11e0-b54c-00137279a832")
    private void setTargetIsClassEAnnotation(Property property) {
        ObjingEAnnotation.setTargetIsClass(property, this.objingElement
                .isTargetIsClass());
    }

    @objid ("0ddcc2a8-4583-11e0-b54c-00137279a832")
    private void setMax(Property ecoreProp) {
        String objingMultMax = this.objingElement.getMultiplicityMax();
        
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreProp.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null){ 
                    if (max != 1)
                        ecoreProp.setUpper(max);
                    //else it is the default value
                }else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMaxWithName",
                            "Attribute", ecoreProp.getName());
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + objingMultMax + "\"", "Integer",
                            contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0ddcc2ab-4583-11e0-b54c-00137279a832")
    private void setReadOnly(Property ecoreProp) {
        switch (this.objingElement.getChangeable()) {
        case READ:
            ecoreProp.setIsReadOnly(true);
        }
    }

}
