/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.statik.IAttribute;
import com.modeliosoft.modelio.api.model.uml.statik.IAttributeLink;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0ddf24f8-4583-11e0-b54c-00137279a832")
public class OAttributeLink extends OElement implements IOElement {
    @objid ("0ddf24f9-4583-11e0-b54c-00137279a832")
    private IAttributeLink objingElement;


    @objid ("0ddf24fa-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createSlot();
    }

    @objid ("0ddf24fe-4583-11e0-b54c-00137279a832")
    public OAttributeLink() {
    }

    @objid ("0ddf2500-4583-11e0-b54c-00137279a832")
    public OAttributeLink(IAttributeLink element) {
        super(element);
        objingElement = element;
    }

    @objid ("0de18758-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IInstance objOwner = objingElement.getAttributed();
        Element temp = GenerationProperties.getInstance().getMappedElement(objOwner);
        
        if (temp instanceof InstanceSpecification){
            InstanceSpecification ecoreOwner = (InstanceSpecification) temp;
            ecoreOwner.getSlots().add((Slot)ecoreElt);
        }
    }

    @objid ("0de1875c-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setDefiningFeature((Slot)ecoreElt);
        setExpressionOfValue((Slot)ecoreElt);
        
        if ( GenerationProperties.getInstance().isRoundtripEnabled()){
            setEAnnotationValue(ecoreElt);
            ObjingEAnnotation.setAttributeLink(ecoreElt, true);
            setEAnnotationName(ecoreElt);
            IInstance objOwner =  objingElement.getAttributed();
        
            ObjingEAnnotation.setOwner(ecoreElt, objOwner.getIdentifier());
           }
    }

    @objid ("0de1875f-4583-11e0-b54c-00137279a832")
    private void setDefiningFeature(Slot slot) {
        IAttribute objingFeature = objingElement.getBase();
        
        if (objingFeature != null) {
        
            Element ecoreFeature =
                    (Element) GenerationProperties.getInstance().getMappedElement(objingFeature);
        
            if (ecoreFeature instanceof StructuralFeature) {
                slot.setDefiningFeature((StructuralFeature)ecoreFeature);
            }
        }
    }

    @objid ("0de3e9b1-4583-11e0-b54c-00137279a832")
    private void setEAnnotationValue(Element ecoreElt) {
        ObjingEAnnotation.setValue(ecoreElt, objingElement.getValue());
    }

    @objid ("0de3e9b4-4583-11e0-b54c-00137279a832")
    private void setEAnnotationName(Element ecoreElt) {
        ObjingEAnnotation.setName(ecoreElt, objingElement.getName());
    }

    @objid ("c98cbbd1-ab2e-11e1-83be-0027103f347d")
    private void setExpression(final Slot slot, final String value) {
        Expression expr = UMLFactory.eINSTANCE.createExpression();
        ValueSpecification result = slot.createValue(null, null, expr.eClass());
        ((Expression)result).setSymbol(value);
    }

    @objid ("c98cbbd5-ab2e-11e1-83be-0027103f347d")
    private void setExpressionOfValue(final Slot slot) {
        IAttribute attribut =  objingElement.getBase();
        if (attribut != null){
            IGeneralClass type = attribut.getType();
            String value = this.objingElement.getValue();
            IGeneralClass objingType = attribut.getType();
        
            // If objingValue is "" then we don't set a default value for the UML2
        
            if (!"".equals(value) ) {
        
                if (objingType != null) {
        
                    IUMLTypes umlTypes = Modelio.getInstance()
                            .getModelingSession().getModel().getUmlTypes();
        
                    if ((ObjingModelNavigation.OBJING_NULL_VALUE != null) 
                            && (ObjingModelNavigation.OBJING_NULL_VALUE.equals(value.toLowerCase())
                                    && (type instanceof IDataType))){
        
                        LiteralNull literalNull = UMLFactory.eINSTANCE.createLiteralNull();
                        ValueSpecification result = slot.createValue(null, null, literalNull.eClass());
                        try{
                            ((LiteralUnlimitedNatural)result).setValue(Integer.valueOf(value.toString()));
                        }catch(NumberFormatException e){
        
                            literalNull.destroy();
                            setExpression(slot, value);
                        }
        
                    }else if ((umlTypes.getBOOLEAN() != null) && (umlTypes.getBOOLEAN().equals(objingType))) { //Boolean Case
                        // If the attribute is a Dynamic Dependency, we don't enable
                        Boolean bool = StringConverter
                                .getBoolean(value);
                        if (bool != null){
                            LiteralBoolean literalBoolean = UMLFactory.eINSTANCE.createLiteralBoolean();
                            ValueSpecification result = slot.createValue(null, null, literalBoolean.eClass());
                            ((LiteralBoolean)result).setValue(bool);
                        }else {
                            setExpression(slot, value);
        
                            XMILogs logs = XMILogs.getInstance();
                            String contextualMsg = XMI.I18N.getMessage(
                                    "logFile.exception.stringConverter.defaultValue", 
                                    this.objingElement.getName(),
                                    "AttributeLink"
                                   );
        
                            String message = XMI.I18N.getMessage(
                                    "logFile.exception.stringConverter.defaultMsg",
                                    "String", "\"" + value + "\"",
                                    "Boolean");
                            logs.writelnInLog(message);
                            GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
        
                        }
                    }else if ((umlTypes.getINTEGER() != null) && (umlTypes.getINTEGER().equals(objingType))) {  //INTEGER case
                        // If the attribute is a Dynamic Dependency, we don't enable
                        // string filters:
                        try{
                            int intValue = Integer.valueOf(value);
                            if (intValue >= 0){
                                LiteralUnlimitedNatural literalUnlimited = UMLFactory.eINSTANCE.createLiteralUnlimitedNatural();
                                ValueSpecification result = slot.createValue(null, null, literalUnlimited.eClass());
                                ((LiteralUnlimitedNatural)result).setValue(intValue);
                            }else{
                                LiteralInteger literalInteger = UMLFactory.eINSTANCE.createLiteralInteger();
                                ValueSpecification result = slot.createValue(null, null, literalInteger.eClass());
                                ((LiteralInteger)result).setValue(intValue);
                            }
        
        
                        }catch(NumberFormatException e){
                            setExpression(slot, value);
        
                            XMILogs logs = XMILogs.getInstance();
                            String contextualMsg = XMI.I18N.getMessage(
                                    "logFile.exception.stringConverter.defaultValue",
                                    this.objingElement.getName(),
                                    "AttributeLink"  );
        
                            String message = XMI.I18N.getMessage("logFile.exception.stringConverter.defaultMsg",
                                    "String", "\"" + value + "\"",
                                    "Integer");
                            logs.writelnInLog(message);
                            GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                        }
                    } else if (((umlTypes.getCHAR() != null) && (umlTypes.getCHAR().equals(objingType)))
                            || ((umlTypes.getSTRING() != null) && (umlTypes.getSTRING().equals(objingType)))){//CHAR and STRING case
        
                        LiteralString literalstring= UMLFactory.eINSTANCE.createLiteralString();
                        ValueSpecification result = slot.createValue(null, null, literalstring.eClass());
                        ((LiteralString)result).setValue(value);
        
                    } else if (objingType instanceof IEnumeration){ //Enumeration case
        
                        if  (ObjingModelNavigation.isEnumerationliteral((IEnumeration) objingType, value)){
        
                            InstanceValue instValue = UMLFactory.eINSTANCE.createInstanceValue();
        
                            InstanceValue result = (InstanceValue) slot.createValue(null, null, instValue.eClass());
        
                            Object ecoreType = GenerationProperties.getInstance().getMappedElement(objingType);
                            if (ecoreType instanceof Type){
                                result.setType((Type) ecoreType);
                            }
        
                            Object ecoreInstance = GenerationProperties.getInstance().getMappedElement(ObjingModelNavigation.getEnumerationliteral((IEnumeration)objingType, value));
                            if (ecoreInstance instanceof InstanceSpecification){
                                result.setInstance((InstanceSpecification) ecoreInstance);
                            }
        
                        }else{
                            XMILogs logs = XMILogs.getInstance();
                            String contextualMsg = XMI.I18N.getMessage(
                                    "logFile.warning.wrongLiteral", value,  objingType.getName());                                
        
                            logs.writelnInLog(contextualMsg);
                            GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement);
        
                            OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
                            ValueSpecification result = slot.createValue(null, null, opaqueExpr.eClass());
                            ((OpaqueExpression)result).getBodies().add(value);
                        }
        
                    } else{ // No possible mapping.
                        setExpression(slot, value);
                    }
        
                } else{ // No type
        
                    OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
                    ValueSpecification result = slot.createValue(null, null, opaqueExpr.eClass());
                    ((OpaqueExpression)result).getBodies().add(value);
                }
            }
        }
    }

}
