/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehaviorParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateParameter;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterEffectKind;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IBehaviorParameter elements
 * @author ebrosse
 */
@objid ("0de8ae6a-4583-11e0-b54c-00137279a832")
public class OBehaviorParameter extends OElement implements IOElement {
    @objid ("0de8ae6b-4583-11e0-b54c-00137279a832")
    private IBehaviorParameter objingElement;


    @Override
    @objid ("0de8ae6c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createParameter();
    }

    /**
     * OBehaviorParameter constructor
     * @param element : the exported Modelio IBehaviorParameter
     */
    @objid ("0deb10c3-4583-11e0-b54c-00137279a832")
    public OBehaviorParameter(final IBehaviorParameter element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0ded7314-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IElement objingParent = ownerVisitor
                .getCompositionOwner(this.objingElement);
        
        Element ecoreParent = genProp.getMappedElement(objingParent);
        
        if (ecoreParent != null) {
            if ( ecoreParent instanceof Behavior) {
                Behavior ecoreBehavior = (Behavior) ecoreParent;
        
                ecoreBehavior.getOwnedParameters().add((Parameter)ecoreElt);
            }
          
        }
    }

    @Override
    @objid ("0ded7318-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setException((Parameter) ecoreElt);
        setStream((Parameter) ecoreElt);
        setEffect((Parameter) ecoreElt);
        setMax((Parameter) ecoreElt);
        setMin((Parameter) ecoreElt);
        setClass((Parameter) ecoreElt);
    }

    @objid ("0ded731b-4583-11e0-b54c-00137279a832")
    private void setEffect(Parameter ecoreElt) {
        switch (this.objingElement.getEffect())
        {
            case CREATE :
                ecoreElt.setEffect(ParameterEffectKind.CREATE_LITERAL);
                break;
            case DELETE :
                ecoreElt.setEffect(ParameterEffectKind.DELETE_LITERAL);
                break;
            case READ :
                ecoreElt.setEffect(ParameterEffectKind.READ_LITERAL);
                break;
            case UPDATE :
                ecoreElt.setEffect(ParameterEffectKind.UPDATE_LITERAL);
                break;
        }
    }

    @objid ("0ded731e-4583-11e0-b54c-00137279a832")
    private void setStream(Parameter ecoreElt) {
        ecoreElt.setIsStream(this.objingElement.isStream());
    }

    @objid ("0ded7321-4583-11e0-b54c-00137279a832")
    private void setException(Parameter ecoreElt) {
        ecoreElt.setIsException(this.objingElement.isException());
    }

    @objid ("0ded7324-4583-11e0-b54c-00137279a832")
    private void setMin(Parameter ecoreElt) {
        String objingMultMin = this.objingElement.getMultiplicityMin();
        
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreElt.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null)
                    ecoreElt.setLower(min);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                                                               "logFile.exception.stringConverter.multiplicityMinWithName",
                                                               "Parameter",ecoreElt.getName());
                    String message = XMI.I18N.getMessage(
                                                         "logFile.exception.stringConverter.defaultMsg",
                                                         "String", "\"" + objingMultMin + "\"", "Integer",
                                                         contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0defd56e-4583-11e0-b54c-00137279a832")
    private void setMax(Parameter ecoreElt) {
        String objingMultMax = this.objingElement.getMultiplicityMax();
        
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreElt.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null)
                    ecoreElt.setUpper(max);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                                                               "logFile.exception.stringConverter.multiplicityMaxWithName",
                                                               "Parameter",ecoreElt.getName());
                    String message = XMI.I18N.getMessage(
                                                         "logFile.exception.stringConverter.defaultMsg",
                                                         "String", "\"" + objingMultMax + "\"", "Integer",
                                                         contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0defd571-4583-11e0-b54c-00137279a832")
    private void setClass(Parameter ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        // Getting type of the Parameter:
        IGeneralClass objingType = this.objingElement.getType();
        if (objingType != null) {
            if (PrimitiveTypeMapper.isPredefinedType(objingType)) {
                PrimitiveTypeMapper.setEcorePredefinedType(ecoreElt, (IDataType) objingType);
            } else {
        
                if (objingType instanceof ITemplateParameter) {
                    return;
        
                } else {
                    Element ecoreEltType = genProp.getMappedElement(objingType);
        
                    if (ecoreEltType instanceof Type)
                        setType(ecoreElt, (Type) ecoreEltType);
                }
            }
        }else {
            ObjingEAnnotation.setNoType(ecoreElt, true);
        }
    }

    @objid ("0defd574-4583-11e0-b54c-00137279a832")
    private void setType(Parameter ecoreParam, Type type) {
        ecoreParam.setType(type);
    }

}
