/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

/**
 * This class manages the export of IBindableInstance elements
 * @author ebrosse
 */
@objid ("0defd578-4583-11e0-b54c-00137279a832")
public class OBindableInstance extends OElement implements IOElement {
    @objid ("0defd579-4583-11e0-b54c-00137279a832")
    private IBindableInstance objingElement;

    @objid ("0defd57a-4583-11e0-b54c-00137279a832")
    private IElement root = null;


    @Override
    @objid ("0defd57b-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        this.root = ObjingModelNavigation.getBindableInstanceOwner(this.objingElement);
        
        if (this.root instanceof IInstance){
            return UMLFactory.eINSTANCE.createSlot();
        }
        
        Property part = null;
        IModelElement representedFeature = this.objingElement.getRepresentedFeature();
        if ((representedFeature != null) && (representedFeature instanceof IAssociationEnd)){
            part = (Property) GenerationProperties.getInstance().getMappedElement(representedFeature);
        }else{
            part = UMLFactory.eINSTANCE.createProperty();
        }
        
        part.setIsComposite(true);
        return part;
    }

    /**
     * Constructor of OBindableInstance
     * @param param : the exported Modelio IBindableInstance
     */
    @objid ("0df237c9-4583-11e0-b54c-00137279a832")
    public OBindableInstance(final IBindableInstance param) {
        super(param);
        this.objingElement = param;
    }

    @Override
    @objid ("0df237d2-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.root = ObjingModelNavigation.getBindableInstanceOwner(this.objingElement);
        if (! (this.objingElement instanceof IPort)){
            if (ecoreElt instanceof Slot){
                attachSlot((Slot)ecoreElt);
            }else if (ecoreElt instanceof Property ){
                attachProperty((Property)ecoreElt);
            }else 
                ecoreElt.destroy();
        }
    }

    @objid ("0df6fc7c-4583-11e0-b54c-00137279a832")
    private void attachProperty(Property ecoreElt) {
        if (!(this.objingElement instanceof IPort)){
            IElement objOwner = this.objingElement.getCompositionOwner();
            Element ecoreOwner = null;
        
            if (objOwner instanceof IBindableInstance){
                setOwnerEAnnotation(ecoreElt, objOwner);
            }
        
            ecoreOwner = GenerationProperties.getInstance().getMappedElement(this.root);
        
            if (ecoreOwner instanceof StructuredClassifier)
        
                ((StructuredClassifier)ecoreOwner).getOwnedAttributes().add(ecoreElt);
            else if (ecoreOwner instanceof Collaboration)
                ((Collaboration)ecoreOwner).getCollaborationRoles().add(ecoreElt);
            else
                ecoreElt.destroy();
        }
    }

    @objid ("0df6fc80-4583-11e0-b54c-00137279a832")
    private void attachSlot(Slot ecoreElt) {
        IElement objingOwner = this.objingElement.getCompositionOwner();
        
        if (objingOwner instanceof IBindableInstance){
            setOwnerEAnnotation(ecoreElt, objingOwner);
        }
        
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof InstanceSpecification) {    
                ((InstanceSpecification) ecoreOwner).getSlots().add(ecoreElt);
            }else{
                ecoreElt.destroy();
                XMILogs logs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage(
                        "logFile.exception.stringConverter.haveNotOwner",
                        ecoreElt.getClass().toString());
                logs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addError(message, this.objingElement);
            }
        
        
            if (( GenerationProperties.getInstance().isRoundtripEnabled()) && (!(this.objingElement instanceof IPort)))
                ObjingEAnnotation.setBindableInstance(ecoreElt, true);
        }
    }

    @objid ("0df6fc84-4583-11e0-b54c-00137279a832")
    private void setOwnerEAnnotation(Element element, IElement owner) {
        ObjingEAnnotation.setOwner(element, String.valueOf(owner.getIdentifier()));
    }

    @Override
    @objid ("0df6fc88-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Property){
            ObjingModelNavigation.setRepresentedFeature(this.objingElement, (Property)ecoreElt);
            setName((NamedElement) ecoreElt);
            setBase((Property) ecoreElt);
            setMin((Property) ecoreElt);
            setMax((Property) ecoreElt);
            setExpressionOfValue((Property) ecoreElt);
        }else if (ecoreElt instanceof Slot){
            setRepresentedFeature((Slot) ecoreElt);
            setEAnnotationName((Slot)ecoreElt);
            setValue((Slot) ecoreElt);
        }
    }

    @objid ("0df6fc8b-4583-11e0-b54c-00137279a832")
    private void setRepresentedFeature(Slot ecoreElt) {
        IModelElement objReprensted = this.objingElement.getRepresentedFeature();
        
        if (objReprensted != null) {
            Element represented = GenerationProperties.getInstance().getMappedElement(objReprensted);
        
            if (represented instanceof StructuralFeature) {
                ecoreElt.setDefiningFeature((StructuralFeature) represented);
            }
        }
    }

    @objid ("0df95ed8-4583-11e0-b54c-00137279a832")
    private void setName(NamedElement ecoreElt) {
        if (ObjingModelNavigation.isNotNullOrEmpty(this.objingElement.getName()))
            ecoreElt.setName(this.objingElement.getName());
    }

    @objid ("0df95edb-4583-11e0-b54c-00137279a832")
    private void setBase(Property ecoreElt) {
        IElement base = this.objingElement.getBase();
        if ( base != null) {
            Element type =  GenerationProperties.getInstance().getMappedElement(base);
        
            if (type instanceof Type)
                ecoreElt.setType((Type)type);
            else{
                XMILogs xmiLogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedTypeExport", 
                        this.objingElement.getName(), this.objingElement.getClass().getSimpleName());
                xmiLogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
            }
        
        }
    }

    @objid ("0df95ee1-4583-11e0-b54c-00137279a832")
    private void setMin(Property ecoreProp) {
        String objingMultMin = this.objingElement.getMultiplicityMin();
        
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreProp.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null){ 
                    if (min != 1)
                        ecoreProp.setLower(min);
                }else {
                    XMILogs logs = XMILogs.getInstance();
        
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMinWithName",
                            "Attribute", ecoreProp.getName());
        
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + objingMultMin + "\"", "Integer",
                            contextualMsg);
        
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0dfbc130-4583-11e0-b54c-00137279a832")
    private void setMax(Property ecoreProp) {
        String objingMultMax = this.objingElement.getMultiplicityMax();
        
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreProp.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null){ 
                    if (max != 1)
                        ecoreProp.setUpper(max);
                }else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMaxWithName",
                            "Attribute", ecoreProp.getName());
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + objingMultMax + "\"", "Integer",
                            contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("0dfbc133-4583-11e0-b54c-00137279a832")
    private void setEAnnotationName(Slot ecoreElt) {
        ObjingEAnnotation.setName(ecoreElt, this.objingElement.getName());
    }

    @objid ("c9817131-ab2e-11e1-83be-0027103f347d")
    private void setExpressionOfValue(final Property ecoreProp) {
        String objingDefaultValue = this.objingElement.getValue();
        INameSpace objingType = this.objingElement.getBase();
        
        // If objingValue is "" then we don't set a default value for the UML2
        
        if (!"".equals(objingDefaultValue) ) {
        
            if (objingType != null) {
                
        
                IUMLTypes umlTypes = Modelio.getInstance()
                        .getModelingSession().getModel().getUmlTypes();
        
                if ((ObjingModelNavigation.OBJING_NULL_VALUE != null) 
                        && (ObjingModelNavigation.OBJING_NULL_VALUE.equals(objingDefaultValue.toLowerCase())))
        
                    ecoreProp.setNullDefaultValue();
        
                else if ((umlTypes.getBOOLEAN() != null) && (umlTypes.getBOOLEAN().equals(objingType))) { //Boolean Case
                    // If the attribute is a Dynamic Dependency, we don't enable
                    StringConverter.setFilterEnabled(false);
                    Boolean bool = StringConverter
                            .getBoolean(objingDefaultValue);
                    if (bool != null)
                        ecoreProp.setBooleanDefaultValue(bool);
                    else {
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultValue",
                                this.objingElement.getName(),
                                "Attribute" );
        
                        String message = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultMsg",
                                "String", "\"" + objingDefaultValue + "\"",
                                "Boolean");
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                        Expression value = UMLFactory.eINSTANCE.createExpression();
                        value.setSymbol(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
                }else if ((umlTypes.getINTEGER() != null) && (umlTypes.getINTEGER().equals(objingType))) {  //INTEGER case
                    // If the attribute is a Dynamic Dependency, we don't enable
                    // string filters:
                    StringConverter.setFilterEnabled(false);
                    Integer objingIntValue = StringConverter
                            .getInteger(objingDefaultValue);
                    
                    if (objingIntValue != null) {
                        if (objingIntValue >= 0)
                            ecoreProp
                            .setUnlimitedNaturalDefaultValue(objingIntValue);
                        else
                            ecoreProp.setIntegerDefaultValue(objingIntValue);
                    } else {
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.exception.stringConverter.defaultValue",
                                this.objingElement.getName(),
                                "Attribute" );
        
                        String message = XMI.I18N.getMessage("logFile.exception.stringConverter.defaultMsg",
                                "String", "\"" + objingDefaultValue + "\"",
                                "Integer");
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                        Expression value = UMLFactory.eINSTANCE.createExpression();
                        value.setSymbol(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
                } else if (((umlTypes.getCHAR() != null) && (umlTypes.getCHAR().equals(objingType)))
                        || ((umlTypes.getSTRING() != null) && (umlTypes.getSTRING().equals(objingType)))){//CHAR and STRING case
                    
                    ecoreProp.setStringDefaultValue(objingDefaultValue);
        
                } else if (objingType instanceof IEnumeration){ //Enumeration case
                    
                    if  (ObjingModelNavigation.isEnumerationliteral((IEnumeration) objingType, objingDefaultValue)){
        
                        InstanceValue value = UMLFactory.eINSTANCE.createInstanceValue();
        
                        Object ecoreType = GenerationProperties.getInstance().getMappedElement(objingType);
                        if (ecoreType instanceof Type){
                            value.setType((Type) ecoreType);
                        }
        
                        Object ecoreInstance = GenerationProperties.getInstance().getMappedElement(ObjingModelNavigation.getEnumerationliteral((IEnumeration)objingType, objingDefaultValue));
                        if (ecoreInstance instanceof InstanceSpecification){
                            value.setInstance((InstanceSpecification) ecoreInstance);
                        }
        
                        ecoreProp.setDefaultValue(value);
                        
                    }else{
                        XMILogs logs = XMILogs.getInstance();
                        String contextualMsg = XMI.I18N.getMessage(
                                "logFile.warning.wrongLiteral", objingDefaultValue,  objingType.getName());                                
                        
                        logs.writelnInLog(contextualMsg);
                        GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement);
        
                        OpaqueExpression value = UMLFactory.eINSTANCE.createOpaqueExpression();
                        value.getBodies().add(objingDefaultValue);
                        ecoreProp.setDefaultValue(value);
                    }
        
                } else{ // No possible mapping.
        
                    Expression value = UMLFactory.eINSTANCE.createExpression();
                    value.setSymbol(objingDefaultValue);
                    ecoreProp.setDefaultValue(value);
                }
        
            } else{ // No type
                OpaqueExpression value = UMLFactory.eINSTANCE.createOpaqueExpression();
                value.getBodies().add(objingDefaultValue);
                ecoreProp.setDefaultValue(value);
            }
        }
    }

    @objid ("c9819841-ab2e-11e1-83be-0027103f347d")
    private void setValue(final Slot ecoreElt) {
        String value = this.objingElement.getValue();
        if ((value != null ) && (!value.equals(""))){
            INameSpace base  = this.objingElement.getBase();
            setValueSpecification(base, this.objingElement.getValue(), ecoreElt);
        }
    }

    @objid ("c9819844-ab2e-11e1-83be-0027103f347d")
    private void setValueSpecification(final INameSpace objType, final Object value, final Slot slot) {
        Type type = null;
        if (objType != null){
            Object temp = GenerationProperties.getInstance().getMappedElement(objType);
            if (temp instanceof Type)
                type = (Type) temp;
        }
        
        if (type != null){
            ValueSpecification result = null;
        
            IUMLTypes umlTypes = Modelio.getInstance()
                    .getModelingSession().getModel().getUmlTypes();
            // primitive type ==> LiteralSpecification
        
            if(objType.equals(umlTypes.getINTEGER())){
                LiteralInteger literalInteger = UMLFactory.eINSTANCE.createLiteralInteger();
                result = slot.createValue(null, type, literalInteger.eClass());
                try{
                    ((LiteralInteger)result).setValue(Integer.valueOf(value.toString()));
                }catch(NumberFormatException e){
                    literalInteger.destroy();
                    Expression expr = UMLFactory.eINSTANCE.createExpression();
                    result = slot.createValue(null, type, expr.eClass());
                    ((Expression)result).setSymbol(value.toString());
        
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultValue",
                            this.objingElement.getName(),
                            "AttributeLink" );
        
                    String message = XMI.I18N.getMessage("logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + value + "\"",
                            "Integer", contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                }
        
            }
        
            else if(objType.getName().equals("UnlimitedNatural")){
                LiteralUnlimitedNatural literalUnlimitedNatural = UMLFactory.eINSTANCE.createLiteralUnlimitedNatural();
                result = slot.createValue(null, type, literalUnlimitedNatural.eClass());
                try{
                    ((LiteralUnlimitedNatural)result).setValue(Integer.valueOf(value.toString()));
                }catch(NumberFormatException e){
                    literalUnlimitedNatural.destroy();
                    Expression expr = UMLFactory.eINSTANCE.createExpression();
                    result = slot.createValue(null, type, expr.eClass());
                    ((Expression)result).setSymbol(value.toString());
                }
            }
        
            else if(objType.equals(umlTypes.getBOOLEAN())){
                LiteralBoolean literalBoolean = UMLFactory.eINSTANCE.createLiteralBoolean();
                result = slot.createValue(null, type, literalBoolean.eClass());
                try{
                    ((LiteralBoolean)result).setValue(Boolean.valueOf(value.toString()));
                }catch(Exception e){
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultValue",
                            this.objingElement.getName(),
                            "AttributeLink");
        
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + value + "\"",
                            "Boolean", contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
                }
            }
        
            else if(objType.equals(umlTypes.getSTRING()) || objType.equals(umlTypes.getCHAR())){
                LiteralString literalString = UMLFactory.eINSTANCE.createLiteralString();
                result = slot.createValue(null, type, literalString.eClass());
                ((LiteralString)result).setValue(value.toString());
              
        
            }else {
                OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
                result = slot.createValue(null, type, opaqueExpr.eClass());
                ((OpaqueExpression)result).getBodies().add(value.toString());
            }
        }else {
            OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
            ValueSpecification result = slot.createValue(null, type, opaqueExpr.eClass());
            ((OpaqueExpression)result).getBodies().add(value.toString());
        }
    }

}
