/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IBinding;
import com.modeliosoft.modelio.api.model.uml.statik.ICollaborationUse;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0dfbc136-4583-11e0-b54c-00137279a832")
public class OBinding extends OElement implements IOElement {
    @objid ("0dfbc137-4583-11e0-b54c-00137279a832")
    private IBinding objElt;


    @objid ("0dfbc138-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createDependency();
    }

    @objid ("0dfbc13c-4583-11e0-b54c-00137279a832")
    public OBinding() {
    }

    @objid ("0dfe238b-4583-11e0-b54c-00137279a832")
    public OBinding(IBinding param) {
        super(param);
        objElt = param;
    }

    @objid ("0dfe2394-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        List<IElement> objingClientList = new ArrayList<IElement>();
                
        Dependency ecoreDependency = (Dependency) ecoreElt;
                
        IElement objingClientTemp = objElt.getConnectorEndRole();
                
        if (objingClientTemp!= null){
            objingClientTemp = objingClientTemp.getCompositionOwner();
            if (objingClientTemp!= null){
                objingClientList.add(objingClientTemp);
            }
        }
                
        objingClientTemp = objElt.getConnectorRole();
                
        if (objingClientTemp!= null){
            objingClientList.add(objingClientTemp);
        }
                
        objingClientTemp = objElt.getRole();
                
        if (objingClientTemp!= null){
            objingClientList.add(objingClientTemp);
        }
                
        if (objingClientList.size() > 0) {
            for (IElement  objingClient : objingClientList){
                
                Element ecoreClient = (Element) genProp
                .getMappedElement(objingClient);
                
                if ((ecoreClient != null) &&  (ecoreClient instanceof NamedElement)) {
                
                        if  (! (ecoreDependency.getClients().contains(ecoreClient)))
                            try{
                                ecoreDependency.getClients().add((NamedElement)ecoreClient);
                            }catch(ArrayStoreException e){
                                e.printStackTrace(Modelio.err);
                            }
                    }
            }
        }
                
        IModelElement objingSupplier = objElt.getRepresentedFeature();
        Element ecoreSupplier = null;
                
        if (objingSupplier != null){
            // Gets or creates the ecore "Supplier" element:
            ecoreSupplier = (Element) genProp.getMappedElement(objingSupplier);
            
        }
                
                
        ICollaborationUse objingCollaboration = (ICollaborationUse) objElt.getCompositionOwner();
                
        CollaborationUse ecoreCollaboration = (CollaborationUse) genProp.getMappedElement(objingCollaboration);
                
        if ((ecoreCollaboration != null) &&
             ((ecoreSupplier != null) && (ecoreSupplier instanceof NamedElement ))){
            ecoreDependency.getClients().add((NamedElement)ecoreCollaboration);
            
            ecoreCollaboration.getRoleBindings().add(ecoreDependency);
            ecoreDependency.getSuppliers().add((NamedElement)ecoreSupplier);
        }else{
            ecoreDependency.destroy();
        }
    }

    @objid ("0e0085e4-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objElt.getCompositionOwner());
        if (ecoreOwner instanceof CollaborationUse){
            ((CollaborationUse) ecoreOwner).getRoleBindings().add((Dependency)ecoreElt);
        }
    }

}
