/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IChoicePseudoState;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.Vertex;

@objid ("0e1d2222-4583-11e0-b54c-00137279a832")
public class OChoicePseudoState extends OModelElement implements IOElement {
    @objid ("0e1d2223-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0e1d2224-4583-11e0-b54c-00137279a832")
    private IChoicePseudoState objingElement;

    @objid ("0e1d2225-4583-11e0-b54c-00137279a832")
    private Region ownerRegion = null;


    @objid ("0e1d2226-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createPseudostate();
    }

    @objid ("0e1f8476-4583-11e0-b54c-00137279a832")
    public OChoicePseudoState() {
    }

    @objid ("0e1f8478-4583-11e0-b54c-00137279a832")
    public OChoicePseudoState(IChoicePseudoState param) {
        super(param);                
        objingElement = param;
    }

    @objid ("0e1f847b-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
        
        if (objingOwner != null) {
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
            if (ecoreOwner != null) {
                
                if (ecoreOwner instanceof State) {
                    
                    EList<?> regions = ((State) ecoreOwner).getRegions();
                    if (regions.size() > 0) {
                        ownerRegion = (Region) regions.get(0);
                        ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                    }
                } else if (ecoreOwner instanceof Region) {
                    ownerRegion = (Region) ecoreOwner;
                    ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                } else if (ecoreOwner instanceof StateMachine) {
                    
                    EList<?> regions = ((StateMachine) ecoreOwner).getRegions();
                    if (regions.size() > 0) {
                    
                        ownerRegion = (Region) regions.get(0);
                        ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                    }
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreOwner.getClass().getSimpleName()
                            + ") Not Found");
                }
            } else {
                ecoreElt.destroy();
                throw new NotFoundException(
                        "Owner Class of ChoicePseudoState Not Found");
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class of ChoicePseudoState Not Found");
        }
    }

    @objid ("0e1f847f-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setKind((Pseudostate)ecoreElt);
    }

    @objid ("0e21e6d0-4583-11e0-b54c-00137279a832")
    private void setKind(Pseudostate pseudostate) {
        pseudostate.setKind(PseudostateKind.CHOICE_LITERAL);
    }

    @objid ("0e21e6d3-4583-11e0-b54c-00137279a832")
    public List<String> getEcoreClassName() {
        return new ArrayList<String>();
    }

}
