/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0e21e6d9-4583-11e0-b54c-00137279a832")
public class OClass extends ONameSpace implements IOElement {
    @objid ("0e21e6da-4583-11e0-b54c-00137279a832")
    private boolean isClassAssociation;


    @objid ("0e21e6db-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        Element element = null;
        if (!isClassAssociation){
            
            GenerationProperties genProp = GenerationProperties.getInstance();         
            IModelTree objingOwner = ((IClass)getObjingElement()).getOwner();
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
            
        //            if (ecoreOwner instanceof TemplateParameter) {
        //                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
        //                Element temp = createEcoreClass();
        //                element = ownerIsTemplateParameter.createOwnedParameteredElement(temp.eClass());
        //                temp.destroy();
        //                
        //            }else {
                element = createEcoreClass();
        //            }
            
        }else
            element = getOrCreateEcoreAssociationClass();
        return element;
    }

    @objid ("0e24492b-4583-11e0-b54c-00137279a832")
    private Element createEcoreClass() {
        return  UMLFactory.eINSTANCE.createClass();
    }

    @objid ("0e24492f-4583-11e0-b54c-00137279a832")
    private Element getOrCreateEcoreAssociationClass() {
        return (AssociationClass) GenerationProperties.getInstance()
                 .getMappedElement(((IClass)getObjingElement()));
    }

    @objid ("0e244933-4583-11e0-b54c-00137279a832")
    public OClass() {
    }

    @objid ("0e244935-4583-11e0-b54c-00137279a832")
    public OClass(IClass element) {
        super(element);
        if (ObjingModelNavigation.isClassAssociation(((IClass)getObjingElement())))
            isClassAssociation = true;
        else
            isClassAssociation = false;
    }

    @objid ("0e26ab8c-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // The "isClassAssociation" test may be done on sub-classes, like
        // "Component" that inherits from "Class".
        if (ObjingModelNavigation.isClassAssociation((IClass)getObjingElement()))
            isClassAssociation = true;
        else
            isClassAssociation = false;
                
        if (!isClassAssociation)
            linkEcoreClass((Class) ecoreElt);
    }

    @objid ("0e26ab90-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (!isClassAssociation)
            setClassProperties((Class) ecoreElt);
        else
            setAssociationClassProperties((AssociationClass) ecoreElt);
    }

    @objid ("0e290de1-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(Class ecoreElt) {
        ObjingEAnnotation.setPrimitive(ecoreElt, ((IClass)getObjingElement()).isElementary());
    }

    @objid ("0e290de4-4583-11e0-b54c-00137279a832")
    private void setLeaf(Class ecoreElt) {
        ecoreElt.setIsLeaf(((IClass)getObjingElement()).isLeaf());
    }

    @objid ("0e290de7-4583-11e0-b54c-00137279a832")
    private void setMainEAnnotation(Class ecoreElt) {
        ObjingEAnnotation.setMain(ecoreElt, ((IClass)getObjingElement()).isMain());
    }

    @objid ("0e290dea-4583-11e0-b54c-00137279a832")
    private void setActive(Class ecoreElt) {
        ecoreElt.setIsActive(((IClass)getObjingElement()).isActive());
    }

    @objid ("0e2b703a-4583-11e0-b54c-00137279a832")
    private void linkEcoreClass(Class ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IModelTree objingOwner = ((IClass)getObjingElement()).getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add(ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add(ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add(ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add(ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ecoreElt.setOwningTemplateParameter(ownerIsTemplateParameter);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0e2b703d-4583-11e0-b54c-00137279a832")
    private void setClassProperties(Class ecoreElt) {
        super.setProperties(ecoreElt);
        this.setLeaf(ecoreElt);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            this.setPrimitiveEAnnotation(ecoreElt);
            this.setMainEAnnotation(ecoreElt);
            this.setRootEAnnotation(ecoreElt);
        }
        
        this.setActive(ecoreElt);
    }

    @objid ("0e2b7040-4583-11e0-b54c-00137279a832")
    private void setAssociationClassProperties(AssociationClass ecoreElt) {
        super.setProperties(ecoreElt);
        this.setLeaf(ecoreElt);
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            this.setPrimitiveEAnnotation(ecoreElt);
            this.setMainEAnnotation(ecoreElt);
            this.setRootEAnnotation(ecoreElt);
        }
        this.setActive(ecoreElt);
    }

    @objid ("0e2b7043-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(Class ecoreElt) {
        ObjingEAnnotation.setRoot(ecoreElt, ((IClass)getObjingElement()).isRoot());
    }

}
