/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IClassAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Property;

@objid ("0e2dd292-4583-11e0-b54c-00137279a832")
public class OClassAssociation extends OElement implements IOElement {
    @objid ("0e2dd293-4583-11e0-b54c-00137279a832")
    private IClassAssociation objingElement;


    @objid ("0e2dd294-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        // The Ecore AssociationClass can be created when parsing the related
        // Ijing Association, Class or
        // current ClassAssociation.
        return getOrCreateEcoreAssociationClass();
    }

    @objid ("0e2dd298-4583-11e0-b54c-00137279a832")
    private AssociationClass getOrCreateEcoreAssociationClass() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        // Gets or creates the Ecore AssociationClass:
        AssociationClass assocClass = (AssociationClass) genProp
                .getMappedElement(objingElement);
                
        // // Tests if the ecore AssociationClass has been created from the
        // objing
        // // ClassAssociation:
        // AssociationClass elt = (AssociationClass)
        // genProp.getMappedElement(objingElement);
        // if (elt == null) {
        // // Tests if the ecore AssociationClass has been created from the
        // // objing Association:
        // elt = (AssociationClass) genProp.getMappedElement(objingElement
        // .getAssociationPart());
        // if (elt == null) {
        // // Tests if the ecore AssociationClass has been created from the
        // // objing Class:
        // elt = (AssociationClass) genProp.getMappedElement(objingElement
        // .getClassPart());
        // }
        // }
        // if (elt == null)
        // elt = createEcoreAssociationClass();
        return assocClass;
    }

    @objid ("0e2dd29c-4583-11e0-b54c-00137279a832")
    private boolean isClassAssociationValid() {
        // If the Class representing the current ClassAssociation represents
        // other
        // ClassAssociations, the ClassAssociation is not valid for export:
        if (ObjingModelNavigation.isRepresentedByAUniqueClass(objingElement))
            return true;
        return false;
    }

    @objid ("0e2dd2a0-4583-11e0-b54c-00137279a832")
    public OClassAssociation() {
    }

    @objid ("0e3034ed-4583-11e0-b54c-00137279a832")
    public OClassAssociation(IClassAssociation param) {
        super(param);
        objingElement = param;
    }

    @objid ("0e3034f6-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (isClassAssociationValid()) {
            GenerationProperties genProp = GenerationProperties.getInstance();
                
            IAssociation objingAssoc = objingElement.getAssociationPart();
                
            if (objingAssoc != null) {
                // The owner of the AssociationClass will be the nearest owner
                // Package of one of the connected Classifiers.
                // The owner Package will be preferentially the nearest Package
                // of a navigable end.
                IClassifier ownerClassifier = null;
                IClassifier ownerNavigableClassifier = null;
                
                for (IAssociationEnd end : objingAssoc.getConnection()) {
                    if (end.isNavigable()) {
                        ownerNavigableClassifier = end.getOwner();
                    }
                    ownerClassifier = end.getOwner();
                    Property ecoreRole = (Property) genProp
                            .getMappedElement(end);
                    if (ecoreRole != null)
                        ecoreRole.setAssociation((AssociationClass) ecoreElt);
                }
                
                if (ownerNavigableClassifier != null)
                    ownerClassifier = ownerNavigableClassifier;
                
                if (ownerClassifier != null) {
                    IPackage objingPkg = ObjingModelNavigation
                            .getNearestPackage(ownerClassifier);
                
                    Package ecorePkg = (Package) genProp
                            .getMappedElement(objingPkg);
                
                    if (ecorePkg != null) {
                        ecorePkg.getPackagedElements().add((PackageableElement)ecoreElt);
                    }
                }
            }
        }
    }

    @objid ("0e329746-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setName((AssociationClass) ecoreElt);
    }

    @objid ("0e329749-4583-11e0-b54c-00137279a832")
    private void setName(AssociationClass ecoreAC) {
        String name = objingElement.getClassPart().getName();
        if (ObjingModelNavigation.isNotNullOrEmpty(name))
            ecoreAC.setName(name);
    }

}
