/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IConditionalNode;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.ConditionalNode;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecificationAction;

@objid ("0e32974c-4583-11e0-b54c-00137279a832")
public class OClause extends OElement implements IOElement {
    @objid ("0e32974d-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = null;

    @objid ("0e32974e-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0e32974f-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createClause();
    }

    @objid ("0e34f9a0-4583-11e0-b54c-00137279a832")
    public OClause() {
    }

    @objid ("0e34f9a2-4583-11e0-b54c-00137279a832")
    public OClause(IClause element) {
        super(element);
    }

    @objid ("0e375bfb-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
                
        IConditionalNode objingOwner = ((IClause) getObjingElement()).getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner instanceof ConditionalNode) {
            ((ConditionalNode) ecoreOwner).getClauses().add((Clause)ecoreElt);
        } else {
            ecoreElt.destroy();
            String errorMsg = "";
            if (ecoreOwner == null)
                errorMsg = "Owner Class not Found";
            else
                errorMsg = "Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found";
            throw new NotFoundException(errorMsg);
        }
    }

    @objid ("0e375bff-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setBody((Clause) ecoreElt);
        setTest((Clause) ecoreElt);
        setName((Clause) ecoreElt);
    }

    @objid ("0e375c02-4583-11e0-b54c-00137279a832")
    private void setBody(Clause clause) {
        IClause objingElement = (IClause) getObjingElement();
        for (IActivityNode objingBody : objingElement.getBody()) {
            Element ecoreBody = genProp.getMappedElement(objingBody);
            if (ecoreBody instanceof ExecutableNode)
                clause.getBodies().add((ExecutableNode)ecoreBody);
        }
    }

    @objid ("0e375c05-4583-11e0-b54c-00137279a832")
    private void setTest(Clause clause) {
        String test = ((IClause) getObjingElement()).getTest();
        if (test.length() > 0) {
            ValueSpecificationAction ecoreTestAction = UMLFactory.eINSTANCE
                    .createValueSpecificationAction();
            ecoreTestAction.setName(test);
            LiteralString value = UMLFactory.eINSTANCE.createLiteralString();
            value.setValue(test);
            ecoreTestAction.setValue(value);
            clause.getTests().add(ecoreTestAction);
            attachActionToActivity(ecoreTestAction);
            ObjingEAnnotation.setDeleted(ecoreTestAction, true);
        }
    }

    @objid ("0e375c08-4583-11e0-b54c-00137279a832")
    private void setName(Clause clause) {
        ObjingEAnnotation.setName(clause, ((IClause) getObjingElement()).getName());
    }

    @objid ("0e39be54-4583-11e0-b54c-00137279a832")
    private void attachActionToActivity(ValueSpecificationAction action) {
        // Setting composition relation (attach to Activity)
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                .getEnclosingElement(((IClause) getObjingElement()), IActivity.class,
                        ownerVisitor);
        if (enclosingActivity != null) {
            Element ecoreActivity = genProp.getMappedElement(enclosingActivity);
            if (ecoreActivity instanceof Activity) {
                ((Activity) ecoreActivity).getNodes().add(action);
            }
        }
    }

}
