/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.ICollaboration;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateParameter;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0e39be57-4583-11e0-b54c-00137279a832")
public class OCollaboration extends ONameSpace implements IOElement {
    @objid ("0e39be58-4583-11e0-b54c-00137279a832")
    private ICollaboration objingElement;

    @objid ("0e39be59-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor theOwnerVisitor;


    @objid ("0e39be5a-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createCollaboration();
    }

    @objid ("0e39be5e-4583-11e0-b54c-00137279a832")
    public OCollaboration() {
    }

    @objid ("0e3c20ae-4583-11e0-b54c-00137279a832")
    public OCollaboration(ICollaboration param) {
        super(param);
        objingElement = param;
    }

    @objid ("0e3c20b7-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        theOwnerVisitor = ownerVisitor;
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
                
        // Possible owner of an objing Collaboration:
        // Behavior ; Operation ; ModelTree ; TemplateParameter
                
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (objingOwner instanceof IBehavior) {
                setOwnerBehavior((Collaboration) ecoreElt, ecoreOwner);
            } else if (objingOwner instanceof IOperation) {
                setOwnerOperation((Collaboration) ecoreElt,
                        (IOperation) objingOwner);
            } else if (objingOwner instanceof IModelTree) {
                setOwnerModelTree((Collaboration) ecoreElt,
                        (IModelTree) objingOwner, ecoreOwner);
            } else {        
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, objingElement,ecoreElt);
                String errorMsg = "";
                if (ecoreOwner == null)
                    errorMsg = "Owner Class not Found";
                else
                    errorMsg = "Owner Class ("
                            + ecoreOwner.getClass().getSimpleName()
                            + ") Not Found";
               
                throw new NotFoundException(errorMsg);
            }
        } else{
            
            ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                    objingOwner, objingElement,ecoreElt);
            String errorMsg = "";
            if (ecoreOwner == null)
                errorMsg = "Owner Class not Found";
            else
                errorMsg = "Owner Class ("
                        + ecoreOwner.getClass().getSimpleName()
                        + ") Not Found";
            throw new NotFoundException(errorMsg);
        }
    }

    @objid ("0e3c20bb-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setLeaf((Collaboration) ecoreElt);
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setConcurrentEAnnotation((Collaboration) ecoreElt);
            setRootEAnnotation((Collaboration)ecoreElt);
        }
    }

    @objid ("0e3e830e-4583-11e0-b54c-00137279a832")
    private void setConcurrentEAnnotation(Collaboration ecoreElt) {
        ObjingEAnnotation.setConcurrent(ecoreElt, objingElement.isConcurrent());
    }

    @objid ("0e3e8311-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(Collaboration ecoreElt) {
        ObjingEAnnotation.setRoot(ecoreElt, objingElement.isRoot());
    }

    @objid ("0e3e8314-4583-11e0-b54c-00137279a832")
    private void setLeaf(Collaboration ecoreElt) {
        ecoreElt.setIsLeaf(objingElement.isLeaf());
    }

    @objid ("0e40e562-4583-11e0-b54c-00137279a832")
    private void setOwnerBehavior(Collaboration ecoreCollab, Element ecoreOwner) {
        if (ecoreOwner instanceof Behavior) {
            Behavior ownerIsBehavior = (Behavior) ecoreOwner;
            ownerIsBehavior.getNestedClassifiers().add(ecoreCollab);
        }
    }

    @objid ("0e40e567-4583-11e0-b54c-00137279a832")
    private void setOwnerOperation(Collaboration ecoreCollab, IOperation objingOperation) {
        // Operations can't own Collaborations in UML2 => attach the
        // collaboration to
        // the owner of the operation
        IElement ownerOp = theOwnerVisitor
                .getCompositionOwner(objingOperation);
                
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        Element ecoreOwnerOp = genProp.getMappedElement(ownerOp);
                
        if (ecoreOwnerOp != null) {
            if (ownerOp instanceof IClassifier) {
                setOwnerModelTree(ecoreCollab, (IClassifier) ownerOp,
                        ecoreOwnerOp);
            } 
        } else{
            ecoreCollab.destroy();
            throw new NotFoundException("Owner Class not found.");
        }
    }

    @objid ("0e40e571-4583-11e0-b54c-00137279a832")
    private void setOwnerModelTree(Collaboration ecoreCollab, IModelTree objingMdlTree, Element ecoreOwner) {
        if (ecoreOwner instanceof Package) {
            Package ownerIsPkg = (Package) ecoreOwner;
            ownerIsPkg.getPackagedElements().add(ecoreCollab);
        } else if (ecoreOwner instanceof Component) {
            Component ownerIsCmpnt = (Component) ecoreOwner;
            ownerIsCmpnt.getPackagedElements().add(ecoreCollab);
        } else if (ecoreOwner instanceof Class) {
            Class ownerIsClass = (Class) ecoreOwner;
            ownerIsClass.getNestedClassifiers().add(ecoreCollab);
        } else if (ecoreOwner instanceof Interface) {
            Interface ownerIsItf = (Interface) ecoreOwner;
            ownerIsItf.getNestedClassifiers().add(ecoreCollab);
        } else {
            ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(objingMdlTree,
                    objingElement,ecoreCollab);
        }
    }

}
