/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0e4cd124-4583-11e0-b54c-00137279a832")
public class OComponent extends ONameSpace implements IOElement {
    @objid ("0e4cd125-4583-11e0-b54c-00137279a832")
    private IComponent objingElement;

    @objid ("0e4cd126-4583-11e0-b54c-00137279a832")
    private boolean isClassAssociation;


    @objid ("0e4cd127-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        Element element = null;
        if (!isClassAssociation)
            element = createEcoreComponent();
        else
            // The Ecore AssociationClass can be created when parsing the
            // related Ijing Association,
            // ClassAssociation, or current Component.
            element = getOrCreateEcoreAssociationClass();
        return element;
    }

    @objid ("0e4cd12b-4583-11e0-b54c-00137279a832")
    private Element createEcoreComponent() {
        return UMLFactory.eINSTANCE.createComponent();
    }

    @objid ("0e4cd12f-4583-11e0-b54c-00137279a832")
    private Element getOrCreateEcoreAssociationClass() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        AssociationClass assocClass = (AssociationClass) genProp
                .getMappedElement(objingElement);
                
        // // Tests if the ecore AssociationClass has been created from the
        // objing
        // // Component:
        // AssociationClass elt = (AssociationClass)
        // genProp.getMappedElement(objingElement);
        // if (elt == null) {
        // // Tests if the ecore AssociationClass has been created from the
        // // objing ClassAssociation:
        // for (IClassAssociation classAssoc : objingElement
        // .getLinkToAssociation()) {
        // elt = (AssociationClass) genProp.getMappedElement(classAssoc);
        // if (elt != null)
        // break;
        // }
        // if (elt == null) {
        // // Tests if the ecore AssociationClass has been created from the
        // // objing Association:
        // for (IClassAssociation classAssoc : objingElement
        // .getLinkToAssociation()) {
        // elt = (AssociationClass) genProp.getMappedElement(classAssoc
        // .getAssociationPart());
        // if (elt != null)
        // break;
        // }
        // }
        // }
        // if (elt == null)
        // elt = OClassAssociation.createEcoreAssociationClass();
        return assocClass;
    }

    @objid ("0e4f337e-4583-11e0-b54c-00137279a832")
    public OComponent() {
    }

    @objid ("0e4f3380-4583-11e0-b54c-00137279a832")
    public OComponent(IComponent element) {
        super(element);
        objingElement = element;
        if (ObjingModelNavigation.isClassAssociation(objingElement))
            isClassAssociation = true;
        else
            isClassAssociation = false;
    }

    @objid ("0e4f3389-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (!isClassAssociation)
            linkEcoreComponent((Component) ecoreElt);
        // else
        // linkEcoreAssociationClass((AssociationClass)ecoreElt); -> Done in
        // OClassAssociation
    }

    @objid ("0e5195db-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (!isClassAssociation)
            setComponentProperties((Component) ecoreElt);
        // else
        // setAssociationClassProperties((AssociationClass)ecoreElt); -> Done
        // when calling the super (Class)
    }

    @objid ("0e5195e4-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(Component ecoreElt) {
        ObjingEAnnotation.setPrimitive(ecoreElt, objingElement.isElementary());
    }

    @objid ("0e5195e7-4583-11e0-b54c-00137279a832")
    private void setLeaf(Component ecoreElt) {
        ecoreElt.setIsLeaf(objingElement.isLeaf());
    }

    @objid ("0e53f832-4583-11e0-b54c-00137279a832")
    private void setMainEAnnotation(Component ecoreElt) {
        ObjingEAnnotation.setMain(ecoreElt, objingElement.isMain());
    }

    @objid ("0e53f835-4583-11e0-b54c-00137279a832")
    private void setActive(Component ecoreElt) {
        ecoreElt.setIsActive(objingElement.isActive());
    }

    @objid ("0e53f83b-4583-11e0-b54c-00137279a832")
    private void linkEcoreComponent(Component ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add(ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add(ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add(ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
            // In case of a TemplateParameter, we don't handle export of
            // Components: the Ijing
            // metamodel supports it but it is not possible for an user to
            // create one through the
            // MMI (man-machine interface).
            // else
            // if (ecoreOwner instanceof TemplateParameter) {
            // TemplateParameter ownerIsTemplateParameter =
            // (TemplateParameter)ecoreOwner;
            // ownerIsTemplateParameter.setOwnedParameteredElement(arg0);
            // }
        }
    }

    @objid ("0e53f83e-4583-11e0-b54c-00137279a832")
    private void setComponentProperties(Component ecoreElt) {
        super.setProperties(ecoreElt);
        setPrimitiveEAnnotation(ecoreElt);
        setLeaf(ecoreElt);
        setMainEAnnotation(ecoreElt);
        // setRoot(); -> Undefined on UML2 Components.
        setActive(ecoreElt);
    }

}
