/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.state.IConnectionPointReference;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEntryPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IExitPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.ConnectionPointReference;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Vertex;

@objid ("0e5b1f42-4583-11e0-b54c-00137279a832")
public class OConnectionPointReference extends ONameSpace implements IOElement {
    @objid ("0e5b1f43-4583-11e0-b54c-00137279a832")
     IConnectionPointReference objingElement;

    @objid ("0e5b1f44-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0e5b1f45-4583-11e0-b54c-00137279a832")
    private Region ownerRegion = null;


    @objid ("0e5b1f46-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IState objOwner = objingElement.getOwnerState();
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objOwner);
        if ((ecoreOwner != null) &&  (ecoreOwner instanceof State)){               
                return ((State) ecoreOwner).createConnection(objingElement.getName());
        }
        return null;
    }

    @objid ("0e5b1f4a-4583-11e0-b54c-00137279a832")
    public OConnectionPointReference() {
    }

    @objid ("0e5b1f4c-4583-11e0-b54c-00137279a832")
    public OConnectionPointReference(IConnectionPointReference objingElt) {
        super(objingElt);
        objingElement = objingElt;
    }

    @objid ("0e5d81a2-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
        
        if (objingOwner != null) {
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
            if (ecoreOwner != null) {
                
                if (ecoreOwner instanceof State) {
                    ((ConnectionPointReference) ecoreElt).setState((State)ecoreOwner);
                } else if (ecoreOwner instanceof Region) {
                    ownerRegion = (Region) ecoreOwner;
                    ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                } else if (ecoreOwner instanceof StateMachine) {
                    
                    EList<?> regions = ((StateMachine) ecoreOwner).getRegions();
                    if (regions.size() > 0) {
                    
                        ownerRegion = (Region) regions.get(0);
                        ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                    }
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreOwner.getClass().getSimpleName()
                            + ") Not Found");
                }
            } else {
                ecoreElt.destroy();
                throw new NotFoundException(
                        "Owner Class of FinalState Not Found");
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class of FinalState Not Found");
        }
                
                //        Modelio.err.println("attach"); 
                //        IState objOwner = objingElement.getOwnerState();
                //        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objOwner);
                //        if (ecoreOwner != null){
                //        if (ecoreOwner instanceof State){
                //        ((State) ecoreOwner).C.getConnectionPoints().add((ConnectionPointReference) ecoreElt);                
                
                //        }else{
                
                //        }
                //        }else{
                //        throw new NotFoundException("Owner Class ("
                //        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
                //        }
    }

    @objid ("0e5d81a6-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setExit((ConnectionPointReference) ecoreElt);
        setEntry((ConnectionPointReference) ecoreElt);
    }

    @objid ("0e5fe3f4-4583-11e0-b54c-00137279a832")
    private void setEntry(ConnectionPointReference ecoreElt) {
        IEntryPointPseudoState objEntry = objingElement.getEntry();
        if (objEntry != null){
            Element ecoreEntry = GenerationProperties.getInstance().getMappedElement(objEntry);
            if (ecoreEntry != null){
                if ((ecoreEntry instanceof Pseudostate) && ((Pseudostate) ecoreEntry).getKind().equals(PseudostateKind.ENTRY_POINT_LITERAL)){
                    ((ConnectionPointReference) ecoreElt).getEntries().add((Pseudostate) ecoreEntry);                
                
                }
            }else{
                ecoreElt.destroy();
                throw new NotFoundException("Entry Point of ("
                        + ecoreElt.getName() + ") Not Found");
            }
        }
    }

    @objid ("0e5fe3f7-4583-11e0-b54c-00137279a832")
    private void setExit(ConnectionPointReference ecoreElt) {
        IExitPointPseudoState objExit = objingElement.getExit();
        if (objExit != null){
            Element ecoreExit = GenerationProperties.getInstance().getMappedElement(objExit);
            if (ecoreExit != null){
                if ((ecoreExit instanceof Pseudostate) && ((Pseudostate) ecoreExit).getKind().equals(PseudostateKind.EXIT_POINT_LITERAL)){
                    ((ConnectionPointReference) ecoreElt).getExits().add((Pseudostate) ecoreExit);                
                
                }
            }else{
                ecoreElt.destroy();
                throw new NotFoundException("Entry Point ("
                        + ecoreElt.getName() + ") Not Found");
            }
        }
    }

}
