/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IConnector;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.ILinkEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorKind;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IConnector elements
 * @author ebrosse
 */
@objid ("0e5fe3fd-4583-11e0-b54c-00137279a832")
public class OConnector extends OModelElement {
    @objid ("0e5fe3fe-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0e5fe3ff-4583-11e0-b54c-00137279a832")
    private IConnector objElt;

    @objid ("0e5fe400-4583-11e0-b54c-00137279a832")
    private IElement root = null;


    @Override
    @objid ("0e5fe401-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        this.root = ObjingModelNavigation.getConnectorOwner(this.objElt);
        
        if (this.root != null){
            if (this.root instanceof IInstance){
                if (this.objElt.getCompositionOwner().getCompositionOwner().equals(this.root))
                    return UMLFactory.eINSTANCE.createInstanceSpecification();
                else{
        
                    XMILogs xmilogs = XMILogs.getInstance();
                    xmilogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.unsupportedExport", 
                                                             this.objElt.getName(), 
                                                             this.objElt.getClass().getName()));
                    return null;
                }
        
            }else 
                return UMLFactory.eINSTANCE.createConnector();
        }else{
            return null;
        }
    }

    /**
     * Constructor
     * @param param : the exported Modelio IConnector
     */
    @objid ("0e624650-4583-11e0-b54c-00137279a832")
    public OConnector(final IConnector param) {
        super(param);                
        this.objElt = param;
    }

    @Override
    @objid ("0e624659-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt instanceof Connector ){
            attachConnector(ecoreElt);
        }else{
            attachInstanceSpecification(ecoreElt);
        }
    }

    @objid ("0e670b02-4583-11e0-b54c-00137279a832")
    private void attachConnector(Element ecoreElt) {
        Element ecoreOwner = null;
        for (ILinkEnd linkEnd : this.objElt.getConnection()) {
        
            if (linkEnd instanceof IConnectorEnd){
                // Links the Association to the OwnerClassifier of one of the
                // Properties
                // (sets the owner of the Association):
        
                IConnectorEnd connectorEnd = (IConnectorEnd) linkEnd;
        
                if (ecoreOwner == null) {
                    IElement objOwner = connectorEnd.getLinked().getCompositionOwner();
                    ecoreOwner =  this.genProp.getMappedElement(objOwner);
                    if (ecoreOwner != null) {
                        // Components and Nodes are handled as Class
                        // (they inherit from Class)
        
                        // Class, Component, Node
                        if (ecoreOwner instanceof StructuredClassifier) {
                            StructuredClassifier ownerIsClass = (StructuredClassifier) ecoreOwner;
                            ownerIsClass.getOwnedConnectors().add((Connector)ecoreElt);
                        } else if (objOwner instanceof IBindableInstance) {
        
                            StructuredClassifier ownerIsClass = (StructuredClassifier)  this.genProp.getMappedElement(ObjingModelNavigation
                                    .getBindableInstanceOwner(objOwner));
                            ownerIsClass.getOwnedConnectors().add((Connector)ecoreElt);
                        } else {
                            ecoreElt.destroy();
                            throw new NotFoundException("Owner Class ("
                                    + ecoreOwner.getClass().getSimpleName()
                                    + ") Not Found");
                        }
                    }
                }
            }
        }
    }

    @objid ("0e696d5e-4583-11e0-b54c-00137279a832")
    private void attachInstanceSpecification(Element ecoreElt) {
        if (this.root == null)
            this.root = ObjingModelNavigation.getBindableInstanceOwner(this.objElt.getCompositionOwner());
        
        Element ecoreRoot = this.genProp.getMappedElement(this.root);
        
        IElement objingOwner = this.root.getCompositionOwner();
        if (ecoreRoot instanceof InstanceSpecification){
            //            Element ecoreOwner =  ((InstanceSpecification) root).getOwner();
            Element ecoreOwner = this.genProp.getMappedElement(objingOwner);
        
            if (ecoreOwner != null) {
                if (ecoreOwner instanceof Package) {
                    Package ownerIsPkg = (Package) ecoreOwner;
                    ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
        
                } else if (ecoreOwner instanceof Component) {
                    Component ownerIsCmpnt = (Component) ecoreOwner;
                    ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
                } else if (objingOwner instanceof IModelTree) {
                    Package ownerIsPkg = (Package) this.genProp.getMappedElement(
                            ObjingModelNavigation.getNearestPackage((IModelTree) objingOwner));
                    ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
        
                    if (!(objingOwner.equals(ownerIsPkg))){
                        XMILogs logs = XMILogs.getInstance();
                        String message = XMI.I18N.getMessage(
                                                            "logFile.exception.stringConverter.ownerChange",
                                                            ecoreElt.getClass().toString(),((IModelTree) objingOwner).getName(),
                                                            ownerIsPkg.getName());
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addWarning(message, this.objElt);
                    }
                }else{
                    XMILogs logs = XMILogs.getInstance();
                    String message = XMI.I18N.getMessage(
                                                         "logFile.exception.stringConverter.haveNotOwner",
                                                         ecoreElt.getClass().toString());
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addError(message, this.objElt);
                }
        
            }
        
        }
    }

    @Override
    @objid ("0e696d61-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof InstanceSpecification)
            setInstanceProperties((InstanceSpecification) ecoreElt);
        else
            setConnectorProperties((Connector) ecoreElt);
        
        if (this.genProp.isRoundtripEnabled() && (ecoreElt instanceof InstanceSpecification))
            ObjingEAnnotation.setConnector(ecoreElt, true);
    }

    @objid ("0e696d64-4583-11e0-b54c-00137279a832")
    private void setInstanceProperties(InstanceSpecification ecoreElt) {
        ecoreElt.setName(this.objElt.getName());
    }

    @objid ("0e696d67-4583-11e0-b54c-00137279a832")
    private void setConnectorProperties(Connector ecoreElt) {
        super.setProperties(ecoreElt);
        setBase(ecoreElt);
        setKind(ecoreElt);
    }

    @objid ("0e696d6a-4583-11e0-b54c-00137279a832")
    private void setBase(Connector ecoreElt) {
        IElement temp = this.objElt.getBase();
        if (temp != null){
            Element ecoreType = this.genProp.getMappedElement(temp);
            if (ecoreType instanceof Association)
                ecoreElt.setType((Association) ecoreType);
        
        }
    }

    @objid ("0e6bcfb6-4583-11e0-b54c-00137279a832")
    private void setKind(Connector ecoreElt) {
        boolean assembly = true;
        
        for (ILinkEnd end : this.objElt.getConnection()){
            if (end.isNavigable())
                assembly = false;
        
        }
        if (!assembly)
            ecoreElt.setKind(ConnectorKind.DELEGATION_LITERAL);
    }

}
