/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.ConnectorKind;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0e6bcfb9-4583-11e0-b54c-00137279a832")
public class OConnectorEnd extends OElement implements IOElement {
    @objid ("0e6bcfba-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0e6bcfbb-4583-11e0-b54c-00137279a832")
    private IConnectorEnd objElt;


    @objid ("0e6bcfbc-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IElement root  = ObjingModelNavigation.getBindableInstanceOwner(objElt);
        IElement owner = objElt.getCompositionOwner();
                
        if ((owner instanceof IInstance) && (owner.getCompositionOwner().equals(root))){
            return UMLFactory.eINSTANCE.createConnectorEnd();
        }else if (root instanceof IInstance){
                return UMLFactory.eINSTANCE.createSlot();
        }else {
            return UMLFactory.eINSTANCE.createConnectorEnd();
        }
    }

    @objid ("0e6bcfbf-4583-11e0-b54c-00137279a832")
    public OConnectorEnd() {
    }

    @objid ("0e6bcfc1-4583-11e0-b54c-00137279a832")
    public OConnectorEnd(IConnectorEnd param) {
        super(param);
        objElt = param;
    }

    @objid ("0e6e3218-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt instanceof ConnectorEnd)
            attachConnectorEnd((ConnectorEnd) ecoreElt);
        else if (ecoreElt instanceof Slot)
            attachSlot((Slot) ecoreElt);
    }

    @objid ("0e6e321c-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Slot){
            setLinked((Slot) ecoreElt);
            setDefiningFeature((Slot) ecoreElt);           
        }
        
        if (ecoreElt instanceof ConnectorEnd){
            ordered((ConnectorEnd) ecoreElt);
        }
    }

    @objid ("0e6e321f-4583-11e0-b54c-00137279a832")
    private void attachConnectorEnd(ConnectorEnd connectorEnd) {
        Element owner =  genProp.getMappedElement(objElt.getLinkNode());
        if (owner instanceof Connector)
            ((Connector) owner).getEnds().add(connectorEnd);
    }

    @objid ("0e70946c-4583-11e0-b54c-00137279a832")
    private void attachSlot(Slot slot) {
        Element owner =  genProp.getMappedElement(objElt.getLinkNode());
        if (owner instanceof InstanceSpecification)
            ((InstanceSpecification)owner).getSlots().add(slot);
    }

    @objid ("0e70946f-4583-11e0-b54c-00137279a832")
    private void setLinked(Slot ecoreElt) {
        Element inst = genProp.getMappedElement(objElt.getLinked());
        if (inst instanceof Slot){
            InstanceValue instanceValue = UMLFactory.eINSTANCE.createInstanceValue();
            ValueSpecification result = ecoreElt.createValue(null, null, instanceValue.eClass());
            ((InstanceValue)result).setInstance(((Slot)inst).getOwningInstance());
            if (ObjingModelNavigation.isNotNullOrEmpty(objElt.getName()))
                ((InstanceValue)result).setName(objElt.getName());
                
        }
        
        ObjingEAnnotation.setOwner(ecoreElt, String.valueOf(objElt.getLinked().getIdentifier()));
    }

    @objid ("0e709472-4583-11e0-b54c-00137279a832")
    private void ordered(ConnectorEnd ecoreElt) {
        Connector connector = (Connector) ecoreElt.getOwner();
        if (connector.getKind().equals(ConnectorKind.DELEGATION_LITERAL)){
            if (objElt.isNavigable()){
                connector.getEnds().remove(ecoreElt);
                
                connector.getEnds().add(connector.getEnds().size() - 1,ecoreElt);
            }
        }
    }

    @objid ("65d5caf2-1a76-11e1-8103-0027103f347d")
    private void setDefiningFeature(final Slot ecoreElt) {
        IInstance instanceOwner = objElt.getLinked();
        ILink link = objElt.getLinkNode();
        if ((instanceOwner != null ) && (link != null)){
            IAssociation assoc = link.getBase();
            INameSpace namespace = instanceOwner.getBase();
            if ((assoc != null ) && (namespace != null)){
                for (IAssociationEnd assocEnd : assoc.getConnection()){
                    if (assocEnd.getOwner().equals(namespace)){
                        Element ecoreProp = genProp.getMappedElement(assocEnd);
                        if (ecoreProp instanceof StructuralFeature){
                            ecoreElt.setDefiningFeature((StructuralFeature)ecoreProp);
                        }
                    }
                }
            }
        }
    }

}
