/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IConstraint;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.DurationObservation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * Export IConstraint
 * @author ebrosse
 */
@objid ("0e709475-4583-11e0-b54c-00137279a832")
public class OConstraint extends OModelElement implements IOElement {
    @objid ("0e709476-4583-11e0-b54c-00137279a832")
    private IConstraint objingElement;


    @Override
    @objid ("0e709477-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (this.objingElement.isStereotyped("DurationObservation_Constraint")){
            return UMLFactory.eINSTANCE.createDurationObservation();
        }
        return UMLFactory.eINSTANCE.createConstraint();
    }

    @objid ("0e72f6c7-4583-11e0-b54c-00137279a832")
    public OConstraint(IConstraint element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0e75591e-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        if (ecoreElt instanceof Constraint){
            //Cosntraint case 
            // Exceptionally sets the Properties of the Constraint, as we need to
            // know the Name and Body of the
            // current constraint.
            setProperties(ecoreElt);
        
            // Gets the ecore constrained elements:
            List<Element> ecoreConstrainedElts = new ArrayList<Element>();
            boolean isMappingPossible = true;
            ObList<IModelElement> objingConstrainedElts = this.objingElement
                    .getConstrainedElement();
            for (IModelElement objingConstrainedElt : objingConstrainedElts) {
                Element ecoreConstrainedElt = genProp
                        .getMappedElement(objingConstrainedElt);
                if (ecoreConstrainedElt == null)
                    return;// The constraint will be linked later: all the
                // constrained elements must have been
                // created.
        
                if (!(ecoreConstrainedElt instanceof Namespace))
                    isMappingPossible = false;
                ecoreConstrainedElts.add(ecoreConstrainedElt);
            }
        
            if (isMappingPossible) {
        
                for (Element ecoreConstrainedElt : ecoreConstrainedElts) {
                    Namespace eConstrainedElt = (Namespace) ecoreConstrainedElt;
                    if (!eConstrainedElt.getOwnedRules().contains(ecoreElt))
                        eConstrainedElt.getOwnedRules().add((Constraint)ecoreElt);
                }
                ((Constraint) ecoreElt).getConstrainedElements().addAll(
                                                                        ecoreConstrainedElts);
                setContext((Constraint) ecoreElt);
        
            } else {
                for (Element ecoreConstrainedElt : ecoreConstrainedElts) {
                    EAnnotation objingEA = ObjingEAnnotation
                            .getOrCreateObjingEAnnotation(ecoreConstrainedElt);
                    if ((objingEA != null)
                            && (!ObjingEAnnotation.existsConstraintInEA(objingEA,
                                                                        (Constraint) ecoreElt))) {
                        objingEA.getContents().add(ecoreElt);
                    }
                }
                ((Constraint) ecoreElt).getConstrainedElements().addAll(
                                                                        ecoreConstrainedElts);
        
            }
        }else{
            if (objingElement.getConstrainedElement().size() == 2 ){
                //Duration Observation
                IModelElement constrainedElt = this.objingElement.getConstrainedElement().get(0);
                Element ecoreOwner = (Element) genProp
                        .getMappedElement(ObjingModelNavigation.getNearestPackage(constrainedElt));
                if (ecoreOwner instanceof Package){
                    DurationObservation durObs = (DurationObservation) ecoreElt;
                    ((Package) ecoreOwner).getPackagedElements().add(durObs);
                    durObs.getEvents().add((NamedElement) genProp
                                           .getMappedElement(this.objingElement.getConstrainedElement().get(0)) );
                    durObs.getEvents().add((NamedElement) genProp
                                           .getMappedElement(this.objingElement.getConstrainedElement().get(1)) );
                    durObs.getFirstEvents().add(true);
                    durObs.getFirstEvents().add(false);
                }else{
                    ecoreElt.destroy();
                }
            }
        }
    }

    @Override
    @objid ("0e755922-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Constraint){
            super.setProperties(ecoreElt);
            setSpecification((Constraint) ecoreElt);
            setContext((Constraint) ecoreElt);
        }else{
            setName((DurationObservation) ecoreElt);
        }
    }

    @objid ("0e755928-4583-11e0-b54c-00137279a832")
    private void setSpecification(Constraint ecoreElt) {
        LiteralString valueSpecification = UMLFactory.eINSTANCE
                .createLiteralString();
        valueSpecification.setValue(this.objingElement.getBody());
        ecoreElt.setSpecification(valueSpecification);
    }

    @objid ("0e75592b-4583-11e0-b54c-00137279a832")
    private void setContext(Constraint constraint) {
        Model ecoreModel = GenerationProperties.getInstance().getEcoreModel();
        constraint.setContext(ecoreModel);
    }

    @objid ("af56f587-3ace-11e1-9527-0027103f347d")
    private void setName(final DurationObservation ecoreElt) {
        String name = this.objingElement.getName();
        
        if ((name != null) &&  (!name.equals(""))){
            ecoreElt.setName(name);
        }
    }

}
