/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IControlFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IDecisionMergeNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IForkJoinNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInterruptibleActivityRegion;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ControlFlow;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InterruptibleActivityRegion;
import org.eclipse.uml2.uml.JoinNode;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.MergeNode;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0e77bb78-4583-11e0-b54c-00137279a832")
public class OControlFlow extends OModelElement implements IOElement {
    @objid ("0e77bb79-4583-11e0-b54c-00137279a832")
    private IControlFlow objingElement;

    @objid ("0e77bb7a-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = null;

    @objid ("0e77bb7b-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0e77bb7c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createControlFlow();
    }

    @objid ("0e77bb80-4583-11e0-b54c-00137279a832")
    public OControlFlow() {
    }

    @objid ("0e77bb82-4583-11e0-b54c-00137279a832")
    public OControlFlow(IControlFlow element) {
        super(element);
        objingElement = element;
    }

    @objid ("0e7a1dd8-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
        
        IActivityNode ownerNode = objingElement.getSource();
        
        IElement ownerOfIjingNode = ownerVisitor
        .getCompositionOwner(ownerNode);
        
        Element ownerOfEcoreNode = genProp.getMappedElement(ownerOfIjingNode);
        
        if (ownerOfEcoreNode != null) {
            if (ownerOfEcoreNode instanceof StructuredActivityNode) {
                ((StructuredActivityNode) ownerOfEcoreNode).getEdges().add(
                        (ActivityEdge)ecoreElt);
            } else {
                attachToActivity(ecoreElt);
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class not found.");
        }
    }

    @objid ("0e7a1ddc-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setSource((ControlFlow) ecoreElt);
        setTarget((ControlFlow) ecoreElt);
        setInterrupts((ControlFlow) ecoreElt);
        setGuard((ControlFlow) ecoreElt);
        setWeight((ControlFlow) ecoreElt);
    }

    @objid ("0e7a1ddf-4583-11e0-b54c-00137279a832")
    private void attachToActivity(Element ecoreElt) {
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
        .getEnclosingElement(objingElement.getSource(),
                IActivity.class, ownerVisitor);
        if (enclosingActivity != null) {
            Element ecoreActivity = genProp.getMappedElement(enclosingActivity);
        
            if (ecoreActivity != null) {
                if (ecoreActivity instanceof Activity) {
                    Activity owner = (Activity) ecoreActivity;
                    owner.getEdges().add((ActivityEdge)ecoreElt);
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreActivity.getClass().getSimpleName()
                            + ") Not Found");
                }
            } else{
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class not found.");
            }
        }
    }

    @objid ("0e7c802f-4583-11e0-b54c-00137279a832")
    private void setSource(ControlFlow flow) {
        IActivityNode objingSource = objingElement.getSource();
        Element ecoreSource = genProp.getMappedElement(objingSource);
        
        if (ecoreSource instanceof ActivityNode) {
            // Particular case of a MergeNode:
            // report to the mapper of DecisionMergeNode for more information.
            // In case of a DecisionMergeNode that is a Merge AND a Decision,
            // a special structure is constructed. In this case, when exporting
            // a flow, its source can't be theMergeNode, it must be the
            // DecisionNode.
            if (ecoreSource instanceof MergeNode
                    && !ObjingModelNavigation
                    .isMergeNode((IDecisionMergeNode) objingSource)
                    && !ObjingModelNavigation
                    .isDecisionNode((IDecisionMergeNode) objingSource)) {
                EList<?> activityEdgeList = ((MergeNode) ecoreSource)
                .getOutgoings();
                if (activityEdgeList != null && activityEdgeList.size() > 0) {
                    ActivityEdge linkingFlow = (ActivityEdge) activityEdgeList
                    .get(0);
                    ActivityNode source = linkingFlow.getTarget();
                    flow.setSource(source);
                }
            } else if (ecoreSource instanceof JoinNode
                    && !ObjingModelNavigation
                    .isJoinNode((IForkJoinNode) objingSource)
                    && !ObjingModelNavigation
                    .isForkNode((IForkJoinNode) objingSource)) {
                // Same principle for ForkJoinNode elements:
                    EList<?> activityEdgeList = ((JoinNode) ecoreSource)
                    .getOutgoings();
                if (activityEdgeList != null && activityEdgeList.size() > 0) {
                    ActivityEdge linkingFlow = (ActivityEdge) activityEdgeList
                    .get(0);
                    ActivityNode source = linkingFlow.getTarget();
                    flow.setSource(source);
                }
            } else {
                flow.setSource((ActivityNode) ecoreSource);
            }
        }
    }

    @objid ("0e7c8032-4583-11e0-b54c-00137279a832")
    private void setTarget(ControlFlow flow) {
        IActivityNode objingTarget = objingElement.getTarget();
        
        if (objingTarget != null) {
            Element ecoreTarget = genProp.getMappedElement(objingTarget);
        
            if (ecoreTarget instanceof ActivityNode) {
                // In case of a DecisionMergeNode as a target, if the
                // mapping is normal, there is no particular treatment.
                // If the mapping is complex, the following structure is
                // created: the DecisionMergeNode is mapped as a Merge
                // connected to a Decision by a flow.
                // the "getMappedElement" method returns the MergeNode.
                // As a consequence, there is no particular treatment
                // to do (the target of a flow must be the MergeNode).
                flow.setTarget((ActivityNode) ecoreTarget);
            }
        }
    }

    @objid ("0e7c8035-4583-11e0-b54c-00137279a832")
    private void setInterrupts(ControlFlow flow) {
        IInterruptibleActivityRegion objingRegion = objingElement
        .getInterrupts();
        
        if (objingRegion != null) {
            Element ecoreRegion = genProp.getMappedElement(objingRegion);
            if (ecoreRegion instanceof InterruptibleActivityRegion)
                flow.setInterrupts((InterruptibleActivityRegion) ecoreRegion);
        }
    }

    @objid ("0e7c8038-4583-11e0-b54c-00137279a832")
    private void setGuard(ControlFlow flow) {
        String guard = objingElement.getGuard();
        if (guard.length() > 0) {
            OpaqueExpression ecoreGuard = UMLFactory.eINSTANCE
            .createOpaqueExpression();
            ecoreGuard.getBodies().add(guard);
            flow.setGuard(ecoreGuard);
        }
    }

    @objid ("0e7c803b-4583-11e0-b54c-00137279a832")
    private void setWeight(ControlFlow flow) {
        String weight = objingElement.getWeight();
        if (weight.length() > 0) {
            try{
                int weightInt = Integer.valueOf(weight);
                LiteralInteger ecoreWeight = UMLFactory.eINSTANCE
                .createLiteralInteger();
                ecoreWeight.setValue(weightInt);
                flow.setWeight(ecoreWeight);
            }catch(Exception e){
                LiteralString ecoreWeight = UMLFactory.eINSTANCE
                .createLiteralString();
                ecoreWeight.setValue(weight);
                flow.setWeight(ecoreWeight);
            }
        }
    }

    @objid ("0e7ee288-4583-11e0-b54c-00137279a832")
    public IControlFlow getObjingElement() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.objingElement;
    }

    @objid ("0e7ee28c-4583-11e0-b54c-00137279a832")
    public CompositionOwnerVisitor getOwnerVisitor() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.ownerVisitor;
    }

    @objid ("0e7ee290-4583-11e0-b54c-00137279a832")
    public GenerationProperties getGenProp() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.genProp;
    }

}
