/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of Modelio IDatatype
 * @author ebrosse
 */
@objid ("0e8ace4a-4583-11e0-b54c-00137279a832")
public class ODataType extends ONameSpace {
    @objid ("0e8ace4b-4583-11e0-b54c-00137279a832")
    private IDataType objingElement;


    @Override
    @objid ("0e8ace4d-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IElement objingOwner = this.objingElement.getCompositionOwner();
        
        if ( objingOwner instanceof ISignal) {
            return null;
        }
        
        if (PrimitiveTypeMapper.isPredefinedType(this.objingElement)){
            return PrimitiveTypeMapper.getEcoreType(this.objingElement);
        }else if (this.objingElement.isElementary())
            return UMLFactory.eINSTANCE.createPrimitiveType();
        else
            return UMLFactory.eINSTANCE.createDataType();
    }

    /**
     * The default constructor
     */
    @objid ("0e8ace50-4583-11e0-b54c-00137279a832")
    public ODataType() {
    }

    /**
     * Constructor with an the exported IDataType
     * @param element : the exported IDataType
     */
    @objid ("0e8ace52-4583-11e0-b54c-00137279a832")
    public ODataType(IDataType element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0e8d30a8-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IModelTree objingOwner = this.objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ((Classifier)ecoreElt).setOwningTemplateParameter(ownerIsTemplateParameter);
            } else if (ecoreOwner instanceof Signal) {
                // In UML2, Signals can't own DataTypes.
                /*
                 * Signal ownerIsSignal = (Signal)ecoreOwner; EAnnotation
                 * objingEA =
                 * ObjingEAnnotation.getOrCreateObjingEAnnotation(ownerIsSignal);
                 * if (objingEA != null) objingEA.getContents().add(ecoreElt);
                 */
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, this.objingElement, ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @Override
    @objid ("0e8d30ac-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        // -> isPrimitive => export as a PrimitiveType element
        // -> else => export as a DataType element.
        setLeaf((DataType) ecoreElt);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setRootEAnnotation( ecoreElt);
            setPrimitiveEAnnotation(ecoreElt, this.objingElement.isElementary());       
        }
    }

    @objid ("0e8f92fc-4583-11e0-b54c-00137279a832")
    private void setLeaf(DataType ecoreElt) {
        ecoreElt.setIsLeaf(this.objingElement.isLeaf());
    }

    @objid ("0e8f9308-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(Element ecoreElt) {
        ObjingEAnnotation.setRoot(ecoreElt, this.objingElement.isRoot());
    }

    @objid ("0e8f930b-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(Element ecoreElt, boolean isPrimitive) {
        ObjingEAnnotation.setPrimitive(ecoreElt, isPrimitive);
    }

}
