/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityPartition;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IConditionalNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IControlFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IDecisionMergeNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IStructuredActivityNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.DecisionNode;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.MergeNode;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0e91f559-4583-11e0-b54c-00137279a832")
public class ODecisionMergeNode extends OModelElement implements IOElement {
    @objid ("0e91f55a-4583-11e0-b54c-00137279a832")
    private boolean partialCreation = true;

    @objid ("0e91f55b-4583-11e0-b54c-00137279a832")
    private boolean isMergeNode = false;

    @objid ("0e91f55c-4583-11e0-b54c-00137279a832")
    private boolean isDecisionNode = false;

    @objid ("0e91f55d-4583-11e0-b54c-00137279a832")
    private IDecisionMergeNode objingElement;

    @objid ("0e91f55e-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = null;

    @objid ("0e91f560-4583-11e0-b54c-00137279a832")
    private DecisionNode decisionNode;


    @objid ("0e91f561-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        this.partialCreation = false;
        
        // If the DecisionMerge is not only Decision or Merge, we create a
        // special structure:
        // a Merge connected to a Decision by a flow.
        if (this.isMergeNode)
            return UMLFactory.eINSTANCE.createMergeNode();
        else if (this.isDecisionNode)
            return UMLFactory.eINSTANCE.createDecisionNode();
        else
            return UMLFactory.eINSTANCE.createMergeNode();
    }

    @objid ("0e91f564-4583-11e0-b54c-00137279a832")
    public ODecisionMergeNode() {
    }

    @objid ("0e96ba0a-4583-11e0-b54c-00137279a832")
    public ODecisionMergeNode(IDecisionMergeNode element) {
        super(element);
        this.objingElement = element;
        if (ObjingModelNavigation.isMergeNode(element))
            this.isMergeNode = true;
        else if (ObjingModelNavigation.isDecisionNode(element))
            this.isDecisionNode = true;
    }

    @objid ("0e96ba13-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
        
        if (!this.isMergeNode && !this.isDecisionNode) {
            if (!this.partialCreation) {
                createStructure((MergeNode) ecoreElt);
            } else {
                // The structure has already been created in the partial
                // creation
                // method.
                this.decisionNode = getDecisionNode((MergeNode) ecoreElt);
            }
        }
        
        IElement objingOwner = ownerVisitor.getCompositionOwner(this.objingElement);
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (objingOwner instanceof IStructuredActivityNode) {
                attachToStructuredActivityNode(ecoreElt, ecoreOwner);
                if (!this.isMergeNode && !this.isDecisionNode)
                    attachToStructuredActivityNode(this.decisionNode, ecoreOwner);
            } else if (objingOwner instanceof IActivityPartition) {
                attachToActivityPartition(ecoreElt, ecoreOwner);
                if (!this.isMergeNode && !this.isDecisionNode)
                    attachToActivityPartition(this.decisionNode, ecoreOwner);
            } else if (objingOwner instanceof IActivity) {
                attachToActivity(ecoreElt, ecoreOwner);
                if (!this.isMergeNode && !this.isDecisionNode)
                    attachToActivity(this.decisionNode, ecoreOwner);
            } else if (objingOwner instanceof IClause) {
                attachToClause(ecoreElt, ecoreOwner, (IClause) objingOwner);
                if (!this.isMergeNode && !this.isDecisionNode)
                    attachToClause(this.decisionNode, ecoreOwner,
                                   (IClause) objingOwner);
            }
        }
    }

    @objid ("0e991c64-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        
        if (this.isDecisionNode)
            setDecisionInput((DecisionNode) ecoreElt);
        
        if (!this.isMergeNode && !this.isDecisionNode) {
            super.setProperties(this.decisionNode);
            setDecisionInput(this.decisionNode);
        }
    }

    @objid ("0e991c67-4583-11e0-b54c-00137279a832")
    private void createStructure(MergeNode mergeNode) {
        this.decisionNode = UMLFactory.eINSTANCE.createDecisionNode();
        
        IActivityEdge typeOfEdge = null;
        ObList<IActivityEdge> objingInc = this.objingElement.getIncoming();
        ObList<IActivityEdge> objingOut = this.objingElement.getOutgoing();
        
        if (objingInc.size() > 0) {
            typeOfEdge = objingInc.get(0);
        } else if (objingOut.size() > 0) {
            typeOfEdge = objingOut.get(0);
        }
        
        ActivityEdge ecoreFlow = null;
        if (typeOfEdge instanceof IControlFlow)
            ecoreFlow = UMLFactory.eINSTANCE.createControlFlow();
        else
            ecoreFlow = UMLFactory.eINSTANCE.createObjectFlow();
        
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                .getEnclosingElement(this.objingElement, IActivity.class,
                                     this.ownerVisitor);
        
        if (enclosingActivity != null) {
            Element ecoreActivity = GenerationProperties.getInstance()
                    .getMappedElement(enclosingActivity);
            if (ecoreActivity instanceof Activity) {
                Activity owner = (Activity) ecoreActivity;
                owner.getEdges().add(ecoreFlow);
            }
        }
        
        ecoreFlow.setSource(mergeNode);
        ecoreFlow.setTarget(decisionNode);
    }

    @objid ("0e991c6a-4583-11e0-b54c-00137279a832")
    private DecisionNode getDecisionNode(MergeNode mergeNode) {
        EList<?> edgeList = mergeNode.getOutgoings();
        if (edgeList != null && edgeList.size() > 0) {
            ActivityEdge flow = (ActivityEdge) edgeList.get(0);
            ActivityNode decisionnode = flow.getTarget();
            if (decisionnode instanceof DecisionNode)
                return (DecisionNode) decisionnode;
        }
        return null;
    }

    @objid ("0e991c6f-4583-11e0-b54c-00137279a832")
    private void attachToStructuredActivityNode(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof StructuredActivityNode) {
            StructuredActivityNode owner = (StructuredActivityNode) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0e991c73-4583-11e0-b54c-00137279a832")
    private void attachToActivityPartition(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof ActivityPartition) {
            ActivityPartition owner = (ActivityPartition) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.objingElement, IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity = GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0e9b7ebf-4583-11e0-b54c-00137279a832")
    private void attachToActivity(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof Activity) {
            Activity owner = (Activity) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0e9b7ec3-4583-11e0-b54c-00137279a832")
    private void attachToClause(Element ecoreElt, Element ecoreOwner, IClause objingOwnerClause) {
        boolean isAttached = false;
        if (ecoreOwner instanceof Clause) {
            Clause owner = (Clause) ecoreOwner;
        
            // A DecisionMergeNode can't have any Pin in Ijing => use of the
            // "body" role:
            if (ecoreElt instanceof ExecutableNode)
                owner.getBodies().add((ExecutableNode)ecoreElt);
        
            // Setting composition relation (in ConditionalNode):
            IConditionalNode objingConditional = objingOwnerClause.getOwner();
        
            if (objingConditional != null) {
                Element ecoreConditional = GenerationProperties.getInstance()
                        .getMappedElement(objingConditional);
                if (ecoreConditional instanceof StructuredActivityNode) {
                    ((StructuredActivityNode) ecoreConditional).getNodes().add(
                                                                               (ActivityNode)ecoreElt);
                    isAttached = true;
                }
            }
        }
        
        if (!isAttached) {
            // Setting composition relation (in Activity):
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.objingElement, IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity = GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        }
    }

    @objid ("0e9b7ecb-4583-11e0-b54c-00137279a832")
    private void setDecisionInput(DecisionNode node) {
        String decisionInput = this.objingElement.getDecisionInputBehavior();
        if (decisionInput.length() > 0) {
            OpaqueBehavior behavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
            ObjingEAnnotation.setDeleted(behavior, true);
            behavior.getBodies().add(decisionInput);
        
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.objingElement, IActivity.class,
                                         this.ownerVisitor);
        
            IPackage objingPkg = ObjingModelNavigation
                    .getNearestPackage(enclosingActivity);
        
            if (objingPkg != null) {
                Package ecorePkg = (Package) GenerationProperties.getInstance()
                        .getMappedElement(objingPkg);
                if (ecorePkg != null)
                    ecorePkg.getPackagedElements().add(behavior);
                else{
                    node.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        
            node.setDecisionInput(behavior);
        }
    }

}
