/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ModelerModuleStereotypes;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0ea2a5cd-4583-11e0-b54c-00137279a832")
public class ODependency extends OModelElement implements IOElement {
    @objid ("0ea2a5ce-4583-11e0-b54c-00137279a832")
    private IDependency objingElement;

    @objid ("0ea2a5cf-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0ea2a5d0-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (objingElement.isStereotyped(XMIStereotypes.UML2ASSOCIATIONREFERENCE) 
                || objingElement.isStereotyped(XMIStereotypes.UML2METHODREFERENCE)
                || objingElement.isStereotyped(XMIStereotypes.UML2STRUCTURALFEATUREREFERENCE)
                || objingElement.isStereotyped(XMIStereotypes.UML2CLASSIFIERREFERENCE)
                || objingElement.isStereotyped(XMIStereotypes.UML2ENDCREATIONDATAREFERENCE)
                || objingElement.isStereotyped(XMIStereotypes.UML2ENDDESTRUCTIONDATAREFERENCE)
                 || objingElement.isStereotyped(XMIStereotypes.UML2EVENT)
                || objingElement.isStereotyped(ModelerModuleStereotypes.PART)){
            return null;
        }else if (objingElement.isStereotyped(XMIStereotypes.UML2COMPONENTREALIZATION)){
            return UMLFactory.eINSTANCE.createComponentRealization();
        }else if (objingElement.isStereotyped(XMIStereotypes.UML2DEPLOYMENT)){
            return UMLFactory.eINSTANCE.createDeployment();
        }else if (objingElement.isStereotyped(XMIStereotypes.UML2PROTOCOLCONFORMANCE)){
            return UMLFactory.eINSTANCE.createProtocolConformance();
        }else if (objingElement.isStereotyped(XMIStereotypes.UML2ABSTRACTION)){
            return UMLFactory.eINSTANCE.createAbstraction();
        }else if (objingElement.isStereotyped(ModelerModuleStereotypes.SATISFY)
                ||objingElement.isStereotyped(ModelerModuleStereotypes.VERIFY)
                ||objingElement.isStereotyped(ModelerModuleStereotypes.DERIVE)){
                    genProp.addSysMLExported(objingElement);
            return UMLFactory.eINSTANCE.createAbstraction();
        }else{
            Dependency ecoreDependency = UMLFactory.eINSTANCE.createDependency();
            ecoreDependency.setName(objingElement.getName());
            return ecoreDependency;
        }
    }

    @objid ("0ea2a5d3-4583-11e0-b54c-00137279a832")
    public ODependency() {
    }

    @objid ("0ea2a5d5-4583-11e0-b54c-00137279a832")
    public ODependency(IDependency element) {
        super(element);
        objingElement = element;
    }

    @objid ("0ea5082d-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (!(ecoreElt instanceof InterfaceRealization)) {
            // This method is also called when linking a Usage, as Usage inherits
            // from Dependency.
           
                
            IModelElement objingClient = objingElement.getImpacted();
            IModelElement objingSupplier = objingElement.getDependsOn();
                
            if (objingClient != null && objingSupplier != null) {
                // Gets or creates the ecore "Client" element:
                Element ecoreClient = (Element) genProp
                .getMappedElement(objingClient);
                
                // Gets or creates the ecore "Supplier" element:
                Element ecoreSupplier = (Element) genProp
                .getMappedElement(objingSupplier);
                
                if (ecoreClient != null && ecoreSupplier != null ) {
                
                    if ((ecoreClient instanceof NamedElement) && (ecoreSupplier instanceof NamedElement)){
                        Dependency ecoreDependency = (Dependency) ecoreElt;
                
                        ecoreDependency.getClients().add((NamedElement)ecoreClient);
                        ecoreDependency.getSuppliers().add((NamedElement)ecoreSupplier);
                
                        Package ecorePkg = ecoreClient.getNearestPackage();
                        if (ecorePkg == null) {
                            genProp.getEcoreModel().getPackagedElements().add(
                                    ecoreDependency);
                        } else {
                            ecorePkg.getPackagedElements().add(ecoreDependency);
                        }
                    }else{
                        ecoreElt.destroy();
                    }
                }else{
                    ecoreElt.destroy();
                }
            }else 
                ecoreElt.destroy();
        }
    }

    @objid ("0ea50831-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setFlow((Dependency) ecoreElt);
        // setDependsOn((Dependency)ecoreElt); -> DONE IN linkEcoreElt()
        // OPERATION
    }

    @objid ("0ea76a80-4583-11e0-b54c-00137279a832")
    private void setFlow(Dependency ecoreElt) {
        if (objingElement.isStereotyped(ModelerModuleStereotypes.FLOW))
            ObjingEAnnotation.setFlow(ecoreElt, true);
    }

}
