/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportProfile;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;

/**
 * This class handle the export of Modelio IElement
 * @author ebrosse
 */
@objid ("0eb0f3e8-4583-11e0-b54c-00137279a832")
public class OElement {
    @objid ("0eb0f3e9-4583-11e0-b54c-00137279a832")
    private IElement objingElt;


    /**
     * return the exported Modelio IElement
     * @return the IElement
     */
    @objid ("0eb0f3ed-4583-11e0-b54c-00137279a832")
    public IElement getObjingElement() {
        return this.objingElt;
    }

    /**
     * return the Modelio identifier of the current Modelio IElement
     * @return the identifier
     */
    @objid ("0eb0f3f1-4583-11e0-b54c-00137279a832")
    public String getObjingID() {
        return this.objingElt.getIdentifier();
    }

    /**
     * Default constructor
     */
    @objid ("0eb35647-4583-11e0-b54c-00137279a832")
    public OElement() {
    }

    /**
     * Constructor with the exported Modelio IElemnt as parameter
     * @param element : the exported Modelio IElement
     */
    @objid ("0eb35649-4583-11e0-b54c-00137279a832")
    public OElement(final IElement element) {
        this.objingElt = element;
        
        if (element instanceof IModelElement){
                
            IModelElement modelElement = (IModelElement) element;
                    
            for( IStereotype obStereotype : modelElement.getExtension()){
                if (ObjingModelNavigation.mustBeExported(obStereotype)) {
                    exportStereotypeProfile(obStereotype);
                    GenerationProperties.getInstance().addStereotypeExported(modelElement);
                }
            }
                
            for( ITaggedValue obTaggedValue : modelElement.getTag()){
                IMetaclassReference reference = obTaggedValue.getDefinition().getOwnerReference();
                if ((reference != null) && (ObjingModelNavigation.mustBeExported(reference)) ){
                    exportReferenceProfile(reference);
                    GenerationProperties.getInstance().addStereotypeExported(modelElement);
                }
            }
        }
    }

    @objid ("0eb5b8a4-4583-11e0-b54c-00137279a832")
    private void exportStereotypeProfile(IStereotype stereotype) {
        exportProfile(stereotype.getOwner());
    }

    @objid ("0eb5b8a7-4583-11e0-b54c-00137279a832")
    private void exportReferenceProfile(IMetaclassReference reference) {
        exportProfile(reference.getOwnerProfile());
    }

    /**
     * Return the name of the corresponding Ecore class
     * @return the name
     */
    @objid ("0ebcdfae-4583-11e0-b54c-00137279a832")
    public List<String> getEcoreClassName() {
        List<String> result = new ArrayList<String>();
        result.add("Element");
        return result;
    }

    @objid ("0ebcdfb4-4583-11e0-b54c-00137279a832")
    private void exportProfile(IProfile profile) {
        Object ecoreProfile =  GenerationProperties.getInstance().getMappedElement(profile);
        
        if (ecoreProfile == null){
            ProfileExportVisitorImpl profileVisitor = new ProfileExportVisitorImpl();       
            PExportProfile pprofile = new PExportProfile(ProfileUtils.getProfileRoot(profile));
            profileVisitor.visit(pprofile);
        }
    }

}
