/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IElementImport;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ElementImport;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("0ebf4204-4583-11e0-b54c-00137279a832")
public class OElementImport extends OElement implements IOElement {
    @objid ("0ebf4205-4583-11e0-b54c-00137279a832")
    private IElementImport objingElement;


    @objid ("0ebf4206-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        ElementImport ecoreEI = UMLFactory.eINSTANCE.createElementImport();
        return ecoreEI;
    }

    @objid ("0ebf420a-4583-11e0-b54c-00137279a832")
    public OElementImport() {
    }

    @objid ("0ebf420c-4583-11e0-b54c-00137279a832")
    public OElementImport(IElementImport element) {
        super(element);
        objingElement = element;
    }

    @objid ("0ec1a463-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        ElementImport ecoreEltImport = (ElementImport) ecoreElt;
                
        IModelElement objingImportingElt = ObjingModelNavigation
                .getImportingElement(objingElement);
        INameSpace objingImportedElt = objingElement.getImportedElement();
                
        if (objingImportingElt != null && objingImportedElt != null) {
                
            // Gets or creates the ecore importing element (the "owner"):
            Element ecoreImportingElt = (Element) genProp
                    .getMappedElement(objingImportingElt);
                
            // Gets or creates the ecore imported element (the element to
            // import):
            Element ecoreImportedElt = (Element) genProp
                    .getMappedElement(objingImportedElt);
                
            if (ecoreImportingElt != null && ecoreImportedElt != null) {
                ((Namespace) ecoreImportingElt).getElementImports().add(
                        (ElementImport)ecoreElt);
                
                // In UML2, an element import is defined as a directed
                // relationship between an
                // importing namespace and a packageable element.
                if (ecoreImportedElt instanceof PackageableElement) {
                    ecoreEltImport
                            .setImportingNamespace((Namespace) ecoreImportingElt);
                    ecoreEltImport
                            .setImportedElement((PackageableElement) ecoreImportedElt);
                } else if (ecoreImportedElt instanceof NamedElement) {
                    ObjingEAnnotation.setImportedElement(ecoreImportingElt,
                            ((NamedElement) ecoreImportedElt)
                                    .getQualifiedName());
                }
            }
        }
    }

    @objid ("0ec1a467-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setName((ElementImport) ecoreElt);
        setVisibility((ElementImport) ecoreElt);
        // setImportedElement((ElementImport) ecoreElt); -> DONE IN
        // linkEcoreElt() OPERATION
    }

    @objid ("0ec1a46a-4583-11e0-b54c-00137279a832")
    private void setName(ElementImport ecoreElt) {
        // Specifies the name that should be added to the namespace of the
        // importing Package in lieu
        // of the name of the imported PackagableElement. The aliased name must
        // not clash with any
        // other member name in the importing Package. By default, no alias is
        // used.
        String name = objingElement.getName();
        // If the name is "", then we don't export the name:
        if (!"".equals(name))
            ecoreElt.setAlias(name);
    }

    @objid ("0ec1a46d-4583-11e0-b54c-00137279a832")
    private void setVisibility(ElementImport ecoreElt) {
        // Note that in theory, the visibility of an ElementImport is either
        // public or private.
        switch (objingElement.getVisibility()) {
        case PACKAGE_VISIBILITY:
            ecoreElt.setVisibility(VisibilityKind.PACKAGE_LITERAL);
            break;
        case PRIVATE:
            ecoreElt.setVisibility(VisibilityKind.PRIVATE_LITERAL);
            break;
        case PROTECTED:
            ecoreElt.setVisibility(VisibilityKind.PROTECTED_LITERAL);
            break;
        case PUBLIC:
            // It is the Default Value
        //          ecoreElt.setVisibility(VisibilityKind.PUBLIC_LITERAL);
            break;
        case VISIBILITY_UNDEFINED:
            ObjingEAnnotation.setUndefined(ecoreElt, true);
            // No undefined field exist in UML2-Ecore for visibility:
            // a visibility not setted corresponds to a PUBLIC visibility.
            break;
        }
    }

}
