/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0ed254d4-4583-11e0-b54c-00137279a832")
public class OEnumeration extends ONameSpace implements IOElement {
    @objid ("0ed254d5-4583-11e0-b54c-00137279a832")
    private IEnumeration objingElement;


    @objid ("0ed254d6-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return  UMLFactory.eINSTANCE.createEnumeration();
    }

    @objid ("0ed254da-4583-11e0-b54c-00137279a832")
    public OEnumeration() {
    }

    @objid ("0ed254dc-4583-11e0-b54c-00137279a832")
    public OEnumeration(IEnumeration element) {
        super(element);
        objingElement = element;
    }

    @objid ("0ed4b734-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = (Element) genProp.getMappedElement(objingOwner);
        
        // In UML2, Signals can't own Enumerations.
        if ((ecoreOwner != null) && (ecoreOwner instanceof Signal)){
            
            ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                    objingOwner, objingElement,ecoreElt);
           
            ecoreOwner = (Element) genProp.getMappedElement(objingOwner.getOwner());
        }
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                ((Package) ecoreOwner).getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                ((Component) ecoreOwner).getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                ((Class) ecoreOwner).getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                ((Interface) ecoreOwner).getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ((Classifier)ecoreElt).setOwningTemplateParameter(ownerIsTemplateParameter);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0ed4b738-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
    }

}
