/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ObEventTypeEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEvent;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.TimeEvent;
import org.eclipse.uml2.uml.TimeExpression;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

@objid ("0edbde3c-4583-11e0-b54c-00137279a832")
public class OEvent extends OModelElement implements IOElement {
    @objid ("0edbde3d-4583-11e0-b54c-00137279a832")
    private IEvent objingElement;


    @objid ("0edbde40-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        ObEventTypeEnum eventType = this.objingElement.getKind();
        if (eventType != null){
            switch (eventType){
                case CALL_EVENT :
                    return UMLFactory.eINSTANCE.createCallEvent();
                case SIGNAL_EVENT :
                    return UMLFactory.eINSTANCE.createSignalEvent();
                case TIME_EVENT :
                    return UMLFactory.eINSTANCE.createTimeEvent();
                case CHANGE_EVENT :
                    return UMLFactory.eINSTANCE.createChangeEvent();
        
            }
        }
        
        if (this.objingElement.isStereotyped(XMIStereotypes.UML2CREATIONEVENT)){
            return UMLFactory.eINSTANCE.createCreationEvent();
        }else if (this.objingElement.isStereotyped(XMIStereotypes.UML2DESTRUCTIONEVENT)){
            return UMLFactory.eINSTANCE.createDestructionEvent();
        }else if (this.objingElement.isStereotyped(XMIStereotypes.UML2EXECUTIONEVENT)){
            return UMLFactory.eINSTANCE.createExecutionEvent();
        }
        return null;
    }

    @objid ("0edbde44-4583-11e0-b54c-00137279a832")
    public OEvent() {
    }

    @objid ("0ede409a-4583-11e0-b54c-00137279a832")
    public OEvent(IEvent event) {
        super(event);
        this.objingElement = event;
    }

    @objid ("0ede409d-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IBehavior oldOwner = this.objingElement.getComposed();
        
        IPackage newOwner = ObjingModelNavigation.getNearestPackage(oldOwner);
        
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(oldOwner);
        
        if ((newOwner != null) && 
                ((ecoreOwner != null) && ((ecoreOwner instanceof BehavioredClassifier) || (ecoreOwner instanceof Transition)
                        || (ecoreOwner instanceof State) || (ecoreOwner instanceof AcceptEventAction) ))){
        
            Element newEcoreOwner = GenerationProperties.getInstance().getMappedElement(newOwner);
            
            if ((newEcoreOwner != null) && (newEcoreOwner instanceof Package))
                ((Package) newEcoreOwner).getPackagedElements().add((Event)ecoreElt);
        
        }
    }

    @objid ("0ede40a1-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Event ){
            super.setProperties(ecoreElt);
            if (ecoreElt instanceof CallEvent ){    
                setCallProperties((CallEvent) ecoreElt);
            }else     if (ecoreElt instanceof SignalEvent ){    
                setSignalProperties((SignalEvent) ecoreElt);
            }else     if (ecoreElt instanceof TimeEvent ){    
                setTimeProperties((TimeEvent) ecoreElt);
            }else     if (ecoreElt instanceof ChangeEvent ){    
                setChangeProperties((ChangeEvent) ecoreElt);
            }
        }
    }

    @objid ("0ee0a2f0-4583-11e0-b54c-00137279a832")
    private void setChangeProperties(ChangeEvent ecoreElt) {
        String changeValue = this.objingElement.getExpression();
        if ((changeValue != null) && (!changeValue.equals(""))){
            LiteralString literalString = UMLFactory.eINSTANCE.createLiteralString();
            ValueSpecification ecoreValue = ecoreElt.createChangeExpression("",PrimitiveTypeMapper.getString(), literalString.eClass());
            ((LiteralString)ecoreValue).setValue(changeValue);
        }
    }

    @objid ("0ee0a2f3-4583-11e0-b54c-00137279a832")
    private void setTimeProperties(TimeEvent ecoreElt) {
        String timeValue = this.objingElement.getExpression();
        if ((timeValue != null)&& (!timeValue.equals(""))){
            TimeExpression ecoreValue = ecoreElt.createWhen("", PrimitiveTypeMapper.getString());
            LiteralString value = UMLFactory.eINSTANCE.createLiteralString();
            value.setValue(timeValue);
            ecoreValue.setExpr(value);
        }
    }

    @objid ("0ee0a2f6-4583-11e0-b54c-00137279a832")
    private void setSignalProperties(SignalEvent ecoreElt) {
        ISignal objSignal = this.objingElement.getModel();
        if (objSignal != null) {
            Element ecoreSignal = GenerationProperties.getInstance().getMappedElement(objSignal);
            if (ecoreSignal instanceof Signal)
                ecoreElt.setSignal((Signal) ecoreSignal);
        }
    }

    @objid ("0ee0a2f9-4583-11e0-b54c-00137279a832")
    private void setCallProperties(CallEvent ecoreElt) {
        IOperation objOperation = this.objingElement.getCalled();
        if (objOperation != null) {
            Element ecoreOperation = GenerationProperties.getInstance().getMappedElement(objOperation);
            if (ecoreOperation instanceof Operation)
                ecoreElt.setOperation((Operation) ecoreOperation);
        }
    }

}
