/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.usecase.IExtensionPoint;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCaseDependency;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Extend;
import org.eclipse.uml2.uml.ExtensionPoint;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("0ef15366-4583-11e0-b54c-00137279a832")
public class OExtensionPoint extends OModelElement implements IOElement {
    @objid ("0ef15367-4583-11e0-b54c-00137279a832")
    private IExtensionPoint objingElement;


    @objid ("0ef15368-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE
                 .createExtensionPoint();
    }

    @objid ("0ef1536c-4583-11e0-b54c-00137279a832")
    public OExtensionPoint() {
    }

    @objid ("0ef1536e-4583-11e0-b54c-00137279a832")
    public OExtensionPoint(IExtensionPoint param) {
        super(param);
        objingElement = param;
    }

    @objid ("0ef3b5c6-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IUseCase objingOwner = objingElement.getOwner();
                
        UseCase ecoreOwner = (UseCase) genProp.getMappedElement(objingOwner);
        ExtensionPoint currentExtensionPoint = (ExtensionPoint) ecoreElt;
        if (ecoreOwner != null) {
            ((ExtensionPoint) currentExtensionPoint).setUseCase(ecoreOwner);
        }
    }

    @objid ("0ef3b5ca-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setVisibility((ExtensionPoint) ecoreElt);
        setUseCaseDependency((ExtensionPoint) ecoreElt);
    }

    @objid ("0ef3b5cd-4583-11e0-b54c-00137279a832")
    private void setUseCaseDependency(ExtensionPoint point) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        ObList<IUseCaseDependency> objingUseCaseDepElts = objingElement
                .getExtended();
        for (IUseCaseDependency objingUseCaseDepElt : objingUseCaseDepElts) {
            Element ecoreUseCaseDep = (Element) genProp
                    .getMappedElement(objingUseCaseDepElt);
            if (ecoreUseCaseDep instanceof Extend) {
                ((Extend) ecoreUseCaseDep).getExtensionLocations().add(point);
            }
        }
    }

    @objid ("0ef6181c-4583-11e0-b54c-00137279a832")
    private void setVisibility(ExtensionPoint ecoreElt) {
        switch (objingElement.getVisibility()) {
        case PACKAGE_VISIBILITY:
            ecoreElt.setVisibility(VisibilityKind.PACKAGE_LITERAL);
            break;
        case PRIVATE:
            ecoreElt.setVisibility(VisibilityKind.PRIVATE_LITERAL);
            break;
        case PROTECTED:
            ecoreElt.setVisibility(VisibilityKind.PROTECTED_LITERAL);
            break;
        case PUBLIC:
            // It is the Default Value
        //          ecoreElt.setVisibility(VisibilityKind.PUBLIC_LITERAL);
            break;
        case VISIBILITY_UNDEFINED:
            ObjingEAnnotation.setUndefined(ecoreElt, true);
            // No undefined field exist in UML2-Ecore for visibility:
            // a visibility not setted corresponds to a PUBLIC visibility.
            break;
        }
    }

}
