/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityPartition;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IClause;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IConditionalNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IControlFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IForkJoinNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IStructuredActivityNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.Clause;
import org.eclipse.uml2.uml.ControlNode;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.ForkNode;
import org.eclipse.uml2.uml.JoinNode;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f06c893-4583-11e0-b54c-00137279a832")
public class OForkJoinNode extends OActivityNode implements IOElement {
    @objid ("0f06c894-4583-11e0-b54c-00137279a832")
    private boolean partialCreation = true;

    @objid ("0f06c895-4583-11e0-b54c-00137279a832")
    private boolean isForkNode = false;

    @objid ("0f06c896-4583-11e0-b54c-00137279a832")
    private boolean isJoinNode = false;

    @objid ("0f06c897-4583-11e0-b54c-00137279a832")
    private IForkJoinNode objingElement;

    @objid ("0f06c89a-4583-11e0-b54c-00137279a832")
    private ForkNode forkNode;


    @objid ("0f06c89b-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        this.partialCreation= false;
        
        // If the ForkJoin is not only Fork or Join, we create a special
        // structure:
        // a Join connected to a Fork by a flow.
        if (this.isJoinNode)
            return UMLFactory.eINSTANCE.createJoinNode();
        else if (this.isForkNode)
            return UMLFactory.eINSTANCE.createForkNode();
        else
            return UMLFactory.eINSTANCE.createJoinNode();
    }

    @objid ("0f06c89e-4583-11e0-b54c-00137279a832")
    public OForkJoinNode() {
    }

    @objid ("0f092aea-4583-11e0-b54c-00137279a832")
    public OForkJoinNode(IForkJoinNode element) {
        super(element);
        this.objingElement = element;
        this.isJoinNode = (ObjingModelNavigation.isJoinNode(element));         
        this.isForkNode = (ObjingModelNavigation.isForkNode(element));
    }

    @objid ("0f092af3-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
        
        if (!this.isJoinNode && !this.isForkNode) {
            if (!this.partialCreation) {
                createStructure((JoinNode) ecoreElt);
            } else {
                // The structure has already been created in the partial
                // creation
                // method.
                this.forkNode = getForkNode((JoinNode) ecoreElt);
            }
        }
        
        IElement objingOwner = ownerVisitor.getCompositionOwner(this.objingElement);
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (objingOwner instanceof IStructuredActivityNode) {
                attachToStructuredActivityNode(ecoreElt, ecoreOwner);
                if (!this.isJoinNode && !this.isForkNode)
                    attachToStructuredActivityNode(this.forkNode, ecoreOwner);
            } else if (objingOwner instanceof IActivityPartition) {
                attachToActivityPartition(ecoreElt, ecoreOwner);
                if (!this.isJoinNode && !this.isForkNode)
                    attachToActivityPartition(this.forkNode, ecoreOwner);
            } else if (objingOwner instanceof IActivity) {
                attachToActivity(ecoreElt, ecoreOwner);
                if (!this.isJoinNode && !this.isForkNode)
                    attachToActivity(forkNode, ecoreOwner);
            } else if (objingOwner instanceof IClause) {
                attachToClause(ecoreElt, ecoreOwner, (IClause) objingOwner);
                if (!this.isJoinNode && !this.isForkNode)
                    attachToClause(this.forkNode, ecoreOwner, (IClause) objingOwner);
            }
        }
    }

    @objid ("0f092af7-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        
        if (this.isJoinNode) {
            setJoinSpec((JoinNode) ecoreElt);
            setCombineDuplicate((JoinNode) ecoreElt);
        }else{
            ObjingEAnnotation.setJoinSpec(ecoreElt, this.objingElement.getJoinSpec());
            ObjingEAnnotation.setCombineDuplicate(ecoreElt, this.objingElement.isCombineDuplicate());
        }
        
        
        if (!this.isJoinNode && !this.isForkNode) {
            setName(this.forkNode);
        }
    }

    @objid ("0f0b8d46-4583-11e0-b54c-00137279a832")
    private void createStructure(JoinNode joinNode) {
        this.forkNode = UMLFactory.eINSTANCE.createForkNode();
        
        IActivityEdge typeOfEdge = null;
        ObList<IActivityEdge> objingInc = this.objingElement.getIncoming();
        ObList<IActivityEdge> objingOut = this.objingElement.getOutgoing();
        
        if (objingInc.size() > 0) {
            typeOfEdge = objingInc.get(0);
        } else if (objingOut.size() > 0) {
            typeOfEdge = objingOut.get(0);
        }
        
        ActivityEdge ecoreFlow = null;
        if (typeOfEdge instanceof IControlFlow)
            ecoreFlow = UMLFactory.eINSTANCE.createControlFlow();
        else
            ecoreFlow = UMLFactory.eINSTANCE.createObjectFlow();
        
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                .getEnclosingElement(this.objingElement, IActivity.class,
                                     this.ownerVisitor);
        
        if (enclosingActivity != null) {
            Element ecoreActivity = GenerationProperties.getInstance()
                    .getMappedElement(enclosingActivity);
            if (ecoreActivity instanceof Activity) {
                Activity owner = (Activity) ecoreActivity;
                owner.getEdges().add(ecoreFlow);
            }
        }
        
        ecoreFlow.setSource(joinNode);
        ecoreFlow.setTarget(this.forkNode);
    }

    @objid ("0f0b8d49-4583-11e0-b54c-00137279a832")
    private ForkNode getForkNode(JoinNode joinNode) {
        EList<?> edgeList = joinNode.getOutgoings();
        if (edgeList != null && edgeList.size() > 0) {
            ActivityEdge flow = (ActivityEdge) edgeList.get(0);
            ActivityNode forknode = flow.getTarget();
            if (forknode instanceof ForkNode)
                return (ForkNode) forknode;
        }
        return null;
    }

    @objid ("0f0b8d4e-4583-11e0-b54c-00137279a832")
    private void attachToStructuredActivityNode(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof StructuredActivityNode) {
            StructuredActivityNode owner = (StructuredActivityNode) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0f0b8d52-4583-11e0-b54c-00137279a832")
    private void attachToActivityPartition(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof ActivityPartition) {
            ActivityPartition owner = (ActivityPartition) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.objingElement, IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity = GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0f0def9e-4583-11e0-b54c-00137279a832")
    private void attachToActivity(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof Activity) {
            Activity owner = (Activity) ecoreOwner;
            owner.getNodes().add((ActivityNode)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0f0defa2-4583-11e0-b54c-00137279a832")
    private void attachToClause(Element ecoreElt, Element ecoreOwner, IClause objingOwnerClause) {
        boolean isAttached = false;
        if (ecoreOwner instanceof Clause) {
            Clause owner = (Clause) ecoreOwner;
        
            // A ForkJoinNode can't have any Pin in Ijing => use of the
            // "body" role:
            if (ecoreElt instanceof ExecutableNode)
                owner.getBodies().add((ExecutableNode) ecoreElt);
        
            // Setting composition relation (in ConditionalNode):
            IConditionalNode objingConditional = objingOwnerClause.getOwner();
        
            if (objingConditional != null) {
                Element ecoreConditional = GenerationProperties.getInstance()
                        .getMappedElement(objingConditional);
                if (ecoreConditional instanceof StructuredActivityNode) {
                    ((StructuredActivityNode) ecoreConditional).getNodes().add(
                                                                               (ActivityNode)ecoreElt);
                    isAttached = true;
                }
            }
        }
        
        if (!isAttached) {
            // Setting composition relation (in Activity):
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.objingElement, IActivity.class,
                                         this.ownerVisitor);
            if (enclosingActivity != null) {
                Element ecoreActivity = GenerationProperties.getInstance()
                        .getMappedElement(enclosingActivity);
                if (ecoreActivity != null)
                    attachToActivity(ecoreElt, ecoreActivity);
                else{
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        }
    }

    @objid ("0f0defa7-4583-11e0-b54c-00137279a832")
    private void setName(ControlNode node) {
        if (ObjingModelNavigation.isNotNullOrEmpty(this.objingElement.getName()))
            node.setName(this.objingElement.getName());
    }

    @objid ("0f0defaa-4583-11e0-b54c-00137279a832")
    private void setJoinSpec(JoinNode node) {
        String value = this.objingElement.getJoinSpec();
        if ((value != null)&& (!value.equals(""))){
            LiteralString joinSpec = UMLFactory.eINSTANCE.createLiteralString();
            joinSpec.setValue(value);
            node.setJoinSpec(joinSpec);
        }
    }

    @objid ("0f0defad-4583-11e0-b54c-00137279a832")
    private void setCombineDuplicate(JoinNode node) {
        node.setIsCombineDuplicate(this.objingElement.isCombineDuplicate());
    }

}
