/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IGeneralization;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f19db61-4583-11e0-b54c-00137279a832")
public class OGeneralization extends OElement implements IOElement {
    @objid ("0f19db62-4583-11e0-b54c-00137279a832")
    private IGeneralization objingElement;


    @objid ("0f19db63-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createGeneralization();
    }

    @objid ("0f19db67-4583-11e0-b54c-00137279a832")
    public OGeneralization() {
    }

    @objid ("0f1c3dba-4583-11e0-b54c-00137279a832")
    public OGeneralization(IGeneralization element) {
        super(element);
        objingElement = element;
    }

    @objid ("0f1c3dc3-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        INameSpace objingSubType = objingElement.getSubType();
        INameSpace objingSuperType = objingElement.getSuperType();
                
        // Gets or creates the ecore subtype element:
        Element ecoreSubType = (Element) genProp
                .getMappedElement(objingSubType);
                
        // Gets or creates the ecore supertype element:
        Element ecoreSuperType = (Element) genProp
                .getMappedElement(objingSuperType);
                
        if (ecoreSubType != null && ecoreSuperType != null) {
            Generalization ecoreLink = (Generalization) ecoreElt;
                
            if (ecoreSubType instanceof Classifier
                    && ecoreSuperType instanceof Classifier) {
                ecoreLink.setSpecific((Classifier) ecoreSubType);
                ecoreLink.setGeneral((Classifier) ecoreSuperType);
            }
            // If the relationship does not link classifiers, we have to export
            // the link in an
            // EAnnotation as it is not supported by UML2:
            else {
                EAnnotation objingEA = ObjingEAnnotation
                        .getOrCreateObjingEAnnotation(ecoreSubType);
                if (objingEA != null)
                    objingEA.getContents().add(ecoreLink);
                // TODO METTRE LOG
            }
        }
    }

    @objid ("0f1c3dc7-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setDiscriminatorEAnnotation((Generalization) ecoreElt);
    }

    @objid ("0f1ea014-4583-11e0-b54c-00137279a832")
    private void setDiscriminatorEAnnotation(Generalization ecoreElt) {
        ObjingEAnnotation.setDiscriminator(ecoreElt, objingElement
                .getDiscriminator());
    }

}
