/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.information.IInformationFlow;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InformationFlow;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Relationship;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f2364c8-4583-11e0-b54c-00137279a832")
public class OInformationFlow extends OModelElement implements IOElement {
    @objid ("0f2364c9-4583-11e0-b54c-00137279a832")
     IInformationFlow objingElement;

    @objid ("0f2364ca-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0f2364cb-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInformationFlow();
    }

    @objid ("0f2364cf-4583-11e0-b54c-00137279a832")
    public OInformationFlow() {
    }

    @objid ("0f2364d1-4583-11e0-b54c-00137279a832")
    public OInformationFlow(IInformationFlow element) {
        super(element);
        objingElement = element;
    }

    @objid ("0f25c72a-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        INameSpace objingOwner = objingElement.getOwner();
        
        if (objingOwner != null) {
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
            
            if (ecoreOwner != null) {
                Package nearestPkg = null;
                if (ecoreOwner instanceof Package)
                    nearestPkg = (Package)ecoreOwner;
                else
                    nearestPkg = ecoreOwner.getNearestPackage();
                
                if (nearestPkg != null) {
                    nearestPkg.getPackagedElements().add((PackageableElement)ecoreElt);
                }
            }
        }
    }

    @objid ("0f25c72e-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setInformationSource((InformationFlow)ecoreElt);
        setInformationTarget((InformationFlow)ecoreElt);
        setRealizingMessage((InformationFlow)ecoreElt);
        setRealizingActivityEdge((InformationFlow)ecoreElt);
        setConveyed((InformationFlow)ecoreElt);
                //        setRealizingConnector((InformationFlow)ecoreElt);
        setRealization((InformationFlow)ecoreElt);
    }

    @objid ("0f28297c-4583-11e0-b54c-00137279a832")
    private void setInformationSource(InformationFlow flow) {
        for (IModelElement objingSource : objingElement.getInformationSource()) {
            Element ecoreSource = genProp.getMappedElement(objingSource);
            if (ecoreSource instanceof NamedElement) {
                flow.getInformationSources().add((NamedElement)ecoreSource);
            }
        }
    }

    @objid ("0f28297f-4583-11e0-b54c-00137279a832")
    private void setInformationTarget(InformationFlow flow) {
        for (IModelElement objingTarget : objingElement.getInformationTarget()) {
            Element ecoreTarget = genProp.getMappedElement(objingTarget);
            if (ecoreTarget instanceof NamedElement) {
                flow.getInformationTargets().add((NamedElement)ecoreTarget);
            }
        }
    }

    @objid ("0f282982-4583-11e0-b54c-00137279a832")
    private void setRealizingMessage(InformationFlow flow) {
        for (IMessage objingMsg : objingElement.getRealizingMessage()) {
            Element ecoreMsg = genProp.getMappedElement(objingMsg);
            if (ecoreMsg instanceof Message)
                flow.getRealizingMessages().add((Message)ecoreMsg);
        }
    }

    @objid ("0f282985-4583-11e0-b54c-00137279a832")
    private void setRealizingActivityEdge(InformationFlow flow) {
        for (IActivityEdge objingEdge : objingElement.getRealizingActivityEdge()) {
            Element ecoreEdge = genProp.getMappedElement(objingEdge);
            if (ecoreEdge instanceof ActivityEdge)
                flow.getRealizingActivityEdges().add((ActivityEdge)ecoreEdge);
        }
    }

    @objid ("0f282988-4583-11e0-b54c-00137279a832")
    private void setConveyed(InformationFlow flow) {
        for (IClassifier objingConveyed : objingElement.getConveyed()) {
            Element ecoreConveyed = genProp.getMappedElement(objingConveyed);
            if (ecoreConveyed instanceof Classifier)
                flow.getConveyeds().add((Classifier)ecoreConveyed);
            
        }
    }

    @objid ("0f2a8bd6-4583-11e0-b54c-00137279a832")
    private void setRealization(InformationFlow flow) {
        for (IAssociation realizingAssoc : objingElement.getRealizingAssociation()) {
            Element realization = genProp.getMappedElement(realizingAssoc);
            if (realization instanceof Relationship)
                flow.getRealizations().add((Relationship)realization);
        }
    }

}
