/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.information.IInformationItem;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InformationItem;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IInformationItem
 * @author ebrosse
 */
@objid ("0f2a8bd9-4583-11e0-b54c-00137279a832")
public class OInformationItem extends ONameSpace {
    @objid ("0f2a8bda-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0f2a8bdb-4583-11e0-b54c-00137279a832")
    private IInformationItem objElement = null;


    @Override
    @objid ("0f2a8bdc-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInformationItem();
    }

    /**
     * Constructor
     * @param param : the exported Modelio IInformationItem
     */
    @objid ("0f2cee30-4583-11e0-b54c-00137279a832")
    public OInformationItem(final IInformationItem param) {
        super(param);
        this.objElement = param;
    }

    @Override
    @objid ("0f2cee39-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objOwner = this.objElement.getOwner();
        Element ecoreOwner = this.genProp.getMappedElement(objOwner);
        
        if (ecoreOwner instanceof Package){
            ((Package) ecoreOwner).getPackagedElements().add((InformationItem)ecoreElt);
        }else if (ecoreOwner instanceof Component){
            ((Component) ecoreOwner).getPackagedElements().add((InformationItem)ecoreElt);
        }else if (ecoreOwner instanceof Interface){
            ((Interface) ecoreOwner).getNestedClassifiers().add((InformationItem)ecoreElt);
        }else if (ecoreOwner instanceof Class){
            ((Class) ecoreOwner).getNestedClassifiers().add((InformationItem)ecoreElt);
        }else{
            ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(objOwner, this.objElement, ecoreElt);
        }
    }

    @Override
    @objid ("0f2cee3d-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setRepresented((InformationItem) ecoreElt);
        if (this.genProp.isRoundtripEnabled()){
            setLeaf((InformationItem) ecoreElt);
            setRoot((InformationItem) ecoreElt);         
        }
    }

    @objid ("0f2f508a-4583-11e0-b54c-00137279a832")
    private void setRoot(InformationItem ecoreElt) {
        ObjingEAnnotation.setRoot(ecoreElt, this.objElement.isRoot());
    }

    @objid ("0f2f508d-4583-11e0-b54c-00137279a832")
    private void setLeaf(InformationItem ecoreElt) {
        ObjingEAnnotation.setLeaf(ecoreElt, this.objElement.isLeaf());
    }

    @objid ("0f2f5093-4583-11e0-b54c-00137279a832")
    private void setRepresented(InformationItem ecoreElt) {
        for (IClassifier classifier : this.objElement.getRepresented()){
            Element ecoreClassifier = this.genProp.getMappedElement(classifier);
            if (ecoreClassifier instanceof Classifier){
                ecoreElt.getRepresenteds().add((Classifier) ecoreClassifier);
            }else{
                XMILogs xmilogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedRelationBetweenTypeExported","represented",
                                                    this.objElement.getName(), this.objElement.getClass().getSimpleName(), 
                                                    classifier.getName(), classifier.getClass().getSimpleName());
                xmilogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, this.objElement);
            }
        }
    }

}
