/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.state.IInitialPseudoState;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.Vertex;

@objid ("0f3b3c4c-4583-11e0-b54c-00137279a832")
public class OInitialPseudoState extends OModelElement implements IOElement {
    @objid ("0f3b3c4d-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0f3b3c4e-4583-11e0-b54c-00137279a832")
    private IInitialPseudoState objingElement;

    @objid ("0f3b3c4f-4583-11e0-b54c-00137279a832")
    private Region ownerRegion = null;


    @objid ("0f3b3c50-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createPseudostate();
    }

    @objid ("0f3b3c54-4583-11e0-b54c-00137279a832")
    public OInitialPseudoState(IInitialPseudoState param) {
        super(param);                
        objingElement = param;
    }

    @objid ("0f3d9eaa-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
                
        if (objingOwner != null) {
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
            if (ecoreOwner != null) {
                if (ecoreOwner instanceof State) {
                    EList<?> regions = ((State) ecoreOwner).getRegions();
                    if (regions.size() > 0) {
                        ownerRegion = (Region) regions.get(0);
                        ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                    }
                } else if (ecoreOwner instanceof Region) {
                    ownerRegion = (Region) ecoreOwner;
                    ownerRegion.getSubvertices().add((Vertex)ecoreElt);
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreOwner.getClass().getSimpleName()
                            + ") Not Found");
                }
            } else {
                ecoreElt.destroy();
                throw new NotFoundException(
                        "Owner Class of InitialPseudoState Not Found");
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class of InitialPseudoState Not Found");
        }
    }

    @objid ("0f3d9eae-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setKind((Pseudostate)ecoreElt);
    }

    @objid ("0f3d9eb4-4583-11e0-b54c-00137279a832")
    private void setKind(Pseudostate pseudostate) {
        pseudostate.setKind(PseudostateKind.INITIAL_LITERAL);
    }

}
