/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInputPin;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IValuePin;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.AttachInputPinToOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.ActionInputPin;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExpansionNode;
import org.eclipse.uml2.uml.ExpansionRegion;
import org.eclipse.uml2.uml.InputPin;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f400100-4583-11e0-b54c-00137279a832")
public class OInputPin extends OPin implements IOElement {
    @objid ("0f400104-4583-11e0-b54c-00137279a832")
    private static AttachInputPinToOwnerVisitor attachInputPin = new AttachInputPinToOwnerVisitor();

    @objid ("0f400106-4583-11e0-b54c-00137279a832")
    private Element ecoreOwnerElt;

    @objid ("671c656a-1a76-11e1-8103-0027103f347d")
    private boolean isValuePin;


    @objid ("0f400107-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (((IInputPin)getObjingElement()).isStereotyped(XMIStereotypes.UML2EXPANSIONNODE))
            return UMLFactory.eINSTANCE.createExpansionNode();
        else if (((IInputPin)getObjingElement()).isStereotyped(XMIStereotypes.UML2ACTIONINPUTPIN))
            return UMLFactory.eINSTANCE.createActionInputPin();
        else if (((IInputPin)getObjingElement()).isStereotyped(XMIStereotypes.UML2VALUE))
            return UMLFactory.eINSTANCE.createValuePin();
        else{
            InputPin pin = null;
            if (this.isValuePin)
                pin = UMLFactory.eINSTANCE.createValuePin();
            else
                pin = UMLFactory.eINSTANCE.createInputPin();
            return pin;
        }
    }

    @objid ("0f40010a-4583-11e0-b54c-00137279a832")
    private boolean allPinsAreMapped() {
        IActivityAction objingOwner = ((IInputPin)getObjingElement()).getInputing();
        ObList<IInputPin> objingInputPinList = objingOwner.getInput();
        
        if (this.ecoreOwnerElt instanceof Action) {
            Action ecoreOwner = (Action) this.ecoreOwnerElt;
            EList<?> ecoreInputPinList = ecoreOwner.getInputs();
        
            if (ecoreInputPinList != null
                    && objingInputPinList.size() == ecoreInputPinList.size())
                return true;
        }
        return false;
    }

    @objid ("0f42635b-4583-11e0-b54c-00137279a832")
    private List<IInputPin> getSortedInputPinList() {
        IActivityAction objingOwner = ((IInputPin)getObjingElement()).getInputing();
        ObList<IParameter> objingParamList = ObjingModelNavigation
                .getRelatedParameters(objingOwner);
        List<IInputPin> objingInputPinList = new ArrayList<IInputPin>(
                objingOwner.getInput());
        List<IInputPin> objingPinSortedList = new ArrayList<IInputPin>();
        
        for (IParameter param : objingParamList) {
            IInputPin matchingPin = getMatchingPin(objingInputPinList, param);
            if (matchingPin != null) {
                objingInputPinList.remove(matchingPin);
                objingPinSortedList.add(matchingPin);
            } else {
                return new ArrayList<IInputPin>();
            }
        }
        
        if (!(objingInputPinList.size() == 0)) {
            return new ArrayList<IInputPin>();
        }
        return objingPinSortedList;
    }

    @objid ("0f426361-4583-11e0-b54c-00137279a832")
    public OInputPin() {
    }

    @objid ("0f426363-4583-11e0-b54c-00137279a832")
    public OInputPin(IInputPin element) {
        super(element);       
        this.ecoreOwnerElt = GenerationProperties.getInstance().getMappedElement(element.getInputing());        
        this.isValuePin =(element instanceof IValuePin);
    }

    @objid ("0f44c5ba-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt instanceof ExpansionNode){
            IElement objingOwner = ownerVisitor.getCompositionOwner((getObjingElement()));
            Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
            if (ecoreOwner instanceof StructuredActivityNode){
                ((StructuredActivityNode) ecoreOwner ).getNodes().add((ExpansionNode)ecoreElt);
            }
        
        }else{
        
            IElement objingOwner = ownerVisitor.getCompositionOwner((getObjingElement()));
            Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingOwner);
        
            if (ecoreOwner != null) {
                if (objingOwner instanceof IActivityAction)
                    attachToActivityAction(ecoreElt, ecoreOwner);
                else
                    super.attach(ecoreElt, ownerVisitor);
            }
        }
    }

    @objid ("0f44c5be-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        if (ecoreElt instanceof ExpansionNode){
            setInputRegion((ExpansionNode) ecoreElt);
        }else{
            setMatched((InputPin) ecoreElt);
        
            if (ecoreElt instanceof ActionInputPin)
                setAction((ActionInputPin) ecoreElt);
        
        }
    }

    @objid ("0f44c5c1-4583-11e0-b54c-00137279a832")
    private void attachToActivityAction(Element ecoreElt, Element ecoreOwner) {
        attachInputPin.attachInputPin((InputPin) ecoreElt, ecoreOwner, ((IInputPin)getObjingElement()));
    }

    @objid ("0f4becca-4583-11e0-b54c-00137279a832")
    private void setMatched(InputPin pin) {
        if ((allPinsAreMapped()) && (this.ecoreOwnerElt instanceof Action)){
            try{
                Action action = (Action) this.ecoreOwnerElt;
                List<IInputPin> objingPinSortedList = getSortedInputPinList();
                EList<InputPin> ecoreInputPinList = action.getInputs();
        
                for (IInputPin objingPin : objingPinSortedList) {
        
                    Element ecorePin = GenerationProperties.getInstance().getMappedElement(objingPin);
                    if (ecorePin instanceof InputPin) {
                        ecoreInputPinList.remove(ecorePin);
                        ecoreInputPinList.add((InputPin) ecorePin);
                    }
        
                }
            }catch(Exception e){
                //TODO
            }
        }
    }

    @objid ("0f4beccd-4583-11e0-b54c-00137279a832")
    private IInputPin getMatchingPin(List<IInputPin> objingInputPinList, IParameter param) {
        for (IInputPin pin : objingInputPinList) {
            if (param.equals(pin.getMatched()))
                return pin;
        }
        return null;
    }

    @objid ("0f4e4f1d-4583-11e0-b54c-00137279a832")
    private void setInputRegion(ExpansionNode ecoreElt) {
        if (ecoreElt.getOwner() instanceof ExpansionRegion)
            ecoreElt.setRegionAsInput((ExpansionRegion)ecoreElt.getOwner());
    }

    @objid ("0f4e4f23-4583-11e0-b54c-00137279a832")
    private void setAction(ActionInputPin ecoreElt) {
        Element element = ecoreElt.getOwner();
        if (element instanceof Action){
            try{    
                ecoreElt.setFromAction((Action) element);
            }catch(Exception e){
                e.printStackTrace(Modelio.err);
            }
        }
    }

}
