/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

/**
 * This classe manages the export of IInstance elements
 * @author ebrosse
 */
@objid ("0f50b176-4583-11e0-b54c-00137279a832")
public class OInstance extends OElement implements IOElement {
    @objid ("0f50b177-4583-11e0-b54c-00137279a832")
    private IInstance objingElement;


    @Override
    @objid ("0f50b178-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInstanceSpecification();
    }

    /**
     * the default constructor
     */
    @objid ("0f50b17c-4583-11e0-b54c-00137279a832")
    public OInstance() {
    }

    /**
     * @param element : The exported IInstance
     */
    @objid ("0f50b17e-4583-11e0-b54c-00137279a832")
    public OInstance(final IInstance element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0f5313d4-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (!(this.objingElement instanceof IBindableInstance)){
            GenerationProperties genProp = GenerationProperties.getInstance();
        
            IElement objingOwner = this.objingElement.getCompositionOwner();
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
            if (ecoreOwner != null) {
                if (ecoreOwner instanceof Package) {
                    Package ownerIsPkg = (Package) ecoreOwner;
                    ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
        
                } else if (ecoreOwner instanceof Component) {
                    Component ownerIsCmpnt = (Component) ecoreOwner;
                    ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
                } else if (objingOwner instanceof IModelTree) {
                    Package ownerIsPkg = (Package) genProp.getMappedElement(
                                                                            ObjingModelNavigation.getNearestPackage((IModelTree) objingOwner));
                    ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
        
                    if (!(objingOwner.equals(ownerIsPkg))){
                        XMILogs logs = XMILogs.getInstance();
                        String message = XMI.I18N.getMessage(
                                                             "logFile.warning.ownerChange",
                                                             this.objingElement.getName(),
                                                             this.objingElement.getClass().getSimpleName().substring(2),
                                                             ((IModelTree) objingOwner).getName(),
                                                             ownerIsPkg.getName());
                        logs.writelnInLog(message);
                        GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
                    }
                }else{
                    XMILogs logs = XMILogs.getInstance();
                    String message = XMI.I18N.getMessage(
                                                         "logFile.exception.stringConverter.haveNotOwner",
                                                         ecoreElt.getClass().toString());
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addError(message, this.objingElement);
                }
        
            }
        }
    }

    @Override
    @objid ("0f5313d8-4583-11e0-b54c-00137279a832")
    public void setProperties(final Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        
        if (ecoreElt instanceof InstanceSpecification){
            setName((InstanceSpecification)ecoreElt);
            setClassifier((InstanceSpecification)ecoreElt);
        
        }
        
        if (genProp.isRoundtripEnabled()){
            setIsConstant(ecoreElt);
            setMultiMax(ecoreElt);
            setMultiMin(ecoreElt);            
        }
        
        setValue(ecoreElt);
    }

    @objid ("0f5313de-4583-11e0-b54c-00137279a832")
    private void setName(InstanceSpecification specification) {
        String eltName = this.objingElement.getName();
        if (ObjingModelNavigation.isNotNullOrEmpty(eltName))
            specification.setName(eltName);
    }

    @objid ("0f55762c-4583-11e0-b54c-00137279a832")
    private void setClassifier(InstanceSpecification ecoreElement) {
        INameSpace objingRepresented = this.objingElement.getBase();
        
        if (objingRepresented != null) {
            GenerationProperties genProp = GenerationProperties.getInstance();
        
            Element ecoreRepresented = genProp.getMappedElement(objingRepresented);
        
            if (ecoreRepresented instanceof Classifier) {
                ecoreElement.getClassifiers().add((Classifier)ecoreRepresented);
            }else if ((objingRepresented instanceof IGeneralClass) 
                    && (PrimitiveTypeMapper.isPredefinedType((IGeneralClass)objingRepresented))
                    && genProp.isRoundtripEnabled()){
                ObjingEAnnotation.setType(ecoreElement, objingRepresented.getName());
            }else{
                XMILogs xmilogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedRelationBetweenTypeExported","Classifier",
                                                     this.objingElement.getName(), 
                                                     this.objingElement.getClass().getSimpleName(), 
                                                     objingRepresented.getName(), 
                                                     objingRepresented.getClass().getSimpleName());
                xmilogs.writelnInLog(message);
                genProp.getReportModel().addWarning(message, this.objingElement);
            }
        }
    }

    @objid ("0f557632-4583-11e0-b54c-00137279a832")
    private void setIsConstant(Element ecoreElt) {
        ObjingEAnnotation.setConstant(ecoreElt, this.objingElement.isConstant());
    }

    @objid ("0f557635-4583-11e0-b54c-00137279a832")
    private void setMultiMax(Element ecoreElt) {
        ObjingEAnnotation.setMultiMax(ecoreElt, this.objingElement.getMultiplicityMax());
    }

    @objid ("0f557638-4583-11e0-b54c-00137279a832")
    private void setValue(Element ecoreElt) {
        String value = this.objingElement.getValue();
        if ((value != null ) && (!value.equals(""))){
            INameSpace base  = ((IInstance) this.objingElement).getBase();
            if (ecoreElt instanceof InstanceSpecification){
                InstanceSpecification spec = (InstanceSpecification) ecoreElt;
                addValueSpecification(base, this.objingElement.getValue(), spec);
            }else
                ObjingEAnnotation.setValue(ecoreElt, this.objingElement.getValue());
        }
    }

    @objid ("0f57d884-4583-11e0-b54c-00137279a832")
    private void addValueSpecification(final INameSpace objType, Object value, final InstanceSpecification spec) {
        Type type = null;
        if (objType != null){
            Object temp = GenerationProperties.getInstance().getMappedElement(objType);
            if (temp instanceof Type)
                type = (Type) temp;
        }
        
            
        
        if (type != null){
            ValueSpecification result = null;
        
            IUMLTypes umlTypes = Modelio.getInstance()
                    .getModelingSession().getModel().getUmlTypes();
            // primitive type ==> LiteralSpecification
        
            if(objType.equals(umlTypes.getINTEGER())){
                LiteralInteger literalInteger = UMLFactory.eINSTANCE.createLiteralInteger();
                result = spec.createSpecification(null, type, literalInteger.eClass());
                try{
                    ((LiteralInteger)result).setValue(Integer.valueOf(value.toString()));
                }catch(NumberFormatException e){
                    literalInteger.destroy();
                    Expression expr = UMLFactory.eINSTANCE.createExpression();
                    result = spec.createSpecification(null, type, expr.eClass());
                    ((Expression)result).setSymbol(value.toString());
                    
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultValue",
                            this.objingElement.getName(),
                            "AttributeLink" );
        
                    String message = XMI.I18N.getMessage("logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + value + "\"",
                            "Integer", contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
        
                }
        
            }
        
            else if(objType.getName().equals("UnlimitedNatural")){
                LiteralUnlimitedNatural literalUnlimitedNatural = UMLFactory.eINSTANCE.createLiteralUnlimitedNatural();
                result = spec.createSpecification(null, type, literalUnlimitedNatural.eClass());
                try{
                    ((LiteralUnlimitedNatural)result).setValue(Integer.valueOf(value.toString()));
                }catch(NumberFormatException e){
                    literalUnlimitedNatural.destroy();
                    Expression expr = UMLFactory.eINSTANCE.createExpression();
                    result = spec.createSpecification(null, type, expr.eClass());
                    ((Expression)result).setSymbol(value.toString());
                }
            }
        
            else if(objType.equals(umlTypes.getBOOLEAN())){
                LiteralBoolean literalBoolean = UMLFactory.eINSTANCE.createLiteralBoolean();
                result = spec.createSpecification(null, type, literalBoolean.eClass());
                try{
                    ((LiteralBoolean)result).setValue(Boolean.valueOf(value.toString()));
                }catch(Exception e){
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultValue",
                            this.objingElement.getName(),
                            "AttributeLink" );
        
                    String message = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.defaultMsg",
                            "String", "\"" + value + "\"",
                            "Boolean", contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
                }
            }
        
            else if(objType.equals(umlTypes.getSTRING()) || objType.equals(umlTypes.getCHAR())){
                LiteralString literalString = UMLFactory.eINSTANCE.createLiteralString();
                result = spec.createSpecification(null, type, literalString.eClass());
                ((LiteralString)result).setValue(value.toString());
              
        
            }else {
                OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
                result = spec.createSpecification(null, type, opaqueExpr.eClass());
                ((OpaqueExpression)result).getBodies().add(value.toString());
            }
        }else {
            OpaqueExpression opaqueExpr = UMLFactory.eINSTANCE.createOpaqueExpression();
            ValueSpecification result = spec.createSpecification(null, type, opaqueExpr.eClass());
            ((OpaqueExpression)result).getBodies().add(value.toString());
        }
    }

    @objid ("0f57d889-4583-11e0-b54c-00137279a832")
    private void setMultiMin(Element ecoreElt) {
        ObjingEAnnotation.setMultiMin(ecoreElt, this.objingElement.getMultiplicityMin());
    }

}
