/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.model.objing.sequence.MainMapper;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IInteraction
 * @author ebrosse
 */
@objid ("0f63c449-4583-11e0-b54c-00137279a832")
public class OInteraction extends OModelElement {
    @objid ("0f63c44a-4583-11e0-b54c-00137279a832")
    private IInteraction objingElement = null;

    @objid ("0f63c44b-4583-11e0-b54c-00137279a832")
     CompositionOwnerVisitor genericOwner = new CompositionOwnerVisitor();


    @Override
    @objid ("0f63c44c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInteraction();
    }

    @objid ("0f63c450-4583-11e0-b54c-00137279a832")
    private void visiteSequenceDiagram() {
        // System.out.println("Liste des lifes Line Pr?sentes pour "
        // + objingElement.getName());
        MainMapper mapper = new MainMapper(this.genericOwner);
        for (ILifeline lf : this.objingElement.getOwnedLine()) {
            // System.out.println("-> " + lf.getName());
            mapper.createLifeline(lf);
        }
        
        IInteractionFragment current = this.objingElement.getStart();
        if (current != null) {
            GenerationProperties genProp = GenerationProperties.getInstance();
            if (!genProp.isMappedElement(current)) {
                // System.out.println("List element en creation pour "
                // + objingElement.getName());
                while (current != null) {
                    mapper.createElement(current);
                    current = this.objingElement.getNext(current);
                }
        
                current = this.objingElement.getStart();
            }
        }
    }

    @objid ("0f6626a0-4583-11e0-b54c-00137279a832")
    private void fixingForSAFRAN() {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        Interaction ecoreElt = (Interaction) genProp.getMappedElement(this.objingElement);
        List<Message> messageList = new ArrayList<Message>();
        
        for (Object fragment : ecoreElt.getMessages()){
            messageList.add((Message) fragment);
        }
        
        
        for (Message messageDel : messageList){
            //            if (fragment instanceof Message){
            //            Message messageDel = (Message) fragment;
            if (messageDel.getMessageSort().equals(MessageSort.DELETE_MESSAGE_LITERAL)){
                MessageOccurrenceSpecification end = (MessageOccurrenceSpecification) messageDel.getReceiveEvent();
                if (end != null){
                    Lifeline lifeline = end.getCovereds().get(0);
                    EList<?> fragmentList = lifeline.getCoveredBys();
                    int indexOfDeleteMessage = fragmentList.indexOf(messageDel);
                    int size  = fragmentList.size();
                    InteractionFragment last = (InteractionFragment) fragmentList.get(size - 1);
                    if (!last.equals(end)) {
        
                        //Change Message Order
        
                        for (int i = indexOfDeleteMessage + 1 ; i < size; i++ ){
                            InteractionFragment current = (InteractionFragment) fragmentList.get(i);
        
                            if (current instanceof MessageOccurrenceSpecification){
                                Message messageCurrent = ((MessageOccurrenceSpecification) current).getMessage();
                                int indexOfMessageCurrent =  ecoreElt.getMessages().indexOf(messageCurrent);
                                int indexOfDelete =  ecoreElt.getMessages().indexOf(messageDel);
                                if (indexOfMessageCurrent > indexOfDelete){
                                    ecoreElt.getMessages().remove(indexOfMessageCurrent);
                                    ecoreElt.getMessages().add(indexOfDelete, messageCurrent);
                                    ecoreElt.getMessages().remove(messageDel);
                                    ecoreElt.getMessages().add(indexOfMessageCurrent, messageDel);
        
                                }
        
                                indexOfMessageCurrent =  ecoreElt.getFragments().indexOf(messageCurrent.getReceiveEvent());
                                indexOfDelete =  ecoreElt.getFragments().indexOf(messageDel.getReceiveEvent());
                                if (indexOfMessageCurrent > indexOfDelete){
                                    ecoreElt.getFragments().remove(indexOfMessageCurrent);
                                    ecoreElt.getFragments().add(indexOfDelete, (InteractionFragment) messageCurrent.getReceiveEvent());
                                    ecoreElt.getFragments().remove(messageDel.getReceiveEvent());
                                    ecoreElt.getFragments().add(indexOfMessageCurrent, (InteractionFragment)messageDel.getReceiveEvent());
        
                                }
        
        
                                indexOfMessageCurrent =  ecoreElt.getFragments().indexOf(messageCurrent.getSendEvent());
                                indexOfDelete =  ecoreElt.getFragments().indexOf(messageDel.getSendEvent());
                                if (indexOfMessageCurrent > indexOfDelete){
                                    ecoreElt.getFragments().remove(indexOfMessageCurrent);
                                    ecoreElt.getFragments().add(indexOfDelete, (InteractionFragment) messageCurrent.getSendEvent());
                                    ecoreElt.getFragments().remove(messageDel.getSendEvent());
                                    ecoreElt.getFragments().add(indexOfMessageCurrent, (InteractionFragment) messageDel.getSendEvent());
        
                                }
        
        
                            }
                        }
                        //Change Message Occurence Specification Order
        
                        lifeline.getCoveredBys().remove(end);
                        lifeline.getCoveredBys().add(end);
                    }
        
                }
        
            }
        
        }
    }

    /**
     * Constructor
     * @param element : the exported Modelio IInteraction
     */
    @objid ("0f6626a4-4583-11e0-b54c-00137279a832")
    public OInteraction(final IInteraction element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0f6888fb-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.genericOwner = ownerVisitor;
        GenerationProperties genProp = GenerationProperties.getInstance();
        IModelElement objingOwner = this.objingElement.getOwner();
        
        if (objingOwner == null) {
            IOperation op = this.objingElement.getOwnerOperation();
            objingOwner = op.getOwner();
            String message = XMI.I18N.getMessage("logFile.warning.moving.interaction",
                                                 this. objingElement.getName(), op.getName(), objingOwner
                                                 .getName());
            XMILogs.getInstance().writelnInLog(message);
        
            GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
        }
        
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof BehavioredClassifier) {
                BehavioredClassifier ownerIsBehavioredClassifier = (BehavioredClassifier) ecoreOwner;
                ownerIsBehavioredClassifier.getOwnedBehaviors().add((Behavior)ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @Override
    @objid ("0f6888ff-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setReentrant((Interaction) ecoreElt);
        visiteSequenceDiagram();
        fixingForSAFRAN();
    }

    @objid ("0f688905-4583-11e0-b54c-00137279a832")
    private void setReentrant(Interaction interaction) {
        interaction.setIsReentrant(this.objingElement.isReentrant());
    }

}
