/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f72126a-4583-11e0-b54c-00137279a832")
public class OInterface extends ONameSpace implements IOElement {
    @objid ("0f72126b-4583-11e0-b54c-00137279a832")
    private IInterface objingElement;


    @objid ("0f72126c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInterface();
    }

    @objid ("0f7474bd-4583-11e0-b54c-00137279a832")
    public OInterface() {
    }

    @objid ("0f7474bf-4583-11e0-b54c-00137279a832")
    public OInterface(IInterface element) {
        super(element);
        objingElement = element;
    }

    @objid ("0f76d716-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = (Element) genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ((Classifier)ecoreElt).setOwningTemplateParameter(ownerIsTemplateParameter);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
            // In case of a TemplateParameter, we don't handle export of
            // Interfaces: the Ijing
            // metamodel supports it but it is not possible for an user to
            // create one through the
            // MMI (man-machine interface).
        }
    }

    @objid ("0f76d71a-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setPrimitiveEAnnotation((Interface) ecoreElt);
        setLeaf((Interface) ecoreElt);
        // this.setRoot(); -> Undefined on UML2 Interfaces.
    }

    @objid ("0f76d723-4583-11e0-b54c-00137279a832")
    private void setPrimitiveEAnnotation(Interface ecoreElt) {
        ObjingEAnnotation.setPrimitive(ecoreElt, objingElement.isElementary());
    }

    @objid ("0f76d726-4583-11e0-b54c-00137279a832")
    private void setLeaf(Interface ecoreElt) {
        ecoreElt.setIsLeaf(objingElement.isLeaf());
    }

}
