/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IInternalTransition;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.TransitionKind;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This classe manages the export of IInternalTransition
 * @author ebrosse
 */
@objid ("0f7dfe27-4583-11e0-b54c-00137279a832")
public class OInternalTransition extends OTransition {
    @objid ("674859aa-1a76-11e1-8103-0027103f347d")
    private Behavior ecoreBehavior = null;


    @Override
    @objid ("0f7dfe2b-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (((IInternalTransition)getObjingElement()).getBehaviorEffect() == null){
            Transition result = UMLFactory.eINSTANCE.createTransition();
            result.setKind(TransitionKind.INTERNAL_LITERAL);
            return result;
        }else{
            setBehavior();
            setAdditionalValues();
            return null;
        }
    }

    /**
     * Constructor
     * @param param : the exported Modelio IInternalTransition
     */
    @objid ("0f80607e-4583-11e0-b54c-00137279a832")
    public OInternalTransition(final IInternalTransition param) {
        super(param);
    }

    @Override
    @objid ("0f806087-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt != null){
             this.ownerVisitor = ownerVisitor;
            super.setProperties(ecoreElt);
        }
    }

    @Override
    @objid ("0f80608b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt != null)
            super.setProperties(ecoreElt);
    }

    @objid ("0f82c2de-4583-11e0-b54c-00137279a832")
    private void setBehavior() {
        IInternalTransition objingElement = (IInternalTransition)getObjingElement();
        String received = objingElement.getReceivedEvents();
        
        IBehavior behavior = objingElement.getBehaviorEffect();
        if ((behavior != null )){
            GenerationProperties genProp = GenerationProperties.getInstance();
            this.ecoreBehavior = (Behavior) genProp.getMappedElement(behavior);
            State ecoreState = (State)genProp.getMappedElement(objingElement.getSComposed());
        
            Element  ecoreOwner = this.ecoreBehavior.getOwner();
            if (ecoreOwner == null){
                ObjingEAnnotation.alwaysSetOwner(this.ecoreBehavior, objingElement.getIdentifier());
        
            }else{
        
                if (ecoreOwner instanceof Package) {
                    Package ownerIsPkg = (Package) ecoreOwner;
                    ownerIsPkg.getPackagedElements().remove(this.ecoreBehavior);
                } else if (ecoreOwner instanceof Component) {
                    Component ownerIsCmpnt = (Component) ecoreOwner;
                    ownerIsCmpnt.getPackagedElements().remove(this.ecoreBehavior);
                } else if (ecoreOwner instanceof Class) {
                    Class ownerIsClass = (Class) ecoreOwner;
                    ownerIsClass.getOwnedBehaviors().remove(this.ecoreBehavior);
                } else if (ecoreOwner instanceof Interface) {
                    Interface ownerIsItf = (Interface) ecoreOwner;
                    ownerIsItf.getNestedClassifiers().remove(this.ecoreBehavior);
                } else if (ecoreOwner instanceof BehavioredClassifier) {
                    BehavioredClassifier ownerIsBehavioredClassifier = (BehavioredClassifier) ecoreOwner;
                    ownerIsBehavioredClassifier.getOwnedBehaviors().remove(this.ecoreBehavior);
                } 
            }
        
            if (received != null) {
                if  (received.equals("Entry")){
                    ecoreState.setEntry(this.ecoreBehavior);               
                }else if  (received.equals("Exit")){
                    ecoreState.setExit(this.ecoreBehavior);
                }else {
                    ecoreState.setDoActivity(this.ecoreBehavior);
                }
            }
        
        }
    }

    @objid ("6748f5f2-1a76-11e1-8103-0027103f347d")
    private void setAdditionalValues() {
        IInternalTransition internalTransition = (IInternalTransition) getObjingElement();
        if (this.ecoreBehavior != null){
            if (internalTransition.getEffect() != null)
                ObjingEAnnotation.setEffect(this.ecoreBehavior, internalTransition.getEffect());
            
            if (internalTransition.getPostCondition() != null)
                ObjingEAnnotation.setPostCondition(this.ecoreBehavior, internalTransition.getPostCondition());
        
            if (internalTransition.getProcessed() != null){
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedUML2RelationExported", 
                                                     "getProcessed", "", "IInternalTransition" );
                XMILogs.getInstance().writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, internalTransition);
            }
        
            if (internalTransition.getTrigger() != null){
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedUML2RelationExported", 
                                                     "getTrigger", "", "IInternalTransition" );
                XMILogs.getInstance().writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, internalTransition);
            }
        }else{
            String message = XMI.I18N.getMessage("logFile.warning.export.internalTransitionHaveNotBehavior", 
                                                internalTransition.getReceivedEvents(), internalTransition.getSComposed().getName());
            XMILogs.getInstance().writelnInLog(message);
            GenerationProperties.getInstance().getReportModel().addWarning(message, internalTransition);
        }
    }

}
