/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInterruptibleActivityRegion;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityGroup;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InterruptibleActivityRegion;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f852532-4583-11e0-b54c-00137279a832")
public class OInterruptibleActivityRegion extends OElement implements IOElement {
    @objid ("0f852533-4583-11e0-b54c-00137279a832")
    private IInterruptibleActivityRegion objingElement;

    @objid ("0f852534-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0f852535-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInterruptibleActivityRegion();
    }

    @objid ("0f852539-4583-11e0-b54c-00137279a832")
    public OInterruptibleActivityRegion() {
    }

    @objid ("0f85253b-4583-11e0-b54c-00137279a832")
    public OInterruptibleActivityRegion(IInterruptibleActivityRegion element) {
        super(element);
        objingElement = element;
    }

    @objid ("0f878790-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objingOwner = ownerVisitor.getCompositionOwner(objingElement);
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null && objingOwner instanceof IActivity)
            attachToActivity(ecoreElt, ecoreOwner);
    }

    @objid ("0f878794-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setInterrupts((InterruptibleActivityRegion) ecoreElt);
        setName((InterruptibleActivityRegion) ecoreElt);
    }

    @objid ("0f878797-4583-11e0-b54c-00137279a832")
    private void attachToActivity(Element ecoreElt, Element ecoreOwner) {
        if (ecoreOwner instanceof Activity) {
            Activity owner = (Activity) ecoreOwner;
            owner.getGroups().add((ActivityGroup)ecoreElt);
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class ("
                    + ecoreOwner.getClass().getSimpleName() + ") Not Found");
        }
    }

    @objid ("0f87879b-4583-11e0-b54c-00137279a832")
    private void setInterrupts(InterruptibleActivityRegion region) {
        for (IActivityEdge objingEdge : objingElement.getInterruptingEdge()) {
            Element ecoreEdge = genProp.getMappedElement(objingEdge);
            if (ecoreEdge instanceof ActivityEdge) {
                region.getInterruptingEdges().add((ActivityEdge)ecoreEdge);
            }
        }
    }

    @objid ("0f89e9e6-4583-11e0-b54c-00137279a832")
    private void setName(InterruptibleActivityRegion region) {
        ObjingEAnnotation.setName(region, objingElement.getName());
    }

}
