/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IConnector;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.ILinkEnd;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0f9a9a5c-4583-11e0-b54c-00137279a832")
public class OLink extends OElement implements IOElement {
    @objid ("0f9a9a5d-4583-11e0-b54c-00137279a832")
    private ILink objingElement;

    @objid ("0f9a9a5e-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0f9a9a5f-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInstanceSpecification();
    }

    @objid ("0f9a9a63-4583-11e0-b54c-00137279a832")
    public OLink() {
    }

    @objid ("0f9a9a65-4583-11e0-b54c-00137279a832")
    public OLink(ILink element) {
        super(element);
        objingElement = element;
    }

    @objid ("0f9cfcba-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (!(objingElement instanceof IConnector)){
            Element ecoreOwner = null;
            for (ILinkEnd connectorEnd : objingElement.getConnection()) {
                // Links the Association to the OwnerClassifier of one of the
                // Properties
                // (sets the owner of the Association):
                if (ecoreOwner == null) {
                    IElement objOwner = connectorEnd.getCompositionOwner().getCompositionOwner();
                    ecoreOwner =  genProp.getMappedElement(objOwner);
                    
                    
                    if (ecoreOwner != null) {
                        // Components and Nodes are handled as Class
                        // (they inherit from Class)
                
                        // Class, Component, Node
                        
                        if (ecoreOwner instanceof Property)
                            ecoreOwner =  genProp.getMappedElement(objOwner.getCompositionOwner());
                        
                        if (ecoreOwner instanceof Package) {
                            Package ownerIsPackage = (Package) ecoreOwner;
                            ownerIsPackage.getPackagedElements().add((PackageableElement)ecoreElt);
                            
                //                        } else if (ecoreOwner instanceof Class) {
                //                            Class ownerIsClass = (Class) ecoreOwner;
                //                            EList attributeList = ownerIsClass.getNestedClassifiers();
                //                            if (!attributeList.contains(ecoreElt))
                //                                attributeList.add(ecoreElt);
                //                        } else if (ecoreOwner instanceof Interface) {// Interface
                //                            Interface ownerIsInterface = (Interface) ecoreOwner;
                //                            ownerIsInterface.getNestedClassifiers().add(ecoreElt);
                            
                            // } else if (ecoreOwner instanceof UseCase) {// UseCase
                            // EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                            // .getOwner(), ecoreElt);
                //                            } else if (ecoreOwner instanceof Actor) {// Actor
                //                            EcoreModelNavigation.attachAssocToNearestPkg(assocEnd
                //                            .getOwner(), ecoreElt);
                //                            } else if (ecoreOwner instanceof DataType) {// DataType
                //                            EcoreModelNavigation.attachAssocToNearestPkg(connectorEnd
                //                            .getCompositionOwner(), ecoreElt);
                //                            } else if (ecoreOwner instanceof Signal) {// Signal
                //                            EcoreModelNavigation.attachAssocToNearestPkg(connectorEnd
                //                            .getCompositionOwner(), ecoreElt);
                        } else {
                            ecoreElt.destroy();
                            throw new NotFoundException("Owner Class ("
                                    + ecoreOwner.getClass().getSimpleName()
                                    + ") Not Found");
                        }
                    }
                }
            }
        }
    }

    @objid ("0f9cfcbe-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof InstanceSpecification){
            setName((InstanceSpecification)ecoreElt);
            setBase((InstanceSpecification)ecoreElt);
            if (genProp.isRoundtripEnabled())
                ObjingEAnnotation.isLink(ecoreElt);
        }
    }

    @objid ("0f9f5f10-4583-11e0-b54c-00137279a832")
    private void setName(InstanceSpecification connector) {
        String eltName = objingElement.getName();
        if (ObjingModelNavigation.isNotNullOrEmpty(eltName))
            connector.setName(eltName);
    }

    @objid ("0f9f5f13-4583-11e0-b54c-00137279a832")
    private void setBase(InstanceSpecification connector) {
        IAssociation base = objingElement.getBase();
                
        if (base != null){
            Element type = (Element) genProp.getMappedElement(base);
            if (type instanceof Classifier)
            connector.getClassifiers().add((Classifier)type);
        }
    }

}
