/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.ILinkEnd;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;

/**
 * This class manages the export of ILinkEnd
 * @author ebrosse
 */
@objid ("0f9f5f16-4583-11e0-b54c-00137279a832")
public class OLinkEnd extends OElement implements IOElement {
    @objid ("0f9f5f17-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("0f9f5f18-4583-11e0-b54c-00137279a832")
    private ILinkEnd objElt;


    @Override
    @objid ("0f9f5f19-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createSlot();
    }

    /**
     * Constructor
     * @param param : the exported Modelio ILinkEnd
     */
    @objid ("0fa1c16c-4583-11e0-b54c-00137279a832")
    public OLinkEnd(final ILinkEnd param) {
        super(param);
        this.objElt = param;
    }

    @Override
    @objid ("0fa1c175-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt instanceof Slot){
            ILink link = this.objElt.getLinkNode();
            if (link != null){ 
                InstanceSpecification owner = (InstanceSpecification) this.genProp.getMappedElement(link);
                
                if (owner != null){   
                    owner.getSlots().add(((Slot) ecoreElt));
                }
            }
        }
    }

    @Override
    @objid ("0fa423c7-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof ConnectorEnd){
            setIsOrdered((ConnectorEnd)ecoreElt);
            setIsUnique((ConnectorEnd)ecoreElt);
            setMax((ConnectorEnd)ecoreElt);
            setMin((ConnectorEnd)ecoreElt);
            setRole((ConnectorEnd)ecoreElt);
            setPartWithPort((ConnectorEnd)ecoreElt);
            if (this.genProp.isRoundtripEnabled()){
                setName((ConnectorEnd)ecoreElt);
                setIsNavigable((ConnectorEnd)ecoreElt);
                
            }
        }else if ((ecoreElt instanceof Slot) && !(this.objElt instanceof IConnectorEnd))
            setSlotProperties((Slot)ecoreElt);
    }

    @objid ("0fa423ca-4583-11e0-b54c-00137279a832")
    private void setSlotProperties(Slot ecoreElt) {
        setLinked(ecoreElt);
        setDefiningFeature(ecoreElt);
        if (this.genProp.isRoundtripEnabled()){
                
            setIsLink(ecoreElt);
            ObjingEAnnotation.setUnique(ecoreElt, this.objElt.isUnique());
            ObjingEAnnotation.setNavigable(ecoreElt, this.objElt.isNavigable());
            ObjingEAnnotation.setOrdered(ecoreElt, this.objElt.isOrdered());
            ObjingEAnnotation.setMultiMax(ecoreElt, this.objElt.getMultiplicityMax());
            ObjingEAnnotation.setMultiMin(ecoreElt, this.objElt.getMultiplicityMin());
            ObjingEAnnotation.setName(ecoreElt, this.objElt.getName());
                
        }
    }

    @objid ("0fa423cd-4583-11e0-b54c-00137279a832")
    private void setLinked(Slot ecoreElt) {
        Element inst = this.genProp.getMappedElement(this.objElt.getLinked());
        if (inst instanceof InstanceSpecification){
            InstanceValue instanceValue = UMLFactory.eINSTANCE.createInstanceValue();
            ValueSpecification result = ecoreElt.createValue(null, null, instanceValue.eClass());
            ((InstanceValue)result).setInstance((InstanceSpecification)inst);
           
            if (ObjingModelNavigation.isNotNullOrEmpty(this.objElt.getName()))
            ((InstanceValue)result).setName(this.objElt.getName());
                
        }
    }

    @objid ("0fa423d0-4583-11e0-b54c-00137279a832")
    private void setIsLink(Slot ecoreElt) {
        ILink link = this.objElt.getLinkNode();
        if (link != null){
            Element inst = this.genProp.getMappedElement(link);
            if (inst instanceof InstanceSpecification){
                ObjingEAnnotation.setLink(inst,true);
            }
        }
    }

    @objid ("0fa423d3-4583-11e0-b54c-00137279a832")
    private void setName(ConnectorEnd ecoreElt) {
        if (ObjingModelNavigation.isNotNullOrEmpty(this.objElt.getName()))
            ObjingEAnnotation.setName(ecoreElt, this.objElt.getName());
    }

    @objid ("0fa6861e-4583-11e0-b54c-00137279a832")
    private void setIsOrdered(ConnectorEnd ecoreElt) {
        ecoreElt.setIsOrdered(this.objElt.isOrdered());
    }

    @objid ("0fa68621-4583-11e0-b54c-00137279a832")
    private void setIsUnique(ConnectorEnd ecoreElt) {
        ecoreElt.setIsUnique(this.objElt.isUnique());
    }

    @objid ("0fa68624-4583-11e0-b54c-00137279a832")
    private void setIsNavigable(ConnectorEnd ecoreElt) {
        ObjingEAnnotation.setNavigable(ecoreElt, this.objElt.isNavigable());
    }

    @objid ("0fa68627-4583-11e0-b54c-00137279a832")
    private void setMin(ConnectorEnd ecoreElt) {
        String objingMultMin = this.objElt.getMultiplicityMin();
                
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreElt.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null)
                    ecoreElt.setLower(min);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg =  XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMin",
                    "Connector end");
                    
                    String message =  XMI.I18N.getMessage(
                                                         "logFile.exception.stringConverter.defaultMsg",
                                                         "String", "\"" + objingMultMin + "\"", "Integer",
                                                         contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objElt);
                }
            }
        }
    }

    @objid ("0fa6862a-4583-11e0-b54c-00137279a832")
    private void setMax(ConnectorEnd ecoreElt) {
        String objingMultMax = this.objElt.getMultiplicityMax();
                
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreElt.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null)
                    ecoreElt.setUpper(max);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg =  XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMax",
                    "Connector end");
                    
                    String message = XMI.I18N.getMessage(
                                                        "logFile.exception.stringConverter.defaultMsg",
                                                        "String", "\"" + objingMultMax + "\"", "Integer",
                                                        contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objElt);
                }
            }
        }
    }

    @objid ("0fa8e878-4583-11e0-b54c-00137279a832")
    private void setRole(ConnectorEnd ecoreElt) {
        Element role = this.genProp.getMappedElement(this.objElt.getLinked());
        if (role instanceof ConnectableElement)
            ecoreElt.setRole((ConnectableElement) role);
    }

    @objid ("0fa8e87b-4583-11e0-b54c-00137279a832")
    private void setPartWithPort(ConnectorEnd ecoreElt) {
        IElement linked = this.objElt.getCompositionOwner();
        if (linked instanceof IPort){
            IElement  portOwner = linked.getCompositionOwner();
            Element part = this.genProp.getMappedElement(portOwner);
            if (part instanceof Property){
                ecoreElt.setPartWithPort((Property) part);
            }
                
        }
    }

    @objid ("6761d668-1a76-11e1-8103-0027103f347d")
    private void setDefiningFeature(final Slot ecoreElt) {
        IInstance instanceOwner = this.objElt.getLinked();
        ILink link = this.objElt.getLinkNode();
        if ((instanceOwner != null ) && (link != null)){
            IAssociation assoc = link.getBase();
            INameSpace namespace = instanceOwner.getBase();
            if ((assoc != null ) && (namespace != null)){
                for (IAssociationEnd assocEnd : assoc.getConnection()){
                    if (assocEnd.getOwner().equals(namespace)){
                        Element ecoreProp = this.genProp.getMappedElement(assocEnd);
                        if (ecoreProp instanceof StructuralFeature){
                            ecoreElt.setDefiningFeature((StructuralFeature)ecoreProp);
                        }
                    }
                }
            }
        }
    }

}
