/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ILoopNode;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LoopNode;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecificationAction;

@objid ("0fb271e2-4583-11e0-b54c-00137279a832")
public class OLoopNode extends OElement implements IOElement {
    @objid ("0fb271e3-4583-11e0-b54c-00137279a832")
    private ILoopNode objingElement;

    @objid ("0fb271e4-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = null;

    @objid ("0fb271e5-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0fb271e6-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createLoopNode();
    }

    @objid ("0fb271ea-4583-11e0-b54c-00137279a832")
    public OLoopNode() {
    }

    @objid ("0fb4d43a-4583-11e0-b54c-00137279a832")
    public OLoopNode(ILoopNode element) {
        super(element);
        objingElement = element;
    }

    @objid ("0fb4d443-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // Done when mapping the super type (StructuredActivityNode)
        this.ownerVisitor = ownerVisitor;
    }

    @objid ("0fb4d447-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        // Part of the Properties are setted when mapping the super type
        // (StructuredActivityNode)
                
        // Properties specific to LoopNodes:
        setIsTestedFirst((LoopNode) ecoreElt);
        setBody((LoopNode) ecoreElt);
        setSetup((LoopNode) ecoreElt);
        setTest((LoopNode) ecoreElt);
    }

    @objid ("0fb73696-4583-11e0-b54c-00137279a832")
    private void setIsTestedFirst(LoopNode node) {
        node.setIsTestedFirst(objingElement.isTestedFirst());
    }

    @objid ("0fb73699-4583-11e0-b54c-00137279a832")
    private void setBody(LoopNode node) {
        for (IActivityNode objingBody : objingElement.getBody()) {
            Element ecoreBody = genProp.getMappedElement(objingBody);
            if (ecoreBody instanceof ExecutableNode)
                node.getBodyParts().add((ExecutableNode)ecoreBody);
        }
    }

    @objid ("0fb7369c-4583-11e0-b54c-00137279a832")
    private void setSetup(LoopNode node) {
        String setup = objingElement.getSetup();
        if (setup.length() > 0) {
            ValueSpecificationAction ecoreSetupAction = UMLFactory.eINSTANCE
                    .createValueSpecificationAction();
            ObjingEAnnotation.setDeleted(ecoreSetupAction,true);
            ecoreSetupAction.setName(setup);
            LiteralString value = UMLFactory.eINSTANCE.createLiteralString();
            value.setValue(setup);
            ecoreSetupAction.setValue(value);
            node.getSetupParts().add(ecoreSetupAction);
            attachActionToActivity(ecoreSetupAction);
        }
    }

    @objid ("0fb7369f-4583-11e0-b54c-00137279a832")
    private void setTest(LoopNode node) {
        String test = objingElement.getTest();
        if (test.length() > 0) {
            ValueSpecificationAction ecoreTestAction = UMLFactory.eINSTANCE
                    .createValueSpecificationAction();
            ObjingEAnnotation.setDeleted(ecoreTestAction,true);
            ecoreTestAction.setName(test);
            LiteralString value = UMLFactory.eINSTANCE.createLiteralString();
            value.setValue(test);
            ecoreTestAction.setValue(value);
            node.getTests().add(ecoreTestAction);
            attachActionToActivity(ecoreTestAction);
        }
    }

    @objid ("0fb736a2-4583-11e0-b54c-00137279a832")
    private void attachActionToActivity(ValueSpecificationAction action) {
        // Setting composition relation (attach to Activity)
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                .getEnclosingElement(objingElement, IActivity.class,
                        ownerVisitor);
        if (enclosingActivity != null) {
            Element ecoreActivity = genProp.getMappedElement(enclosingActivity);
            if (ecoreActivity instanceof Activity) {
                ((Activity) ecoreActivity).getNodes().add(action);
            }
        }
    }

}
