/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IArtifact;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Artifact;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Manifestation;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0fb998ee-4583-11e0-b54c-00137279a832")
public class OManifestation extends OElement implements IOElement {
    @objid ("0fb998ef-4583-11e0-b54c-00137279a832")
    private IManifestation objingElement;

    @objid ("0fb998f0-4583-11e0-b54c-00137279a832")
    private Artifact ecoreArtifact = null;


    @objid ("0fb998f1-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (ObjingModelNavigation.isManifestationMappable(objingElement)) {
            return UMLFactory.eINSTANCE.createManifestation();
        } else {
        //            XMILogs.getInstance().writelnInLog(Messages.getString("logFile.warning.export.noumlmanifestation", objingElement.getName()));
            return null;
        }
    }

    @objid ("0fb998f8-4583-11e0-b54c-00137279a832")
    public OManifestation() {
    }

    @objid ("0fbbfb48-4583-11e0-b54c-00137279a832")
    public OManifestation(IManifestation param) {
        super(param);
        objingElement = param;
    }

    @objid ("0fbbfb51-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IArtifact objingArtifact = objingElement.getOwner();
        
        if (objingArtifact != null) {
            // Gets the ecore owner Artifact element:
            ecoreArtifact = (Artifact) genProp.getMappedElement(objingArtifact);
        
            if (ecoreArtifact != null) {
                EList<Manifestation> ecoreManifList = ecoreArtifact.getManifestations();
                if (!ecoreManifList.contains(ecoreElt))
                    ecoreManifList.add((Manifestation)ecoreElt);
            }
        }
    }

    @objid ("0fbbfb55-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setName((Manifestation) ecoreElt);
        setUtilizedElement((Manifestation) ecoreElt);
    }

    @objid ("0fbe5da2-4583-11e0-b54c-00137279a832")
    private void setName(Manifestation manifestation) {
        if (ObjingModelNavigation.isNotNullOrEmpty(objingElement.getName()))
            manifestation.setName(objingElement.getName());
    }

    @objid ("0fbe5da5-4583-11e0-b54c-00137279a832")
    private void setUtilizedElement(Manifestation manifestation) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IModelElement objingUtilizedElt = objingElement.getUtilizedElement();
        
        if (objingUtilizedElt != null) {
            // Gets the ecore utilized element:
            Element ecoreUtilizedElt = genProp
            .getMappedElement(objingUtilizedElt);
        
            if (ecoreUtilizedElt instanceof PackageableElement) {
                manifestation
                .setUtilizedElement((PackageableElement) ecoreUtilizedElt);
            } else if (ecoreArtifact != null) {
                ecoreArtifact.getManifestations().remove(manifestation);
                /*
                 * if (objingUtilizedElt instanceof INameSpace) { UMLPath path =
                 * new UMLPath((INameSpace)objingUtilizedElt);
                 * ObjingEAnnotation.setSupplier(manifestation,
                 * path.getPath(".")); } else
                 * ObjingEAnnotation.setSupplier(manifestation,
                 * objingUtilizedElt.getName()); EAnnotation objingEA =
                 * ObjingEAnnotation.getOrCreateObjingEAnnotation(ecoreArtifact);
                 * if (objingEA != null)
                 * objingEA.getContents().add(manifestation);
                 */
            }
        }
    }

}
