/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  

package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;

/**
 * THis class manages the export of Modelio IModelElement
 * @author ebrosse
 */
@objid ("0fc7e70d-4583-11e0-b54c-00137279a832")
public class OModelElement extends OElement implements IOElement {

	@Override
	@objid ("0fc7e70e-4583-11e0-b54c-00137279a832")
	public Element createEcoreElt() {
		return null;
	}

	@objid ("0fc7e712-4583-11e0-b54c-00137279a832")
	public OModelElement() {
	}

	/**
	 * Constructor
	 * @param element : the exported Modelio IModelElement
	 */
	@objid ("0fca496a-4583-11e0-b54c-00137279a832")
	public OModelElement(final IModelElement element) {
		super(element);
	}

	@Override
	@objid ("0fca496d-4583-11e0-b54c-00137279a832")
	public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
	}

	@Override
	@objid ("0fca4971-4583-11e0-b54c-00137279a832")
	public void setProperties(Element ecoreElt) {
		if (ecoreElt instanceof NamedElement){
			setName((NamedElement) ecoreElt); 
		}

		setEcoreIds(ecoreElt);
	}


	@objid ("653256a4-1a76-11e1-8103-0027103f347d")
	private void setName(final NamedElement ecoreElt) {
		String name = ((IModelElement)getObjingElement()).getName();
		if (ObjingModelNavigation.isNotNullOrEmpty(name))
			ecoreElt.setName(name);
	}

	private void setEcoreIds(final Element ecoreElt) {
		
		String ecoreId = "";
		
		IModelElement obElt = ((IModelElement)getObjingElement());

		for (ITaggedValue tag : obElt.getTag()){
			if (tag.getDefinition().getName().equals(XMITagTypes.MODELELEMENT_ECOREID) 
					&& (tag.getQualifier() != null)){

				ecoreId =  tag.getQualifier().getValue();
				break;
			}
		}
		
		if (ecoreId.equals(""))
			ecoreId = obElt.getIdentifier();

		ObjingEAnnotation.setEcoreId(ecoreElt, ecoreId);
	}
	
	

}
