/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("676f92bc-1a76-11e1-8103-0027103f347d")
public class ONameSpace extends OModelElement implements IOElement {

    @objid ("676fb9ce-1a76-11e1-8103-0027103f347d")
    public Element createEcoreElt() {
        return null;
    }

    @objid ("676fe0e1-1a76-11e1-8103-0027103f347d")
    public ONameSpace() {
    }

    @objid ("677007f2-1a76-11e1-8103-0027103f347d")
    public ONameSpace(final IModelElement element) {
        super(element);
    }

    @objid ("67705616-1a76-11e1-8103-0027103f347d")
    public void attach(final Element ecoreElt, final CompositionOwnerVisitor ownerVisitor) {
    }

    @objid ("6770a43a-1a76-11e1-8103-0027103f347d")
    public void setProperties(final Element ecoreElt) {
        super.setProperties(ecoreElt);
        if (ecoreElt instanceof Classifier)
            this.setAbstract((Classifier)ecoreElt);
        else if (GenerationProperties.getInstance().isRoundtripEnabled()){
            this.setAbstractEAnnotation(ecoreElt);
        }
        
        if (ecoreElt instanceof NamedElement)
            setVisibility((NamedElement) ecoreElt); 
        else if (GenerationProperties.getInstance().isRoundtripEnabled()){
            this.setVisibilityEAnnotation(ecoreElt);
        }
    }

    @objid ("6770f25e-1a76-11e1-8103-0027103f347d")
    private void setAbstract(final Classifier ecoreElt) {
        ecoreElt.setIsAbstract(((INameSpace)getObjingElement()).isAbstract());
    }

    @objid ("67711972-1a76-11e1-8103-0027103f347d")
    private void setVisibility(final NamedElement ecoreElt) {
        IElement objingElement = getObjingElement();
        if (objingElement instanceof INameSpace){
            INameSpace nameSpace = (INameSpace) objingElement;
            switch (nameSpace.getVisibility()) {
            case PACKAGE_VISIBILITY:
                ecoreElt.setVisibility(VisibilityKind.PACKAGE_LITERAL);
                break;
            case PRIVATE:
                ecoreElt.setVisibility(VisibilityKind.PRIVATE_LITERAL);
                break;
            case PROTECTED:
                ecoreElt.setVisibility(VisibilityKind.PROTECTED_LITERAL);
                break;
            case PUBLIC:
                // It is the Default Value
                //          ecoreElt.setVisibility(VisibilityKind.PUBLIC_LITERAL);
                break;
            case VISIBILITY_UNDEFINED:
                ObjingEAnnotation.setUndefined(ecoreElt, true);
                // No undefined field exist in UML2-Ecore for visibility:
                // a visibility not setted corresponds to a PUBLIC visibility.
                break;
            }
        }
    }

    @objid ("67714082-1a76-11e1-8103-0027103f347d")
    private void setAbstractEAnnotation(final Element ecoreElt) {
        IElement objElement = getObjingElement(); 
        if (objElement instanceof INameSpace)
            ObjingEAnnotation.setAbstract(ecoreElt, ((INameSpace)objElement ).isAbstract());
    }

    @objid ("67714086-1a76-11e1-8103-0027103f347d")
    private void setVisibilityEAnnotation(final Element ecoreElt) {
        IElement objElement = getObjingElement(); 
        if (objElement instanceof INameSpace)
            ObjingEAnnotation.setVisibility(ecoreElt, ((INameSpace)objElement ).getVisibility().ordinal());
    }

}
