/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.INode;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Node;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0fd17072-4583-11e0-b54c-00137279a832")
public class ONode extends ONameSpace implements IOElement {
    @objid ("0fd17073-4583-11e0-b54c-00137279a832")
     INode objingElement;


    @objid ("0fd17074-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (objingElement.isStereotyped(XMIStereotypes.UML2DEVICE))
            return UMLFactory.eINSTANCE.createDevice();
        else if (objingElement.isStereotyped(XMIStereotypes.UML2EXECUTIONENVIRONMENT))
            return UMLFactory.eINSTANCE.createExecutionEnvironment();
        else
            return UMLFactory.eINSTANCE.createNode();
    }

    @objid ("0fd17077-4583-11e0-b54c-00137279a832")
    public ONode() {
    }

    @objid ("0fd17079-4583-11e0-b54c-00137279a832")
    public ONode(INode param) {
        super(param);
        objingElement = param;
    }

    @objid ("0fd3d2d0-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = (Element) genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Node) {
                Node ownerIsNode = (Node) ecoreOwner;
                ownerIsNode.getNestedNodes().add((Node)ecoreElt);
            } else if (ecoreOwner instanceof TemplateParameter) {
                TemplateParameter ownerIsTemplateParameter = (TemplateParameter) ecoreOwner;
                ((Classifier)ecoreElt).setOwningTemplateParameter(ownerIsTemplateParameter);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("0fd3d2d4-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
    }

}
