/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IClassAssociation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.SysMLProfileUtils;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.TimeObservation;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class handles the export of INote elements
 * @author ebrosse
 */
@objid ("0fd63526-4583-11e0-b54c-00137279a832")
public class ONote extends OElement implements IOElement {
    @objid ("0fd63527-4583-11e0-b54c-00137279a832")
    private INote objingElement;


    @Override
    @objid ("0fd63528-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IModelElement subject = this.objingElement.getSubject();
        Element annotatedElement =GenerationProperties.getInstance()
                .getMappedElement(subject);
        
        if (this.objingElement.getModel().getName().equals("TimedObservation") && (annotatedElement instanceof NamedElement)){
            return UMLFactory.eINSTANCE.createTimeObservation();
        
        }else{
            Comment ecoreComment = UMLFactory.eINSTANCE.createComment();
            if (SysMLProfileUtils.isSysML(this.objingElement.getModel().getCompositionOwner().getCompositionOwner()))
                GenerationProperties.getInstance().addSysMLExported(this.objingElement);
        
            return ecoreComment;
        
        }
    }

    /**
     * @param element : the exported INote
     */
    @objid ("0fd6352e-4583-11e0-b54c-00137279a832")
    public ONote(final INote element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("0fdaf9e2-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IModelElement subjectModelElement = this.objingElement.getSubject();
        
        if (subjectModelElement != null) {
            // Gets or creates the ecore element that owns the Note:
            Element annotatedElement = genProp.getMappedElement(subjectModelElement);
        
            if (annotatedElement != null) {
        
                if (ecoreElt instanceof Comment){
                    annotatedElement.getOwnedComments().add((Comment)ecoreElt);
                }else{
                    Element ecoreOwner = genProp.getMappedElement(ObjingModelNavigation.getNearestPackage(subjectModelElement));
                    if (ecoreOwner instanceof Package){
                        TimeObservation timeObs = (TimeObservation) ecoreElt;
                        ((Package) ecoreOwner).getPackagedElements().add(timeObs);
                        timeObs.setEvent((NamedElement) annotatedElement );
                        
                    }else{
                        ecoreElt.destroy();
                    }
                }
            }
        }
    }

    @Override
    @objid ("0fdaf9e6-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Comment){
            setTypeEAnnotation((Comment) ecoreElt);
            setContents((Comment) ecoreElt);
            setOwnerAnnotation((Comment) ecoreElt);
        }else{
            setName((TimeObservation) ecoreElt);
        }
    }

    @objid ("0fdd5c34-4583-11e0-b54c-00137279a832")
    private void setContents(Comment ecoreElt) {
        ecoreElt.setBody(this.objingElement.getContent());
    }

    @objid ("0fdd5c37-4583-11e0-b54c-00137279a832")
    private void setTypeEAnnotation(Comment ecoreElt) {
        INoteType noteType = this.objingElement.getModel();
        ObjingEAnnotation.setNoteTypeName(ecoreElt, noteType.getName());
    }

    @objid ("677a68ba-1a76-11e1-8103-0027103f347d")
    private void setOwnerAnnotation(final Comment ecoreElt) {
        if (ecoreElt.getOwner() instanceof AssociationClass){
            if (this.objingElement.getSubject() instanceof IAssociation)
                ObjingEAnnotation.setOwnedByAssociation(ecoreElt, true);
            else if (this.objingElement.getSubject() instanceof IClassAssociation){
                ObjingEAnnotation.setOwnedByAssociationClass(ecoreElt, true);
            }
        }
    }

    @objid ("af44f49d-3ace-11e1-9527-0027103f347d")
    private void setName(final TimeObservation ecoreElt) {
        String name = this.objingElement.getName();
        
        if ((name != null) &&  (!name.equals(""))){
            ecoreElt.setName(name);
        }
    }

}
