/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IDecisionMergeNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IForkJoinNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInputPin;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInterruptibleActivityRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IObjectFlow;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOutputPin;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IStructuredActivityNode;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Activity;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExceptionHandler;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.InterruptibleActivityRegion;
import org.eclipse.uml2.uml.JoinNode;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.MergeNode;
import org.eclipse.uml2.uml.ObjectFlow;
import org.eclipse.uml2.uml.ObjectNode;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0fe220ec-4583-11e0-b54c-00137279a832")
public class OObjectFlow extends OModelElement implements IOElement {
    @objid ("0fe220ed-4583-11e0-b54c-00137279a832")
    private IObjectFlow objingElement;

    @objid ("0fe220ee-4583-11e0-b54c-00137279a832")
    private CompositionOwnerVisitor ownerVisitor = null;

    @objid ("0fe220ef-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @objid ("0fe220f0-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (objingElement.isStereotyped(XMIStereotypes.UML2EXCEPTIONHANDLER))
            return UMLFactory.eINSTANCE.createExceptionHandler();
        else
            return UMLFactory.eINSTANCE.createObjectFlow();
    }

    @objid ("0fe220f3-4583-11e0-b54c-00137279a832")
    public OObjectFlow() {
    }

    @objid ("0fe48342-4583-11e0-b54c-00137279a832")
    public OObjectFlow(IObjectFlow element) {
        super(element);
        objingElement = element;
    }

    @objid ("0fe4834b-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        this.ownerVisitor = ownerVisitor;
                
        IActivityNode ownerNode = objingElement.getSource();
                
        IElement ownerOfIjingNode = ownerVisitor.getCompositionOwner(ownerNode);
                
        Element ownerOfEcoreNode = genProp.getMappedElement(ownerOfIjingNode);
                
        if (ownerOfEcoreNode != null) {
            if (ecoreElt instanceof ExceptionHandler){    
                if (ownerNode instanceof IStructuredActivityNode)
                    ((StructuredActivityNode) genProp.getMappedElement(ownerNode)).getHandlers().add(
                        (ExceptionHandler)ecoreElt);
                else if ((ownerNode instanceof IInputPin) && (ownerNode.getCompositionOwner() instanceof IStructuredActivityNode)) 
                    ((StructuredActivityNode) genProp.getMappedElement(ownerNode.getCompositionOwner())).getHandlers().add(
                            (ExceptionHandler)ecoreElt);
            }else{
                if (ownerOfEcoreNode instanceof StructuredActivityNode) {
                
                    ((StructuredActivityNode) ownerOfEcoreNode).getEdges().add(
                            (ActivityEdge)ecoreElt);
                
                } else {
                    attachToActivity(ecoreElt);
                }
            }
        } else {
            ecoreElt.destroy();
            throw new NotFoundException("Owner Class not found.");
        }
    }

    @objid ("0fe4834f-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof ExceptionHandler){
            setHandlerBody((ExceptionHandler)ecoreElt);    
            setExceptionType((ExceptionHandler)ecoreElt);    
            setExceptionInput((ExceptionHandler)ecoreElt);
            setProtectedNode((ExceptionHandler)ecoreElt);
        }else{
                
            // Properties defined on super classes:
             super.setProperties(ecoreElt);
            setSource((ObjectFlow) ecoreElt);
            setTarget((ObjectFlow) ecoreElt);
            setInterrupts((ObjectFlow) ecoreElt);
            setGuard((ObjectFlow) ecoreElt);
            setWeight((ObjectFlow) ecoreElt);
                
            // Properties of ObjectFlows:
            setEffectEAnnotation((ObjectFlow) ecoreElt);
            setMultiCast((ObjectFlow) ecoreElt);
            setMultiReceive((ObjectFlow) ecoreElt);
            setSelectionBehavior((ObjectFlow) ecoreElt);
            setTransformationBehavior((ObjectFlow) ecoreElt);
        }
    }

    @objid ("0fe6e59c-4583-11e0-b54c-00137279a832")
    private void attachToActivity(Element ecoreElt) {
        IActivity enclosingActivity = (IActivity) ObjingModelNavigation
        .getEnclosingElement(objingElement.getSource(),
                IActivity.class, ownerVisitor);
        if (enclosingActivity != null) {
            Element ecoreActivity = genProp.getMappedElement(enclosingActivity);
                
            if (ecoreActivity != null) {
                if (ecoreActivity instanceof Activity) {
                    Activity owner = (Activity) ecoreActivity;
                    owner.getEdges().add((ActivityEdge)ecoreElt);
                } else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreActivity.getClass().getSimpleName()
                            + ") Not Found");
                }
            } else{
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class not found.");
            }
        }
    }

    @objid ("0fe6e5a2-4583-11e0-b54c-00137279a832")
    private void setSource(ObjectFlow flow) {
        IActivityNode objingSource = objingElement.getSource();
        Element ecoreSource = genProp.getMappedElement(objingSource);
                
        if (ecoreSource instanceof ActivityNode) {
            // Particular case of a MergeNode:
            // report to the mapper of DecisionMergeNode for more information.
            // In case of a DecisionMergeNode that is a Merge AND a Decision,
            // a special structure is constructed. In this case, when exporting
            // a flow, its source can't be theMergeNode, it must be the
            // DecisionNode.
            if (ecoreSource instanceof MergeNode
                    && !ObjingModelNavigation
                    .isMergeNode((IDecisionMergeNode) objingSource)
                    && !ObjingModelNavigation
                    .isDecisionNode((IDecisionMergeNode) objingSource)) {
                EList<?> activityEdgeList = ((MergeNode) ecoreSource)
                .getOutgoings();
                if (activityEdgeList != null && activityEdgeList.size() > 0) {
                    ActivityEdge linkingFlow = (ActivityEdge) activityEdgeList
                    .get(0);
                    ActivityNode source = linkingFlow.getTarget();
                    flow.setSource(source);
                }
            } else if (ecoreSource instanceof JoinNode
                    && !ObjingModelNavigation
                    .isJoinNode((IForkJoinNode) objingSource)
                    && !ObjingModelNavigation
                    .isForkNode((IForkJoinNode) objingSource)) {
                // Same principle for ForkJoinNode elements:
                EList<?> activityEdgeList = ((JoinNode) ecoreSource)
                .getOutgoings();
                if (activityEdgeList != null && activityEdgeList.size() > 0) {
                    ActivityEdge linkingFlow = (ActivityEdge) activityEdgeList
                    .get(0);
                    ActivityNode source = linkingFlow.getTarget();
                    flow.setSource(source);
                }
            } else {
                flow.setSource((ActivityNode) ecoreSource);
            }
        }
    }

    @objid ("0fe6e5a5-4583-11e0-b54c-00137279a832")
    private void setTarget(ObjectFlow flow) {
        IActivityNode objingTarget = objingElement.getTarget();
        Element ecoreTarget = genProp.getMappedElement(objingTarget);
                
        if (ecoreTarget instanceof ActivityNode) {
            // In case of a DecisionMergeNode as a target, if the
            // mapping is normal, there is no particular treatment.
            // If the mapping is complex, the following structure is
            // created: the DecisionMergeNode is mapped as a Merge
            // connected to a Decision by a flow.
            // the "getMappedElement" method returns the MergeNode.
            // As a consequence, there is no particular treatment
            // to do (the target of a flow must be the MergeNode).
            flow.setTarget((ActivityNode) ecoreTarget);
        }
    }

    @objid ("0fe6e5a8-4583-11e0-b54c-00137279a832")
    private void setInterrupts(ObjectFlow flow) {
        IInterruptibleActivityRegion objingRegion = objingElement
        .getInterrupts();
                
        if (objingRegion != null) {
            Element ecoreRegion = genProp.getMappedElement(objingRegion);
            if (ecoreRegion instanceof InterruptibleActivityRegion)
                flow.setInterrupts((InterruptibleActivityRegion) ecoreRegion);
        }
    }

    @objid ("0fe6e5ab-4583-11e0-b54c-00137279a832")
    private void setGuard(ObjectFlow flow) {
        String guard = objingElement.getGuard();
        if (guard.length() > 0) {
            OpaqueExpression ecoreGuard = UMLFactory.eINSTANCE
            .createOpaqueExpression();
            ecoreGuard.getBodies().add(guard);
            flow.setGuard(ecoreGuard);
        }
    }

    @objid ("0fe947f8-4583-11e0-b54c-00137279a832")
    private void setWeight(ObjectFlow flow) {
        String weight = objingElement.getWeight();
        if (weight.length() > 0) {
            try{
                int weightInt = Integer.valueOf(weight);
                LiteralInteger ecoreWeight = UMLFactory.eINSTANCE
                .createLiteralInteger();
                ecoreWeight.setValue(weightInt);
                flow.setWeight(ecoreWeight);
            }catch(Exception e){
                LiteralString ecoreWeight = UMLFactory.eINSTANCE
                .createLiteralString();
                ecoreWeight.setValue(weight);
                flow.setWeight(ecoreWeight);
            }
        }
    }

    @objid ("0fe947fb-4583-11e0-b54c-00137279a832")
    private void setEffectEAnnotation(ObjectFlow flow) {
        String effect;
        switch (objingElement.getEffect()) {
        case CREATE_FLOW:
            effect = "CREATE_FLOW";
            break;
        case DELETE_FLOW:
            effect = "DELETE_FLOW";
            break;
        case EXCEPTION_FLOW:
            effect = "EXCEPTION_FLOW";
            break;
        case READ_FLOW:
            effect = "READ_FLOW";
            break;
        case UPDATEFLOW:
            effect = "UPDATEFLOW";
            break;
        default:
            effect = "READ_FLOW";
        }
                
        ObjingEAnnotation.setEffect(flow, effect);
    }

    @objid ("0fe947fe-4583-11e0-b54c-00137279a832")
    private void setMultiCast(ObjectFlow flow) {
        flow.setIsMulticast(objingElement.isMultiCast());
    }

    @objid ("0fe94801-4583-11e0-b54c-00137279a832")
    private void setMultiReceive(ObjectFlow flow) {
        flow.setIsMultireceive(objingElement.isMultiReceive());
    }

    @objid ("0fe94804-4583-11e0-b54c-00137279a832")
    private void setSelectionBehavior(ObjectFlow flow) {
        String selectionBehavior = objingElement.getSelectionBehavior();
        if (selectionBehavior.length() > 0) {
            OpaqueBehavior behavior = UMLFactory.eINSTANCE
            .createOpaqueBehavior();
            behavior.setName("SelectionBehavior");
            behavior.getBodies().add(selectionBehavior);
            flow.setSelection(behavior);
                
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
            .getEnclosingElement(objingElement, IActivity.class,
                    ownerVisitor);
                
            IPackage objingPkg = ObjingModelNavigation
            .getNearestPackage(enclosingActivity);
                
            if (objingPkg != null) {
                Package ecorePkg = (Package) genProp
                .getMappedElement(objingPkg);
                if (ecorePkg != null)
                    ecorePkg.getPackagedElements().add(behavior);
            }
        }
    }

    @objid ("0febaa50-4583-11e0-b54c-00137279a832")
    private void setTransformationBehavior(ObjectFlow flow) {
        String transformationBehavior = objingElement
        .getTransformationBehavior();
        if (transformationBehavior.length() > 0) {
            OpaqueBehavior behavior = UMLFactory.eINSTANCE
            .createOpaqueBehavior();
            behavior.setName("TransformationBehavior");
            behavior.getBodies().add(transformationBehavior);
            flow.setSelection(behavior);
        }
    }

    @objid ("0febaa53-4583-11e0-b54c-00137279a832")
    private void setExceptionType(ExceptionHandler ecoreElt) {
        for (IDependency exceptionType : ((IModelElement) getObjingElement()).getDependsOnDependency()){
            if (exceptionType.isStereotyped(XMIStereotypes.UML2EXCEPTIONTYPEREFERENCE)){
                IModelElement type = exceptionType.getDependsOn();
                Element ecoreType = GenerationProperties.getInstance().getMappedElement(type);
                if ((ecoreType != null) && (ecoreType instanceof Classifier)){
                    ecoreElt.getExceptionTypes().add((Classifier) ecoreType);
                }
            }
        }
    }

    @objid ("0febaa56-4583-11e0-b54c-00137279a832")
    private void setHandlerBody(ExceptionHandler ecoreElt) {
        IActivityNode body = ((IObjectFlow) getObjingElement()).getTarget();
        Object ecoreBody = null;
        if (body instanceof IInputPin) {
            ecoreBody = GenerationProperties.getInstance().getMappedElement(body.getCompositionOwner());
        }else {
            ecoreBody = GenerationProperties.getInstance().getMappedElement(body);    
        }
                
        if (ecoreBody instanceof ExecutableNode)
            ecoreElt.setHandlerBody((ExecutableNode) ecoreBody);
    }

    @objid ("0febaa59-4583-11e0-b54c-00137279a832")
    private void setProtectedNode(ExceptionHandler ecoreElt) {
        IActivityNode protectedNode = ((IObjectFlow) getObjingElement()).getSource();
        Object ecoreProtectedNode = null;
        if (protectedNode instanceof IOutputPin) {
            ecoreProtectedNode = GenerationProperties.getInstance().getMappedElement(protectedNode.getCompositionOwner());
        }else {
            ecoreProtectedNode = GenerationProperties.getInstance().getMappedElement(protectedNode);    
        }
                
        if ((ecoreProtectedNode != null) && (ecoreProtectedNode instanceof ExecutableNode)){
            ecoreElt.setProtectedNode((ExecutableNode) ecoreProtectedNode);
        }else{
            ecoreElt.destroy();
        }
    }

    @objid ("0febaa5c-4583-11e0-b54c-00137279a832")
    private void setExceptionInput(ExceptionHandler ecoreElt) {
        IActivityNode source = ((IObjectFlow) getObjingElement()).getTarget();
                
        if (source instanceof IInputPin) {
            Object ecoreTarget = GenerationProperties.getInstance().getMappedElement(source);
            if (ecoreTarget instanceof ObjectNode)
                ecoreElt.setExceptionInput((ObjectNode) ecoreTarget);
        }
    }

}
