/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivity;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IObjectNode;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.ObjectNode;
import org.eclipse.uml2.uml.ObjectNodeOrderingKind;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("0fee0caa-4583-11e0-b54c-00137279a832")
public class OObjectNode extends OActivityNode implements IOElement {

    @objid ("0fee0cb1-4583-11e0-b54c-00137279a832")
    public OObjectNode(IObjectNode element) {
        super(element);
    }

    @objid ("0ff06f04-4583-11e0-b54c-00137279a832")
    public OObjectNode() {
    }

    @Override
    @objid ("0ff06f06-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        super.attach(ecoreElt, ownerVisitor);
    }

    @Override
    @objid ("0ff06f0b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setControlType((ObjectNode) ecoreElt);
        setOrdering((ObjectNode) ecoreElt);
        setSelectionBehavior((ObjectNode) ecoreElt);
        setUpperBound((ObjectNode) ecoreElt);
        setType((ObjectNode) ecoreElt);
        setState((ObjectNode) ecoreElt);
    }

    @Override
    @objid ("0ff2d15e-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return null;
    }

    @Override
    @objid ("66921fb6-1a76-11e1-8103-0027103f347d")
    public IObjectNode getObjingElement() {
        return (IObjectNode) super.getObjingElement();
    }

    @objid ("669246c9-1a76-11e1-8103-0027103f347d")
    private void setControlType(final ObjectNode node) {
        node.setIsControlType(this.getObjingElement().isControlType());
    }

    @objid ("66926dda-1a76-11e1-8103-0027103f347d")
    private void setOrdering(final ObjectNode node) {
        switch (this.getObjingElement().getOrdering()) {
        case F_I_F_O:
            node.setOrdering(ObjectNodeOrderingKind.FIFO_LITERAL);
            break;
        case L_I_F_O:
            node.setOrdering(ObjectNodeOrderingKind.LIFO_LITERAL);
            break;
        case ORDERED:
            node.setOrdering(ObjectNodeOrderingKind.ORDERED_LITERAL);
            break;
        case UNORDERED:
            node.setOrdering(ObjectNodeOrderingKind.UNORDERED_LITERAL);
            break;
        default:
            node.setOrdering(ObjectNodeOrderingKind.FIFO_LITERAL);
        }
    }

    @objid ("66926dde-1a76-11e1-8103-0027103f347d")
    private void setSelectionBehavior(final ObjectNode node) {
        String selectionBehavior = this.getObjingElement().getSelectionBehavior();
        if (selectionBehavior.length() > 0) {
            OpaqueBehavior behavior = UMLFactory.eINSTANCE
                    .createOpaqueBehavior();
            behavior.setName("SelectionBehavior");
            behavior.getBodies().add(selectionBehavior);
            node.setSelection(behavior);
                
            // Setting composition relation
            IActivity enclosingActivity = (IActivity) ObjingModelNavigation
                    .getEnclosingElement(this.getObjingElement(), IActivity.class,
                            ownerVisitor);
                
            IPackage objingPkg = ObjingModelNavigation
                    .getNearestPackage(enclosingActivity);
                
            if (objingPkg != null) {
                Package ecorePkg = (Package)  GenerationProperties.getInstance()
                        .getMappedElement(objingPkg);
                if (ecorePkg != null)
                    ecorePkg.getPackagedElements().add(behavior);
                else{
                    node.destroy();
                    throw new NotFoundException("Owner Class not found.");
                }
            }
        }
    }

    @objid ("669294ee-1a76-11e1-8103-0027103f347d")
    private void setUpperBound(final ObjectNode node) {
        String upperBound = this.getObjingElement().getUpperBound();
        if (upperBound.length() > 0) {
            LiteralString literal = UMLFactory.eINSTANCE.createLiteralString();
            literal.setName("UpperBound");
            literal.setValue(upperBound);
            node.setUpperBound(literal);
        }
    }

    @objid ("6692bbfe-1a76-11e1-8103-0027103f347d")
    private void setType(final ObjectNode node) {
        IGeneralClass objingType = this.getObjingElement().getType();
        if (objingType != null) {
                
            if (PrimitiveTypeMapper.isPredefinedType(objingType)) {
                PrimitiveTypeMapper.setEcorePredefinedType(node, (IDataType) objingType);
            }else{
                Element ecoreType =  GenerationProperties.getInstance().getMappedElement(objingType);
                if (ecoreType instanceof Type)
                    node.setType((Type) ecoreType);
            }
            
            
        }
    }

    @objid ("6692bc02-1a76-11e1-8103-0027103f347d")
    private void setState(final ObjectNode node) {
        IState objingState = this.getObjingElement().getInState();
        if (objingState != null) {
            Element ecoreState =  GenerationProperties.getInstance().getMappedElement(objingState);
            if (ecoreState instanceof State)
                node.getInStates().add((State) ecoreState);
        }
    }

}
