/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.usecase.IActor;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Artifact;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioralFeature;
import org.eclipse.uml2.uml.CallConcurrencyKind;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Node;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Reception;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.TemplateSignature;
import org.eclipse.uml2.uml.TemplateableElement;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("10011f81-4583-11e0-b54c-00137279a832")
public class OOperation extends OFeature implements IOElement {
    @objid ("10011f82-4583-11e0-b54c-00137279a832")
    private IOperation objingElement;


    @objid ("10011f83-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IElement objingOwner = objingElement.getCompositionOwner();
        if (((IModelElement) getObjingElement()).isStereotyped(XMIStereotypes.UML2RECEPTION)){
            if (objingOwner instanceof IActor
                    || objingOwner instanceof IUseCase
                    || objingOwner instanceof ISignal) {
        
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, objingElement);
                return null;
        
            }
            return UMLFactory.eINSTANCE.createReception();
        }else if (((IModelElement) getObjingElement()).isStereotyped(XMIStereotypes.UML2REDEFINABLETEMPLATESIGNATURE)){
            return UMLFactory.eINSTANCE.createRedefinableTemplateSignature();
        }else if (((IModelElement) getObjingElement()).isStereotyped(XMIStereotypes.UML2TEMPLATESIGNATURE)){
            return UMLFactory.eINSTANCE.createTemplateSignature();
        }else{
        
            if (objingOwner instanceof IActor
                    || objingOwner instanceof IUseCase
                    || objingOwner instanceof ISignal) {
        
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, objingElement);
                return null;
        
            }
            return UMLFactory.eINSTANCE.createOperation();
        }
    }

    @objid ("10011f86-4583-11e0-b54c-00137279a832")
    public OOperation() {
    }

    @objid ("100381d5-4583-11e0-b54c-00137279a832")
    public OOperation(IOperation element) {
        super(element);
        objingElement = element;
    }

    @objid ("100381de-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        if (ecoreElt.getOwner() == null){
            if (ecoreElt instanceof Operation){
                attachOperation((Operation) ecoreElt);
            } else if (ecoreElt instanceof Reception){
                attachReception((Reception) ecoreElt);
            } else if (ecoreElt instanceof TemplateSignature){
                attachTemplateSignature((TemplateSignature) ecoreElt);
            }
        }
    }

    @objid ("1005e42e-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        
        if (ecoreElt instanceof BehavioralFeature){
            setAbstract((BehavioralFeature) ecoreElt);
            setFinal((BehavioralFeature) ecoreElt);
            setIsClass((BehavioralFeature) ecoreElt);
            setPassingMode((BehavioralFeature) ecoreElt);
            setConcurrency((BehavioralFeature) ecoreElt);        
            setMethod((BehavioralFeature) ecoreElt);
        }
        
        if (ecoreElt instanceof Reception){
            setSignal((Reception) ecoreElt);
        }
        if (ecoreElt instanceof Operation){
            setRedefined((Operation) ecoreElt);
        }
    }

    @objid ("1005e437-4583-11e0-b54c-00137279a832")
    private void setAbstract(BehavioralFeature ecoreElt) {
        ecoreElt.setIsAbstract(objingElement.isAbstract());
    }

    @objid ("1005e43a-4583-11e0-b54c-00137279a832")
    private void setFinal(BehavioralFeature ecoreElt) {
        ecoreElt.setIsLeaf(objingElement.isFinal());
    }

    @objid ("1005e43d-4583-11e0-b54c-00137279a832")
    private void setIsClass(BehavioralFeature ecoreElt) {
        ecoreElt.setIsStatic(objingElement.isClass());
    }

    @objid ("10084689-4583-11e0-b54c-00137279a832")
    private void setPassingMode(BehavioralFeature ecoreElt) {
        boolean isQuery = false;
        switch (objingElement.getPassing()) {
        case METHOD_IN:
            isQuery = true;
            break;
        case METHOD_OUT:
            isQuery = false;
            break;
        }
        
        if (ecoreElt instanceof Operation)
            ((Operation)ecoreElt).setIsQuery(isQuery);
    }

    @objid ("1008468c-4583-11e0-b54c-00137279a832")
    private void setConcurrency(BehavioralFeature ecoreElt) {
        // Since concurrency is represented as a boolean in Ijing, the mapping
        // is true for the
        // CONCURRENT value, and false else (SEQUENTIAL by default):
        if (objingElement.isConcurrency())
            ecoreElt.setConcurrency(CallConcurrencyKind.CONCURRENT_LITERAL);
        else
            ecoreElt.setConcurrency(CallConcurrencyKind.SEQUENTIAL_LITERAL);
    }

    @objid ("1008468f-4583-11e0-b54c-00137279a832")
    private void setMethod(BehavioralFeature ecoreElt) {
        for (IDependency depend : objingElement.getDependsOnDependency()){
            if (depend.isStereotyped(XMIStereotypes.UML2METHODREFERENCE)){
                Element behavior = GenerationProperties.getInstance().getMappedElement(depend.getDependsOn());
                if (behavior instanceof Behavior){
                    ecoreElt.getMethods().add((Behavior) behavior);
                }
            }
        }
    }

    @objid ("10084692-4583-11e0-b54c-00137279a832")
    private void attachReception(Reception ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            // Operations defined on Actor, UseCase and Signal are not exported
            // in
            // UML2. They are
            // added into an EAnnotation:
            if (objingOwner instanceof IActor
                    || objingOwner instanceof IUseCase
                    || objingOwner instanceof ISignal) {
                /*
                 * if (ecoreOwner == null) { GenerationProperties genProp =
                 * GenerationProperties.getInstance();
                 * objingOwner.accept(genProp.getIjingVisitor()); ecoreOwner =
                 * (Element)revProp.getMappedElement(objingElement.getOwner().getId()); }
                 * if (ecoreOwner != null) { EAnnotation objingEA =
                 * ObjingEAnnotation.getOrCreateObjingEAnnotation(ecoreOwner);
                 * if (objingEA != null) objingEA.getContents().add(ecoreElt); }
                 */
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, objingElement, ecoreElt);
            } else if (ecoreOwner != null) {
                if (ecoreOwner instanceof Component) {
                    Component ownerIsComponent = (Component) ecoreOwner;
                    ownerIsComponent.getOwnedReceptions().add(ecoreElt);
                } else if (ecoreOwner instanceof Node) {
                    Node ownerIsNode = (Node) ecoreOwner;
                    ownerIsNode.getOwnedReceptions().add(ecoreElt);
                } else
                    // An AssociationClass is a Class.
                    if (ecoreOwner instanceof Class) {
                        Class ownerIsClass = (Class) ecoreOwner;
                        ownerIsClass.getOwnedReceptions().add(ecoreElt);
                    } else if (ecoreOwner instanceof Interface) {
                        Interface ownerIsInterface = (Interface) ecoreOwner;
                        ownerIsInterface.getOwnedReceptions().add(ecoreElt);
                    } else {
                        ecoreElt.destroy();
                        //                    XMILogs xmilogs = XMILogs.getInstance();
                        //                    xmilogs.writelnInLog(Messages.getString("logFile.warning.unsupportedOwnerExport", ))
                        throw new NotFoundException("Owner Class ("
                                + ecoreOwner.getClass().getSimpleName()
                                + ") Not Found");
                    }
            }
        }
    }

    @objid ("10084696-4583-11e0-b54c-00137279a832")
    private void attachOperation(Operation ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            // Operations defined on Actor, UseCase and Signal are not exported
            // in
            // UML2. They are
            // added into an EAnnotation:
            if (objingOwner instanceof IActor
                    || objingOwner instanceof IUseCase
                    || objingOwner instanceof ISignal) {
                /*
                 * if (ecoreOwner == null) { GenerationProperties genProp =
                 * GenerationProperties.getInstance();
                 * objingOwner.accept(genProp.getIjingVisitor()); ecoreOwner =
                 * (Element)revProp.getMappedElement(objingElement.getOwner().getId()); }
                 * if (ecoreOwner != null) { EAnnotation objingEA =
                 * ObjingEAnnotation.getOrCreateObjingEAnnotation(ecoreOwner);
                 * if (objingEA != null) objingEA.getContents().add(ecoreElt); }
                 */
                ObjingModelNavigation.infoOfUnsupportedOwnedWithEMF(
                        objingOwner, objingElement, ecoreElt);
        
            } else if (ecoreOwner != null) {
                if (ecoreOwner instanceof Component) {
                    Component ownerIsComponent = (Component) ecoreOwner;
                    ownerIsComponent.getOwnedOperations().add(ecoreElt);
                } else if (ecoreOwner instanceof Node) {
                    Node ownerIsNode = (Node) ecoreOwner;
                    ownerIsNode.getOwnedOperations().add(ecoreElt);
                } else
                    // An AssociationClass is a Class.
                    if (ecoreOwner instanceof Class) {
                        Class ownerIsClass = (Class) ecoreOwner;
                        ownerIsClass.getOwnedOperations().add(ecoreElt);
                    } else if (ecoreOwner instanceof Interface) {
                        Interface ownerIsInterface = (Interface) ecoreOwner;
                        ownerIsInterface.getOwnedOperations().add(ecoreElt);
                    } else if (ecoreOwner instanceof DataType) {
                        DataType ownerIsDataType = (DataType) ecoreOwner;
                        ownerIsDataType.getOwnedOperations().add(ecoreElt);
                    } else if (ecoreOwner instanceof Artifact) {
                        Artifact ownerIsArtifact = (Artifact) ecoreOwner;
                        ownerIsArtifact.getOwnedOperations().add(ecoreElt);
                    } else {
                        ecoreElt.destroy();
                        //                    XMILogs xmilogs = XMILogs.getInstance();
                        //                    xmilogs.writelnInLog(Messages.getString("logFile.warning.unsupportedOwnerExport", ))
                        throw new NotFoundException("Owner Class ("
                                + ecoreOwner.getClass().getSimpleName()
                                + ") Not Found");
                    }
            }
        }
    }

    @objid ("100aa8e4-4583-11e0-b54c-00137279a832")
    private void setSignal(Reception ecoreElt) {
        ObList<ISignal> lists = ((IOperation) getObjingElement()).getSRepresentation();
        if (lists.size()>0){
            Element ecoreSignal = GenerationProperties.getInstance().getMappedElement(lists.get(0));
            if (ecoreSignal instanceof Signal)
                ecoreElt.setSignal((Signal) ecoreSignal);
        }
    }

    @objid ("100aa8e7-4583-11e0-b54c-00137279a832")
    private void setRedefined(Operation ecoreElt) {
        for (IOperation redefines : objingElement.getRedefinition()){
            Element ecoreOperation = GenerationProperties.getInstance().getMappedElement(redefines);
            if (ecoreOperation instanceof Operation){
                ecoreElt.getRedefinedOperations().add((Operation)ecoreOperation);
            }
        }
    }

    @objid ("27202073-b08d-11e1-bf0d-0027103f347d")
    private void attachTemplateSignature(TemplateSignature ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        if (ecoreOwner instanceof TemplateableElement)
            ecoreElt.setTemplate((TemplateableElement)ecoreOwner);
        else 
            ecoreElt.destroy();
    }

}
