/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IOutputPin;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.AttachOutputPinToOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExpansionNode;
import org.eclipse.uml2.uml.ExpansionRegion;
import org.eclipse.uml2.uml.OutputPin;
import org.eclipse.uml2.uml.Pin;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("100aa8ea-4583-11e0-b54c-00137279a832")
public class OOutputPin extends OPin implements IOElement {
    @objid ("100aa8ed-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("100d0b3c-4583-11e0-b54c-00137279a832")
    private static AttachOutputPinToOwnerVisitor attachOutputPin = new AttachOutputPinToOwnerVisitor();

    @objid ("100d0b3e-4583-11e0-b54c-00137279a832")
    private Element ecoreOwnerElt;


    @objid ("100d0b3f-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (getObjingElement().isStereotyped(XMIStereotypes.UML2EXPANSIONNODE))
            return UMLFactory.eINSTANCE.createExpansionNode();
        else{
            return UMLFactory.eINSTANCE.createOutputPin();
        }
    }

    @objid ("100d0b42-4583-11e0-b54c-00137279a832")
    private boolean allPinsAreMapped() {
        IActivityAction objingOwner = ((IOutputPin)getObjingElement()).getOutputing();
        ObList<IOutputPin> objingOutputPinList = objingOwner.getOutput();
        
        if (ecoreOwnerElt instanceof Action) {
            Action ecoreOwner = (Action) ecoreOwnerElt;
            EList<?> ecoreOutputPinList = ecoreOwner.getOutputs();
        
            if (ecoreOutputPinList != null
                    && objingOutputPinList.size() == ecoreOutputPinList.size())
                return true;
        }
        return false;
    }

    @objid ("100d0b46-4583-11e0-b54c-00137279a832")
    private List<IOutputPin> getSortedOutputPinList() {
        IActivityAction objingOwner = ((IOutputPin)getObjingElement()).getOutputing();
        ObList<IParameter> objingParamList = ObjingModelNavigation
        .getRelatedParameters(objingOwner);
        List<IOutputPin> objingOutputPinList = new ArrayList<IOutputPin>(
                objingOwner.getOutput());
        List<IOutputPin> objingPinSortedList = new ArrayList<IOutputPin>();
        
        for (IParameter param : objingParamList) {
            IOutputPin matchingPin = getMatchingPin(objingOutputPinList, param);
            if (matchingPin != null) {
                objingOutputPinList.remove(matchingPin);
                objingPinSortedList.add(matchingPin);
            } else {
                return new ArrayList<IOutputPin>();
            }
        }
        
        if (!(objingOutputPinList.size() == 0)) {
            return new ArrayList<IOutputPin>();
        }
        return objingPinSortedList;
    }

    @objid ("100d0b4c-4583-11e0-b54c-00137279a832")
    public OOutputPin() {
    }

    @objid ("100f6d97-4583-11e0-b54c-00137279a832")
    public OOutputPin(IOutputPin element) {
        super(element);
        ecoreOwnerElt = genProp.getMappedElement(element.getOutputing());
    }

    @objid ("100f6da0-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IElement objingOwner = ownerVisitor.getCompositionOwner(((IOutputPin)getObjingElement()));
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        if (ecoreElt instanceof ExpansionNode){
        
        
            if (ecoreOwner instanceof StructuredActivityNode){
                ((StructuredActivityNode) ecoreOwner ).getNodes().add((ExpansionNode)ecoreElt);
            }
        
        }else{
            if (ecoreOwner != null) {
                if (objingOwner instanceof IActivityAction)
                    attachToActivityAction(ecoreElt, ecoreOwner);
                else
                    super.attach(ecoreElt, ownerVisitor);
            }
        }
    }

    @objid ("1011cff0-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        if (ecoreElt instanceof ExpansionNode){
            setOutputRegion((ExpansionNode) ecoreElt);
        
        }
        
        
        setMatched((Pin) ecoreElt);
    }

    @objid ("1011cff3-4583-11e0-b54c-00137279a832")
    private void attachToActivityAction(Element ecoreElt, Element ecoreOwner) {
        attachOutputPin.attachOutputPin((OutputPin) ecoreElt, ecoreOwner);
    }

    @objid ("1018f6fe-4583-11e0-b54c-00137279a832")
    private void setMatched(Pin pin) {
        if (allPinsAreMapped()) {
            List<IOutputPin> objingPinSortedList = getSortedOutputPinList();
            List<OutputPin> ecoreOutputPinList = ((Action)ecoreOwnerElt).getOutputs();
        
            for (IOutputPin objingPin : objingPinSortedList) {
                if (ecoreOwnerElt instanceof Action) {
                    Element ecorePin = genProp.getMappedElement(objingPin);
                    if (ecorePin instanceof OutputPin) {
                        ecoreOutputPinList.remove(ecorePin);
                        ecoreOutputPinList.add((OutputPin) ecorePin);
                    }
                }
            }
        }
    }

    @objid ("1018f701-4583-11e0-b54c-00137279a832")
    private IOutputPin getMatchingPin(List<IOutputPin> objingOutputPinList, IParameter param) {
        for (IOutputPin pin : objingOutputPinList) {
            if (param.equals(pin.getMatched()))
                return pin;
        }
        return null;
    }

    @objid ("1018f709-4583-11e0-b54c-00137279a832")
    private void setOutputRegion(ExpansionNode ecoreElt) {
        if (ecoreElt.getOwner() instanceof ExpansionRegion)
            ecoreElt.setRegionAsOutput((ExpansionRegion)ecoreElt.getOwner());
    }

}
