/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("101b595b-4583-11e0-b54c-00137279a832")
public class OPackage extends ONameSpace implements IOElement {

    @objid ("101b595c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createPackage();
    }

    @objid ("101b5960-4583-11e0-b54c-00137279a832")
    public OPackage() {
    }

    @objid ("101dbbb2-4583-11e0-b54c-00137279a832")
    public OPackage(IPackage element) {
        super(element);
    }

    @objid ("101dbbbb-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IElement objingOwner = ownerVisitor.getCompositionOwner((IPackage)getObjingElement());
        
        if (objingOwner != null) {// else : case of the root package
            Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
            if (ecoreOwner != null) {
                if (ecoreOwner instanceof Package) {
                    Package ownerIsPkg = (Package) ecoreOwner;
                    ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
                } else if (ecoreOwner instanceof Component) {
                    Component ownerIsCmpnt = (Component) ecoreOwner;
                    ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
                }
                else {
                    ecoreElt.destroy();
                    throw new NotFoundException("Owner Class ("
                            + ecoreOwner.getClass().getSimpleName()
                            + ") Not Found");
                }
            }
        }
    }

    @objid ("101dbbbf-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            this.setLeafEAnnotation((Package) ecoreElt); 
            this.setRootEAnnotation((Package) ecoreElt);
            this.setInstantiableEAnnotation((Package) ecoreElt);
        }
    }

    @objid ("10201e16-4583-11e0-b54c-00137279a832")
    private void setInstantiableEAnnotation(Package ecoreElt) {
        ObjingEAnnotation.setInstantiable(ecoreElt, ((IPackage)getObjingElement()).isInstantiable());
    }

    @objid ("10201e19-4583-11e0-b54c-00137279a832")
    private void setLeafEAnnotation(Package ecoreElt) {
        ObjingEAnnotation.setLeaf(ecoreElt, ((IPackage)getObjingElement()).isLeaf());
    }

    @objid ("10201e1c-4583-11e0-b54c-00137279a832")
    private void setRootEAnnotation(Package ecoreElt) {
        ObjingEAnnotation.setRoot(ecoreElt, ((IPackage)getObjingElement()).isRoot());
    }

}
