/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.model.uml.statik.IPackageImport;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageImport;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.VisibilityKind;

@objid ("10228068-4583-11e0-b54c-00137279a832")
public class OPackageImport extends OElement implements IOElement {
    @objid ("10228069-4583-11e0-b54c-00137279a832")
    private IPackageImport objingElement;


    @objid ("1022806a-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        PackageImport ecorePI = UMLFactory.eINSTANCE.createPackageImport();
        return ecorePI;
    }

    @objid ("1022806e-4583-11e0-b54c-00137279a832")
    public OPackageImport() {
    }

    @objid ("1024e2c0-4583-11e0-b54c-00137279a832")
    public OPackageImport(IPackageImport element) {
        super(element);
        objingElement = element;
    }

    @objid ("1024e2c9-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        PackageImport ecorePkgImport = (PackageImport) ecoreElt;
                
        IModelElement objingImportingElt = ObjingModelNavigation
                .getImportingElement(objingElement);
        IPackage objingImportedPkg = objingElement.getImportedPackage();
                
        if (objingImportingElt != null && objingImportedPkg != null) {
                
            // Gets or creates the ecore importing element (the "owner"):
            Element ecoreImportingElt = (Element) genProp
                    .getMappedElement(objingImportingElt);
                
            // Gets or creates the ecore imported Package (the Package to
            // import):
            Package ecoreImportedPkg = (Package) genProp
                    .getMappedElement(objingImportedPkg);
                
            if (ecoreImportingElt != null && ecoreImportedPkg != null) {
                ((Namespace) ecoreImportingElt).getPackageImports().add(
                        (PackageImport)ecoreElt);
                
                // In UML2, a Package import is defined as a directed
                // relationship between an
                // importing namespace and a Package.
                if (ecoreImportingElt instanceof Namespace) {
                    ecorePkgImport
                            .setImportingNamespace((Namespace) ecoreImportingElt);
                    ecorePkgImport.setImportedPackage(ecoreImportedPkg);
                }
            }
        }
    }

    @objid ("1024e2cd-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setNameEAnnotation((PackageImport) ecoreElt);
        setVisibility((PackageImport) ecoreElt);
    }

    @objid ("1027451b-4583-11e0-b54c-00137279a832")
    private void setNameEAnnotation(PackageImport ecoreElt) {
        ObjingEAnnotation.setName(ecoreElt, objingElement.getName());
    }

    @objid ("1027451e-4583-11e0-b54c-00137279a832")
    private void setVisibility(PackageImport ecoreElt) {
        // Note that in theory, the visibility of a PackageImport is either
        // public or private.
        switch (objingElement.getVisibility()) {
        case PACKAGE_VISIBILITY:
            ecoreElt.setVisibility(VisibilityKind.PACKAGE_LITERAL);
            break;
        case PRIVATE:
            ecoreElt.setVisibility(VisibilityKind.PRIVATE_LITERAL);
            break;
        case PROTECTED:
            ecoreElt.setVisibility(VisibilityKind.PROTECTED_LITERAL);
            break;
        case PUBLIC:
            // It is the Default Value
        //          ecoreElt.setVisibility(VisibilityKind.PUBLIC_LITERAL);
            break;
        case VISIBILITY_UNDEFINED:
            ObjingEAnnotation.setUndefined(ecoreElt, true);
            // No undefined field exist in UML2-Ecore for visibility:
            // a visibility not setted corresponds to a PUBLIC visibility.
            break;
        }
    }

}
