/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateParameter;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Reception;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IParameter elements
 * @author ebrosse
 */
@objid ("102c09d5-4583-11e0-b54c-00137279a832")
public class OParameter extends OModelElement {
    @objid ("102c09d6-4583-11e0-b54c-00137279a832")
    private IParameter objingElement = null;


    @Override
    @objid ("102c09d7-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (this.objingElement.isStereotyped(XMIStereotypes.UML2CLASSIFIERTEMPLATEPARAMETER))
            return UMLFactory.eINSTANCE.createClassifierTemplateParameter();
        else if (this.objingElement.isStereotyped(XMIStereotypes.UML2OPERATIONTEMPLATEPARAMETER))
            return UMLFactory.eINSTANCE.createOperationTemplateParameter();
        else    
            return UMLFactory.eINSTANCE.createParameter();
    }

    /**
     * Constructor
     * @param element : the exported Modelio IParameter
     */
    @objid ("102e6c2a-4583-11e0-b54c-00137279a832")
    public OParameter(final IParameter element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("102e6c33-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IElement objingParent = ownerVisitor
        .getCompositionOwner(this.objingElement);
        
        Element ecoreParent = genProp.getMappedElement(objingParent);
        
        if (ecoreParent != null) {
            if (objingParent instanceof IOperation){
                if(  ecoreParent instanceof Operation) {
                    Operation ecoreOperation = (Operation) ecoreParent;
                    ecoreOperation.getOwnedParameters().add((Parameter)ecoreElt);
                }else  if(  ecoreParent instanceof Reception) {
                    Reception ecoreOperation = (Reception) ecoreParent;
                    ecoreOperation.getOwnedParameters().add((Parameter)ecoreElt);
                }
            }
        }
    }

    @Override
    @objid ("1030ce84-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (!ObjingModelNavigation
                .isReturnParameter(this.objingElement))
             super.setProperties(ecoreElt);
        
        setParameterPassingMode((Parameter) ecoreElt);
        setMin((Parameter) ecoreElt);
        setMax((Parameter) ecoreElt);
        setTypeConstraintEAnnotation((Parameter) ecoreElt);      
        setDefaultValue((Parameter) ecoreElt);        
        setClass((Parameter) ecoreElt);
        setOrdered((Parameter) ecoreElt);
        setUnique((Parameter) ecoreElt);
    }

    @objid ("1030ce8a-4583-11e0-b54c-00137279a832")
    private void setParameterPassingMode(Parameter ecoreElt) {
        switch (this.objingElement.getParameterPassing()) {
        case IN:
            ecoreElt.setDirection(ParameterDirectionKind.IN_LITERAL);
            break;
        case OUT:
            ecoreElt.setDirection(ParameterDirectionKind.OUT_LITERAL);
            break;
        case INOUT:
            ecoreElt.setDirection(ParameterDirectionKind.INOUT_LITERAL);
            break;
        }
        if (ObjingModelNavigation.isReturnParameter(this.objingElement))
            ecoreElt.setDirection(ParameterDirectionKind.RETURN_LITERAL);
    }

    @objid ("1030ce8d-4583-11e0-b54c-00137279a832")
    private void setMin(Parameter ecoreElt) {
        String objingMultMin = this.objingElement.getMultiplicityMin();
        
        // If objingMultMin is "" then we don't set a lower multiplicity for the
        // UML2 element.
        if (!"".equals(objingMultMin)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMin))
                ecoreElt.setLower(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer min = StringConverter.getInteger(objingMultMin);
                if (min != null)
                    ecoreElt.setLower(min);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg = XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMinWithName",
                            "Parameter",ecoreElt.getName());
                    String message = XMI.I18N.getMessage(
                                                        "logFile.exception.stringConverter.defaultMsg",
                                                        "String", "\"" + objingMultMin + "\"", "Integer",
                                                        contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("1030ce90-4583-11e0-b54c-00137279a832")
    private void setMax(Parameter ecoreElt) {
        String objingMultMax = this.objingElement.getMultiplicityMax();
        
        // If objingMultMax is "" then we don't set an upper multiplicity for
        // the UML2 element.
        if (!"".equals(objingMultMax)) {
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE
                    .equals(objingMultMax))
                ecoreElt.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                StringConverter.setFilterEnabled(true);
                Integer max = StringConverter.getInteger(objingMultMax);
                if (max != null)
                    ecoreElt.setUpper(max);
                else {
                    XMILogs logs = XMILogs.getInstance();
                    String contextualMsg =XMI.I18N.getMessage(
                            "logFile.exception.stringConverter.multiplicityMaxWithName",
                            "Parameter",ecoreElt.getName());
                    String message = XMI.I18N.getMessage(
                                                        "logFile.exception.stringConverter.defaultMsg",
                                                        "String", "\"" + objingMultMax + "\"", "Integer",
                                                        contextualMsg);
                    logs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addInfo(message, this.objingElement);
                }
            }
        }
    }

    @objid ("103330dc-4583-11e0-b54c-00137279a832")
    private void setTypeConstraintEAnnotation(Parameter ecoreElt) {
        ObjingEAnnotation.setTypeConstraint(ecoreElt, this.objingElement
                .getTypeConstraint());
    }

    @objid ("103330df-4583-11e0-b54c-00137279a832")
    private void setDefaultValue(Parameter ecoreParam) {
        String objingDefaultValue = this.objingElement.getDefaultValue();
        IGeneralClass objingType = this.objingElement.getType();
        IUMLTypes umlTypes = Modelio.getInstance() .getModelingSession().getModel().getUmlTypes();
        
        // If objingValue is "" then we don't set a default value for the UML2
        // element.
        if (!("".equals(objingDefaultValue))){
            if (objingType != null) {
                if (PrimitiveTypeMapper.isPredefinedType(objingType)) {
        
                    IDataType objingPredefinedType = (IDataType) objingType;
        
                    if (( ObjingModelNavigation.OBJING_NULL_VALUE != null) 
                            && ( ObjingModelNavigation.OBJING_NULL_VALUE.equals(objingDefaultValue.toLowerCase()))){
                        ecoreParam.setNullDefaultValue();
                    }else if ((umlTypes.getBOOLEAN() != null) && (umlTypes.getBOOLEAN().equals(objingPredefinedType))) {
                        // If the attribute is a Dynamic Dependency, we don't enable
                        // string filters:
                        StringConverter.setFilterEnabled(true);
                        Boolean bool = StringConverter
                        .getBoolean(objingDefaultValue);
                        if (bool != null)
                            ecoreParam.setBooleanDefaultValue(bool);
                        else {
                            XMILogs logs = XMILogs.getInstance();
                            String contextualMsg = XMI.I18N.getMessage(
                                    "logFile.exception.stringConverter.defaultValue",
                                    this.objingElement.getName(),
                            "Parameter");
                            String message = XMI.I18N.getMessage(
                                                                "logFile.exception.stringConverter.defaultMsg",
                                                                "String", "\"" + objingDefaultValue + "\"",
                                                                "Boolean", contextualMsg);
                            logs.writelnInLog(message);
                            GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
                            ecoreParam.setStringDefaultValue(objingDefaultValue);
                        }
                    } else if ((umlTypes.getCHAR() != null) && (umlTypes.getCHAR().equals(objingPredefinedType))) {
                        ecoreParam.setStringDefaultValue(objingDefaultValue);
                    } else if ((umlTypes.getSTRING() != null) && (umlTypes.getSTRING().equals(objingPredefinedType))){
        
                        ecoreParam.setStringDefaultValue(objingDefaultValue);
                    } else if ((umlTypes.getINTEGER() != null) && (umlTypes.getINTEGER().equals(objingPredefinedType))){
                        // If the attribute is a Dynamic Dependency, we don't enable
                        // string filters:
                        StringConverter.setFilterEnabled(true);
                        Integer objingIntValue = StringConverter
                        .getInteger(objingDefaultValue);
                        if (objingIntValue != null) {
                            if (objingIntValue >= 0)
                                ecoreParam
                                .setUnlimitedNaturalDefaultValue(objingIntValue);
                            else
                                ecoreParam.setIntegerDefaultValue(objingIntValue);
                        } else {
                            XMILogs logs = XMILogs.getInstance();
                            String contextualMsg = XMI.I18N.getMessage(
                                    "logFile.exception.stringConverter.defaultValue",
                                    this.objingElement.getName(),
                            "Parameter");
                            
                            String message = XMI.I18N.getMessage(
                                                                "logFile.exception.stringConverter.defaultMsg",
                                                                "String", "\"" + objingDefaultValue + "\"",
                                                                "Integer", contextualMsg);
                            logs.writelnInLog(message);
                            GenerationProperties.getInstance().getReportModel().addInfo(contextualMsg, this.objingElement,  message);
                            ecoreParam.setStringDefaultValue(objingDefaultValue);
                        }
                    } else 
                        ecoreParam.setStringDefaultValue(objingDefaultValue);
                } else if ((objingType instanceof IEnumeration) 
                        && (ObjingModelNavigation.isEnumerationliteral((IEnumeration)objingType, objingDefaultValue))){
        
                    InstanceValue value = UMLFactory.eINSTANCE.createInstanceValue();
        
                    Object ecoreType = GenerationProperties.getInstance().getMappedElement(objingType);
                    if (ecoreType instanceof Type){
                        value.setType((Type) ecoreType);
                    }
        
                    Object ecoreInstance = GenerationProperties.getInstance().getMappedElement(ObjingModelNavigation.getEnumerationliteral((IEnumeration)objingType, objingDefaultValue));
                    if (ecoreInstance instanceof InstanceSpecification){
                        value.setInstance((InstanceSpecification) ecoreInstance);
                    }
        
                    ecoreParam.setDefaultValue(value);
                }else{
                    ecoreParam.setStringDefaultValue(objingDefaultValue);
                }
            }else{
                ecoreParam.setStringDefaultValue(objingDefaultValue);
            }
        }else
            ecoreParam.setStringDefaultValue(objingDefaultValue);
    }

    @objid ("103330e2-4583-11e0-b54c-00137279a832")
    private void setClass(Parameter ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        // Getting type of the Parameter:
        IGeneralClass objingType = this.objingElement.getType();
        if (objingType != null) {
            if (PrimitiveTypeMapper.isPredefinedType(objingType)) {
                PrimitiveTypeMapper.setEcorePredefinedType(ecoreElt, (IDataType) objingType);
            } else {
        
                if (objingType instanceof ITemplateParameter) {
                    return;
        
                } else {
                    Element ecoreEltType = genProp.getMappedElement(objingType);
        
                    if (ecoreEltType instanceof Type)
                        setType(ecoreElt, (Type) ecoreEltType);
                }
            }
        }else {
            ObjingEAnnotation.setNoType(ecoreElt, true);
        }
    }

    @objid ("103330e5-4583-11e0-b54c-00137279a832")
    private void setType(Parameter ecoreParam, Type type) {
        ecoreParam.setType(type);
    }

    @objid ("5ec2ff22-63b7-11e1-904f-0027103f347d")
    private void setUnique(final Parameter ecoreElt) {
        ecoreElt.setIsUnique(this.objingElement.isUnique());
    }

    @objid ("5ec32635-63b7-11e1-904f-0027103f347d")
    private void setOrdered(final Parameter ecoreElt) {
        ecoreElt.setIsOrdered(this.objingElement.isOrdered());
    }

}
