/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1037f59b-4583-11e0-b54c-00137279a832")
public class OPort extends OElement implements IOElement {
    @objid ("103a57ea-4583-11e0-b54c-00137279a832")
    private IPort objingElement;

    @objid ("103a57eb-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @Override
    @objid ("103a57ed-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        Element ecoreElement = null;
                
        IElement objOwner = ObjingModelNavigation.getBindableInstanceOwner(this.objingElement);
                
        if ( objOwner instanceof IInstance)
            ecoreElement =  UMLFactory.eINSTANCE.createSlot();
        else{
            IElement owner  = this.objingElement.getCluster();
                
            boolean exist = false; 
            IModelElement representedFeature = this.objingElement.getRepresentedFeature();
            if (owner instanceof IInstance){
                INameSpace base = ((IInstance) owner).getBase();
                exist = ((base!= null) && (representedFeature != null) 
                        && (representedFeature instanceof IPort)
                        && (representedFeature.getCompositionOwner().equals(base)));
            }
                
            if (exist){
                 return this.genProp.getMappedElement(representedFeature);            
            }else{ 
                  if ((representedFeature != null) && (representedFeature instanceof IPort)
                          && !(representedFeature.getCompositionOwner() instanceof IInstance)){
                      ecoreElement = this.genProp.getMappedElement(representedFeature);
                  }else
                      ecoreElement =  UMLFactory.eINSTANCE.createPort();
            }
        }
        return ecoreElement;
    }

    @objid ("103a57f3-4583-11e0-b54c-00137279a832")
    public OPort(IPort element) {
        super(element);
        this.objingElement = element;
    }

    @Override
    @objid ("103cba48-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        /*  Type of possible owners of a Port in Modelio: // - IClassifier
         * (from IBindableInstance) // - IInstance (from '' ) // - INameSpace
         * (from IInstance) // - ITemplateParameter (from IModelElement) //
         * Type of possible owners of a Slot in Uml2: // - InstanceSpecification
         * (from slot) // - Element (from Element) // - EAnnotation (from
          EObject) 
         */
                
        IElement objingOwner =  ownerVisitor.getCompositionOwner(this.objingElement);
                
        if (objingOwner instanceof IBindableInstance) { 
             attachPortToBindableInstance((IBindableInstance)objingOwner, ecoreElt);
        }else if (objingOwner instanceof IInstance){
            attachPortToInstance((IInstance)objingOwner, (Slot) ecoreElt);
        }else{
            attachPortToClassifier( objingOwner, (Port) ecoreElt);
        }
    }

    @objid ("103cba4c-4583-11e0-b54c-00137279a832")
    private void attachPortToClassifier(IElement objingOwner, Port ecoreElt) {
        Element ecoreOwner =  this.genProp.getMappedElement(objingOwner);
                
        if (objingOwner instanceof INameSpace ){
            if ( ecoreOwner instanceof  Class) { 
                
                if (!((Class) ecoreOwner).getOwnedPorts().contains(ecoreElt))
                    ((Class) ecoreOwner).getOwnedPorts().add(ecoreElt); 
                
            }else if (ecoreOwner instanceof Signal){
                
                if (!((Signal) ecoreOwner).getOwnedAttributes().contains(ecoreElt))
                    ((Signal) ecoreOwner).getOwnedAttributes().add(ecoreElt); 
                
            }else    if (ecoreOwner instanceof Component){
                
                if (!((Component) ecoreOwner).getOwnedAttributes().contains(ecoreElt))
                    ((Component) ecoreOwner).getOwnedAttributes().add(ecoreElt); 
                
            }else    if (ecoreOwner instanceof Collaboration){
                
                if (!((Collaboration) ecoreOwner).getOwnedAttributes().contains(ecoreElt))
                    ((Collaboration) ecoreOwner).getOwnedAttributes().add(ecoreElt); 
                
                
            }else { 
                ecoreElt.destroy();
                throw new NotFoundException( "No possible mapping for " +
                        ecoreElt.getClass().getSimpleName() + " contained in " +
                        ecoreOwner.getClass().getSimpleName()); 
            }
        }else{
            ecoreElt.destroy();
        }
    }

    @objid ("103cba50-4583-11e0-b54c-00137279a832")
    private void attachPortToInstance(IInstance objingOwner, Slot ecoreElt) {
        InstanceSpecification ecoreOwner = (InstanceSpecification) this.genProp.getMappedElement(objingOwner);
        ecoreOwner.getSlots().add(ecoreElt);
    }

    @objid ("103cba54-4583-11e0-b54c-00137279a832")
    private void attachPortToBindableInstance(IBindableInstance objingOwner, Element ecoreElt) {
        INameSpace base = objingOwner.getBase();
                
        if ((base != null) && (ecoreElt instanceof Port)){
            attachPortToClassifier( base,(Port)ecoreElt);
        }else{
            IElement objOwner = ObjingModelNavigation.getBindableInstanceOwner(this.objingElement);
            if ( objOwner instanceof IInstance)
                attachPortToInstance((IInstance)objOwner,(Slot) ecoreElt);
            else{
                attachPortToClassifier( objOwner,(Port)ecoreElt);
            }
        }
    }

    @Override
    @objid ("103f1c9f-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof Port){           
            ((Port) ecoreElt).setIsBehavior(this.objingElement.isBehavior());
            ((Port) ecoreElt).setIsService(this.objingElement.isService());
        }else if (ecoreElt instanceof Slot){
            setDefiningFeature((Slot) ecoreElt);
            ObjingEAnnotation.setBehavior(ecoreElt, this.objingElement.isBehavior());
            ObjingEAnnotation.setService(ecoreElt, this.objingElement.isService());
        }
                
        if (this.genProp.isRoundtripEnabled()){
            ObjingEAnnotation.setPort(ecoreElt, true);
            setOwnerEAnnotation(ecoreElt,this.objingElement.getCompositionOwner());
            ObjingEAnnotation.setPortDirection(ecoreElt, this.objingElement.getDirection().toString());
        }
    }

    @objid ("103f1ca2-4583-11e0-b54c-00137279a832")
    private void setDefiningFeature(Slot slot) {
        IElement represented = this.objingElement.getRepresentedFeature();
        if (represented != null){
                
            Object temp = this.genProp.getMappedElement(represented);
            if (temp instanceof StructuralFeature){
                slot.setDefiningFeature((StructuralFeature) temp);
            }
        }
    }

    @objid ("103f1ca5-4583-11e0-b54c-00137279a832")
    private void setOwnerEAnnotation(Element element, IElement owner) {
        ObjingEAnnotation.setOwner(element, String.valueOf(owner.getIdentifier()));
    }

}
