/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.statik.IProvidedInterface;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("10607d90-4583-11e0-b54c-00137279a832")
public class OProvidedInterface extends OElement implements IOElement {
    @objid ("10607d91-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("10607d92-4583-11e0-b54c-00137279a832")
    private IProvidedInterface objingElement = null;


    @objid ("10607d93-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IPort owner = objingElement.getProviding();
        INameSpace base = owner.getBase();
        
        if ((base == null)  &&  (genProp.isRoundtripEnabled())) { 
            if (objingElement.getProvidedElement().size() == 0)
                return UMLFactory.eINSTANCE.createInterfaceRealization();
            else
                return UMLFactory.eINSTANCE.createDependency();
        }
        return null;
    }

    @objid ("1062dfe5-4583-11e0-b54c-00137279a832")
    public OProvidedInterface() {
    }

    @objid ("1062dfe7-4583-11e0-b54c-00137279a832")
    public OProvidedInterface(IProvidedInterface param) {
        super(param);
        objingElement = param;
    }

    @objid ("1065423e-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IPort objingPort = objingElement.getProviding();
        Element ecorePort = genProp.getMappedElement(objingPort);
        INameSpace base = objingPort.getBase();
        
        if (base == null){
        
            if (genProp.isRoundtripEnabled()){
        
                if (objingElement.getProvidedElement().size() == 0){
                    setNumberProvidedInterface(ecorePort);    
                    ecoreElt.destroy();
                }else{
                    if (ecorePort instanceof NamedElement){
                        Dependency dependency =  (Dependency) ecoreElt;
                        dependency.getClients().add((NamedElement)ecorePort);
        
                        for (IInterface inter : objingElement.getProvidedElement())
                            dependency.getSuppliers().add((NamedElement)genProp.getMappedElement(inter));
        
                        ObjingEAnnotation.setProvidedInterface(dependency, true);
        
                        Package pack = ecorePort.getNearestPackage();
                        Element temp = ecorePort.getOwner();
                        while((pack == null) && (temp != null)){
                            pack = temp.getNearestPackage();
                            temp = temp.getOwner();
                        }
        
                        if (pack == null)
                            pack =  genProp.getEcoreModel();
        
                        pack.getPackagedElements().add(dependency);
                    }else{
                        ecoreElt.destroy();
                    }
                }
            }
        
        }else{
        
            Element ecoreClient = (Element) genProp.getMappedElement(base);
        
            InterfaceRealization temp = (InterfaceRealization) ecoreElt;
        
            for (IInterface objingSupplier : objingElement.getProvidedElement()){
        
                if (temp == null)
                    temp = UMLFactory.eINSTANCE.createInterfaceRealization();
        
                if ( objingSupplier != null) {
        
                    Element ecoreSupplier = (Element) genProp
                    .getMappedElement(objingSupplier);
        
                    if ((ecoreClient != null) && ecoreSupplier != null 
                            && ecoreClient instanceof BehavioredClassifier 
                            && ecoreSupplier instanceof Interface) {
        
                        Boolean exist = false;
        
                        for (Object interRealization : ((BehavioredClassifier) ecoreClient).getInterfaceRealizations()){
                            if (((InterfaceRealization) interRealization).getContract().equals(ecoreSupplier))
                                exist = true;
                        }
        
                        if (!exist){
        
                            InterfaceRealization interRealiz = temp;
                            interRealiz.setImplementingClassifier((BehavioredClassifier)ecoreClient);
                            interRealiz.setContract((Interface) ecoreSupplier);
                        }
        
                    }
                    temp = null;
                }
            }
        }
    }

    @objid ("10654242-4583-11e0-b54c-00137279a832")
    private void setNumberProvidedInterface(Element ecoreElt) {
        int number = 0;
        for (IProvidedInterface provided : objingElement.getProviding().getProvided()){
            if (provided.getProvidedElement().size() == 0)
                number++;
        }
        
        if (number != 0)
            ObjingEAnnotation.setNumberProvidedInterface(
                    genProp.getMappedElement(objingElement.getProviding()), number);
    }

    @objid ("10654245-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
    }

}
