/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IRaisedException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Type;

@objid ("106c694c-4583-11e0-b54c-00137279a832")
public class ORaisedException extends OElement implements IOElement {
    @objid ("106c694d-4583-11e0-b54c-00137279a832")
    private IRaisedException objingElement;


    @objid ("106c694e-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        // RaisedException is a property directly defined on Operation in UML2.
        // No ecore element is
        // created. The exception is added to the related Operation:
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IOperation objingThrower = objingElement.getThrower();
        IClassifier objingThrownType = objingElement.getThrownType();
        Operation ecoreThrower = null;
                
        if (objingThrower != null && objingThrownType != null) {
            // Gets or creates the ecore thrower (the Operation):
            ecoreThrower = (Operation) genProp.getMappedElement(objingThrower);
                
            // Gets or creates the ecore thrown type (the Type):
            Type ecoreThrownType = (Type) genProp
                    .getMappedElement(objingThrownType);
                
            if (ecoreThrower != null && ecoreThrownType != null) {
                ecoreThrower.getRaisedExceptions().add(ecoreThrownType);
            }
        }
        // Returns the operation: if we return null, if Notes are defined on the
        // RaisedException element, the
        // visitor will produce an infinite loop. In this case, as we return the
        // operation, the Note will be
        // added on the operation (as no model element exists for
        // "RaisedException" in UML2).
        return ecoreThrower;
    }

    @objid ("106c6952-4583-11e0-b54c-00137279a832")
    public ORaisedException() {
    }

    @objid ("106c6954-4583-11e0-b54c-00137279a832")
    public ORaisedException(IRaisedException element) {
        super(element);
        objingElement = element;
    }

    @objid ("106ecbae-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // Nothing to do: RaisedException is a property directly defined on
        // Operation in UML2.
    }

    @objid ("106ecbb2-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        // Nothing to do: RaisedException is a property directly defined on
        // Operation in UML2.
    }

}
