/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.statik.IRequiredInterface;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.Usage;

@objid ("1075f2b4-4583-11e0-b54c-00137279a832")
public class ORequiredInterface extends OElement implements IOElement {
    @objid ("1075f2b6-4583-11e0-b54c-00137279a832")
    private IRequiredInterface objingElement = null;


    @objid ("1075f2b7-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IPort owner = this.objingElement.getRequiring();
        Element ecoreOwner =  GenerationProperties.getInstance().getMappedElement(owner);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            if (this.objingElement.getRequiredElement().size() == 0){
                ObjingEAnnotation.setNumberRequiredInterface(ecoreOwner,
                        ObjingEAnnotation.getNumberRequiredInterface(ecoreOwner) + 1 ) ;
            }else{
                Dependency dep = UMLFactory.eINSTANCE.createDependency();
                ObjingEAnnotation.setRequiredInterface(dep, true);
                return dep;
            }
        }
        return null;
    }

    @objid ("1075f2bb-4583-11e0-b54c-00137279a832")
    public ORequiredInterface() {
    }

    @objid ("1075f2bd-4583-11e0-b54c-00137279a832")
    public ORequiredInterface(IRequiredInterface param) {
        super(param);
        this.objingElement = param;
    }

    @objid ("107ab76c-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IPort objPort = this.objingElement.getRequiring();
        IModelElement objingClient = objPort.getBase();
        Element ecorePort = GenerationProperties.getInstance().getMappedElement(objPort);
        
        if (objingClient != null) {
            if (this.objingElement.getRequiredElement().size() >0){
                for (IModelElement objingSupplier : this.objingElement.getRequiredElement()){
        
                    Element ecoreClient = GenerationProperties.getInstance()
                    .getMappedElement(objingClient);
        
                    Element ecoreSupplier = GenerationProperties.getInstance()
                    .getMappedElement(objingSupplier);
        
                    if (ecoreClient != null && ecoreSupplier != null) {
        
                        Boolean exist = false;
        
                        for (Object usage : ((NamedElement)ecoreClient).getClientDependencies()){
                            if (usage instanceof Usage)
                                if (((Usage) usage).getSuppliers().contains(ecoreSupplier))
                                    exist = true;
                        }        
        
                        if (!exist){
        
                            Usage ecoreUsage = (Usage) ecoreElt;
        
                            ecoreUsage.getClients().add((NamedElement)ecoreClient);
                            ecoreUsage.getSuppliers().add((NamedElement)ecoreSupplier);
        
                            Package ecorePkg = ecoreClient.getNearestPackage();
                            if (ecorePkg == null) {
                                GenerationProperties.getInstance().getEcoreModel().getPackagedElements().add(
                                        ecoreUsage);
                            } else {
                                ecorePkg.getPackagedElements().add(ecoreUsage);
                            }
                        }else
                            ecoreElt.destroy();
                    }else
                        ecoreElt.destroy();
        
                }
            }else
                ecoreElt.destroy();
        }else{
            if (GenerationProperties.getInstance().isRoundtripEnabled()) {
        
                if (this.objingElement.getRequiredElement().size() == 0){
                    setNumberRequiredInterface(ecorePort);    
                    ecoreElt.destroy();
                }else{
        
                    Dependency dependency =  (Dependency) ecoreElt;
        
                    Element port = GenerationProperties.getInstance().getMappedElement(objPort);
                    Package pack = ecorePort.getNearestPackage();
        
                    if ((port instanceof NamedElement) && (pack!= null)){
        
                        dependency.getSuppliers().add((NamedElement) port);
        
                        for (IInterface inter : this.objingElement.getRequiredElement())
                            dependency.getClients().add((NamedElement)GenerationProperties.getInstance().getMappedElement(inter));
        
                        ObjingEAnnotation.setRequiredInterface(dependency, true);
        
                        pack.getPackagedElements().add(dependency);
                    }else{
                        GenerationProperties.getInstance().getEcoreModel().getPackagedElements().add(
                                dependency);
                    }
                }
            }
        
        }
    }

    @objid ("107ab770-4583-11e0-b54c-00137279a832")
    private void setNumberRequiredInterface(Element ecoreElt) {
        int number = ObjingEAnnotation.getNumberRequiredInterface(ecoreElt);
        
        for (IRequiredInterface required : this.objingElement.getRequiring().getRequired()){
            if (required.getRequiredElement().size() == 0)
                number++;
        }
        
        if (number != 0)
            ObjingEAnnotation.setNumberRequiredInterface(
                    GenerationProperties.getInstance().getMappedElement(this.objingElement.getRequiring()), number);
    }

    @objid ("107ab773-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
    }

}
