/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of ISignal elements
 * @author ebrosse
 */
@objid ("1099b600-4583-11e0-b54c-00137279a832")
public class OSignal extends ONameSpace {
    @objid ("1099b601-4583-11e0-b54c-00137279a832")
    private ISignal objingElement;


    @Override
    @objid ("1099b603-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createSignal();
    }

    /**
     * The default constructor
     */
    @objid ("109c1854-4583-11e0-b54c-00137279a832")
    public OSignal() {
    }

    /**
     * @param param : the exported ISignal
     */
    @objid ("109c1856-4583-11e0-b54c-00137279a832")
    public OSignal(final ISignal param) {
        super(param);
        this.objingElement = param;
    }

    @Override
    @objid ("109e7aae-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor theOwnerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        IElement objingOwner = theOwnerVisitor.getCompositionOwner(this.objingElement);
        Element ecoreOwner = genProp.getMappedElement(objingOwner);
        
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add((Signal)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getNestedClassifiers().add((Classifier)ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @Override
    @objid ("109e7ab2-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        
        setBaseEAnnotation((Signal) ecoreElt);
        setPBaseEAnnotation((Signal) ecoreElt);
        setOBaseEAnnotation((Signal) ecoreElt);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setIsEventEAnnotation((Signal) ecoreElt);
            setIsExceptionEAnnotation((Signal) ecoreElt);
        }
    }

    @objid ("109e7ab5-4583-11e0-b54c-00137279a832")
    private void setBase(IModelElement base, final Signal ecoreElt) {
        Element ecoreBase = GenerationProperties.getInstance().getMappedElement(base);
        
        if  ((ecoreBase != null)
                && (ecoreBase instanceof Classifier)){
            ecoreElt.getRedefinedClassifiers().add((Classifier) ecoreBase);
        
        }else{
            XMILogs xmilogs = XMILogs.getInstance();
        
            String message = XMI.I18N.getMessage("logFile.warning.unsupportedRelationBetweenTypeExported",
                                                "Base", this.objingElement.getName(), this.objingElement.getClass().getSimpleName(),
                                                base.getName(), base.getClass().getSimpleName());
        
            xmilogs.writelnInLog(message);
            GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
        }
    }

    @objid ("109e7abc-4583-11e0-b54c-00137279a832")
    private void setPBaseEAnnotation(final Signal ecoreElt) {
        IModelElement temp = this.objingElement.getPBase();
        if (temp != null)
            setBase(temp, ecoreElt);
    }

    @objid ("109e7abf-4583-11e0-b54c-00137279a832")
    private void setOBaseEAnnotation(final Signal ecoreElt) {
        IModelElement temp = this.objingElement.getOBase();
        if (temp != null)
            setBase(temp, ecoreElt);
    }

    @objid ("10a0dd09-4583-11e0-b54c-00137279a832")
    private void setBaseEAnnotation(final Signal ecoreElt) {
        IModelElement temp = this.objingElement.getBase();
        if (temp != null)
            setBase(temp, ecoreElt);
    }

    @objid ("10a0dd0c-4583-11e0-b54c-00137279a832")
    private void setIsEventEAnnotation(Signal signal) {
        ObjingEAnnotation.setEvent(signal, this.objingElement.isEvent());
    }

    @objid ("10a0dd0f-4583-11e0-b54c-00137279a832")
    private void setIsExceptionEAnnotation(Signal signal) {
        ObjingEAnnotation.setException(signal, this.objingElement.isEvent());
    }

}
