/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioredClassifier;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of IStateMachine
 * @author ebrosse
 */
@objid ("10acc8ca-4583-11e0-b54c-00137279a832")
public class OStateMachine extends OModelElement {
    @objid ("10acc8cb-4583-11e0-b54c-00137279a832")
     boolean isProtocol = false;

    @objid ("10acc8cc-4583-11e0-b54c-00137279a832")
    private IStateMachine objingElement = null;

    @objid ("10acc8ce-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();


    @Override
    @objid ("10acc8cf-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (this.isProtocol)
            return UMLFactory.eINSTANCE.createProtocolStateMachine();
        return UMLFactory.eINSTANCE.createStateMachine();
    }

    /**
     * Constructor
     * @param element : the exported Modelio IStateMachine
     */
    @objid ("10af2b24-4583-11e0-b54c-00137279a832")
    public OStateMachine(final IStateMachine element) {
        super(element);
        this.objingElement = element;
        if (ObjingModelNavigation.isProtocolStateMachine(element))
            this.isProtocol = true;
        else
            this.isProtocol = false;
    }

    @Override
    @objid ("10af2b2d-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        IModelElement objingOwner = this.objingElement.getOwner();
        if (objingOwner == null) {
            IOperation op = this.objingElement.getOwnerOperation();
            objingOwner = op.getOwner();
            String message =  XMI.I18N.getMessage("logFile.warning.moving.stateMachine",
                                                  this.objingElement.getName(), op.getName(), objingOwner
                                                  .getName());
            XMILogs.getInstance().writelnInLog(message);
            GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
        }
        Element ecoreOwner = this.genProp.getMappedElement(objingOwner);
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                Package ownerIsPkg = (Package) ecoreOwner;
                ownerIsPkg.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                Component ownerIsCmpnt = (Component) ecoreOwner;
                ownerIsCmpnt.getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                Class ownerIsClass = (Class) ecoreOwner;
                ownerIsClass.getOwnedBehaviors().add((Behavior)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                Interface ownerIsItf = (Interface) ecoreOwner;
                ownerIsItf.getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof BehavioredClassifier) {
                BehavioredClassifier ownerIsBehavioredClassifier = (BehavioredClassifier) ecoreOwner;
                ownerIsBehavioredClassifier.getOwnedBehaviors().add((Behavior)ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @Override
    @objid ("10af2b31-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        if (this.genProp.isRoundtripEnabled())
            setReentrant((StateMachine) ecoreElt);
    }

    @objid ("10b18d7e-4583-11e0-b54c-00137279a832")
    private void setReentrant(StateMachine ecoreElt) {
        ObjingEAnnotation.setReentrant(ecoreElt, this.objingElement.isReentrant());
    }

}
