/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateBinding;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.TemplateBinding;
import org.eclipse.uml2.uml.TemplateSignature;
import org.eclipse.uml2.uml.TemplateableElement;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of ITemplateBinding
 * @author ebrosse
 */
@objid ("10e13c86-4583-11e0-b54c-00137279a832")
public class OTemplateBinding extends OElement implements IOElement {

    @Override
    @objid ("10e13c87-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createTemplateBinding();
    }

    /**
     * constructor
     * @param param : the exported Modelio ITemplateBinding
     */
    @objid ("10e39ee0-4583-11e0-b54c-00137279a832")
    public OTemplateBinding(final ITemplateBinding param) {
        super(param);
    }

    @Override
    @objid ("10e39ee9-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        ITemplateBinding templateBinding = (ITemplateBinding) getObjingElement();
        IElement objOwner =  templateBinding.getBoundElement();
        if (objOwner == null)
            objOwner =  templateBinding.getBoundOperation();
        
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objOwner);
        
        if (ecoreOwner instanceof TemplateableElement){
        
            ((TemplateableElement) ecoreOwner).getTemplateBindings().add((TemplateBinding) ecoreElt);
        
        }else{
            ecoreElt.destroy();
        }
    }

    @Override
    @objid ("10e39eed-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setTemplate(ecoreElt);
    }

    @objid ("10e6013a-4583-11e0-b54c-00137279a832")
    private void setTemplate(Element ecoreElt) {
        ITemplateBinding objElt = (ITemplateBinding) getObjingElement();
        TemplateBinding ecoreElement = (TemplateBinding) ecoreElt;
        
        IElement  template = objElt.getInstanciatedTemplate();
        if (template == null)
            template = objElt.getInstanciatedTemplateOperation();
        
        if (template != null){
            Element ecoreTemplate = GenerationProperties.getInstance().getMappedElement(template);
        
            if (ecoreTemplate instanceof TemplateableElement){
                TemplateSignature signature = ((TemplateableElement) ecoreTemplate).getOwnedTemplateSignature();
        
                if (signature == null){
                    signature = ((TemplateableElement) ecoreTemplate).createOwnedTemplateSignature();
                    ObjingEAnnotation.setDeleted(signature, true);
                }
                ecoreElement.setSignature(signature);
                
                try{
                    signature.setTemplate((TemplateableElement) ecoreTemplate);
                }catch(IllegalArgumentException e){
                    signature.destroy();
                    ecoreTemplate.destroy();
                    XMILogs xmiLogs = XMILogs.getInstance();
                    String message = XMI.I18N.getMessage("logFile.warning.export.unsupportedRelation.wrongEcoreType",
                                                        "instanciedTemplate", objElt.getName(), " ITemplateBinding",  "TemplateableElement");
                    xmiLogs.writelnInLog(message);
                    GenerationProperties.getInstance().getReportModel().addWarning(message, objElt);
                }
               
            }
        }
    }

}
