/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateParameter;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.ParameterableElement;
import org.eclipse.uml2.uml.RedefinableTemplateSignature;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.TemplateSignature;
import org.eclipse.uml2.uml.TemplateableElement;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of ITemplateParameter
 * @author ebrosse
 */
@objid ("10e6013d-4583-11e0-b54c-00137279a832")
public class OTemplateParameter extends ONameSpace {
    @objid ("10e6013e-4583-11e0-b54c-00137279a832")
    private TemplateSignature signature = null;


    @Override
    @objid ("10e6013f-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (((IModelElement) getObjingElement()).isStereotyped(XMIStereotypes.UML2CONNECTABLEELEMENTTEMPLATEPARAMETER)){
            return UMLFactory.eINSTANCE.createConnectableElementTemplateParameter();
        }else  if (((IModelElement) getObjingElement()).isStereotyped(XMIStereotypes.UML2CLASSIFIERTEMPLATEPARAMETER)){
            return UMLFactory.eINSTANCE.createClassifierTemplateParameter();
        }else{
            IElement objOwner =  ((ITemplateParameter)getObjingElement()).getParameterized();
            if (objOwner == null)
                objOwner =  ((ITemplateParameter)getObjingElement()).getParameterizedOperation();
        
            Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objOwner);
            if (ecoreOwner instanceof TemplateableElement){
        
                this.signature = ((TemplateableElement) ecoreOwner).getOwnedTemplateSignature();
                try{
                    if (this.signature == null){
                        this.signature = ((TemplateableElement) ecoreOwner).createOwnedTemplateSignature();
                        ObjingEAnnotation.setDeleted(this.signature, true);
                    }
                    return this.signature.createOwnedParameter();
        
                }catch(IllegalArgumentException e){
                    this.signature.destroy();
                }
        
            }
            return null;
        }
    }

    /**
     * Constructor
     * @param element : the exported Modelio ITemplateParameter
     */
    @objid ("10e86394-4583-11e0-b54c-00137279a832")
    public OTemplateParameter(final ITemplateParameter element) {
        super(element);
    }

    @Override
    @objid ("10e8639d-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // (voir OElementImport pour l'exemple)
        
        
        IElement objOwner =  ((ITemplateParameter)getObjingElement()).getParameterized();
        
        if (objOwner == null)
            objOwner =  ((ITemplateParameter)getObjingElement()).getParameterizedOperation();
        
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objOwner);
        
        if (ecoreOwner instanceof TemplateableElement){
        
            TemplateSignature signature = ((TemplateableElement) ecoreOwner).getOwnedTemplateSignature();
        
            try{
                if (signature == null){
                    signature = UMLFactory.eINSTANCE.createTemplateSignature();
                    signature.getOwnedElements();
                    signature.getOwnedParameters().add((TemplateParameter) ecoreElt);
                    ((TemplateableElement) ecoreOwner).setOwnedTemplateSignature(signature);
        
                }
        
            }catch(IllegalArgumentException e){     
                ecoreElt.destroy();
                signature.destroy();
            }
        
        }else if ((ecoreOwner instanceof RedefinableTemplateSignature) && (ecoreElt instanceof TemplateParameter)){
            ((RedefinableTemplateSignature) ecoreOwner).getOwnedParameters().add((TemplateParameter)ecoreElt);
        }else{
            ecoreElt.destroy();
        }
    }

    @Override
    @objid ("10e863a1-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setTemplateType((TemplateParameter) ecoreElt);
        setIsValueParameter((TemplateParameter) ecoreElt);
        setDefaultType((TemplateParameter) ecoreElt);
    }

    @objid ("10eac5f1-4583-11e0-b54c-00137279a832")
    private void setTemplateType(TemplateParameter ecoreElt) {
        Element ecoreType = getEcoreType(((ITemplateParameter)getObjingElement()).getType());
        if (ecoreType != null)
            setParameteredElement(ecoreElt, ecoreType);
    }

    @objid ("10eac5f4-4583-11e0-b54c-00137279a832")
    private Element getEcoreType(IModelElement type) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        // Getting type of the Template:
        IModelElement objingType = type;
        Element ecoreType = null;
        if (objingType != null) {
            ecoreType = genProp.getMappedElement(objingType);
        }
        return ecoreType;
    }

    @objid ("10eac5f9-4583-11e0-b54c-00137279a832")
    private void setParameteredElement(TemplateParameter ecoreTemplateParam, Element ecoreElement) {
        if (ecoreElement instanceof ParameterableElement){
            try{
                ParameterableElement parameterableElement = (ParameterableElement) ecoreElement;
                ecoreTemplateParam.setParameteredElement(parameterableElement);
            }catch(IllegalArgumentException e){
                XMILogs xmiLogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.export.unsupportedRelation.wrongEcoreType", 
                        "Type", "ParameterableElement");
                xmiLogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, getObjingElement());
            }
        }
    }

    @objid ("10eac5fd-4583-11e0-b54c-00137279a832")
    private void setIsValueParameter(TemplateParameter ecoreElt) {
        ObjingEAnnotation.setValueParameter(ecoreElt, ((ITemplateParameter) getObjingElement()).isValueParameter());
    }

    @objid ("10ed2848-4583-11e0-b54c-00137279a832")
    private void setDefaultType(TemplateParameter ecoreElt) {
        ITemplateParameter templateParam = ((ITemplateParameter)getObjingElement());
        IModelElement type = templateParam.getDefaultType();
        if (type != null){
            Element ecoreType = getEcoreType(type);
            setDefault(ecoreElt, ecoreType);
        }else{
            LiteralString literalString = UMLFactory.eINSTANCE.createLiteralString();
            literalString.setValue(templateParam.getDefaultValue());
            setDefault(ecoreElt, literalString);
        }
    }

    @objid ("10ed284b-4583-11e0-b54c-00137279a832")
    private void setDefault(TemplateParameter ecoreTemplateParam, Element ecoreElement) {
        if (ecoreElement instanceof ParameterableElement){
            ParameterableElement parameterableElement = (ParameterableElement) ecoreElement;
            ecoreTemplateParam.setOwnedDefault(parameterableElement);
        }
    }

}
