/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.ITemplateParameterSubstitution;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ParameterableElement;
import org.eclipse.uml2.uml.TemplateBinding;
import org.eclipse.uml2.uml.TemplateParameter;
import org.eclipse.uml2.uml.TemplateParameterSubstitution;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of Modelio ITemplateParameterSubstitution
 * @author ebrosse
 */
@objid ("10ed2852-4583-11e0-b54c-00137279a832")
public class OTemplateParameterSubstitution extends OElement implements IOElement {

    @Override
    @objid ("10ef8aa2-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(getObjingElement().getCompositionOwner());
        
        if (ecoreOwner instanceof TemplateBinding)   
            return UMLFactory.eINSTANCE.createTemplateParameterSubstitution();
        else 
            return null;
    }

    /**
     * Constructor
     * @param param : the exported Modelio ITemplateParameterSubstitution
     */
    @objid ("10ef8aa8-4583-11e0-b54c-00137279a832")
    public OTemplateParameterSubstitution(final ITemplateParameterSubstitution param) {
        super(param);
    }

    @Override
    @objid ("10f1ecfe-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(getObjingElement().getCompositionOwner());
        
        if (ecoreOwner instanceof TemplateBinding){              
            ((TemplateBinding) ecoreOwner).getParameterSubstitutions().add((TemplateParameterSubstitution) ecoreElt);               
        }
    }

    @Override
    @objid ("10f1ed02-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setValue(ecoreElt);
        setFormal(ecoreElt);
        setActual(ecoreElt);
    }

    @objid ("10f1ed05-4583-11e0-b54c-00137279a832")
    private void setFormal(Element ecoreElt) {
        IElement formal = ((ITemplateParameterSubstitution) getObjingElement()).getFormalParameter();
        if (formal != null){
        
            Element ecoreFormal = GenerationProperties.getInstance().getMappedElement(formal);
            if ((ecoreFormal != null) && (ecoreFormal instanceof TemplateParameter)){
                ((TemplateParameterSubstitution) ecoreElt)
                .setFormal((TemplateParameter) ecoreFormal);
            }else{
                XMILogs xmiLogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.export.unsupportedRelation.wrongEcoreType"
                                                    , "FormalParameter", "" ,"TemplateParameterSubtitution", "TemplateParameter");
                xmiLogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, getObjingElement());
        
            }
        }
    }

    @objid ("10f1ed08-4583-11e0-b54c-00137279a832")
    private void setValue(Element ecoreElt) {
        ObjingEAnnotation.setValue(ecoreElt, ((ITemplateParameterSubstitution) getObjingElement()).getValue());
    }

    @objid ("10f1ed0b-4583-11e0-b54c-00137279a832")
    private void setActual(Element ecoreElt) {
        IElement actual = ((ITemplateParameterSubstitution) getObjingElement()).getActual();
        
        if (actual != null){
            Element ecoreFormal = GenerationProperties.getInstance().getMappedElement(actual);
            if ((ecoreFormal != null) && (ecoreFormal instanceof ParameterableElement)){
                ((TemplateParameterSubstitution) ecoreElt).setActual((ParameterableElement) ecoreFormal);
        
            }else{
                XMILogs xmiLogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.export.unsupportedRelation.wrongEcoreType",
                                                    "Actual", "" ,"TemplateParameterSubtitution" , "ParameterableElement");
                xmiLogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, getObjingElement());
        
            }
        }
    }

}
