/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IConnectionPointReference;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEntryPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IEvent;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IExitPointPseudoState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IInternalTransition;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateVertex;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.BehavioralFeature;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.ProtocolTransition;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Trigger;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.Vertex;

/**
 * This class handles the export of ITransition.
 * @author ebrosse
 */
@objid ("10fdd8c7-4583-11e0-b54c-00137279a832")
public class OTransition extends OModelElement {
    @objid ("11003b18-4583-11e0-b54c-00137279a832")
    private boolean isProtocolTransition;

    @objid ("11003b19-4583-11e0-b54c-00137279a832")
    private ITransition objingElement;

    @objid ("11003b1a-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("11003b1b-4583-11e0-b54c-00137279a832")
    protected CompositionOwnerVisitor ownerVisitor;


    @Override
    @objid ("11003b1c-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (!( this.objingElement instanceof IInternalTransition)){
            if (this.isProtocolTransition)
                return UMLFactory.eINSTANCE.createProtocolTransition();
            return UMLFactory.eINSTANCE.createTransition();
        }
        return null;
    }

    /**
     * Constructor with the exported ITransition as parameter
     * @param param : the exported ITransition
     */
    @objid ("11003b22-4583-11e0-b54c-00137279a832")
    public OTransition(ITransition param) {
        super(param);
        this.objingElement = param;
        this. isProtocolTransition = ObjingModelNavigation
                .isProtocolTransition(param);
    }

    @Override
    @objid ("11029d76-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor theOwnerVisitor) {
        this.ownerVisitor = theOwnerVisitor;
        IElement objingOwner = this.objingElement.getSource();
        
        if (objingOwner != null) {
            Element ecoreOwner = this.genProp.getMappedElement(objingOwner);
            Region ownerRegion = null;
            if (ecoreOwner instanceof Vertex) {
                ownerRegion = ((Vertex) ecoreOwner).getContainer();
                
                if (ownerRegion == null){
                    IRegion objRegion = null;
                    if (objingOwner instanceof IConnectionPointReference){
                        objRegion = ((IConnectionPointReference) objingOwner).getOwnerState().getParent();
                      
                    }else if (objingOwner instanceof IEntryPointPseudoState){
                        IEntryPointPseudoState entry = (IEntryPointPseudoState) objingOwner;
                        IStateMachine sm = entry.getEntryOfMachine();
                        
                        if (sm != null)
                            objRegion= sm.getTop();
                        else{
                            objRegion= entry.getEntryOf().getParent();
                        }
                       
                    } else if (objingOwner instanceof IExitPointPseudoState){
                    IExitPointPseudoState exit = (IExitPointPseudoState) objingOwner;
                    IStateMachine sm = exit.getExitOfMachine();
                    
                    if (sm != null)
                        objRegion= sm.getTop();
                    else{
                        objRegion= exit.getExitOf().getParent();
                    }
                   
                }
                    
                    ownerRegion= (Region) this.genProp.getMappedElement(objRegion);
                }
            } else if (ecoreOwner instanceof Region) {
                ownerRegion = ((Region) ecoreOwner);
            }
            
            if (ownerRegion != null){
            ownerRegion.getTransitions().add((Transition)ecoreElt);
            }
            
        }
    }

    @Override
    @objid ("11029d7a-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        if (this.objingElement instanceof IInternalTransition){
            setOwner((Transition) ecoreElt);
        
        }else{
            setSource((Transition) ecoreElt);
            setTarget((Transition) ecoreElt);
        
        }
        setEffect((Transition)ecoreElt);
        
        setTrigger((Transition) ecoreElt);
        setGuard((Transition) ecoreElt);
        setPostCondition((Transition) ecoreElt);
        setEffects((Transition)ecoreElt);   
        setProcessed((Transition) ecoreElt);
        
        if (ecoreElt instanceof ProtocolTransition){
            setReferred((Transition) ecoreElt);
        }
        
        if (this.genProp.isRoundtripEnabled()){
            setReceivedEvent(ecoreElt);
            setSentEvents(ecoreElt);
        
        }
    }

    @objid ("11029d7d-4583-11e0-b54c-00137279a832")
    private void setGuard(Transition ecoreElt) {
        String objGuard = this.objingElement.getGuard();
        if ((objGuard != null) && (!objGuard.equals("")) && (ecoreElt.getGuard() == null)){
            Constraint guard = UMLFactory.eINSTANCE.createConstraint();
            guard.setName(objGuard);
            ecoreElt.setGuard(guard);
            guard.getConstrainedElements().add(ecoreElt);
            LiteralString valueSpecification = UMLFactory.eINSTANCE.createLiteralString();
            valueSpecification.setValue(objGuard);
            guard.setSpecification(valueSpecification);
        
            if (this.isProtocolTransition){
                ((ProtocolTransition) ecoreElt).setPreCondition(guard);
            }
        }
    }

    @objid ("1104ffcc-4583-11e0-b54c-00137279a832")
    private void setEffect(Transition ecoreElt) {
        String effect = this.objingElement.getEffect();
        if ((effect != null) && (!effect.equals(""))){
            // String Case
            OpaqueBehavior opaqueBehavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
            opaqueBehavior.getBodies().add(effect);
            ecoreElt.setEffect(opaqueBehavior);
        }else{
        
            //Behavior Case
        
            IBehavior behavior = this.objingElement.getBehaviorEffect();
            if (behavior != null){
                Element ecoreBehavior = this.genProp.getMappedElement(behavior);
        
                if ((ecoreBehavior != null) && (ecoreBehavior instanceof Behavior)){
        
                    OpaqueBehavior opaqueBehavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
                    ecoreElt.setEffect(opaqueBehavior);
                    opaqueBehavior.getRedefinedBehaviors().add((Behavior)ecoreBehavior);
                }
        
            }
        }
    }

    @objid ("1104ffcf-4583-11e0-b54c-00137279a832")
    private void setSentEvents(Element ecoreElt) {
        String signalString = this.objingElement.getSentEvents();
        if (signalString != null)
            ObjingEAnnotation.setSignal(ecoreElt, signalString);
    }

    @objid ("1104ffd2-4583-11e0-b54c-00137279a832")
    private void setSource(Transition transition) {
        IStateVertex objingSource = this.objingElement.getSource();
        if (objingSource != null) {
            Element ecoreSource = this.genProp.getMappedElement(objingSource);
            if (ecoreSource instanceof Vertex) {
                transition.setSource((Vertex) ecoreSource);
            }
        
        }
    }

    @objid ("1104ffd5-4583-11e0-b54c-00137279a832")
    private void setTarget(Transition transition) {
        IStateVertex objingTarget = this.objingElement.getTarget();
        if (objingTarget != null) {
            Element ecoreTarget = this.genProp.getMappedElement(objingTarget);
            if (ecoreTarget instanceof Vertex) {
                transition.setTarget((Vertex) ecoreTarget);
            }
        }
    }

    @objid ("1104ffd8-4583-11e0-b54c-00137279a832")
    private void setEffects(Transition transition) {
        ISignal signal = this.objingElement.getEffects();
        if (signal != null) {
            Behavior behavior = transition.getEffect();
            if (behavior == null){
                behavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
                behavior.setName(signal.getName());
                transition.setEffect(behavior);
            }
        
            Signal ecoreSignal = (Signal) this.genProp.getMappedElement(signal);
            if (ecoreSignal != null) {
                IRegion ownerRegion = this.objingElement.getTarget().getParent();
        
                IStateMachine objingSM =  ObjingModelNavigation
                        .getOwnerStateMachine(ownerRegion);
        
                if (objingSM != null) {
                    IPackage smOwner = ObjingModelNavigation
                            .getNearestPackage(objingSM);
        
                    if (smOwner != null) {
                        Element ecorePkg = this.genProp.getMappedElement(smOwner);
        
                        if (ecorePkg instanceof Package) {
                            Dependency dependency = UMLFactory.eINSTANCE.createDependency();
                            dependency.getClients().add(behavior);
                            dependency.getSuppliers().add(ecoreSignal);
                            ((Package) ecorePkg).getPackagedElements().add(dependency);
                        }
                    }
                }
            }
        }
    }

    @objid ("1104ffdb-4583-11e0-b54c-00137279a832")
    private void setTrigger(Transition transition) {
        IEvent event = this.objingElement.getTrigger();
        if (event != null) {
            setEventTrigger(transition, event);
        }
    }

    @objid ("11076231-4583-11e0-b54c-00137279a832")
    private void setReferred(Transition transition) {
        if (transition instanceof ProtocolTransition){
            IEvent event = this.objingElement.getTrigger();
            if (event != null){
                Object temp = this.genProp.getMappedElement(event);
                if (temp instanceof Event){
                    Trigger trigger = UMLFactory.eINSTANCE.createTrigger();
                    trigger.setEvent((Event)temp);
                    transition.getTriggers().add(trigger);
                }
            }
        }
    }

    @objid ("6620e18a-1a76-11e1-8103-0027103f347d")
    private void setPostCondition(final Transition ecoreElt) {
        String objPost = this.objingElement.getPostCondition();
        if ((objPost != null) && (!objPost.equals(""))){
            if ((ecoreElt instanceof ProtocolTransition) && (this.objingElement.getProcessed() != null)){
                Constraint post = UMLFactory.eINSTANCE.createConstraint();
                post.setName(objPost);
                ((ProtocolTransition)ecoreElt).setPostCondition(post);
                post.getConstrainedElements().add(ecoreElt);
                LiteralString valueSpecification = UMLFactory.eINSTANCE.createLiteralString();
                valueSpecification.setValue(objPost);
                post.setSpecification(valueSpecification);
            }else{
                if (this.genProp.isRoundtripEnabled())
                    ObjingEAnnotation.setPostCondition(ecoreElt, objPost);
            }
        }
    }

    @objid ("662156c0-1a76-11e1-8103-0027103f347d")
    private void setReceivedEvent(final Element ecoreElt) {
        String receivedEvent = this.objingElement.getReceivedEvents();
        if (receivedEvent != null)
            ObjingEAnnotation.setReceivedEvent(ecoreElt, receivedEvent);
    }

    @objid ("6621a4e4-1a76-11e1-8103-0027103f347d")
    private void setEventTrigger(final Transition transition, final IEvent objingEvent) {
        StateMachine sm = EcoreModelNavigation.getMostEnclosingStateMachine(transition);
        
        Event ecorEvent =  (Event) this.genProp.getMappedElement(objingEvent);
        Trigger trigger = null;
        boolean existTrigger = false; 
        for (Trigger ownedTrigger : sm.getOwnedTriggers()){
            if (ownedTrigger.getEvent().equals(ecorEvent)){
                trigger = ownedTrigger;
                existTrigger = true;
            }
        }
        
        if (existTrigger){
            transition.getTriggers().add(trigger);
        }else{
        
            trigger = UMLFactory.eINSTANCE.createTrigger();
            trigger.setName(objingEvent.getName());
            transition.getTriggers().add(trigger);
        
            trigger.setEvent(ecorEvent);
        
        }
    }

    @objid ("66221a1a-1a76-11e1-8103-0027103f347d")
    private void setProcessed(final Transition transition) {
        IOperation objingOperation = this.objingElement.getProcessed();
        if (objingOperation != null) {
        
            Object effect = this.genProp.getMappedElement(objingOperation);
            if (effect instanceof BehavioralFeature){
                Behavior behavior = transition.getEffect();
                if (behavior == null){
                    behavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
                    transition.setEffect(behavior);
                }
        
                behavior.setSpecification((BehavioralFeature) effect);
        
            }
        }
    }

    @objid ("6622683e-1a76-11e1-8103-0027103f347d")
    private void setOwner(final Transition ecoreElt) {
        IStateVertex objingSource = ((IInternalTransition) this.objingElement).getSComposed();
        
        Element ecoreSource = this.genProp.getMappedElement(objingSource);
        if (ecoreSource instanceof Vertex) {
            ecoreElt.setSource((Vertex) ecoreSource);
            ecoreElt.setTarget((Vertex) ecoreSource);
        }
    }

}
