/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;

@objid ("1110eb8e-4583-11e0-b54c-00137279a832")
public class OUseCase extends ONameSpace implements IOElement {
    @objid ("1110eb8f-4583-11e0-b54c-00137279a832")
    private IUseCase objingElement;


    @objid ("1110eb90-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createUseCase();
    }

    @objid ("1110eb94-4583-11e0-b54c-00137279a832")
    public OUseCase() {
    }

    @objid ("1110eb96-4583-11e0-b54c-00137279a832")
    public OUseCase(IUseCase param) {
        super(param);
        objingElement = param;
    }

    @objid ("11134dee-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IModelTree objingOwner = objingElement.getOwner();
        Element ecoreOwner = (Element) genProp.getMappedElement(objingOwner);
                
        if (ecoreOwner != null) {
            if (ecoreOwner instanceof Package) {
                ((Package) ecoreOwner).getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Component) {
                ((Component) ecoreOwner).getPackagedElements().add((PackageableElement)ecoreElt);
            } else if (ecoreOwner instanceof Class) {
                ((Class) ecoreOwner).getNestedClassifiers().add((Classifier)ecoreElt);
            } else if (ecoreOwner instanceof Interface) {
                ((Interface) ecoreOwner).getNestedClassifiers().add((Classifier)ecoreElt);
            } else {
                ecoreElt.destroy();
                throw new NotFoundException("Owner Class ("
                        + ecoreOwner.getClass().getSimpleName() + ") Not Found");
            }
        }
    }

    @objid ("11134df2-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setSubject((UseCase) ecoreElt);
    }

    @objid ("1115b042-4583-11e0-b54c-00137279a832")
    private void setSubject(UseCase ecoreElt) {
        Element ecoreOwner = GenerationProperties.getInstance().getMappedElement(objingElement.getOwner());
        if (ecoreOwner instanceof Classifier)
            ecoreElt.getSubjects().add((Classifier)  ecoreOwner);
    }

}
