/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing;

import com.modeliosoft.modelio.api.model.uml.usecase.IExtensionPoint;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCaseDependency;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.NotFoundException;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Extend;
import org.eclipse.uml2.uml.ExtensionPoint;
import org.eclipse.uml2.uml.Include;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UseCase;

/**
 * This class manages the export of Modelio IUseCaseDependency
 * @author ebrosse
 */
@objid ("1115b045-4583-11e0-b54c-00137279a832")
public class OUseCaseDependency extends OModelElement {
    @objid ("1115b046-4583-11e0-b54c-00137279a832")
    private boolean isExtend = false;

    @objid ("1115b047-4583-11e0-b54c-00137279a832")
    private boolean isInclude = false;

    @objid ("1115b048-4583-11e0-b54c-00137279a832")
    private IUseCaseDependency objingElement;


    @Override
    @objid ("1115b049-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        if (this.isExtend) {
           return UMLFactory.eINSTANCE.createExtend();
        } else if (this.isInclude) {
            return UMLFactory.eINSTANCE.createInclude();
        } else {            
            throw new NotFoundException("UseCaseDependency type not found");
        }
    }

    /**
     * Constructor
     * @param param : the exported Modelio IUseCaseDependency
     */
    @objid ("1118129c-4583-11e0-b54c-00137279a832")
    public OUseCaseDependency(final IUseCaseDependency param) {
        super(param);
        this.objingElement = param;
        if (ObjingModelNavigation.isStereotyped(this.objingElement, XMI.I18N
                .getString("objing.java.stereotype.extend")))
            this.isExtend = true;
        else if (ObjingModelNavigation.isStereotyped(this.objingElement, XMI.I18N
                .getString("objing.java.stereotype.include")))
            this.isInclude = true;
        else{
            throw new NotFoundException("UseCaseDependency type not found");
        }
    }

    @Override
    @objid ("111812a5-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt, CompositionOwnerVisitor ownerVisitor) {
        // This method is also called when linking a Usage, as Usage inherits
        // from Dependency.
        GenerationProperties genProp = GenerationProperties.getInstance();
        IUseCase objingTarget = this.objingElement.getTarget();
        IUseCase objingOrigin = this.objingElement.getOrigin();
        UseCase ecoreTarget = (UseCase) genProp.getMappedElement(objingTarget);
        UseCase ecoreOrigin = (UseCase) genProp.getMappedElement(objingOrigin);
        
        if (ecoreTarget != null && ecoreOrigin != null) {
            if (this.isExtend) {
                Extend ecoreElementExtend = (Extend) ecoreElt;
                ecoreElementExtend.setExtendedCase(ecoreTarget);
                ecoreElementExtend.setExtension(ecoreOrigin);
            } else if (this.isInclude) {
                Include ecoreElementInclude = (Include) ecoreElt;
                ecoreElementInclude.setAddition(ecoreTarget);
                ecoreElementInclude.setIncludingCase(ecoreOrigin);
            }
        }
    }

    @Override
    @objid ("111812a9-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
                if (this.isExtend){
           setExtensionLocation((Extend) ecoreElt);
           setExtendedCase((Extend) ecoreElt);
                }
    }

    @objid ("111a74f6-4583-11e0-b54c-00137279a832")
    private void setExtendedCase(Extend ecoreElt) {
        Element ecoreUseCase = GenerationProperties.getInstance().getMappedElement((this.objingElement).getTarget());
        
        if (ecoreUseCase instanceof UseCase)
            ecoreElt.setExtendedCase((UseCase) ecoreUseCase);
    }

    @objid ("111a74f9-4583-11e0-b54c-00137279a832")
    private void setExtensionLocation(Extend ecoreElt) {
        for (IExtensionPoint extPoint : this.objingElement.getExtensionLocation() ){
            Element ecoreExtPoint = GenerationProperties.getInstance().getMappedElement(extPoint);
        
        if (ecoreExtPoint instanceof ExtensionPoint)
            ecoreElt.getExtensionLocations().add((ExtensionPoint) ecoreExtPoint);
        }
    }

}
