/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.profile;

import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.StringConverter;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

@objid ("11219c07-4583-11e0-b54c-00137279a832")
public class PExportAttribut implements IExportProfileElement {
    @objid ("11219c08-4583-11e0-b54c-00137279a832")
    private ITagType objingElt = null;


    @objid ("11219c09-4583-11e0-b54c-00137279a832")
    public PExportAttribut(ITagType attribut) {
        objingElt = attribut;
    }

    @objid ("11219c0c-4583-11e0-b54c-00137279a832")
    public void accept(ProfileExportVisitorImpl visitor) {
        visitor.visit(this);
    }

    @objid ("11219c0f-4583-11e0-b54c-00137279a832")
    public void visit() {
        Stereotype stereotype = null;
        
        if (objingElt.getOwnerStereotype() != null){
            stereotype = (Stereotype) TotalExportMap.getInstance().get(objingElt.getOwnerStereotype().getIdentifier());
        }else{
            stereotype = (Stereotype) TotalExportMap.getInstance().get(objingElt.getOwnerReference().getIdentifier());
        }
        
        
        Property attr = null;
        String name = ProfileUtils.getTagTypeName(objingElt);
        
        
        if (objingElt.getParamNumber().equals("0")){
            attr = stereotype.getOwnedAttribute(name, PrimitiveTypeMapper.getBoolean());
            if (attr == null)
                attr = stereotype.createOwnedAttribute(name, PrimitiveTypeMapper.getBoolean());
            attr.setLower(1);
            attr.setUpper(1);
        }else{
        
            attr = stereotype.getOwnedAttribute(name, PrimitiveTypeMapper.getString());
            if (attr == null)
                attr = stereotype.createOwnedAttribute(name, PrimitiveTypeMapper.getString());
        
            attr.setLower(0);
        
            String max = objingElt.getParamNumber();
        
            if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE.equals(max))
                attr.setUpper(LiteralUnlimitedNatural.UNLIMITED);
            else {
                Integer intMax = StringConverter.getInteger(max);
                if (intMax != null)
                    attr.setUpper(intMax);
            }
        
        }
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setHidden(attr);
            setLabel(attr);
            setPartSignature(attr);
        }
        
        ObjingEAnnotation.addObjingID(attr, objingElt.getIdentifier());
    }

    @objid ("11219c11-4583-11e0-b54c-00137279a832")
    private void setLabel(Property attr) {
        ObjingEAnnotation.setLabel(attr, objingElt.getLabel());
    }

    @objid ("1123fe5f-4583-11e0-b54c-00137279a832")
    private void setHidden(Property attr) {
        ObjingEAnnotation.setHidden(attr, objingElt.isHidden());
    }

    @objid ("1123fe62-4583-11e0-b54c-00137279a832")
    private void setPartSignature(Property attr) {
        ObjingEAnnotation.setPartSignature(attr, objingElt.isBelongToPrototype());
    }

}
