/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.profile;

import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.PrimitiveTypeMapper;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

@objid ("112660bd-4583-11e0-b54c-00137279a832")
public class PExportNoteType implements IExportProfileElement {
    @objid ("112660be-4583-11e0-b54c-00137279a832")
    private INoteType objingElt = null;


    @objid ("112660bf-4583-11e0-b54c-00137279a832")
    public PExportNoteType(INoteType attribut) {
        objingElt = attribut;
    }

    @objid ("112660c2-4583-11e0-b54c-00137279a832")
    public void accept(ProfileExportVisitorImpl visitor) {
        visitor.visit(this);
    }

    @objid ("112660c5-4583-11e0-b54c-00137279a832")
    public void visit() {
        Stereotype stereotype = null;
        
        if (objingElt.getOwnerStereotype() != null){
            stereotype = (Stereotype)TotalExportMap.getInstance().get(objingElt.getOwnerStereotype().getIdentifier());
        }else{
            stereotype = (Stereotype)TotalExportMap.getInstance().get(objingElt.getOwnerReference().getIdentifier());
        }
        
        String name = objingElt.getName();
        
        Property attr  = stereotype.getOwnedAttribute(name, PrimitiveTypeMapper.getString());
        if (attr == null)
            attr = stereotype.createOwnedAttribute(name, PrimitiveTypeMapper.getString());
        
        attr.setLower(1);
        attr.setUpper(1);
        
        ObjingEAnnotation.setNoteType(attr, true);
        
        if (GenerationProperties.getInstance().isRoundtripEnabled()){
            setHidden(attr);
            setLabel(attr);
        
        }
        ObjingEAnnotation.addObjingID(attr, objingElt.getIdentifier());
    }

    @objid ("1128c312-4583-11e0-b54c-00137279a832")
    private void setLabel(Property attr) {
        ObjingEAnnotation.setLabel(attr, objingElt.getLabel());
    }

    @objid ("1128c315-4583-11e0-b54c-00137279a832")
    private void setHidden(Property attr) {
        ObjingEAnnotation.setHidden(attr, objingElt.isHidden());
    }

}
