/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.profile;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import org.eclipse.uml2.uml.Profile;

@objid ("1128c318-4583-11e0-b54c-00137279a832")
public class PExportProfile implements IExportProfileElement {
    @objid ("1128c319-4583-11e0-b54c-00137279a832")
    private IProfile objingElt = null;

    @objid ("1128c31a-4583-11e0-b54c-00137279a832")
    private TotalExportMap totalMap = TotalExportMap.getInstance();


    @objid ("1128c31b-4583-11e0-b54c-00137279a832")
    public PExportProfile(IProfile profile) {
        objingElt = profile;
    }

    @objid ("1128c31e-4583-11e0-b54c-00137279a832")
    public void accept(ProfileExportVisitorImpl visitor) {
        visitor.visit(this);
    }

    @objid ("112b256c-4583-11e0-b54c-00137279a832")
    public void visit() {
        Profile ecoreElt = (Profile) GenerationProperties.getInstance().getMappedElement(objingElt);
        
        if (ecoreElt == null){
            ecoreElt = ProfileUtils.createEcoreProfile(objingElt);  
            totalMap.put(objingElt.getIdentifier(), ecoreElt);
        }
        
        setProperties(ecoreElt);
        attach(ecoreElt);
    }

    @objid ("112b256e-4583-11e0-b54c-00137279a832")
    private void setProperties(Profile ecoreElt) {
        ObjingEAnnotation.setModule(ecoreElt, objingElt.getOwnerModule().getIdentifier());
        ObjingEAnnotation.addObjingID(ecoreElt, objingElt.getIdentifier());
        String name = ProfileUtils.getName(objingElt);
        ecoreElt.setName(name);
    }

    @objid ("112b2575-4583-11e0-b54c-00137279a832")
    public List<PExportProfile> getSubProfiles() {
        List<PExportProfile> subProfiles = new ArrayList<PExportProfile>();
        
        for (IModelTree sub : objingElt.getOwnedElement()){
            if (sub instanceof IProfile){
                PExportProfile temp = new PExportProfile((IProfile) sub);
                subProfiles.add(temp);
            }
        
        }
        return subProfiles;
    }

    @objid ("112d87c6-4583-11e0-b54c-00137279a832")
    public List<PExportStereotype> getStereotypes() {
        List<PExportStereotype> result = new ArrayList<PExportStereotype>();
        for (IStereotype sub : this.objingElt.getDefinedStereotype()){
            if (ObjingModelNavigation.mustBeExported(sub)){
                PExportStereotype temp = new PExportStereotype(sub);
                result.add(temp);
            }
        
        }
        return result;
    }

    @objid ("112d87cc-4583-11e0-b54c-00137279a832")
    public List<PExportReference> getReferences() {
        List<PExportReference> result = new ArrayList<PExportReference>();
        for (IMetaclassReference sub : this.objingElt.getOwnedReference()){
            PExportReference temp = new PExportReference(sub);
            result.add(temp);
        
        }
        return result;
    }

    @objid ("65a590ba-1a76-11e1-8103-0027103f347d")
    public IProfile getObjElt() {
        return this.objingElt;
    }

    @objid ("65a5b7cc-1a76-11e1-8103-0027103f347d")
    private void attach(final Profile ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        
        if (this.objingElt.getOwner() instanceof IProfile){
            Profile ecoreOwner = (Profile) genProp.getMappedElement(this.objingElt.getOwner());
            ecoreOwner.getPackagedElements().add(ecoreElt);
        }else{
        
            IElement owner = ProfileUtils.getProfileOwner(this.objingElt);
            Profile ecoreOwner = null;
            if ((owner != null) && (owner instanceof IProfile)){
                PExportProfile export = new PExportProfile((IProfile) owner);
                export.visit();
                ecoreOwner = (Profile) genProp.getMappedElement(owner);
                
                if  ((ecoreOwner != null) && (!(ecoreOwner.equals(ecoreElt))))
                    ecoreOwner.getPackagedElements().add(ecoreElt);
            }
          
        }
    }

}
