/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.profile;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.UMLMetamodel;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.ElementImport;
import org.eclipse.uml2.uml.Extension;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

@objid ("112d87d2-4583-11e0-b54c-00137279a832")
public class PExportReference implements IExportProfileElement {
    @objid ("112fea20-4583-11e0-b54c-00137279a832")
    private IMetaclassReference objingElt = null;

    @objid ("112fea22-4583-11e0-b54c-00137279a832")
    private TotalExportMap totalMap = TotalExportMap.getInstance();


    @objid ("112fea24-4583-11e0-b54c-00137279a832")
    public PExportReference(IMetaclassReference reference) {
        objingElt = reference;
    }

    @objid ("112fea27-4583-11e0-b54c-00137279a832")
    public void accept(ProfileExportVisitorImpl visitor) {
        visitor.visit(this);
    }

    @objid ("112fea2a-4583-11e0-b54c-00137279a832")
    private Stereotype createEcoreStereotype() {
        Profile profile = (Profile) totalMap.get(objingElt.getOwnerProfile().getIdentifier());
        
        Stereotype stereotype = ProfileUtils.createStereotype(objingElt, profile);
        
        ObjingEAnnotation.setReference(stereotype, true);
        return stereotype;
    }

    @objid ("112fea2e-4583-11e0-b54c-00137279a832")
    private void setProperties(Stereotype stereotype) {
        setInheritance(stereotype);
        ObjingEAnnotation.addObjingID(stereotype, objingElt.getIdentifier());
    }

    @objid ("112fea31-4583-11e0-b54c-00137279a832")
    private void setInheritance(Stereotype stereotype) {
        Profile profile = (Profile) totalMap.get(objingElt.getOwnerProfile().getIdentifier());
        
        Stereotype stereotypeEcore = (Stereotype) totalMap.get(objingElt.getIdentifier());
        
        for (String newMetaclassName : ProfileUtils.getEcoreNameClass(objingElt.getReferencedClassName())){
        
            Class metaclassClass = (Class) UMLMetamodel.getInstance().getUMLMetamodel().getOwnedType(newMetaclassName);
        
            boolean found = false;
            Class reference = metaclassClass;
        
            for (Object metaclass :  profile.getMetaclassReferences()){
                if (((Class)((ElementImport)metaclass).getImportedElement()).getName().equals(metaclassClass.getName())){
                    reference = (Class)((ElementImport)metaclass).getImportedElement();
                    found = true; 
                }
            }
        
            if (!found){ 
                profile.createMetaclassReference(reference);
                stereotypeEcore.createExtension(reference, false);
            }else{
        
                boolean typed = false ;
                for (Object extension : reference.getExtensions())
                    for (Object extensionEnd : ((Extension) extension).getOwnedEnds())
                        if (((Property) extensionEnd).getType().equals(stereotypeEcore))
                            typed = true;
        
                if (!typed)
                    stereotypeEcore.createExtension(reference, false);
            }    
        }
    }

    @objid ("11324c7a-4583-11e0-b54c-00137279a832")
    public List<PExportAttribut> getAttribute() {
        List<PExportAttribut> result = new ArrayList<PExportAttribut>();
        
        for (ITagType part : objingElt.getDefinedTagType()){
        
            PExportAttribut attribut = new PExportAttribut((ITagType) part);
            result.add(attribut);
        
        }
        return result;
    }

    @objid ("11324c80-4583-11e0-b54c-00137279a832")
    public List<PExportNoteType> getNoteTypes() {
        List<PExportNoteType> result = new ArrayList<PExportNoteType>();
        for (INoteType part : objingElt.getDefinedNoteType()){
        
            PExportNoteType attribut = new PExportNoteType((INoteType) part);
            result.add(attribut);
        
        }
        return result;
    }

    @objid ("11324c86-4583-11e0-b54c-00137279a832")
    public IElement getElt() {
        return objingElt;
    }

    @objid ("1134aed5-4583-11e0-b54c-00137279a832")
    public void visit() {
        Stereotype ecoreElt = (Stereotype) totalMap.get(objingElt.getIdentifier());
        
        if ((ecoreElt == null) && ObjingModelNavigation.mustBeExported(objingElt)){
            ecoreElt = createEcoreStereotype();
            totalMap.put(objingElt.getIdentifier(), ecoreElt);
            setProperties(ecoreElt);
        }
    }

}
