/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.profile;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ProfileUtils;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;

@objid ("1134aed7-4583-11e0-b54c-00137279a832")
public class PExportStereotype implements IExportProfileElement {
    @objid ("1134aed8-4583-11e0-b54c-00137279a832")
    private IStereotype objingElt = null;

    @objid ("1134aed9-4583-11e0-b54c-00137279a832")
    private GenerationProperties genProp = GenerationProperties.getInstance();

    @objid ("1134aeda-4583-11e0-b54c-00137279a832")
    private TotalExportMap totalMap = TotalExportMap.getInstance();

    @objid ("65a67b26-1a76-11e1-8103-0027103f347d")
    private ProfileExportVisitorImpl _visitor = null;


    @objid ("1134aedb-4583-11e0-b54c-00137279a832")
    public PExportStereotype(IStereotype stereotype) {
        objingElt = stereotype;
    }

    @objid ("1134aede-4583-11e0-b54c-00137279a832")
    public void accept(ProfileExportVisitorImpl visitor) {
        _visitor = visitor;
        visitor.visit(this);
    }

    @objid ("1137112e-4583-11e0-b54c-00137279a832")
    private Stereotype createEcoreStereotype() {
        return ProfileUtils.createStereotype(objingElt);
    }

    @objid ("11371132-4583-11e0-b54c-00137279a832")
    private void setProperties(Stereotype stereotype) {
        setInheritance(stereotype);
        
        ObjingEAnnotation.addObjingID(stereotype, objingElt.getIdentifier());
        
        if (genProp.isRoundtripEnabled()){
            setHidden(stereotype);
            setLabel(stereotype);
            setBaseClass(stereotype);
        
        }
    }

    @objid ("11371135-4583-11e0-b54c-00137279a832")
    private void setLabel(Stereotype stereotype) {
        ObjingEAnnotation.setLabel(stereotype, objingElt.getLabel());
    }

    @objid ("11371138-4583-11e0-b54c-00137279a832")
    private void setHidden(Stereotype stereotype) {
        ObjingEAnnotation.setHidden(stereotype, objingElt.isHidden());
    }

    @objid ("1137113b-4583-11e0-b54c-00137279a832")
    private void setIcon(Stereotype stereotype) {
        String iconPath = null;
        
        String smallIconPath = null;
        
        String explorerIconPath = null;
        for (ITaggedValue tag : objingElt.getTag()){
        
            String typeName = tag.getDefinition().getName();
        
            if (typeName.equals("icon")){
                for (ITagParameter actual : tag.getActual()){
                    iconPath = actual.getValue();
                }
            }else if (typeName.equals("smallIcon")){
                for (ITagParameter actual : tag.getActual()){
                    smallIconPath = actual.getValue();
                }
            }else if (typeName.equals("explorerIcon")){
                for (ITagParameter actual : tag.getActual()){
                    explorerIconPath = actual.getValue();
                }
            }
        }        
        
        String genroot = ResourceLoader.getInstance().getProjectRoot();
        
        if ((iconPath != null) && (!iconPath.equals("")))
            ProfileUtils.setStereotypeImage(genroot + iconPath, stereotype);
        else if ((smallIconPath != null) && (!smallIconPath.equals("")))
            ProfileUtils.setStereotypeImage(genroot + smallIconPath, stereotype);
        else if ((explorerIconPath != null) && (!explorerIconPath.equals("")))
            ProfileUtils.setStereotypeImage(genroot + explorerIconPath, stereotype);
    }

    @objid ("1137113e-4583-11e0-b54c-00137279a832")
    private void setInheritance(Stereotype stereotype) {
        // inheritance link
        
        IStereotype parent = objingElt.getParent();
        
        if(( parent != null) && (ProfileUtils.isInScope(parent))){
            Stereotype parentEcore = (Stereotype) genProp.getMappedElement(parent);
        
            if (parentEcore == null)  {
                IProfile obParentProfile = objingElt.getParent().getOwner();
                PExportProfile parentProfile = new PExportProfile(obParentProfile);
                _visitor.visit(parentProfile);
        
                Profile ecoreParentProfile = (Profile) genProp.getMappedElement(obParentProfile);
        
                Profile ecoreProfile = (Profile) genProp.getMappedElement(objingElt.getOwner());
                genProp.inverseProfiles(ecoreParentProfile, ecoreProfile);
        
                parentEcore = (Stereotype) genProp.getMappedElement(parent);
            }
        
            if (!stereotype.getSuperClasses().contains(parentEcore))
                stereotype.getSuperClasses().add(parentEcore);
        }
    }

    @objid ("1139738c-4583-11e0-b54c-00137279a832")
    public List<PExportAttribut> getAttribute() {
        List<PExportAttribut> result = new ArrayList<PExportAttribut>();
        for (ITagType part : objingElt.getDefinedTagType()){
        
            PExportAttribut attribut = new PExportAttribut((ITagType) part);
            result.add(attribut);
        
        }
        return result;
    }

    @objid ("11397392-4583-11e0-b54c-00137279a832")
    public IElement getElt() {
        return objingElt;
    }

    @objid ("113bd5e2-4583-11e0-b54c-00137279a832")
    public void visit() {
        Stereotype ecoreElt = (Stereotype) totalMap.get(objingElt.getIdentifier());
        
        if ((ecoreElt == null) && ObjingModelNavigation.mustBeExported(objingElt)){
            ecoreElt = createEcoreStereotype();
            totalMap.put(objingElt.getIdentifier(), ecoreElt);
            setProperties(ecoreElt);
        }
    }

    @objid ("113bd5e4-4583-11e0-b54c-00137279a832")
    private void setBaseClass(Stereotype stereotype) {
        ObjingEAnnotation.addBaseClass(stereotype, objingElt.getBaseClassName());
    }

    @objid ("113bd5e7-4583-11e0-b54c-00137279a832")
    public List<PExportNoteType> getNoteTypes() {
        List<PExportNoteType> result = new ArrayList<PExportNoteType>();
        for (INoteType part : objingElt.getDefinedNoteType()){
        
            PExportNoteType attribut = new PExportNoteType((INoteType) part);
            result.add(attribut);
        
        }
        return result;
    }

}
