/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import java.lang.reflect.Method;
import com.modeliosoft.modelio.api.model.change.IModelChangeHandler;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IConstraint;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INote;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.model.objing.OConstraint;
import com.modeliosoft.modelio.xmi.model.objing.ODependency;
import com.modeliosoft.modelio.xmi.model.objing.OElement;
import com.modeliosoft.modelio.xmi.model.objing.ONote;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.eclipse.uml2.uml.PackageableElement;

@objid ("113bd5ee-4583-11e0-b54c-00137279a832")
public abstract class BaseMapper extends OElement {
    @objid ("113e383c-4583-11e0-b54c-00137279a832")
    protected IElement objingElement;

    @objid ("113e383d-4583-11e0-b54c-00137279a832")
    protected MainMapper mapper;


    @objid ("113e383e-4583-11e0-b54c-00137279a832")
    public abstract Element createEcoreElt();

    @objid ("113e3840-4583-11e0-b54c-00137279a832")
    public BaseMapper(IElement param, MainMapper mapper) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        // If element doesn't exist
        this.objingElement = param;
        if (!genProp.isMappedElement(param)) {
            this.mapper = mapper;
            Element ecoreElt = createEcoreElt();
        
            // If the current element is an Operation, we don't add it in the
            // HashMap,
            // as is would be associated to an event (instead of an Operation):
            if (!((this.objingElement instanceof IOperation) || (this.objingElement instanceof ISignal)))
                TotalExportMap.getInstance().put(this.objingElement.getIdentifier(),
                                                 ecoreElt);
        
            setName(ecoreElt);
        
            attach(ecoreElt);
            setProperties(ecoreElt);
            if (this.objingElement instanceof IModelElement){
                setCommonRelations(ecoreElt);
            }
        
        }
    }

    @objid ("113e3844-4583-11e0-b54c-00137279a832")
    public abstract void attach(Element ecoreElt);

    @objid ("113e3846-4583-11e0-b54c-00137279a832")
    public abstract void setProperties(Element ecoreElt);

    @objid ("113e3848-4583-11e0-b54c-00137279a832")
    public void attachInteractionFragment(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        ObList<ILifeline> objingLifeline = ((IInteractionFragment) this.objingElement)
                .getCovered();
        IInteraction interaction = getInteraction((IInteractionFragment) this.objingElement);
        IInteractionOperand interactionOperand = ((IInteractionFragment) this.objingElement)
                .getEnclosingOperand();
        if (interaction != null || interactionOperand != null) {
            // We add the fragement to the lifeline
            for (ILifeline lifeline : objingLifeline) {
                Lifeline ecoreLifeline = (Lifeline) genProp
                        .getMappedElement(lifeline);
                if (ecoreLifeline != null) {
                    try{
                        //                    ecoreLifeline.getCoveredBys().add(ecoreElt);
                        ecoreLifeline.getCoveredBys().add((InteractionFragment)ecoreElt);
                    }catch (ArrayStoreException e){
                        e.printStackTrace();
                    }
                }
            }
            // form ecore we add the fragement to the interaction
            if (interaction != null) {
                Interaction ecoreInteraction = (Interaction) genProp
                        .getMappedElement(interaction);
                if (ecoreInteraction != null)
                    addFragementToInteraction(ecoreInteraction, ecoreElt);
            } else if (interactionOperand != null) {
                InteractionOperand ecoreInteractionOperand = (InteractionOperand) genProp
                        .getMappedElement(interactionOperand);
                if (ecoreInteractionOperand != null)
                    addFragementToInteractionOperand(ecoreInteractionOperand,
                                                     ecoreElt);
            }
        }
    }

    @objid ("113e384b-4583-11e0-b54c-00137279a832")
    protected IInteraction getInteraction(IInteractionFragment objingElement) {
        return objingElement.getEnclosingInteraction();
    }

    @objid ("113e3850-4583-11e0-b54c-00137279a832")
    protected void addFragementToInteraction(Interaction ecoreInteraction, Element ecoreElt) {
        if ((ecoreInteraction != null) && (ecoreElt instanceof InteractionFragment))
            ecoreInteraction.getFragments().add((InteractionFragment)ecoreElt);
    }

    @objid ("11409a98-4583-11e0-b54c-00137279a832")
    protected void attachEvent(OccurrenceSpecification ecoreOccSpec, Element ecoreElt) {
        if ((ecoreOccSpec != null) && (ecoreElt instanceof PackageableElement)){
            EList<PackageableElement>   list = ecoreOccSpec.getNearestPackage().getPackagedElements();
            int size = list.size();
            if (size > 0 ){ 
                PackageableElement last = list.get(list.size()-1);
                if ((last.getClass().equals(ecoreElt.getClass())) && (last.getName().equals(((PackageableElement)ecoreElt).getName()))){
                    list.remove(last);
                    last.destroy();
        
                }
                list.add((PackageableElement)ecoreElt);
        
            }else
                list.add((PackageableElement)ecoreElt);
        }
    }

    @objid ("11409a9c-4583-11e0-b54c-00137279a832")
    protected IInteraction getOwnerInteraction(IElement objingElement) {
        CompositionOwnerVisitor ownerVisitor = this.mapper.getGenericOwner();
        IElement owner = ownerVisitor.getCompositionOwner(objingElement);
        while (owner != null && !(owner instanceof IInteraction)) {
            owner = ownerVisitor.getCompositionOwner(owner);
        }
        // special case for gate on PartDecomposition
        if (owner == null) {
            if (objingElement instanceof IGate
                    && ((IGate) objingElement).getOwnerUse() instanceof IPartDecomposition) {
                return PartDecompositionMapper
                        .getIInteraction((IPartDecomposition) ((IGate) objingElement)
                                         .getOwnerUse());
            } else if (objingElement instanceof IPartDecomposition) {
                return PartDecompositionMapper
                        .getIInteraction((IPartDecomposition) objingElement);
            }
        }
        return (IInteraction) owner;
    }

    @objid ("11409aa1-4583-11e0-b54c-00137279a832")
    protected void addFragementToInteractionOperand(InteractionOperand ecoreInteractionOperand, Element ecoreElt) {
        if (ecoreElt instanceof InteractionFragment)
            ecoreInteractionOperand.getFragments().add((InteractionFragment)ecoreElt);
    }

    @objid ("11409aa5-4583-11e0-b54c-00137279a832")
    private void setName(Element element) {
        try {
            // Get name
            Method methode = objingElement.getClass().getMethod("getName", (Class[]) null);
            String mame = (String) methode.invoke(this.objingElement);
            // set Name
            Object[] params = { mame };
            methode = element.getClass().getMethod("setName", String.class );
            methode.invoke(element, params);
        } catch (Exception e) {
            Modelio.out.println("Set name NOK");
        }
    }

    @objid ("e9f79f1e-6c3d-11e1-b236-0027103f347d")
    private void setCommonRelations(final Element ecoreElt) {
        setNotes();
        setStereotypes();
        setDependencies();
        setConstraints();
    }

    @objid ("e9f94ce4-6c3d-11e1-b236-0027103f347d")
    private void setStereotypes() {
        new OElement(this.objingElement);
    }

    @objid ("e9f973f6-6c3d-11e1-b236-0027103f347d")
    private void setDependencies() {
        for (IDependency dep : ((IModelElement) this.objingElement).getDependsOnDependency()){
            if (GenerationProperties.getInstance().getMappedElement(dep.getImpacted()) != null){
                ODependency odep = new ODependency(dep);
                Element ecoreDep = GenerationProperties.getInstance().getMappedElement(dep);
                if (ecoreDep == null){
                    ecoreDep = odep.createEcoreElt();
                }
                
                odep.attach(ecoreDep, new CompositionOwnerVisitor());
                odep.setProperties(ecoreDep);
            }
        }
        
        
        for (IDependency dep : ((IModelElement) this.objingElement).getImpactedDependency()){
            if (GenerationProperties.getInstance().getMappedElement(dep.getDependsOn()) != null){
                ODependency odep = new ODependency(dep);
                Element ecoreDep = GenerationProperties.getInstance().getMappedElement(dep);
                if (ecoreDep == null){
                    ecoreDep = odep.createEcoreElt();
                }
                
                odep.attach(ecoreDep, new CompositionOwnerVisitor());
                odep.setProperties(ecoreDep);
            }
        }
    }

    @objid ("e9f99b08-6c3d-11e1-b236-0027103f347d")
    private void setConstraints() {
        for (IConstraint constraint : ((IModelElement) this.objingElement).getConstraintDefinition()){
            boolean allExist = true;
            for (IModelElement constrainted : constraint.getConstrainedElement()){
                if (GenerationProperties.getInstance().getMappedElement(constrainted) == null){
                 allExist = false;
                 break;
                }
            }
            
            if (allExist){
                OConstraint oConstraint = new OConstraint(constraint);
                Element ecoreConstraint = GenerationProperties.getInstance().getMappedElement(constraint);
                if (ecoreConstraint == null){
                    ecoreConstraint = oConstraint.createEcoreElt();
                }
                
                oConstraint.attach(ecoreConstraint, new CompositionOwnerVisitor());
                oConstraint.setProperties(ecoreConstraint);
            }
        }
    }

    @objid ("e9f9c21a-6c3d-11e1-b236-0027103f347d")
    private void setNotes() {
        for (INote note : ((IModelElement) this.objingElement).getDescriptor()){
            ONote oNote = new ONote(note);
            Element ecoreNote = GenerationProperties.getInstance().getMappedElement(note);
            if (ecoreNote == null){
                ecoreNote = oNote.createEcoreElt();
            }
            
            oNote.attach(ecoreNote, new CompositionOwnerVisitor());
            oNote.setProperties(ecoreNote);
        }
    }

}
