/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ICombinedFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionOperand;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InteractionOperatorKind;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1142fcf0-4583-11e0-b54c-00137279a832")
public class CombinedFragmentMapper extends BaseMapper {

    @Override
    @objid ("1142fcf1-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createCombinedFragment();
    }

    @objid ("1142fcf6-4583-11e0-b54c-00137279a832")
    public CombinedFragmentMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("1142fcfa-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        attachInteractionFragment(ecoreElt);
    }

    @Override
    @objid ("11455f4b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setInteractionOperator((CombinedFragment) ecoreElt);
        setInteractionOperand();
    }

    @objid ("11455f4f-4583-11e0-b54c-00137279a832")
    private void setInteractionOperand() {
        ObList<IInteractionOperand> objiingInteractionOps = ((ICombinedFragment) objingElement)
                .getOperand();
        for (IInteractionOperand objiingInteractionOp : objiingInteractionOps) {
            new InteractionOperandMapper(objiingInteractionOp, mapper);
        }
    }

    @objid ("11455f52-4583-11e0-b54c-00137279a832")
    private void setInteractionOperator(CombinedFragment ecoreElt) {
        switch (((ICombinedFragment) objingElement).getOperator()) {
        case ALT_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.ALT_LITERAL);
            break;
        case ASSERT_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.ASSERT_LITERAL);
            break;
        case BREAK_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.BREAK_LITERAL);
            break;
        case CONSIDER_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.CONSIDER_LITERAL);
            break;
        case CRITICAL_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.CRITICAL_LITERAL);
            break;
        case IGNORE_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.IGNORE_LITERAL);
            break;
        case LOOP_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.LOOP_LITERAL);
            break;
        case NEG_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.NEG_LITERAL);
            break;
        case OPT_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.OPT_LITERAL);
            break;
        case PAR_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.PAR_LITERAL);
            break;
        case SEQ_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.SEQ_LITERAL);
            break;
        case STRICT_OP:
            ecoreElt
                    .setInteractionOperator(InteractionOperatorKind.STRICT_LITERAL);
            break;
        }
    }

}
