/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.CompositionOwnerVisitor;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;

@objid ("114a2404-4583-11e0-b54c-00137279a832")
public abstract class EventMapper {
    @objid ("114a2405-4583-11e0-b54c-00137279a832")
     MainMapper mapper;

    @objid ("114a2406-4583-11e0-b54c-00137279a832")
     IElement objingElement;

    @objid ("114a2407-4583-11e0-b54c-00137279a832")
     NamedElement ecoreMessOccSpec;


    @objid ("114a2408-4583-11e0-b54c-00137279a832")
    public abstract Element createEcoreElt();

    @objid ("114a240a-4583-11e0-b54c-00137279a832")
    protected IInteraction getOwnerInteraction() {
        CompositionOwnerVisitor ownerVisitor = mapper.getGenericOwner();
        IElement owner = ownerVisitor.getCompositionOwner(objingElement);
        while (owner != null && !(owner instanceof IInteraction)) {
            owner = ownerVisitor.getCompositionOwner(owner);
        }
        // special case for gate on PartDecomposition
        if (owner == null) {
            if (objingElement instanceof IGate
                    && ((IGate) objingElement).getOwnerUse() instanceof IPartDecomposition) {
                return PartDecompositionMapper
                .getIInteraction((IPartDecomposition) ((IGate) objingElement)
                        .getOwnerUse());
            }
        }
        return (IInteraction) owner;
    }

    @objid ("114c8658-4583-11e0-b54c-00137279a832")
    public EventMapper(NamedElement ecoreMessOccSpec, IElement objingElement, MainMapper mapper) {
        this.ecoreMessOccSpec = ecoreMessOccSpec;
        this.mapper = mapper;
        this.objingElement = objingElement;
        Event event = mapper.getEvent(this);
        if (event == null) {
            Element ecoreElt = createEcoreElt();
            mapper.addEvent(this, (Event) ecoreElt);
            attach(ecoreElt);
            setProperties(ecoreElt);
        } else {
            setProperties(event);
        }
    }

    @objid ("114c865d-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        // We add the event to the Nearest Package if the element don't have
        // Nearest Package we add to the Nearest Package of the Interaction
        Package thePackage = ecoreMessOccSpec.getNearestPackage();
        if (thePackage == null) {
            GenerationProperties genProp = GenerationProperties.getInstance();
            IInteraction objingInteraction = getOwnerInteraction();
                
            if (objingInteraction != null) {
                if (genProp.isMappedElement(objingInteraction)) {
                    Interaction ecoreInteraction = (Interaction) genProp
                    .getMappedElement(objingInteraction);
                    if (ecoreInteraction != null)
                        thePackage = ecoreInteraction.getNearestPackage();
                }
            }
        }
        
        if (thePackage != null){
            thePackage.getPackagedElements().add((PackageableElement)ecoreElt);
        }
    }

    @objid ("114c8660-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setEvent(ecoreElt);
    }

    @objid ("114c8663-4583-11e0-b54c-00137279a832")
    private void setEvent(Element ecoreElt) {
        if (ecoreMessOccSpec instanceof OccurrenceSpecification) {
            ((OccurrenceSpecification) ecoreMessOccSpec)
            .setEvent((Event) ecoreElt);
        }
    }

}
