/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionOccurenceSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionSpecification;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutionOccurrenceSpecification;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1153ad69-4583-11e0-b54c-00137279a832")
public class ExecutionSpecificationMapper extends BaseMapper {

    @Override
    @objid ("1153ad6a-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createBehaviorExecutionSpecification();
    }

    @objid ("1153ad6f-4583-11e0-b54c-00137279a832")
    public ExecutionSpecificationMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @objid ("11560fc0-4583-11e0-b54c-00137279a832")
    public ExecutionSpecificationMapper(IElement param, MainMapper mapper, boolean setProperty) {
        super(param, mapper);
        if (setProperty) {
            GenerationProperties genProp = GenerationProperties.getInstance();
            Element ecoreElt = genProp.getMappedElement(objingElement);
            if (ecoreElt != null)
                setProperties(ecoreElt);
                attach(ecoreElt);
                
        }
    }

    @Override
    @objid ("11560fc5-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        attachInteractionFragment(ecoreElt);
    }

    @Override
    @objid ("11560fc9-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setStartFinish(ecoreElt);
    }

    @objid ("1158721a-4583-11e0-b54c-00137279a832")
    public void setStartFinish(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IExecutionSpecification objingExc = (IExecutionSpecification) objingElement;
                
        BehaviorExecutionSpecification ecoreExc = null;
        if (ecoreElt instanceof BehaviorExecutionSpecification)
            ecoreExc = (BehaviorExecutionSpecification) ecoreElt;
                
                
        IExecutionOccurenceSpecification objingStart = objingExc.getStart();
        Element ecoreStart = null;
        if (genProp.isMappedElement(objingStart))
            ecoreStart = genProp.getMappedElement(objingStart);
                
        IExecutionOccurenceSpecification objingFinish = objingExc.getFinish();
        Element ecoreFinish = null;
        if (genProp.isMappedElement(objingFinish))
            ecoreFinish = genProp.getMappedElement(objingFinish);
                
        // We check if excution is set to ExecutionOccurrenceSpecification if
        // not we set it
                
        if (ecoreExc != null){
            if (ecoreStart instanceof OccurrenceSpecification) {
                if (ecoreStart instanceof ExecutionOccurrenceSpecification)
                    ((ExecutionOccurrenceSpecification) ecoreStart)
                    .setExecution(ecoreExc);
                ecoreExc.setStart((OccurrenceSpecification) ecoreStart);
            }
                
            if (ecoreFinish instanceof OccurrenceSpecification) {
                if (ecoreFinish instanceof ExecutionOccurrenceSpecification)
                    ((ExecutionOccurrenceSpecification) ecoreFinish)
                    .setExecution(ecoreExc);
                ecoreExc.setFinish((OccurrenceSpecification) ecoreFinish);
        
        }
                
        }
    }

}
