/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IGate;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionUse;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionUse;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1161fb82-4583-11e0-b54c-00137279a832")
public class InteractionUseMapper extends BaseMapper {

    @Override
    @objid ("1161fb83-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createInteractionUse();
    }

    @objid ("1161fb88-4583-11e0-b54c-00137279a832")
    public InteractionUseMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("1161fb8c-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        attachInteractionFragment(ecoreElt);
    }

    @Override
    @objid ("11645dde-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setInteraction((InteractionUse) ecoreElt);
        setActualGates((InteractionUse) ecoreElt);
    }

    @objid ("11645de2-4583-11e0-b54c-00137279a832")
    private void setInteraction(InteractionUse ecoreElt) {
        IInteractionUse objingInteractionUse = (IInteractionUse) objingElement;
        IInteraction objingInteractionRef = objingInteractionUse.getRefersTo();
        if (objingInteractionRef == null) {
            objingInteractionRef = getOwnerInteraction(objingInteractionUse);
        }
        GenerationProperties genProp = GenerationProperties.getInstance();
        Interaction ecoreInteraction = (Interaction) genProp
                .getMappedElement(objingInteractionRef);
        if (ecoreInteraction != null)
            ecoreElt.setRefersTo(ecoreInteraction);
    }

    @objid ("11645de5-4583-11e0-b54c-00137279a832")
    private void setActualGates(InteractionUse ecoreElt) {
        IInteractionUse objingInteractionUse = (IInteractionUse) objingElement;
        ObList<IGate> objiingGateListe = objingInteractionUse.getActualGate();
        for (IGate objingGate : objiingGateListe) {
            new GateMapper(objingGate, this.mapper);
            GenerationProperties genProp = GenerationProperties.getInstance();
            if (genProp.isMappedElement(objingGate)) {
                Gate ecoreFormalGate = (Gate) genProp
                        .getMappedElement(objingGate);
                if (ecoreFormalGate != null)
                    ecoreElt.getActualGates().add(ecoreFormalGate);
            }
        }
    }

}
