/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.UMLFactory;

/**
 * This class manages the export of ILifeline elements
 * @author ebrosse
 */
@objid ("11645de8-4583-11e0-b54c-00137279a832")
public class LifelineMapper extends BaseMapper {

    @Override
    @objid ("1166c036-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createLifeline();
    }

    /**
     * Constructor
     * @param param the exported ILifeline
     * @param mapper the mainmapper
     */
    @objid ("1166c03b-4583-11e0-b54c-00137279a832")
    public LifelineMapper(final IElement param, final MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("1166c03f-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        IInteraction objingOwner = ((ILifeline) this.objingElement).getOwner();
        Interaction ecoreOwner = (Interaction) genProp
        .getMappedElement(objingOwner);
        if (ecoreOwner != null) {
            ecoreOwner.getLifelines().add((Lifeline)ecoreElt);
        }
    }

    @Override
    @objid ("1166c043-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setSelector((Lifeline) ecoreElt);
        setDecomposedAs();
        setRepresented((Lifeline) ecoreElt);
    }

    @objid ("11692293-4583-11e0-b54c-00137279a832")
    private void setRepresented(Lifeline ecoreElt) {
        IElement type = ((ILifeline) this.objingElement).getRepresented();
        if (type != null) {
            Element temp = GenerationProperties.getInstance().getMappedElement(type);
            if (temp instanceof ConnectableElement){
                ecoreElt.setRepresents((ConnectableElement)temp);
            }else{
                XMILogs xmiLogs = XMILogs.getInstance();
                String message = XMI.I18N.getMessage("logFile.warning.unsupportedTypeExport", 
                                                     ((IInstance)type).getName(), 
                                                     this.objingElement.getClass().getName());
                xmiLogs.writelnInLog(message);
                GenerationProperties.getInstance().getReportModel().addWarning(message, this.objingElement);
            }
                
        }
    }

    @objid ("116b84f1-4583-11e0-b54c-00137279a832")
    private void setSelector(Lifeline lifeline) {
        String selector = ((ILifeline) this.objingElement).getSelector();
        if (selector != null && selector.trim().length() > 0) {
            LiteralString valueSpecification = UMLFactory.eINSTANCE
            .createLiteralString();
            valueSpecification.setValue(selector);
            lifeline.setSelector(valueSpecification);
        }
    }

    @objid ("116b84f4-4583-11e0-b54c-00137279a832")
    private void setDecomposedAs() {
        IPartDecomposition objingPartDec = ((ILifeline) this.objingElement)
        .getDecomposedAs();
        if (objingPartDec != null) {
            new PartDecompositionMapper(objingPartDec, this.mapper);
        }
    }

}
