/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessageEnd;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObMessageSortEnum;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OccurrenceSpecification;

@objid ("1172abf8-4583-11e0-b54c-00137279a832")
public abstract class MessageEndMapper extends BaseMapper {
    @objid ("1172abf9-4583-11e0-b54c-00137279a832")
    protected IMessage objingMessageSent = null;

    @objid ("1172abfa-4583-11e0-b54c-00137279a832")
    protected IMessage objingMessageReceived = null;


    @Override
    @objid ("1172abfb-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        IMessageEnd objiingMessageEnd = (IMessageEnd) objingElement;
        objingMessageSent = objiingMessageEnd.getSentMessage();
        objingMessageReceived = objiingMessageEnd.getReceivedMessage();
        return null;
    }

    @objid ("1172ac00-4583-11e0-b54c-00137279a832")
    public MessageEndMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("1172ac04-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        attachInteractionFragment(ecoreElt);
    }

    @Override
    @objid ("11750e52-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        if (ecoreElt instanceof MessageOccurrenceSpecification
                || ecoreElt instanceof Gate) {
            setMessageAndSignatureMessage((MessageEnd) ecoreElt, objingMessageReceived);
            setMessageAndSignatureMessage((MessageEnd) ecoreElt, objingMessageSent);
        } else {
            setEvent((OccurrenceSpecification) ecoreElt, objingMessageSent);
        }
    }

    @objid ("11750e56-4583-11e0-b54c-00137279a832")
    private void setMessageAndSignatureMessage(MessageEnd ecoreMessOccSpec, IMessage message) {
        // we set the message on Execution occurence because if we set on
        // Message we may
        // create Execution Occurence in bad order
        if (message != null) {
            GenerationProperties genProp = GenerationProperties.getInstance();
                
            IOperation objinOperation = message.getInvoked();
            ISignal objingSignal = message.getSignalSignature();
                
            Message ecoreMessage = null;
            if (!genProp.isMappedElement(message)) {
                new MessageMapper(message, mapper);
            }
                
            ecoreMessage = (Message) genProp.getMappedElement(message);
                
                
            if (ecoreMessage != null) {
                if (objingElement.equals(message.getSendEvent())) {
                    ecoreMessage.setSendEvent(ecoreMessOccSpec);
                    ecoreMessOccSpec.setMessage(ecoreMessage);
                
                    boolean setEvent = true;
                
                    if (message.getSortOfMessage().equals(ObMessageSortEnum.CREATE_MESSAGE)
                            && objinOperation != null
                            && ecoreMessOccSpec instanceof MessageOccurrenceSpecification){
                        setEvent = false;
                        new CreationEventMapper(
                                (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                objinOperation,
                                mapper);
                    }
                
                    // We set a SentEvent Operation
                    if (objinOperation != null
                            && ecoreMessOccSpec instanceof MessageOccurrenceSpecification) {
                        setEvent = false;
                        new SendOperationEventMapper(
                                objinOperation,
                                (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                mapper);
                    }
                
                    if (objingSignal != null
                            && ecoreMessOccSpec instanceof MessageOccurrenceSpecification) {
                        setEvent = false;
                        new SendSignalEventMapper(
                                objingSignal,
                                (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                mapper);
                    }
                
                    if (setEvent) {
                        setEvent((NamedElement) ecoreMessOccSpec, message);
                    }
                
                
                } else if (objingElement.equals(message.getReceiveEvent())){
                    
                        ecoreMessage.setReceiveEvent(ecoreMessOccSpec);
                        ecoreMessOccSpec.setMessage(ecoreMessage);
                
                        boolean setEvent = true;
                
                        if (message.getSortOfMessage().equals(ObMessageSortEnum.CREATE_MESSAGE)
                                && objinOperation != null
                                && ecoreMessOccSpec instanceof MessageOccurrenceSpecification){
                            setEvent = false;
                            new CreationEventMapper(
                                    (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                    objinOperation,
                                    mapper);
                        }
                        
                        // We set a ReceiveEvent Operation
                        if (objinOperation != null
                                && ecoreMessOccSpec instanceof MessageOccurrenceSpecification
                                && ((MessageOccurrenceSpecification)ecoreMessOccSpec).getMessage()!= null) {
                            setEvent = false;
                            new ReceiveOperationEventMapper(
                                    objinOperation,
                                    (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                    mapper);
                        }
                
                        if (objingSignal != null
                                && ecoreMessOccSpec instanceof MessageOccurrenceSpecification) {
                            setEvent = false;
                            new ReceiveSignalEventMapper(
                                    objingSignal,
                                    (MessageOccurrenceSpecification) ecoreMessOccSpec,
                                    mapper);
                        }
                
                        if (setEvent) {
                            setEvent((NamedElement) ecoreMessOccSpec, message);
                        }
                    }
            }
        }
    }

    @objid ("11750e5a-4583-11e0-b54c-00137279a832")
    private void setEvent(NamedElement ecoreMessOccSpec, IMessage objingMessage) {
        if (objingMessage != null) {
            if (haveUML2Event(objingMessage)){ 
                 new UMLEventMapper(ecoreMessOccSpec, objingElement,
                        mapper);
            }else if (objingMessage.getSortOfMessage().equals(
                    ObMessageSortEnum.DESTROY_MESSAGE)) {
                new DestructionEventMapper(ecoreMessOccSpec, objingElement,
                        mapper);
            } else if (objingMessage.getSortOfMessage().equals(
                    ObMessageSortEnum.CREATE_MESSAGE)) {
                new CreationEventMapper(ecoreMessOccSpec, objingElement, mapper);
            } else {
                new ExecutionEventMapper(ecoreMessOccSpec, objingElement,
                        mapper);
            }
        } else {
            new ExecutionEventMapper(ecoreMessOccSpec, objingElement, mapper);
        }
    }

    @objid ("65886a4c-1a76-11e1-8103-0027103f347d")
    private boolean haveUML2Event(final IMessage objingMessage) {
        IMessageEnd end = objingMessage.getSendEvent();
        IMessageEnd start = objingMessage.getReceiveEvent();
        for (IDependency dep : end.getDependsOnDependency()){
            if (dep.isStereotyped("UML2Event"))
                return true;
        }
        
        for (IDependency dep : start.getDependsOnDependency()){
            if (dep.isStereotyped("UML2Event"))
                return true;
        }
        return false;
    }

}
