/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.usecase.IActor;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.MessageEvent;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ReceiveOperationEvent;
import org.eclipse.uml2.uml.ReceiveSignalEvent;
import org.eclipse.uml2.uml.SendOperationEvent;
import org.eclipse.uml2.uml.SendSignalEvent;
import org.eclipse.uml2.uml.Signal;

@objid ("11750e5e-4583-11e0-b54c-00137279a832")
public abstract class MessageEventMapper extends BaseMapper {
    @objid ("11750e5f-4583-11e0-b54c-00137279a832")
    protected MessageOccurrenceSpecification ecoreMessOccSpec;


    @Override
    @objid ("117770ac-4583-11e0-b54c-00137279a832")
    public abstract Element createEcoreElt();

    @objid ("117770af-4583-11e0-b54c-00137279a832")
    public MessageEventMapper(IElement operation, MessageOccurrenceSpecification ecoreMessOccSpec, MainMapper mapper) {
        super(operation, mapper);
        objingElement = operation;
        this.ecoreMessOccSpec = ecoreMessOccSpec;
        Element ecoreElt = createEcoreElt();
        setProperties(ecoreElt);
        attach(ecoreElt);
    }

    @Override
    @objid ("117770b4-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        // We add the event to the selected root package
                setMessageEventName(ecoreElt);
        attachEvent(ecoreMessOccSpec, ecoreElt);
    }

    @Override
    @objid ("117770b8-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setEvent(ecoreElt);
        if (ecoreElt instanceof SendOperationEvent
                || ecoreElt instanceof ReceiveOperationEvent) {
            setOperation(ecoreElt);
        } else if (ecoreElt instanceof SendSignalEvent
                || ecoreElt instanceof ReceiveSignalEvent) {
            setSignal(ecoreElt);
        }
    }

    @objid ("1179d306-4583-11e0-b54c-00137279a832")
    private void setMessageEventName(Element ecoreElt) {
        MessageEvent messageEvent = (MessageEvent) ecoreElt;
                
        String theClass = messageEvent.getClass().getSimpleName();
        if (theClass != null) {
            if (theClass.endsWith("Impl")) {
                theClass = theClass.substring(0, theClass.length() - 4);
            }
        }
                
        if (ecoreMessOccSpec != null)
            messageEvent
            .setName(theClass
                    + (ecoreMessOccSpec.getMessage().getName() != null ? ecoreMessOccSpec
                            .getMessage().getName()
                            : ""));
    }

    @objid ("1179d309-4583-11e0-b54c-00137279a832")
    private void setSignal(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        Signal ecoreSignal = (Signal) genProp.getMappedElement(objingElement);
                
        if (ecoreSignal != null) {
            if (ecoreElt instanceof SendSignalEvent) {
                ((SendSignalEvent) ecoreElt).setSignal(ecoreSignal);
            } else {
                ((ReceiveSignalEvent) ecoreElt).setSignal(ecoreSignal);
            }
        }
    }

    @objid ("1179d30c-4583-11e0-b54c-00137279a832")
    private void setOperation(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        Operation ecoreOp = (Operation) genProp.getMappedElement(objingElement);
                
                
        if (ecoreOp != null) {
            IElement objingOwner = objingElement.getCompositionOwner();
            if (!(objingOwner instanceof IActor
                    || objingOwner instanceof IUseCase
                    || objingOwner instanceof ISignal)) {
                
                if (ecoreElt instanceof SendOperationEvent) {
                    ((SendOperationEvent) ecoreElt).setOperation(ecoreOp);
                } else {
                    ((ReceiveOperationEvent) ecoreElt).setOperation(ecoreOp);
                }
            }
                
        }
    }

    @objid ("1179d30f-4583-11e0-b54c-00137279a832")
    private void setEvent(Element ecoreElt) {
        if (ecoreMessOccSpec != null)
            ecoreMessOccSpec.setEvent(((Event) ecoreElt));
    }

}
