/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessageEnd;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1179d312-4583-11e0-b54c-00137279a832")
public class MessageMapper extends BaseMapper {

    @Override
    @objid ("1179d313-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createMessage();
    }

    @objid ("117c3563-4583-11e0-b54c-00137279a832")
    public MessageMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("117c3567-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
        // we need to take the interaction on the send event
        IMessageEnd objingMess = ((IMessage) this.objingElement).getSendEvent();
        IInteraction interaction = null;
        interaction = getOwnerInteraction(objingMess);
        Interaction ecoreInteraction = (Interaction) genProp
        .getMappedElement(interaction);
        if (ecoreInteraction != null)
            ((Message) ecoreElt).setInteraction(ecoreInteraction);
    }

    @Override
    @objid ("117c356b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        // For Message Kind we don t have to set it, because it's determine by
        // the value of message sort
                
        setMessageSort((Message) ecoreElt);
        setArgument((Message) ecoreElt);
        setConnector((Message) ecoreElt);
    }

    @objid ("117e97bb-4583-11e0-b54c-00137279a832")
    private void setMessageSort(Message ecoreElt) {
        switch (((IMessage) objingElement).getSortOfMessage()) {
        case A_SYNC_CALL:
            ecoreElt.setMessageSort(MessageSort.ASYNCH_CALL_LITERAL);
            break;
        case CREATE_MESSAGE:
            ecoreElt.setMessageSort(MessageSort.CREATE_MESSAGE_LITERAL);
            break;
        case A_SYNC_SIGNAL:
            ecoreElt.setMessageSort(MessageSort.ASYNCH_SIGNAL_LITERAL);
            break;
        case DESTROY_MESSAGE:
            ecoreElt.setMessageSort(MessageSort.DELETE_MESSAGE_LITERAL);
            break;
        case SYNC_CALL:
            ecoreElt.setMessageSort(MessageSort.SYNCH_CALL_LITERAL);
            break;
        case RETURN_MESSAGE:
            ecoreElt.setMessageSort(MessageSort.REPLY_LITERAL);
            break;
        }
    }

    @objid ("117e97be-4583-11e0-b54c-00137279a832")
    private void setArgument(Message ecoreElt) {
        String argument = ((IMessage) objingElement).getArgument();
        if (argument != null && argument.trim().length() > 0) {
            LiteralString valueSpecification = UMLFactory.eINSTANCE
            .createLiteralString();
            valueSpecification.setValue(argument);
            ecoreElt.getArguments().add(valueSpecification);
        }
    }

    @objid ("117e97c1-4583-11e0-b54c-00137279a832")
    private void setConnector(Message ecoreElt) {
        ObjingModelNavigation.setConnector(ecoreElt, ((IMessage) objingElement));
    }

}
