/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IPartDecomposition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.PartDecomposition;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("117e97c4-4583-11e0-b54c-00137279a832")
public class PartDecompositionMapper extends InteractionUseMapper {

    @Override
    @objid ("117e97c5-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createPartDecomposition();
    }

    @objid ("1180fa17-4583-11e0-b54c-00137279a832")
    public PartDecompositionMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("1180fa1b-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        super.setProperties(ecoreElt);
        setDecomposedAs((PartDecomposition) ecoreElt);
    }

    @Override
    @objid ("1180fa1f-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        ((PartDecomposition) ecoreElt)
                .setEnclosingInteraction(getInteraction((IPartDecomposition) objingElement));
    }

    @objid ("11835c6e-4583-11e0-b54c-00137279a832")
    public static Interaction getInteraction(IPartDecomposition objingElement) {
        IInteraction objingInteraction = getIInteraction(objingElement);
        GenerationProperties genProp = GenerationProperties.getInstance();
        // if (genProp.isMappedElement(objingInteraction))
        return (Interaction) genProp.getMappedElement(objingInteraction);
        // else
        // return null;
    }

    @objid ("11835c73-4583-11e0-b54c-00137279a832")
    public static IInteraction getIInteraction(IPartDecomposition objingElement) {
        ILifeline objingLifeLine = objingElement.getDecomposed();
        return objingLifeLine.getOwner();
    }

    @objid ("11835c78-4583-11e0-b54c-00137279a832")
    private void setDecomposedAs(PartDecomposition ecorePartDec) {
        ILifeline objingLifeLine = ((IPartDecomposition) objingElement)
                .getDecomposed();
        GenerationProperties genProp = GenerationProperties.getInstance();
        Lifeline ecoreLifeline = (Lifeline) genProp
                .getMappedElement(objingLifeLine);
        if (ecoreLifeline != null) {
            ecoreLifeline.setDecomposedAs(ecorePartDec);
        }
    }

}
