/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.model.objing.sequence;

import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateInvariant;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.util.GenerationProperties;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.StateInvariant;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("118ce5db-4583-11e0-b54c-00137279a832")
public class StateInvariantMapper extends BaseMapper {

    @Override
    @objid ("118ce5dc-4583-11e0-b54c-00137279a832")
    public Element createEcoreElt() {
        return UMLFactory.eINSTANCE.createStateInvariant();
    }

    @objid ("118ce5e1-4583-11e0-b54c-00137279a832")
    public StateInvariantMapper(IElement param, MainMapper mapper) {
        super(param, mapper);
    }

    @Override
    @objid ("118f4833-4583-11e0-b54c-00137279a832")
    public void attach(Element ecoreElt) {
        attachInteractionFragment(ecoreElt);
    }

    @Override
    @objid ("118f4837-4583-11e0-b54c-00137279a832")
    public void setProperties(Element ecoreElt) {
        setInvariant((StateInvariant) ecoreElt);
    }

    @objid ("118f483b-4583-11e0-b54c-00137279a832")
    private void setInvariant(StateInvariant ecoreElt) {
        GenerationProperties genProp = GenerationProperties.getInstance();
                
        IStateInvariant objingStateInvariant = (IStateInvariant) objingElement;
        String stringConstraint = objingStateInvariant.getBody();
                
        // Constraint creation
        Constraint constraint = UMLFactory.eINSTANCE.createConstraint();
        // new LiteralString for constraint value
        LiteralString valueSpecification = UMLFactory.eINSTANCE
                .createLiteralString();
        valueSpecification.setValue(stringConstraint);
        constraint.setSpecification(valueSpecification);
                
        // Gets the Lifeline:
        ObList<ILifeline> lifelineList = objingStateInvariant.getCovered();
        ILifeline objingLL = null;
        if (lifelineList != null && lifelineList.size() > 0)
            objingLL = lifelineList.get(0);
                
        if (objingLL != null) {
            Lifeline ecoreLifeline = (Lifeline) genProp
                    .getMappedElement(objingLL);
                
            if (ecoreLifeline != null) {
                try {
                    constraint.getConstrainedElements().add(ecoreLifeline);
                } catch (ArrayStoreException e) {
                    Modelio.err
                            .println("ArrayStoreException when adding a Constraint on a Lifeline:");
                    e.printStackTrace();
                    constraint.getConstrainedElements().add(ecoreElt);
                } catch (Exception e) {
                    e.printStackTrace();
                    constraint.getConstrainedElements().add(ecoreElt);
                }
            }
        }
                
        // set the constraint
        ecoreElt.setInvariant(constraint);
    }

}
