/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.plugin;

import java.util.ResourceBundle;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.ui.i18n.BundledMessages;
import com.modeliosoft.modelio.xmi.api.ExportConfiguration;
import com.modeliosoft.modelio.xmi.api.ImportConfiguration;
import com.modeliosoft.modelio.xmi.impl.XMIImpl;

/**
 * XMI plugin central class.
 * 
 * This class listen to session to be notified when a project is opened saved or closed. It stores the modeling session
 * and make it available for actions.
 * 
 * This is currently a singleton.
 * 
 * @author ebrosse
 */
@objid ("119ff8b1-4583-11e0-b54c-00137279a832")
public class XMI {
    @objid ("11a25b01-4583-11e0-b54c-00137279a832")
    private static final XMI INSTANCE = new XMI();

    /**
     * Id of the XMI plugin
     */
    @objid ("fd70c40a-5c7a-11e0-adb5-00137279a832")
    public static final String PLUGIN_ID = "org.modelio.xmi.internal";

    /**
     * Bundle for the internationalization of XMI plugin
     */
    @objid ("a6e5f9c0-5d31-11e1-bceb-0027103f347d")
    public static final BundledMessages I18N = new BundledMessages(PLUGIN_ID,
                                                                   ResourceBundle.getBundle("xmi"));

    @objid ("83c14bff-5d6e-11e1-bceb-0027103f347d")
    private IModelingSession modelingSession;

    @objid ("6d621aa9-5ed0-11e1-9159-0027103f347d")
    private XMIImpl xmiservice = new XMIImpl();


    @objid ("11a25b04-4583-11e0-b54c-00137279a832")
    private XMI() {
    }

    /**
     * This method returns the XMI singleton
     * @return XMI Instance
     */
    @objid ("11a25b06-4583-11e0-b54c-00137279a832")
    public static XMI getInstance() {
        return INSTANCE;
    }

    /**
     * This method returns the current Modeling Session
     * @return the current Modeling session
     */
    @objid ("11a4bd5e-4583-11e0-b54c-00137279a832")
    public IModelingSession getModelingSession() {
        return this.modelingSession;
    }

    /**
     * This method sets the current Modeling session
     * @param session : the current modeling session
     */
    @objid ("83c14c00-5d6e-11e1-bceb-0027103f347d")
    public void sessionOpened(final IModelingSession session) {
        this.modelingSession = session;
    }

    /**
     * This methods is called when closing the modeling session
     */
    @objid ("83c17312-5d6e-11e1-bceb-0027103f347d")
    public void sessionClosed() {
        this.modelingSession = null;
    }

    /**
     * This service exports the given IPackage in the File xmiFile
     * @throws java.lang.Exception : the first occuring exception
     * @param configuration : the configuration of export process.
     */
    @objid ("6d643dc1-5ed0-11e1-9159-0027103f347d")
    public void exportXMIFile(final ExportConfiguration configuration) throws Exception {
        this.xmiservice.exportXMIFile(configuration);
    }

    /**
     * This service imports the Model saved in the given XMI file.
     * The import result will be owned by the given IPackage.
     * @throws java.lang.Exception : the first occuring exception
     * @param configuration : the configuration of import process.
     */
    @objid ("6d657661-5ed0-11e1-9159-0027103f347d")
    public void importXMIModel(final ImportConfiguration configuration) throws Exception {
        this.xmiservice.importXMIModel(configuration);
    }

    /**
     * This service imports the Profile saved in the given XMI file.
     * The import result will be owned by the given IModule.
     * @throws java.lang.Exception : the first occuring exception
     * @param configuration : the configuration of import process.
     */
    @objid ("6d65c48a-5ed0-11e1-9159-0027103f347d")
    public void importXMIProfile(final ImportConfiguration configuration) throws Exception {
        this.xmiservice.importXMIProfile(configuration);
    }

    /**
     * This service export the Profile saved in the given XMI file.
     * The export result will be owned by the given IModule.
     * @throws java.lang.Exception : the first Exception occurring during the export
     * @param configuration : the configuration of import process.
     */
    @objid ("2ce4c6f0-94ff-11e1-bc83-bc305ba4815c")
    public void exportXMIProfile(final ExportConfiguration configuration) throws Exception {
        this.xmiservice.exportXMIProfile(configuration);;
    }

}
