/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.plugin;

import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.app.events.ISessionListener;
import com.modeliosoft.modelio.core.jni.IProjectPropertiesService;
import com.modeliosoft.modelio.javadesigner.annotations.objid;

/**
 * Session Listener of the XMI plugin.
 * @author ebrosse
 */
@objid ("55387436-5d32-11e1-bceb-0027103f347d")
public class XMISessionListener implements ISessionListener {

    @Override
    @objid ("55389b40-5d32-11e1-bceb-0027103f347d")
    public void sessionClosed(final IModelingSession modelingSession) {
        XMI.getInstance().sessionClosed();
    }

    @Override
    @objid ("5538c248-5d32-11e1-bceb-0027103f347d")
    public void sessionOpened(final IModelingSession modelingSession) {
        XMI.getInstance().sessionOpened(modelingSession);
        initApplicationOptions();
    }

    @objid ("5538c24c-5d32-11e1-bceb-0027103f347d")
    private void initApplicationOptions() {
        IProjectPropertiesService propertiesService = XMI.getInstance().getModelingSession().getProjectPropertiesService();
        
        String param = propertiesService.readProperty("Modelio", "Ui.Parameter.ExportPath");
        if (param == null || param.equals("")) {
            propertiesService.writeProperty("Modelio", "Ui.Parameter.ExportPath", "");
        }
        
        param = propertiesService.readProperty("Modelio", "Ui.Parameter.ImportPath");
        if (param == null || param.equals("")) {
            propertiesService.writeProperty("Modelio", "Ui.Parameter.ImportPath", "");
        }
        
        param = propertiesService.readProperty("Modelio", "Ui.Parameter.CompatibilityActivated");
        
        if (param == null || param.equals("")) {
            propertiesService.writeProperty("Modelio", "Ui.Parameter.CompatibilityActivated", "true");
        }
        
        param = propertiesService.readProperty("Modelio", "Ui.Parameter.VersionExport");
        
        if (param == null || param.equals("")) {
            propertiesService.writeProperty("Modelio", "Ui.Parameter.VersionExport", "UML211");
        }
        
        param = propertiesService.readProperty("Modelio", "Ui.Parameter.FileExtension");
        
        if (param == null || param.equals("")) {
            propertiesService.writeProperty("Modelio", "Ui.Parameter.FileExtension", XMI.I18N.getString("Ui.Parameter.Extension.XMI"));
        }
        
        XMI.getInstance().getModelingSession().save();
    }

}
