/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.plugin.handlers;

import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.SwtWizardImportProfile;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * Handler of the  XMI "ImportProfile" command.
 * @author ebrosse
 */
@objid ("119b33f9-4583-11e0-b54c-00137279a832")
public class ImportProfile extends AbstractHandler {
    @objid ("119b33fa-4583-11e0-b54c-00137279a832")
     IModule selectedModule = null;


    @Override
    @objid ("119b33fb-4583-11e0-b54c-00137279a832")
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ReverseProperties.getInstance().initialize();
        ReverseProperties.getInstance().setProfileRoot(this.selectedModule);
        selectImportFile();
        return null;
    }

    @objid ("119d964d-4583-11e0-b54c-00137279a832")
    private void selectImportFile() {
        try {        
            Shell objing = null;
            if (!(System.getProperty("os.name").equals("Linux"))) {
                objing = Display.getDefault().getActiveShell();
            }
        
            final SwtWizardImportProfile dialog = new SwtWizardImportProfile(objing);
            dialog.setSelectedElt(this.selectedModule);
            dialog.open();
        } catch (final Exception e) {
            LOG.error(XMI.PLUGIN_ID, "Exception : ");
            LOG.error(XMI.PLUGIN_ID, e);
        }
    }

    @Override
    @objid ("119d964f-4583-11e0-b54c-00137279a832")
    public boolean isEnabled() {
        ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService().getSelection();
        if (selection instanceof IStructuredSelection){
            StructuredSelection structuredSelection = (StructuredSelection) selection;        
            if (structuredSelection.size() == 1)
                if (structuredSelection.getFirstElement() instanceof IAdaptable){
                    IAdaptable adaptable = (IAdaptable) structuredSelection.getFirstElement();
                    Object adapted = adaptable.getAdapter(IElement.class);
                    if(adapted instanceof IModule){
                        this.selectedModule = (IModule) adapted;
                        return true;
                    }
                }
        }
        return false;
    }

}
