/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.plugin.handlers;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.SwtWizardImport;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * Handler of the XMI "Import" command.
 * @author ebrosse
 */
@objid ("119d9654-4583-11e0-b54c-00137279a832")
public class ImportXMI extends AbstractHandler {
    @objid ("119d9655-4583-11e0-b54c-00137279a832")
    private IPackage selectedPackage;


    @Override
    @objid ("119d9656-4583-11e0-b54c-00137279a832")
    public Object execute(final ExecutionEvent event) throws ExecutionException {
        ReverseProperties.getInstance().initialize();
        selectImportFile();
        return null;
    }

    @objid ("119ff8aa-4583-11e0-b54c-00137279a832")
    private void selectImportFile() {
        Shell objing = null;
        if (!(System.getProperty("os.name").equals("Linux"))) {
            objing = Display.getDefault().getActiveShell();
        }
        ReverseProperties.getInstance().setUMLRoot(this.selectedPackage);
        final SwtWizardImport dialog = new SwtWizardImport(objing);
        dialog.setSelectedElt(this.selectedPackage);
        dialog.open();
    }

    @Override
    @objid ("119ff8ac-4583-11e0-b54c-00137279a832")
    public boolean isEnabled() {
        final ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService().getSelection();
        if (selection instanceof IStructuredSelection){
            final StructuredSelection structuredSelection = (StructuredSelection) selection;        
            if (structuredSelection.size() == 1)
                if (structuredSelection.getFirstElement() instanceof IAdaptable){
                    final IAdaptable adaptable = (IAdaptable) structuredSelection.getFirstElement();
                    final Object adapted = adaptable.getAdapter(IElement.class);
                    if(adapted instanceof IPackage){
                        this.selectedPackage = (IPackage) adapted;
                        return true;
                    }
                }
        }
        return false;
    }

}
