/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.preferences;

import com.modeliosoft.modelio.core.ui.ktable.editors.EditorFactory;
import com.modeliosoft.modelio.core.ui.ktable.renderers.RendererFactory;
import com.modeliosoft.modelio.core.ui.ktable.types.IPropertyType;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableDefaultModel;
import de.kupzog.ktable.renderers.DefaultCellRenderer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * @author ebrosse
 */
@objid ("6f39d682-1a76-11e1-8103-0027103f347d")
public class KTableModel extends KTableDefaultModel {
    @objid ("6f39fd95-1a76-11e1-8103-0027103f347d")
    private IPreferencesModel data = null;

    @objid ("6f39fd97-1a76-11e1-8103-0027103f347d")
    private Color oddColor = null;

    @objid ("6f3a24a6-1a76-11e1-8103-0027103f347d")
    private Color evenColor = null;

    @objid ("b3943ab7-5d66-11e1-bceb-0027103f347d")
    private KTable table = null;


    /**
     * @param table : the Ktable
     * @param dataModel : the current values
     */
    @objid ("6f3a24a7-1a76-11e1-8103-0027103f347d")
    public KTableModel(final KTable table, final IPreferencesModel dataModel) {
        this.table = table;
        
        this.data = dataModel;
        
        // Initialize colors
        this.oddColor = new Color(Display.getCurrent(), new RGB(245, 255, 255));
        this.evenColor = new Color(Display.getCurrent(), new RGB(230, 240, 255));
    }

    @Override
    @objid ("6f3a4bbc-1a76-11e1-8103-0027103f347d")
    public KTableCellEditor doGetCellEditor(final int col, final int row) {
        KTableCellEditor editor = null;
        
        // Row 0 is not editable
        if (row <= 0)
            return null;
        
        // Col 0 is not editable
        if (col <= 0)
            return null;
        
        if (!this.data.isEditable(row, col)) {
            return null;
        }
        
        final IPropertyType type = this.data.getTypeAt(row, col);
        
        if (type == null) {
            return null;
        }
        
        EditorFactory editorfactory = new EditorFactory();
        editor = editorfactory.getEditor(type);
        return editor;
    }

    @Override
    @objid ("6f3a99dc-1a76-11e1-8103-0027103f347d")
    public KTableCellRenderer doGetCellRenderer(final int col, final int row) {
        DefaultCellRenderer renderer = null;
        
        // Other rows depends on column
        RendererFactory rendererfactory = new RendererFactory(this.table);
        
        // Header row: return header renderer for all columns
        if (row == 0) {
            return rendererfactory.getHeaderRenderer();
        }
        
        if (col == 0) {
            renderer = rendererfactory.getLabelRenderer();
        } else {
            final IPropertyType type = this.data.getTypeAt(row, col);
        
            if (type == null) {
                return null;
            }
        
            renderer = rendererfactory.getRenderer(type);
        }
        
        renderer.setBackground((row % 2 == 0) ? this.oddColor : this.evenColor);
        return renderer;
    }

    @Override
    @objid ("6f3ac0f1-1a76-11e1-8103-0027103f347d")
    public int doGetColumnCount() {
        return 2;
    }

    @Override
    @objid ("6f3ae802-1a76-11e1-8103-0027103f347d")
    public Object doGetContentAt(final int col, final int row) {
        final Object value = this.data.getValueAt(row, col);
        
        if(row == 0 || col == 0)        
            return XMI.I18N.getString(value.toString());
        else if(row == 1)
            return new Boolean(value.toString());
        
        else return value;
    }

    @Override
    @objid ("6f3b0f16-1a76-11e1-8103-0027103f347d")
    public int doGetRowCount() {
        return this.data.getRowsNumber();
    }

    @Override
    @objid ("6f3b3626-1a76-11e1-8103-0027103f347d")
    public void doSetContentAt(final int col, final int row, final Object value) {
        // Row 0 is not editable
        if (row == 0)
            return;
        
        // Col 0 is not editable
        if (col == 0)
            return;
        
        this.data.setValueAt(row, col, value);
        
        this.table.redraw();
    }

    @Override
    @objid ("6f3b8448-1a76-11e1-8103-0027103f347d")
    public int getInitialColumnWidth(final int column) {
        final double firstColumnWeight = 0.35;
        
        final int colCount = 2;
        final int availableWidth = this.table.getClientArea().width - 18;
        
        if (column == 0)
            return (int) (availableWidth * firstColumnWeight);
        else
            return (int) ((availableWidth) * (1 - firstColumnWeight)) / (colCount - 1);
    }

    @Override
    @objid ("6f3bab5a-1a76-11e1-8103-0027103f347d")
    public int getInitialRowHeight(final int row) {
        return (row == 0) ? 22 : 18;
    }

    @Override
    @objid ("6f3bd26d-1a76-11e1-8103-0027103f347d")
    public int getFixedHeaderColumnCount() {
        return 0;
    }

    @Override
    @objid ("6f3bf97e-1a76-11e1-8103-0027103f347d")
    public int getFixedHeaderRowCount() {
        return 1;
    }

    @Override
    @objid ("6f3bf983-1a76-11e1-8103-0027103f347d")
    public int getFixedSelectableColumnCount() {
        return 0;
    }

    @Override
    @objid ("6f3c2094-1a76-11e1-8103-0027103f347d")
    public int getFixedSelectableRowCount() {
        return 0;
    }

    @Override
    @objid ("6f3c47a5-1a76-11e1-8103-0027103f347d")
    public int getRowHeightMinimum() {
        return 10;
    }

    @Override
    @objid ("6f3c6eb6-1a76-11e1-8103-0027103f347d")
    public boolean isColumnResizable(final int col) {
        return true;
    }

    @Override
    @objid ("6f3c95c9-1a76-11e1-8103-0027103f347d")
    public void setColumnWidth(final int col, final int value) {
        super.setColumnWidth(col, value);
    }

    @Override
    @objid ("6f3ce3ea-1a76-11e1-8103-0027103f347d")
    public boolean isRowResizable(final int row) {
        return false;
    }

    @Override
    @objid ("6f3d0afd-1a76-11e1-8103-0027103f347d")
    public int getColumnWidth(final int col) {
        return super.getColumnWidth(col);
    }

    /**
     * @param row : the desired row
     * @return the desription associated to the row
     */
    @objid ("6f3d3210-1a76-11e1-8103-0027103f347d")
    public String doGetDescription(final int row) {
        final String descriptionKey = this.data.getDescriptionKey(row); 
        if (descriptionKey != null) {
            return  XMI.I18N.getString(descriptionKey);
        } else {
            return "";
        }
    }

}
