/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.preferences;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.ui.ktable.types.BooleanType;
import com.modeliosoft.modelio.core.ui.ktable.types.IPropertyType;
import com.modeliosoft.modelio.core.ui.ktable.types.ListStringType;
import com.modeliosoft.modelio.core.ui.ktable.types.StringType;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

/**
 * @author ebrosse
 */
@objid ("6f4a792c-1a76-11e1-8103-0027103f347d")
public class XmiPreferenceModel implements IPreferencesModel {
    @objid ("6f4aa03f-1a76-11e1-8103-0027103f347d")
    private List<PreferenceData> data = null;

    @objid ("947b5978-4734-11e1-947a-0027103f347d")
    private StringType labelStringType = null;

    @objid ("947b5979-4734-11e1-947a-0027103f347d")
    private ListStringType compatibilityListType = null;

    @objid ("947b597a-4734-11e1-947a-0027103f347d")
    private ListStringType extensionListType = null;

    @objid ("947b80a6-4734-11e1-947a-0027103f347d")
    private BooleanType checkType = null;


    /**
     * @param modelingSession : the modelio modelling Sessions
     * @param data : all data
     */
    @objid ("6f4b1575-1a76-11e1-8103-0027103f347d")
    public XmiPreferenceModel(final IModelingSession modelingSession, final List<PreferenceData> data) {
        this.data = data;
        this.labelStringType = new StringType(false);
        this.checkType = new BooleanType();
        final List<String> compatibilityItems = getDefaultCompatibilities();
        this.compatibilityListType = new ListStringType(false, compatibilityItems);
               
        final List<String> extensionItems = getDefaultExtensions();
        this.extensionListType = new ListStringType(false, extensionItems);
    }

    @objid ("6f4b3c8b-1a76-11e1-8103-0027103f347d")
    private List<String> getDefaultExtensions() {
        final List<String> defaultExtensions = new ArrayList<String>();
        defaultExtensions.add( XMI.I18N.getString("Ui.Parameter.Extension.XMI"));
        defaultExtensions.add( XMI.I18N.getString("Ui.Parameter.Extension.UML"));
        return defaultExtensions;
    }

    @objid ("6f4b639d-1a76-11e1-8103-0027103f347d")
    private List<String> getDefaultCompatibilities() {
        final List<String> defaultCompatibilities = new ArrayList<String>();
        defaultCompatibilities.add("UML211");
        defaultCompatibilities.add("UML22");
        defaultCompatibilities.add("UML23");
        defaultCompatibilities.add("UML241");
        defaultCompatibilities.add("EMF300");
        return defaultCompatibilities;
    }

    @Override
    @objid ("6f4b8aaf-1a76-11e1-8103-0027103f347d")
    public int getColumnNumber() {
        return 2;
    }

    @Override
    @objid ("6f4bb1bf-1a76-11e1-8103-0027103f347d")
    public String getDescriptionKey(final int row) {
        if (row > 0) {
            return this.data.get(row).getKey() + ".Description";
        } else {
            return null;
        }
    }

    @Override
    @objid ("6f4bd8d0-1a76-11e1-8103-0027103f347d")
    public int getRowsNumber() {
        return this.data.size();
    }

    @Override
    @objid ("6f4bffe1-1a76-11e1-8103-0027103f347d")
    public IPropertyType getTypeAt(final int row, final int col) {
        switch (col) {
        case 0: // col 0 is the property key type
            return this.labelStringType;
        case 1: // col 1 is the property value type
            switch (row) {
            case 0: // Header
                return this.labelStringType;
            case 1:
                return this.checkType;
            case 2:
                return this.compatibilityListType;
            case 3:
                return this.extensionListType;
            default:
                return null;
            }
        default:
            return null;
        }
    }

    @Override
    @objid ("6f4c26f6-1a76-11e1-8103-0027103f347d")
    public Object getValueAt(final int row, final int col) {
        switch (col) {
        case 0: // col 0 is the property key type
            return this.data.get(row).getKey() + ".Label";
        case 1: // col 1 is the property value type
            if (row == 0) {
                return this.data.get(row).getValue() + ".Value";
            } else {
                return this.data.get(row).getValue();
            }
        default:
            return null;
        }
    }

    @Override
    @objid ("6f4c7516-1a76-11e1-8103-0027103f347d")
    public boolean isEditable(final int row, final int col) {
        switch (col) {
        case 0: // col 0 is the property key type
            return false;
        case 1: // col 1 is the property value type
            switch (row) {
            case 0: // Header
                return false;
            case 1:
                return true;
            case 2:
                return true;
            case 3:
                return true;
            default:
                return false;
            }
        default:
            return false;
        }
    }

    @Override
    @objid ("6f4c9c2b-1a76-11e1-8103-0027103f347d")
    public void setValueAt(final int row, final int col, final Object value) {
        switch (col) {
        case 0: // col 0 is the property key type
            return;
        case 1: // col 1 is the property value type
            if (row == 1) {
                this.data.get(row).setValue(value.toString());
            }
            else if(row != 0)
                this.data.get(row).setValue((String) value);
            return;
        default:
            return;
        }
    }

}
