/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.preferences;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.ui.UIColor;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.jni.IProjectPropertiesService;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellSelectionListener;
import de.kupzog.ktable.SWTX;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author ebrosse
 */
@objid ("6f4cea4c-1a76-11e1-8103-0027103f347d")
public class XmiPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {
    @objid ("6f4cea4e-1a76-11e1-8103-0027103f347d")
     KTableModel preferencesModel = null;

    @objid ("6f4d115e-1a76-11e1-8103-0027103f347d")
    private List<PreferenceData> data = null;

    @objid ("6f4d1161-1a76-11e1-8103-0027103f347d")
    private IModelingSession modelingSession = null;

    @objid ("6f4d3870-1a76-11e1-8103-0027103f347d")
     Text descriptionText = null;

    @objid ("6f4d3871-1a76-11e1-8103-0027103f347d")
    private ScrolledComposite scrolledComposite = null;

    @objid ("e50d562d-4735-11e1-947a-0027103f347d")
    private KTable table = null;


    /**
     * The default contructor
     */
    @objid ("6f4d3872-1a76-11e1-8103-0027103f347d")
    public XmiPreferencePage() {
        super();
        noDefaultAndApplyButton();
        this.modelingSession = XMI.getInstance().getModelingSession();
    }

    /**
     * @param title : title of the page
     */
    @objid ("6f4d8694-1a76-11e1-8103-0027103f347d")
    public XmiPreferencePage(final String title) {
        super(title);
        noDefaultAndApplyButton();
        this.modelingSession = XMI.getInstance().getModelingSession();
    }

    /**
     * @param title : title of the page
     * @param image : icon of the page
     */
    @objid ("6f4dada7-1a76-11e1-8103-0027103f347d")
    public XmiPreferencePage(final String title, final ImageDescriptor image) {
        super(title, image);
        noDefaultAndApplyButton();
        this.modelingSession = XMI.getInstance().getModelingSession();
    }

    @Override
    @objid ("6f4dfbcc-1a76-11e1-8103-0027103f347d")
    protected Control createContents(final Composite parent) {
        createTableViewer(parent);
        createDescription(parent);
        return null;
    }

    @objid ("6f4e22df-1a76-11e1-8103-0027103f347d")
    private void createTableViewer(final Composite parent) {
        final int tableStyle = SWTX.AUTO_SCROLL | SWTX.FILL_WITH_LASTCOL | SWT.BORDER | SWT.FULL_SELECTION;
        this.table = new KTable(parent, tableStyle);
        final GridData tableData = new GridData(SWT.FILL, SWT.FILL, true, true);
        this.table.setLayoutData(tableData);
        
        initPreferences();
        
        this.preferencesModel = new KTableModel(this.table, new XmiPreferenceModel(this.modelingSession, this.data));
        
        this.table.setModel(this.preferencesModel);
        
        this.table.setSelection(0, 1, true);
        
        this.table.addCellSelectionListener(new TableSelectionListener());
    }

    @objid ("6f4e49ee-1a76-11e1-8103-0027103f347d")
    private void createDescription(final Composite parent) {
        this.scrolledComposite = new ScrolledComposite(parent, SWT.V_SCROLL);
        
        final GridLayout gl_scrolledComposite = new GridLayout(1, false);
        this.scrolledComposite.setLayout(gl_scrolledComposite);
        
        this.descriptionText = new Text(this.scrolledComposite, SWT.WRAP | SWT.MULTI);
        this.descriptionText.setEditable(false);
        this.descriptionText.setForeground(UIColor.LABEL_TIP_FG);
        final GridData gd_descriptionText = new GridData(SWT.FILL, SWT.FILL, true, true);
        this.descriptionText.setLayoutData(gd_descriptionText);
        
        this.scrolledComposite.setContent(this.descriptionText);
        
        final Point point = parent.getSize();
        this.scrolledComposite.setMinSize(point);
        this.scrolledComposite.setExpandVertical(true);
        this.scrolledComposite.setExpandHorizontal(true);
        final GridData gd_scrolledComposite = new GridData(SWT.FILL, SWT.BOTTOM, true, false);
        gd_scrolledComposite.heightHint = 80;
        gd_scrolledComposite.widthHint = 400;
        this.scrolledComposite.setLayoutData(gd_scrolledComposite);
        
        this.table.addCellSelectionListener(new TableSelectionListener());
    }

    /**
     * Initialize the preference page
     */
    @objid ("6f4e49f2-1a76-11e1-8103-0027103f347d")
    public void initPreferences() {
        final IProjectPropertiesService propertiesService = this.modelingSession.getProjectPropertiesService();
        
        this.data = new ArrayList<PreferenceData>();
        
        String key = null;
        String value = null;
        
        key = "Preferences.Option";
        value = "Preferences.Option";
        this.data.add(new PreferenceData(key, value));
        
        key = "Ui.Parameter.CompatibilityActivated";
        value = propertiesService.readProperty("Modelio", key);
        this.data.add(new PreferenceData(key, value));
        
        key = "Ui.Parameter.VersionExport";
        value = propertiesService.readProperty("Modelio", key);
        this.data.add(new PreferenceData(key, value));
        
        key = "Ui.Parameter.FileExtension";
        value = propertiesService.readProperty("Modelio", key);
        this.data.add(new PreferenceData(key, value));
    }

    @Override
    @objid ("6f4e7100-1a76-11e1-8103-0027103f347d")
    public boolean performOk() {
        final IProjectPropertiesService propertiesService = this.modelingSession.getProjectPropertiesService();
        
        String key = null;
        String value = null;
        
        for (final PreferenceData pref : this.data) {
            key = pref.getKey();
            value = pref.getValue();           
            propertiesService.writeProperty("Modelio", key, value);
        }
        return super.performOk();
    }

    @Override
    @objid ("6f4e9812-1a76-11e1-8103-0027103f347d")
    public void init(final IWorkbench workbench) {
    }

    @objid ("6f4e9817-1a76-11e1-8103-0027103f347d")
    private class TableSelectionListener implements KTableCellSelectionListener {

        @Override
        @objid ("6f4ebf24-1a76-11e1-8103-0027103f347d")
        public void cellSelected(final int col, final int row, final int statemask) {
            updateDescription(row);
        }

        @Override
        @objid ("6f4ee63a-1a76-11e1-8103-0027103f347d")
        public void fixedCellSelected(final int col, final int row, final int statemask) {
            updateDescription(row);
        }

        @objid ("6f4f345e-1a76-11e1-8103-0027103f347d")
        private void updateDescription(final int row) {
            final String description = XmiPreferencePage.this.preferencesModel.doGetDescription(row);
            if (description != null) {
                XmiPreferencePage.this.descriptionText.setText(description);
            } else {
                XmiPreferencePage.this.descriptionText.setText("");
            }
        }

    }

}
