/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  

package com.modeliosoft.modelio.xmi.reverse;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.exceptions.InvalidModelManipulationException;
import com.modeliosoft.modelio.api.utils.exceptions.ObElementNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.model.ecore.IEElement;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import com.modeliosoft.modelio.xmi.util.ObjingEAnnotation;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMILogs;

@objid ("13de8f65-4583-11e0-b54c-00137279a832")
public class ImportModel {
	@objid ("13de8f66-4583-11e0-b54c-00137279a832")
	private ProgressBarComposite progressBar = null;

	@objid ("13e0f1bb-4583-11e0-b54c-00137279a832")
	private boolean external = false;




	@objid ("13e0f1bc-4583-11e0-b54c-00137279a832")
	public void importXMI(IEElement ieelem) {
		PartialImportMap partialCreationMap = PartialImportMap.getInstance();
		TotalImportMap totalCreationMap = TotalImportMap.getInstance();
		boolean isNamespace = false;

		Element ecoreElt = ieelem.getEcoreElement();

		if (ecoreElt instanceof Namespace)
			isNamespace = true;

		Object objingElt = totalCreationMap.get(ecoreElt);

		if (objingElt == null) {
			objingElt = partialCreationMap.get(ecoreElt);

			if (objingElt == null &&
					isMapperOfCurrentElt(getCurrentClassName(ieelem), ieelem.getEcoreElement())) {
				// Creation:

				//Galaxy sera
				objingElt = ObjingModelNavigation.searchElement(ieelem.getEcoreElement());

				if (objingElt == null) {
					objingElt = ieelem.createObjingElt();
				}
				
				if (objingElt != null) {
					partialCreationMap.put(ecoreElt, objingElt);
				}

				if (isNamespace && this.progressBar != null) {
					this.progressBar.addValue();
					this.progressBar.addElement();
				}
			}

			if (objingElt != null) {

				// Finish creation:
				setMapping(objingElt, ieelem);

				partialCreationMap.remove(ecoreElt);
				totalCreationMap.put(ecoreElt, objingElt);
				if (this.progressBar != null) {
					this.progressBar.addElement();
					if (isNamespace) {
						this.progressBar.addValue();
					}
				}

			}
		}
	}

	@objid ("13e35415-4583-11e0-b54c-00137279a832")
	public ImportModel(ProgressBarComposite progressBar) {
		this.progressBar = progressBar;
	}

	@objid ("13e35418-4583-11e0-b54c-00137279a832")
	private void setMapping(Object objingElt, IEElement ieelem) {
		XMILogs log = XMILogs.getInstance();
		try {
			if ((objingElt != null) &&
					(objingElt instanceof IElement) &&
					(((IElement) objingElt).getElementStatus().isModifiable())) {
				if (this.external) {
					if (objingElt instanceof IModelTree) {
						((IModelTree) objingElt).setOwner(ReverseProperties.getInstance()
								.getExternalPackage());
						this.external = false;
					}
				} else {
					ieelem.attach((IElement) objingElt);
				}
				ieelem.setProperties((IElement) objingElt);
				ieelem.setStereotypes();

			} else if (objingElt instanceof ArrayList) {
				ieelem.attach((ArrayList<Object>) objingElt);
				for (Object elt : (ArrayList<IElement>) objingElt) {
					if (((IElement) elt).getElementStatus().isModifiable()) {
						ieelem.setProperties((IElement) elt);
						ieelem.setStereotypes();
					}
				}
			}

		} catch (ObElementNotFoundException e) {

			String eltName = "";
			if (ieelem.getEcoreElement() instanceof NamedElement)
				eltName = ((NamedElement) ieelem.getEcoreElement()).getName();

			log.writelnInLog(XMI.I18N.getMessage("logFile.warning.import.unknowObjingElement",
					eltName,
					ieelem.getEcoreElement().getClass().getCanonicalName()));

		} catch (InvalidModelManipulationException e) {
			if ((objingElt != null) && (objingElt instanceof IModelElement)) {
				String eltName = ((IModelElement) objingElt).getName();
				log.writelnInLog(XMI.I18N.getMessage("logFile.warning.import.invalidObjingModelManipulation",
						eltName,
						objingElt.getClass().getCanonicalName()));
			}
		} catch (RuntimeException e) {

			if ("Joni Element Not Found".equals(e.getMessage())) {
				PartialImportMap.getInstance().remove(ieelem.getEcoreElement());
				e.printStackTrace(Modelio.err);
				log.writelnInLog(e.getMessage());
			} else {
				e.printStackTrace(Modelio.err);
				log.writelnInLog(e.getMessage());
			}

		}
	}

	@objid ("13e3541c-4583-11e0-b54c-00137279a832")
	private String getCurrentClassName(IEElement ieelem) {
		String name = ieelem.getClass().getSimpleName();
		return name.substring(1, name.length());
	}

	@objid ("13e5b66e-4583-11e0-b54c-00137279a832")
	public ImportModel() {
	}

	@objid ("13e5b674-4583-11e0-b54c-00137279a832")
	private boolean isMapperOfCurrentElt(String mapperName, Element ecoreElement) {
		String ecoreEltName = ecoreElement.getClass().getSimpleName();

		if (ecoreEltName.contains(mapperName))
			return true;
		return false;
	}

}
