/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.reverse;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.core.IModelingSession;
import com.modeliosoft.modelio.core.jni.ITransaction;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.EcoreModelNavigation;
import com.modeliosoft.modelio.xmi.util.ResourceLoader;
import com.modeliosoft.modelio.xmi.util.ReverseProperties;
import com.modeliosoft.modelio.xmi.util.XMIFileUtils;
import com.modeliosoft.modelio.xmi.util.XMILogs;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.impl.PackageImpl;
import org.eclipse.uml2.uml.internal.impl.ProfileImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

/**
 * This class provides all import services
 * @author ebrosse
 */
@objid ("13e818ce-4583-11e0-b54c-00137279a832")
public class ImportServices {
    @objid ("13e818cf-4583-11e0-b54c-00137279a832")
    private int ownedElements = 0;

    @objid ("9fcb5abc-1a76-11e1-8103-0027103f347d")
    private List<String> profiles = new ArrayList<String>();

    @objid ("35a4fdc6-94ff-11e1-bc83-bc305ba4815c")
    private Shell _shell = null;


    @objid ("13e818d1-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel(File xmiFile) {
        // Create a resource set.
        ResourceSet resourceSet = new ResourceSetImpl();
        // Register the default resource factory -- only needed for
        // stand-alone!
        resourceSet.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet.getURIConverter().getURIMap();
        
        URI uri = URI.createURI(ResourceLoader.getInstance().getUMLRessource());
        
        URIConverter.URI_MAP.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        URIConverter.URI_MAP.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        URIConverter.URI_MAP.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        URI fileURI = URI.createFileURI(xmiFile.getAbsolutePath());
        
        Resource result = null;
        
        try{
            result = resourceSet.getResource(fileURI, true);
        }catch(Exception e){
        
            deleteUnresolvedReference(xmiFile, fileURI, resourceSet );
        
            result = resourceSet.getResource(fileURI, false);
        
            //ecriture du log
            if (result != null){
                XMILogs xmilogs = XMILogs.getInstance();
        
                for (Resource.Diagnostic diagnostic : result.getErrors()){
                    xmilogs.writelnInLog(XMI.I18N.getMessage("info.import.emf.error", diagnostic.getMessage()));
                }
        
            }
        }
        return result;
    }

    @objid ("13ea7b28-4583-11e0-b54c-00137279a832")
    private void attachExternalElements(OwnedCompositionUml2Visitor visitEcoreModel) {
        PartialImportMap partialImportMap = PartialImportMap.getInstance();
        TotalImportMap totalImportMap = TotalImportMap.getInstance();
        
        List<Element> externalElements = new ArrayList<Element>();
        
        for (Object key : partialImportMap.keySet()) {
            if (key != null){
                if (key instanceof Element) {
                    externalElements.add((Element) key);
                }
            }else{
                partialImportMap.remove(key);
            }
        }
        
        for (Element externalElt : externalElements) {  
            if (EcoreModelNavigation.isModelChild(externalElt)){
                if (externalElt instanceof Model){        
                    totalImportMap.put(externalElt, partialImportMap.remove(externalElt));
                } else if (totalImportMap.get(externalElt) != null) {
                    partialImportMap.remove(externalElt);
                } else if (externalElt instanceof PrimitiveType) {
                    attachExternalPrimitiveType((PrimitiveType) externalElt, partialImportMap, totalImportMap);        
                } else {
                    visitEcoreModel.doSwitch(externalElt);
        
                }
            }else{
        
                Object result = partialImportMap.get(externalElt);
                if (result != null){
                    if (result instanceof IElement){ 
                        if(((IElement) result).getElementStatus().isModifiable()){
                            Modelio.getInstance().getModelingSession().getModel().deleteElement((IElement) result);
                        }
                    }else{
                        for (IElement elt: (List<IElement>) result){
                            if(elt.getElementStatus().isModifiable()){
                                Modelio.getInstance().getModelingSession().getModel().deleteElement(elt);
                            }
                        }
                    }
                }
            }
            partialImportMap.remove(externalElt);
        }
        
        if (partialImportMap.size() != 0) {
            attachExternalElements(visitEcoreModel);
        }
    }

    /**
     * @throws Exception
     * @param xmiFile @return
     */
    @objid ("13ea7b2b-4583-11e0-b54c-00137279a832")
    public Resource getResource(File xmiFile) throws Exception {
        //load file
        if (XMIFileUtils.containsPatterns(xmiFile, getOldUMLPatterns())){
            return loadModel(xmiFile, getOldUMLPatterns(), getNewUMLPatterns());
        }else{
            return loadModel(xmiFile, getOldEMFPatterns(), getNewEMFPatterns());
        }
    }

    /**
     * @param resource : the Ecore resource
     * @param progressBar : the progressBar
     * @param shell : the parent Shell
     * @return false if any errors occurs
     */
    @objid ("13ea7b30-4583-11e0-b54c-00137279a832")
    public boolean importEcoreModel(Resource resource, ProgressBarComposite progressBar, Shell shell) {
        boolean error = false;
        
        this._shell = shell;
        
        if (resource != null) {
        
            boolean isAEcoreModel = isEcoreModel(resource);
        
            if (isAEcoreModel) {
                importModel(progressBar);
                cleanResource(resource);
            } else {
                error = true;
        
                Display.getDefault().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        MessageBox messageBox = new MessageBox(_shell, SWT.ICON_INFORMATION);
                        messageBox.setMessage(XMI.I18N.getString("info.import.result_failed.notModel"));
                        messageBox.setText(XMI.I18N.getString("info.import.result_failed.notModel"));
                        messageBox.open();
                        _shell.dispose();
                    }
                });
        
                Modelio.err.println(XMI.I18N
                                    .getString("info.import.result_failed.notModel"));
            }
        
            resource.unload();
        
        }
        return error;
    }

    /**
     * @param model
     * @param progressBar
     * @param shell
     * @param session
     * @param t @return
     */
    @objid ("13ecdd7d-4583-11e0-b54c-00137279a832")
    public boolean importEcoreModel(Model model, ProgressBarComposite progressBar, Shell shell, IModelingSession session, ITransaction t) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        TotalImportMap totalImportMap = TotalImportMap.getInstance();
        boolean error = false;
        
        revProp.addEcoreModel(model);
        
        revProp.storeImportModelRoundtripMode(model);
        
        revProp.setOpenedTransaction(t);
        XMIImportBehavior importBehavior = new XMIImportBehavior(
                                                                 progressBar);
        OwnedCompositionUml2Visitor visitEcoreModel = new OwnedCompositionUml2Visitor(
                                                                                      importBehavior, model);
        revProp.setEcoreVisitor(visitEcoreModel);
        
        IPackage objingTechnicalRoot = revProp.getUMLRoot();
        
        totalImportMap.put(model, objingTechnicalRoot);
        
        // Launch visit of the model:
        visitEcoreModel.doSwitch(model);
        
        progressBar.setLabel(XMI.I18N
                             .getString("progressBar.content.import.checkConsistency"));
        
        attachExternalElements(visitEcoreModel);
        cleanExternalElements();
        
        // Close opened sequence diagrams:
        revProp.closeSequenceDiagrams();
        
        error = session.getAudit().auditTransaction(t);
        
        progressBar.addFinalValue();
        
        
        if (!error) {
            Modelio.err.println(XMI.I18N
                                .getString("info.import.result_done"));
        }
        return error;
    }

    @objid ("13f1a230-4583-11e0-b54c-00137279a832")
    private int initProgressBar(Package modelToImport) {
        this.ownedElements = 0;
        
        for (Object elem : modelToImport.getOwnedMembers()) {
            if (elem instanceof Namespace)
                countOwnedElements((Namespace) elem);
        }
        return this.ownedElements * 2;
    }

    @objid ("13f1a235-4583-11e0-b54c-00137279a832")
    private void countOwnedElements(Namespace elem) {
        this.ownedElements++;
        for (Object elt : elem.getOwnedElements()) {
            if (elt instanceof Namespace)
                countOwnedElements((Namespace) elt);
        }
    }

    @objid ("13f1a238-4583-11e0-b54c-00137279a832")
    private void attachExternalPrimitiveType(PrimitiveType externalElt, PartialImportMap partialImportMap, TotalImportMap totalImportMap) {
        IDataType temp = (IDataType) partialImportMap.remove(externalElt);
        
        if (temp.getOwner() == null){
            temp.setElementary(true);
            temp.setName(externalElt.getName());
        
            IPackage root = ReverseProperties.getInstance().getExternalPackage();
            root.addOwnedElement(temp);
            temp.setOwner(root);
        
            totalImportMap.put(externalElt, partialImportMap.remove(externalElt));
        }
    }

    /**
     * This Method export a given profile in a given Ecore Resource
     * @param resource : the Ecore Resource
     * @param root : the root of the export
     * @param progressBar : the progressBar
     * @param shell : the current shell
     * @return true if no error happen
     */
    @objid ("13f1a23d-4583-11e0-b54c-00137279a832")
    public boolean importEcoreProfile(Resource resource, IElement root, ProgressBarComposite progressBar, Shell shell) {
        boolean error = false;
        
        this._shell = shell;
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        Modelio.getInstance().getModelingSession();
        
        if (resource != null) {
            ProfileImpl profileToImport = null;
            boolean isAProfile = false;
        
            for (Object elem : resource.getContents()) {
                if (elem.getClass().getSimpleName().compareTo("ProfileImpl") == 0) {
                    isAProfile = true;
                    profileToImport = (ProfileImpl) elem;
                    revProp.setEcoreProfile(profileToImport);
                }
                if (elem.getClass().getSimpleName().compareTo("ModelImpl") == 0) {
                    for (Object packaged : ((Model) elem).getPackagedElements()) {
                        if (packaged.getClass().getSimpleName().compareTo("ProfileImpl") == 0) {
                            isAProfile = true;
                            profileToImport = (ProfileImpl) packaged;
                            revProp.setEcoreProfile(profileToImport);
                        }
                    }
                }
            }    
        
            if (isAProfile) {
        
                revProp.getOpenedTransaction();
        
                revProp.storeImportModelRoundtripMode(profileToImport);
        
                revProp.addEcoreModel(profileToImport);
        
                if(progressBar != null) progressBar.setNumberElement(initProgressBar(profileToImport));
        
                XMIImportBehavior importBehavior = new XMIImportBehavior(
                                                                         progressBar);
                OwnedCompositionUml2Visitor visitEcoreModel = new OwnedCompositionUml2Visitor(
                                                                                              importBehavior, profileToImport);
                revProp.setEcoreVisitor(visitEcoreModel);
        
                visitEcoreModel.doSwitch(profileToImport); 
        
                if(progressBar != null)  progressBar.setLabel(XMI.I18N
                                     .getString("progressBar.content.import.checkConsistency"));
        
            } else {
                error = true;
        
                Display.getDefault().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        MessageBox messageBox = new MessageBox(_shell, SWT.ICON_INFORMATION);
                        messageBox.setMessage(XMI.I18N.getString("info.import.result_failed.notProfile"));
                        messageBox.setText(XMI.I18N.getString("info.import.result_failed.notProfile"));
                        messageBox.open();
                        _shell.dispose();
                    }
                });
        
                Modelio.err.println(XMI.I18N
                                    .getString("info.import.result_failed.notProfile"));
            }
        
        }
        return error;
    }

    @objid ("13f666e9-4583-11e0-b54c-00137279a832")
    private boolean isEcoreModel(Resource resource) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        
        boolean result = false;
        
        for (Object elem : resource.getContents()) {
        
            String className = elem.getClass().getSimpleName();
            if (className.equals("ModelImpl") || className.equals("PackageImpl")) {
                revProp.addEcoreModel((PackageImpl) elem);
                result = true;      
            }
        
        }
        return result;
    }

    @objid ("13f8c946-4583-11e0-b54c-00137279a832")
    private void cleanResource(Resource resource) {
        File tempFolder = ResourceLoader.getInstance().getXMITempFolder();
        tempFolder.delete();
        resource.unload();
    }

    @objid ("13f8c949-4583-11e0-b54c-00137279a832")
    private void importModel(ProgressBarComposite progressBar) {
        ReverseProperties revProp = ReverseProperties.getInstance();
        PartialImportMap.getInstance().clear();
        TotalImportMap.getInstance().clear();
        
        for(Package modelToImport : revProp.getEcoreModel()){
        
            revProp.storeImportModelRoundtripMode(modelToImport);
        
            if(progressBar != null) progressBar.setNumberElement(initProgressBar(modelToImport));
        
            XMIImportBehavior importBehavior = new XMIImportBehavior(
                                                                     progressBar);
            OwnedCompositionUml2Visitor visitEcoreModel = new OwnedCompositionUml2Visitor(
                                                                                          importBehavior, modelToImport);
            revProp.setEcoreVisitor(visitEcoreModel);
        
            IPackage objingTechnicalRoot = revProp.getUMLRoot();
            TotalImportMap.getInstance().put(modelToImport, objingTechnicalRoot);
            visitEcoreModel.doSwitch(modelToImport); 
        
            attachExternalElements(visitEcoreModel);
        
        }
        
        if(progressBar != null) progressBar.setLabel(XMI.I18N.getString("progressBar.content.import.checkConsistency"));
        
        // Close opened sequence diagrams:
        revProp.closeSequenceDiagrams();
        
        // To finish when it.next working !
        
        if(progressBar != null) progressBar.addFinalValue();
        
        revProp.clean();
    }

    @objid ("13fb2b98-4583-11e0-b54c-00137279a832")
    private void deleteUnresolvedReference(File currentfile, URI fileURI2, ResourceSet resourceSet2) {
        Resource result = resourceSet2.getResource(fileURI2, false);    
        
        if ((result != null) && (result.getErrors() != null)&& (result.getErrors().size() != 0)){
        
            XMIFileUtils.removeReferences(currentfile.getAbsolutePath(),result.getErrors());
        
        }
    }

    @objid ("7091fe10-1a76-11e1-8103-0027103f347d")
    private Resource loadModel(final File oldFile, final List<String> oldPatterns, final List<String> newPatterns) {
        File newFile = null;
        File tempFile = null;
        List<File> listeFile = new ArrayList<File>();
        Resource newresource = null; 
        
        try{
            newFile = updateFile(oldFile, oldPatterns, newPatterns);
        
            this.profiles = XMIFileUtils.getAllAppliedProfiles(oldFile.getAbsolutePath());
        
            String directory = oldFile.getParent();
        
            for (String profile : this.profiles){
                try{
                    tempFile = updateFile(new File(directory + java.io.File.separator + profile)
                    , oldPatterns, newPatterns);
                    listeFile.add(tempFile);
                }catch(Exception e){
                    e.printStackTrace(Modelio.err);
                }
        
            }
        
            newresource = loadEcoreModel(newFile);
        
            ResourceSet resourceset = newresource.getResourceSet();
        
            for (Resource resource : resourceset.getResources()){
        
                if (resource.getURI().toFileString() != null){ 
                    //                        && (!resource.getURI().toFileString().endsWith(".emf"))){
                    tempFile = updateFile(new File(resource.getURI().toFileString()), 
                                          oldPatterns, newPatterns);
        
                    resource.setURI(URI.createFileURI(tempFile.getAbsolutePath()));
                    listeFile.add(tempFile);
                }
            }
        
            newresource = loadEcoreModel(newFile);
        
        }catch (Exception e){
            e.printStackTrace(Modelio.err);
            if (newFile != null){
                newFile.delete();
                for (File fileTemp : listeFile){
                    fileTemp.delete();
                }
            }
        }
        return newresource;
    }

    @objid ("70924c36-1a76-11e1-8103-0027103f347d")
    private File updateFile(final File oldFile, final List<String> oldPatterns, final List<String> newPatterns) {
        String fileName = oldFile.getName();
        
        File tempFolder = ResourceLoader.getInstance().getXMITempFolder();
        if (!tempFolder.exists()){
            tempFolder.mkdirs();
            tempFolder.mkdir();
        }
        
        String tempFolderPath =  tempFolder.getAbsolutePath();
        
        String newFilePath = tempFolderPath + java.io.File.separator  + fileName;
        File newFile = new File(newFilePath);
        
        String tempPath = tempFolderPath + java.io.File.separator  + fileName + ".temp";
        File tempFile = new File(tempPath);
        
        String tempPath2 = tempFolderPath + java.io.File.separator + fileName + ".temp2";
        File tempFile2 = new File(tempPath2);
        
        String tempPath3 = tempFolderPath + java.io.File.separator + fileName  + ".temp3";
        File tempFile3 = new File(tempPath3);
        
        String tempPath4 = tempFolderPath + java.io.File.separator + fileName  + ".temp4";
        File tempFile4 = new File(tempPath4);
        
        try{
        
            if (tempFile.exists())
                tempFile.delete();
        
            if (tempFile2.exists())
                tempFile2.delete();
        
           
            XMIFileUtils.replace(oldFile.getAbsolutePath(), tempPath, oldPatterns, newPatterns); 
        
            File xslImportFile = ResourceLoader.getInstance().getResource("xslt" + java.io.File.separator + "import.xsl");
            XMIFileUtils.applyXSLT(tempPath, tempPath2, xslImportFile); 
        
        
            if (tempFile3.exists())
                tempFile3.delete();
        
            File xslImport2File = ResourceLoader.getInstance().getResource("xslt" + java.io.File.separator + "import2.xsl");
            XMIFileUtils.applyXSLT(tempPath2, tempPath3, xslImport2File); 
        
            if (tempFile4.exists())
                tempFile4.delete();
        
            File xslImport3File = ResourceLoader.getInstance().getResource("xslt" + java.io.File.separator + "import3.xsl");
            XMIFileUtils.applyXSLT(tempPath3, tempPath4, xslImport3File); 
        
            try{
                newFile.delete();
                newFile.createNewFile();
            }catch(IOException e){
                e.printStackTrace(Modelio.err);
            }
        
            XMIFileUtils.copyFile(tempFile4, newFile);
        
        }finally{
            tempFile.delete();
            tempFile2.delete();
            tempFile3.delete();
            tempFile4.delete();
            System.gc();
        }
        return newFile;
    }

    @objid ("7092c16a-1a76-11e1-8103-0027103f347d")
    private List<String> getOldUMLPatterns() {
        List<String> patterns = new LinkedList<String>();
        patterns.add("xmlns:uml=\"http://schema.omg.org/spec/UML/2.1.1/uml.xml");  
        patterns.add("xmlns:uml[\\s]?=[\\s]?[\"]?[\']?http://www.omg.org/spec/UML/20090901[\"]?[\']?");
        patterns.add("xmlns:uml[\\s]?=[\\s]?[\"]?[\']?http://www.omg.org/spec/UML/20100901[\"]?[\']?");
        patterns.add("xmlns:uml[\\s]?=[\\s]?\'?\"?http://schema.omg.org/spec/UML/2.[1.1]?[1.2]?[1]?[2]?[3]?\'?\"");
        patterns.add("http://schema.omg.org/spec/UML/2.2/ [http://www.eclipse.org/uml2/3.0.0/UML]?");
        return patterns;
    }

    @objid ("7092e87c-1a76-11e1-8103-0027103f347d")
    private List<String> getNewUMLPatterns() {
        List<String> patterns = new LinkedList<String>();
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML" );
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML\"" );
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML\"" );
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML\"" );
        patterns.add("http://www.eclipse.org/uml2/3.0.0/UML/");
        return patterns;
    }

    @objid ("70930f8e-1a76-11e1-8103-0027103f347d")
    private List<String> getNewEMFPatterns() {
        List<String> patterns = new LinkedList<String>();
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML\"");
        patterns.add("xmlns:uml=\"http://www.eclipse.org/uml2/3.0.0/UML\"");
        patterns.add("href=\"http://www.eclipse.org/uml2/3.0.0/UML");
        patterns.add("pathmaphref=\"http://www.eclipse.org/uml2/3.0.0/UML");
        return patterns;
    }

    @objid ("709336a0-1a76-11e1-8103-0027103f347d")
    private List<String> getOldEMFPatterns() {
        List<String> patterns = new LinkedList<String>();
        patterns.add("xmlns:uml[\\s]?=[\\s]?\"?\'?http://www.eclipse.org/uml2/3.1.0/UML[/]?\"?\'?");
        patterns.add("xmlns:uml[\\s]?=[\\s]?\"?\'?http://www.eclipse.org/uml2/2.[01]+.0/UML[/]?\"?\'?");
        patterns.add("href[\\s]?=[\\s]?\"?\'?http://www.eclipse.org/uml2/2.[01]+.0/UML");
        return patterns;
    }

    @objid ("70935db2-1a76-11e1-8103-0027103f347d")
    private void cleanExternalElements() {
        PartialImportMap partialImportMap = PartialImportMap.getInstance();
        
        List<Element> externalElements = new ArrayList<Element>();
        
        for (Object key : partialImportMap.keySet()) {
            if (key != null){
                if (key instanceof Element) {
                    externalElements.add((Element) key);
                }
            }else{
                partialImportMap.remove(key);
            }
        }
        
        IUmlModel model = Modelio.getInstance().getModelingSession().getModel();
        ReverseProperties revProp = ReverseProperties.getInstance();
        for (Element externalElt : externalElements) {  
            Object objElts = revProp.getMappedElement(externalElt);
            if (objElts instanceof IElement){
                model.deleteElement((IElement) objElts);
            }else{
                for(Object objElt : (List<IElement>)objElts ){
                    if (objElt instanceof IElement){
                        model.deleteElement((IElement) objElt);
                    }
                }
            }
        }
    }

}
