/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.reverse;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.ActionExecutionSpecification;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.ConsiderIgnoreFragment;
import org.eclipse.uml2.uml.Continuation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ExecutionOccurrenceSpecification;
import org.eclipse.uml2.uml.ExecutionSpecification;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.InteractionOperand;
import org.eclipse.uml2.uml.InteractionUse;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.eclipse.uml2.uml.PartDecomposition;
import org.eclipse.uml2.uml.StateInvariant;
import org.eclipse.uml2.uml.util.UMLSwitch;

@objid ("17ee10a9-4583-11e0-b54c-00137279a832")
public class Uml2InsertableIFVisitor {
    @objid ("17ee10aa-4583-11e0-b54c-00137279a832")
    private InsertableIFSelector selector;

    @objid ("17ee10ab-4583-11e0-b54c-00137279a832")
    private List<Element> insertableElements = new ArrayList<Element>();


    @objid ("17ee10ae-4583-11e0-b54c-00137279a832")
    public Uml2InsertableIFVisitor() {
        selector = new InsertableIFSelector();
    }

    @objid ("17ee10b0-4583-11e0-b54c-00137279a832")
    public List<Element> getInsertableElements(InteractionFragment theIF) {
        insertableElements.clear();
        selector.doSwitch(theIF);
        return insertableElements;
    }

    @objid ("17f07303-4583-11e0-b54c-00137279a832")
    private class InsertableIFSelector extends UMLSwitch<Object> {
        @objid ("8589ff58-2b14-11e1-8e26-0027103f347d")
        private static final int result = 42;


        @Override
        @objid ("17f07304-4583-11e0-b54c-00137279a832")
        public Object doSwitch(EObject inputElement) {
            if (inputElement != null)
                return super.doSwitch(inputElement);
            else
                return null;
        }

        @Override
        @objid ("17f07309-4583-11e0-b54c-00137279a832")
        public Object caseActionExecutionSpecification(ActionExecutionSpecification fragment) {
            // If this ExecutionSpecification is not connected to a message,
            // add it to the list of insertableElements:
            OccurrenceSpecification start = fragment.getStart();
            OccurrenceSpecification finish = fragment.getFinish();
                        
            if (!(start instanceof MessageOccurrenceSpecification)
                    && !(finish instanceof MessageOccurrenceSpecification)) {
                insertableElements.add(fragment);
            }
            return result;
        }

        @Override
        @objid ("17f0730f-4583-11e0-b54c-00137279a832")
        public Object caseBehaviorExecutionSpecification(BehaviorExecutionSpecification fragment) {
            // If this ExecutionSpecification is not connected to a message,
            // add it to the list of insertableElements:
            OccurrenceSpecification start = fragment.getStart();
            OccurrenceSpecification finish = fragment.getFinish();
                        
            if ((start instanceof ExecutionOccurrenceSpecification)
                    && (finish instanceof ExecutionOccurrenceSpecification)) {
                insertableElements.add(fragment);
            }
            return result;
        }

        @Override
        @objid ("17f2d560-4583-11e0-b54c-00137279a832")
        public Object caseCombinedFragment(CombinedFragment fragment) {
            // /!\ order is important: the CombinedFragment shall be in 1st
            // position so as to be inserted 1st in the diagram.
            insertableElements.add(fragment);
            // We add the Interaction operands to the list of insertable
            // elements:
            for (Object op : fragment.getOperands()) {
                insertableElements.add((InteractionOperand) op);
            }
            return result;
        }

        @Override
        @objid ("17f2d566-4583-11e0-b54c-00137279a832")
        public Object caseConsiderIgnoreFragment(ConsiderIgnoreFragment fragment) {
            insertableElements.add(fragment);
            return result;
        }

        @Override
        @objid ("17f537b7-4583-11e0-b54c-00137279a832")
        public Object caseContinuation(Continuation fragment) {
            // Continuation are not supported by Modelio.
            return result;
        }

        @Override
        @objid ("17f537bd-4583-11e0-b54c-00137279a832")
        public Object caseExecutionOccurrenceSpecification(ExecutionOccurrenceSpecification fragment) {
            return result;
        }

        @Override
        @objid ("17f537c3-4583-11e0-b54c-00137279a832")
        public Object caseExecutionSpecification(ExecutionSpecification fragment) {
            return result;
        }

        @Override
        @objid ("17f79a15-4583-11e0-b54c-00137279a832")
        public Object caseInteraction(Interaction fragment) {
            return result;
        }

        @Override
        @objid ("17f79a1b-4583-11e0-b54c-00137279a832")
        public Object caseInteractionOperand(InteractionOperand fragment) {
            insertableElements.add(fragment);
            return result;
        }

        @Override
        @objid ("17f9fc6b-4583-11e0-b54c-00137279a832")
        public Object caseInteractionUse(InteractionUse fragment) {
            insertableElements.add(fragment);
            return result;
        }

        @Override
        @objid ("17f9fc71-4583-11e0-b54c-00137279a832")
        public Object caseMessageOccurrenceSpecification(MessageOccurrenceSpecification fragment) {
            // Returns the message itself
            Message msg = fragment.getMessage();
            if (msg != null ) {
                insertableElements.add(msg);
            }
            return result;
        }

        @Override
        @objid ("17fc5ec4-4583-11e0-b54c-00137279a832")
        public Object caseOccurrenceSpecification(OccurrenceSpecification fragment) {
            return result;
        }

        @Override
        @objid ("17fc5eca-4583-11e0-b54c-00137279a832")
        public Object casePartDecomposition(PartDecomposition fragment) {
            // Nothing to do, PartDecompositions are handled after lifelines.
            return result;
        }

        @Override
        @objid ("17fc5ed0-4583-11e0-b54c-00137279a832")
        public Object caseStateInvariant(StateInvariant fragment) {
            insertableElements.add(fragment);
            return result;
        }

    }

}
