/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import com.modeliosoft.modelio.api.model.uml.behavior.activity.IInputPin;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.AddStructuralFeatureValueAction;
import org.eclipse.uml2.uml.AddVariableValueAction;
import org.eclipse.uml2.uml.CallOperationAction;
import org.eclipse.uml2.uml.ClearAssociationAction;
import org.eclipse.uml2.uml.CreateLinkAction;
import org.eclipse.uml2.uml.DestroyObjectAction;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InputPin;
import org.eclipse.uml2.uml.InvocationAction;
import org.eclipse.uml2.uml.LinkAction;
import org.eclipse.uml2.uml.LoopNode;
import org.eclipse.uml2.uml.OpaqueAction;
import org.eclipse.uml2.uml.RaiseExceptionAction;
import org.eclipse.uml2.uml.ReadIsClassifiedObjectAction;
import org.eclipse.uml2.uml.ReadLinkObjectEndAction;
import org.eclipse.uml2.uml.ReadLinkObjectEndQualifierAction;
import org.eclipse.uml2.uml.ReadStructuralFeatureAction;
import org.eclipse.uml2.uml.ReclassifyObjectAction;
import org.eclipse.uml2.uml.ReduceAction;
import org.eclipse.uml2.uml.RemoveStructuralFeatureValueAction;
import org.eclipse.uml2.uml.RemoveVariableValueAction;
import org.eclipse.uml2.uml.ReplyAction;
import org.eclipse.uml2.uml.SendObjectAction;
import org.eclipse.uml2.uml.SendSignalAction;
import org.eclipse.uml2.uml.StartClassifierBehaviorAction;
import org.eclipse.uml2.uml.StructuralFeatureAction;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.TestIdentityAction;
import org.eclipse.uml2.uml.UnmarshallAction;
import org.eclipse.uml2.uml.WriteLinkAction;
import org.eclipse.uml2.uml.WriteStructuralFeatureAction;
import org.eclipse.uml2.uml.WriteVariableAction;
import org.eclipse.uml2.uml.util.UMLSwitch;

@objid ("195613be-4583-11e0-b54c-00137279a832")
public class AttachInputPinToOwnerVisitor {
    @objid ("195613bf-4583-11e0-b54c-00137279a832")
    private InputPinLinker linker;

    @objid ("195613c0-4583-11e0-b54c-00137279a832")
    private IInputPin obInputPin;

    @objid ("195613c1-4583-11e0-b54c-00137279a832")
    private InputPin inputPin;


    @objid ("195613c2-4583-11e0-b54c-00137279a832")
    public AttachInputPinToOwnerVisitor() {
        linker = new InputPinLinker();
    }

    @objid ("195613c4-4583-11e0-b54c-00137279a832")
    public void attachInputPin(InputPin inputPin, Element owner, IInputPin obInputPin) {
        this.inputPin = inputPin;
        this.obInputPin = obInputPin;
        linker.accept(owner);
    }

    @objid ("195613c9-4583-11e0-b54c-00137279a832")
    class InputPinLinker extends UMLSwitch<Object> {
        @objid ("dcf3dad7-2b10-11e1-8e26-0027103f347d")
        private static final int result = 42;


        @objid ("195613ca-4583-11e0-b54c-00137279a832")
        public void accept(Element ecoreElt) {
            doSwitch(ecoreElt);
        }

        @Override
        @objid ("195613cd-4583-11e0-b54c-00137279a832")
        public Object caseAction(Action owner) {
            if (!owner.getInputs().contains(inputPin))
                owner.getInputs().add(inputPin);
            return result;
        }

        @Override
        @objid ("1958761c-4583-11e0-b54c-00137279a832")
        public Object caseAddStructuralFeatureValueAction(AddStructuralFeatureValueAction owner) {
            if (obInputPin.isStereotyped(XMIStereotypes.UML2INSERTAT)){
                if ((owner.getInsertAt() == null ) ||(!owner.getInsertAt().equals(inputPin)))
                    owner.setInsertAt(inputPin);
                        
            }else if (obInputPin.isStereotyped(XMIStereotypes.UML2VALUE)){
                        
                if ((owner.getValue() == null ) || (!owner.getValue().equals(inputPin)))
                    owner.setValue(inputPin);
                        
            }else {
                if ((owner.getObject() == null) ||(!owner.getObject().equals(inputPin)))
                    owner.setObject(inputPin);
            }
            return result;
        }

        @Override
        @objid ("19587622-4583-11e0-b54c-00137279a832")
        public Object caseAddVariableValueAction(AddVariableValueAction owner) {
            if ((owner.getInsertAt() == null ) || (!owner.getInsertAt().equals(inputPin)))
                owner.setInsertAt(inputPin);
            return result;
        }

        @Override
        @objid ("195ad873-4583-11e0-b54c-00137279a832")
        public Object caseCallOperationAction(CallOperationAction owner) {
            if (obInputPin.isStereotyped(XMIStereotypes.UML2TARGET)){
                if ((owner.getTarget() == null ) || (!owner.getTarget().equals(inputPin)))
                    owner.setTarget(inputPin);
            }else{
                        
                owner.getArguments().add(inputPin);
            }
            return result;
        }

        @Override
        @objid ("195ad879-4583-11e0-b54c-00137279a832")
        public Object caseClearAssociationAction(ClearAssociationAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("195ad87f-4583-11e0-b54c-00137279a832")
        public Object caseDestroyObjectAction(DestroyObjectAction owner) {
            if ((owner.getTarget() == null ) || (!owner.getTarget().equals(inputPin)))
                owner.setTarget(inputPin);
            return result;
        }

        @Override
        @objid ("195d3ad0-4583-11e0-b54c-00137279a832")
        public Object caseInvocationAction(InvocationAction owner) {
            owner.getArguments().add(inputPin);
            return result;
        }

        @Override
        @objid ("195d3ad6-4583-11e0-b54c-00137279a832")
        public Object caseLinkAction(LinkAction owner) {
            owner.getInputValues().add(inputPin);
            return result;
        }

        @Override
        @objid ("195f9d27-4583-11e0-b54c-00137279a832")
        public Object caseLoopNode(LoopNode owner) {
            owner.getLoopVariableInputs().add(inputPin);
            return result;
        }

        @Override
        @objid ("195f9d2d-4583-11e0-b54c-00137279a832")
        public Object caseOpaqueAction(OpaqueAction owner) {
            owner.getInputValues().add(inputPin);
            return result;
        }

        @Override
        @objid ("1961ff80-4583-11e0-b54c-00137279a832")
        public Object caseRaiseExceptionAction(RaiseExceptionAction owner) {
            if ((owner.getException() == null ) || (!owner.getException().equals(inputPin)))
                owner.setException(inputPin);
            return result;
        }

        @Override
        @objid ("1961ff86-4583-11e0-b54c-00137279a832")
        public Object caseReadIsClassifiedObjectAction(ReadIsClassifiedObjectAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("1961ff8c-4583-11e0-b54c-00137279a832")
        public Object caseReadLinkObjectEndAction(ReadLinkObjectEndAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("196461de-4583-11e0-b54c-00137279a832")
        public Object caseReadLinkObjectEndQualifierAction(ReadLinkObjectEndQualifierAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("196461e4-4583-11e0-b54c-00137279a832")
        public Object caseReclassifyObjectAction(ReclassifyObjectAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("196461ea-4583-11e0-b54c-00137279a832")
        public Object caseReduceAction(ReduceAction owner) {
            if ((owner.getCollection() == null ) || (!owner.getCollection().equals(inputPin)))
                owner.setCollection(inputPin);
            return result;
        }

        @Override
        @objid ("1966c435-4583-11e0-b54c-00137279a832")
        public Object caseRemoveStructuralFeatureValueAction(RemoveStructuralFeatureValueAction owner) {
            if ((owner.getRemoveAt() == null ) || (!owner.getRemoveAt().equals(inputPin)))
                owner.setRemoveAt(inputPin);
            return result;
        }

        @Override
        @objid ("1966c43b-4583-11e0-b54c-00137279a832")
        public Object caseRemoveVariableValueAction(RemoveVariableValueAction owner) {
            if ((owner.getRemoveAt() == null ) || (!owner.getRemoveAt().equals(inputPin)))
                owner.setRemoveAt(inputPin);
            return result;
        }

        @Override
        @objid ("1969268e-4583-11e0-b54c-00137279a832")
        public Object caseReplyAction(ReplyAction owner) {
            owner.getReplyValues().add(inputPin);
            return result;
        }

        @Override
        @objid ("19692694-4583-11e0-b54c-00137279a832")
        public Object caseSendObjectAction(SendObjectAction owner) {
            if ((owner.getRequest() == null ) ||(!owner.getRequest().equals(inputPin)))
                owner.setRequest(inputPin);
            return result;
        }

        @Override
        @objid ("1969269a-4583-11e0-b54c-00137279a832")
        public Object caseSendSignalAction(SendSignalAction owner) {
            if ((owner.getTarget() == null ) ||(owner.getTarget() == null))
                owner.setTarget(inputPin);
            return null;
        }

        @Override
        @objid ("196b88ec-4583-11e0-b54c-00137279a832")
        public Object caseStartClassifierBehaviorAction(StartClassifierBehaviorAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("196b88f2-4583-11e0-b54c-00137279a832")
        public Object caseStructuralFeatureAction(StructuralFeatureAction owner) {
            if ((owner.getObject() == null) ||(!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("19704d9d-4583-11e0-b54c-00137279a832")
        public Object caseStructuredActivityNode(StructuredActivityNode owner) {
            owner.getNodes().add(inputPin);
            return result;
        }

        @Override
        @objid ("1972aff6-4583-11e0-b54c-00137279a832")
        public Object caseTestIdentityAction(TestIdentityAction owner) {
            if (obInputPin.isStereotyped(XMIStereotypes.UML2FIRST)){
                owner.setFirst(inputPin);
            }else if (obInputPin.isStereotyped(XMIStereotypes.UML2SECOND)){
                owner.setSecond(inputPin);
            }
            return result;
        }

        @Override
        @objid ("1972affc-4583-11e0-b54c-00137279a832")
        public Object caseUnmarshallAction(UnmarshallAction owner) {
            if ((owner.getObject() == null ) ||(!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("19751250-4583-11e0-b54c-00137279a832")
        public Object caseWriteStructuralFeatureAction(WriteStructuralFeatureAction owner) {
            if (obInputPin.isStereotyped(XMIStereotypes.UML2VALUE)){
                        
                if ((owner.getValue() == null ) || (!owner.getValue().equals(inputPin)))
                    owner.setValue(inputPin);
                        
            }
            return result;
        }

        @Override
        @objid ("19751256-4583-11e0-b54c-00137279a832")
        public Object caseWriteVariableAction(WriteVariableAction owner) {
            if ((owner.getValue() == null ) || (!owner.getValue().equals(inputPin)))
                owner.setValue(inputPin);
            return result;
        }

        @Override
        @objid ("1975125c-4583-11e0-b54c-00137279a832")
        public Object caseCreateLinkAction(CreateLinkAction owner) {
            return null;
        }

        @Override
        @objid ("197774ae-4583-11e0-b54c-00137279a832")
        public Object caseReadStructuralFeatureAction(ReadStructuralFeatureAction owner) {
            if ((owner.getObject() == null ) || (!owner.getObject().equals(inputPin)))
                owner.setObject(inputPin);
            return result;
        }

        @Override
        @objid ("197774b4-4583-11e0-b54c-00137279a832")
        public Object caseWriteLinkAction(WriteLinkAction owner) {
            return null;
        }

    }

}
