/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.eclipse.uml2.uml.AcceptCallAction;
import org.eclipse.uml2.uml.AcceptEventAction;
import org.eclipse.uml2.uml.Action;
import org.eclipse.uml2.uml.AddStructuralFeatureValueAction;
import org.eclipse.uml2.uml.CallAction;
import org.eclipse.uml2.uml.ConditionalNode;
import org.eclipse.uml2.uml.CreateLinkObjectAction;
import org.eclipse.uml2.uml.CreateObjectAction;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LoopNode;
import org.eclipse.uml2.uml.OpaqueAction;
import org.eclipse.uml2.uml.OutputPin;
import org.eclipse.uml2.uml.ReadExtentAction;
import org.eclipse.uml2.uml.ReadIsClassifiedObjectAction;
import org.eclipse.uml2.uml.ReadLinkAction;
import org.eclipse.uml2.uml.ReadLinkObjectEndAction;
import org.eclipse.uml2.uml.ReadLinkObjectEndQualifierAction;
import org.eclipse.uml2.uml.ReadSelfAction;
import org.eclipse.uml2.uml.ReadStructuralFeatureAction;
import org.eclipse.uml2.uml.ReadVariableAction;
import org.eclipse.uml2.uml.ReduceAction;
import org.eclipse.uml2.uml.SendSignalAction;
import org.eclipse.uml2.uml.StructuredActivityNode;
import org.eclipse.uml2.uml.TestIdentityAction;
import org.eclipse.uml2.uml.UnmarshallAction;
import org.eclipse.uml2.uml.ValueSpecificationAction;
import org.eclipse.uml2.uml.util.UMLSwitch;

@objid ("1979d706-4583-11e0-b54c-00137279a832")
public class AttachOutputPinToOwnerVisitor {
    @objid ("1979d707-4583-11e0-b54c-00137279a832")
    private OutputPinLinker linker;

    @objid ("1979d708-4583-11e0-b54c-00137279a832")
    private OutputPin outputPin;


    @objid ("1979d709-4583-11e0-b54c-00137279a832")
    public AttachOutputPinToOwnerVisitor() {
        this.linker = new OutputPinLinker();
    }

    @objid ("1979d70b-4583-11e0-b54c-00137279a832")
    public void attachOutputPin(OutputPin outputPin, Element owner) {
        this.outputPin = outputPin;
        this.linker.accept(owner);
    }

    @objid ("1979d70f-4583-11e0-b54c-00137279a832")
    private class OutputPinLinker extends UMLSwitch<Object> {
        @objid ("7be1f53f-2b10-11e1-8e26-0027103f347d")
        private static final int result = 42;


        @objid ("1979d710-4583-11e0-b54c-00137279a832")
        public void accept(Element ecoreElt) {
            doSwitch(ecoreElt);
        }

        @Override
        @objid ("197c395e-4583-11e0-b54c-00137279a832")
        public Object caseAcceptCallAction(AcceptCallAction owner) {
            owner.setReturnInformation(outputPin);
            return result;
        }

        @Override
        @objid ("197c3964-4583-11e0-b54c-00137279a832")
        public Object caseAcceptEventAction(AcceptEventAction owner) {
            owner.getResults().add(outputPin);
            return result;
        }

        @Override
        @objid ("197c396a-4583-11e0-b54c-00137279a832")
        public Object caseAction(Action owner) {
            try{
                if (!(owner.getOutputs().contains(outputPin)))
                    owner.getOutputs().add(outputPin);
            }catch(UnsupportedOperationException e){
                outputPin.destroy();
            }
            return result;
        }

        @Override
        @objid ("197e9bbc-4583-11e0-b54c-00137279a832")
        public Object caseCallAction(CallAction owner) {
            owner.getResults().add(outputPin);
            return result;
        }

        @Override
        @objid ("197e9bc2-4583-11e0-b54c-00137279a832")
        public Object caseConditionalNode(ConditionalNode owner) {
            owner.getResults().add(outputPin);
            return result;
        }

        @Override
        @objid ("1980fe13-4583-11e0-b54c-00137279a832")
        public Object caseCreateLinkObjectAction(CreateLinkObjectAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("1980fe19-4583-11e0-b54c-00137279a832")
        public Object caseCreateObjectAction(CreateObjectAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("1980fe1f-4583-11e0-b54c-00137279a832")
        public Object caseLoopNode(LoopNode owner) {
            owner.getResults().add(outputPin);
            return result;
        }

        @Override
        @objid ("19836070-4583-11e0-b54c-00137279a832")
        public Object caseOpaqueAction(OpaqueAction owner) {
            owner.getOutputValues().add(outputPin);
            return result;
        }

        @Override
        @objid ("19836076-4583-11e0-b54c-00137279a832")
        public Object caseReadExtentAction(ReadExtentAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("1985c2c7-4583-11e0-b54c-00137279a832")
        public Object caseReadIsClassifiedObjectAction(ReadIsClassifiedObjectAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("1985c2cd-4583-11e0-b54c-00137279a832")
        public Object caseReadLinkAction(ReadLinkAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("19882524-4583-11e0-b54c-00137279a832")
        public Object caseReadLinkObjectEndAction(ReadLinkObjectEndAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("1988252a-4583-11e0-b54c-00137279a832")
        public Object caseReadLinkObjectEndQualifierAction(ReadLinkObjectEndQualifierAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198a877b-4583-11e0-b54c-00137279a832")
        public Object caseReadSelfAction(ReadSelfAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198a8781-4583-11e0-b54c-00137279a832")
        public Object caseReadStructuralFeatureAction(ReadStructuralFeatureAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198ce9d4-4583-11e0-b54c-00137279a832")
        public Object caseReadVariableAction(ReadVariableAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198ce9da-4583-11e0-b54c-00137279a832")
        public Object caseReduceAction(ReduceAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198ce9e0-4583-11e0-b54c-00137279a832")
        public Object caseStructuredActivityNode(StructuredActivityNode owner) {
            owner.getNodes().add(outputPin);
            return result;
        }

        @Override
        @objid ("198f4c33-4583-11e0-b54c-00137279a832")
        public Object caseTestIdentityAction(TestIdentityAction owner) {
            owner.setResult(outputPin);
            return result;
        }

        @Override
        @objid ("198f4c39-4583-11e0-b54c-00137279a832")
        public Object caseUnmarshallAction(UnmarshallAction owner) {
            owner.getResults().add(outputPin);
            return result;
        }

        @Override
        @objid ("1991ae8b-4583-11e0-b54c-00137279a832")
        public Object caseValueSpecificationAction(ValueSpecificationAction owner) {
            owner.setResult(outputPin);
            return null;
        }

        @objid ("1991ae91-4583-11e0-b54c-00137279a832")
        public Object caseSendSignalAction(SendSignalAction owner) {
            // pas supporter en UML2 
            outputPin.destroy();
            return null;
        }

        @Override
        @objid ("6df3d852-1a76-11e1-8103-0027103f347d")
        public Object caseAddStructuralFeatureValueAction(final AddStructuralFeatureValueAction owner) {
            if (owner.getResult() == null){
                try{    
                    owner.setResult(outputPin);
                }catch(UnsupportedOperationException e){
                    outputPin.destroy();
                    e.printStackTrace(Modelio.err);
                }
            }
            return result;
        }

    }

}
