/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.lang.reflect.Method;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

@objid ("199410e2-4583-11e0-b54c-00137279a832")
public class BareBonesBrowserLaunch {

    @objid ("199410e3-4583-11e0-b54c-00137279a832")
    public static void openURL(String url, Shell shell) {
        String osName = System.getProperty("os.name");
        try {
            if (url == null) {
                throw new Exception(XMI.I18N
                        .getString("warning.helpDocNotFound.voidURL"));
            } else if (osName.startsWith("Windows")) {
                Runtime.getRuntime().exec(
                        "rundll32 url.dll,FileProtocolHandler " + url);
            } else if (osName.startsWith("Mac OS")) {
                Class<?> fileMgr = Class.forName("com.apple.eio.FileManager");
                Method openURL = fileMgr.getDeclaredMethod("openURL",
                        new Class[] { String.class });
                openURL.invoke(null, new Object[] { url });
            } else { // assume Unix or Linux
                String[] browsers = { "firefox", "opera", "konqueror",
                        "epiphany", "mozilla", "netscape" };
                String browser = null;
                for (int count = 0; count < browsers.length && browser == null; count++) {
                    if (Runtime.getRuntime().exec(
                            new String[] { "which", browsers[count] })
                            .waitFor() == 0) {
                        browser = browsers[count];
                    }
                }
                
                if (browser == null)
                    throw new Exception(
                            XMI.I18N
                                    .getString("warning.helpDocNotFound.browserNotFound"));
                else
                    Runtime.getRuntime().exec(new String[] { browser, url });
            }
        } catch (Exception e) {
            displayErrorMessage(e, shell);
        }
    }

    @objid ("199410e7-4583-11e0-b54c-00137279a832")
    private static void displayErrorMessage(final Exception e, final Shell shell) {
        MessageBox messageBox = new MessageBox(shell, SWT.OK | SWT.ICON_WARNING);
                
        String errorMsg = e.getLocalizedMessage();
        if (errorMsg == null || "".compareTo(errorMsg) == 0) {
            errorMsg = XMI.I18N
                    .getString("warning.helpDocNotFound.browserLaunchingError");
        }
                
        messageBox.setText(XMI.I18N.getString("warning.helpDocNotFound.title"));
        messageBox.setMessage(errorMsg);
        messageBox.open();
    }

}
