/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.core.app.O;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.api.XMIExtension;
import com.modeliosoft.modelio.xmi.generation.GenericMetamodelVisitor;
import com.modeliosoft.modelio.xmi.generation.PartialCreationExportVisitor;
import com.modeliosoft.modelio.xmi.generation.PartialExportMap;
import com.modeliosoft.modelio.xmi.generation.ProfileExportVisitorImpl;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.gui.report.ReportModel;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportProfile;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;

@objid ("1aafc8c0-4583-11e0-b54c-00137279a832")
public class GenerationProperties {
    @objid ("1aafc8c1-4583-11e0-b54c-00137279a832")
    private boolean roundtripEnabled = false;

    @objid ("1aafc8c2-4583-11e0-b54c-00137279a832")
    private static GenerationProperties INSTANCE = null;

    @objid ("1aafc8c3-4583-11e0-b54c-00137279a832")
    private GenericMetamodelVisitor objingVisitor = null;

    @objid ("1aafc8c4-4583-11e0-b54c-00137279a832")
    private List<IModelTree> exportScopeElts;

    @objid ("1aafc8c8-4583-11e0-b54c-00137279a832")
    private boolean timeDisplayerActivated = false;

    @objid ("1aafc8c9-4583-11e0-b54c-00137279a832")
    private IPackage selectedPackage = Modelio.getInstance().getModelingSession().getModel().getRoot();

    @objid ("1aafc8ca-4583-11e0-b54c-00137279a832")
    private PartialCreationExportVisitor partialCreationExportVisitor = new PartialCreationExportVisitor();

    @objid ("1aafc8cb-4583-11e0-b54c-00137279a832")
    private ScopeChecker scopeChecker;

    @objid ("1aafc8cc-4583-11e0-b54c-00137279a832")
    private List<IModelElement> stereotypesExported = new ArrayList<IModelElement>();

    @objid ("1ab22b15-4583-11e0-b54c-00137279a832")
    private List<Stereotype> defaultStereotyped = new ArrayList<Stereotype>();

    @objid ("1ab22b18-4583-11e0-b54c-00137279a832")
    private Model ecoreModel = null;

    @objid ("6e483e34-1a76-11e1-8103-0027103f347d")
    private List<IModelElement> sysMLElementExported = new ArrayList<IModelElement>();

    @objid ("6e486548-1a76-11e1-8103-0027103f347d")
    private String filePath = "";

    @objid ("6e488c58-1a76-11e1-8103-0027103f347d")
    private Map<String, ArrayList<String>> classTabConvertion = null;

    @objid ("6e48da7c-1a76-11e1-8103-0027103f347d")
    private Profile sysMLProfile = null;

    @objid ("6e48da7d-1a76-11e1-8103-0027103f347d")
    private List<Profile> exportedProfiles = new ArrayList<Profile>();

    @objid ("9d77af4d-1a76-11e1-8103-0027103f347d")
    private VersionExport exportVersion = VersionExport.UML211;

    @objid ("9d784b94-1a76-11e1-8103-0027103f347d")
    private XMIExtension fileExtension = XMIExtension.XMI;

    @objid ("0c9ed465-3b71-11e1-8577-0027103f347d")
    private ReportModel report = null;

    @objid ("d7d25530-71bf-11e1-bffd-0027103f347d")
    private boolean sysmlApplied = false;


    @objid ("1ab48d71-4583-11e0-b54c-00137279a832")
    private GenerationProperties() {
    }

    @objid ("1ab48d73-4583-11e0-b54c-00137279a832")
    public static GenerationProperties getInstance() {
        if (INSTANCE == null)
            INSTANCE = new GenerationProperties();
        return INSTANCE;
    }

    @objid ("1ab48d77-4583-11e0-b54c-00137279a832")
    public void setRoundtripEnabled(boolean enableRoundtrip) {
        this.roundtripEnabled = enableRoundtrip;
    }

    @objid ("1ab48d7a-4583-11e0-b54c-00137279a832")
    public void setObjingVisitor(GenericMetamodelVisitor objingVisitor) {
        this.objingVisitor = objingVisitor;
    }

    @objid ("1ab95227-4583-11e0-b54c-00137279a832")
    public Model getEcoreModel() {
        return this.ecoreModel;
    }

    @objid ("1ab9522b-4583-11e0-b54c-00137279a832")
    public void setTimeDisplayerActivated(boolean activateTimeDisplayer) {
        this.timeDisplayerActivated = activateTimeDisplayer;
    }

    @objid ("1abbb47a-4583-11e0-b54c-00137279a832")
    public void setScopeChecker(ScopeChecker theScopeChecker) {
        this.scopeChecker = theScopeChecker;
    }

    @objid ("1abbb483-4583-11e0-b54c-00137279a832")
    public void cleanStereotypesExported() {
        this.stereotypesExported = new ArrayList<IModelElement>();
    }

    @objid ("1abbb485-4583-11e0-b54c-00137279a832")
    public void supressProfile() {
        for (Profile current : exportedProfiles){
            if (this.ecoreModel.isProfileApplied(current))
                this.ecoreModel.unapplyProfile(current);
        }
        this.exportedProfiles = new ArrayList<Profile>();
    }

    @objid ("1abe16e2-4583-11e0-b54c-00137279a832")
    public void setExportScopeElts(List<IModelTree> rootElements) {
        this.exportScopeElts = rootElements;
    }

    @objid ("1ac07932-4583-11e0-b54c-00137279a832")
    public void setEcoreModel(Model ecoreModel) {
        this.ecoreModel = ecoreModel;
    }

    @objid ("1ac07938-4583-11e0-b54c-00137279a832")
    public void setSelectedPackage(IPackage selectedPkg) {
        this.selectedPackage = selectedPkg;
    }

    @objid ("1ac0793b-4583-11e0-b54c-00137279a832")
    public Element getMappedElement(IElement objingElt) {
        PartialExportMap partialCreationMap = PartialExportMap.getInstance();
        TotalExportMap totalCreationMap = TotalExportMap.getInstance();
        
        Element mappedElement = totalCreationMap.get(objingElt.getIdentifier());
        
        if (mappedElement == null)
            mappedElement = partialCreationMap.get(objingElt.getIdentifier());
        
        if (mappedElement == null) {
            // If the element is not in the scope, it will not be created
            // partially:
            if (this.scopeChecker.contains(objingElt)) {
                mappedElement = this.partialCreationExportVisitor
                        .createPartialEcoreElt(objingElt);
            }
        }
        return mappedElement;
    }

    @objid ("1ac2db89-4583-11e0-b54c-00137279a832")
    public boolean isMappedElement(IElement objingElt) {
        if (TotalExportMap.getInstance().get(objingElt.getIdentifier()) != null)
            return true;
        else if (PartialExportMap.getInstance().get(objingElt.getIdentifier()) != null)
            return true;
        else
            return false;
    }

    @objid ("1ac2db92-4583-11e0-b54c-00137279a832")
    public void addStereotypeExported(IModelElement element) {
        if (!this.stereotypesExported.contains(element))
            this.stereotypesExported.add(element);
    }

    @objid ("1ac2db95-4583-11e0-b54c-00137279a832")
    public VersionExport getExportVersion() {
        return this.exportVersion;
    }

    @objid ("1ac53de6-4583-11e0-b54c-00137279a832")
    public void setVersionExport(VersionExport version) {
        this. exportVersion = version;
    }

    @objid ("1ac53deb-4583-11e0-b54c-00137279a832")
    public XMIExtension getFileExtension() {
        return this.fileExtension;
    }

    @objid ("1ac7a03c-4583-11e0-b54c-00137279a832")
    public void setFileExtension(XMIExtension ext) {
        this.fileExtension = ext;
    }

    @objid ("1ac7a041-4583-11e0-b54c-00137279a832")
    public boolean isRoundtripEnabled() {
        // Automatically generated method. Please delete this comment before entering specific code.
        return this.roundtripEnabled;
    }

    @objid ("1ac7a045-4583-11e0-b54c-00137279a832")
    public GenericMetamodelVisitor getObjingVisitor() {
        return this.objingVisitor;
    }

    @objid ("1aca0296-4583-11e0-b54c-00137279a832")
    public List<IModelTree> getExportScopeElts() {
        return this.exportScopeElts;
    }

    @objid ("1aca029c-4583-11e0-b54c-00137279a832")
    public boolean isTimeDisplayerActivated() {
        return this.timeDisplayerActivated;
    }

    @objid ("1aca02a0-4583-11e0-b54c-00137279a832")
    public IPackage getSelectedPackage() {
        return this.selectedPackage;
    }

    @objid ("1acc64f1-4583-11e0-b54c-00137279a832")
    public PartialCreationExportVisitor getPartialCreationExportVisitor() {
        return this.partialCreationExportVisitor;
    }

    @objid ("1acc64f5-4583-11e0-b54c-00137279a832")
    public ScopeChecker getScopeChecker() {
        return this.scopeChecker;
    }

    @objid ("1acc64f9-4583-11e0-b54c-00137279a832")
    public List<IModelElement> getStereotypesExported() {
        return this.stereotypesExported;
    }

    @objid ("1acec74d-4583-11e0-b54c-00137279a832")
    public void initialize() {
        PrimitiveTypeMapper.clean();     
        TotalExportMap.getInstance().clear();
        PartialExportMap.getInstance().clear();
    }

    @objid ("1ad129a4-4583-11e0-b54c-00137279a832")
    public List<Stereotype> getDefaultStereotyped() {
        return this.defaultStereotyped;
    }

    @objid ("6e5229c7-1a76-11e1-8103-0027103f347d")
    public List<Profile> getExportedProfiles() {
        return this.exportedProfiles;
    }

    @objid ("6e5250dc-1a76-11e1-8103-0027103f347d")
    public void addExportedProfile(final Profile newProfile) {
        if (!(this.exportedProfiles.contains(newProfile)))
            this.exportedProfiles.add(newProfile);
    }

    @objid ("6e5277ed-1a76-11e1-8103-0027103f347d")
    public void addSysMLExported(final IModelElement element) {
        if (!this.sysMLElementExported.contains(element))
            this.sysMLElementExported.add(element);
    }

    @objid ("6e529efe-1a76-11e1-8103-0027103f347d")
    public List<IModelElement> getSysMLExported() {
        return this.sysMLElementExported;
    }

    @objid ("6e52ed20-1a76-11e1-8103-0027103f347d")
    public void supressSysMLProfile() {
        if (this.sysMLProfile != null){
            if (this.ecoreModel.isProfileApplied(this.sysMLProfile)){
                this.ecoreModel.unapplyProfile(this.sysMLProfile);
            }
            this.sysMLProfile = null;   
        }
    }

    @objid ("6e52ed22-1a76-11e1-8103-0027103f347d")
    public void cleanSysMLElementExported() {
        this.sysMLElementExported = new ArrayList<IModelElement>();
    }

    @objid ("6e533b45-1a76-11e1-8103-0027103f347d")
    public Profile getSysMLProfile() {
        if (this.sysMLProfile == null){
        
            IProfile obSysMLProfile = null;
            for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
                if (module.getName().equals("SysMLArchitect")){
                    for (IProfile profile :module.getOwnedProfile()){
                        if (SysMLProfileUtils.isSysML(profile)){
                            obSysMLProfile = profile;
                            break;
                        }
                    }
                }
            }
        
            if (obSysMLProfile != null){
                this.sysMLProfile = (Profile) TotalExportMap.getInstance().get(obSysMLProfile.getIdentifier());
                //SysML export
                if (this.sysMLProfile == null) {
                    ProfileExportVisitorImpl profileVisitor = new ProfileExportVisitorImpl();
                    PExportProfile pprofile = new PExportProfile(obSysMLProfile);
                    profileVisitor.visit(pprofile);
                    this.sysMLProfile = (Profile) TotalExportMap.getInstance().get(obSysMLProfile.getIdentifier());
        
                }
        
        
            }
        }
        return this.sysMLProfile;
    }

    @objid ("6e536257-1a76-11e1-8103-0027103f347d")
    public void inverseProfiles(final Profile profileFirst, final Profile profileLast) {
        int indexFirst = this.exportedProfiles.indexOf(profileFirst);
        int indexLast = this.exportedProfiles.indexOf(profileLast);
        if (indexFirst > indexLast){
            this.exportedProfiles.remove(profileFirst);
            this.exportedProfiles.remove(profileLast);
            if (indexFirst < this.exportedProfiles.size() -1)
                this.exportedProfiles.add(indexFirst,  profileFirst);
            else 
                this.exportedProfiles.add(profileFirst);
        
            if (indexLast < this.exportedProfiles.size() -1)
                this.exportedProfiles.add(indexLast, profileLast);
            else 
                this.exportedProfiles.add(profileLast);
        
        }
    }

    @objid ("6e53b07a-1a76-11e1-8103-0027103f347d")
    public void setFilePath(final File file) {
        this.filePath = file.getAbsolutePath();
    }

    @objid ("6e53d78c-1a76-11e1-8103-0027103f347d")
    public void setFilePath(final String path) {
        this.filePath = path;
    }

    @objid ("6e53fea1-1a76-11e1-8103-0027103f347d")
    public String getFilePath() {
        return this.filePath;
    }

    @objid ("6e5425b3-1a76-11e1-8103-0027103f347d")
    public String getFileDirectory() {
        File temp = new File(this.filePath);
        return temp.getParentFile().getAbsolutePath();
    }

    @objid ("6e54c1f8-1a76-11e1-8103-0027103f347d")
    public List<String> getEcoreClassNames(final String objClassName) {
        if (this.classTabConvertion == null)
            initClassTabConvertion();
        
        List<String> result = this.classTabConvertion.get(objClassName);
        
        if (result == null)
            return new ArrayList<String>();
        else
            return result;
    }

    @SuppressWarnings("serial")
    @objid ("6e54e90c-1a76-11e1-8103-0027103f347d")
    private void initClassTabConvertion() {
        this.classTabConvertion = new HashMap<String, ArrayList<String>>();
        this.classTabConvertion.put("Abstraction",  new ArrayList<String>(){{add("Abstraction");}});
        this.classTabConvertion.put("AcceptCallEventAction",  new ArrayList<String>(){{add("AcceptEventAction");}});
        this.classTabConvertion.put("AcceptChangeEventAction",  new ArrayList<String>(){{add("AcceptEventAction");}});
        this.classTabConvertion.put("AcceptSignalAction",  new ArrayList<String>(){{add("AcceptEventAction");}});
        this.classTabConvertion.put("AcceptTimeEventAction",  new ArrayList<String>(){{add("AcceptEventAction");}});
        this.classTabConvertion.put("Activity",  new ArrayList<String>(){{add("Activity");}});
        this.classTabConvertion.put("ActivityEdge",  new ArrayList<String>(){{add("ActivityEdge");}});
        this.classTabConvertion.put("ActivityFinalNode",  new ArrayList<String>(){{add("ActivityFinalNode");}});
        this.classTabConvertion.put("ActivityNode",  new ArrayList<String>(){{add("ActivityNode");}});
        this.classTabConvertion.put("ActivityParameterNode",  new ArrayList<String>(){{add("ActivityParameterNode");}});
        this.classTabConvertion.put("ActivityPartition",  new ArrayList<String>(){{add("ActivityPartition");}});
        this.classTabConvertion.put("Actor",  new ArrayList<String>(){{add("Actor");}});
        this.classTabConvertion.put("Artifact",  new ArrayList<String>(){{add("Artifact");}});
        this.classTabConvertion.put("Association",  new ArrayList<String>(){{add("Association"); add("AssociationClass");}});
        this.classTabConvertion.put("AssociationEnd",  new ArrayList<String>(){{add("Property");}});
        this.classTabConvertion.put("Attribute",  new ArrayList<String>(){{add("Property");}});
        this.classTabConvertion.put("AttributeLink",  new ArrayList<String>(){{add("Slot");}});
        this.classTabConvertion.put("Behavior",  new ArrayList<String>(){{add("Behavior");}});
        this.classTabConvertion.put("BehaviorParameter",  new ArrayList<String>(){{add("Parameter");}});
        this.classTabConvertion.put("BindableInstance",  new ArrayList<String>(){{add("Property");}});
        this.classTabConvertion.put("Binding",  new ArrayList<String>(){{add("Dependency");}});
        this.classTabConvertion.put("CallBehaviorAction",  new ArrayList<String>(){{add("CallBehaviorAction");}});
        this.classTabConvertion.put("CallOperationAction",  new ArrayList<String>(){{add("CallOperationAction");}});
        this.classTabConvertion.put("CentralBufferNode",  new ArrayList<String>(){{add("CentralBufferNode");}});
        this.classTabConvertion.put("Class",  new ArrayList<String>(){{add("Class");}});
        this.classTabConvertion.put("ClassAssociation",  new ArrayList<String>(){{add("AssociationClass");}});
        this.classTabConvertion.put("Classifier",  new ArrayList<String>(){{add("Classifier");}});
        this.classTabConvertion.put("Clause",  new ArrayList<String>(){{add("Clause");}});
        this.classTabConvertion.put("Collaboration",  new ArrayList<String>(){{add("Collaboration");}});
        this.classTabConvertion.put("CollaborationUse",  new ArrayList<String>(){{add("CollaborationUse");}});
        this.classTabConvertion.put("Component",  new ArrayList<String>(){{add("Component");}});
        this.classTabConvertion.put("ConditionalNode",  new ArrayList<String>(){{add("ConditionalNode");}});
        this.classTabConvertion.put("Connector",  new ArrayList<String>(){{add("Connector");}});
        this.classTabConvertion.put("ConnectorEnd",  new ArrayList<String>(){{add("ConnectorEnd");add("Slot");}});
        this.classTabConvertion.put("Constraint",  new ArrayList<String>(){{add("Constraint");}});
        this.classTabConvertion.put("ControlFlow",  new ArrayList<String>(){{add("ControlFlow");}});
        this.classTabConvertion.put("DataFlow",  new ArrayList<String>(){{add("InformationFlow");}});
        this.classTabConvertion.put("DataStoreNode",  new ArrayList<String>(){{add("DataStoreNode");}});
        this.classTabConvertion.put("DataType",  new ArrayList<String>(){{add("DataType"); add("PrimitiveType");}});
        this.classTabConvertion.put("DecisionMergeNode",  new ArrayList<String>(){{add("DecisionNode"); add("MergeNode");}});
        this.classTabConvertion.put("DeepHistoryPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("Dependency",  new ArrayList<String>(){{add("Dependency"); add("Extend"); add("Include");}});
        this.classTabConvertion.put("ElementImport",  new ArrayList<String>(){{add("ElementImport");}});
        this.classTabConvertion.put("ElementRealization",  new ArrayList<String>(){{add("Realization");}});
        this.classTabConvertion.put("EntryPointPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("Enumeration",  new ArrayList<String>(){{add("Enumeration");}});
        this.classTabConvertion.put("EnumerationLiteral",  new ArrayList<String>(){{add("EnumerationLiteral");}});
        this.classTabConvertion.put("Event",  new ArrayList<String>(){{add("CallEvent"); add("SignalEvent");
        add("TimeEvent"); add("ChangeEvent"); add("CreationEvent"); add("DestructionEvent"); add("ExecutionEvent");}});
        this.classTabConvertion.put("ExecutionOccurenceSpecification",  new ArrayList<String>(){{add("ExecutionOccurenceSpecification");}});
        this.classTabConvertion.put("ExecutionSpecification",  new ArrayList<String>(){{add("ExecutionSpecification");}});
        this.classTabConvertion.put("ExitPointPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("ExtensionPoint",  new ArrayList<String>(){{add("ExtensionPoint");}});
        this.classTabConvertion.put("Feature",  new ArrayList<String>(){{add("Feature");}});
        this.classTabConvertion.put("FinalState",  new ArrayList<String>(){{add("FinalState");}});
        this.classTabConvertion.put("FlowFinalNode",  new ArrayList<String>(){{add("FlowFinalNode");}});
        this.classTabConvertion.put("ForkJoinNode",  new ArrayList<String>(){{add("ForkNode"); add("JoinNode");}});
        this.classTabConvertion.put("ForkPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("Generalization",  new ArrayList<String>(){{add("Generalization");}});
        this.classTabConvertion.put("InformationFlow",  new ArrayList<String>(){{add("InformationFlow");}});
        this.classTabConvertion.put("InformationItem",  new ArrayList<String>(){{add("InformationItem");}});
        this.classTabConvertion.put("InitialNode",  new ArrayList<String>(){{add("InitialNode");}});
        this.classTabConvertion.put("InitialPseudoState",  new ArrayList<String>(){{add("PseudoState");}});      
        this.classTabConvertion.put("InputPin",  new ArrayList<String>(){{add("InputPin");add("ValuePin");
        add("ExpansionNode");add("ActionInputPin");}});
        this.classTabConvertion.put("Instance",  new ArrayList<String>(){{add("InstanceSpecification");}});
        this.classTabConvertion.put("InstanceNode",  new ArrayList<String>(){{add("CentralBufferNode");}});
        this.classTabConvertion.put("Interaction",  new ArrayList<String>(){{add("Interaction");}});
        this.classTabConvertion.put("Interface",  new ArrayList<String>(){{add("Interface");}});
        this.classTabConvertion.put("InterfaceRealization",  new ArrayList<String>(){{add("InterfaceRealization");}});
        this.classTabConvertion.put("InternalTransition",  new ArrayList<String>(){{add("Transition");}});
        this.classTabConvertion.put("InterruptibleActivityRegion",  new ArrayList<String>(){{add("InterruptibleActivityRegion");}});
        this.classTabConvertion.put("JoinPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("JunctionPseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("Lifeline",  new ArrayList<String>(){{add("Lifeline");}});
        this.classTabConvertion.put("Link",  new ArrayList<String>(){{add("InstanceSpecification");}});
        this.classTabConvertion.put("LinkEnd",  new ArrayList<String>(){{add("Slot");}});
        this.classTabConvertion.put("LoopNode",  new ArrayList<String>(){{add("LoopNode");}});
        this.classTabConvertion.put("Manifestation",  new ArrayList<String>(){{add("Manifestation");}});
        this.classTabConvertion.put("ModelElement",  new ArrayList<String>(){{add("NamedElement");}});
        this.classTabConvertion.put("Node",  new ArrayList<String>(){{add("Node");}});
        this.classTabConvertion.put("Note",  new ArrayList<String>(){{add("Comment");}});
        this.classTabConvertion.put("ObjectFlow",  new ArrayList<String>(){{add("ObjectFlow");add("ExceptionHandler");}});
        this.classTabConvertion.put("ObjectNode",  new ArrayList<String>(){{add("ObjectNode");}});
        this.classTabConvertion.put("OpaqueAction",  new ArrayList<String>(){{add("OpaqueAction");add("ValueSpecificationAction");}});
        this.classTabConvertion.put("Operation",  new ArrayList<String>(){{add("Operation");add("Reception");
        add("RedefinableTemplateSignature");add("TemplateSignature");}});
        this.classTabConvertion.put("OutputPin",  new ArrayList<String>(){{add("OutputPin");}});
        this.classTabConvertion.put("Package",  new ArrayList<String>(){{add("Package");}});
        this.classTabConvertion.put("PackageImport",  new ArrayList<String>(){{add("PackageImport");}});
        this.classTabConvertion.put("PackageMerge",  new ArrayList<String>(){{add("PackageMerge");}});
        this.classTabConvertion.put("Parameter",  new ArrayList<String>(){{add("Parameter");add("ClassifierTemplateParameter");
        add("OperationTemplateParameter");}});
        this.classTabConvertion.put("Pin",  new ArrayList<String>(){{add("Pin");}});
        this.classTabConvertion.put("Port",  new ArrayList<String>(){{add("Port");}});
        this.classTabConvertion.put("RaisedException",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("Region",  new ArrayList<String>(){{add("Region");}});
        this.classTabConvertion.put("RequirementContainer",  new ArrayList<String>(){{add("Package");}});
        this.classTabConvertion.put("SendSignalAction",  new ArrayList<String>(){{add("SendSignalAction");}});
        this.classTabConvertion.put("Signal",  new ArrayList<String>(){{add("Signal");}});
        this.classTabConvertion.put("State",  new ArrayList<String>(){{add("State");}});
        this.classTabConvertion.put("StateMachine",  new ArrayList<String>(){{add("StateMachine");add("ProtocolStateMachine");}});
        this.classTabConvertion.put("StructuredActivityNode",  new ArrayList<String>(){{add("StructuredActivityNode");add("ExpansionRegion");}});
        this.classTabConvertion.put("Substitution",  new ArrayList<String>(){{add("Substitution");}});
        this.classTabConvertion.put("TemplateBinding",  new ArrayList<String>(){{add("TemplateBinding");}});
        this.classTabConvertion.put("TemplateParameter",  new ArrayList<String>(){{add("TemplateParameter");add("ConnectableElementTemplateParameter");}});
        this.classTabConvertion.put("TemplateParameterSubstitution",  new ArrayList<String>(){{add("TemplateParameterSubstitution");}});
        this.classTabConvertion.put("TerminatePseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("Transition",  new ArrayList<String>(){{add("Transition");add("ProtocolTransition");}});
        this.classTabConvertion.put("Usage",  new ArrayList<String>(){{add("Usage");}});
        this.classTabConvertion.put("UseCase",  new ArrayList<String>(){{add("UseCase");}});
        this.classTabConvertion.put("UseCaseDependency",  new ArrayList<String>(){{add("Extend");add("Include");}});
    }

    @objid ("0cb089ea-3b71-11e1-8577-0027103f347d")
    public void setReportModel(final ReportModel newReport) {
        this.report = newReport;
    }

    @objid ("0cb0b0fe-3b71-11e1-8577-0027103f347d")
    public ReportModel getReportModel() {
        return this.report;
    }

    @objid ("d7d4c5f0-71bf-11e1-bffd-0027103f347d")
    public boolean isSysMLApplied() {
        return this.sysmlApplied;
    }

    @objid ("d7d4ecfd-71bf-11e1-bffd-0027103f347d")
    public void setSysMLApplied() {
        this.sysmlApplied = true;
    }

}
