/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import com.modeliosoft.modelio.api.model.IModelingSession;
import com.modeliosoft.modelio.api.model.IUmlModel;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityEdge;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IObjectFlow;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMIStereotypes;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ExceptionHandler;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.ObjectNode;

@objid ("6e55372f-1a76-11e1-8103-0027103f347d")
public class ModelUtils {

    /**
     * @param assocEnd : the tested associationEnd
     * @return true if the association is tagged as "Owned by the classifier"
     */
    @objid ("6e55ac64-1a76-11e1-8103-0027103f347d")
    public static boolean isOwnedByClassifier(final IAssociationEnd assocEnd) {
        for (ITaggedValue taggedValue : assocEnd.getTag()){
            if (taggedValue.getDefinition().getName().equals(XMITagTypes.ASSOCIATIONEND_XMIISOWNEDBYCLASSIFIER)){
                for (ITagParameter actual : taggedValue.getActual()){
                    if (actual.getValue().equals("true"))
                        return true;
                }
            }
        }
        return false;
    }

    /**
     * @param ecoreElt : the ecore ExceptionHandler
     * @return the corresponding element
     */
    @objid ("6e55d376-1a76-11e1-8103-0027103f347d")
    public static IObjectFlow createExceptionHandler(final ExceptionHandler ecoreElt) {
        IObjectFlow controlFlow = Modelio.getInstance().getModelingSession().getModel().createObjectFlow();
        
        try{
            controlFlow.addExtension(Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotype(IObjectFlow.class, XMIStereotypes.UML2EXCEPTIONHANDLER));
        }catch(Exception e){
            e.printStackTrace(Modelio.err);
        }
        
        setProtectedNode(controlFlow, ecoreElt);
        setHandlerBody(controlFlow, ecoreElt);    
        setExceptionType(controlFlow, ecoreElt);    
        setExceptionInput(controlFlow, ecoreElt);
        if (sourceAndTargetExist(controlFlow))
            return controlFlow;
        else{
            Modelio.getInstance().getModelingSession().getModel().deleteElement(controlFlow);
            return null;
        }
    }

    @objid ("6e55d37b-1a76-11e1-8103-0027103f347d")
    private static void setExceptionInput(final IObjectFlow objingElt, final ExceptionHandler ecoreElt) {
        ObjectNode ecoreTarget = ecoreElt.getExceptionInput();
        if (ecoreTarget != null) {
            Object objingTarget = ReverseProperties.getInstance().getMappedElement(ecoreTarget);
            if (objingTarget instanceof IActivityNode)
                ((IActivityEdge) objingElt).setTarget((IActivityNode) objingTarget);
        }
    }

    @objid ("6e56219a-1a76-11e1-8103-0027103f347d")
    private static void setExceptionType(final IObjectFlow objingElt, final ExceptionHandler ecoreElt) {
        IModelingSession session = Modelio.getInstance().getModelingSession();
        IUmlModel model = session.getModel();
        for (Classifier type : ecoreElt.getExceptionTypes()){
            IModelElement objType = (IModelElement) ReverseProperties.getInstance().getMappedElement(type);
            if (objType != null) {
                IDependency dependency  = model.createDependency();
                dependency.setDependsOn(objType);
                dependency.setImpacted(objingElt);
                try{
                    dependency.addExtension(session.getMetamodelExtensions().getStereotype(IDependency.class, XMIStereotypes.UML2EXCEPTIONTYPEREFERENCE));
                }catch(Exception e){
                    e.printStackTrace(Modelio.err);
                }
            }
        }
    }

    @objid ("6e5648ac-1a76-11e1-8103-0027103f347d")
    private static void setHandlerBody(final IObjectFlow objingElt, final ExceptionHandler ecoreElt) {
        ExecutableNode ecoreTarget = ecoreElt.getHandlerBody();
        if (ecoreTarget != null) {
            Object objingTarget = ReverseProperties.getInstance().getMappedElement(ecoreTarget);
            if ((objingTarget != null) && (objingTarget instanceof IActivityNode))
                ((IActivityEdge) objingElt).setTarget((IActivityNode) objingTarget);
        }
    }

    @objid ("6e566fbe-1a76-11e1-8103-0027103f347d")
    private static void setProtectedNode(final IObjectFlow objingElt, final ExceptionHandler ecoreElt) {
        ExecutableNode ecoreSource = ecoreElt.getProtectedNode();
        if (ecoreSource != null) {
            Object objingSource = ReverseProperties.getInstance().getMappedElement(ecoreSource);
            if ((objingSource != null) && (objingSource instanceof IActivityNode)){
                ((IActivityEdge) objingElt).setSource((IActivityNode) objingSource);
            }
        }
    }

    @objid ("6e5696d0-1a76-11e1-8103-0027103f347d")
    private static boolean sourceAndTargetExist(final IObjectFlow objingElt) {
        return ((objingElt.getSource() != null) && (objingElt.getTarget() != null));
    }

    /**
     * @param elt : the tested Modelio element
     * @return false is tagged as "NotExported"
     */
    @objid ("6e56bde2-1a76-11e1-8103-0027103f347d")
    public static boolean mustBeExported(final IElement elt) {
        if (elt instanceof IModelElement){
            for (ITaggedValue tag : ((IModelElement) elt).getTag()){
                if (tag.getDefinition().getName().equals(XMITagTypes.MODELELEMENT_NOTEXPORTED))
                    return false;
            }
        }
        return true;
    }

    @objid ("6e56e4f4-1a76-11e1-8103-0027103f347d")
    public static ITaggedValue getTaggedValue(final IModelElement element, final String tagTypeName) {
        for (ITaggedValue taggedValueTemp : element.getTag()){
            if (taggedValueTemp.getDefinition().getName().equals(tagTypeName)){
                return taggedValueTemp;
            }
        }
        return null;
    }

}
