/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  

package com.modeliosoft.modelio.xmi.util;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import com.modeliosoft.modelio.api.model.IMetamodelExtensions;
import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IActivityAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ICallBehaviorAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.ICallOperationAction;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IDecisionMergeNode;
import com.modeliosoft.modelio.api.model.uml.behavior.activity.IForkJoinNode;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehavior;
import com.modeliosoft.modelio.api.model.uml.behavior.common.IBehaviorParameter;
import com.modeliosoft.modelio.api.model.uml.behavior.common.ISignal;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionOccurenceSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IExecutionSpecification;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteractionFragment;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessage;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IMessageEnd;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ObMessageSortEnum;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IInternalTransition;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IRegion;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IState;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateMachine;
import com.modeliosoft.modelio.api.model.uml.behavior.state.IStateVertex;
import com.modeliosoft.modelio.api.model.uml.behavior.state.ITransition;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IUsage;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IAttribute;
import com.modeliosoft.modelio.api.model.uml.statik.IAttributeLink;
import com.modeliosoft.modelio.api.model.uml.statik.IBindableInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IClassAssociation;
import com.modeliosoft.modelio.api.model.uml.statik.IClassifier;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IConnector;
import com.modeliosoft.modelio.api.model.uml.statik.IConnectorEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IElementImport;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumeration;
import com.modeliosoft.modelio.api.model.uml.statik.IEnumerationLiteral;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralization;
import com.modeliosoft.modelio.api.model.uml.statik.IInstance;
import com.modeliosoft.modelio.api.model.uml.statik.IInterface;
import com.modeliosoft.modelio.api.model.uml.statik.IInterfaceRealization;
import com.modeliosoft.modelio.api.model.uml.statik.ILink;
import com.modeliosoft.modelio.api.model.uml.statik.ILinkEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IManifestation;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.INode;
import com.modeliosoft.modelio.api.model.uml.statik.IOperation;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.model.uml.statik.IPackageImport;
import com.modeliosoft.modelio.api.model.uml.statik.IParameter;
import com.modeliosoft.modelio.api.model.uml.statik.IPort;
import com.modeliosoft.modelio.api.model.uml.usecase.IActor;
import com.modeliosoft.modelio.api.model.uml.usecase.IUseCase;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.NoteTypeNotFoundException;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.core.app.O;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.plugin.XMITagTypes;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;

import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;

/**
 * Services for the Modelio model
 * @author ebrosse
 */
@objid ("1c7e50ef-4583-11e0-b54c-00137279a832")
public abstract class ObjingModelNavigation {
	@objid ("1c7e50f0-4583-11e0-b54c-00137279a832")
	public static final String OBJING_NULL_VALUE = "null";

	@objid ("1c7e50f2-4583-11e0-b54c-00137279a832")
	public static final String OBJING_UNLIMITED_VALUE = "*";


	@objid ("1c7e50f6-4583-11e0-b54c-00137279a832")
	public static IDataType getPrimitiveType(String ecoreTypeName) {
		if (ecoreTypeName != null) {

			IUMLTypes umlTypes = Modelio.getInstance().getModelingSession()
					.getModel().getUmlTypes();
			IDataType obBOOLEAN, obBYTE, obCHAR, obDATE, obDOUBLE, obFLOAT, obSHORT, obINTEGER, obLONG, obSTRING, obUNDEFINED;

			obBOOLEAN = umlTypes.getBOOLEAN();
			obBYTE = umlTypes.getBYTE();
			obCHAR = umlTypes.getCHAR();
			obINTEGER = umlTypes.getINTEGER();
			//            REAL = umlTypes.REAL;
			obDATE = umlTypes.getDATE();
			obDOUBLE = umlTypes.getDOUBLE();
			obFLOAT = umlTypes.getFLOAT();
			obLONG = umlTypes.getLONG();
			obSHORT = umlTypes.getSHORT();
			obSTRING = umlTypes.getSTRING();
			obUNDEFINED = umlTypes.getUNDEFINED();

			if ((obBOOLEAN != null) && (ecoreTypeName.equals(obBOOLEAN.getName())))
				return obBOOLEAN;
			else if  ((obCHAR != null) &&(ecoreTypeName.equals(obCHAR.getName())))
				return obCHAR;
			else if ((obINTEGER != null) && (ecoreTypeName.equals(obINTEGER.getName())))
				return obINTEGER;
			else if ((obBYTE != null) &&(ecoreTypeName.equals(obBYTE.getName())))
				return obBYTE;
			else if ((obDATE != null) &&(ecoreTypeName.equals(obDATE.getName())))
				return obDATE;
			else if ((obDOUBLE != null) &&(ecoreTypeName.equals(obDOUBLE.getName())))
				return obDOUBLE;
			else if ((obFLOAT != null) &&(ecoreTypeName.equals(obFLOAT.getName())))
				return obFLOAT;
			else if ((obFLOAT != null) &&(ecoreTypeName.equals("real")))
				return obFLOAT;
			else if ((obLONG != null) &&(ecoreTypeName.equals(obLONG.getName())))
				return obLONG;
			else if ((obSHORT != null) &&(ecoreTypeName.equals(obSHORT.getName())))
				return obSHORT;
			else if ((obSTRING != null) &&(ecoreTypeName.equals(obSTRING.getName())))
				return obSTRING;
			else if ((obUNDEFINED != null) &&(ecoreTypeName.equals(obUNDEFINED.getName())))
				return obUNDEFINED;
		}
		return null;
	}


	public static IModelElement searchElement(Element ecoreElt){
		String ecoreId = ObjingEAnnotation.getEcoreId(ecoreElt);
		if ((ecoreId != null) && (!ecoreId.equals(""))){
			List<IElement> modelElts =  Modelio.getInstance().getModelingSession().findByClass(IModelElement.class);
			for (IElement elt : modelElts){
				IModelElement modelElt = (IModelElement) elt;
				for (ITaggedValue tag : modelElt.getTag()){
					if (tag.getDefinition().getName().equals(XMITagTypes.MODELELEMENT_ECOREID) &&
							(tag.getQualifier() != null ) &&
							tag.getQualifier().getValue().equals(ecoreId)){

						TotalImportMap.getInstance().put(ecoreElt, modelElt);
						return modelElt;

					}

				}
			}

		}

		return null; 
	}


	@objid ("1c80b347-4583-11e0-b54c-00137279a832")
	public static INoteType getNoteType(String baseMetaclassName, String noteTypeName) {
		INoteType noteType = null;
		try {
			noteType = Modelio
					.getInstance()
					.getModelingSession()
					.getMetamodelExtensions()
					.getNoteType((Class<IElement>) 
							(Class.forName( baseMetaclassName)), noteTypeName);
		} catch (NoteTypeNotFoundException e) {
			XMILogs xmiLogs = XMILogs.getInstance();
			xmiLogs.writelnInLog(XMI.I18N.getMessage(
					"logFile.warning.noteTypeNotFoundExceptionWithOutCreation", noteTypeName));

		} catch (ClassNotFoundException e) {
			e.printStackTrace(Modelio.err); 
		}
		return noteType;
	}

	@objid ("1c80b34d-4583-11e0-b54c-00137279a832")
	public static ObList<IStereotype> getAllStereotypes() {
		List<IStereotype> stereoList = new ArrayList<IStereotype>();

		List<Class<? extends IElement>> metaclasses = getAllMetaclasses();

		IMetamodelExtensions ext = Modelio.getInstance()
				.getModelingSession().getMetamodelExtensions();

		for (Class<? extends IElement> metaclass : metaclasses) {
			try {
				ObList<IStereotype> stereoListForCurrentMetaClass = ext
						.getStereotypes(metaclass);
				if (stereoListForCurrentMetaClass != null
						&& stereoListForCurrentMetaClass.size() > 0) {
					for (IStereotype s : stereoListForCurrentMetaClass) {
						if (!stereoList.contains(s)) {
							stereoList.add(s);
						}
					}
				}
			} catch (NullPointerException e) {
				Modelio.err.println("No Stereotypes defined on \""
						+ metaclass.getSimpleName() + "\".");
			}
		}
		return new ObList<IStereotype>(stereoList);
	}

	@objid ("1c80b353-4583-11e0-b54c-00137279a832")
	public static ObList<ITagType> getAllTagTypes() {
		List<ITagType> tagTypeList = new ArrayList<ITagType>();

		List<Class<? extends IElement>> metaclasses = getAllMetaclasses();

		IMetamodelExtensions ext = Modelio.getInstance()
				.getModelingSession().getMetamodelExtensions();

		for (Class<? extends IElement> metaclass : metaclasses) {
			try {
				ObList<ITagType> tagTypes = ext.getTagTypes(metaclass);
				if (tagTypes != null && tagTypes.size() > 0) {
					for (ITagType tagType : tagTypes) {
						if (!tagTypeList.contains(tagType))
							tagTypeList.add(tagType);
					}
				}
			} catch (NullPointerException e) {
				Modelio.err.println("No TagTypes defined on \""
						+ metaclass.getSimpleName() + "\".");
			}
		}
		return new ObList<ITagType>(tagTypeList);
	}

	@objid ("1c8315a4-4583-11e0-b54c-00137279a832")
	public static ObList<INoteType> getAllNoteTypes() {
		List<INoteType> noteTypeList = new ArrayList<INoteType>();

		List<Class<? extends IElement>> metaclasses = getAllMetaclasses();

		IMetamodelExtensions ext = Modelio.getInstance()
				.getModelingSession().getMetamodelExtensions();

		for (Class<? extends IElement> metaclass : metaclasses) {
			try {
				ObList<INoteType> noteTypes = ext.getNoteTypes(metaclass);
				if (noteTypes != null && noteTypes.size() > 0) {
					for (INoteType tagType : noteTypes) {
						if (!noteTypeList.contains(tagType))
							noteTypeList.add(tagType);
					}
				}
			} catch (NullPointerException e) {
				Modelio.err.println("No NoteTypes defined on \""
						+ metaclass.getSimpleName() + "\".");
			}
		}
		return new ObList<INoteType>(noteTypeList);
	}

	@objid ("1c8315aa-4583-11e0-b54c-00137279a832")
	private static List<Class<? extends IElement>> getAllMetaclasses() {
		String pkgName = "com.modeliosoft.modelio.api.model";

		String jarPath = getJarPath(pkgName);

		if (jarPath != null) {
			List<Class<? extends IElement>> classList = getClassesInPackageFromJar(jarPath, pkgName);

			return classList;
		}
		return null;
	}

	@objid ("1c8577fa-4583-11e0-b54c-00137279a832")
	private static String getJarPath(String pkgName) {
		File file = null;
		try {
			file = getFileFromPackage(pkgName);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		if (file != null) {
			String path = file.getPath();

			int index = path.indexOf(".jar!");
			String jarPath = path.substring(0, index + 4);
			if (jarPath.startsWith("file:/") || jarPath.startsWith("file:\\"))
				jarPath = jarPath.substring(6);

			return jarPath;
		}
		return null;
	}

	@objid ("1c8577ff-4583-11e0-b54c-00137279a832")
	private static File getFileFromPackage(String pckgname) throws ClassNotFoundException {
		File directory = null;
		try {
			ClassLoader cld = Thread.currentThread().getContextClassLoader();
			if (cld == null) {
				throw new ClassNotFoundException("Can't get class loader.");
			}
			String path = pckgname.replace('.', '/');
			URL resource = cld.getResource(path);

			if (resource == null) {
				throw new ClassNotFoundException("No resource for " + path);
			}
			directory = new File(resource.getFile());
			return directory;
		} catch (NullPointerException x) {
			throw new ClassNotFoundException(pckgname + " (" + directory
					+ ") does not appear to be a valid package");
		}
	}

	@objid ("1c857803-4583-11e0-b54c-00137279a832")
	private static List<Class<? extends IElement>> getClassesInPackageFromJar(final String jarName, final String packageName) {
		List<Class<? extends IElement>> classList = new ArrayList<Class<? extends IElement>>();

		try {
			JarInputStream jarFile = new JarInputStream(new FileInputStream(
					jarName));
			JarEntry jarEntry;
			final String jonimodelEntryName = packageName + "/";

			while (true) {
				jarEntry = jarFile.getNextJarEntry();
				if (jarEntry == null) {
					break;
				}

				if (jarEntry.getName().equals(jonimodelEntryName)) {
					JarEntry jonimodelEntry = jarEntry;
					while (true) {
						jonimodelEntry = jarFile.getNextJarEntry();
						if (jonimodelEntry == null)
							break;

						if (jonimodelEntry.getName().endsWith(".class")) {
							String metaclassNamespace = getNamespace(jonimodelEntry
									.getName());

							String metaclassName = getSimpleName(metaclassNamespace);

							if (metaclassNamespace.contains("model."
									+ metaclassName)) {
								Class<? extends IElement> metaclass = (Class<IElement>) Class.forName(metaclassNamespace);

								if (IModelElement.class
										.isAssignableFrom(metaclass))
									classList.add(metaclass);
							}
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return classList;
	}

	@objid ("1c87da54-4583-11e0-b54c-00137279a832")
	private static String getNamespace(String jarEntryName) {
		String namespace = jarEntryName;
		namespace = namespace.replaceAll("/", "\\.");
		String separator = "/";

		int index = namespace.lastIndexOf(separator);
		namespace = namespace.substring(index + 1);

		if (namespace.endsWith(".class"))
			namespace = namespace.substring(0, namespace.length() - 6);
		return namespace;
	}

	@objid ("1c87da59-4583-11e0-b54c-00137279a832")
	private static String getSimpleName(String namespace) {
		int index = namespace.lastIndexOf(".");
		namespace = namespace.substring(index + 1, namespace.length());
		return namespace;
	}

	@objid ("1c87da5e-4583-11e0-b54c-00137279a832")
	public static List<IAssociationEnd> getOppositeAssociationEnds(IAssociationEnd a1) {
		IAssociation assoc = a1.getRelated();
		if (assoc != null)
			return assoc.getConnection().select(new ExcludeElementFilter(a1));
		else
			return null;
	}

	@objid ("1c87da64-4583-11e0-b54c-00137279a832")
	public static List<ILinkEnd> getOppositeLinkEnds(ILinkEnd a1) {
		ILink assoc = a1.getLinkNode();
		if (assoc != null)
			return assoc.getConnection().select(new ExcludeElementFilter(a1));
		else
			return null;
	}

	@objid ("1c8a3cb8-4583-11e0-b54c-00137279a832")
	public static boolean isCreate(IOperation operation) {
		return isStereotyped(operation, XMI.I18N
				.getString("objing.java.stereotype.constructor"));
	}

	@objid ("1c8a3cbd-4583-11e0-b54c-00137279a832")
	public static boolean isDestroy(IOperation operation) {
		return isStereotyped(operation, XMI.I18N
				.getString("objing.java.stereotype.destructor"));
	}

	@objid ("1c8c9f09-4583-11e0-b54c-00137279a832")
	public static IOperation getOwnerOperation(IParameter parameter) {
		IOperation operation = parameter.getComposed();
		if (operation == null)
			operation = parameter.getReturned();
		return operation;
	}

	@objid ("1c8c9f0e-4583-11e0-b54c-00137279a832")
	public static boolean isReturnParameter(IParameter parameter) {
		if (parameter != null)
			if (parameter.getReturned() != null)
				return true;
		return false;
	}

	@objid ("1c8c9f13-4583-11e0-b54c-00137279a832")
	public static IModelElement getImportingElement(IModelElement mdlEltImport) {
		IModelElement importingElt = null;
		if (mdlEltImport != null) {
			if (mdlEltImport instanceof IElementImport) {
				IElementImport eltImport = (IElementImport) mdlEltImport;
				importingElt = eltImport.getImportingOperation();
				if (importingElt == null)
					importingElt = eltImport.getImportingNameSpace();
			} else if (mdlEltImport instanceof IPackageImport) {
				IPackageImport pkgImport = (IPackageImport) mdlEltImport;
				importingElt = pkgImport.getImportingOperation();
				if (importingElt == null)
					importingElt = pkgImport.getImportingNameSpace();
			}
		}
		return importingElt;
	}

	@objid ("1c8f0168-4583-11e0-b54c-00137279a832")
	public static boolean isClassAssociation(IAssociation assoc) {
		if (assoc != null)
			if (assoc.getLinkToClass() != null)
				return true;
		return false;
	}

	@objid ("1c9163bc-4583-11e0-b54c-00137279a832")
	public static boolean isClassAssociation(IClass class_) {
		if (class_ != null) {
			ObList<IClassAssociation> classAssocList = class_
					.getLinkToAssociation();
			if (classAssocList != null && classAssocList.size() > 0)
				return true;
		}
		return false;
	}

	@objid ("1c9163c1-4583-11e0-b54c-00137279a832")
	public static boolean isClassAssociation(IAssociationEnd assocEnd) {
		if (assocEnd != null)
			if (assocEnd.getRelated().getLinkToClass() != null)
				return true;
		return false;
	}

	@objid ("1c93c616-4583-11e0-b54c-00137279a832")
	public static boolean isBinary(IAssociation assoc) {
		if (assoc != null) {
			ObList<IAssociationEnd> memberEnds = assoc.getConnection();
			if (memberEnds != null && memberEnds.size() == 2)
				return true;
		}
		return false;
	}

	@objid ("1c93c61b-4583-11e0-b54c-00137279a832")
	public static boolean isRepresentedByAUniqueClass(IClassAssociation classAssoc) {
		IClass objingClass = classAssoc.getClassPart();
		if (objingClass != null) {
			ObList<IClassAssociation> classAssocList = objingClass
					.getLinkToAssociation();
			if (classAssocList != null && classAssocList.size() == 1)
				return true;
		}
		return false;
	}

	@objid ("1c93c620-4583-11e0-b54c-00137279a832")
	public static IPackage getNearestPackage(IModelTree modelTreeElt) {
		if (modelTreeElt != null) {
			IModelTree owner = modelTreeElt.getOwner();
			if (owner instanceof IPackage)
				return (IPackage) owner;
			else
				return getNearestPackage(owner);
		}
		return null;
	}

	@objid ("1c962870-4583-11e0-b54c-00137279a832")
	public static void infoOfUnsupportedOwnedWithEMF(IElement objingOwner, IModelElement objingElement, Element ecoreElt) {
		infoOfUnsupportedOwnedWithEMF(objingOwner,objingElement);
		if (ecoreElt != null)
			ecoreElt.destroy();
	}

	@objid ("1c962875-4583-11e0-b54c-00137279a832")
	public static IStereotype getStereotype(Class<? extends IElement> metaclass, String stereotypeName) throws StereotypeNotFoundException {
		return Modelio.getInstance().getModelingSession()
				.getMetamodelExtensions().getStereotype(metaclass,
						stereotypeName);
	}

	@objid ("1c962882-4583-11e0-b54c-00137279a832")
	public static boolean isManifestationMappable(IManifestation manif) {
		IModelElement objingUtilizedElt = manif.getUtilizedElement();
		if (objingUtilizedElt != null) {

			// Gets the ecore utilized element:
			Element ecoreUtilizedElt = GenerationProperties.getInstance()
					.getMappedElement(objingUtilizedElt);

			return ecoreUtilizedElt instanceof PackageableElement;
		}
		return false;
	}

	@objid ("1c962887-4583-11e0-b54c-00137279a832")
	public static IInteraction getEnclosingInteractionRecursively_DEPRECATED(IInteractionFragment sequenceElt) {
		CompositionOwnerVisitor compositionVisitor = new CompositionOwnerVisitor();
		IElement owner = compositionVisitor.getCompositionOwner(sequenceElt);
		if (owner instanceof IInteraction) {
			return (IInteraction) owner;
		} else if (owner instanceof IInteractionFragment) {
			return getEnclosingInteractionRecursively_DEPRECATED((IInteractionFragment) owner);
		}
		// else
			// if (owner instanceof ICombinedFragment) {
		// return
		// getEnclosingInteractionRecursively((IInteractionFragment)owner);
		// }
		// else
		// if (owner instanceof IExecutionSpecification) {
		// return
		// getEnclosingInteractionRecursively((IInteractionFragment)owner);
		// }
		// else
		// if (owner instanceof IInteractionOperand) {
		//            
		// }
		// else
		// if (owner instanceof IPartDecomposition) {
		//            
		// }
		// else
		// if (owner instanceof IInteractionUse) {
		//            
		// }
		// else
		// if (owner instanceof IExecutionOccurenceSpecification) {
		//            
		// }
		// else
		// if (owner instanceof IGate) {
		//            
		// }
		// else
		// if (owner instanceof IMessageEnd) {
		//            
		// }
		// else
		// if (owner instanceof IStateInvariant) {
		//            
		// }
		// else
		// if (owner instanceof IOccurrenceSpecification) {
		//            
		// }
		else
			return null;
	}

	@objid ("1c96288b-4583-11e0-b54c-00137279a832")
	public static IMessage getReplyMessage(IMessage message) {
		boolean isSynchronous = false;
		ObMessageSortEnum msgSort = message.getSortOfMessage();
		if (msgSort == ObMessageSortEnum.SYNC_CALL)
			isSynchronous = true;

		if (isSynchronous) {
			IMessageEnd receiveEnd = message.getReceiveEvent();
			ILifeline coveredLL = receiveEnd.getCovered().get(0);

			ObList<IInteractionFragment> coveringFragments = coveredLL
					.getCoveredBy();
			int index = coveringFragments.indexOf(receiveEnd);
			int fragmentsNumber = coveringFragments.size();
			int indexOfEOS = index + 1;
			IInteractionFragment next = coveringFragments.get(indexOfEOS);
			while (indexOfEOS < fragmentsNumber - 1
					&& !(next instanceof IExecutionSpecification)) {
				next = coveringFragments.get(++indexOfEOS);
			}

			if (next instanceof IExecutionSpecification) {
				IExecutionOccurenceSpecification finishEnd = ((IExecutionSpecification) next)
						.getFinish();
				return getMessage(finishEnd);
			}
		}
		return null;
	}

	@objid ("1c962890-4583-11e0-b54c-00137279a832")
	public static IMessage getMessage(IMessageEnd end) {
		IMessage message = end.getSentMessage();
		if (message == null)
			message = end.getReceivedMessage();
		return message;
	}

	@objid ("1c962895-4583-11e0-b54c-00137279a832")
	public static IInteractionFragment getConnectedFragment(IModelElement sequenceElt) {
		IInteractionFragment connectedFragment = null;
		if (sequenceElt instanceof IMessage) {
			IMessageEnd end = ((IMessage) sequenceElt).getReceiveEvent();
			if (end == null)
				end = ((IMessage) sequenceElt).getSendEvent();

			if (end != null)
				connectedFragment = end;
		}
		return connectedFragment;
	}

	@objid ("1c988acb-4583-11e0-b54c-00137279a832")
	public static Boolean IsInstanceOwner(IBindableInstance bindableInstance) {
		IElement owner = bindableInstance.getCompositionOwner();
		while ((owner != null) && (owner instanceof IBindableInstance)){    
			bindableInstance = (IBindableInstance) owner;
			owner = bindableInstance.getCompositionOwner();
		}
		return (owner instanceof IInstance);
	}

	@objid ("1c988ad0-4583-11e0-b54c-00137279a832")
	public static IElement getBindableInstanceOwner(IElement bindableInstance) {
		IElement owner = bindableInstance.getCompositionOwner();
		while ((owner != null) && (owner instanceof IBindableInstance)){    
			bindableInstance = (IBindableInstance) owner;
			owner = bindableInstance.getCompositionOwner();
		}
		return owner;
	}

	@objid ("1c988ad5-4583-11e0-b54c-00137279a832")
	public static IElement getConnectorOwner(IConnector connector) {
		IElement owner = null;
		for (IElement connectorEnd : connector.getConnection()){
			if (connectorEnd instanceof IConnectorEnd){
				IElement temp = getBindableInstanceOwner(((IConnectorEnd)connectorEnd).getLinked());
				if (owner == null){
					owner = temp;
				}else {
					if (!owner.equals(temp)){
						return null;
					}
				}
			}
		}
		return owner;
	}

	@objid ("1c9aed25-4583-11e0-b54c-00137279a832")
	public static <T extends com.modeliosoft.modelio.api.model.uml.infrastructure.IElement> IElement getEnclosingElement(IElement child, Class<?> typeOfEnclosing, CompositionOwnerVisitor ownerVisitor) {
		IElement owner = ownerVisitor.getCompositionOwner(child);

		while (owner != null) {
			if (typeOfEnclosing.isAssignableFrom(owner.getClass()))
				return owner;
			else
				owner = ownerVisitor.getCompositionOwner(owner);
		}
		return null;
	}

	@objid ("1ca93b45-4583-11e0-b54c-00137279a832")
	public static ObList<IParameter> getRelatedParameters(IActivityAction objingOwner) {
		List<IParameter> paramList = null;

		if (objingOwner instanceof ICallOperationAction) {
			IOperation operation = ((ICallOperationAction) objingOwner)
					.getCalled();
			if (operation != null)
				paramList = operation.getIO();
		} else if (objingOwner instanceof ICallBehaviorAction) {
			IBehavior behavior = ((ICallBehaviorAction) objingOwner)
					.getCalled();
			if (behavior != null) {
				paramList = new ArrayList<IParameter>();
				for (IBehaviorParameter param : behavior.getParameter()) {
					paramList.add(param);
				}
			}
		}

		if (paramList == null)
			paramList = new ArrayList<IParameter>();
		return new ObList<IParameter>(paramList);
	}

	@objid ("1ca93b4c-4583-11e0-b54c-00137279a832")
	public static boolean isMergeNode(IDecisionMergeNode node) {
		int incomings = node.getIncoming().size();
		int outgoings = node.getOutgoing().size();

		if (incomings > 1 && outgoings <= 1)
			return true;
		else
			return false;
	}

	@objid ("1cab9d9d-4583-11e0-b54c-00137279a832")
	public static boolean isDecisionNode(IDecisionMergeNode node) {
		if (node.getIncoming().size() <= 1)
			return true;
		else
			return false;
	}

	@objid ("1cab9da1-4583-11e0-b54c-00137279a832")
	public static boolean isJoinNode(IForkJoinNode node) {
		int incomings = node.getIncoming().size();
		int outgoings = node.getOutgoing().size();

		if (incomings > 1 && outgoings <= 1)
			return true;
		else
			return false;
	}

	@objid ("1cab9da5-4583-11e0-b54c-00137279a832")
	public static boolean isForkNode(IForkJoinNode node) {
		if (node.getIncoming().size() <= 1)
			return true;
		else
			return false;
	}

	@objid ("1cadfff4-4583-11e0-b54c-00137279a832")
	public static boolean isProtocolStateMachine(IStateMachine element) {
		boolean isProtocol = false;
		switch (element.getKind()) {
		case PROTOCOL:
			isProtocol = true;
			break;
		default:
			isProtocol = false;
			break;
		}
		return isProtocol;
	}

	@objid ("1cadfffd-4583-11e0-b54c-00137279a832")
	public static boolean isJunctionState(IStateVertex state) {
		int incomings = state.getIncoming().size();
		int outgoings = state.getOutGoing().size();

		if (incomings > 1 && outgoings <= 1)
			return true;
		return false;
	}

	@objid ("1cb0624e-4583-11e0-b54c-00137279a832")
	public static boolean isChoiceState(IStateVertex state) {
		if (state.getIncoming().size() <= 1)
			return true;
		return false;
	}

	@objid ("1cb0625b-4583-11e0-b54c-00137279a832")
	public static boolean isProtocolTransition(ITransition transition) {
		if (!(transition instanceof IInternalTransition) && (transition.getProcessed() != null))
			return true;
		return false;
	}

	@objid ("1cb2c4ac-4583-11e0-b54c-00137279a832")
	public static IStateMachine getOwnerStateMachine(IState state) {
		IStateMachine sm = state.getSubMachine();
		if (sm == null)
			sm = getOwnerStateMachine(state.getParent());
		return sm;
	}

	@objid ("1cb2c4b1-4583-11e0-b54c-00137279a832")
	public static IStateMachine getOwnerStateMachine(IRegion region) {
		IStateMachine sm = region.getRepresented();
		if (sm == null)
			sm = getOwnerStateMachine(region.getParent());
		return sm;
	}

	@objid ("1cb9ebba-4583-11e0-b54c-00137279a832")
	public static IPackage getNearestPackage(IElement element) {
		IElement newOwner = element.getCompositionOwner();
		while (!(newOwner instanceof IPackage) ){
			newOwner = newOwner.getCompositionOwner();
		}
		return (IPackage) newOwner;
	}

	@objid ("1cbc4e11-4583-11e0-b54c-00137279a832")
	public static int countNumberOfLocalPofile() {
		int i= 0;

		for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
			for(IProfile profile : module.getOwnedProfile()){
				if (profile.getName().equals("LocalProfile")){
					i++;
				}
			}
		}
		return i;
	}

	@objid ("1cbc4e15-4583-11e0-b54c-00137279a832")
	public static boolean isNotOwnedByActor(IAssociation assoc) {
		int nbNoActor = 0;
		for (IAssociationEnd assocEnd : assoc.getConnection()){
			if (!(assocEnd.getOwner() instanceof IActor))
				nbNoActor++;
		}
		return nbNoActor >=2 ;
	}

	@objid ("1cbeb06a-4583-11e0-b54c-00137279a832")
	public static boolean mustBeExported(IStereotype obStereotype) {
		IProfile profile =  obStereotype.getOwner();


		if (profile.getOwnerModule().getName().equals("ModelerModule")  && profile.getName().equals("default") 
				&& ((obStereotype.getName().equals(ModelerModuleStereotypes.EXTEND)) || (obStereotype.getName().equals(ModelerModuleStereotypes.INCLUDE) || (obStereotype.getName().equals(ModelerModuleStereotypes.REQUIREMENT))
						|| (obStereotype.getName().equals(ModelerModuleStereotypes.REQUIREMENT_CONTAINER))
						|| (obStereotype.getName().equals(ModelerModuleStereotypes.DERIVE)) || (obStereotype.getName().equals(ModelerModuleStereotypes.SATISFY))|| (obStereotype.getName().equals(ModelerModuleStereotypes.PART))
						|| (obStereotype.getName().equals(ModelerModuleStereotypes.VERIFY)))))
			return false;
		return !(profile.getIdentifier().equals("01ec045c-0000-3739-0000-000000000000"));
	}

	@objid ("1cbeb06f-4583-11e0-b54c-00137279a832")
	public static boolean mustBeExported(IMetaclassReference metaclass) {
		IProfile profile =  metaclass.getOwnerProfile();
		return !(profile.getIdentifier().equals("01ec045c-0000-3739-0000-000000000000"));
	}

	@objid ("1cc112c4-4583-11e0-b54c-00137279a832")
	public static boolean mustBeExported(ITagType tagType) {
		if (tagType.getOwnerStereotype() != null){
			return mustBeExported(tagType.getOwnerStereotype());
		}else
			return mustBeExported(tagType.getOwnerReference());
	}

	@objid ("1cc112c8-4583-11e0-b54c-00137279a832")
	public static boolean mustBeExported(ITaggedValue taggedValue) {
		return mustBeExported(taggedValue.getDefinition());
	}

	@objid ("1cc3751e-4583-11e0-b54c-00137279a832")
	public static void setConnector(Message ecoreElt, IMessage objElt) {
		IMessageEnd end1 = objElt.getReceiveEvent();
		IMessageEnd end2 = objElt.getSendEvent();
		if ((end1 != null ) && (end2 != null) && (end1.getCovered().size() != 1) &&  (end2.getCovered().size() != 1)) {
			ILifeline lifeline1 = end1.getCovered().get(0);
			ILifeline lifeline2 = end2.getCovered().get(0);
			if ((lifeline1 != null) && (lifeline2 != null)){
				IInstance instance1 = lifeline1.getRepresented();
				IInstance instance2 = lifeline2.getRepresented();
				if ((instance1 != null) && (instance2 != null)){
					for (ILinkEnd linkEnd : instance1.getConnection()){
						ILink link = linkEnd.getLinkNode();
						for (ILinkEnd linkend2 : link.getConnection()){
							if (linkend2.getLinked().equals(instance2)){
								Element ecoreConnector = GenerationProperties.getInstance().getMappedElement(link);
								if (ecoreConnector instanceof Connector){
									ecoreElt.setConnector((Connector) ecoreConnector);

								}
								break;
							}
						}

					}
				}
			}

		}
	}

	@objid ("1cc37522-4583-11e0-b54c-00137279a832")
	public static String getTaggedValue(IModelElement owner, String tagType) {
		String result = "";
		for (ITaggedValue tagged : owner.getTag()){
			if (tagged.getDefinition().getName().equals(tagType)){
				if (tagged.getActual().size() > 0){
					result = tagged.getActual().get(0).getValue();
				}
				break;

			}
		}
		return result;
	}

	@objid ("1cc37528-4583-11e0-b54c-00137279a832")
	public static void attachAssociationEnd(IAssociationEnd objingEnd, Property ecoreElement) {
		try {
			List<Property> oppositeEnds = EcoreModelNavigation.getOppositeAssociationEnds(ecoreElement);

			int nbEnds = oppositeEnds.size();

			if (nbEnds == 1) {
				// Case of a binary Association
				attachToAssoc(objingEnd, oppositeEnds.get(0).getType());
			} else if (nbEnds > 1) {// (in Ecore, an association may have only
				// one member end (which does
				// not have any sense ...)
				// Case of a N ary association
				attachToAssoc(objingEnd, ecoreElement.getType());
			}
		} catch (AssociationNotFoundException e) {
			e.printStackTrace(Modelio.err);
			XMILogs.getInstance().writelnInLog(
					"The Property \"" + ecoreElement.getName()
					+ "\" is not connected to an Association.");
		}
	}

	@objid ("1cc5d778-4583-11e0-b54c-00137279a832")
	private static void attachToAssoc(IAssociationEnd objingEnd, Type ecoreType) {
		if (ecoreType != null) {
			Object objingOwner = ReverseProperties.getInstance().getMappedElement(ecoreType);
			if (objingOwner != null){
				//Rule Core R22
				if ((objingOwner instanceof IClass) || (objingOwner instanceof IInterface) || (objingOwner instanceof IComponent)
						|| (objingOwner instanceof IActor) || (objingOwner instanceof IDataType) || (objingOwner instanceof INode) 
						|| (objingOwner instanceof ISignal)|| (objingOwner instanceof IUseCase)){
					objingEnd.setOwner((IClassifier) objingOwner);
				}else{      
					XMILogs xmilogs = XMILogs.getInstance();
					xmilogs.writelnInLog(XMI.I18N.getMessage(
							"logFile.warning.unsupportedOwnerImport", objingEnd.getName(), 
							objingEnd.getClass().getName(), objingOwner.getClass().getName()));       
				}
			}
		}
	}

	@objid ("1cc5d77c-4583-11e0-b54c-00137279a832")
	public static Type getType(IAttributeLink attributeLink) {
		Type result = null;
		IAttribute attribute  = attributeLink.getBase();
		if (attribute != null){
			attribute.getType();
		}
		return result;
	}

	@objid ("1cc5d781-4583-11e0-b54c-00137279a832")
	public static IBindableInstance copyAssocEndToBindableInstance(IAssociationEnd initial, Property ecoreElt) {
		IBindableInstance copy = Modelio.getInstance().getModelingSession().getModel().createBindableInstance();
		setInstance(copy, ecoreElt);
		copy.setOwner(initial.getOwner());
		copy.setRepresentedFeature(initial);
		return copy;
	}

	@objid ("1cc839d5-4583-11e0-b54c-00137279a832")
	public static void attachPort(IElement objingElt, Port ecoreElement) {
		ReverseProperties revProp = ReverseProperties.getInstance();

		Element ecoreOwner = ecoreElement.getOwner();


		if (revProp.isRoundtripEnabled() && (ObjingEAnnotation.getOwner(ecoreElement) != null)){
			String ownerId = ObjingEAnnotation.getOwner(ecoreElement);
			for (Object object : ((Classifier)ecoreOwner).getAllAttributes()){
				Property property = (Property) object;
				List<String> ids = ObjingEAnnotation.getObjingIDs(property);
				if (ids.size() > 0){
					String id =  ids.get(0);
					if (id.equals(ownerId)){
						Object objOwner = revProp.getMappedElement(property);
						if (objOwner instanceof IBindableInstance)
							if (objingElt instanceof IBindableInstance){
								if (((IBindableInstance) objingElt).getInternalOwner()== null)
									((IBindableInstance) objingElt).setCluster((IInstance)objOwner);
								break;
							}else{
								((IAttributeLink) objingElt).setAttributed((IInstance)objOwner);
								break;
							}
					}
				}
			}
		} 

		if (ecoreOwner != null) {
			Object objOwner = ReverseProperties.getInstance().getMappedElement(ecoreOwner);
			if (objOwner instanceof IClassifier){
				if (((IBindableInstance) objingElt).getCluster()== null)
					((IPort) objingElt).setInternalOwner(((IClassifier) objOwner));
			}else if (objOwner instanceof IBindableInstance)
				if (((IBindableInstance) objingElt).getInternalOwner()== null)
					((IPort) objingElt).setCluster((IBindableInstance) objOwner);
				else if  (objOwner instanceof IAssociationEnd){
					objOwner = ObjingModelNavigation.copyAssocEndToBindableInstance(
							(IAssociationEnd) objOwner, (Property) ecoreOwner);
					if (((IBindableInstance) objingElt).getInternalOwner()== null)
						((IPort) objingElt).setCluster((IBindableInstance) objOwner);

				}

		}
	}

	@objid ("1cc839d9-4583-11e0-b54c-00137279a832")
	private static void setBase(IInstance objingElt, Property ecoreElt) {
		Type type =  ecoreElt.getType();
		if (type != null){
			IElement base =(IElement) ReverseProperties.getInstance().getMappedElement(type);
			if (base instanceof INameSpace)
				objingElt.setBase((INameSpace) base);
		}
	}

	@objid ("1cca9c2c-4583-11e0-b54c-00137279a832")
	public static void setRepresentedFeature(IInstance objingElt, Property ecoreElt) {
		if (objingElt instanceof IBindableInstance){

			EList<?> listProperty =  ecoreElt.getRedefinedProperties();

			if ((listProperty != null) &&(listProperty.size() > 0) ){
				Property represented = (Property) listProperty.get(0);
				IElement base =(IElement) ReverseProperties.getInstance().getMappedElement(represented);

				if ((base instanceof IModelElement) && (!base.equals(objingElt)))
					((IBindableInstance)objingElt).setRepresentedFeature((IModelElement)base);
			}
		}
	}

	@objid ("1cca9c30-4583-11e0-b54c-00137279a832")
	private static void setIsConstant(IInstance objingElt, Property ecoreElt) {
		objingElt.setConstant(ObjingEAnnotation.isConstant(ecoreElt));
	}

	@objid ("1cca9c34-4583-11e0-b54c-00137279a832")
	public static void setMultiMax(IInstance objingElt, Property ecoreElt) {
		String value =  ObjingEAnnotation.getMultiMax(ecoreElt);
		if (value != null)
			objingElt.setMultiplicityMax(value);
	}

	@objid ("1cccfe86-4583-11e0-b54c-00137279a832")
	private static void setValue(IInstance objingElt, Property ecoreElt) {
		String value =  ObjingEAnnotation.getValue(ecoreElt);
		if (value != null)
			objingElt.setValue(value);
	}

	@objid ("1cccfe8a-4583-11e0-b54c-00137279a832")
	public static void setMultiMin(IInstance objingElt, Property ecoreElt) {
		String value =  ObjingEAnnotation.getMultiMin(ecoreElt);
		if (value != null)
			objingElt.setMultiplicityMin(value);
	}

	@objid ("1cccfe8e-4583-11e0-b54c-00137279a832")
	private static void setMultiplicity(IInstance objingElt, Property ecoreElt) {
		String multMin = EcoreModelNavigation.getMultiplicityMin(ecoreElt);
		String multMax = EcoreModelNavigation.getMultiplicityMax(ecoreElt);
		if (!("".equals(multMin)))
			objingElt.setMultiplicityMin(multMin);
		if (!("".equals(multMax)))
			objingElt.setMultiplicityMax(multMax);
	}

	@objid ("1cd1c33a-4583-11e0-b54c-00137279a832")
	public static void setInstance(IInstance objingElt, Property ecoreElt) {
		setBase(objingElt, ecoreElt);
		setRepresentedFeature(objingElt, ecoreElt);
		setMultiplicity((IInstance) objingElt, ecoreElt);

		if (ReverseProperties.getInstance().isRoundtripEnabled()){

			setIsConstant((IInstance) objingElt, ecoreElt);
			setValue((IInstance) objingElt, ecoreElt);
			setMultiMax((IInstance) objingElt, ecoreElt);
			setMultiMin((IInstance) objingElt, ecoreElt);
		}
	}

	@objid ("1cd1c33e-4583-11e0-b54c-00137279a832")
	public static void infoOfUnsupportedOwnedWithEMF(IElement objingOwner, IModelElement objingElement) {
		String eltName = objingElement.getName();
		String eltClassName = objingElement.getClass().getSimpleName()
				.substring(2);
		String ownerClassName = objingOwner.getClass().getSimpleName()
				.substring(2);
		XMILogs.getInstance().writelnInLog(
				XMI.I18N.getMessage("logFile.warning.unsupportedOwnerExport",
						eltClassName, eltName, ownerClassName));
	}

	@objid ("6d53987e-1a76-11e1-8103-0027103f347d")
	public static List<IInterface> getAllProvidedInterface(final IPort port) {
		List<IInterface> result = new ArrayList<IInterface>();

		INameSpace type = port.getBase();

		if (type != null){
			if (type instanceof IInterface)
				result.add((IInterface)type);
			else 
				result.addAll(getAllProvidedInterfaces(type));
		}
		return result;
	}

	@objid ("6d6796b4-1a76-11e1-8103-0027103f347d")
	public static IEnumerationLiteral getEnumerationliteral(final IEnumeration enumeration, final String stringToTest) {
		for (IEnumerationLiteral literal : enumeration.getValue()){
			if (literal.getName().equals(stringToTest))
				return literal;
		}
		return null;
	}

	@objid ("6d6db184-1a76-11e1-8103-0027103f347d")
	public static boolean haveInterfaceRealization(final INameSpace eltToTest, final IInterface inter) {
		for (IInterfaceRealization realization : eltToTest.getRealized() ){
			if (realization.getImplemented().equals(inter))
				return true;
		}
		return false;
	}

	@objid ("6d6dffa8-1a76-11e1-8103-0027103f347d")
	public static boolean haveInterfaceRealizationParent(final INameSpace eltToTest, final IInterface inter) {
		if (haveInterfaceRealization(eltToTest, inter))
			return true;
		else{
			for (IGeneralization generalization : eltToTest.getParent()){
				if (haveInterfaceRealizationParent(generalization.getSuperType(), inter))
					return true;
			}
		}
		return false;
	}

	@objid ("6d786070-1a76-11e1-8103-0027103f347d")
	public static boolean isEnumerationliteral(final IEnumeration enumeration, final String stringToTest) {
		for (IEnumerationLiteral literal : enumeration.getValue()){
			if (literal.getName().equals(stringToTest))
				return true;
		}
		return false;
	}

	@objid ("6d799900-1a76-11e1-8103-0027103f347d")
	public static boolean isNotNullOrEmpty(final String toTest) {
		return ((toTest != null) &&  (!toTest.equals("")));
	}

	@objid ("6d7ad190-1a76-11e1-8103-0027103f347d")
	public static boolean isStereotyped(final IModelElement elt, final String stereotypeName) {
		for (IStereotype stereo : elt.getExtension()) {
			if (stereo.getName().equals(stereotypeName))
				return true;
		}
		return false;
	}

	@objid ("6d7d69c2-1a76-11e1-8103-0027103f347d")
	private static List<IInterface> getAllProvidedInterfaces(final INameSpace parent) {
		List<IInterface> result = new ArrayList<IInterface>();

		result.addAll(getProvidedInterfaces(parent));

		for (IGeneralization generalization : parent.getParent()){
			result.addAll(getAllProvidedInterfaces( generalization.getSuperType()));
		}
		return result;
	}

	@objid ("6d7db7e6-1a76-11e1-8103-0027103f347d")
	private static List<IInterface> getAllRequiredInterfaces(final INameSpace parent) {
		List<IInterface> result = new ArrayList<IInterface>();

		result.addAll(getRequiredInterfaces(parent));

		for (IGeneralization generalization : parent.getParent()){
			result.addAll(getAllRequiredInterfaces( generalization.getSuperType()));
		}
		return result;
	}

	@objid ("6d805018-1a76-11e1-8103-0027103f347d")
	private static List<IInterface> getProvidedInterfaces(final INameSpace parent) {
		List<IInterface> result = new ArrayList<IInterface>();

		for (IInterfaceRealization interfaceRealization : parent.getRealized()){
			result.add(interfaceRealization.getImplemented());
		}
		return result;
	}

	@objid ("6d809e3c-1a76-11e1-8103-0027103f347d")
	private static List<IInterface> getRequiredInterfaces(final INameSpace parent) {
		List<IInterface> result = new ArrayList<IInterface>();

		for (IDependency dependency : parent.getDependsOnDependency()){
			if ((dependency instanceof IUsage)
					&& (dependency.getDependsOn() instanceof IInterface))
				result.add((IInterface)dependency.getDependsOn());
		}
		return result;
	}

}
