/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

@objid ("1d0afbb1-4583-11e0-b54c-00137279a832")
public class Parseur {
    @objid ("1d0afbb2-4583-11e0-b54c-00137279a832")
    private DocumentBuilderFactory factory;

    @objid ("1d0afbb3-4583-11e0-b54c-00137279a832")
    private DocumentBuilder builder;

    @objid ("1d0afbb4-4583-11e0-b54c-00137279a832")
    private Document doc;

    @objid ("1d0afbb5-4583-11e0-b54c-00137279a832")
    private Element nroot;

    @objid ("1d0afbb6-4583-11e0-b54c-00137279a832")
    private File file;


    /**
     * @param path path of XML file
     * @throws Exception
     */
    @objid ("1d0afbb7-4583-11e0-b54c-00137279a832")
    public Parseur(File file) {
        //Chargement du document
        try{
            factory = DocumentBuilderFactory.newInstance();
            builder = factory.newDocumentBuilder();
            doc = builder.parse(file);
            this.nroot = doc.getDocumentElement();
            this.file = file;
        }
        catch(Exception e){
            e.printStackTrace();
        }
    }

    /**
     * Traite les diff?rentes erreurs EA
     */
    @objid ("1d0d5e07-4583-11e0-b54c-00137279a832")
    public void clean() {
        if (isEAModel()){
            deleteDefaultValues();
            deleteEAnone();
            deleteNoAnnotatedElement();
            deleteWriteVariableAction();
            ecrireDocument();
        }
    }

    @objid ("1d0d5e0a-4583-11e0-b54c-00137279a832")
    private boolean isEAModel() {
        boolean isEAModel = false;
        NodeList listeparcours = this.nroot.getElementsByTagName("xmi:Documentation");
        
        
        int nbparcours=listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if ( (e != null ) && e.hasAttribute("exporter") 
                    && e.getAttribute("exporter").equals("Enterprise Architect")){
        
                isEAModel = true;
            }
        }
        return isEAModel;
    }

    /**
     * supprimer les elements defaultValue n'ayant pas de type
     */
    @objid ("1d0d5e0e-4583-11e0-b54c-00137279a832")
    private void deleteDefaultValues() {
        // Parcour du document
        
        NodeList listeparcours = this.nroot.getElementsByTagName("defaultValue");
        
        int nbparcours=listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if ( (e != null) && (!e.hasAttribute("xmi:type"))){
                ((Element)e.getParentNode()).removeAttribute("type");
                e.getParentNode().removeChild(e);
        
            }
        }
    }

    /**
     * supprimer les elements WriteVariableAction
     */
    @objid ("1d0fc05e-4583-11e0-b54c-00137279a832")
    private void deleteWriteVariableAction() {
        // Parcours du document et supression des elements de type WriteVariableAction 
        
        NodeList listeparcours = this.nroot.getElementsByTagName("node");
        
        List<String> listId = new ArrayList<String>();
        List<Element> toDelete = new ArrayList<Element>();
        
        int nbparcours=listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if (e.hasAttribute("xmi:type") && (e.getAttribute("xmi:type").equals("uml:WriteVariableAction"))){
                NodeList childList = e.getChildNodes();
                int nbChild = childList.getLength();
                for(int j = 0; j < nbChild; j++){
                    Element child = (Element)listeparcours.item(j);
                    // case incoming
                    if (child.getNodeName().equals("incoming") && (child.hasAttribute("xmi:idref")))
                        listId.add(child.getAttribute("xmi:idref"));
                    //case output
                    else if (child.getNodeName().equals("output")){
                        NodeList outputChildList = child.getChildNodes();
                        int nbOutputChild = outputChildList.getLength();
                        for(int z = 0; z < nbOutputChild; z++){
                            if (child.getNodeName().equals("outgoing") && (child.hasAttribute("xmi:idref")))
                                listId.add(child.getAttribute("xmi:idref"));
                        }
                    }
        
                }
                toDelete.add(e);
            }
        }
        
        
        
        
        // Parcours du document et supression des elements de type WriteVariableAction 
        
        listeparcours = this.nroot.getElementsByTagName("edge");
        
        nbparcours = listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if ((e != null) && e.hasAttribute("xmi:id") && (listId.contains(e.getAttribute("xmi:id"))))
                toDelete.add(e);
        }
        
        for(Element e : toDelete){
            e.getParentNode().removeChild(e);
        }
    }

    /**
     * supprimer les elements ayant pour type EAnone_void
     */
    @objid ("1d0fc061-4583-11e0-b54c-00137279a832")
    private void deleteEAnone() {
        // Parcour du document
        
        NodeList listeparcours = this.nroot.getElementsByTagName("ownedParameter");
        
        List<Element> toDelete = new ArrayList<Element>();
        
        int nbparcours=listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if ((e != null) 
                    && e.hasAttribute("type") && (e.getAttribute("type").equals("EAnone_void")))
                toDelete.add(e);
        }
        
        for (Element e : toDelete){
            e.getParentNode().removeChild(e);
        }
    }

    /**
     * supprimer les elements de
     */
    @objid ("1d0fc064-4583-11e0-b54c-00137279a832")
    private void deleteNoAnnotatedElement() {
        // Parcour du document
        
        NodeList listeparcours = this.nroot.getElementsByTagName("annotatedElement");
        
        List<String> listId = new ArrayList<String>();
        
        int nbparcours = listeparcours.getLength();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if (e.hasAttribute("xmi:idref")) 
                listId.add(e.getAttribute("xmi:idref"));
        }
        
        List<Element> all = getAllElement();
        
        for (Element e : all){
            if (e.hasAttribute("xmi:id") && (listId.contains(e.getAttribute("xmi:id"))))
                listId.remove(e.getAttribute("xmi:id"));
        }
        
        List<Element> toDelete = new ArrayList<Element>();
        for(int i = 0; i < nbparcours; i++){
            Element e = (Element)listeparcours.item(i);
            if (e.hasAttribute("xmi:idref") && (listId.contains(e.getAttribute("xmi:idref"))))
                toDelete.add(e);
        }
        
        for (Element e : toDelete){
            e.getParentNode().removeChild(e);
        }
    }

    @objid ("1d0fc067-4583-11e0-b54c-00137279a832")
    private List<Element> getAllElement() {
        List<Element> result = new ArrayList<Element>();
        NodeList nodeList = this.nroot.getChildNodes();
        
        int nbChild = nodeList.getLength();
        
        List<Element> current = new ArrayList<Element>();
        for(int i = 0; i < nbChild; i++){
            if (nodeList.item(i) instanceof Element)
                current.add((Element)nodeList.item(i));
        }
        
        List<Element> temp = new ArrayList<Element>();
        boolean haveChild = true;
        
        while(haveChild){
            temp = getAllChild(current);
            haveChild = (temp.size() != 0);
            result.addAll(current);
            current.clear();
            current.addAll(temp);
            temp.clear();
        
        }
        return result;
    }

    /**
     * ?crit dans un fichier un document DOM, ?tant donn? un nom de fichier.
     * @param nomFichier le nom du fichier de sortie
     */
    @objid ("1d1222b9-4583-11e0-b54c-00137279a832")
    private void ecrireDocument() {
        // on consid?re le document "doc" comme ?tant la source d'une 
        // transformation XML
        Source source = new DOMSource(doc);
        
        // le r?sultat de cette transformation sera un flux d'?criture dans
        // un fichier
        Result resultat = new StreamResult(new File(file.getAbsolutePath() + "test"));
        
        // cr?ation du transformateur XML
        Transformer transfo = null;
        try {
            transfo = TransformerFactory.newInstance().newTransformer();
        } catch(TransformerConfigurationException e) {
            Modelio.err.println("Impossible de cr?er un transformateur XML.");
        
        }
        
        // configuration du transformateur
        
        
        if (transfo != null){
            // sortie en XML
        
            transfo.setOutputProperty(OutputKeys.METHOD, "xml");
        
            // inclut une d?claration XML (recommand?)
            transfo.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
        
            // codage des caract?res : UTF-8. Ce pourrait ?tre ?galement ISO-8859-1
            transfo.setOutputProperty(OutputKeys.ENCODING, "utf-8");
        
            // idente le fichier XML
            transfo.setOutputProperty(OutputKeys.INDENT, "yes");
        
            try {
                transfo.transform(source, resultat);
            } catch(TransformerException e) {
                Modelio.err.println("La transformation a ?chou? : " + e);
            }
        }
    }

    @objid ("1d1222bc-4583-11e0-b54c-00137279a832")
    private List<Element> getAllChild(List<Element> old) {
        List<Element> result = new ArrayList<Element>();
        
        for (Element current : old){
            NodeList currentChild = current.getChildNodes();
            int nbChild = currentChild.getLength();
            for(int i = 0; i < nbChild; i++){
                if (currentChild.item(i) instanceof Element)
                    result.add( (Element)currentChild.item(i));
        
            }
        }
        return result;
    }

}
