/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.api.model.IUMLTypes;
import com.modeliosoft.modelio.api.model.uml.statik.IDataType;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralClass;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.impl.PrimitiveTypeImpl;

/**
 * This class made the mapping between Modelio predefined type and Ecore DataType
 * @author ebrosse
 */
@objid ("1d148512-4583-11e0-b54c-00137279a832")
public class PrimitiveTypeMapper {
    @objid ("1d148513-4583-11e0-b54c-00137279a832")
    private static final String booleanEcoreName = "Boolean";

    @objid ("1d148515-4583-11e0-b54c-00137279a832")
    private static final String byteEcoreName = "EByte";

    @objid ("1d148517-4583-11e0-b54c-00137279a832")
    private static final String charEcoreName = "EChar";

    @objid ("1d148519-4583-11e0-b54c-00137279a832")
    private static final String dateEcoreName = "EDate";

    @objid ("1d14851b-4583-11e0-b54c-00137279a832")
    private static final String doubleEcoreName = "EDouble";

    @objid ("1d14851d-4583-11e0-b54c-00137279a832")
    private static final String floatEcoreName = "EFloat";

    @objid ("1d16e76d-4583-11e0-b54c-00137279a832")
    private static final String integerEcoreName = "Integer";

    @objid ("1d16e76f-4583-11e0-b54c-00137279a832")
    private static final String longEcoreName = "ELong";

    @objid ("1d16e771-4583-11e0-b54c-00137279a832")
    private static final String shortEcoreName = "EShort";

    @objid ("1d16e773-4583-11e0-b54c-00137279a832")
    private static final String stringEcoreName = "String";

    @objid ("1d16e777-4583-11e0-b54c-00137279a832")
    private static final String unlimitedNaturalName = "UnlimitedNatural";

    @objid ("1d1949c7-4583-11e0-b54c-00137279a832")
    private static List<PrimitiveType> predefinedType = new ArrayList<PrimitiveType>();

    @objid ("1d1949ca-4583-11e0-b54c-00137279a832")
    private static PrimitiveType BYTE = null;

    @objid ("1d1949cb-4583-11e0-b54c-00137279a832")
    private static PrimitiveType CHAR = null;

    @objid ("1d1949cc-4583-11e0-b54c-00137279a832")
    private static PrimitiveType DATE = null;

    @objid ("1d1949cd-4583-11e0-b54c-00137279a832")
    private static PrimitiveType DOUBLE = null;

    @objid ("1d1949ce-4583-11e0-b54c-00137279a832")
    private static PrimitiveType FLOAT = null;

    @objid ("1d1949cf-4583-11e0-b54c-00137279a832")
    private static PrimitiveType LONG = null;

    @objid ("1d1949d0-4583-11e0-b54c-00137279a832")
    private static PrimitiveType SHORT = null;

    @objid ("1d1949d1-4583-11e0-b54c-00137279a832")
    private static PrimitiveType UNDEFINED = null;

    @objid ("1d1949d4-4583-11e0-b54c-00137279a832")
    private static PrimitiveType BOOLEAN = null;

    @objid ("1d1949d5-4583-11e0-b54c-00137279a832")
    private static PrimitiveType STRING = null;

    @objid ("1d1949d6-4583-11e0-b54c-00137279a832")
    private static PrimitiveType INTEGER = null;

    @objid ("1d1949d7-4583-11e0-b54c-00137279a832")
    private static PrimitiveType UNLIMITED = null;


    /**
     * This methods return the Ecore PrimitiveType associated to the Byte Modelio Predefined Type
     * @return the 'Byte' Ecore PrimitiveType
     */
    @objid ("1d1949d8-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getByte() {
        if (BYTE == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                BYTE = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(byteEcoreName);
            }else{
                BYTE = UMLFactory.eINSTANCE.createPrimitiveType();
                BYTE.setName(Modelio.getInstance().getModelingSession()
                             .getModel().getUmlTypes().getBYTE().getName());
                predefinedType.add(BYTE);
            }
        
        }
        return BYTE;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Char Modelio Predefined Type
     * @return the 'Char' Ecore PrimitiveType
     */
    @objid ("1d1bac20-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getChar() {
        if (CHAR == null){
        
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                CHAR = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(charEcoreName);
            }else{
                CHAR = UMLFactory.eINSTANCE.createPrimitiveType();
                CHAR.setName(Modelio.getInstance().getModelingSession()
                             .getModel().getUmlTypes().getCHAR().getName());
                predefinedType.add(CHAR);
            }
        }
        return CHAR;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Date Modelio Predefined Type
     * @return the 'Date' Ecore PrimitiveType
     */
    @objid ("1d1bac24-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getDate() {
        if (DATE == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                DATE = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(dateEcoreName);
            }else{
                DATE = UMLFactory.eINSTANCE.createPrimitiveType();
                DATE.setName(Modelio.getInstance().getModelingSession()
                             .getModel().getUmlTypes().getDATE().getName());
                predefinedType.add(DATE);
            }
        }
        return DATE;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Double Modelio Predefined Type
     * @return the 'Double' Ecore PrimitiveType
     */
    @objid ("1d1bac28-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getDouble() {
        if (DOUBLE == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                DOUBLE = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(doubleEcoreName);
            }else{
                DOUBLE = UMLFactory.eINSTANCE.createPrimitiveType();
                DOUBLE.setName(Modelio.getInstance().getModelingSession()
                               .getModel().getUmlTypes().getDOUBLE().getName());
                predefinedType.add(DOUBLE);
            }
        }
        return DOUBLE;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Float Modelio Predefined Type
     * @return the 'Float' Ecore PrimitiveType
     */
    @objid ("1d1bac2c-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getFloat() {
        if (FLOAT == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                FLOAT = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(floatEcoreName);
            }else{
                FLOAT = UMLFactory.eINSTANCE.createPrimitiveType();
                FLOAT.setName(Modelio.getInstance().getModelingSession()
                              .getModel().getUmlTypes().getFLOAT().getName());
                predefinedType.add(FLOAT);
            }
        }
        return FLOAT;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Long Modelio Predefined Type
     * @return the 'Long' Ecore PrimitiveType
     */
    @objid ("1d1e0e7a-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getLong() {
        if (LONG == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                LONG = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(longEcoreName);
            }else{
                LONG = UMLFactory.eINSTANCE.createPrimitiveType();
                LONG.setName(Modelio.getInstance().getModelingSession()
                             .getModel().getUmlTypes().getLONG().getName());
                predefinedType.add(LONG);
            }
        }
        return LONG;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the String Modelio Predefined Type
     * @return the 'String' Ecore PrimitiveType
     */
    @objid ("1d1e0e7e-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getString() {
        if (STRING == null){
        
            STRING = (PrimitiveType) UMLMetamodel.getInstance().getUMLLibrary().getOwnedType(stringEcoreName);
        }
        return STRING;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Boolean Modelio Predefined Type
     * @return the 'Boolean' Ecore PrimitiveType
     */
    @objid ("1d1e0e82-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getBoolean() {
        if (BOOLEAN == null){
            BOOLEAN = (PrimitiveType ) UMLMetamodel.getInstance().getUMLLibrary().getOwnedType(booleanEcoreName);
        }
        return BOOLEAN;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Integer Modelio Predefined Type
     * @return the 'Integer' Ecore PrimitiveType
     */
    @objid ("1d1e0e86-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getInteger() {
        if (INTEGER == null){
            INTEGER = (PrimitiveType ) UMLMetamodel.getInstance().getUMLLibrary().getOwnedType(integerEcoreName);
        }
        return INTEGER;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Unlimited Modelio Predefined Type
     * @return the 'Unlimited' Ecore PrimitiveType
     */
    @objid ("1d2070d5-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getUnlimited() {
        if (UNLIMITED == null){
            UNLIMITED = (PrimitiveType ) UMLMetamodel.getInstance().getUMLLibrary().getOwnedType(unlimitedNaturalName);
        }
        return UNLIMITED;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Short Modelio Predefined Type
     * @return the 'Short' Ecore PrimitiveType
     */
    @objid ("1d2070d9-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getShort() {
        if (SHORT == null){
            if (GenerationProperties.getInstance().getExportVersion().equals(VersionExport.EMF300)){
                SHORT = (PrimitiveType) UMLMetamodel.getInstance().getEcoreLibrary().getOwnedType(shortEcoreName);
            }else{
                SHORT = UMLFactory.eINSTANCE.createPrimitiveType();
                SHORT.setName(Modelio.getInstance().getModelingSession()
                              .getModel().getUmlTypes().getSHORT().getName());
                predefinedType.add(SHORT);
            }
        }
        return SHORT;
    }

    /**
     * This methods return the Ecore PrimitiveType associated to the Undefined Modelio Predefined Type
     * @return the 'Undefined' Ecore PrimitiveType
     */
    @objid ("1d2070dd-4583-11e0-b54c-00137279a832")
    public static PrimitiveType getUndefined() {
        if (UNDEFINED == null){
            UNDEFINED = UMLFactory.eINSTANCE.createPrimitiveType();
            UNDEFINED.setName(Modelio.getInstance().getModelingSession()
                              .getModel().getUmlTypes().getUNDEFINED().getName());
            predefinedType.add(UNDEFINED);
        }
        return UNDEFINED;
    }

    /**
     * This method returns a list containing all Ecore Predefined Type
     * @return the list of all Ecore Predefined Type
     */
    @objid ("1d2070e1-4583-11e0-b54c-00137279a832")
    public static List<PrimitiveType> getPredifinedTypeList() {
        return predefinedType;
    }

    /**
     * This method returns the Ecore Type corresponding to a given Modelio Predefined Type
     * @param objingPredefinedType : the given Modelio Predefined Type
     * @return the matching Ecore Type
     */
    @objid ("1d22d332-4583-11e0-b54c-00137279a832")
    public static Type getEcoreType(IDataType objingPredefinedType) {
        IUMLTypes umlTypes = Modelio.getInstance().getModelingSession().getModel().getUmlTypes();
        
        PrimitiveType primitiveType = null;
        
        if ((umlTypes.getBOOLEAN() != null) && umlTypes.getBOOLEAN().equals(objingPredefinedType)) {
            primitiveType = getBoolean();
        }else if ((umlTypes.getBYTE() != null) &&  umlTypes.getBYTE().equals(objingPredefinedType)) {
            primitiveType = getByte();
        } else if ((umlTypes.getCHAR() != null) &&  umlTypes.getCHAR().equals(objingPredefinedType)) {
            primitiveType = getChar();
        }else if ((umlTypes.getDOUBLE() != null) &&  umlTypes.getDOUBLE().equals(objingPredefinedType)) {
            primitiveType = getDouble();
        }else if ((umlTypes.getFLOAT() != null) &&  umlTypes.getFLOAT().equals(objingPredefinedType)) {
            primitiveType = getFloat();
        }else if ((umlTypes.getLONG() != null) &&  umlTypes.getLONG().equals(objingPredefinedType)) {
            primitiveType = getLong();
        }else if ((umlTypes.getSHORT() != null) &&  umlTypes.getSHORT().equals(objingPredefinedType)) {
            primitiveType = getShort();
        } else if ((umlTypes.getINTEGER() != null) &&  umlTypes.getINTEGER().equals(objingPredefinedType)){
            primitiveType = getInteger();
        }else if ((umlTypes.getDATE() != null) &&  umlTypes.getDATE().equals(objingPredefinedType)) {
            primitiveType = getDate();
        } else if ((umlTypes.getSTRING() != null) &&  umlTypes.getSTRING().equals(objingPredefinedType)){
            primitiveType = getString();
        }else if ((umlTypes.getUNDEFINED() != null) && umlTypes.getUNDEFINED().equals(objingPredefinedType)) {      
            primitiveType = getUndefined();
        }
        return primitiveType;
    }

    /**
     * This method types an Ecore TypedElement by a given Modelio DataType
     * @param typed : the Ecore TypedElement
     * @param objingPredefinedType : the Modelio DataType
     */
    @objid ("1d22d337-4583-11e0-b54c-00137279a832")
    public static void setEcorePredefinedType(TypedElement typed, IDataType objingPredefinedType) {
        IUMLTypes umlTypes = Modelio.getInstance().getModelingSession()
                .getModel().getUmlTypes();
        
        PrimitiveType primitiveType = null;
        
        if ((umlTypes.getBOOLEAN() != null) && umlTypes.getBOOLEAN().equals(objingPredefinedType)) {
            primitiveType = getBoolean();
        }else if ((umlTypes.getBYTE() != null) &&  umlTypes.getBYTE().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "byte"
            primitiveType = getByte();
        } else if ((umlTypes.getCHAR() != null) &&  umlTypes.getCHAR().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "char"
            primitiveType = getChar();
        }else if ((umlTypes.getDOUBLE() != null) &&  umlTypes.getDOUBLE().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "double"
            primitiveType = getDouble();
        }else if ((umlTypes.getFLOAT() != null) &&  umlTypes.getFLOAT().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "float"
            primitiveType = getFloat();
        }else if ((umlTypes.getLONG() != null) &&  umlTypes.getLONG().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "long"
            primitiveType = getLong();
        }else if ((umlTypes.getSHORT() != null) &&  umlTypes.getSHORT().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "short"
            primitiveType = getShort();
        } else if ((umlTypes.getINTEGER() != null) &&  umlTypes.getINTEGER().equals(objingPredefinedType)){
            primitiveType = getInteger();
        }else if ((umlTypes.getDATE() != null) &&  umlTypes.getDATE().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "date"
            primitiveType = getDate();
        
        } else if ((umlTypes.getSTRING() != null) &&  umlTypes.getSTRING().equals(objingPredefinedType))
            primitiveType = getString();
        else if ((umlTypes.getUNDEFINED() != null) && umlTypes.getUNDEFINED().equals(objingPredefinedType)) {
            // Stores the original Modelio predefined type: "undefined"
            primitiveType = getUndefined();
        }
        
        if (primitiveType != null)
            typed.setType(primitiveType);
    }

    /**
     * Suppress all Primitive Type matching done in a previous export
     */
    @objid ("1d25358e-4583-11e0-b54c-00137279a832")
    public static void clean() {
        BYTE = null;
        CHAR = null;
        DATE = null;
        DOUBLE = null;
        FLOAT = null;
        LONG = null;
        SHORT = null;
        UNDEFINED = null;
        BOOLEAN = null;
        STRING = null;
        INTEGER = null;
        UNLIMITED = null;
        predefinedType = new ArrayList<PrimitiveType>();
    }

    /**
     * Test if a given IGeneralClass is a Modelio Predefined Type
     * @param type : the tested IGeneralClass
     * @return true if it is a Modelio Predefined Type
     */
    @objid ("6e5e1142-1a76-11e1-8103-0027103f347d")
    public static boolean isPredefinedType(final IGeneralClass type) {
        if (type != null) {
        
            IUMLTypes umlTypes = Modelio.getInstance().getModelingSession()
                    .getModel().getUmlTypes();
        
            if ((umlTypes.getBOOLEAN() != null) && (type.equals(umlTypes.getBOOLEAN())))
                return true;
            else if  ((umlTypes.getCHAR() != null) &&(type.equals(umlTypes.getCHAR())))
                return true;
            else if ((umlTypes.getINTEGER() != null) && (type.equals(umlTypes.getINTEGER())))
                return true;
            else if ((umlTypes.getBYTE() != null) &&(type.equals(umlTypes.getBYTE())))
                return true;
            else if ((umlTypes.getDATE() != null) &&(type.equals(umlTypes.getDATE())))
                return true;
            else if ((umlTypes.getDOUBLE()!= null) &&(type.equals(umlTypes.getDOUBLE())))
                return true;
            else if ((umlTypes.getFLOAT() != null) &&(type.equals(umlTypes.getFLOAT())))
                return true;
            else if ((umlTypes.getLONG() != null) &&(type.equals(umlTypes.getLONG())))
                return true;
            else if ((umlTypes.getSHORT() != null) &&(type.equals(umlTypes.getSHORT())))
                return true;
            else if ((umlTypes.getSTRING() != null) &&(type.equals(umlTypes.getSTRING())))
                return true;
            else if ((umlTypes.getUNDEFINED() != null) &&(type.equals(umlTypes.getUNDEFINED())))
                return true;
        }
        return false;
    }

    /**
     * provide the Modelio DataType cooresponding to a given Ecore Type
     * @param ecoreType : the given Ecore Type
     * @return the corresponding Modelio DataType
     */
    @objid ("6e5e5f66-1a76-11e1-8103-0027103f347d")
    public static IDataType getPredefinedType(final Type ecoreType) {
        //UMLPredefined type
        
        IUMLTypes umlTypes = Modelio.getInstance().getModelingSession().getModel().getUmlTypes();
        
        //Compute primitiveType name
        String currentEcoreTypeName = ecoreType.getName();
        
        if (currentEcoreTypeName == null) {
            if (ecoreType instanceof PrimitiveTypeImpl){
                currentEcoreTypeName = ((PrimitiveTypeImpl) ecoreType).eProxyURI().fragment();
            }else
                currentEcoreTypeName = ecoreType.eResource().getURI().fragment();
        }
        
        if ((ecoreType.eContainer() instanceof ModelImpl) && ((ModelImpl) ecoreType.eContainer()).getName().equals("EcorePrimitiveTypes")){
            if (currentEcoreTypeName.equals(byteEcoreName)){
                return umlTypes.getBYTE();
            }else if (currentEcoreTypeName.equals(dateEcoreName)){
                return umlTypes.getDATE();
            }else if (currentEcoreTypeName.equals(doubleEcoreName)){
                return umlTypes.getDOUBLE();
            }else if (currentEcoreTypeName.equals(floatEcoreName)){
                return umlTypes.getFLOAT();
            }else if (currentEcoreTypeName.equals(longEcoreName)){
                return umlTypes.getLONG();
            }else if (currentEcoreTypeName.equals(shortEcoreName)){
                return umlTypes.getSHORT();
            }else if (currentEcoreTypeName.equals(charEcoreName)){
                return umlTypes.getCHAR();
            }
        }
        
        if (currentEcoreTypeName != null) {
            currentEcoreTypeName = currentEcoreTypeName.toLowerCase();
            if (currentEcoreTypeName.equals(umlTypes.getINTEGER().getName())){
                return umlTypes.getINTEGER();
            }else if (currentEcoreTypeName.equals(umlTypes.getBOOLEAN().getName())){
                return umlTypes.getBOOLEAN();
            }else if (currentEcoreTypeName.toLowerCase().equals(umlTypes.getSTRING().getName())){
                return umlTypes.getSTRING();
            }else if (currentEcoreTypeName.equals(umlTypes.getBYTE().getName())){
                return umlTypes.getBYTE();
            }else if (currentEcoreTypeName.equals(unlimitedNaturalName)){
                return umlTypes.getINTEGER();
            }else if (currentEcoreTypeName.equals(umlTypes.getDATE().getName())){
                return umlTypes.getDATE();
            }else if (currentEcoreTypeName.equals(umlTypes.getDOUBLE().getName())){
                return umlTypes.getDOUBLE();
            }else if ((umlTypes.getFLOAT() != null) && (currentEcoreTypeName.equals(umlTypes.getFLOAT().getName()))){
                return umlTypes.getFLOAT();
            }else if (currentEcoreTypeName.equals(umlTypes.getLONG().getName())){
                return umlTypes.getLONG();
            }else if (currentEcoreTypeName.equals(umlTypes.getSTRING().getName())){
                return umlTypes.getSTRING();
            }else if (currentEcoreTypeName.equals(umlTypes.getSHORT().getName())){
                return umlTypes.getSHORT();
            }else if (currentEcoreTypeName.equals(umlTypes.getCHAR().getName())){
                return umlTypes.getCHAR();
            }else if (currentEcoreTypeName.equals(umlTypes.getUNDEFINED().getName())){
                return umlTypes.getUNDEFINED();
            }
        }
        return null;
    }

    /**
     * Test if a given Ecore Type is a Modelio Predefined Type
     * @param ecoreType : the tested Ecore Type
     * @return true if the Ecore Type corresponds to a Modelio Predefined Type
     */
    @objid ("6e5e8678-1a76-11e1-8103-0027103f347d")
    public static boolean isPredefinedType(final Type ecoreType) {
        String ecoreTypeName = ecoreType.getName();
        
        if (ecoreTypeName == null) {
            if (ecoreType instanceof PrimitiveTypeImpl){
                ecoreTypeName = ((PrimitiveTypeImpl) ecoreType).eProxyURI().fragment();
            }else if (ecoreType.eResource() != null)
                ecoreTypeName = ecoreType.eResource().getURI().fragment();
        }
        
        if (ecoreTypeName != null) {
        
            if ((ecoreType.eContainer() instanceof ModelImpl) && ((ModelImpl) ecoreType.eContainer()).getName().equals("EcorePrimitiveTypes")){
                if ((ecoreTypeName.equals(byteEcoreName)) 
                        || (ecoreTypeName.equals(dateEcoreName))
                        || (ecoreTypeName.equals(doubleEcoreName))
                        || (ecoreTypeName.equals(floatEcoreName)) 
                        || (ecoreTypeName.equals(longEcoreName))
                        || (ecoreTypeName.equals(shortEcoreName))
                        || (ecoreTypeName.equals(charEcoreName))){
                    return true;
                }
            }
        
            IUMLTypes umlTypes = Modelio.getInstance().getModelingSession()
                    .getModel().getUmlTypes();
        
            String uml2IntegerTypeName = "Integer";
        
            String uml2BooleanTypeName = "Boolean";
        
            String uml2StringTypeName = "String";
        
            String uml2UnlimitedNaturalTypeName = "UnlimitedNatural";
        
            ecoreTypeName = ecoreTypeName.toLowerCase();
        
            return  ((ecoreTypeName.equals(umlTypes.getBOOLEAN().getName()))
                    || (ecoreTypeName.equals(umlTypes.getBYTE().getName()))
                    || (ecoreTypeName.equals(umlTypes.getDATE().getName()))
                    || (ecoreTypeName.equals(umlTypes.getDOUBLE().getName()))
                    || (ecoreTypeName.equals(umlTypes.getFLOAT().getName()))
                    || (ecoreTypeName.equals(umlTypes.getLONG().getName()))
                    || (ecoreTypeName.equals(umlTypes.getSHORT().getName()))
                    || (ecoreTypeName.equals(umlTypes.getCHAR().getName()))
                    || (ecoreTypeName.equals(umlTypes.getINTEGER().getName()))
                    || (ecoreTypeName.equals(umlTypes.getSTRING().getName()))
                    || (ecoreTypeName.equals(umlTypes.getUNDEFINED().getName()))
                    || (ecoreTypeName.equals(uml2IntegerTypeName))
                    || (ecoreTypeName.equals(uml2BooleanTypeName))
                    || (ecoreTypeName.equals(uml2StringTypeName))
                    || (ecoreTypeName.equals(uml2UnlimitedNaturalTypeName))
                    );
        
        }
        return false;
    }

    /**
     * @param ecoreType : the tested Type
     * @return true if the test Type is the Boolean Type
     */
    @objid ("6e5ead8d-1a76-11e1-8103-0027103f347d")
    public static boolean isBoolean(final Type ecoreType) {
        if (PrimitiveTypeMapper.isPredefinedType(ecoreType)){
            IDataType obType = PrimitiveTypeMapper.getPredefinedType(ecoreType);
            return  (obType.equals(Modelio.getInstance().getModelingSession().getModel().getUmlTypes().getBOOLEAN()));
        }
        return false;
    }

}
