/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.imageio.ImageIO;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.extension.IProject;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElementStatus;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.INoteType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IProfile;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IGeneralization;
import com.modeliosoft.modelio.api.model.uml.statik.INameSpace;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.core.app.O;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.model.objing.profile.PExportProfile;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.ElementImport;
import org.eclipse.uml2.uml.Extension;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Image;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.internal.impl.PrimitiveTypeImpl;

/**
 * This class provides services for Profile Import/export
 * @author ebrosse
 */
@objid ("8482b9f4-1a74-11e1-8103-0027103f347d")
public class ProfileUtils {
    @objid ("6e5f70e4-1a76-11e1-8103-0027103f347d")
    private static String nameSpacingSeparator = "::";

    @objid ("6e5f70e5-1a76-11e1-8103-0027103f347d")
    private static String stereotypeIdSeparator = ";";

    @objid ("6e5f70e6-1a76-11e1-8103-0027103f347d")
    private static CompositionOwnerVisitor ownerVisitor = new CompositionOwnerVisitor();

    @objid ("6e5f97f6-1a76-11e1-8103-0027103f347d")
    private static ScopeChecker scopeChecker = new ScopeChecker(GenerationProperties
			.getInstance().getSelectedPackage(), ownerVisitor);


    /**
     * Get the content of an {@link Image} as {@link org.eclipse.swt.graphics.Image}
     * @link Image} to set
     * @link org.eclipse.swt.graphics.Image} content
     * @param image the UML {
     * @return {
     */
    @objid ("6e5f97f7-1a76-11e1-8103-0027103f347d")
    public static org.eclipse.swt.graphics.Image getContent(final Image image) {
        if (image == null) {
            // null parameter
            return null;
        }
        
        if (image.getContent() == null) {
            // null image
            return null;
        }
        
        
        String rawData = image.getContent();
        StringTokenizer strToken = new StringTokenizer(rawData, " ");
        byte[]target=new byte[strToken.countTokens()];
        
        // decoding image
        int j=0;
        String tempString = null;
        
        while(strToken.hasMoreTokens()){
            try {
                tempString = strToken.nextToken();
                Long.valueOf(tempString,16);
                Byte tempByte = new Byte(Byte.parseByte(tempString, 16));
                target[j]=tempByte.byteValue();
            }catch(NumberFormatException e){
                int temp = - (Integer.valueOf("ff",16) - Integer.valueOf(tempString,16) + 1);
                Byte tempByte = new Byte(String.valueOf(temp));
                tempByte = Byte.decode(String.valueOf(temp));
                target[j]=tempByte.byteValue();
            }
            j++;
        }
        
        org.eclipse.swt.graphics.Image decodedImage =  new org.eclipse.swt.graphics.Image(null, new ByteArrayInputStream(target));
        return decodedImage;
    }

    @objid ("6e5fbf0c-1a76-11e1-8103-0027103f347d")
    public static List<String> getObjingNameClass(final String ecoreClassName) {
        List<String> result = ReverseProperties.getInstance().getObjClassNames(ecoreClassName);
        
        if (result.size() == 0){
            result.add("ModelElement");
        }
        return result;
    }

    @objid ("6e600d2c-1a76-11e1-8103-0027103f347d")
    private static List<String> getObjNameClass(final java.lang.Class<?> classe, final Method methodCreateElt) {
        List<String> result = new ArrayList<String>();
        
        try{
        
            Constructor<?> createClasse = null;
        
            for (Constructor<?> current : classe.getDeclaredConstructors()){
                if (current.getParameterTypes().length == 0)
                    createClasse= current;
            }
        
            methodCreateElt.invoke(UMLFactory.eINSTANCE, (Object[])null);
        
            Method methodIjingClass = classe.getMethod("getIjingClassName");
        
            if (createClasse != null) {
                Object classeInstance = createClasse.newInstance();
                result = (List<String>) methodIjingClass.invoke(classeInstance);
            }
        
        }catch (Exception e){
            String errorMsg = e.getMessage();
            LOG.warning(XMI.PLUGIN_ID, errorMsg);  
        }
        
        if (result.size() == 0)
            result.add("ModelElement");
        return result;
    }

    @objid ("6e605b51-1a76-11e1-8103-0027103f347d")
    public static List<String> getEcoreNameClass(final String objClassName) {
        List<String> result = GenerationProperties.getInstance().getEcoreClassNames(objClassName);
        
        if (result.size() == 0)
            result.add("Element");
        return result;
    }

    @objid ("6e608265-1a76-11e1-8103-0027103f347d")
    public static Stereotype createStereotype(final IStereotype obStereotype) {
        Profile profile = (Profile) GenerationProperties.getInstance().getMappedElement(obStereotype.getOwner());
        
        String name = ProfileUtils.getStereotypeName(obStereotype);
        Stereotype stereotype  = profile.getOwnedStereotype(name);
        
        if (stereotype == null){
            stereotype = profile.createOwnedStereotype(name, false);
            setIconsProperties(stereotype, obStereotype);
            TotalExportMap.getInstance().put(obStereotype.getIdentifier(), stereotype);
        
        }
        
        ObjingEAnnotation.setNamedWithConvention(stereotype, ProfileUtils.isNamedWithConvention(obStereotype));
        ObjingEAnnotation.addObjingID(stereotype, obStereotype.getIdentifier());
        return stereotype;
    }

    @objid ("6e60a977-1a76-11e1-8103-0027103f347d")
    public static Image setStereotypeImage(final String iconPath, final Stereotype stereotype) {
        try {                
        
            Image icon = stereotype.createIcon(iconPath);
        
            File imageFile = new File(iconPath);
        
            setContent(icon, imageFile);
        
            icon.setFormat(iconPath.substring(iconPath.lastIndexOf(".") + 1, iconPath.length()));
        
            return icon;
        
        }catch(SWTException e) {
            return null;
        }
    }

    @objid ("6e60d08a-1a76-11e1-8103-0027103f347d")
    public static String getNameSpaceName(final IClass obStereotype) {
        String name = "";
        IPackage owner = (IPackage) obStereotype.getOwner();
        while (!(owner.getOwner() instanceof IComponent)){
            name = owner.getName() + nameSpacingSeparator + name;
            owner = (IPackage) owner.getOwner();
        }
        
        String temp = obStereotype.getName();
        if (temp.startsWith(name))
            return temp;
        else
            return (name + temp);
    }

    @objid ("6e60f79a-1a76-11e1-8103-0027103f347d")
    public static String getLocalName(final IClass obStereotype) {
        String path = "";
        IPackage owner = (IPackage) obStereotype.getOwner();
        while (!(owner.getOwner() instanceof IComponent)){
            path = owner.getName() + nameSpacingSeparator + path;
            owner = (IPackage)owner.getOwner();
        }
        
        String name = obStereotype.getName();
        if (name.startsWith(path))
            return name.replaceFirst(path, "");
        else
            return name ;
    }

    /**
     * Set the content of an {@link Image} with a file (containing an image)
     * @link Image} to set
     * @param image the UML {
     * @param imageFile the icon
     */
    @objid ("6e611eaa-1a76-11e1-8103-0027103f347d")
    private static void setContent(final Image image, final File imageFile) {
        try{
            String rawImageData = "";
            if (imageFile != null) {
                byte[] byteFlow = getBytesFromFile(imageFile);
        
                rawImageData = "";
        
                // file reading
                for (int i=0; i<byteFlow.length; i++ ) {
        
                    rawImageData = rawImageData + Integer.toString(byteFlow[i],0x10) + " ";
        
        
                }
            } else {
                rawImageData = null;
            }
        
            image.setContent(rawImageData);
        
        } catch (Exception ex) {}
    }

    /**
     * Read an image file content
     * @param file
     * @throws IOException
     * @return a table of bytes of the file content
     */
    @objid ("6e6145be-1a76-11e1-8103-0027103f347d")
    private static byte[] getBytesFromFile(final File file) throws IOException {
        InputStream is = new FileInputStream(file);
        
        // Get the size of the file
        long length = file.length();
        
        // You cannot create an array using a long type.
        // It needs to be an int type.
        // Before converting to an int type, check
        // to ensure that file is not larger than Integer.MAX_VALUE.
        if (length > Integer.MAX_VALUE) {
            throw new IOException("Image too big to encode");
        }
        
        // Create the byte array to hold the data
        byte[] bytes = new byte[(int)length];
        
        // Read in the bytes
        int offset = 0;
        int numRead = 0;
        while ((offset < bytes.length)
                && ((numRead=is.read(bytes, offset, bytes.length-offset)) >= 0)) {
            offset += numRead;
        }
        
        // Ensure all the bytes have been read in
        if (offset < bytes.length) {
            throw new IOException("Could not completely read file "+file.getName());
        }
        
        // Close the input stream and return bytes
        is.close();
        return bytes;
    }

    @objid ("6e61e205-1a76-11e1-8103-0027103f347d")
    public static List<String> getMetaclassHeritage(final INameSpace metaclass) {
        List<String> result = new  ArrayList<String>();
        
        List<IClass> parentList = new  ArrayList<IClass>();
        parentList.add((IClass) metaclass);
        result.add( metaclass.getName());
        
        while (parentList.size() != 0){
            List<IClass> temp = new  ArrayList<IClass>();
            for(IClass parent : parentList){
                for(IGeneralization generalization : parent.getParent()){
                    temp.add((IClass) generalization.getSuperType());
                }
            }
            parentList.clear();
            parentList.addAll(temp);
            for(IClass parent : parentList)
                result.add(parent.getName());
        }
        return result;
    }

    @objid ("6e620919-1a76-11e1-8103-0027103f347d")
    public static String getNameSpacingSeparator() {
        return nameSpacingSeparator;
    }

    @objid ("6e623029-1a76-11e1-8103-0027103f347d")
    private static IMetaclassReference createReference(final Stereotype ecoreElt) {
        IMetaclassReference objReference  = O.getDefault().getModelingSession().getModelFactory().createMetaclassReference();
        objReference.setReferencedClassName(ecoreElt.getName());
        Object owner =  TotalImportMap.getInstance().get(ecoreElt.getOwner()); 
        if ((owner != null) && (owner instanceof IProfile)){
            ((IProfile) owner).addOwnedReference(objReference);
            objReference.setOwnerProfile(((IProfile) owner));
        }
        TotalImportMap.getInstance().put(ecoreElt, objReference);
        return objReference;
    }

    @objid ("6e62573c-1a76-11e1-8103-0027103f347d")
    public static Object visitStereotype(final Stereotype ecoreElement) {
        Object temp =  TotalImportMap.getInstance().get(ecoreElement);
        if (temp == null){
            if (ObjingEAnnotation.isReference(ecoreElement)){
                return getReference(ecoreElement);
            }else
                return createObjStereotype(ecoreElement);
        }
        return temp;
    }

    @objid ("6e627e4d-1a76-11e1-8103-0027103f347d")
    public static void visitProperty(final Property ecoreElement) {
        Object result = TotalImportMap.getInstance().get(ecoreElement);
        
        if (result == null){
            Element ecoreOwner = ecoreElement.getOwner();
            Object owner = TotalImportMap.getInstance().get(ecoreOwner);
        
            if ((owner == null) && (ecoreOwner instanceof Stereotype)){
                visitStereotype((Stereotype)ecoreOwner);
                owner = TotalImportMap.getInstance().get(owner);
            }
        
            if (owner != null){
                if (owner instanceof ArrayList)
                    visitStereotype((ArrayList<IStereotype>) owner, ecoreElement);
                else
                    visitReference((IMetaclassReference)owner, ecoreElement);
            }
        }
    }

    @objid ("6e62a560-1a76-11e1-8103-0027103f347d")
    private static void setProperties(final ITagType tagType, final Property ecoreProperty) {
        setLabel(tagType, ecoreProperty);
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setHidden(tagType, ecoreProperty);
            setPartSignature(tagType, ecoreProperty);
        }
    }

    @objid ("6e62cc72-1a76-11e1-8103-0027103f347d")
    private static void setHidden(final ITagType objElt, final Property ecoreElement) {
        objElt.setHidden(ObjingEAnnotation.isHidden(ecoreElement));
    }

    @objid ("6e62f384-1a76-11e1-8103-0027103f347d")
    private static void setPartSignature(final ITagType objElt, final Property ecoreElement) {
        objElt.setBelongToPrototype(ObjingEAnnotation.isPartSignature(ecoreElement));
    }

    @objid ("6e631a96-1a76-11e1-8103-0027103f347d")
    private static void setLabel(final ITagType objElt, final Property ecoreElement) {
        String name = objElt.getName();
        if (name != null)
            objElt.setLabel(name);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            String label = ObjingEAnnotation.getLabel(ecoreElement);
            if (label != null)
                objElt.setLabel(label);
        }
    }

    @objid ("6e6341a9-1a76-11e1-8103-0027103f347d")
    private static boolean needCreation(final IMetaclassReference ref) {
        IElementStatus status = ref.getElementStatus();
        return ((!status.isRamcObject()) || (status.isModifiable()));
    }

    @objid ("6e6368bb-1a76-11e1-8103-0027103f347d")
    private static void visitReference(final IMetaclassReference reference, final Property ecoreElement) {
        if (needCreation(reference))
            createUnderReference(reference, ecoreElement);
        else{
        
            List<String> objingIds = ObjingEAnnotation.getObjingIDs(ecoreElement);
        
            if (objingIds.size() > 0){
                String objingId = objingIds.get(0);
                if (ObjingEAnnotation.isNoteType(ecoreElement)){
                    for(INoteType tagtype : reference.getDefinedNoteType()){
                        if (tagtype.getIdentifier().equals(objingId)){
                            PartialImportMap.getInstance().remove(ecoreElement);
                            TotalImportMap.getInstance().put(ecoreElement, tagtype);
                            break;
                        }
                    }
                }else{
                    for(ITagType tagtype : reference.getDefinedTagType()){
                        if (tagtype.getIdentifier().equals(objingId)){
                            PartialImportMap.getInstance().remove(ecoreElement);
                            TotalImportMap.getInstance().put(ecoreElement, tagtype);
                            break;
                        }
                    }
                }
            }
        
        }
    }

    @objid ("6e638fce-1a76-11e1-8103-0027103f347d")
    public static void addExtension(final IModelElement objModelElement, final Stereotype stereotype) {
        if (!ObjingEAnnotation.isReference(stereotype)){
        
            Object imported = visitStereotype(stereotype);
        
        
            List<IStereotype> importedStereotypes = new ArrayList<IStereotype>();
        
            if (imported instanceof IStereotype){
                importedStereotypes.add((IStereotype)imported);
            }else if (imported instanceof List){
                importedStereotypes = (List<IStereotype>) imported;
            }
        
            ObList<IStereotype> existingStereotypes = Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotypes(objModelElement.getClass());
        
            boolean notApply = true;
            for (IStereotype existingStereotype : existingStereotypes){
                if (importedStereotypes.contains(existingStereotype)){
                    objModelElement.addExtension(existingStereotype);
                    notApply = false;
                    break;
                }
            }
        
        
            if ((notApply) && !(ObjingEAnnotation.isReference(stereotype))) {
        
                String name = stereotype.getName();
                try {
        
                    IStereotype objStereotype = Modelio.getInstance().getModelingSession().getMetamodelExtensions().getStereotype(objModelElement.getClass(), name);
        
                    if (!objModelElement.isStereotyped(name))
                        objModelElement.addExtension(objStereotype);
        
                } catch (StereotypeNotFoundException e) {
                    String errorMsg = e.getMessage();
                    LOG.warning(XMI.PLUGIN_ID, errorMsg);  
                }
            }
        }
    }

    @objid ("6e63ddee-1a76-11e1-8103-0027103f347d")
    private static void createUnderReference(final IMetaclassReference reference, final Property ecoreElement) {
        if ((reference != null) ){            
            if (!reference.getElementStatus().isRamcObject()){
                if (!exist(reference, ecoreElement)){
                    if (ObjingEAnnotation.isNoteType(ecoreElement)){
                        createNoteType(reference, ecoreElement);
                    }else{
                        createTagType(reference, ecoreElement);
                    }
        
                }
            }
        }
    }

    @objid ("6e640500-1a76-11e1-8103-0027103f347d")
    public static IProfile createObjProfile(final Profile ecoreElement) {
        IProfile objProfile = (IProfile) TotalImportMap.getInstance().get(ecoreElement);
        
        if (objProfile == null){
            if ( (ecoreElement.getName() != null) && (ecoreElement.getName().equals("LocalProfile"))){
                for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
                    if (module.getName().equals("LocalModule")){
                        for(IProfile profile : module.getOwnedProfile()){
                            if (profile.getName().equals("LocalProfile")){
                                objProfile =  profile;
                                break;
                            }
                        }
                    }
                }
        
            }else{
        
                String moduleId = ObjingEAnnotation.getModule(ecoreElement);
        
                List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreElement);
                if  (ids.size() > 0){
        
                    objProfile = (IProfile) Modelio.getInstance().getModelingSession().findElementById("Profile", ids.get(0));
                    if (objProfile == null){
                        for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
                            if (module.getIdentifier().equals(moduleId)){
                                for (IProfile ownedProfile : module.getOwnedProfile()){
                                    if (ownedProfile.getIdentifier().equals(ids.get(0))){
                                        objProfile =  ownedProfile;
                                    }
                                }
                            }
                        }
                    }
                }
        
        
                if (objProfile == null){
                    objProfile = O.getDefault().getModelingSession().getModelFactory().createProfile();
                    attach(objProfile, ecoreElement);
                    setProperties(objProfile, ecoreElement);
                }
        
            }
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().put(ecoreElement, objProfile);      
        }
        return objProfile;
    }

    @objid ("6e642c12-1a76-11e1-8103-0027103f347d")
    private static void setProperties(final IProfile objProfile, final Profile ecoreElement) {
        String name = ecoreElement.getName();
        Element owner = ecoreElement.getOwner();
        
        while (owner instanceof org.eclipse.uml2.uml.Package){
            org.eclipse.uml2.uml.Package ownerPck = (org.eclipse.uml2.uml.Package) owner;
            name = ownerPck.getName() +  nameSpacingSeparator + name;
            owner =  ownerPck.getOwner();
        }
        
        if (name != null)
            objProfile.setName(name);
    }

    @objid ("6e647a36-1a76-11e1-8103-0027103f347d")
    private static Object createObjStereotype(final Stereotype ecoreElement) {
        List<String> baseClasses = new ArrayList<String>();
        List<IStereotype> results = new ArrayList<IStereotype>();
        List<IStereotype> parents = new ArrayList<IStereotype>();
        
        List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreElement);
        for (String id : ids){
            IStereotype result = (IStereotype) Modelio.getInstance().getModelingSession().findElementById("Stereotype", id);
            if (result != null)
                results.add(result);        
        }
        
        
        if (results.size() == 0){
            baseClasses = ObjingEAnnotation.getBaseClass(ecoreElement);
            if (baseClasses.size() == 0){
                for (Element ownedElt : ecoreElement.getOwnedElements()){
                    if (ownedElt instanceof Property ){
                        Property ecoreProperty = (Property) ownedElt;
        
                        if ((ecoreProperty.getAssociation() instanceof Extension)
                                &&  (ecoreProperty.getType() instanceof org.eclipse.uml2.uml.Class) ){
                            String typeName = ecoreProperty.getType().getName();
                            if (typeName == null)
                                typeName = ((InternalEObject)ecoreProperty.getType()).eProxyURI().fragment();
                            baseClasses.addAll(ProfileUtils.getObjingNameClass(typeName));
                        }
                    }
                }
            }
        
            if (baseClasses.size() == 0){
                baseClasses.add("ModelElement");
            }
        
            for (String baseClass : baseClasses){
        
                IStereotype result =  O.getDefault().getModelingSession().getModelFactory().createStereotype();
                result.setBaseClassName(baseClass);
        
                for (IStereotype parent : parents){
                    if (parent.getBaseClassName().equals(baseClass)){
                        result.setParent(parent);
                        break;
                    }
                }
        
                if ((result.getParent() == null) && (parents.size() != 0)){
                    result.setParent(parents.get(0));
                }
        
                if (result.getElementStatus().isModifiable()){
                    attach(result, ecoreElement);
                    setProperties(result, ecoreElement);
                }
                results.add(result);
            }
            setName(results, ecoreElement);
        }
        
        PartialImportMap.getInstance().remove(ecoreElement);
        TotalImportMap.getInstance().put(ecoreElement, results);
        return results;
    }

    @objid ("6e64a148-1a76-11e1-8103-0027103f347d")
    private static void attach(final IStereotype objingSterotype, final Stereotype ecoreSterotype) {
        Object owner =  TotalImportMap.getInstance().get(ecoreSterotype.getOwner()); 
        
        if ((owner != null) && (owner instanceof IProfile)){
            ((IProfile)owner).addDefinedStereotype(objingSterotype);
        }else{
            for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
                if (module.getName().equals("LocalModule")){
                    for(IProfile profile : module.getOwnedProfile()){
                        if (profile.getName().equals("LocalProfile")){
                            profile.addDefinedStereotype(objingSterotype);
                        }
                    }
                }
            }
        }
    }

    @objid ("6e64c85a-1a76-11e1-8103-0027103f347d")
    private static void attach(final IProfile objProfile, final Profile ecoreElement) {
        boolean attach = false;
        IProject projet = O.getDefault().getModelingSession().getModel().getProject();
        String moduleId = ObjingEAnnotation.getModule(ecoreElement);
        
        if (moduleId != null){
        
            for (IModule  module : projet.getInstalled()){
        
                if (module.getIdentifier().equals(moduleId)){
                    List<String> ids =  ObjingEAnnotation.getObjingIDs(ecoreElement);
        
                    if (ids.size()>0) {
                        String profileId = ids.get(0);
        
                        if (profileId != null){
        
                            for (IProfile  profile : module.getOwnedProfile()){
        
                                if (profile.getIdentifier().equals(profileId)){
                                    attach = true;
                                }
                            }
                        }
                    }
                }
            }
        }
        
        if (!attach)
            ReverseProperties.getInstance().getProfileRoot().addOwnedProfile(objProfile);
    }

    @objid ("6e65167e-1a76-11e1-8103-0027103f347d")
    private static void setProperties(final IStereotype objingElt, final Stereotype ecoreElement) {
        setInheritance(objingElt, ecoreElement);
        setIcons(objingElt, ecoreElement);
        setLabel(objingElt, ecoreElement);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled())
            setHidden(objingElt, ecoreElement);
    }

    @objid ("6e653d90-1a76-11e1-8103-0027103f347d")
    private static void setName(final List<IStereotype> objElts, final Stereotype stereotype) {
        String sterName = stereotype.getName();
        if (sterName != null){
        
            if (objElts.size() > 1){
                //List of stereotype
                for (IStereotype objElt : objElts){
                    objElt.setName(sterName + "_" + objElt.getBaseClassName());
                    objElt.setLabel(sterName);
                }
        
            }else{
                //Unique stereotype
                objElts.get(0).setName(sterName);
                objElts.get(0).setLabel(sterName);
        
            }
        }
    }

    @objid ("6e658bb4-1a76-11e1-8103-0027103f347d")
    private static void setHidden(final IStereotype objElt, final Stereotype ecoreElement) {
        if (ReverseProperties.getInstance().isRoundtripEnabled())
            objElt.setHidden(ObjingEAnnotation.isHidden(ecoreElement));
    }

    @objid ("6e65d9d8-1a76-11e1-8103-0027103f347d")
    private static void setLabel(final IStereotype objElt, final Stereotype ecoreElement) {
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            String label = ObjingEAnnotation.getLabel(ecoreElement);
            if ((label != null) && (label.equals("")))
                objElt.setLabel(label);
        }
    }

    @objid ("6e6600ea-1a76-11e1-8103-0027103f347d")
    private static void setIcons(final IStereotype objElt, final Stereotype ecoreElement) {
        EList<Image> icons = ecoreElement.getIcons(); 
        if (icons.size() > 0 ){
            if (ReverseProperties.getInstance().isRoundtripEnabled()){
                for(Object temp :icons){
                    org.eclipse.uml2.uml.Image icon = (org.eclipse.uml2.uml.Image) temp;
                    String iconType = ObjingEAnnotation.getIconType(icon);
                    if (iconType != null){
                        setIcon(objElt, icon, iconType);
                    }    
                }
            }else{
                setIcon(objElt, (org.eclipse.uml2.uml.Image) icons.get(0), "icon");
                setIcon(objElt, (org.eclipse.uml2.uml.Image) icons.get(0), "explorerIcon");
                setIcon(objElt, (org.eclipse.uml2.uml.Image) icons.get(0), "smallIcon");
            }
        
        }
    }

    @objid ("6e6627fe-1a76-11e1-8103-0027103f347d")
    public static void write(final org.eclipse.swt.graphics.Image img, final String fileName, final String format) throws IOException {
        BufferedImage buffi = convertToAWT(img.getImageData());
        File outputFile = new File(fileName);
        try{
            outputFile.createNewFile();
            ImageIO.write(buffi, format, outputFile);
        }catch(IOException e){
            LOG.warning(XMI.PLUGIN_ID,"File not Found");
        }
    }

    /**
     * a ver.... este m?todo fue extra?do de uno de los snippets de
     * swt para la transformaci?n de una im?gen swt en una
     * bufferedImage de awt. Sin embargo, este c?digo fue mejorado por sgurin
     * para el manejo de las transparencias. Su misi?n es que el uso:
     * 
     * Image img = new Image(null, "pepe");
     * BufferedImage bimg = convertToAWT(img.getImageData())
     * 
     * funcione correctamente para "pepe" con formato png, jpg, gif
     * @param data la ImageData (image swt) a convertir
     * @return la conversi?n de data a una BufferedImage swt
     */
    @objid ("6e667620-1a76-11e1-8103-0027103f347d")
    static BufferedImage convertToAWT(final ImageData data) {
        ColorModel colorModel = null;
        PaletteData palette = data.palette;
        
        if (palette.isDirect) {            
        
            // no tenemos canal alfa
            if(data.alphaData==null) {
                colorModel = new DirectColorModel(24, palette.redMask, 
                        palette.greenMask, palette.blueMask);                
                BufferedImage bufferedImage = new BufferedImage(colorModel, 
                        colorModel.createCompatibleWritableRaster(data.width, data.height), 
                        false, null);                
                WritableRaster raster = bufferedImage.getRaster();                
                int[] pixelArray = new int[3];
                for (int y = 0; y < data.height; y++) {
                    for (int x = 0; x < data.width; x++) {
                        int pixel = data.getPixel(x, y);                        
                        RGB rgb = palette.getRGB(pixel);
                        pixelArray[0] = rgb.red;
                        pixelArray[1] = rgb.green;
                        pixelArray[2] = rgb.blue;
                        raster.setPixels(x, y, 1, 1, pixelArray);                        
                    }
                }
        
                return bufferedImage;
            }
        
            //tenemos canal alfa
            else {
                colorModel = new DirectColorModel(32, palette.redMask, 
                        palette.greenMask, palette.blueMask, 0xff000000);                
                BufferedImage bufferedImage = new BufferedImage(colorModel, 
                        colorModel.createCompatibleWritableRaster(data.width, data.height), 
                        false, null);                
                WritableRaster raster = bufferedImage.getRaster();                
                int[] pixelArray = new int[4];
                for (int y = 0; y < data.height; y++) {
                    for (int x = 0; x < data.width; x++) {
                        int pixel = data.getPixel(x, y);                        
                        RGB rgb = palette.getRGB(pixel);
                        pixelArray[0] = rgb.red;
                        pixelArray[1] = rgb.green;
                        pixelArray[2] = rgb.blue;
                        pixelArray[3]=data.getAlpha(x,y);
                        raster.setPixels(x, y, 1, 1, pixelArray);                        
                    }
                }
                return bufferedImage;
            }
        
            //la paleta swt no es directa ??????
        } else { //no sabemos qu? pasa ac?
            RGB[] rgbs = palette.getRGBs();
            byte[] red = new byte[rgbs.length];
            byte[] green = new byte[rgbs.length];
            byte[] blue = new byte[rgbs.length];
            for (int i = 0; i < rgbs.length; i++) {
                RGB rgb = rgbs[i];
                red[i] = (byte)rgb.red;
                green[i] = (byte)rgb.green;
                blue[i] = (byte)rgb.blue;
            }
            if (data.transparentPixel != -1) {
                colorModel = new IndexColorModel(data.depth, rgbs.length, red, 
                        green, blue, data.transparentPixel);
            } else {
                colorModel = new IndexColorModel(data.depth, rgbs.length, red, 
                        green, blue);
            }        
            BufferedImage bufferedImage = new BufferedImage(colorModel, 
                    colorModel.createCompatibleWritableRaster(data.width, data.height), 
                    false, null);
            WritableRaster raster = bufferedImage.getRaster();
            int[] pixelArray = new int[1];
            for (int y = 0; y < data.height; y++) {
                for (int x = 0; x < data.width; x++) {
                    int pixel = data.getPixel(x, y);
                    pixelArray[0] = pixel;
                    raster.setPixel(x, y, pixelArray);
                }
            }
            return bufferedImage;
        }
    }

    @objid ("6e669d32-1a76-11e1-8103-0027103f347d")
    private static void setInheritance(final IStereotype objElt, final Stereotype ecoreElement) {
        for (Stereotype  ecoreParent : getParent(ecoreElement)){
            Object parents =  visitStereotype(ecoreParent);
            objElt.setParent(((ArrayList<IStereotype>)parents).get(0));
        }
    }

    @objid ("6e66c446-1a76-11e1-8103-0027103f347d")
    private static List<Stereotype> getParent(final Stereotype stereotype) {
        List<Stereotype> result = new ArrayList<Stereotype>();
        for (Object obj : stereotype.getGeneralizations()){
            if (obj instanceof Generalization){
                Classifier sterParent = ((Generalization) obj).getGeneral();
                if (sterParent instanceof Stereotype){
                    result.add((Stereotype)sterParent);
                }
            }
        }
        return result;
    }

    @objid ("6e671268-1a76-11e1-8103-0027103f347d")
    private static void setIconsProperties(final Stereotype stereotype, final IStereotype obStereotype) {
        ResourceLoader.getInstance();
        
        String iconPath = obStereotype.getIcon();
        
        if ((iconPath != null) && (!iconPath.equals("") )){
            Image icon = setStereotypeImage( iconPath, stereotype);
            ObjingEAnnotation.setIconType(icon, "icon");
        }
        
        iconPath = obStereotype.getIconBrowser();
        
        if ((iconPath != null) && (!iconPath.equals("") ))  {
            Image icon = setStereotypeImage( iconPath, stereotype);
        
            ObjingEAnnotation.setIconType(icon, "explorerIcon");
        }
    }

    @objid ("6e67aeb0-1a76-11e1-8103-0027103f347d")
    private static void setProperties(final INoteType attri, final Property ecoreElement) {
        setLabel(attri, ecoreElement);
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            setHidden(attri,ecoreElement);
        }
    }

    @objid ("6e67fcd4-1a76-11e1-8103-0027103f347d")
    private static void setHidden(final INoteType objElt, final Property ecoreElement) {
        objElt.setHidden(ObjingEAnnotation.isHidden(ecoreElement));
    }

    @objid ("6e684af8-1a76-11e1-8103-0027103f347d")
    private static void setLabel(final INoteType objElt, final Property ecoreElement) {
        String name = objElt.getName();
        if (name != null)
            objElt.setLabel(name);
        
        if (ReverseProperties.getInstance().isRoundtripEnabled()){
            String label = ObjingEAnnotation.getLabel(ecoreElement);
            if (label != null)
                objElt.setLabel(label);
        }
    }

    @objid ("6e68720e-1a76-11e1-8103-0027103f347d")
    private static INoteType createNoteType(final IStereotype stereotype, final Property ecoreElement) {
        INoteType noteType  =  O.getDefault().getModelingSession().getModelFactory().createNoteType();
        stereotype.addDefinedNoteType(noteType);
        
        noteType.setName(ecoreElement.getName());
        
        setProperties(noteType, ecoreElement);
        return noteType;
    }

    @objid ("6e68c02e-1a76-11e1-8103-0027103f347d")
    private static ITagType createTagType(final IStereotype stereotype, final Property ecoreElement) {
        String name = ecoreElement.getName(); 
        Type type = ecoreElement.getType();
        ITagType attr = O.getDefault().getModelingSession().getModelFactory().createTagType();
        
        stereotype.addDefinedTagType(attr);
        attr.setName(name);
        
        String max = EcoreModelNavigation.getMultiplicityMax(ecoreElement);
        
        if (PrimitiveTypeMapper.isBoolean(type) && (max.equals("1"))){
            attr.setParamNumber("0");
        }else{
            attr.setParamNumber(max);
        }
        
        setProperties(attr, ecoreElement);
        return attr;
    }

    @objid ("6e690e52-1a76-11e1-8103-0027103f347d")
    private static void createNoteType(final IMetaclassReference reference, final Property ecoreElement) {
        List<String> ids =  ObjingEAnnotation.getObjingIDs(ecoreElement);
        INoteType noteType = null;
        
        if (ids.size()>0) {
            String id = ids.get(0);
            noteType = (INoteType) Modelio.getInstance().getModelingSession().findElementById("NoteType", id);
        }
        
        if (noteType == null){
            noteType = O.getDefault().getModelingSession().getModelFactory().createNoteType();
            reference.addDefinedNoteType(noteType);
        
            noteType.setName(ecoreElement.getName());
        
            ecoreElement.getType();
        
            setProperties(noteType, ecoreElement);
        }
    }

    @objid ("6e693565-1a76-11e1-8103-0027103f347d")
    private static ITagType createTagType(final IMetaclassReference reference, final Property ecoreElement) {
        List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreElement);
        ITagType tagType = null;
        
        if (ids.size() > 0){
            tagType = (ITagType) Modelio.getInstance().getModelingSession().findElementById("TagType", ObjingEAnnotation.getObjingIDs(ecoreElement).get(0));
        
            Type type = ecoreElement.getType();
            String name = ecoreElement.getName();
        
            if ((name != null ) && (!name.equals("")) 
                    && (!name.startsWith("base_")) && (type != null) 
                    && (tagType == null)){
                tagType =  O.getDefault().getModelingSession().getModelFactory().createTagType();
                reference.addDefinedTagType(tagType);
        
                tagType.setName(name);
        
        
                if ((((type.getName() != null) && (type.getName().equals("Boolean"))) 
                        || ((type instanceof PrimitiveTypeImpl) && (((PrimitiveTypeImpl) type).eProxyURI() != null) 
                                && ((PrimitiveTypeImpl) type).eProxyURI().fragment().equals("Boolean")))) {
                    tagType.setParamNumber("0");
                }
        
        
                String max = String.valueOf(ecoreElement.getUpper());
                if (max != null){
                    if (ObjingModelNavigation.OBJING_UNLIMITED_VALUE.equals(max) || (max.equals("-1")))
                        tagType.setParamNumber("*");
                    else{
                        String min = String.valueOf(ecoreElement.getLower());
                        tagType.setParamNumber(String.valueOf(Integer.valueOf(max) - Integer.valueOf(min)));
                    }
        
                }
        
                setProperties(tagType, ecoreElement);
            }
        }
        return tagType;
    }

    @objid ("6e698388-1a76-11e1-8103-0027103f347d")
    private static boolean exist(final IStereotype stereotype, final Property ecoreElement) {
        String name = ecoreElement.getName();
        if (name != null){
        
            if (ObjingEAnnotation.isNoteType(ecoreElement)){
                for (INoteType noteType : stereotype.getDefinedNoteType()){
                    if (noteType.getName().equals(name)){
                        return true;
                    }
                }
            }else{
                for (ITagType tagtype : stereotype.getDefinedTagType()){
                    if (tagtype.getName().equals(name)){
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @objid ("6e69aa9b-1a76-11e1-8103-0027103f347d")
    private static boolean exist(final IMetaclassReference reference, final Property ecoreElement) {
        String name = ecoreElement.getName();
        if (name != null){
        
            if (ObjingEAnnotation.isNoteType(ecoreElement)){
                for (INoteType noteType : reference.getDefinedNoteType()){
                    if (noteType.getName().equals(name)){
                        return true;
                    }
                }
            }else{
                for (ITagType tagtype : reference.getDefinedTagType()){
                    if (tagtype.getName().equals(name)){
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @objid ("6e69f8be-1a76-11e1-8103-0027103f347d")
    private static boolean needCreation(final List<IStereotype> stereotypes) {
        for (IStereotype ster : stereotypes){
            IElementStatus status = ster.getElementStatus();
        
            if ((!status.isRamcObject()) || (status.isModifiable())){
                return true;
            }
        }
        return false;
    }

    @objid ("6e6a46e2-1a76-11e1-8103-0027103f347d")
    private static void visitStereotype(final List<IStereotype> stereotypes, final Property ecoreElement) {
        if (needCreation(stereotypes))
            createUnderStereotypes(stereotypes, ecoreElement);
        else{
            List<String> objingIds = ObjingEAnnotation.getObjingIDs(ecoreElement);
            if (objingIds.size() >0){
                String objingId = objingIds.get(0);
                for (IStereotype ster : stereotypes){
        
                    if (ObjingEAnnotation.isNoteType(ecoreElement)){
                        for(INoteType tagtype : ster.getDefinedNoteType()){
                            if (tagtype.getIdentifier().equals(objingId)){
                                PartialImportMap.getInstance().remove(ecoreElement);
                                TotalImportMap.getInstance().put(ecoreElement, tagtype);
                                break;
                            }
                        }
                    }else{
                        for(ITagType tagtype : ster.getDefinedTagType()){
                            if (tagtype.getIdentifier().equals(objingId)){
                                PartialImportMap.getInstance().remove(ecoreElement);
                                TotalImportMap.getInstance().put(ecoreElement, tagtype);
                                break;
                            }
                        }
                    }
                }
            }
        }
    }

    @objid ("6e6a9506-1a76-11e1-8103-0027103f347d")
    private static void createUnderStereotypes(final List<IStereotype> stereotypes, final Property ecoreElement) {
        if ((stereotypes != null) ){     
        
            List<IModelElement> results = new ArrayList<IModelElement>();
        
            for (IStereotype stereotype : stereotypes){
                if ((!stereotype.getElementStatus().isRamcObject()) 
                        &&  (!exist(stereotype, ecoreElement))){     
        
                    String name = ecoreElement.getName();
        
                    if (ObjingEAnnotation.isNoteType(ecoreElement)){
                        results.add(createNoteType(stereotype, ecoreElement));
        
                    }else if ((name != null ) && (!name.equals("")) 
                            && (!name.startsWith("base_")) && (ecoreElement.getType() != null) 
                            ){
                        results.add(createTagType(stereotype, ecoreElement));
        
                    }
                }
            }
        
            PartialImportMap.getInstance().remove(ecoreElement);
            TotalImportMap.getInstance().put(ecoreElement, results);
        }
    }

    @objid ("6e6ae32a-1a76-11e1-8103-0027103f347d")
    private static void setIcon(final IStereotype objElt, final org.eclipse.uml2.uml.Image icon, final String tag) {
        String format = icon.getFormat();
        
        String location = icon.getLocation();
        
        if(format != null && !format.equals("") && location != null && !location.equals("")){
        
            //creation du fichier image
        
            File file = new File(location);
            if (!file.exists()){
                file.mkdirs();
        
                try{
                    org.eclipse.swt.graphics.Image image = getContent(icon);
                    if (image != null) {
                        write(image, location, format);
                    }
        
                }catch(SWTException e){
        
                    if (e.getMessage().equals("Unsupported or unrecognized format")){
                        XMILogs xmilogs = XMILogs.getInstance();
                        xmilogs.writelnInLog(XMI.I18N.getMessage("logFile.warning.import.unsupportedFormat", location, format));
                    }else{
                        String errorMsg = e.getMessage();
                        LOG.warning(XMI.PLUGIN_ID, errorMsg);  
                    }
        
                }catch(IOException e){
                    String errorMsg = e.getMessage();
                    LOG.warning(XMI.PLUGIN_ID, errorMsg);     
                }
            }
        
            //ajout de la taggedvalue
        
            if (tag.equals("icon")){
                objElt.setIcon(location);
            }else if (tag.equals("explorerIcon")){
                objElt.setIconBrowser(location);
            }
        
        
        }
    }

    @objid ("6e6b314e-1a76-11e1-8103-0027103f347d")
    public static boolean isNamedWithConvention(final IStereotype stereotype) {
        if ((stereotype.getLabel() != null) &&  (stereotype.getBaseClassName() != null))
            return stereotype.getName().equals(stereotype.getLabel() + "_" + stereotype.getBaseClassName());
        else 
            return false;
    }

    @objid ("6e6b5861-1a76-11e1-8103-0027103f347d")
    public static String getStereotypeName(final IStereotype stereotype) {
        if (isNamedWithConvention(stereotype))
            return stereotype.getLabel();
        else
            return stereotype.getName();
    }

    @objid ("6e6b7f74-1a76-11e1-8103-0027103f347d")
    public static String getStereotypeIdSeparator() {
        return stereotypeIdSeparator;
    }

    @objid ("6e6bf4a8-1a76-11e1-8103-0027103f347d")
    public static boolean isNamedWithConvention(final ITagType tagtype) {
        if ((tagtype.getLabel() != null) && (tagtype.getOwnerStereotype() != null) && (tagtype.getOwnerStereotype().getBaseClassName() != null))
            return tagtype.getName().equals(tagtype.getOwnerStereotype().getLabel() + "_" + tagtype.getOwnerStereotype().getBaseClassName()+ "_"+ tagtype.getLabel() );
        else 
            return false;
    }

    @objid ("6e6c1bba-1a76-11e1-8103-0027103f347d")
    public static String getTagTypeName(final ITagType tagtype) {
        if (isNamedWithConvention(tagtype))
            return tagtype.getLabel();
        else
            return tagtype.getName();
    }

    @objid ("6e6c42ce-1a76-11e1-8103-0027103f347d")
    public static void addReference(final Stereotype stereotype, final String metaclassName) {
        Class metaclassClass = (Class) UMLMetamodel.getInstance().getUMLMetamodel().getOwnedType(metaclassName);
        
        Profile profile = stereotype.getProfile();
        
        if (metaclassClass != null){
        
            boolean found = false;
            Class reference = metaclassClass;
        
            for (Object metaclass : profile.getMetaclassReferences()){
                Class importedClas = (Class)((ElementImport)metaclass).getImportedElement();
                if (importedClas != null){
                    String clasname = importedClas.getName();
                    String metaClassName = metaclassClass.getName();
                    if (metaClassName != null){
                        if (clasname.equals(metaClassName)){
                            reference = (Class)((ElementImport)metaclass).getImportedElement();
                            found = true; 
                        }
                    }
                }
            }
        
            if (!found){ 
                profile.createMetaclassReference(reference);
                stereotype.createExtension(reference, false);
            }else{
        
                boolean typed = false ;
                for (Object extension : reference.getExtensions())
                    for (Object extensionEnd : ((Extension) extension).getOwnedEnds())
                        if (((Property) extensionEnd).getType().equals(stereotype))
                            typed = true;
        
                if (!typed)
                    stereotype.createExtension(reference, false);
            }  
        
        }else{
            String errorMsg = "Not found correspondance for : " + metaclassName;
            LOG.warning(XMI.PLUGIN_ID, errorMsg);  
        }
    }

    @objid ("6e6cb802-1a76-11e1-8103-0027103f347d")
    public static Stereotype createStereotype(final IMetaclassReference reference, final Profile profile) {
        Stereotype result = profile.createOwnedStereotype(reference.getReferencedClassName(), false);
        ObjingEAnnotation.addObjingID(result, reference.getIdentifier());
        return result;
    }

    @objid ("6e6d0626-1a76-11e1-8103-0027103f347d")
    public static boolean isInScope(final IStereotype stereotype) {
        return scopeChecker.contains(stereotype);
    }

    @objid ("6e6da26e-1a76-11e1-8103-0027103f347d")
    public static String getName(final IProfile profile) {
        String name = profile.getName();
        if (name.contains(nameSpacingSeparator)){
            String[] names = name.split(nameSpacingSeparator);
            return names[names.length - 1]; 
        }
        return name;
    }

    @objid ("6e6dc981-1a76-11e1-8103-0027103f347d")
    public static IProfile getProfileOwner(final IProfile profile) {
        String name = profile.getName();
        IProfile owner = null;
        
        if (name.contains(nameSpacingSeparator)){
            String[] names = name.split(nameSpacingSeparator);
            String ownerName = names[0];
            int size = names.length;
            for (int  i = 1 ;  i < size - 1; i++ ){
                ownerName = ownerName + nameSpacingSeparator + names[i];              
            }
        
            for (IProfile otherProfile: profile.getOwnerModule().getOwnedProfile()){
                if (otherProfile.getName().equals(ownerName)){
                    return otherProfile;
                }
            }
        }
        return owner;
    }

    @objid ("6e6e17a4-1a76-11e1-8103-0027103f347d")
    public static List<PExportProfile> getSubProfiles(final PExportProfile profile) {
        List<PExportProfile> result = new ArrayList<PExportProfile>();
        IProfile objProfil = profile.getObjElt();
        for (IProfile subprofile : objProfil.getOwnerModule().getOwnedProfile() ){
            String profileName = objProfil.getName();
            String subProfileName = subprofile.getName();
            if (subProfileName.contains(profileName)
                    && subProfileName.replaceAll(profileName, "").contains(nameSpacingSeparator)){
                result.add(new PExportProfile(subprofile));
            }
        }
        return result;
    }

    @objid ("6e6e3eb7-1a76-11e1-8103-0027103f347d")
    public static IProfile getProfileRoot(final IProfile profile) {
        String name = profile.getName();
        IProfile owner = profile;
        
        if (name.contains(nameSpacingSeparator)){
            String[] names = name.split(nameSpacingSeparator);
            String ownerName = names[0];
        
            for (IProfile otherProfile: profile.getOwnerModule().getOwnedProfile()){
                if (otherProfile.getName().equals(ownerName)){
                    return otherProfile;
                }
            }
        }
        return owner;
    }

    @objid ("6e6e65cb-1a76-11e1-8103-0027103f347d")
    private static IMetaclassReference getReference(final Stereotype ecoreElt) {
        List<String> ids = ObjingEAnnotation.getObjingIDs(ecoreElt);
        
        if (ids.size() > 0){
            String id = ids.get(0);
            Object profile = ReverseProperties.getInstance().getMappedElement(ecoreElt.getProfile());
            if ((profile != null ) && (profile instanceof IProfile)){
                for (IMetaclassReference reference : ((IProfile) profile).getOwnedReference()){
                    if (reference.getIdentifier().equals(id)){
                        return reference;
                    }
                }
            }
        }
        return createReference(ecoreElt);
    }

    @objid ("6e6eb3ec-1a76-11e1-8103-0027103f347d")
    public static Profile createEcoreProfile(final IProfile objingElt) {
        Profile ecoreProfile = UMLFactory.eINSTANCE.createProfile();
        
        ObjingEAnnotation.setModule(ecoreProfile, objingElt.getOwnerModule().getIdentifier());
        ObjingEAnnotation.addObjingID(ecoreProfile, objingElt.getIdentifier());
        ecoreProfile.setName(ProfileUtils.getName(objingElt));
        
        
        if (SysMLProfileUtils.isSysML(objingElt)){
            GenerationProperties.getInstance().setSysMLApplied();
            ecoreProfile.setName("SysML");
            SysMLProfileUtils.completeSysMLprofile(ecoreProfile);
            TotalExportMap.getInstance().put(objingElt.getIdentifier(), ecoreProfile);
        }else 
            GenerationProperties.getInstance().addExportedProfile(ecoreProfile);
        return ecoreProfile;
    }

    /**
     * @return
     */
    @objid ("6e8748fc-2a4a-11e1-9b22-0027103f347d")
    public static boolean isAnalystDeployed() {
        for (IModule module : Modelio.getInstance().getModelingSession().getModel().getUmlProject().getInstalled()){
            if (module.getIdentifier().equals("87d35db0-2fb3-11e0-b88e-002564c97630"))
                return true;
        }
        return false;
    }

}
