/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.osgi.framework.Bundle;

@objid ("1d65950c-4583-11e0-b54c-00137279a832")
public class ResourceLoader {
    @objid ("1d65950d-4583-11e0-b54c-00137279a832")
    private String iconCommonPath = "res" + java.io.File.separator + "bmp" +  java.io.File.separator;

    @objid ("1d65950e-4583-11e0-b54c-00137279a832")
    private static ResourceLoader INSTANCE = null;

    @objid ("55b7d290-62d0-11e1-8db4-0027103f347d")
    private final String umlResourcesJarName = "org.eclipse.uml2.uml.resources_3.1.1.v201008191505.jar";


    @objid ("1d659510-4583-11e0-b54c-00137279a832")
    private ResourceLoader() {
    }

    @objid ("1d67f760-4583-11e0-b54c-00137279a832")
    public static ResourceLoader getInstance() {
        if (INSTANCE == null)
            INSTANCE = new ResourceLoader();
        return INSTANCE;
    }

    @objid ("1d67f764-4583-11e0-b54c-00137279a832")
    public Image getImage(String resourceName) {
        final Bundle bundle = Platform.getBundle(XMI.PLUGIN_ID);
        
        final URL url = FileLocator.find(bundle, new Path(this.iconCommonPath + resourceName ), null);
        
        File file = null;
        try {
            file = new File(FileLocator.toFileURL(url).getPath());
        } catch (IOException e) {
          
            e.printStackTrace(Modelio.err);
        }
        return new Image(Display.getDefault(), file.getAbsolutePath());
    }

    @objid ("1d67f769-4583-11e0-b54c-00137279a832")
    public File getResource(final String resourceName) {
        final Bundle bundle = Platform.getBundle(XMI.PLUGIN_ID);
        
        final URL url = FileLocator.find(bundle, new Path("res"+ java.io.File.separator + resourceName ), null);
        
        File file = null;
        try {
            file = new File(FileLocator.toFileURL(url).getPath());
        } catch (IOException e) {
            e.printStackTrace(Modelio.err);
        }
        return file;
    }

    @objid ("1d6a59be-4583-11e0-b54c-00137279a832")
    public String getUMLRessource() {
        return "jar:file:" + this.getLibraryResource() + "!/";
    }

    @objid ("1d6a59c2-4583-11e0-b54c-00137279a832")
    public String getProjectRoot() {
        return Modelio.getInstance().getContext().getProjectSpacePath().getAbsolutePath();
    }

    @objid ("55ba6ab2-62d0-11e1-8db4-0027103f347d")
    private String getLibraryResource() {
        final Bundle bundle = Platform.getBundle(XMI.PLUGIN_ID);
        
        final URL url = FileLocator.find(bundle, new Path("lib"+ java.io.File.separator +  this.umlResourcesJarName ), null);
        
        try {
            return FileLocator.toFileURL(url).getPath();
        } catch (IOException e) {
            e.printStackTrace(Modelio.err);
        }
        return "";
    }

    @objid ("55bb2e06-62d0-11e1-8db4-0027103f347d")
    public File getXMITempFolder() {
        return new File(getProjectRoot() + java.io.File.separator  + "mda" + java.io.File.separator + "XMI" + java.io.File.separator + "temp");
    }

}
