/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import com.modeliosoft.modelio.api.model.extension.IModule;
import com.modeliosoft.modelio.api.model.extension.IProject;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.IInteraction;
import com.modeliosoft.modelio.api.model.uml.behavior.interaction.ILifeline;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.core.app.O;
import com.modeliosoft.modelio.core.jni.ITransaction;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.reverse.OwnedCompositionUml2Visitor;
import com.modeliosoft.modelio.xmi.reverse.PartialCreationImportVisitor;
import com.modeliosoft.modelio.xmi.reverse.PartialImportMap;
import com.modeliosoft.modelio.xmi.reverse.SequenceDiagramEltCreationVisitor;
import com.modeliosoft.modelio.xmi.reverse.SequenceElementVisitor;
import com.modeliosoft.modelio.xmi.reverse.TotalImportMap;
import com.modeliosoft.modelio.xmi.reverse.Uml2InsertableIFVisitor;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Profile;

/**
 * This class handles the properties needed during an XMI import
 * @author ebrosse
 */
@objid ("1d6cbc14-4583-11e0-b54c-00137279a832")
public class ReverseProperties {
    @objid ("1d6cbc17-4583-11e0-b54c-00137279a832")
    private static ReverseProperties INSTANCE = null;

    @objid ("1d6cbc18-4583-11e0-b54c-00137279a832")
    private OwnedCompositionUml2Visitor ecoreVisitor = null;

    @objid ("1d6cbc19-4583-11e0-b54c-00137279a832")
    private boolean roundtripEnabled = false;

    @objid ("1d6cbc1a-4583-11e0-b54c-00137279a832")
    private boolean timeDisplayerActivated = false;

    @objid ("1d6cbc1b-4583-11e0-b54c-00137279a832")
    private PartialCreationImportVisitor partialCreationImportVisitor = null;

    @objid ("1d6cbc1d-4583-11e0-b54c-00137279a832")
    private Uml2InsertableIFVisitor insertableIFVisitor = null;

    @objid ("1d6cbc1e-4583-11e0-b54c-00137279a832")
    private SequenceDiagramEltCreationVisitor sequenceDiagramEltCreationVisitor = null;

    @objid ("1d6cbc1f-4583-11e0-b54c-00137279a832")
    private SequenceElementVisitor sequenceElementVisitor = null;

    @objid ("1d6cbc20-4583-11e0-b54c-00137279a832")
    private IModule profileRoot = null;

    @objid ("1d6cbc21-4583-11e0-b54c-00137279a832")
    private boolean rollback = false;

    @objid ("1d6cbc22-4583-11e0-b54c-00137279a832")
    private Set<Package> ecoreModel = null;

    @objid ("1d6cbc26-4583-11e0-b54c-00137279a832")
    private Profile ecoreProfile = null;

    @objid ("8634080f-5668-11e1-8a6e-0027103f347d")
    private ITransaction openedTransaction = null;

    @objid ("6d85f5b2-1a76-11e1-8103-0027103f347d")
    private String filePath = "";

    @objid ("6d85f5b3-1a76-11e1-8103-0027103f347d")
    private IPackage externalPackage = null;

    @objid ("6d85f5b4-1a76-11e1-8103-0027103f347d")
    private IPackage umlRoot = null;

    @objid ("6d861cc4-1a76-11e1-8103-0027103f347d")
    private Map<String, ArrayList<String>> classTabConvertion = null;

    @objid ("6d8643d8-1a76-11e1-8103-0027103f347d")
    private List<Interaction> interactionList = null;

    @objid ("9d02ee83-1a76-11e1-8103-0027103f347d")
    private List<String> appliedProfiles = new ArrayList<String>();


    @objid ("1d6cbc28-4583-11e0-b54c-00137279a832")
    private ReverseProperties() {
    }

    /**
     * Provide the instance of the class
     * @return the ReverseProperties instance
     */
    @objid ("1d6f1e6e-4583-11e0-b54c-00137279a832")
    public static ReverseProperties getInstance() {
        if (INSTANCE == null)
            INSTANCE = new ReverseProperties();
        return INSTANCE;
    }

    /**
     * provide the current Ecore Visitor
     * @return EcoreVisitor
     */
    @objid ("1d7180d0-4583-11e0-b54c-00137279a832")
    public OwnedCompositionUml2Visitor getEcoreVisitor() {
        return this.ecoreVisitor;
    }

    /**
     * set the Ecore visitor for the current import
     * @param ecoreVisitor : the Ecore visitor
     */
    @objid ("1d7180d4-4583-11e0-b54c-00137279a832")
    public void setEcoreVisitor(OwnedCompositionUml2Visitor ecoreVisitor) {
        this.ecoreVisitor = ecoreVisitor;
    }

    /**
     * Provide the Ecore model resulting of the XMI export
     * @return Ecore model
     */
    @objid ("1d73e32a-4583-11e0-b54c-00137279a832")
    public Set<Package> getEcoreModel() {
        return this.ecoreModel;
    }

    /**
     * Provide the Transaction opened for the XMI import
     * @return Import transaction
     */
    @objid ("1d73e330-4583-11e0-b54c-00137279a832")
    public ITransaction getOpenedTransaction() {
        return this.openedTransaction;
    }

    /**
     * This method allows to know if the Modelio annotation are available
     * @return true if the imported XMI files contains Modelio annotations
     */
    @objid ("1d76457d-4583-11e0-b54c-00137279a832")
    public boolean isRoundtripEnabled() {
        return this.roundtripEnabled;
    }

    /**
     * This method allows to know if the time displayer is activated
     * @return true if the time displayer is activated
     */
    @objid ("1d764581-4583-11e0-b54c-00137279a832")
    public boolean isTimeDisplayerActivated() {
        return this.timeDisplayerActivated;
    }

    /**
     * This method allows to activated or not the time displayer for the current export
     * @param activateTimeDisplayer : if true is able the time displayer
     */
    @objid ("1d764585-4583-11e0-b54c-00137279a832")
    public void setTimeDisplayerActivated(final boolean activateTimeDisplayer) {
        this.timeDisplayerActivated = activateTimeDisplayer;
    }

    /**
     * This method is called in order to closed the behaviours dedicated to the sequence import
     */
    @objid ("1d78a7d6-4583-11e0-b54c-00137279a832")
    public void closeSequenceDiagrams() {
        for (Interaction interaction : this.interactionList) {
            IInteraction objInteraction = (IInteraction) getMappedElement(interaction);
        
            ObList<ILifeline> objLines = objInteraction.getOwnedLine();
            for (ILifeline objLine : objLines){
                objInteraction.removeOwnedLine(objLine);
            }
        
            for (Lifeline line : interaction.getLifelines()){
                ILifeline objLine = (ILifeline) getMappedElement(line);
                objInteraction.addOwnedLine(objLine);
        
            }
        }
        
        this.interactionList.clear();
    }

    /**
     * This method returns the partial creation visitor of the current import
     * @return the current PartialCreationImportVisitor
     */
    @objid ("1d78a7d8-4583-11e0-b54c-00137279a832")
    public PartialCreationImportVisitor getPartialCreationImportVisitor() {
        return this.partialCreationImportVisitor;
    }

    /**
     * This methods initializes the properties of the import
     */
    @objid ("1d78a7dc-4583-11e0-b54c-00137279a832")
    public void initialize() {
        PrimitiveTypeMapper.clean();
        TotalImportMap.getInstance().clear();
        PartialImportMap.getInstance().clear();
        
        if (this.interactionList == null)
            this.interactionList = new ArrayList<Interaction>();
        else
            this.interactionList.clear();
        
        this.partialCreationImportVisitor = new PartialCreationImportVisitor();
        this.insertableIFVisitor = new Uml2InsertableIFVisitor();
        this.sequenceDiagramEltCreationVisitor = new SequenceDiagramEltCreationVisitor();
        this.sequenceElementVisitor = new SequenceElementVisitor();
        IProject project = Modelio.getInstance().getModelingSession().getModel().getUmlProject();
        
        boolean found = false; 
        for (IModule module : project.getInstalled()){
            if (module.getName().equals("LocalModule")){
                found = true;
                setProfileRoot(module);
            }
        }
        
        if (!found){
            IModule localModule = O.getDefault().getModelingSession().getModelFactory().createModule();
            localModule.setName("LocalModule");
            project.addInstalled(localModule);
            setProfileRoot(localModule);                   
        }
        
        if (this.ecoreModel != null)
            this.ecoreModel.clear();
        
        this.rollback = false;
    }

    /**
     * This method returns the current visitor of the interaction fragments
     * @return the current Uml2InsertableIFVisitor
     */
    @objid ("1d78a7de-4583-11e0-b54c-00137279a832")
    public Uml2InsertableIFVisitor getInsertableIFVisitor() {
        return this.insertableIFVisitor;
    }

    /**
     * This method returns the current creation visitor of the sequence diagrams
     * @return the current SequenceDiagramEltCreationVisitor
     */
    @objid ("1d7b0a30-4583-11e0-b54c-00137279a832")
    public SequenceDiagramEltCreationVisitor getSequenceDiagramEltCreationVisitor() {
        return this.sequenceDiagramEltCreationVisitor;
    }

    /**
     * This method returns the current creation visitor of the sequence diagrams elements
     * @return the current SequenceElementVisitor
     */
    @objid ("1d7b0a34-4583-11e0-b54c-00137279a832")
    public SequenceElementVisitor getSequenceElementVisitor() {
        return this.sequenceElementVisitor;
    }

    /**
     * This method allows to add an Ecore model to the current reverses
     * @param model : the Ecore model to add
     */
    @objid ("1d7b0a3b-4583-11e0-b54c-00137279a832")
    public void addEcoreModel(Package model) {
        if (this.ecoreModel == null)
            this.ecoreModel = new HashSet<Package>();
        this.ecoreModel.add(model);
    }

    /**
     * This method allows to store the transaction of the import
     * @param openedTransaction : the import transaction
     */
    @objid ("1d7b0a3e-4583-11e0-b54c-00137279a832")
    public void setOpenedTransaction(ITransaction openedTransaction) {
        this.openedTransaction = openedTransaction;
    }

    /**
     * This method stores the availability of Modelio annotation in a specific Ecore model
     * @param importedPck : the given Ecore model
     */
    @objid ("1d7d6c8c-4583-11e0-b54c-00137279a832")
    public void storeImportModelRoundtripMode(final Package importedPck) {
        this.roundtripEnabled = ObjingEAnnotation
        .isRoundTrip(importedPck);
    }

    /**
     * This method returns the import result of a given Ecore element
     * @param ecoreElt : the imported Ecore element
     * @return the result of the import
     */
    @objid ("1d7d6c8f-4583-11e0-b54c-00137279a832")
    public Object getMappedElement(Element ecoreElt) {
        PartialImportMap partialCreationMap = PartialImportMap.getInstance();
        TotalImportMap totalCreationMap = TotalImportMap.getInstance();
        
        Object mappedElement = totalCreationMap.get(ecoreElt);
        
        //Galaxy
        if (mappedElement == null)
        	mappedElement = ObjingModelNavigation.searchElement(ecoreElt);
        
        if (mappedElement == null)
            mappedElement = partialCreationMap.get(ecoreElt);
        
        if (mappedElement == null) {
            mappedElement = this.partialCreationImportVisitor
            .createPartialObjingElt(ecoreElt);
        }
        return mappedElement;
    }

    /**
     * This method tests if a given Ecore element has been already mapped
     * @param ecoreElt : the tested element
     * @return true if the tested element has been mapped
     */
    @objid ("1d7d6c95-4583-11e0-b54c-00137279a832")
    public boolean isMappedElement(Element ecoreElt) {
        if (TotalImportMap.getInstance().get(ecoreElt) != null)
            return true;
        else if (PartialImportMap.getInstance().get(ecoreElt) != null)
            return true;
        else
            return false;
    }

    /**
     * This method supress an element form the import
     * @param ecoreElt : the ecore element to supress
     * @param objingElt : the corresponding modelio element to supress
     */
    @objid ("1d7fcee4-4583-11e0-b54c-00137279a832")
    public void deleteElement(Element ecoreElt, IElement objingElt) {
        PartialImportMap.getInstance().remove(ecoreElt);
        TotalImportMap.getInstance().remove(ecoreElt);
        Modelio.getInstance().getModelingSession().getModel()
        .deleteElement(objingElt);
    }

    /**
     * This method sets the profile root
     * @param root : the profile root
     */
    @objid ("1d7fcee8-4583-11e0-b54c-00137279a832")
    public void setProfileRoot(IModule root) {
        this.profileRoot = root;
    }

    /**
     * This method returns the profile root of the current export
     * @return the profile root
     */
    @objid ("1d7fceeb-4583-11e0-b54c-00137279a832")
    public IModule getProfileRoot() {
        if (this.profileRoot != null)
            return this.profileRoot;
        
        for (IModule module : O.getDefault().getModelingSession().getModel().getProject().getInstalled()){
            if (module.getName().equals("LocalModule"))
                return module;
        }
        return null;
    }

    /**
     * This method sets the imported Ecore profile
     * @param profile : the imported profile
     */
    @objid ("1d7fceef-4583-11e0-b54c-00137279a832")
    public void setEcoreProfile(final Profile profile) {
        this.ecoreProfile = profile;
    }

    /**
     * This method returns the imported Ecore profile
     * @return the imported Ecore profile
     */
    @objid ("1d82313e-4583-11e0-b54c-00137279a832")
    public Profile getEcoreProfile() {
        return this.ecoreProfile;
    }

    /**
     * This method allows to know if the import would be rollbacked
     * @return true if the import must be rollbacked
     */
    @objid ("1d84939d-4583-11e0-b54c-00137279a832")
    public boolean isRollback() {
        return this.rollback;
    }

    /**
     * This method allows to specify if the import transaction must be rollbacked
     * @param rollback : need of rollbacking the current transaction
     */
    @objid ("1d8493a1-4583-11e0-b54c-00137279a832")
    public void setRollback(boolean rollback) {
        this.rollback = rollback;
    }

    /**
     * This method clean the properties of the current import
     */
    @objid ("1d8493a4-4583-11e0-b54c-00137279a832")
    public void clean() {
        this.interactionList.clear();
        this.ecoreModel.clear();
        this.externalPackage = null;
        this.appliedProfiles = new ArrayList<String>();
    }

    /**
     * THis methods allow to add an interaction to the list ofg imported interactions
     * @param interaction : the Interaction to add
     */
    @objid ("6d8c5ea6-1a76-11e1-8103-0027103f347d")
    public void addInteraction(final Interaction interaction) {
        this.interactionList.add(interaction);
    }

    /**
     * This methods returns the path of the imported file
     * @return the imported file path
     */
    @objid ("6d8c85b8-1a76-11e1-8103-0027103f347d")
    public String getFilePath() {
        return this.filePath;
    }

    /**
     * This methods allows to add a profile to the list of applied profiles
     * @param appliedProfile : the profile to add
     */
    @objid ("6d8cacca-1a76-11e1-8103-0027103f347d")
    public void addAppliedProfile(final String appliedProfile) {
        if (!this.appliedProfiles.contains(appliedProfile))
            this.appliedProfiles.add(appliedProfile);
    }

    /**
     * This method allows to specify the path of the imported file
     * @param filePath : the imported file path
     */
    @objid ("6d8cd3dc-1a76-11e1-8103-0027103f347d")
    public void setFilePath(final String filePath) {
        this.filePath = filePath;
    }

    /**
     * This method returns the 'external' Package used in order to store additional elements
     * @return the 'external' Package
     */
    @objid ("6d8cfaee-1a76-11e1-8103-0027103f347d")
    public IPackage getExternalPackage() {
        if (this.externalPackage == null){
            this.externalPackage = Modelio.getInstance().getModelingSession().getModel().createPackage();
            this.externalPackage.setName( XMI.I18N.getString("Ui.ExternalPackage.Name"));
            this.externalPackage.setOwner( Modelio.getInstance().getModelingSession().getModel().getRoot());
        }
        return this.externalPackage;
    }

    /**
     * This method returns the list of imported profiles
     * @return imported profiles
     */
    @objid ("6d8d2200-1a76-11e1-8103-0027103f347d")
    public List<String> getAppliedProfiles() {
        return this.appliedProfiles;
    }

    /**
     * This method adds a given profile to the list of imported profile
     * @param importedProfiles : the profiles to add
     */
    @objid ("6d8d4912-1a76-11e1-8103-0027103f347d")
    public void addAppliedProfiles(final ArrayList<String> importedProfiles) {
        this.appliedProfiles.addAll(importedProfiles);
    }

    /**
     * This methods specify the path of the imported file
     * @param file : the imported file
     */
    @objid ("6d8d9736-1a76-11e1-8103-0027103f347d")
    public void setFilePath(final File file) {
        this.filePath = file.getAbsolutePath();
    }

    /**
     * This method returns the Modelio package owner of the launched import
     * @return import owner package
     */
    @objid ("6d8e3380-1a76-11e1-8103-0027103f347d")
    public IPackage getUMLRoot() {
        if (this.umlRoot != null)
            return this.umlRoot;
        return Modelio.getInstance().getModelingSession().getModel().getRoot();
    }

    /**
     * This method allows to specify the owner of the XMI import
     * @param root : the package which will own the import result
     */
    @objid ("6d8e5a91-1a76-11e1-8103-0027103f347d")
    public void setUMLRoot(final IPackage root) {
        this.umlRoot = root;
    }

    @SuppressWarnings("serial")
    @objid ("6d8e81a4-1a76-11e1-8103-0027103f347d")
    private void initClassTabConvertion() {
        this.classTabConvertion = new HashMap<String, ArrayList<String>>();
        this.classTabConvertion.put("Abstraction",  new ArrayList<String>(){{add("Dependency");}});
        this.classTabConvertion.put("AcceptCallEventAction",  new ArrayList<String>(){{add("AcceptCallEventAction");}});
        this.classTabConvertion.put("AcceptEventAction",  new ArrayList<String>(){{add("AcceptCallEventAction"); 
        add("AcceptSignalAction");    add("AcceptChangeEventAction");add("AcceptTimeEventAction");}});
        this.classTabConvertion.put("Action",  new ArrayList<String>(){{add("ActivityAction");}});
        this.classTabConvertion.put("ActionInputPin",  new ArrayList<String>(){{add("InputPin");}});
        this.classTabConvertion.put("Activity",  new ArrayList<String>(){{add("Activity");}});
        this.classTabConvertion.put("ActivityEdge",  new ArrayList<String>(){{add("ActivityEdge");}});
        this.classTabConvertion.put("ActivityFinalNode",  new ArrayList<String>(){{add("ActivityFinalNode");}});
        this.classTabConvertion.put("ActivityParameterNode",  new ArrayList<String>(){{add("ActivityParameterNode");}});
        this.classTabConvertion.put("ActivityPartition",  new ArrayList<String>(){{add("ActivityPartition");}});
        this.classTabConvertion.put("Actor",  new ArrayList<String>(){{add("Actor");}});
        this.classTabConvertion.put("AddStructuralFeature",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("AddVariableValueAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("AnyReceiveEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("Artifact",  new ArrayList<String>(){{add("Artifact");}});
        this.classTabConvertion.put("Association",  new ArrayList<String>(){{add("Association");}});
        this.classTabConvertion.put("AssociationClass",  new ArrayList<String>(){{add("Class");}});
        this.classTabConvertion.put("Behavior",  new ArrayList<String>(){{add("Behavior");}});
        this.classTabConvertion.put("BehaviorFeature",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("BroadcastSignalAction",  new ArrayList<String>(){{add("SendSignalAction");}});
        this.classTabConvertion.put("CallBehaviorAction",  new ArrayList<String>(){{add("CallBehaviorAction");}});
        this.classTabConvertion.put("CallEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("CallOperationAction",  new ArrayList<String>(){{add("CallOperationAction");}});        
        this.classTabConvertion.put("CentralBufferNode",  new ArrayList<String>(){{add("CentralBufferNode");}});
        this.classTabConvertion.put("ChangeEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("Class",  new ArrayList<String>(){{add("Class");}});
        this.classTabConvertion.put("Classifier",  new ArrayList<String>(){{add("Classifier");
        add("Association"); add("Collaboration");}});
        this.classTabConvertion.put("ClassifierTemplateParameter",  new ArrayList<String>(){{add("TemplateParameter");}});
        this.classTabConvertion.put("Clause",  new ArrayList<String>(){{add("Clause");}});
        this.classTabConvertion.put("ClearAssociationAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ClearStructuralFeatureAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ClearVariableAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("Collaboration",  new ArrayList<String>(){{add("Collaboration");}});        
        this.classTabConvertion.put("CollaborationUse",  new ArrayList<String>(){{add("CollaborationUse");}});
        this.classTabConvertion.put("Comment",  new ArrayList<String>(){{add("Note");}});
        this.classTabConvertion.put("CommunicationPath",  new ArrayList<String>(){{add("Association");}});
        this.classTabConvertion.put("Component",  new ArrayList<String>(){{add("Component");}});       
        this.classTabConvertion.put("ConditionalNode",  new ArrayList<String>(){{add("ConditionalNode");}});
        this.classTabConvertion.put("ConnectableElement",  new ArrayList<String>(){{add("Parameter");
        add("BehaviorParameter"); add("AssociationEnd"); add("BindableInstance"); add("Attribute");}});
        this.classTabConvertion.put("Connector",  new ArrayList<String>(){{add("Connector");}});
        this.classTabConvertion.put("ConnectorEnd",  new ArrayList<String>(){{add("ConnectorEnd");}});
        this.classTabConvertion.put("Constraint",  new ArrayList<String>(){{add("Class");}});
        this.classTabConvertion.put("ControlFlow",  new ArrayList<String>(){{add("ControlFlow");}});
        this.classTabConvertion.put("CreateLinkAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("CreateLinkObject",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("CreateObjectAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("CreationEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("DataStoreNode",  new ArrayList<String>(){{add("DataStoreNode");}});
        this.classTabConvertion.put("DataType",  new ArrayList<String>(){{add("DataType");}});
        this.classTabConvertion.put("DecisionNode",  new ArrayList<String>(){{add("DecisionMergeNode");}});
        this.classTabConvertion.put("Dependency",  new ArrayList<String>(){{add("Dependency");}});
        this.classTabConvertion.put("Deployment",  new ArrayList<String>(){{add("Dependency");}});
        this.classTabConvertion.put("DeploymentSpecification",  new ArrayList<String>(){{add("Artifact");}});
        this.classTabConvertion.put("DestroyLinkAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("DestroyObjectAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("DestructionEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("Device",  new ArrayList<String>(){{add("Node");}});
        this. classTabConvertion.put("Element",  new ArrayList<String>(){{add("Element");}});
        this.classTabConvertion.put("ElementImport",  new ArrayList<String>(){{add("ElementImport");}});
        this.classTabConvertion.put("Enumeration",  new ArrayList<String>(){{add("Enumeration");}});
        this.classTabConvertion.put("EnumerationLiteral",  new ArrayList<String>(){{add("EnumerationLiteral");}});
        this.classTabConvertion.put("Event",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("ExceptionHandler",  new ArrayList<String>(){{add("ObjectFlow");}});
        this.classTabConvertion.put("ExecutionEnvironment",  new ArrayList<String>(){{add("Node");}});
        this.classTabConvertion.put("ExecutionEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("ExpansionNode",  new ArrayList<String>(){{add("Pin");}});
        this.classTabConvertion.put("ExpansionRegion",  new ArrayList<String>(){{add("StructuredActivityNode");}});
        this.classTabConvertion.put("Extend",  new ArrayList<String>(){{add("UseCaseDependency");}});
        this.classTabConvertion.put("ExtensionPoint",  new ArrayList<String>(){{add("ExtensionPoint");}});
        this.classTabConvertion.put("Feature",  new ArrayList<String>(){{add("Feature");}});
        this.classTabConvertion.put("FinalState",  new ArrayList<String>(){{add("FinalState");}});
        this.classTabConvertion.put("FlowFinalNode",  new ArrayList<String>(){{add("FlowFinalNode");}});
        this.classTabConvertion.put("ForkNode",  new ArrayList<String>(){{add("ForkJoinNode");}});
        this.classTabConvertion.put("Gate",  new ArrayList<String>(){{add("Gate");}});
        this.classTabConvertion.put("Generalization",  new ArrayList<String>(){{add("Generalization");}});
        this.classTabConvertion.put("Include",  new ArrayList<String>(){{add("UseCaseDependency");}});
        this.classTabConvertion.put("InformationFlow",  new ArrayList<String>(){{add("InformationFlow");}});
        this.classTabConvertion.put("InformationItem",  new ArrayList<String>(){{add("InformationItem");}});
        this.classTabConvertion.put("InitialNode",  new ArrayList<String>(){{add("InitialNode");}});
        this.classTabConvertion.put("InputPin",  new ArrayList<String>(){{add("InputPin");}});
        this. classTabConvertion.put("InstanceSpecification",  new ArrayList<String>(){{add("Link");
        add("Instance");add("EnumerationLiteral");}});
        this.classTabConvertion.put("Interaction",  new ArrayList<String>(){{add("Interaction");}});
        this.classTabConvertion.put("Interface",  new ArrayList<String>(){{add("Interface");}});
        this.classTabConvertion.put("InterfaceRealization",  new ArrayList<String>(){{add("InterfaceRealization");}});
        this.classTabConvertion.put("InterruptibleActivityRegion",  new ArrayList<String>(){{add("InterruptibleActivityRegion");}});
        this.classTabConvertion.put("InvocationAction",  new ArrayList<String>(){{add("CallAction");
        add("SendSignalAction"); add("OpaqueAction");}});
        this.classTabConvertion.put("JoinNode",  new ArrayList<String>(){{add("ForkJoinNode");}});
        this.classTabConvertion.put("Lifeline",  new ArrayList<String>(){{add("Lifeline");}});
        this.classTabConvertion.put("LoopNode",  new ArrayList<String>(){{add("LoopNode");}});
        this.classTabConvertion.put("Manifestation",  new ArrayList<String>(){{add("Manifestation");}});
        this.classTabConvertion.put("MergeNode",  new ArrayList<String>(){{add("DecisionMergeNode");}});
        this.classTabConvertion.put("Message",  new ArrayList<String>(){{add("Message");}});
        this.classTabConvertion.put("MessageOccurrenceSpecification",  new ArrayList<String>(){{add("MessageOccurrenceSpecification");}});
        this.classTabConvertion.put("Model",  new ArrayList<String>(){{add("Package");}});
        this.classTabConvertion.put("ModelElement",  new ArrayList<String>(){{add("NamedElement");}});
        this.classTabConvertion.put("Node",  new ArrayList<String>(){{add("Node");}});
        this.classTabConvertion.put("ObjectFlow",  new ArrayList<String>(){{add("ObjectFlow");}});
        this.classTabConvertion.put("ObjectNode",  new ArrayList<String>(){{add("ObjectNode");}});
        this.classTabConvertion.put("OccurenceSpecification",  new ArrayList<String>(){{add("OccurenceSpecification");}});
        this.classTabConvertion.put("OpaqueAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("OpaqueBehavior",  new ArrayList<String>(){{add("Signal");}});
        this.classTabConvertion.put("Operation",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("OperationTemplateParameter",  new ArrayList<String>(){{add("Parameter");}});
        this.classTabConvertion.put("OutputPin",  new ArrayList<String>(){{add("OutputPin");}});
        this.classTabConvertion.put("Package",  new ArrayList<String>(){{add("Package");}});
        this.classTabConvertion.put("PackageImport",  new ArrayList<String>(){{add("PackageImport");}});
        this.classTabConvertion.put("PackageMerge",  new ArrayList<String>(){{add("PackageMerge");}});
        this.classTabConvertion.put("Parameter",  new ArrayList<String>(){{add("Parameter");}});
        this.classTabConvertion.put("Pin",  new ArrayList<String>(){{add("Pin");}});
        this.classTabConvertion.put("Port",  new ArrayList<String>(){{add("Port");}});
        this.classTabConvertion.put("PrimitiveType",  new ArrayList<String>(){{add("DataType");}});
        this.classTabConvertion.put("Profile",  new ArrayList<String>(){{add("Profile");}});
        this.classTabConvertion.put("Property",  new ArrayList<String>(){{add("AssociationEnd");
        add("BindableInstance"); add("Attribute");}});
        this.classTabConvertion.put("ProtocolConformance",  new ArrayList<String>(){{add("Dependency");}});
        this.classTabConvertion.put("ProtocolStateMachine",  new ArrayList<String>(){{add("StateMachine");}});
        this.classTabConvertion.put("ProtocolTransition",  new ArrayList<String>(){{add("Transition");add("InternalTransition");}});
        this.classTabConvertion.put("PseudoState",  new ArrayList<String>(){{add("PseudoState");}});
        this.classTabConvertion.put("RaiseExceptionAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadExtentAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadIsClassifierAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadLinkAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadLinkObjectEndAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadLinkObjectEndQualifierAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadSelfAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadStructuralFeatureAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReadVariableAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("Realization",  new ArrayList<String>(){{add("Realization");}});
        this.classTabConvertion.put("Reception",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("ReclassifyObjectAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("RedefinableTemplateSignature",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("ReduceAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("Region",  new ArrayList<String>(){{add("Region");}});
        this.classTabConvertion.put("RemoveStructuralFeatureValueAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("RemoveVariableValueAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("ReplyAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("SendObjectAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("SendSignalAction",  new ArrayList<String>(){{add("SendSignalAction");}});
        this.classTabConvertion.put("SequenceNode",  new ArrayList<String>(){{add("StructuredActivityNode");}});
        this.classTabConvertion.put("Signal",  new ArrayList<String>(){{add("Signal");}});
        this.classTabConvertion.put("Slot",  new ArrayList<String>(){{add("Port");
        add("BindableInstance");add("ConnectorEnd");add("LinkEnd");}});
        this.classTabConvertion.put("StartClassifierBehaviorAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("State",  new ArrayList<String>(){{add("State");}});
        this.classTabConvertion.put("StateInvariant",  new ArrayList<String>(){{add("StateInvariant");}});
        this.classTabConvertion.put("StateMachine",  new ArrayList<String>(){{add("StateMachine");}});
        this.classTabConvertion.put("Stereotype",  new ArrayList<String>(){{add("Stereotype");}});
        this.classTabConvertion.put("StructuredActivityNode",  new ArrayList<String>(){{add("StructuredActivityNode");}});
        this.classTabConvertion.put("StructuredClassifier",  new ArrayList<String>(){{add("Collaboration");
        add("Class");add("Behavior");add("Node");}});
        this.classTabConvertion.put("Substitution",  new ArrayList<String>(){{add("Substitution");}});
        this.classTabConvertion.put("TemplateBinding",  new ArrayList<String>(){{add("TemplateBinding");}});
        this.classTabConvertion.put("TemplateParameter",  new ArrayList<String>(){{add("TemplateParameter");}});
        this.classTabConvertion.put("TemplateParameterSubstitution",  new ArrayList<String>(){{add("TemplateParameterSubstitution");}});
        this.classTabConvertion.put("TemplateSignature",  new ArrayList<String>(){{add("Operation");}});
        this.classTabConvertion.put("TestIdentityAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("TimeEvent",  new ArrayList<String>(){{add("Event");}});
        this.classTabConvertion.put("Transition",  new ArrayList<String>(){{add("Transition");
        add("InternalTransition");}});
        this.classTabConvertion.put("UnmarshallAction",  new ArrayList<String>(){{add("OpaqueAction");}});
        this.classTabConvertion.put("Usage",  new ArrayList<String>(){{add("Usage");}});
        this.classTabConvertion.put("UseCase",  new ArrayList<String>(){{add("UseCase");}});
        this.classTabConvertion.put("ValuePin",  new ArrayList<String>(){{add("InputPin");}});
        this.classTabConvertion.put("ValueSpecificationAction",  new ArrayList<String>(){{add("OpaqueAction");}});
    }

    /**
     * This methods returns the list of Modelio classes corresponding to a given Ecore class
     * @param ecoreClassName : the name of the Ecore class
     * @return the list of the corresponding Modelio classes
     */
    @objid ("6d8ea8b4-1a76-11e1-8103-0027103f347d")
    public List<String> getObjClassNames(final String ecoreClassName) {
        if (this.classTabConvertion == null)
                initClassTabConvertion();
        
        List<String> result = this.classTabConvertion.get(ecoreClassName);
        
        if (result == null)
            return new ArrayList<String>();
        else
            return result;
    }

}
