/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import com.modeliosoft.modelio.api.model.analyst.IRequirement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IDependency;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.api.utils.StereotypeNotFoundException;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

@objid ("6e985e1e-1a76-11e1-8103-0027103f347d")
public class SysMLProfileUtils {
    @objid ("6e988530-1a76-11e1-8103-0027103f347d")
    private static String sysmlId = "00bc42a4-0000-1968-0000-000000000000";


    @objid ("6e988531-1a76-11e1-8103-0027103f347d")
    public static boolean isSysML(final IElement profile) {
        return profile.getIdentifier().equals(sysmlId);
    }

    @objid ("6e98ac43-1a76-11e1-8103-0027103f347d")
    public static void completeSysMLprofile(final Profile sysMLProfile) {
        addRequirementStereotype(sysMLProfile);
        addDeriveStereotype(sysMLProfile);
        addSatisfyStereotype(sysMLProfile);
        addVerifyStereotype(sysMLProfile);
        addSysMLNoteStereotypes(sysMLProfile);
    }

    @objid ("6e98d354-1a76-11e1-8103-0027103f347d")
    private static void addVerifyStereotype(final Profile sysMLProfile) {
        Stereotype stereotype = sysMLProfile.createOwnedStereotype("Verify", false); 
        ProfileUtils.addReference(stereotype, "Abstraction");
        try{
            IStereotype obStereotype = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IDependency.class, "verify");
            TotalExportMap.getInstance().put(obStereotype.getIdentifier(), stereotype);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
    }

    @objid ("6e98d358-1a76-11e1-8103-0027103f347d")
    private static void addSatisfyStereotype(final Profile sysMLProfile) {
        Stereotype stereotype = sysMLProfile.createOwnedStereotype("Satisfy", false); 
        ProfileUtils.addReference(stereotype, "Abstraction");
        try{
            IStereotype obStereotype = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IDependency.class, "satisfy");
            TotalExportMap.getInstance().put(obStereotype.getIdentifier(), stereotype);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
    }

    @objid ("6e98fa68-1a76-11e1-8103-0027103f347d")
    private static void addDeriveStereotype(final Profile sysMLProfile) {
        Stereotype stereotype = sysMLProfile.createOwnedStereotype("DeriveReqt", false); 
        ProfileUtils.addReference(stereotype, "Abstraction");
        
        try{
            IStereotype obStereotype = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IDependency.class, "derive");
            TotalExportMap.getInstance().put(obStereotype.getIdentifier(), stereotype);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
    }

    @objid ("6e98fa6c-1a76-11e1-8103-0027103f347d")
    private static void addRequirementStereotype(final Profile sysMLProfile) {
        Stereotype stereotype = sysMLProfile.createOwnedStereotype("Requirement", false); 
        ProfileUtils.addReference(stereotype, "Class");
        
        try{
            IStereotype obStereotype = Modelio.getInstance().getModelingSession().getMetamodelExtensions()
            .getStereotype(IRequirement.class, "requirement");
            TotalExportMap.getInstance().put(obStereotype.getIdentifier(), stereotype);
        }catch(StereotypeNotFoundException e){
            e.printStackTrace(Modelio.err);
        }
        
        Property text = stereotype.createOwnedAttribute("Text", PrimitiveTypeMapper.getString());
        text.setLower(1);
        text.setUpper(1);
        
        Property id = stereotype.createOwnedAttribute("Id", PrimitiveTypeMapper.getString());
        id.setLower(1);
        id.setUpper(1);
    }

    @objid ("6e99217b-1a76-11e1-8103-0027103f347d")
    private static void addSysMLNoteStereotypes(final Profile sysMLProfile) {
        Stereotype rationaleStereotype = sysMLProfile.createOwnedStereotype("Rationale", false); 
        ProfileUtils.addReference(rationaleStereotype, "Comment");
        
        Stereotype problemStereotype = sysMLProfile.createOwnedStereotype("Problem", false); 
        ProfileUtils.addReference(problemStereotype, "Comment");
        
        Stereotype relatedStereotype = sysMLProfile.createOwnedStereotype("RequirementRelated", false); 
        ProfileUtils.addReference(relatedStereotype, "NamedElement");
    }

}
