/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;

import com.modeliosoft.modelio.api.model.uml.infrastructure.IMetaclassReference;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IModelElement;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IStereotype;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagParameter;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITagType;
import com.modeliosoft.modelio.api.model.uml.infrastructure.ITaggedValue;
import com.modeliosoft.modelio.api.utils.ObList;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.generation.TotalExportMap;
import com.modeliosoft.modelio.xmi.plugin.XMI;

/**
 * This class provides services for the export of ITaggedValue
 * @author ebrosse
 */
@objid ("8bc5c56e-1a74-11e1-8103-0027103f347d")
public class TaggedValueUtils {

    @objid ("6e9996ae-1a76-11e1-8103-0027103f347d")
    public static void setValue(final IModelElement element, final Element ecoreElement) {
        TotalExportMap totalCreationMap = TotalExportMap.getInstance();
        
        List<ITaggedValue> listTag = sortByType(element);
        
        ITagType tagType = null; 
        
        for (ITaggedValue taggedValue : listTag){
        
            if (!(taggedValue.getDefinition().equals(tagType))){
                tagType = taggedValue.getDefinition();
            }
        
            // find stereotype
            Stereotype stereotype = null;
            IStereotype obStereotype = taggedValue.getDefinition().getOwnerStereotype();
        
            if (obStereotype != null){
                IStereotype appliedSterotype = null;
                for (IStereotype extension : taggedValue.getAnnoted().getExtension()){
                    IStereotype temp = extension;
                    while (temp != null ){
                        if (temp.equals(obStereotype)){
                            appliedSterotype = extension;
                            stereotype = (Stereotype) TotalExportMap.getInstance().get(appliedSterotype.getIdentifier());
                        }
                        temp = temp.getParent();
                    }
                }
            }else{
                IMetaclassReference obReference = taggedValue.getDefinition().getOwnerReference();    
                stereotype = (Stereotype) totalCreationMap.get(obReference.getIdentifier());
            }
        
            if (stereotype != null){
        
                String tagTypeName = ProfileUtils.getTagTypeName(taggedValue.getDefinition());
        
                //Cas boolean 
                if (taggedValue.getDefinition().getParamNumber().equals("0")){
                    if (stereotype != null){
                        Object value = ecoreElement.getValue(stereotype, tagTypeName);
        
                        try{
        
                            if ((value != null) && value.equals(false)){
                                ecoreElement.setValue(stereotype, tagTypeName, true);
                            }
        
                        }catch(IllegalArgumentException e){
                            XMILogs xmilogs = XMILogs.getInstance();
                            String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                    taggedValue.getDefinition().getName(), 
                                    element.getName());
        
                            xmilogs.writelnInLog(message);
        
                            GenerationProperties.getInstance().getReportModel().addWarning(message, element);
        
                        }
                    }
                }else {
                    //Cas String
        
                    if ((tagType != null) && (tagType.getParamNumber().equals("1"))){
        
                        if (taggedValue.getActual().size() > 0){
                            String value = taggedValue.getActual().get(0).getValue();
                            try{
        
                                ecoreElement.setValue(stereotype, tagTypeName, value);
        
                            }catch(IllegalArgumentException e){
                                XMILogs xmilogs = XMILogs.getInstance();
                                String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                        taggedValue.getDefinition().getName(), 
                                        element.getName());
        
                                xmilogs.writelnInLog(message);
        
                                GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                            }
                        }
                    }else{
                        //Cas tableau de string
                        try{
                            EDataTypeUniqueEList<String> valueList = (EDataTypeUniqueEList<String>) ecoreElement.getValue(stereotype, tagTypeName);
                            for (ITagParameter tagParameter : taggedValue.getActual() ){
                                if (tagParameter.getValue() != null)
                                    valueList.add(tagParameter.getValue());
        
                            }
                        }catch(IllegalArgumentException e){
                            XMILogs xmilogs = XMILogs.getInstance();
                            String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                    taggedValue.getDefinition().getName(), 
                                    element.getName());
        
                            xmilogs.writelnInLog(message);
        
                            GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                        }
                    }
                }
            }
        }
    }

    @objid ("6e99bdc3-1a76-11e1-8103-0027103f347d")
    public static void setValueEAnnotation(final IModelElement element, final Element ecoreElement) {
        int currentString = 0;
        
        List<ITaggedValue> listTag = sortByType(element);
        
        ITagType tagType = null; 
        
        for (ITaggedValue taggedValue : listTag){
        
            if (!(taggedValue.getDefinition().equals(tagType))){
                tagType = taggedValue.getDefinition();
        
                currentString = 0;
            }
        
            //get stereotype owner
        
            IStereotype obStereotype = taggedValue.getDefinition().getOwnerStereotype();
            Stereotype stereotype = null;
        
            if (obStereotype != null){
        
                IStereotype appliedSterotype = null;
                for (IStereotype extension : taggedValue.getAnnoted().getExtension()){
                    IStereotype temp = extension;
                    while (temp != null ){
                        if (temp.equals(obStereotype)){
                            appliedSterotype = extension;
                            stereotype = (Stereotype) TotalExportMap.getInstance().get(appliedSterotype.getIdentifier());
                        }
                        temp = temp.getParent();
                    }
                }
        
            }else{
                IMetaclassReference obReference = taggedValue.getDefinition().getOwnerReference();    
                TotalExportMap totalCreationMap = TotalExportMap.getInstance();
                stereotype = (Stereotype) totalCreationMap.get(obReference.getIdentifier());
            }
        
            if (stereotype != null){
        
                String tagTypeName = ProfileUtils.getTagTypeName(taggedValue.getDefinition());
                String name = null; 
                if (taggedValue.getDefinition().getParamNumber().equals("0")){
        
                    Object value = ecoreElement.getValue(stereotype, tagTypeName);
                    try{
                        if ((value != null) && value.equals(false))
                            ecoreElement.setValue(stereotype,tagTypeName,true);
                    }catch(IllegalArgumentException e){
                        XMILogs xmilogs = XMILogs.getInstance();
                        String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                taggedValue.getDefinition().getName(), 
                                element.getName());
        
                        xmilogs.writelnInLog(message);
        
                        GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                    }
        
                }else {
                    //Cas String
        
                    if ((tagType != null) && (tagType.getParamNumber().equals("1"))){
        
                        if ((taggedValue.getActual().size() >= 1)
                                && (taggedValue.getActual().get(0).getValue() != null)){
                            try{
                                ecoreElement.setValue(stereotype, tagTypeName, taggedValue.getActual().get(0).getValue());
                            }catch(IllegalArgumentException e){
                                XMILogs xmilogs = XMILogs.getInstance();
                                String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                        taggedValue.getDefinition().getName(), 
                                        element.getName());
        
                                xmilogs.writelnInLog(message);
        
                                GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                            }
                        }
                        
                    }else{
                        //Cas du tableau de string
        
                        EDataTypeUniqueEList<String> value =  (EDataTypeUniqueEList<String>) ecoreElement.getValue(stereotype, tagTypeName);
        
                        for (ITagParameter tagParameter : taggedValue.getActual() ){
        
        
                            try{
                                if (tagParameter.getValue() != null)
                                    value.add(tagParameter.getValue());
                            }catch(IllegalArgumentException e){
                                XMILogs xmilogs = XMILogs.getInstance();
                                String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                        taggedValue.getDefinition().getName(), 
                                        element.getName());
        
                                xmilogs.writelnInLog(message);
        
                                GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                            }
                        }
        
        
                        if (taggedValue.getQualifier() != null){
                            name = getNextFreeQualifier(tagTypeName, stereotype,
                                    ecoreElement, currentString);
        
        
                            try{
                                ecoreElement.setValue(stereotype, name, taggedValue.getQualifier().getValue());
                            }catch(IllegalArgumentException e){
                                XMILogs xmilogs = XMILogs.getInstance();
                                String message = XMI.I18N.getMessage("logFile.warning.unexportedTaggedValue", 
                                        taggedValue.getDefinition().getName(), 
                                        element.getName());
        
                                xmilogs.writelnInLog(message);
        
                                GenerationProperties.getInstance().getReportModel().addWarning(message, element);
                            }
        
                        }
                        currentString++;
                    }
                }
            }
        }
    }

    @objid ("6e99e4d5-1a76-11e1-8103-0027103f347d")
    static List<ITaggedValue> sortByType(final IModelElement element) {
        ObList<ITaggedValue> listTag = element.getTag();
        List<ITaggedValue> listTagSorted = new ArrayList<ITaggedValue>();
        
        for (ITaggedValue current : listTag){
            if (!(listTagSorted.contains(current))){
                listTagSorted.add(current);
                for (ITaggedValue other : listTag){
                    if ((!(listTagSorted.contains(other))) 
                            && (current.getDefinition().equals(other.getDefinition()) )){
                        listTagSorted.add(other);
                    }
                }
            }
        }
        return listTagSorted;
    }

    @objid ("6e9a0be9-1a76-11e1-8103-0027103f347d")
    static int getNextFreeValue(final String name, final Stereotype stereotype, final Element ecoreElement, final Type type, final int i) {
        int currenti = i;
        
        PrimitiveType booleanType = PrimitiveTypeMapper.getBoolean();
        
        String currentName = null;
        
        if (type.equals(booleanType))
            currentName = name;
        else
            currentName = name + "_" + String.valueOf(currenti);
        
        boolean find = false;
        
        while (!find) {
            Object value = ecoreElement.getValue(stereotype, currentName);
        
            if (value != null) {
                if ((type.equals(booleanType)) && (value.equals(false)))
                    return currenti;
                else
                    currentName = name + "_" + String.valueOf(++currenti);
            }else {
                if (!(type.equals(booleanType)))
                    return currenti;
                else
                    currentName = name + "_" + String.valueOf(++currenti);
        
            }
        }
        return currenti;
    }

    @objid ("6e9a5a0c-1a76-11e1-8103-0027103f347d")
    static Stereotype returnAssociatedStereotype(final Stereotype stereotype, final String currentName) {
        PrimitiveType stringType = PrimitiveTypeMapper.getString();
        
        Property propertyTemp = stereotype.getOwnedAttribute(currentName, stringType); 
        
        if (propertyTemp != null)
            return stereotype;     
        
        for (Object parent : stereotype.getSuperClasses()){
            Stereotype stereotypeTemp = (Stereotype) parent;
            propertyTemp = stereotypeTemp.getOwnedAttribute(currentName, stringType); 
            if (propertyTemp == null ) 
                stereotypeTemp = returnAssociatedStereotype(stereotypeTemp, currentName);
            else
                return stereotypeTemp;
        }
        return null;
    }

    @objid ("6e9aa82c-1a76-11e1-8103-0027103f347d")
    static String getNextFreeQualifier(final String name, final Stereotype stereotype, final Element ecoreElement, final int current) {
        PrimitiveType stringType = PrimitiveTypeMapper.getString();
        
        int i = 0;
        
        String currentName = name + "_" + String.valueOf(i);
        
        Stereotype propertyOwner = returnAssociatedStereotype(stereotype, currentName);
        
        boolean find = false;
        
        while (!find) {
            Object Value = ecoreElement.getValue(stereotype, currentName);
        
            if  (!(Value != null)){
                Property property = propertyOwner.getOwnedAttribute(currentName, stringType);
        
                if ( (Integer.valueOf(ObjingEAnnotation.getTaggedValue(property)) == current)
                        && (!ObjingEAnnotation.isTagParameter(property))){
        
                    return currentName;
                }
            }
            currentName = name + "_" + String.valueOf(++i);
        }
        return currentName;
    }

}
