/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

@objid ("1ddcca9a-4583-11e0-b54c-00137279a832")
public class TimeDisplayer {
    @objid ("1ddcca9b-4583-11e0-b54c-00137279a832")
    private String defaultFormat = XMI.I18N
			.getString("mdacProperties.timeDisplayer.defaultFormat");

    @objid ("1ddcca9c-4583-11e0-b54c-00137279a832")
    private static TimeDisplayer INSTANCE;

    @objid ("1ddcca9d-4583-11e0-b54c-00137279a832")
    private SimpleDateFormat dateFormat;

    @objid ("1ddcca9e-4583-11e0-b54c-00137279a832")
    private Date beginning = null;


    @objid ("1ddcca9f-4583-11e0-b54c-00137279a832")
    private TimeDisplayer() {
        boolean error = false;
        try {
            this.dateFormat = new SimpleDateFormat(this.defaultFormat);
        } catch (NullPointerException nullE) {
            error = true;
            Modelio.err
                    .println(XMI.I18N
                            .getString("warning.exception.timeDisplayer.NullPointerException"));
        } catch (IllegalArgumentException illegalE) {
            error = true;
            Modelio.err.println(XMI.I18N.getMessage(
                    "warning.exception.timeDisplayer.IllegalArgumentException",
                    this.defaultFormat));
        }
                
        if (error)
            this.dateFormat.applyPattern("HH:mm:ss");
    }

    @objid ("1ddccaa1-4583-11e0-b54c-00137279a832")
    public static TimeDisplayer getInstance() {
        if (INSTANCE == null)
            INSTANCE = new TimeDisplayer();
        return INSTANCE;
    }

    @objid ("1ddccaa5-4583-11e0-b54c-00137279a832")
    public void initialize() {
        this.beginning = null;
    }

    @objid ("1ddccaa7-4583-11e0-b54c-00137279a832")
    public String getCurrentTime() {
        Date currentDate = new Date();
                
        if (this.beginning == null)
            this.beginning = currentDate;
        return this.dateFormat.format(currentDate);
    }

}
