/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLFactory;

@objid ("1de65409-4583-11e0-b54c-00137279a832")
public class XMIEAnnotation {
    @objid ("6ea53006-1a76-11e1-8103-0027103f347d")
    public static final String OBJING_NAME = "Objing";

    @objid ("6eab98fb-1a76-11e1-8103-0027103f347d")
    public static final String TRUE_VALUE = "true";

    @objid ("6eabc00d-1a76-11e1-8103-0027103f347d")
    public static final String FALSE_VALUE = "false";


    @objid ("1de6540a-4583-11e0-b54c-00137279a832")
    public static void setOrCreateEAnnotationContent(Element ecoreElt, String eaName, String fieldName, String fieldContent) {
        if ((ecoreElt != null) && (fieldContent != null) && (!(fieldContent.equals("")))) {
            // Gets or creates the EAnnotation:
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea == null)
                ea = ecoreElt.createEAnnotation(eaName);
                
            // Gets or creates the Property:
            Property prop = null;
            for (Object content : ea.getContents()) {
                if (content instanceof Property) {
                    Property property = (Property) content;
                    if (fieldName.equals(property.getName())) {
                        prop = property;
                        break;
                    }
                }
            }
            if (prop == null) {
                prop = UMLFactory.eINSTANCE.createProperty();
                prop.setName(fieldName);
            }
                
            if (fieldContent == null)
                fieldContent = "";
                
            // Sets the field's content and adds the Property to the
            // EAnnotation:
            prop.setDefault(fieldContent);
            ea.getContents().add(prop);
        }
    }

    @objid ("1de8b65c-4583-11e0-b54c-00137279a832")
    protected static String getEAnnotationContent(Element ecoreElt, String eaName, String fieldName) {
        String strContent = "";
        if (ecoreElt != null) {
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea != null) {
                for (Object content : ea.getContents()) {
                    if (content instanceof Property) {
                        Property prop = (Property) content;
                        if (fieldName.equals(prop.getName())) {                               
                            strContent = prop.getDefault();    
                            break;
                        }
                    }
                }
            }
        }
        
        if (strContent == null){
            return "";
        }
        return strContent;
    }

    @objid ("1de8b663-4583-11e0-b54c-00137279a832")
    protected static boolean hasEAnnotationContent(Element ecoreElt, String eaName, String fieldName) {
        if (ecoreElt != null) {
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea != null) {
                for (Object content : ea.getContents()) {
                    if (content instanceof Property)
                        if (fieldName.equals(((Property) content).getName()))
                            return true;
                }
            }
        }
        return false;
    }

    @objid ("1de8b66a-4583-11e0-b54c-00137279a832")
    protected static EAnnotation getEAnnotation(Element ecoreElt, String eaName) {
        if (ecoreElt != null)
            return ecoreElt.getEAnnotation(eaName);
        return null;
    }

    @objid ("1deb18b6-4583-11e0-b54c-00137279a832")
    protected static EAnnotation createEAnnotation(Element ecoreElt, String eaName) {
        EAnnotation ea = null;
        if (ecoreElt != null)
            ea = ecoreElt.createEAnnotation(eaName);
        return ea;
    }

    @objid ("1deb18bc-4583-11e0-b54c-00137279a832")
    protected static EAnnotation getOrCreateEAnnotation(Element ecoreElt, String eaName) {
        EAnnotation ea = null;
        if (ecoreElt != null) {
            // Gets or creates the EAnnotation:
            ea = ecoreElt.getEAnnotation(eaName);
            if (ea == null)
                ea = ecoreElt.createEAnnotation(eaName);
        }
        return ea;
    }

    @objid ("1deb18c2-4583-11e0-b54c-00137279a832")
    public static void createEAnnotation(Element ecoreElt, String eaName, String fieldName) {
        if (ecoreElt != null) {
            // Gets or creates the EAnnotation:
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea == null)
                ea = ecoreElt.createEAnnotation(eaName);
                
            // Gets or creates the Property:
            Property prop = null;
            for (Object content : ea.getContents()) {
                if (content instanceof Property) {
                    Property property = (Property) content;
                    if (fieldName.equals(property.getName())) {
                        prop = property;
                        break;
                    }
                }
            }
            if (prop == null) {
                prop = UMLFactory.eINSTANCE.createProperty();
                prop.setName(fieldName);
            }
                
            ea.getContents().add(prop);
        }
    }

    @objid ("1deb18c7-4583-11e0-b54c-00137279a832")
    public static void deleteEAnnotation(Element ecoreElt, String eaName, String fieldName) {
        if (ecoreElt != null) {
            // Gets or creates the EAnnotation:
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea != null){
                
                Property field = null;
                for (Object content : ea.getContents()) {
                    if (content instanceof Property) {
                        Property prop = (Property) content;
                        if (fieldName.equals(prop.getName())) {
                            field = prop;
                            break;
                        }
                    }
                }
                
                if (field != null){
                    ea.getContents().remove(field);
                    if (ea.getContents().size() == 0){
                        ecoreElt.getEAnnotations().remove(ea);
                    }
                }
                
            }
            
        }
    }

    @objid ("1defdd6c-4583-11e0-b54c-00137279a832")
    protected static String getAndDestroyEAnnotationContent(Element ecoreElt, String eaName, String fieldName) {
        String strContent = "";
        if (ecoreElt != null) {
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea != null) {
                for (Object content : ea.getContents()) {
                    if (content instanceof Property) {
                        Property prop = (Property) content;
                        if (fieldName.equals(prop.getName())) {
                            
                            strContent = prop.getDefault();    
                            ea.getContents().remove(prop);
                            prop.destroy();
                            break;
                        }
                    }
                }
            }
        }
        
        if (strContent == null){
            return "";
        }
        return strContent;
    }

    @objid ("6eb4730e-1a76-11e1-8103-0027103f347d")
    public static void addEAnnotationContent(final Element ecoreElt, final String eaName, final String fieldName, final String fieldContent) {
        if ((ecoreElt != null) && (fieldContent != null) && (!(fieldContent.equals("")))) {
            // Gets or creates the EAnnotation:
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea == null)
                ea = ecoreElt.createEAnnotation(eaName);
                
            // Gets or creates the Property:
            Property prop = null;
            for (Object content : ea.getContents()) {
                if (content instanceof Property) {
                    Property property = (Property) content;
                    if (fieldName.equals(property.getName()) && fieldContent.equals(property.getDefault())) {
                        prop = property;
                        break;
                    }
                }
            }
            
            if (prop == null) {
                prop = UMLFactory.eINSTANCE.createProperty();
                prop.setName(fieldName);
               
                // Sets the field's content and adds the Property to the
                // EAnnotation:
                prop.setDefault(fieldContent);
                ea.getContents().add(prop);
            }
                
        }
    }

    @objid ("6eb4c132-1a76-11e1-8103-0027103f347d")
    protected static List<String> getEAnnotationContents(final Element ecoreElt, final String eaName, final String fieldName) {
        List<String> strContent = new ArrayList<String>();
        if (ecoreElt != null) {
            EAnnotation ea = ecoreElt.getEAnnotation(eaName);
            if (ea != null) {
                for (Object content : ea.getContents()) {
                    if (content instanceof Property) {
                        Property prop = (Property) content;
                        if (fieldName.equals(prop.getName())) {                               
                            strContent.add(prop.getDefault());    
                        }
                    }
                }
            }
        }
        
        //        if (strContent.size() == 0)
        //            strContent.add("");
        return strContent;
    }

    @objid ("6eb50f56-1a76-11e1-8103-0027103f347d")
    public static boolean getBooleanContent(final Element ecoreElt, final String annotation) {
        if (hasEAnnotationContent(ecoreElt, OBJING_NAME, annotation)){
            return (!(FALSE_VALUE.equals(getEAnnotationContent(ecoreElt, OBJING_NAME, annotation))));
        }
        return false;
    }

}
