/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.api.VersionExport;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.resource.Resource.Diagnostic;
import org.eclipse.emf.ecore.xmi.UnresolvedReferenceException;

/**
 * This class provides all service dedicated to XMI file treatment.
 * @author ebrosse
 */
@objid ("1aa3dd03-4583-11e0-b54c-00137279a832")
public class XMIFileUtils {

    /**
     * This service copies a file content into another file.
     * @param source The source file
     * @param target The target file
     */
    @objid ("1aa63f57-4583-11e0-b54c-00137279a832")
    public static void copyFile(File source, File target) {
        FileChannel in = null; 
        FileChannel out = null; 
        
        if (!source.getAbsolutePath ().contentEquals (target.getAbsolutePath ())) {
            try {
                // Init
                in = new FileInputStream (source).getChannel ();
                out = new FileOutputStream (target).getChannel ();
        
                //Copy
                in.transferTo (0, in.size (), out);
            } catch (Exception e) {
                e.printStackTrace (Modelio.err); 
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (IOException e) {
                        e.printStackTrace (Modelio.err); 
                    }
                }
                if (out != null) {
                    try {
                        out.close ();
                    } catch (IOException e) {
                        e.printStackTrace (Modelio.err); 
                    }
                }
            }
        }
    }

    /**
     * This method converts an EMF UML file into the desired UML OMG format
     * @param filePath The file location
     */
    @objid ("1aab040e-4583-11e0-b54c-00137279a832")
    public static void changeToUML(final String filePath) {
        File file = new File(filePath);
        
        List<String> oldPatterns = new LinkedList<String>();
        List<String> newPatterns = new LinkedList<String>();
        
        
        File xslExportFile = null;
        
        VersionExport versionExport = GenerationProperties.getInstance().getExportVersion();
        
        if (versionExport.equals(VersionExport.UML211)){
        
        
            oldPatterns.add("href[\\s]?=[\\s]?\"pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml");
            oldPatterns.add("xmlns:uml[\\s]?=[\\s]?\"http://www.eclipse.org/uml2/3.0.0/UML");
            oldPatterns.add("xsi:schemaLocation[\\s]?=[\\s]?\"");
        
            newPatterns.add("href=\"http://schema.omg.org/spec/UML/2.1.1/uml.xml" );
            newPatterns.add("xmlns:uml=\"http://schema.omg.org/spec/UML/2.1.1" );
            newPatterns.add("xsi:schemaLocation=\"http://www.eclipse.org/uml2/3.0.0/UML http://schema.omg.org/spec/UML/2.1.1 ");
        
        
            xslExportFile = ResourceLoader.getInstance().getResource("xslt"+ java.io.File.separator + "export211.xsl");
        
        }else if (versionExport.equals(VersionExport.UML22)){
        
            oldPatterns.add("href[\\s]?=[\\s]?\"pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml");
            oldPatterns.add("xmlns:uml[\\s]?=[\\s]?\"http://www.eclipse.org/uml2/3.0.0/UML");
            oldPatterns.add("xsi:schemaLocation[\\s]?=[\\s]?\"");
        
            newPatterns.add("href=\"http://schema.omg.org/spec/UML/2.2/uml.xml" );
            newPatterns.add("xmlns:uml=\"http://schema.omg.org/spec/UML/2.2" );
            newPatterns.add("xsi:schemaLocation=\"http://www.eclipse.org/uml2/3.0.0/UML http://schema.omg.org/spec/UML/2.2 ");
        
        
            xslExportFile = ResourceLoader.getInstance().getResource("xslt"+ java.io.File.separator + "export22.xsl");
        
        }else if (versionExport.equals(VersionExport.UML23)){
        
            oldPatterns.add("href[\\s]?=[\\s]?\"pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml");
            oldPatterns.add("xmlns:uml[\\s]?=[\\s]?\"http://www.eclipse.org/uml2/3.0.0/UML");
            oldPatterns.add("xsi:schemaLocation[\\s]?=[\\s]?\"");
        
            newPatterns.add("href=\"http://schema.omg.org/spec/UML/2.3/uml.xml" );
            newPatterns.add("xmlns:uml=\"http://www.omg.org/spec/UML/20090901" );
            newPatterns.add("xsi:schemaLocation=\"http://www.eclipse.org/uml2/3.0.0/UML http://schema.omg.org/spec/UML/20090901 ");
        
            xslExportFile = ResourceLoader.getInstance().getResource("xslt" + java.io.File.separator + "export23.xsl");
        
        }else if (versionExport.equals(VersionExport.UML241)){
        
            oldPatterns.add("href[\\s]?=[\\s]?\"pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml");
            oldPatterns.add("xmlns:uml[\\s]?=[\\s]?\"http://www.eclipse.org/uml2/3.0.0/UML");
            oldPatterns.add("xsi:schemaLocation[\\s]?=[\\s]?\"");
        
            newPatterns.add("href=\"http://www.omg.org/spec/UML/20100901/PrimitiveTypes.xmi#" );
            newPatterns.add("xmlns:uml=\"http://www.omg.org/spec/UML/20100901" );
            newPatterns.add("xsi:schemaLocation=\"http://www.eclipse.org/uml2/3.0.0/UML http://schema.omg.org/spec/UML/20100901 ");
        
            xslExportFile = ResourceLoader.getInstance().getResource("xslt" + java.io.File.separator + "export241.xsl");
        
        }
        
        //        if (GenerationProperties.getInstance().isSysMLApplied()){
        //
        //            oldPatterns.add("xmlns:sysml=\"[^\"]*\"");
        //            newPatterns.add("xmlns:sysml=\"http://www.omg.org/spec/SysML/20100301/SysML-profile\"");
        //
        //            oldPatterns.add("http:///schemas/sysml/[^\\s]*\\s");
        //            newPatterns.add("");
        //
        //            oldPatterns.add("href=\"sysml.profile.[^\\s]*#[^\\s]*\"");
        //            newPatterns.add("href=\"http://www.omg.org/spec/SysML/20100301/SysML-profile.uml#_0\"");
        //
        //            oldPatterns.add("[^\\s]*.sysml.profile.[^\\s]*\\s");
        //            newPatterns.add("");
        //
        //            oldPatterns.add("[^\\s]*.sysml.profile.[^\\s]*\"");
        //            newPatterns.add("\"");
        //
        //        }
        
        File tempFolder = ResourceLoader.getInstance().getXMITempFolder();
        
        if (!tempFolder.exists()){
            tempFolder.mkdirs();
            tempFolder.mkdir();
        }
        
        String tempPath = tempFolder.getAbsolutePath() + java.io.File.separator  + file.getName() + ".temp";
        File tempFile = new File(tempPath);
        if (tempFile.exists()){
            tempFile.delete();
        }
        
        applyXSLT(file.getAbsolutePath(), tempPath, xslExportFile); 
        
        if (file.exists()){
            file.delete();
        }
        
        replace(tempPath, file.getAbsolutePath(), oldPatterns, newPatterns);
        
        if (tempFile.exists()){
            tempFile.delete();
        }
        
        System.gc();
    }

    /**
     * remove wrong references listed in the listErrors
     * @param filePath The location of the file
     * @param listErrors The list of unavailable references
     */
    @objid ("1aad6661-4583-11e0-b54c-00137279a832")
    public static void removeReferences(final String filePath, EList<Diagnostic> listErrors) {
        String line = "";
        int errorsSize = listErrors.size();
        int currentError = 0;
        Diagnostic current = listErrors.get(currentError);
        
        while((!(current instanceof UnresolvedReferenceException))&& (currentError < errorsSize -1)) {
            currentError++;
            current = listErrors.get(currentError);
        }
        
        if  ((currentError < errorsSize) && (current instanceof UnresolvedReferenceException)){
            String unresolvedRef = ((UnresolvedReferenceException) current).getReference();
            int errorPos = ((UnresolvedReferenceException) current).getLine();
        
            File file = new File(filePath);
        
            try {
                File tempFile = new File(filePath + ".temp");
                tempFile.createNewFile();
                XMIFileUtils.copyFile(file, tempFile);
        
                file.createNewFile();
        
                FileWriter output = new FileWriter(filePath);
                FileInputStream fis = new FileInputStream(tempFile);
                BufferedReader reader = new BufferedReader ( new InputStreamReader(fis));
                int currentLine = 0;
                while((line = reader.readLine()) != null) {
                    currentLine++;
        
                    while(currentLine == errorPos){
                        line = line.replaceAll(unresolvedRef, "");
                        if  (currentError < errorsSize - 1 ){
                            currentError++;
                            current = listErrors.get(currentError);
                            while((!(current instanceof UnresolvedReferenceException))&& (currentError < errorsSize -1 )) {
                                current = listErrors.get(++currentError);
                            }
        
                            if  ((currentError < errorsSize ) && (current instanceof UnresolvedReferenceException)){
                                unresolvedRef = ((UnresolvedReferenceException) current).getReference();
                                errorPos = ((UnresolvedReferenceException) current).getLine();
                            }else{
                                errorPos = 0;
                            }
                        }else{
                            errorPos = 0;
                        }
                    }
        
                    output.write(line +"\n");
                }
        
                reader.close();
                output.flush();
                output.close();
                tempFile.delete();
            }catch (IOException e) {
                e.printStackTrace(Modelio.err);
            }
        }
    }

    /**
     * Apply the XSL Transformation (located in XSLTFile file) to the file located in oldFilePath.
     * The result is stored in newFilePath location.
     * @param oldFilePath The file location on which the XSL will be applied
     * @param newFilePath The result location
     * @param XSLTFile The file containing the XSL Transformation
     * @return return true is successfully processed
     */
    @objid ("1aafc8bb-4583-11e0-b54c-00137279a832")
    public static boolean applyXSLT(final String oldFilePath, String newFilePath, final File XSLTFile) {
        File oldFile = new File(oldFilePath);
        
        if (oldFile.exists() && oldFile.canRead()){
        
            try {
        
        
                TransformerFactory tFactory = TransformerFactory.newInstance();
        
                Transformer transformer = tFactory.newTransformer
                        (new javax.xml.transform.stream.StreamSource(XSLTFile));
        
                transformer.transform(new javax.xml.transform.stream.StreamSource
                                      (oldFilePath), new javax.xml.transform.stream.StreamResult( new FileOutputStream(newFilePath)));
        
                transformer = null;
                return true;
        
            }catch (TransformerException e) {
                XMILogs.getInstance().writelnInLog(XMI.I18N.getString("fileChooser.dialog.wrongEcoreFormat.description"));
                return false;
            }catch (Exception e) {
                e.printStackTrace(Modelio.err );
                return false;
            }
        }
        return false;
    }

    /**
     * replace all oldPatterns[i] by newPatterns[i] in the file located in filePath.
     * @param filePath The file location
     * @param oldPatterns The strings to replace
     * @param newPatterns The new strings
     * @return return true is successfully processed
     */
    @objid ("6d436b0a-1a76-11e1-8103-0027103f347d")
    public static boolean replace(final String filePath, final List<String> oldPatterns, final List<String> newPatterns) {
        File file = new File(filePath);
        
        if (file.exists() && (oldPatterns.size() == newPatterns.size())){
        
            int i = 0;
            String filePathTemp = filePath + "_" + String.valueOf(i);
            while((new File(filePathTemp)).exists()){
                i++;
                filePathTemp = filePath + "_" + String.valueOf(i);
            }
        
            replace(filePath, filePathTemp, oldPatterns, newPatterns);
        
            File fileTemp = new File(filePathTemp);
            file.delete();
        
            fileTemp.renameTo(file);
            return true;
        }
        return false;
    }

    /**
     * This service return the list of applied diagrams
     * @param source
     * @param target
     */
    @objid ("6d43b92f-1a76-11e1-8103-0027103f347d")
    private static List<String> getAppliedProfiles(final String filePath) {
        List<String> result = new ArrayList<String>();
        String line = "";
        
        try {
            FileInputStream fis = new FileInputStream(filePath);
            BufferedReader reader = new BufferedReader ( new InputStreamReader(fis));
        
        
            while((line = reader.readLine()) != null) {
        
                if (line.contains("appliedProfile") && line.contains(" href=\"") 
                        && line.contains("#") && (!line.contains("pathmap:")) ){               
                    result.add(line.split(" href=\"")[1].split("#")[0]);
                }
        
            }
        
            reader.close();
        
        }catch (IOException e) {
            e.printStackTrace(Modelio.err);
        }
        return result;
    }

    /**
     * Test if the file contains at least one of the patterns.
     * @param file The file to test
     * @param patterns The list of patterns
     * @return
     * true if the file contains at least one of the patterns.
     */
    @objid ("6d440752-1a76-11e1-8103-0027103f347d")
    public static boolean containsPatterns(final File file, final List<String> patterns) {
        String line = "";
        int cmpt = 0;
        
        try {
            FileInputStream fis = new FileInputStream(file.getAbsolutePath());
            BufferedReader reader = new BufferedReader ( new InputStreamReader(fis));
        
            List<Pattern> listPatterns = new ArrayList<Pattern>();
        
            for (String oldPattern : patterns){
                listPatterns.add(Pattern.compile(oldPattern));
            }
        
            while(((line = reader.readLine()) != null) &&  (cmpt < 30)){
        
                for (Pattern pattern : listPatterns){
                    if (pattern.matcher(line).find())
                        return true;
                }
                cmpt++;
        
            }
        
            reader.close();
        
        }catch (IOException e) {
            e.printStackTrace(Modelio.err);
        }
        return false;
    }

    /**
     * Replace all the oldPatterns[i] by the newPatterns[i] from the oldFile to the newFile
     * @param oldFilePath The location of the oldFile
     * @param newFilePath The location of the newFile i.e. the result file
     * @param oldPatterns The list of patterns to be replaced
     * @param newPatterns The list of the new patterns
     */
    @objid ("6d445576-1a76-11e1-8103-0027103f347d")
    public static void replace(final String oldFilePath, final String newFilePath, final List<String> oldPatterns, final List<String> newPatterns) {
        String line = "";
        try {
            FileWriter output = new FileWriter(newFilePath);
            FileInputStream fis = new FileInputStream(oldFilePath);
            BufferedReader reader = new BufferedReader ( new InputStreamReader(fis));
        
            List<Pattern> listPatterns = new ArrayList<Pattern>();
        
            for (String oldPattern : oldPatterns){
                listPatterns.add(Pattern.compile(oldPattern));
            }
        
            while((line = reader.readLine()) != null) {
        
                for (Pattern pattern : listPatterns){
                    line = pattern.matcher(line).replaceAll(newPatterns.get(listPatterns.indexOf(pattern)));
                }
        
                output.write(line + "\n");
            }
        
            reader.close();
        
            output.flush();
            output.close();
        }catch (FileNotFoundException e) {
            XMILogs.getInstance().writelnInLog(e.getLocalizedMessage());
        }catch (IOException e) {
            e.printStackTrace(Modelio.err);
        }
    }

    /**
     * This service returns the list of applied profiles present
     * @param filePath The location of the file
     * @return
     * The list of applied profile
     */
    @objid ("6d44a39e-1a76-11e1-8103-0027103f347d")
    public static List<String> getAllAppliedProfiles(final String filePath) {
        List<String> result = new ArrayList<String>();
        List<String> temp = new ArrayList<String>();
        String directory = new File(filePath).getParent();
        
        List<String> appliedProfilescurrent = getAppliedProfiles(filePath);
        
        while (result.size() != appliedProfilescurrent.size()){
            result.clear();
            temp.clear();
            result.addAll(appliedProfilescurrent);
            temp.addAll(appliedProfilescurrent);
            for (String appliedProfile : result){
                temp.addAll(getAppliedProfiles(directory + java.io.File.separator + appliedProfile));
                appliedProfilescurrent = new ArrayList<String>(new HashSet<String>(temp));
            }
        }
        return result;
    }

}
