/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import com.modeliosoft.modelio.api.modelio.Modelio;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

/**
 * @author ebrosse
 */
@objid ("1defdd73-4583-11e0-b54c-00137279a832")
public class XMILogs {
    @objid ("1defdd74-4583-11e0-b54c-00137279a832")
    private final String LINE_SEPARATOR = System.getProperty("line.separator");

    @objid ("1df23fc5-4583-11e0-b54c-00137279a832")
    private File fileToSave = null;

    @objid ("1df23fc6-4583-11e0-b54c-00137279a832")
    private DateFormat dateFormat = DateFormat.getDateTimeInstance(
                                                                   DateFormat.SHORT, DateFormat.MEDIUM, Locale.getDefault());

    @objid ("1df23fc7-4583-11e0-b54c-00137279a832")
    private StringBuffer log = null;

    @objid ("1df23fc8-4583-11e0-b54c-00137279a832")
    private static XMILogs INSTANCE = null;

    @objid ("1df23fc9-4583-11e0-b54c-00137279a832")
    private boolean empty = true;

    @objid ("1df23fca-4583-11e0-b54c-00137279a832")
    private String logFile = "";

    @objid ("9e0f6333-1a76-11e1-8103-0027103f347d")
    private List<String> elements = new ArrayList<String>();


    @objid ("1df23fcf-4583-11e0-b54c-00137279a832")
    private XMILogs() {
        this.logFile = Modelio.getInstance().getContext().getProjectSpacePath().getAbsolutePath() 
        +  java.io.File.separator + "XMI";
        fileToSave();
        this.log = null;
    }

    /**
     * @return the instance
     */
    @objid ("1df23fd1-4583-11e0-b54c-00137279a832")
    public static XMILogs getInstance() {
        if (INSTANCE == null)
            INSTANCE = new XMILogs();
        return INSTANCE;
    }

    @objid ("1df23fd5-4583-11e0-b54c-00137279a832")
    private void fileToSave() {
        this.fileToSave = new File(this.logFile + ".log");
    }

    /**
     * This method saves the log file
     */
    @objid ("1df23fd7-4583-11e0-b54c-00137279a832")
    public void saveLog() {
        if (this.log != null) {
            this.empty = false;
            String logFilepath = this.fileToSave.getAbsolutePath();
            Modelio.err.println(XMI.I18N.getMessage("warning.log.check",
                                                   logFilepath));
            this.log = null;
        }
    }

    /**
     * @param inputMsg : the message wrote in the log file
     */
    @objid ("1df4a21f-4583-11e0-b54c-00137279a832")
    public void writelnInLog(final String inputMsg) {
        if (inputMsg != null){
            this.empty = false;
            if ( this.log == null)
                this.log = new StringBuffer();
        
            if (! this.elements.contains(inputMsg)) {
                this.elements.add(inputMsg);
                this.log.append(inputMsg).append( this.LINE_SEPARATOR);
                try {
                    if (! this.fileToSave.exists()) {
                        if (( this.fileToSave.getParentFile() == null) 
                                ||(! this.fileToSave.getParentFile().exists())) {
                            this.fileToSave.mkdirs();
                            try {
                                Thread.sleep(100);
                            } catch (InterruptedException e) {
                                e.printStackTrace();
                            }
                        }
                        this.fileToSave.createNewFile();
                    }
                    FileWriter f = new FileWriter( this.fileToSave, true);
                    BufferedWriter bf = new BufferedWriter(f);
                    bf.write("[" +  this.dateFormat.format(Calendar.getInstance().getTime()) + "]: " + inputMsg);
                    bf.newLine();
                    bf.close();
        
                } catch (IOException e) {
                    e.printStackTrace(Modelio.err);
                }
            }
        }
    }

    /**
     * @return the path of the log file
     */
    @objid ("1df4a222-4583-11e0-b54c-00137279a832")
    public String getLogFile() {
        return  this.logFile;
    }

    /**
     * @param logFile : path of the log file
     */
    @objid ("1df4a226-4583-11e0-b54c-00137279a832")
    public void setLogFile(final String logFile) {
        this.elements = new ArrayList<String>();
        this.logFile = logFile;
        fileToSave();
    }

    /**
     * @return the elements
     */
    @objid ("1df7047a-4583-11e0-b54c-00137279a832")
    public List<String> elements() {
        return this.elements;
    }

    /**
     * @return true if a log exist
     */
    @objid ("1df70480-4583-11e0-b54c-00137279a832")
    public boolean haveLog() {
        return ( this.log != null);
    }

    /**
     * @return true if the log is empty
     */
    @objid ("1df70484-4583-11e0-b54c-00137279a832")
    public boolean isEmpty() {
        return  this.empty;
    }

    /**
     * @param empty : set if log is empty or not
     */
    @objid ("1df966d2-4583-11e0-b54c-00137279a832")
    public void setEmpty(final boolean empty) {
        this.empty = empty;
    }

}
