/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

@objid ("1ad129ab-4583-11e0-b54c-00137279a832")
public class AttachInputPinToOwnerVisitorGen {
    @objid ("1ad129ac-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1ad129ad-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1ad129ae-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;

    @objid ("9ccce79f-1a76-11e1-8103-0027103f347d")
    private List<String> inputPinOwnerList = new ArrayList<String>();


    @objid ("1ad129b2-4583-11e0-b54c-00137279a832")
    public AttachInputPinToOwnerVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
        
        this.inputPinOwnerList.add("Action");
        this.inputPinOwnerList.add("AddStructuralFeatureValueAction");
        this.inputPinOwnerList.add("AddVariableValueAction");
        this.inputPinOwnerList.add("CallOperationAction");
        this.inputPinOwnerList.add("ClearAssociationAction");
        this.inputPinOwnerList.add("DestroyObjectAction");
        this.inputPinOwnerList.add("InvocationAction");
        this.inputPinOwnerList.add("LinkAction");
        this.inputPinOwnerList.add("LoopNode");
        this.inputPinOwnerList.add("OpaqueAction");
        this.inputPinOwnerList.add("RaiseExceptionAction");
        this.inputPinOwnerList.add("ReadIsClassifiedObjectAction");
        this.inputPinOwnerList.add("ReadLinkObjectEndAction");
        this.inputPinOwnerList.add("ReadLinkObjectEndQualifierAction");
        this.inputPinOwnerList.add("ReclassifyObjectAction");
        this.inputPinOwnerList.add("ReduceAction");
        this.inputPinOwnerList.add("RemoveStructuralFeatureValueAction");
        this.inputPinOwnerList.add("RemoveVariableValueAction");
        this.inputPinOwnerList.add("ReplyAction");
        this.inputPinOwnerList.add("SendObjectAction");
        this.inputPinOwnerList.add("SendSignalAction");
        this.inputPinOwnerList.add("StartClassifierBehaviorAction");
        this.inputPinOwnerList.add("StructuralFeatureAction");
        this.inputPinOwnerList.add("StructuredActivityNode");
        this.inputPinOwnerList.add("TestIdentityAction");
        this.inputPinOwnerList.add("UnmarshallAction");
        this.inputPinOwnerList.add("WriteStructuralFeatureAction");
        this.inputPinOwnerList.add("WriteVariableAction");
    }

    @objid ("1ad38c00-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1ad38c02-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1ad38c07-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1ad38c0c-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1ad5ee59-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1ad5ee5c-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.util;\r\n");
        
        this.printer.println("import org.eclipse.uml2.uml.*;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;");
        this.printer.println();
    }

    @objid ("1ad5ee5e-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to attach an InputPin to its Owner. " + "Call the \"attachInputPin()\" method to attach the InputPin to its owner." + "\r\n*/";
        generateJavadoc(docText);
        
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassCode();
        generateClassFooter();
    }

    @objid ("1ad5ee60-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class AttachInputPinToOwnerVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1ad5ee62-4583-11e0-b54c-00137279a832")
    private void generateClassFooter() {
        this.printer.println("}");
    }

    @objid ("1ad5ee64-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Visitor that will realize a switch on the owner of the InputPin to attach them." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private InputPinLinker linker;\r\n");
        
        docText = "/**\r\n* " + "Stores the InputPin to attach to its parent." + "\r\n*/";
        this.printer.println("private InputPin inputPin;\r\n");
    }

    @objid ("1ad850b2-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public AttachInputPinToOwnerVisitorGenerated()");
        this.printer.println("{");
        this.printer.println("linker = new InputPinLinker();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1ad850b4-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1ad850b6-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        String docText = "/**\r\n* " + "Attaches the given InputPin to its owner.\r\n" + "@param inputPin The InputPin to attach.\r\n" + "@param owner The owner of the InputPin to attach.\r\n" + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public void attachInputPin(");
        this.printer.println("InputPin inputPin, Element owner)");
        this.printer.println("{");
        
        this.printer.println("this.inputPin = inputPin;");
        this.printer.println("linker.accept(owner);");
        
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1ad850b8-4583-11e0-b54c-00137279a832")
    private void generateInnerClassCode() {
        String docText = "/**\r\n* " + "This class extends UMLSwitch and realizes a switch.\r\n" + "The class is inner and private, encapsulated to mask the \"visit\" methods to the user of the AttachInputPinToOwnerVisitor class.\r\n" + "The user will not see this class.\r\n" + "Some visit methods return 42 to avoid calling the super (if return null => super)." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerClassHeader();
        
        generateInnerMemberAttributes();
        generateInnerConstructor();
        
        docText = "/**\r\n* " + "Launches the switch." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerPublicMethods();
        
        generateInnerRedefinedMethods();
        
        generateInnerClassFooter();
    }

    @objid ("1ad850ba-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        this.printer.println("private class InputPinLinker");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1ad850bc-4583-11e0-b54c-00137279a832")
    private void generateInnerClassFooter() {
        this.printer.println("}");
    }

    @objid ("1adab30c-4583-11e0-b54c-00137279a832")
    private void generateInnerMemberAttributes() {
        // No member attributes
    }

    @objid ("1adab30e-4583-11e0-b54c-00137279a832")
    private void generateInnerConstructor() {
        // No constructor
    }

    @objid ("1adab310-4583-11e0-b54c-00137279a832")
    private void generateInnerPublicMethods() {
        this.printer.println("public void accept(");
        this.printer.println("Element ecoreElt)");
        this.printer.println("{");
        this.printer.println("doSwitch(ecoreElt);");
        this.printer.println("}");
    }

    @objid ("1adab312-4583-11e0-b54c-00137279a832")
    private void generateInnerRedefinedMethods() {
        int i = 0;
        for (String metaclassName : this.inputPinOwnerList) {
            LOG.info(XMI.PLUGIN_ID, metaclassName);
            i++;
            visitMetaclass(metaclassName);
        }
        LOG.info(XMI.PLUGIN_ID, "" + i);
    }

    @objid ("1adab314-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(String metaclassName) {
        generateMethodHeader(metaclassName);
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1add1567-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader(String metaclassName) {
        String docText = "/**\r\n* " + "Attach the InputPin to its owner \"" + metaclassName + "\" element." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.print("public Object case");
        this.printer.print(metaclassName);
        this.printer.println("(");
        
        this.printer.print(metaclassName);
        this.printer.println(" owner)");
        this.printer.println("{");
    }

    @objid ("1add156a-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1add156c-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return 42;");
    }

    @objid ("1add156e-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        
        AttachInputPinToOwnerVisitorGen generator = new AttachInputPinToOwnerVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/util/AttachInputPinToOwnerVisitorGenerated.java");
        
        generator.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

}
