/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

@objid ("1adf77c0-4583-11e0-b54c-00137279a832")
public class AttachOutputPinToOwnerVisitorGen {
    @objid ("1adf77c1-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1adf77c2-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1adf77c3-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;

    @objid ("9cce6e55-1a76-11e1-8103-0027103f347d")
    private List<String> outputPinOwnerList = new ArrayList<String>();


    @objid ("1adf77c7-4583-11e0-b54c-00137279a832")
    public AttachOutputPinToOwnerVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
        
        this.outputPinOwnerList.add("AcceptCallAction");
        this.outputPinOwnerList.add("AcceptEventAction");
        this.outputPinOwnerList.add("Action");
        this.outputPinOwnerList.add("CallAction");
        this.outputPinOwnerList.add("ConditionalNode");
        this.outputPinOwnerList.add("CreateLinkObjectAction");
        this.outputPinOwnerList.add("CreateObjectAction");
        this.outputPinOwnerList.add("LoopNode");
        this.outputPinOwnerList.add("OpaqueAction");
        this.outputPinOwnerList.add("ReadExtentAction");
        this.outputPinOwnerList.add("ReadIsClassifiedObjectAction");
        this.outputPinOwnerList.add("ReadLinkAction");
        this.outputPinOwnerList.add("ReadLinkObjectEndAction");
        this.outputPinOwnerList.add("ReadLinkObjectEndQualifierAction");
        this.outputPinOwnerList.add("ReadSelfAction");
        this.outputPinOwnerList.add("ReadStructuralFeatureAction");
        this.outputPinOwnerList.add("ReadVariableAction");
        this.outputPinOwnerList.add("ReduceAction");
        this.outputPinOwnerList.add("StructuredActivityNode");
        this.outputPinOwnerList.add("TestIdentityAction");
        this.outputPinOwnerList.add("UnmarshallAction");
        this.outputPinOwnerList.add("ValueSpecificationAction");
    }

    @objid ("1adf77ca-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1adf77cc-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1ae1da1b-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1ae1da20-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1ae1da25-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1ae1da28-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.util;\r\n");
        
        this.printer.println("import org.eclipse.uml2.uml.*;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;");
        this.printer.println();
    }

    @objid ("1ae43c74-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to attach an OutputPin to its Owner. " + "Call the \"attachOutputPin()\" method to attach the OutputPin to its owner." + "\r\n*/";
        generateJavadoc(docText);
        
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassCode();
        generateClassFooter();
    }

    @objid ("1ae43c76-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class AttachOutputPinToOwnerVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1ae43c78-4583-11e0-b54c-00137279a832")
    private void generateClassFooter() {
        this.printer.println("}");
    }

    @objid ("1ae43c7a-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Visitor that will realize a switch on the owner of the OutputPin to attach them." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private OutputPinLinker linker;\r\n");
        
        docText = "/**\r\n* " + "Stores the OutputPin to attach to its parent." + "\r\n*/";
        this.printer.println("private OutputPin outputPin;\r\n");
    }

    @objid ("1ae43c7c-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public AttachOutputPinToOwnerVisitorGenerated()");
        this.printer.println("{");
        this.printer.println("linker = new OutputPinLinker();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1ae43c7e-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1ae43c80-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        String docText = "/**\r\n* " + "Attaches the given OutputPin to its owner.\r\n" + "@param outputPin The OutputPin to attach.\r\n" + "@param owner The owner of the OutputPin to attach.\r\n" + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public void attachOutputPin(");
        this.printer.println("OutputPin outputPin, Element owner)");
        this.printer.println("{");
        
        this.printer.println("this.outputPin = outputPin;");
        this.printer.println("linker.accept(owner);");
        
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1ae69ecf-4583-11e0-b54c-00137279a832")
    private void generateInnerClassCode() {
        String docText = "/**\r\n* " + "This class extends UMLSwitch and realizes a switch.\r\n" + "The class is inner and private, encapsulated to mask the \"visit\" methods to the user of the AttachOutputPinToOwnerVisitor class.\r\n" + "The user will not see this class.\r\n" + "Some visit methods return 42 to avoid calling the super (if return null => super)." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerClassHeader();
        
        generateInnerMemberAttributes();
        generateInnerConstructor();
        
        docText = "/**\r\n* " + "Launches the switch." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerPublicMethods();
        
        generateInnerRedefinedMethods();
        
        generateInnerClassFooter();
    }

    @objid ("1ae69ed1-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        this.printer.println("private class OutputPinLinker");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1ae69ed3-4583-11e0-b54c-00137279a832")
    private void generateInnerClassFooter() {
        this.printer.println("}");
    }

    @objid ("1ae69ed5-4583-11e0-b54c-00137279a832")
    private void generateInnerMemberAttributes() {
        // No member attributes
    }

    @objid ("1ae69ed7-4583-11e0-b54c-00137279a832")
    private void generateInnerConstructor() {
        // No constructor
    }

    @objid ("1ae69ed9-4583-11e0-b54c-00137279a832")
    private void generateInnerPublicMethods() {
        this.printer.println("public void accept(");
        this.printer.println("Element ecoreElt)");
        this.printer.println("{");
        this.printer.println("doSwitch(ecoreElt);");
        this.printer.println("}");
    }

    @objid ("1ae90128-4583-11e0-b54c-00137279a832")
    private void generateInnerRedefinedMethods() {
        int i = 0;
        for (String metaclassName : this.outputPinOwnerList) {
            LOG.info(XMI.PLUGIN_ID, metaclassName);
            i++;
            visitMetaclass(metaclassName);
        }
        LOG.info(XMI.PLUGIN_ID, "" + i);
    }

    @objid ("1ae9012a-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(String metaclassName) {
        generateMethodHeader(metaclassName);
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1ae9012d-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader(String metaclassName) {
        String docText = "/**\r\n* " + "Attach the OutputPin to its owner \"" + metaclassName + "\" element." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.print("public Object case");
        this.printer.print(metaclassName);
        this.printer.println("(");
        
        this.printer.print(metaclassName);
        this.printer.println(" owner)");
        this.printer.println("{");
    }

    @objid ("1ae90130-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1ae90132-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return 42;");
    }

    @objid ("1aeb6382-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        
        AttachOutputPinToOwnerVisitorGen generator = new AttachOutputPinToOwnerVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/util/AttachOutputPinToOwnerVisitorGenerated.java");
        
        generator.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

}
