/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IAssociationEnd;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.model.uml.statik.ObAggregationKindEnum;
import com.modeliosoft.modelio.api.utils.IObListFilter;
import com.modeliosoft.modelio.api.utils.NameFilter;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import com.modeliosoft.modelio.xmi.util.ObjingModelNavigation;

@objid ("1aeb6387-4583-11e0-b54c-00137279a832")
public class CompositionOwnerObjingVisitorGen {
    @objid ("1aeb6388-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1aeb6389-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1aeb638a-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;


    @objid ("1aeb638b-4583-11e0-b54c-00137279a832")
    public CompositionOwnerObjingVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
    }

    @objid ("1aeb638e-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1aedc5dc-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1aedc5e1-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1aedc5e6-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1aedc5eb-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1af02836-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.util;\r\n");
        
        this.printer.println("import java.util.HashMap;");
        this.printer.println("import com.softeam.modelio.joni.model.utils.DefaultMetamodelVisitor;");
        this.printer.println("import com.softeam.modelio.joni.model.*;");
    }

    @objid ("1af02838-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to get the composition owner of a given model element.\r\n" + "Call the \"getCompositionOwner()\" method to retrieve the Parent.\r\n" + "An element may have several different types of owners, defined by composition links at meta-level. " + "These relations are exclusive: only one of them returns the owner element." + "\r\n*/";
        
        generateJavadoc(docText);
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassCode();
        generateClassClosure();
    }

    @objid ("1af0283a-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class CompositionOwnerVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1af0283c-4583-11e0-b54c-00137279a832")
    private void generateClassClosure() {
        this.printer.println("}");
    }

    @objid ("1af0283e-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Represents the composition owner that will be returned." + "\r\n*/";
        
        generateJavadoc(docText);
        this.printer.println("private IElement theParent;\r\n");
        
        docText = "/**\r\n* " + "HashMap that relates an element (key) and its composition owner (value)." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private HashMap<IElement, IElement> ownershipMap;\r\n");
        
        docText = "/**\r\n* " + "Visitor that will navigate relations of composition to retrive the owner." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private CompositionOwnerSelector selector;\r\n");
        this.printer.println();
    }

    @objid ("1af02840-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public CompositionOwnerVisitorGenerated()");
        this.printer.println("{");
        this.printer.println("ownershipMap = new HashMap<IElement, IElement>();");
        this.printer.println("selector = new CompositionOwnerSelector();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1af02842-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1af28a91-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        String docText = "/**\r\n* " + "Returns the composition owner of the given element.\r\n" + "@param theChild The element from which we will get the owner.\r\n" + "@return the composition owner of the element given as parameter." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public IElement getCompositionOwner(");
        this.printer.println("IElement theChild)");
        this.printer.println("{");
        this.printer.println("theParent = ownershipMap.get(theChild);");
        this.printer.println("if (theParent == null) {");
        this.printer.println("selector.launchVisit(theChild);");
        this.printer.println("ownershipMap.put(theChild, theParent);");
        this.printer.println("}");
        this.printer.println("return theParent;");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1af28a93-4583-11e0-b54c-00137279a832")
    private void generateInnerClassCode() {
        String docText = "/**\r\n* " + "This class extends DefaultMetamodelVisitor and visits the composition links defined at meta-level.\r\n" + "The class is inner and private, encapsulated to mask the \"visit\" methods to the user of the CompositionOwnerVisitor class.\r\n" + "The user will not see this class." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerClassHeader();
        
        generateInnerMemberAttributes();
        generateInnerConstructor();
        
        docText = "/**\r\n* " + "Launches the visit of the composition relations." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerPublicMethods();
        
        generateInnerRedefinedMethods();
        
        generateInnerClassClosure();
    }

    @objid ("1af28a95-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        this.printer.println("private class CompositionOwnerSelector");
        this.printer.println("extends DefaultMetamodelVisitor");
        this.printer.println("{");
    }

    @objid ("1af28a97-4583-11e0-b54c-00137279a832")
    private void generateInnerClassClosure() {
        this.printer.println("}");
    }

    @objid ("1af28a99-4583-11e0-b54c-00137279a832")
    private void generateInnerMemberAttributes() {
        // No member attributes
    }

    @objid ("1af28a9b-4583-11e0-b54c-00137279a832")
    private void generateInnerConstructor() {
        // No constructor
    }

    @objid ("1af4ecea-4583-11e0-b54c-00137279a832")
    private void generateInnerPublicMethods() {
        this.printer.println("public void launchVisit(");
        this.printer.println("IElement theChild)");
        this.printer.println("{");
        this.printer.println("theChild.accept(this);");
        this.printer.println("}");
    }

    @objid ("1af4ecec-4583-11e0-b54c-00137279a832")
    private void generateInnerRedefinedMethods() {
        IPackage metamodelPkg = selectVisitEntryPoint();
        if (metamodelPkg != null) {
            for (IClass metaclass : metamodelPkg.getOwnedElement(IClass.class).select(new IObListFilter() {
                public boolean select(IElement element) {
                    if (element.getName().equals("Item") || element.getName().equals("Object") || element.getName().equals("PointOfView")) {
                        return false;
                    } else
                        return true;
                }
            })) {
                visitMetaclass(metaclass);
            }
        } else {
            LOG.info(XMI.PLUGIN_ID, "Metamodel RAMC not found.");
        }
    }

    @objid ("1af4ecee-4583-11e0-b54c-00137279a832")
    private IPackage selectVisitEntryPoint() {
        IPackage root = XMI.getInstance().getModelingSession().getModel().getProject().getModel();
        IPackage metamodelPackage = null;
        
        for (IModelTree ramc : root.getOwnedElement(IComponent.class).select(new NameFilter("metamodel"))) {
            for (IModelTree refMdl : ramc.getOwnedElement(IPackage.class).select(new NameFilter("Reference Metamodels"))) {
                for (IModelTree objingUML : refMdl.getOwnedElement(IPackage.class).select(new NameFilter("Modelio UML"))) {
                    for (IModelTree metamodelPkg : objingUML.getOwnedElement(IPackage.class).select(new NameFilter("Metamodel"))) {
                        metamodelPackage = (IPackage) metamodelPkg;
                    }
                }
            }
        }
        return metamodelPackage;
    }

    @objid ("1af4ecf2-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader(String metaclassName) {
        String docText = "/**\r\n* " + "Visits composition relations for \"" + metaclassName + "\" elements and its super classes." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.print("public Object visit");
        this.printer.print(metaclassName);
        this.printer.println("(");
        
        this.printer.print("I");
        this.printer.print(metaclassName);
        this.printer.println(" child)");
        this.printer.println("{");
    }

    @objid ("1af4ecf5-4583-11e0-b54c-00137279a832")
    private void generateMethodClosure() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1af74f44-4583-11e0-b54c-00137279a832")
    private void generateMethodTestResultIsNull() {
        this.printer.println("if (theParent == null)");
    }

    @objid ("1af74f46-4583-11e0-b54c-00137279a832")
    private void generateMethodSuperCall(String metaclassName) {
        generateMethodTestResultIsNull();
        this.printer.print("super.visit");
        this.printer.print(metaclassName);
        this.printer.println("(child);");
        this.printer.println();
    }

    @objid ("1af74f49-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return null;");
    }

    @objid ("1af74f4b-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        
        CompositionOwnerObjingVisitorGen generator = new CompositionOwnerObjingVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/util/CompositionOwnerVisitorGenerated.java");
        
        generator.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1af9b19e-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(IClass metaclass) {
        String metaclassName = getMetaclassName(metaclass);
        
        generateMethodHeader(metaclassName);
        
        generateMethodComposition(metaclass);
        generateMethodSuperCall(metaclassName);
        
        generateMethodReturnValue();
        generateMethodClosure();
    }

    @objid ("1af9b1a1-4583-11e0-b54c-00137279a832")
    private String getMetaclassName(IClass metaclass) {
        return metaclass.getName();
    }

    @objid ("1af9b1a6-4583-11e0-b54c-00137279a832")
    private void generateMethodComposition(IClass metaclass) {
        // for (IFeature part :
        // metaclass.getPart(IAssociationEnd.class).select(new
        // OppositeAssociationEndIsComposition()).select(new
        // EndNotConnectedToMetaclass_Project())) {
        for (IAssociationEnd assocEnd : metaclass.getPart(IAssociationEnd.class)) {
        
            List<IAssociationEnd> oppositeEnds = ObjingModelNavigation.getOppositeAssociationEnds(assocEnd);
        
            if (oppositeEnds.size() == 1) {
                IAssociationEnd oppositeEnd = oppositeEnds.get(0);
                if (oppositeEnd.getAggregation() == ObAggregationKindEnum.KIND_IS_COMPOSITION && !("Project".equals(oppositeEnd.getOwner().getName()))) {
        
                    generateMethodTestResultIsNull();
                    this.printer.print("theParent = child.get");
                    this.printer.print(assocEnd.getName());
                    this.printer.println("();");
                    this.printer.println();
                }
            }
        }
    }

}
