/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.internal.impl.ClassImpl;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1afc13ff-4583-11e0-b54c-00137279a832")
public class OwnedAssociationUml2VisitorGen {
    @objid ("1afc1400-4583-11e0-b54c-00137279a832")
    private File visitorFile = null;

    @objid ("1afc1401-4583-11e0-b54c-00137279a832")
    private BufferedWriter visitorBuffer = null;

    @objid ("1afc1402-4583-11e0-b54c-00137279a832")
    private PrintWriter visitorPrinter = null;

    @objid ("1afc1403-4583-11e0-b54c-00137279a832")
    private File behaviorFile = null;

    @objid ("1afc1404-4583-11e0-b54c-00137279a832")
    private BufferedWriter behaviorBuffer = null;

    @objid ("1afc1405-4583-11e0-b54c-00137279a832")
    private PrintWriter behaviorPrinter = null;


    @objid ("1afc1406-4583-11e0-b54c-00137279a832")
    public OwnedAssociationUml2VisitorGen() {
    }

    @objid ("1afe7652-4583-11e0-b54c-00137279a832")
    private void generateUml2Visitor() {
        Resource res = loadEcoreModel();
        if (res != null) {
            parseUml2Metamodel(res);
        }
    }

    @objid ("1afe7654-4583-11e0-b54c-00137279a832")
    private void compareTypes(String className, String resultVar, String tab) {
        this.visitorPrinter.println(tab + "// In particular case of this concrete element is inherited by another concrete element, it ");
        this.visitorPrinter.println(tab + "// shall be in the visitorMap. Also do call the treatment defined at the current super level:");
        this.visitorPrinter.println(tab + "if (" + resultVar + " == null || !(\"" + className + "Impl\".equals(" + resultVar + ".getClass().getSimpleName()))) {");
    }

    @objid ("1afe7659-4583-11e0-b54c-00137279a832")
    private void generateVisitorDefaultSwitch() {
        // visitorPrinter.println("\t@Override");
        // visitorPrinter.println("\tpublic Object defaultCase(");
        // visitorPrinter.println("\t\tEObject inputElement)");
        // visitorPrinter.println("\t{");
        // visitorPrinter.println("\t\treturn
        // super.defaultCase(inputElement);");
        // visitorPrinter.println("\t}\r\n");
        
        this.visitorPrinter.println("\t/**");
        this.visitorPrinter.println("\t * Calls caseXXX for each class of the model until one" + " returns a non null result; it yields that result.");
        this.visitorPrinter.println("\t * ");
        this.visitorPrinter.println("\t * @param inputElement the target of the switch.");
        this.visitorPrinter.println("\t * @return the first non-null result returned by a " + "caseXXX call.");
        this.visitorPrinter.println("\t */");
        
        this.visitorPrinter.println("\t@Override");
        this.visitorPrinter.println("\tpublic Object doSwitch(");
        this.visitorPrinter.println("\t\tEObject inputElement)");
        this.visitorPrinter.println("\t{");
        this.visitorPrinter.println("\t\tif (inputElement != null)");
        this.visitorPrinter.println("\t\t\treturn super.doSwitch(inputElement);");
        this.visitorPrinter.println("\t\telse");
        this.visitorPrinter.println("\t\t\treturn null;");
        this.visitorPrinter.println("\t}\r\n");
        
        // visitorPrinter.println("\t@Override");
        // visitorPrinter.println("\tprotected Object doSwitch(");
        // visitorPrinter.println("\t\tEClass inputClass,");
        // visitorPrinter.println("\t\tEObject inputElement)");
        // visitorPrinter.println("\t{");
        // visitorPrinter.println("\t\treturn super.doSwitch(inputClass,
        // inputElement);");
        // visitorPrinter.println("\t}\r\n");
        
        // visitorPrinter.println("\t@Override");
        // visitorPrinter.println("\tprotected Object doSwitch(");
        // visitorPrinter.println("\t\tint inputIntValue,");
        // visitorPrinter.println("\t\tEObject inputElement)");
        // visitorPrinter.println("\t{");
        // visitorPrinter.println("\t\treturn super.doSwitch(inputIntValue,
        // inputElement);");
        // visitorPrinter.println("\t}");
    }

    @objid ("1afe765b-4583-11e0-b54c-00137279a832")
    private void generateBehaviorDefaultSwitch() {
        // behaviorPrinter.println("\tpublic void defaultvisit(");
        // behaviorPrinter.println("\t\tEObject inputElement)");
        // behaviorPrinter.println("\t{");
        // behaviorPrinter.println("\t}\r\n");
        
        this.behaviorPrinter.println("\tpublic void doSwitch(");
        this.behaviorPrinter.println("\t\tEObject inputElement)");
        this.behaviorPrinter.println("\t{");
        this.behaviorPrinter.println("\t}\r\n");
        
        // behaviorPrinter.println("\tprotected void doSwitch(");
        // behaviorPrinter.println("\t\tEClass inputClass,");
        // behaviorPrinter.println("\t\tEObject inputElement)");
        // behaviorPrinter.println("\t{");
        // behaviorPrinter.println("\t}\r\n");
        
        // behaviorPrinter.println("\tprotected void doSwitch(");
        // behaviorPrinter.println("\t\tint inputIntValue,");
        // behaviorPrinter.println("\t\tEObject inputElement)");
        // behaviorPrinter.println("\t{");
        // behaviorPrinter.println("\t}");
    }

    @objid ("1afe765d-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        String path;
        path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/eclipse new/workspace2/aaa/lib/uml2metamodel/metamodels/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1afe7661-4583-11e0-b54c-00137279a832")
    private void initVisitorFile() {
        this.visitorPrinter.println("public class Uml2MetamodelVisitor");
        this.visitorPrinter.println("\textends UMLSwitch");
        this.visitorPrinter.println("{");
        this.visitorPrinter.println("\tprivate XMIImportBehavior behavior;\r\n");
        this.visitorPrinter.println("\tprivate Map<Object, Object> visitorMap;\r\n");
    }

    @objid ("1b00d8ad-4583-11e0-b54c-00137279a832")
    private void initBehaviorFile() {
        this.behaviorPrinter.println("public class XMIImportBehavior");
        this.behaviorPrinter.println("{");
        this.behaviorPrinter.println("\tprivate ImportModel importModel;\r\n");
    }

    @objid ("1b00d8af-4583-11e0-b54c-00137279a832")
    private void generateVisitorConstructor() {
        this.visitorPrinter.println("\tpublic Uml2MetamodelVisitor(");
        this.visitorPrinter.println("\t\tXMIImportBehavior behavior)");
        this.visitorPrinter.println("\t{");
        this.visitorPrinter.println("\t\tthis.behavior = behavior;");
        this.visitorPrinter.println("\t\tthis.visitorMap = new HashMap();\r\n");
        
        this.visitorPrinter.println("\t\tReverseProperties revProp = ReverseProperties." + "getInstance();");
        this.visitorPrinter.println("\t\tModelImpl ecoreRootModel = revProp.getEcoreModel();");
        this.visitorPrinter.println("\t\tthis.visitorMap.put(ecoreRootModel, ecoreRootModel);");
        this.visitorPrinter.println("\t}\r\n");
    }

    @objid ("1b00d8b1-4583-11e0-b54c-00137279a832")
    private void generateBehaviorConstructor() {
        this.behaviorPrinter.println("\tpublic XMIImportBehavior()");
        this.behaviorPrinter.println("\t{");
        this.behaviorPrinter.println("\t\timportModel = new ImportModel();");
        this.behaviorPrinter.println("\t}\r\n");
    }

    @objid ("1b00d8b3-4583-11e0-b54c-00137279a832")
    private void closeVisitorFile() {
        this.visitorPrinter.println("}");
        this.visitorPrinter.flush();
        this.visitorPrinter.close();
    }

    @objid ("1b00d8b5-4583-11e0-b54c-00137279a832")
    private void closeBehaviorFile() {
        this.behaviorPrinter.println("}");
        this.behaviorPrinter.flush();
        this.behaviorPrinter.close();
    }

    @objid ("1b00d8b7-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File file) {
        BufferedWriter buffer = null;
        try {
            buffer = new BufferedWriter(new FileWriter(file));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return buffer;
    }

    @objid ("1b033b07-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter buffer) {
        PrintWriter print = null;
        if (buffer != null)
            print = new PrintWriter(buffer);
        return print;
    }

    @objid ("1b033b0c-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b033b11-4583-11e0-b54c-00137279a832")
    private void generateVisitorImports() {
        this.visitorPrinter.println("package com.modeliosoft.modelio.xmi.uml2visitorGen;\r\n");
        
        this.visitorPrinter.println("import java.util.HashMap;");
        this.visitorPrinter.println("import java.util.Map;\r\n");
        
        this.visitorPrinter.println("import org.eclipse.uml2.uml.util.UMLSwitch;");
        this.visitorPrinter.println("import java.lang.Object;");
        this.visitorPrinter.println("import org.eclipse.emf.ecore.EModelElement;");
        this.visitorPrinter.println("import org.eclipse.emf.ecore.EObject;");
        this.visitorPrinter.println("import org.eclipse.emf.ecore.EClass;");
        this.visitorPrinter.println("import org.eclipse.uml2.uml.Class;");
        this.visitorPrinter.println("import org.eclipse.uml2.uml.Package;");
        this.visitorPrinter.println("import org.eclipse.uml2.uml.*;\r\n");
    }

    @objid ("1b059d60-4583-11e0-b54c-00137279a832")
    private void generateBehaviorImports() {
        this.behaviorPrinter.println("package com.modeliosoft.modelio.xmi.uml2visitorGen;\r\n");
        
        this.visitorPrinter.println("import org.eclipse.uml2.uml.Class;");
        this.visitorPrinter.println("import org.eclipse.uml2.uml.Package;");
        this.behaviorPrinter.println("import org.eclipse.uml2.uml.*;\r\n");
    }

    @objid ("1b059d62-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        OwnedAssociationUml2VisitorGen gen = new OwnedAssociationUml2VisitorGen();
        
        gen.visitorFile = gen.createFile("C:/eclipse new/workspace2/Uml2MetamodelVisitor.java");
        gen.behaviorFile = gen.createFile("C:/eclipse new/workspace2/XMIImportBehavior.java");
        
        gen.visitorBuffer = gen.createBuffer(gen.visitorFile);
        gen.behaviorBuffer = gen.createBuffer(gen.behaviorFile);
        
        gen.visitorPrinter = gen.createPrinter(gen.visitorBuffer);
        gen.behaviorPrinter = gen.createPrinter(gen.behaviorBuffer);
        
        gen.generateVisitorImports();
        gen.generateBehaviorImports();
        
        gen.initVisitorFile();
        gen.initBehaviorFile();
        
        gen.generateVisitorConstructor();
        gen.generateBehaviorConstructor();
        
        gen.generateUml2Visitor();
        
        gen.generateVisitorDefaultSwitch();
        gen.generateBehaviorDefaultSwitch();
        
        gen.closeVisitorFile();
        gen.closeBehaviorFile();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b059d67-4583-11e0-b54c-00137279a832")
    private void parseUml2Metamodel(Resource res) {
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                LOG.info(XMI.PLUGIN_ID, elem.getClass().getSimpleName());
                ModelImpl mdl = (ModelImpl) elem;
                LOG.info(XMI.PLUGIN_ID, mdl.getName());
                String umlTypeName;
                int i = 0;
                for (Object umlElt : mdl.getPackagedElements()) {
                    umlTypeName = umlElt.getClass().getSimpleName();
                    if (umlTypeName.equals("ClassImpl")) {
                        computeElement((NamedElement) umlElt);
                        i++;
                    }
                    // else {
                    // System.out.println("\t"+umlTypeName);
                    // if (umlTypeName.equals("EnumerationImpl") ||
                    // umlTypeName.equals("PrimitiveTypeImpl"))
                    // {
                    // System.out.println("\t\t"+((NamedElement)umlElt).getName());
                    // }
                    // }
                }
                LOG.info(XMI.PLUGIN_ID, "nb elts : " + i);
            }
        }
    }

    @objid ("1b059d6a-4583-11e0-b54c-00137279a832")
    private void computeElement(NamedElement umlElt) {
        if (umlElt instanceof ClassImpl) {
            this.visitorPrinter.flush();
            this.behaviorPrinter.flush();
            computeClass((ClassImpl) umlElt);
        }
    }

    @objid ("1b059d6d-4583-11e0-b54c-00137279a832")
    private void computeClass(ClassImpl class_) {
        boolean isAbstract = class_.isAbstract();
        
        String className = class_.getName();
        
        if ("Class".equals(className))
            LOG.info(XMI.PLUGIN_ID, "Class");
        
        String inArgName = "input" + className;
        String tab = "\t";
        
        // Javadoc generation
        String abstractOrConcreteStr;
        if (isAbstract) {
            abstractOrConcreteStr = "abstract";
        } else {
            abstractOrConcreteStr = "concrete";
        }
        this.visitorPrinter.println(tab + "/**");
        
        this.visitorPrinter.print(tab + " * Returns the result of interpreting the ");
        this.visitorPrinter.print(abstractOrConcreteStr);
        this.visitorPrinter.println(" object as an instance of '" + className + "'.<BR>");
        this.visitorPrinter.println(tab + " * This implementation returns null; returning a " + "non-null result will terminate the switch.");
        this.visitorPrinter.println(tab + " * ");
        this.visitorPrinter.println(tab + " * @param " + inArgName + " the " + className + " to visit");
        this.visitorPrinter.println(tab + " * @return the result of interpreting the object " + "as an instance of '" + className + "'.");
        this.visitorPrinter.println(tab + " */");
        
        // Generation of the content of the method:
        this.visitorPrinter.println(tab + "@Override");
        this.visitorPrinter.println(tab + "public Object case" + className + "(");
        this.behaviorPrinter.println(tab + "public void visit" + className + "(");
        
        tab = "\t\t";
        this.visitorPrinter.println(tab + className + " " + inArgName + ")");
        this.behaviorPrinter.println(tab + className + " " + inArgName + ")");
        
        tab = "\t";
        this.visitorPrinter.println(tab + "{");
        this.behaviorPrinter.println(tab + "{");
        
        tab = "\t\t";
        // visitorPrinter.println(tab+"Object superRet = null;");
        
        if (!isAbstract) {
            // EList concreteSubClasses = getConcreteSubClasses(class_);
            // compareTypes(className, inArgName);
        
            this.visitorPrinter.println(tab + "Object theResult = visitorMap.get(" + inArgName + ");");
        
            this.behaviorPrinter.println(tab + "E" + className + " element = new E" + className + "(" + inArgName + ");");
            this.behaviorPrinter.println(tab + "importModel.importXMI(element);");
        
            this.visitorPrinter.println(tab + "if (theResult == null) {");
            tab = "\t\t\t";
            this.visitorPrinter.println(tab + "visitorMap.put(" + inArgName + ", " + inArgName + ");");
            tab = "\t\t";
            this.visitorPrinter.println(tab + "}\r\n");
        
            compareTypes(className, "theResult", tab);
            tab = "\t\t\t";
            // visitorPrinter.println(tab+"theResult = "+inArgName+";");
            this.visitorPrinter.println(tab + "behavior.visit" + className + "(" + inArgName + ");");
        
            this.visitorPrinter.println(tab + "theResult = super.case" + className + "(" + inArgName + ");");
            visitLinks(class_, inArgName, tab);
        
            tab = "\t\t";
            this.visitorPrinter.println(tab + "}");
        } else {
            this.visitorPrinter.println(tab + "Object theResult = super.case" + className + "(" + inArgName + ");");
            visitLinks(class_, inArgName, tab);
        }
        // visitorPrinter.println(tab+"superRet = super.case" + className +
        // "("+inArgName+");");
        
        this.visitorPrinter.println("\t\treturn theResult;");
        this.visitorPrinter.println("\t}\r\n");
        this.behaviorPrinter.println("\t}\r\n");
    }

    @objid ("1b07ffba-4583-11e0-b54c-00137279a832")
    private void visitLinks(ClassImpl class_, String inArgName, String tab) {
        for (Object feature : class_.getFeatures()) {
            if (feature instanceof Property) {
                Property prop = (Property) feature;
                Association assoc = prop.getAssociation();
                if (assoc != null) {
                    if (!isUnlimitedMultiplicity(prop)) {
                        String propName = prop.getName();
                        if ("".equals(propName)) {
                            LOG.warning(XMI.PLUGIN_ID, "The name of the Association end is \"\".");
                            propName = "ownedElement";
                        } else if ("class".equals(propName)) {
                            propName = "class_";
                        }
        
                        // ex : transformation from "nameExpression" to
                        // "getNameExpression"
                        String methodName = propName.substring(1);
                        String prefix = String.valueOf(propName.charAt(0)).toUpperCase();
                        prefix = "get" + prefix;
                        methodName = prefix + methodName;
        
                        // visitorPrinter.println(tab+"if
                        // (visitorMap.get("+inArgName+"."+methodName+"()) ==
                        // null)");
                        // visitorPrinter.println(tab+"\tthis.doSwitch((EObject)"+inArgName+"."+methodName+"());");
                        this.visitorPrinter.println(tab + "this.doSwitch((EObject)" + inArgName + "." + methodName + "());");
                    } else {
                        String propName = prop.getName();
                        if ("".equals(propName)) {
                            LOG.warning(XMI.PLUGIN_ID, "The name of the Association end is \"\".");
                            propName = "ownedElement";
                        }
                        // ex : transformation from "ownedElement" to
                        // "getOwnedElements"
                        String methodName = propName.substring(1);
                        String prefix = String.valueOf(propName.charAt(0)).toUpperCase();
                        prefix = "get" + prefix;
                        methodName = prefix + methodName;
        
                        String plural = "";
        
                        if ("endData".equals(propName)) {
                            plural = "";
                        } else if (propName.charAt(propName.length() - 1) == 'y') {
                            // Replaces "y" by "ies" in "clientDependency" ->
                            // "getClientDependencies"
                            methodName = methodName.substring(0, methodName.length() - 1);
                            if (propName.contains("pendency") || propName.contains("try") || propName.contains("ody") || propName.contains("roperty")) {
                                plural = "ies";
                            } else if (propName.contains("overedBy"))
                                plural = "ys";
                            else
                                plural = "y";
                        } else if (propName.charAt(propName.length() - 1) == 'x' && propName.contains("subvertex")) {
                            methodName = methodName.substring(0, methodName.length() - 2);
                            plural = "ices";
                        } else if (propName.charAt(propName.length() - 1) == 's') {
                            plural = "es";
                        } else {
                            plural = "s";
                        }
        
                        methodName = methodName + plural;
        
                        this.visitorPrinter.println(tab + "for (Object " + propName + " : " + inArgName + "." + methodName + "()) {");
        
                        // visitorPrinter.println(tab+"\tif
                        // (visitorMap.get("+propName+") == null)");
                        // visitorPrinter.println(tab+"\t\tthis.doSwitch((EObject)"+propName+");");
                        this.visitorPrinter.println(tab + "\tthis.doSwitch((EObject)" + propName + ");");
                        this.visitorPrinter.println(tab + "}");
                    }
                }
            }
        }
    }

    @objid ("1b07ffc4-4583-11e0-b54c-00137279a832")
    private boolean isUnlimitedMultiplicity(Property prop) {
        ValueSpecification upperValue = prop.getUpperValue();
        
        if (upperValue != null && upperValue.isComputable()) {
            if (upperValue instanceof LiteralUnlimitedNatural) {
                LiteralUnlimitedNatural naturalIntValue = (LiteralUnlimitedNatural) upperValue;
                if (naturalIntValue.getValue() == 1)
                    return false;
                else
                    return true;
            } else if (upperValue instanceof LiteralInteger) {
                if (upperValue.integerValue() > 1)
                    return true;
            } else if (upperValue instanceof LiteralBoolean) {
                return false;
            } else if (upperValue instanceof LiteralString) {
                String upperValueStr = upperValue.stringValue();
                if ("*".equals(upperValueStr))
                    return true;
                else if (Integer.valueOf(upperValueStr) > 1) {
                    return true;
                }
                return false;
            } else if (upperValue instanceof LiteralNull) {
                return false;
            }
            // int upperValueInt = upperValue.integerValue();
            // boolean uppervalueBool = upperValue.booleanValue();
            // String upperValueStr = upperValue.stringValue();
            // int upperValueUnlimited = upperValue.unlimitedValue();
            // boolean upperValueIsNull = upperValue.isNull();
        }
        return false;
    }

}
