/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1b0a6219-4583-11e0-b54c-00137279a832")
public class OwnedCompositionUml2VisitorGen {
    @objid ("1b0a621a-4583-11e0-b54c-00137279a832")
    private boolean isCurrentClassAbstract;

    @objid ("1b0a621b-4583-11e0-b54c-00137279a832")
    private String metaclassName;

    @objid ("1b0a621c-4583-11e0-b54c-00137279a832")
    private String inArgName;

    @objid ("1b0a621d-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b0a621e-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b0a621f-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;


    @objid ("1b0a6220-4583-11e0-b54c-00137279a832")
    public OwnedCompositionUml2VisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
        this.isCurrentClassAbstract = false;
    }

    @objid ("1b0a6223-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b0cc46e-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b0cc473-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b0cc478-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b0cc47d-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b0f26c8-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.reverse;\r\n");
        
        this.printer.println("import java.util.HashMap;");
        this.printer.println("import java.util.Map;\r\n");
        
        this.printer.println("import org.eclipse.emf.ecore.EObject;");
        this.printer.println("import org.eclipse.uml2.uml.*;\r\n");
        this.printer.println("import org.eclipse.uml2.uml.Class;");
        this.printer.println("import org.eclipse.uml2.uml.Package;");
        this.printer.println("import org.eclipse.uml2.uml.internal.impl.ModelImpl;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;");
        
        // printer.println("import
        // com.modeliosoft.modelio.xmi.gui.ProgressBarComposite;");
        this.printer.println("import com.modeliosoft.modelio.xmi.util.ReverseProperties;\r\n");
    }

    @objid ("1b0f26ca-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to visit the EMF-UML2 metamodel along composition relations." + "\r\n*/";
        
        generateJavadoc(docText);
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generateUml2VisitMethods();
        generateClassClosure();
    }

    @objid ("1b0f26cc-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class OwnedCompositionUml2VisitorGenerated");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1b0f26ce-4583-11e0-b54c-00137279a832")
    private void generateClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b0f26d0-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        this.printer.println("private XMIImportBehavior behavior;\r\n");
        this.printer.println("private Map<Object, Object> visitorMap;\r\n");
        // printer.println("private ProgressBarComposite progressBar =
        // null;\r\n");
    }

    @objid ("1b0f26d2-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public OwnedCompositionUml2VisitorGenerated(");
        this.printer.println("XMIImportBehavior behavior)");
        this.printer.println("{");
        this.printer.println("this.behavior = behavior;");
        this.printer.println("this.visitorMap = new HashMap<Object, Object>();\r\n");
        // printer.println("this.progressBar =
        // behavior.getTheProgressBar();\r\n");
        this.printer.println("ReverseProperties revProp = ReverseProperties." + "getInstance();");
        this.printer.println("ModelImpl ecoreRootModel = revProp.getEcoreModel();");
        this.printer.println("this.visitorMap.put(ecoreRootModel, ecoreRootModel);");
        this.printer.println("}\r\n");
    }

    @objid ("1b0f26d4-4583-11e0-b54c-00137279a832")
    private void generateUml2VisitMethods() {
        generateDefaultSwitch();
        Resource res = loadEcoreModel();
        if (res != null) {
            for (Class uml2Class : getUml2Metaclasses(res)) {
                LOG.info(XMI.PLUGIN_ID, "metaclass = " + uml2Class.getName());
                visitMetaclass(uml2Class);
            }
        }
    }

    @objid ("1b118923-4583-11e0-b54c-00137279a832")
    private void generateDefaultSwitch() {
        String docText = "/**\r\n" + "Calls caseXXX for each class of the model until one returns a non null" + " result; it yields that result.\r\n" + "@param inputElement the target of the switch.\r\n" + "@return the first non-null result returned by a caseXXX call.\r\n" + "*/";
        generateJavadoc(docText);
        
        this.printer.println("@Override");
        this.printer.println("public Object doSwitch(");
        this.printer.println("EObject inputElement)");
        this.printer.println("{");
        this.printer.println("if (inputElement != null)");
        this.printer.println("return super.doSwitch(inputElement);");
        this.printer.println("else");
        this.printer.println("return null;");
        this.printer.println("}\r\n");
    }

    @objid ("1b118925-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        String path;
        path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1b118929-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader() {
        String abstractOrConcreteStr;
        if (this.isCurrentClassAbstract)
            abstractOrConcreteStr = "abstract";
        else
            abstractOrConcreteStr = "concrete";
        
        this.inArgName = "input" + this.metaclassName;
        
        String docText = "/**\r\n* " + " Returns the result of interpreting the " + abstractOrConcreteStr + " object as an instance of '" + this.metaclassName + "'.<BR>\r\n" + " * This implementation returns null; returning a " + "non-null result will terminate the switch.\r\n" + " *\r\n" + " * @param " + this.inArgName + " the " + this.metaclassName + " to visit\r\n" + " * @return the result of interpreting the object " + "as an instance of '" + this.metaclassName + "'." + "\r\n*/";
        generateJavadoc(docText);
        
        this.printer.println("@Override");
        this.printer.print("public Object case");
        this.printer.print(this.metaclassName);
        this.printer.println("(");
        
        this.printer.print(this.metaclassName);
        this.printer.println(" " + this.inArgName + ")");
        this.printer.println("{");
    }

    @objid ("1b11892b-4583-11e0-b54c-00137279a832")
    private void compareTypes(String className, String resultVar) {
        this.printer.print("// If this concrete element is inherited by another concrete element, it");
        this.printer.print(" shall be in the visitorMap. Also do call the treatment defined at the");
        this.printer.println(" current super level:");
        
        this.printer.println("if (" + resultVar + " == null || !(\"" + className + "Impl\".equals(" + resultVar + ".getClass().getSimpleName()))) {");
    }

    @objid ("1b13eb7c-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        if ("PrimitiveType".equals(this.metaclassName)) {
            this.printer.println("// [CVI]: case of a primitive type: read only in Ijing => do not call super classes");
            this.printer.println("return 42;");
        } else
            this.printer.println("return null;");
    }

    @objid ("1b13eb7e-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b13eb80-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b13eb82-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        OwnedCompositionUml2VisitorGen gen = new OwnedCompositionUml2VisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/reverse/OwnedCompositionUml2VisitorGenerated.java");
        
        gen.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b13eb87-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2Metaclasses(Resource res) {
        List<Class> uml2Classes = new ArrayList<Class>();
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                ModelImpl mdl = (ModelImpl) elem;
                for (Object umlElt : mdl.getPackagedElements()) {
                    if (umlElt instanceof Class) {
                        // && !((Class)umlElt).isAbstract()) {
                        uml2Classes.add((Class) umlElt);
                    }
                }
            }
        }
        // Sort classes by name:
        uml2Classes = sortByName(uml2Classes);
        return uml2Classes;
    }

    @objid ("1b164ddb-4583-11e0-b54c-00137279a832")
    private List<Class> sortByName(List<Class> uml2Classes) {
        int listSize = uml2Classes.size();
        boolean isDisordered = true;
        
        for (int i = 0; i < listSize && isDisordered; i++) {
            isDisordered = false;
            for (int j = 1; j < listSize - i; j++) {
                if (compareTo(uml2Classes.get(j), uml2Classes.get(j - 1)) < 0) {
                    Class tmpClass = uml2Classes.get(j - 1);
                    uml2Classes.set(j - 1, uml2Classes.get(j));
                    uml2Classes.set(j, tmpClass);
                    isDisordered = true;
                }
            }
        }
        return uml2Classes;
    }

    @objid ("1b164de4-4583-11e0-b54c-00137279a832")
    private int compareTo(Class class1, Class class2) {
        if (class1 != null && class2 != null)
            return class1.getName().toUpperCase().compareTo(class2.getName().toUpperCase());
        return -1;
    }

    @objid ("1b18b033-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(Class metaclass) {
        this.metaclassName = metaclass.getName();
        this.isCurrentClassAbstract = metaclass.isAbstract();
        
        generateMethodHeader();
        
        generateMethodBody(metaclass);
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1b18b036-4583-11e0-b54c-00137279a832")
    private void generateMethodBody(Class metaclass) {
        if (this.isCurrentClassAbstract)
            generateBodyForAbstractClass(metaclass);
        else
            generateBodyForConcreteClass(metaclass);
    }

    @objid ("1b18b039-4583-11e0-b54c-00137279a832")
    private void generateBodyForAbstractClass(Class metaclass) {
        // addProgressBarValue(metaclass);// NOT CALLED IN THE VISITOR
        // BECAUSE OF THE SUPER CALL TO NAMESPACE => CALLED TOO MANY TIMES.
        // printer.println("Object theResult = super.case"
        // + metaclassName + "(" + inArgName + ");");
        visitCompositionLinks(metaclass);
        // addProgressBarValue(metaclass);
    }

    @objid ("1b1b128d-4583-11e0-b54c-00137279a832")
    private void generateBodyForConcreteClass(Class metaclass) {
        this.printer.println("Object theResult = visitorMap.get(" + this.inArgName + ");");
        //
        // printer.println("if (theResult == null) {");
        compareTypes(this.metaclassName, "theResult");
        this.printer.println("visitorMap.put(" + this.inArgName + ", " + this.inArgName + ");");
        // printer.println("}\r\n");
        
        ignoreModelVisit();
        
        // compareTypes(metaclassName, "theResult");
        // printer.println(tab+"theResult = "+inArgName+";");
        this.printer.println("behavior.visit" + this.metaclassName + "(" + this.inArgName + ");");
        
        // printer.println("theResult = super.case" + metaclassName
        // + "(" + inArgName + ");");
        visitCompositionLinks(metaclass);
        
        this.printer.println("}");
    }

    @objid ("1b1b1290-4583-11e0-b54c-00137279a832")
    private void ignoreModelVisit() {
        if ("Package".equals(this.metaclassName)) {
            this.printer.println("// [CVI]: ignore visit of relations defined at \"Package\" level for an element of type \"Model\"");
            this.printer.println("if (inputPackage instanceof Model)");
            this.printer.println("return null;\r\n");
        }
    }

    @objid ("1b1b1293-4583-11e0-b54c-00137279a832")
    private void visitCompositionLinks(Class metaclass) {
        for (Object feature : metaclass.getFeatures()) {
            if (feature instanceof Property) {
                Property prop = (Property) feature;
                if (prop.getAssociation() != null && prop.getAggregation() == AggregationKind.COMPOSITE_LITERAL) {
                    visitCompositionLink(prop);
                }
            }
        }
    }

    @objid ("1b1d74e4-4583-11e0-b54c-00137279a832")
    private void visitCompositionLink(Property prop) {
        if (!isUnlimitedMultiplicity(prop) && !isUnlimitedFilter(prop)) {
            generateGetter(prop);
        } else {
            generateForEach(prop);
        }
    }

    @objid ("1b1d74e7-4583-11e0-b54c-00137279a832")
    private boolean isUnlimitedFilter(Property prop) {
        if ("Extension".equals(this.metaclassName) && "ownedEnd".equals(prop.getName())) {
            return true;
        }
        return false;
    }

    @objid ("1b1d74ec-4583-11e0-b54c-00137279a832")
    private void generateGetter(Property prop) {
        String propName = prop.getName();
        if ("".equals(propName)) {
            LOG.error(XMI.PLUGIN_ID, "The name of the Association end is \"\".");
            propName = "ownedElement";
        } else if ("class".equals(propName)) {
            propName = "class_";
        }
        
        // ex : transformation from "nameExpression" to
        // "getNameExpression"
        String methodName = propName.substring(1);
        String prefix = String.valueOf(propName.charAt(0)).toUpperCase();
        prefix = "get" + prefix;
        methodName = prefix + methodName;
        
        this.printer.println("this.doSwitch((EObject)" + this.inArgName + "." + methodName + "());");
    }

    @objid ("1b1d74ef-4583-11e0-b54c-00137279a832")
    private void generateForEach(Property prop) {
        String propName = prop.getName();
        if ("".equals(propName)) {
            LOG.error(XMI.PLUGIN_ID, "The name of the Association end is \"\".");
            propName = "ownedElement";
        }
        // ex : transformation from "ownedElement" to
        // "getOwnedElements"
        String methodName = propName.substring(1);
        String prefix = String.valueOf(propName.charAt(0)).toUpperCase();
        prefix = "get" + prefix;
        methodName = prefix + methodName;
        
        String plural = "";
        
        if ("endData".equals(propName)) {
            plural = "";
        } else if (propName.charAt(propName.length() - 1) == 'y') {
            // Replaces "y" by "ies" in "clientDependency" ->
            // "getClientDependencies"
            methodName = methodName.substring(0, methodName.length() - 1);
            if (propName.contains("pendency") || propName.contains("try") || propName.contains("ody") || propName.contains("roperty")) {
                plural = "ies";
            } else if (propName.contains("overedBy"))
                plural = "ys";
            else
                plural = "y";
        } else if (propName.charAt(propName.length() - 1) == 'x' && propName.contains("subvertex")) {
            methodName = methodName.substring(0, methodName.length() - 2);
            plural = "ices";
        } else if (propName.charAt(propName.length() - 1) == 's') {
            plural = "es";
        } else {
            plural = "s";
        }
        
        methodName = methodName + plural;
        
        this.printer.println("for (Object " + propName + " : " + this.inArgName + "." + methodName + "()) {");
        
        this.printer.println("this.doSwitch((EObject)" + propName + ");");
        this.printer.println("}");
    }

    @objid ("1b1fd73e-4583-11e0-b54c-00137279a832")
    private boolean isUnlimitedMultiplicity(Property prop) {
        ValueSpecification upperValue = prop.getUpperValue();
        
        if (upperValue != null && upperValue.isComputable()) {
            if (upperValue instanceof LiteralUnlimitedNatural) {
                LiteralUnlimitedNatural naturalIntValue = (LiteralUnlimitedNatural) upperValue;
                if (naturalIntValue.getValue() == 1)
                    return false;
                else
                    return true;
            } else if (upperValue instanceof LiteralInteger) {
                if (upperValue.integerValue() > 1)
                    return true;
            } else if (upperValue instanceof LiteralBoolean) {
                return false;
            } else if (upperValue instanceof LiteralString) {
                String upperValueStr = upperValue.stringValue();
                if ("*".equals(upperValueStr))
                    return true;
                else if (Integer.valueOf(upperValueStr) > 1) {
                    return true;
                }
                return false;
            } else if (upperValue instanceof LiteralNull) {
                return false;
            }
        }
        return false;
    }

}
