/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import com.modeliosoft.modelio.api.model.uml.infrastructure.IElement;
import com.modeliosoft.modelio.api.model.uml.statik.IClass;
import com.modeliosoft.modelio.api.model.uml.statik.IComponent;
import com.modeliosoft.modelio.api.model.uml.statik.IModelTree;
import com.modeliosoft.modelio.api.model.uml.statik.IPackage;
import com.modeliosoft.modelio.api.utils.IObListFilter;
import com.modeliosoft.modelio.api.utils.NameFilter;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;

@objid ("1b1fd743-4583-11e0-b54c-00137279a832")
public class PartialCreationExportVisitorGen {
    @objid ("1b1fd744-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b1fd745-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b1fd746-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;

    @objid ("1b1fd747-4583-11e0-b54c-00137279a832")
    private Class<?> ecoreFactory = null;


    @objid ("1b1fd748-4583-11e0-b54c-00137279a832")
    public PartialCreationExportVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
        try {
            this.ecoreFactory = Class.forName("org.eclipse.uml2.uml.UMLFactory");
        } catch (ClassNotFoundException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
    }

    @objid ("1b1fd74b-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b223998-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b22399d-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b2239a2-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b2239a7-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b249bf4-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.generation;\r\n");
        
        this.printer.println("import org.eclipse.uml2.uml.Element;");
        this.printer.println("import org.eclipse.uml2.uml.UMLFactory;");
        this.printer.println("import com.softeam.modelio.joni.model.utils.DefaultMetamodelVisitor;");
        this.printer.println("import com.softeam.modelio.joni.model.*;");
        this.printer.println();
    }

    @objid ("1b249bf6-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to get an Ecore element mapped from the given Modelio " + "element." + "Call the \"createPartialEcoreElt()\" method to get the corresponding Ecore " + "element." + "\r\n*/";
        
        generateJavadoc(docText);
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassCode();
        generateClassFooter();
    }

    @objid ("1b249bf8-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class PartialCreationExportVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1b249bfa-4583-11e0-b54c-00137279a832")
    private void generateClassFooter() {
        this.printer.println("}");
    }

    @objid ("1b249bfc-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Represents the Ecore element that will be returned." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private Element ecoreElt;\r\n");
        
        docText = "/**\r\n* " + "Visitor that will realize a switch to call the appropriate \"creation\" methods." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private CreationExportMapper mapper;\r\n");
        this.printer.println();
        
        docText = "/**\r\n* " + "HashMap containing as key an Modelio element, and, as value, the corresponding " + "Ecore element partially created." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private PartialExportMap partialMap;\r\n");
    }

    @objid ("1b249bfe-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public PartialCreationExportVisitorGenerated()");
        this.printer.println("{");
        this.printer.println("ecoreElt = null;");
        this.printer.println("mapper = new CreationExportMapper();");
        this.printer.println("partialMap = PartialExportMap.getInstance();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b26fe4c-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b26fe4e-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        String docText = "/**\r\n* " + "Returns the Ecore element mapped from the given element.\r\n" + "@param objingElt The element from which we will do the mapping.\r\n" + "@return the Ecore element mapped from the element given as parameter." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public Element createPartialEcoreElt(");
        this.printer.println("IElement objingElt)");
        this.printer.println("{");
        
        this.printer.println("Element ecoreEltFromMap = partialMap.get(objingElt.getId());");
        this.printer.println("if (ecoreEltFromMap == null) {");
        this.printer.println("mapper.accept(objingElt);");
        this.printer.println("partialMap.put(objingElt.getId(), ecoreElt);");
        this.printer.println("}");
        this.printer.println("return ecoreElt;");
        
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b26fe50-4583-11e0-b54c-00137279a832")
    private void generateInnerClassCode() {
        String docText = "/**\r\n* " + "This class extends DefaultMetamodelVisitor and realizes a switch.\r\n" + "The class is inner and private, encapsulated to mask the \"visit\" methods to the user of the PartialCreationExportVisitor class.\r\n" + "The user will not see this class." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerClassHeader();
        
        generateInnerMemberAttributes();
        generateInnerConstructor();
        
        docText = "/**\r\n* " + "Launches the switch." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerPublicMethods();
        
        generateInnerRedefinedMethods();
        
        generateInnerClassFooter();
    }

    @objid ("1b26fe52-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        this.printer.println("private class CreationExportMapper");
        this.printer.println("extends DefaultMetamodelVisitor");
        this.printer.println("{");
    }

    @objid ("1b26fe54-4583-11e0-b54c-00137279a832")
    private void generateInnerClassFooter() {
        this.printer.println("}");
    }

    @objid ("1b26fe56-4583-11e0-b54c-00137279a832")
    private void generateInnerMemberAttributes() {
        // No member attributes
    }

    @objid ("1b26fe58-4583-11e0-b54c-00137279a832")
    private void generateInnerConstructor() {
        // No constructor
    }

    @objid ("1b2960a7-4583-11e0-b54c-00137279a832")
    private void generateInnerPublicMethods() {
        this.printer.println("public void accept(");
        this.printer.println("IElement objingElt)");
        this.printer.println("{");
        this.printer.println("objingElt.accept(this);");
        this.printer.println("}");
    }

    @objid ("1b2960a9-4583-11e0-b54c-00137279a832")
    private void generateInnerRedefinedMethods() {
        IPackage metamodelPkg = selectVisitEntryPoint();
        if (metamodelPkg != null) {
            for (IModelTree metaclass : metamodelPkg.getOwnedElement(IClass.class).select(new IObListFilter() {
                public boolean select(IElement element) {
                    if (((IClass) element).getName().equals("Item") || ((IClass) element).getName().equals("Object") || ((IClass) element).getName().equals("PointOfView")) {
                        return false;
                    }
        
                    if (isAbstract(element)) {
                        return false;
                    } else {
                        return true;
                    }
                }
        
                private boolean isAbstract(IElement element) {
                    try {
                        Class<?> objingClass = Class.forName(IElement.class.getPackage().getName() + ".I" + ((IClass) element).getName());
        
                        if (Modifier.isAbstract(objingClass.getModifiers()))
                            return true;
                    } catch (ClassNotFoundException e) {
                        LOG.error(XMI.PLUGIN_ID, e);
                    }
                    return false;
                }
            })) {
                visitMetaclass((IClass) metaclass);
            }
        } else {
            LOG.info(XMI.PLUGIN_ID, "Metamodel RAMC not found.");
        }
    }

    @objid ("1b2960ab-4583-11e0-b54c-00137279a832")
    private IPackage selectVisitEntryPoint() {
        IPackage root =  XMI.getInstance().getModelingSession().getModel().getProject().getModel();
        IPackage metamodelPackage = null;
        
        for (IModelTree ramc : root.getOwnedElement(IComponent.class).select(new NameFilter("metamodel"))) {
            for (IModelTree refMdl : ramc.getOwnedElement(IPackage.class).select(new NameFilter("Reference Metamodels"))) {
                for (IModelTree objingUML : refMdl.getOwnedElement(IPackage.class).select(new NameFilter("Modelio UML"))) {
                    for (IModelTree metamodelPkg : objingUML.getOwnedElement(IPackage.class).select(new NameFilter("Metamodel"))) {
                        metamodelPackage = (IPackage) metamodelPkg;
                    }
                }
            }
        }
        return metamodelPackage;
    }

    @objid ("1b2960af-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader(String metaclassName) {
        String docText = "/**\r\n* " + "Create the Ecore element from the given \"" + metaclassName + "\" Modelio element." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.print("public Object visit");
        this.printer.print(metaclassName);
        this.printer.println("(");
        
        this.printer.print("I");
        this.printer.print(metaclassName);
        this.printer.println(" objingElt)");
        this.printer.println("{");
    }

    @objid ("1b2bc300-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b2bc302-4583-11e0-b54c-00137279a832")
    private void generateEcoreEltCreation(String metaclassName) {
        if (this.ecoreFactory != null) {
            try {
                Method method = this.ecoreFactory.getMethod("create" + metaclassName, (Class[])null);
                if (method != null) {
                    this.printer.println("ecoreElt = UMLFactory.eINSTANCE.create" + metaclassName + "();");
                }
            } catch (SecurityException e) {
                // TODO Auto-generated catch block
                LOG.error(XMI.PLUGIN_ID, e);
            } catch (NoSuchMethodException e) {
                this.printer.println("// TODO map objing to ecore");
                LOG.error(XMI.PLUGIN_ID, "? create" + metaclassName + "() doesn't exist.");
            }
        }
    }

    @objid ("1b2bc305-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return null;");
    }

    @objid ("1b2bc307-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        
        PartialCreationExportVisitorGen generator = new PartialCreationExportVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/generation/PartialCreationExportVisitorGenerated.java");
        
        generator.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b2bc30c-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(IClass metaclass) {
        String metaclassName = getMetaclassName(metaclass);
        
        generateMethodHeader(metaclassName);
        
        generateEcoreEltCreation(metaclassName);
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1b2e255a-4583-11e0-b54c-00137279a832")
    private String getMetaclassName(IClass metaclass) {
        return metaclass.getName();
    }

}
