/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1b2e255f-4583-11e0-b54c-00137279a832")
public class PartialCreationImportVisitorGen {
    @objid ("1b2e2560-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b2e2561-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b2e2562-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;

    @objid ("1b2e2563-4583-11e0-b54c-00137279a832")
    private Class<?> objingFactory = null;


    @objid ("1b2e2564-4583-11e0-b54c-00137279a832")
    public PartialCreationImportVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
        try {
            this.objingFactory = Class.forName("com.softeam.modelio.mda.model.UMLModel");
        } catch (ClassNotFoundException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
    }

    @objid ("1b2e2567-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b3087b5-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b3087ba-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b3087bf-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b3087c4-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b32ea0f-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.reverse;\r\n");
        
        this.printer.println("import org.eclipse.uml2.uml.*;");
        this.printer.println("import org.eclipse.uml2.uml.Class;");
        this.printer.println("import org.eclipse.uml2.uml.Package;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;");
        this.printer.println();
        
        this.printer.println("import com.softeam.modelio.mda.objing.Modelio;");
        
        this.printer.println();
    }

    @objid ("1b32ea11-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to get an Ijing element mapped from the given Ecore " + "element." + "Call the \"createPartialIjingElt()\" method to get the corresponding Ijing " + "element." + "\r\n*/";
        generateJavadoc(docText);
        
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassCode();
        generateClassFooter();
    }

    @objid ("1b32ea13-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class PartialCreationImportVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1b32ea15-4583-11e0-b54c-00137279a832")
    private void generateClassFooter() {
        this.printer.println("}");
    }

    @objid ("1b32ea17-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Represents the Ijing element that will be returned (may be a List)." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private Object objingElt;\r\n");
        
        docText = "/**\r\n* " + "Visitor that will realize a switch to call the appropriate \"creation\" methods." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private CreationImportMapper mapper;\r\n");
        this.printer.println();
        
        docText = "/**\r\n* " + "HashMap containing as key an Ecore element, and, as value, the corresponding " + "Modelio element partially created." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private PartialImportMap partialMap;\r\n");
    }

    @objid ("1b354c68-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public PartialCreationImportVisitorGenerated()");
        this.printer.println("{");
        this.printer.println("objingElt = null;");
        this.printer.println("mapper = new CreationImportMapper();");
        this.printer.println("partialMap = PartialImportMap.getInstance();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b354c6a-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b354c6c-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        String docText = "/**\r\n* " + "Returns the Ijing element mapped from the given element.\r\n" + "@param ecoreElt The element from which we will do the mapping.\r\n" + "@return the Ijing element mapped from the element given as parameter." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public Object createPartialIjingElt(");
        this.printer.println("Element ecoreElt)");
        this.printer.println("{");
        
        this.printer.println("Object objingEltFromMap = partialMap.get(ecoreElt);");
        this.printer.println("if (objingEltFromMap == null) {");
        this.printer.println("mapper.accept(ecoreElt);");
        this.printer.println("partialMap.put(ecoreElt, objingElt);");
        this.printer.println("}");
        this.printer.println("return objingElt;");
        
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b354c6e-4583-11e0-b54c-00137279a832")
    private void generateInnerClassCode() {
        String docText = "/**\r\n* " + "This class extends UMLSwitch and realizes a switch.\r\n" + "The class is inner and private, encapsulated to mask the \"visit\" methods to the user of the PartialCreationImportVisitor class.\r\n" + "The user will not see this class." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerClassHeader();
        
        generateInnerMemberAttributes();
        generateInnerConstructor();
        
        docText = "/**\r\n* " + "Launches the switch." + "\r\n*/";
        generateJavadoc(docText);
        generateInnerPublicMethods();
        
        generateInnerRedefinedMethods();
        
        generateInnerClassFooter();
    }

    @objid ("1b354c70-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        this.printer.println("private class CreationImportMapper");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1b354c72-4583-11e0-b54c-00137279a832")
    private void generateInnerClassFooter() {
        this.printer.println("}");
    }

    @objid ("1b354c74-4583-11e0-b54c-00137279a832")
    private void generateInnerMemberAttributes() {
        // No member attributes
    }

    @objid ("1b37aec2-4583-11e0-b54c-00137279a832")
    private void generateInnerConstructor() {
        // No constructor
    }

    @objid ("1b37aec4-4583-11e0-b54c-00137279a832")
    private void generateInnerPublicMethods() {
        this.printer.println("public void accept(");
        this.printer.println("Element ecoreElt)");
        this.printer.println("{");
        this.printer.println("doSwitch(ecoreElt);");
        this.printer.println("}");
    }

    @objid ("1b37aec6-4583-11e0-b54c-00137279a832")
    private List<org.eclipse.uml2.uml.Class> getUML2Metaclasses() {
        Resource res = loadEcoreModel();
        if (res != null) {
            List<org.eclipse.uml2.uml.Class> classes = parseUml2Metamodel(res);
            return classes;
        }
        return null;
    }

    @objid ("1b37aecd-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
                
        String path;
        path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1b3a111f-4583-11e0-b54c-00137279a832")
    private void generateInnerRedefinedMethods() {
        List<org.eclipse.uml2.uml.Class> uml2Classes = getUML2Metaclasses();
        for (org.eclipse.uml2.uml.Class uml2Class : uml2Classes) {
            LOG.info(XMI.PLUGIN_ID, uml2Class.getName());
            visitMetaclass(uml2Class);
        }
    }

    @objid ("1b3a1121-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader(String metaclassName) {
        String docText = "/**\r\n* " + "Create the Modelio element from the given \"" + metaclassName + "\" Ecore element." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.print("public Object case");
        this.printer.print(metaclassName);
        this.printer.println("(");
        
        this.printer.print(metaclassName);
        this.printer.println(" ecoreElt)");
        this.printer.println("{");
    }

    @objid ("1b3a1124-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b3a1126-4583-11e0-b54c-00137279a832")
    private void generateEcoreEltCreation(String metaclassName) {
        if (this.objingFactory != null) {
            try {
                Method method = this.objingFactory.getMethod("create" + metaclassName, (Class[])null);
                if (method != null) {
                    this.printer.println("// TODO assert mapping is correct");
                    this.printer.println("objingElt = Modelio.getInstance().getModelingSession().getModel().create" + metaclassName + "();");
                }
            } catch (SecurityException e) {
                LOG.error(XMI.PLUGIN_ID, e);
            } catch (NoSuchMethodException e) {
                this.printer.println("// TODO map ecore to objing");
                LOG.error(XMI.PLUGIN_ID, "? create" + metaclassName + "() doesn't exist.");
            }
        }
    }

    @objid ("1b3c7376-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return null;");
    }

    @objid ("1b3c7378-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        
        PartialCreationImportVisitorGen generator = new PartialCreationImportVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/generation/PartialCreationImportVisitorGenerated.java");
        
        generator.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b3c737d-4583-11e0-b54c-00137279a832")
    private List<org.eclipse.uml2.uml.Class> parseUml2Metamodel(Resource res) {
        List<org.eclipse.uml2.uml.Class> uml2Classes = new ArrayList<org.eclipse.uml2.uml.Class>();
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                ModelImpl mdl = (ModelImpl) elem;
                for (Object umlElt : mdl.getPackagedElements()) {
                    if (umlElt instanceof org.eclipse.uml2.uml.Class && !((org.eclipse.uml2.uml.Class) umlElt).isAbstract()) {
                        uml2Classes.add((org.eclipse.uml2.uml.Class) umlElt);
                    }
                }
            }
        }
        // Sort classes by name:
        uml2Classes = sortByName(uml2Classes);
        return uml2Classes;
    }

    @objid ("1b3ed5d1-4583-11e0-b54c-00137279a832")
    private List<org.eclipse.uml2.uml.Class> sortByName(List<org.eclipse.uml2.uml.Class> uml2Classes) {
        int listSize = uml2Classes.size();
        boolean isDisordered = true;
        
        for (int i = 0; i < listSize && isDisordered; i++) {
            isDisordered = false;
            for (int j = 1; j < listSize - i; j++) {
                if (compareTo(uml2Classes.get(j), uml2Classes.get(j - 1)) < 0) {
                    org.eclipse.uml2.uml.Class tmpClass = uml2Classes.get(j - 1);
                    uml2Classes.set(j - 1, uml2Classes.get(j));
                    uml2Classes.set(j, tmpClass);
                    isDisordered = true;
                }
            }
        }
        return uml2Classes;
    }

    @objid ("1b41382a-4583-11e0-b54c-00137279a832")
    private int compareTo(org.eclipse.uml2.uml.Class class1, org.eclipse.uml2.uml.Class class2) {
        if (class1 != null && class2 != null)
            return class1.getName().compareTo(class2.getName());
        return -1;
    }

    @objid ("1b413832-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(org.eclipse.uml2.uml.Class metaclass) {
        String metaclassName = metaclass.getName();
        
        generateMethodHeader(metaclassName);
        
        generateEcoreEltCreation(metaclassName);
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

}
