/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1b56ad5a-4583-11e0-b54c-00137279a832")
public class Uml2InheritanceGen {
    @objid ("1b56ad5b-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b56ad5c-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b56ad5d-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;


    @objid ("1b56ad5e-4583-11e0-b54c-00137279a832")
    public Uml2InheritanceGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
    }

    @objid ("1b56ad61-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b5b7209-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b5b720e-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b5b7213-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b5dd462-4583-11e0-b54c-00137279a832")
    private void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b5dd464-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.util;\r\n");
        
        this.printer.println("import java.util.ArrayList;");
        this.printer.println("import java.util.List;\r\n");
        
        this.printer.println("import org.eclipse.uml2.uml.*;");
        this.printer.println("\r\n");
    }

    @objid ("1b5dd466-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generateMemberMethods();
        generateClassClosure();
    }

    @objid ("1b5dd468-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        String docText = "/**\r\n* " + "This class is used to parse EMF-UML2 metamodel along inheritance relations." + "\r\n*/";
        generateJavadoc(docText);
        
        this.printer.println("public class Uml2InheritanceGenerated");
        this.printer.println("{");
    }

    @objid ("1b5dd46a-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "List containing names of classes that have at least one concrete super class." + "\r\n*/";
        generateJavadoc(docText);
        
        this.printer.println("private List<String> concreteSuperClasses;");
        this.printer.println("\r\n");
    }

    @objid ("1b5dd46c-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        this.printer.println("public Uml2InheritanceGenerated() {");
        this.printer.println("concreteSuperClasses = new ArrayList<String>();");
        this.printer.println("initList();");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b5dd46e-4583-11e0-b54c-00137279a832")
    private void generateMemberMethods() {
        generateHasConcreteSuperMethod();
        generateInitializeMethod();
    }

    @objid ("1b6036bd-4583-11e0-b54c-00137279a832")
    private void generateHasConcreteSuperMethod() {
        String docText = "/**\r\n* " + "Returns true if the given class has at least one concrete super class;<BR>\r\n* " + "Returns false, else." + "\r\n*/";
        generateJavadoc(docText);
        
        this.printer.println("public boolean hasConcreteSuperClass(String metaclassName)");
        this.printer.println("{");
        this.printer.println("if (concreteSuperClasses.contains(metaclassName))");
        this.printer.println("return true;");
        this.printer.println("else");
        this.printer.println("return false;");
        this.printer.println("}");
        this.printer.println();
    }

    @objid ("1b6036bf-4583-11e0-b54c-00137279a832")
    private void generateInitializeMethod() {
        String docText = "/**\r\n* " + "Initializes the List of classes that have at least one concrete super class." + "\r\n*/";
        generateJavadoc(docText);
        
        this.printer.println("private void initList()");
        this.printer.println("{");
        generateInitializeMethodBody();
        this.printer.println("}");
    }

    @objid ("1b6036c1-4583-11e0-b54c-00137279a832")
    private void generateInitializeMethodBody() {
        Resource res = loadEcoreModel();
        if (res != null) {
            for (Class uml2Class : getUml2Metaclasses(res)) {
                visitMetaclass(uml2Class);
            }
        }
    }

    @objid ("1b629916-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        String path;
        path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1b62991a-4583-11e0-b54c-00137279a832")
    private void generateClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b62991c-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b62991e-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        Uml2InheritanceGen gen = new Uml2InheritanceGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/util/Uml2InheritanceGenerated.java");
        
        gen.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b64fb70-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2Metaclasses(Resource res) {
        List<Class> uml2Classes = new ArrayList<Class>();
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                ModelImpl mdl = (ModelImpl) elem;
                for (Object umlElt : mdl.getPackagedElements()) {
                    if (umlElt instanceof Class) {
                        uml2Classes.add((Class) umlElt);
                    }
                }
            }
        }
        // Sort classes by name:
        uml2Classes = sortByName(uml2Classes);
        return uml2Classes;
    }

    @objid ("1b64fb77-4583-11e0-b54c-00137279a832")
    private List<Class> sortByName(List<Class> uml2Classes) {
        int listSize = uml2Classes.size();
        boolean isDisordered = true;
        
        for (int i = 0; i < listSize && isDisordered; i++) {
            isDisordered = false;
            for (int j = 1; j < listSize - i; j++) {
                if (compareTo(uml2Classes.get(j), uml2Classes.get(j - 1)) < 0) {
                    Class tmpClass = uml2Classes.get(j - 1);
                    uml2Classes.set(j - 1, uml2Classes.get(j));
                    uml2Classes.set(j, tmpClass);
                    isDisordered = true;
                }
            }
        }
        return uml2Classes;
    }

    @objid ("1b675dca-4583-11e0-b54c-00137279a832")
    private int compareTo(Class class1, Class class2) {
        if (class1 != null && class2 != null)
            return class1.getName().toUpperCase().compareTo(class2.getName().toUpperCase());
        return -1;
    }

    @objid ("1b675dd0-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(Class uml2Class) {
        if (hasConcreteSuperClass(uml2Class)) {
            this.printer.println("concreteSuperClasses.add(");
            this.printer.print(uml2Class.getName());
            this.printer.println(".class.getSimpleName());");
        }
    }

    @objid ("1b675dd3-4583-11e0-b54c-00137279a832")
    private boolean hasConcreteSuperClass(Class uml2Class) {
        boolean hasConcreteSC = false;
        
        if (!"Element".equals(uml2Class.getName())) {
            for (Object superIj : uml2Class.getSuperClasses()) {
                Class superClass = (Class) superIj;
                if (!superClass.isAbstract()) {
                    hasConcreteSC = true;
                    break;
                } else if (hasConcreteSuperClass(superClass)) {
                    hasConcreteSC = true;
                    break;
                }
            }
        }
        return hasConcreteSC;
    }

}
