/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1b69c024-4583-11e0-b54c-00137279a832")
public class Uml2InsertableIFVisitorGen {
    @objid ("1b69c025-4583-11e0-b54c-00137279a832")
    private String metaclassName;

    @objid ("1b69c026-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b69c027-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b69c028-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;


    @objid ("1b69c029-4583-11e0-b54c-00137279a832")
    public Uml2InsertableIFVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
    }

    @objid ("1b69c02c-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b69c02e-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b69c033-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b6c2280-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b6c2285-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b6c2288-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.reverse;\r\n");
        
        this.printer.println("import java.util.ArrayList;");
        this.printer.println("import java.util.List;\r\n");
        this.printer.println("import org.eclipse.emf.ecore.EObject;");
        this.printer.println("import org.eclipse.uml2.uml.ActionExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.BehaviorExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.CombinedFragment;");
        this.printer.println("import org.eclipse.uml2.uml.ConsiderIgnoreFragment;");
        this.printer.println("import org.eclipse.uml2.uml.Continuation;");
        this.printer.println("import org.eclipse.uml2.uml.Element;");
        this.printer.println("import org.eclipse.uml2.uml.ExecutionOccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.ExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.Interaction;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionFragment;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionOperand;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionUse;");
        this.printer.println("import org.eclipse.uml2.uml.MessageOccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.OccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.PartDecomposition;");
        this.printer.println("import org.eclipse.uml2.uml.StateInvariant;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;\r\n");
    }

    @objid ("1b6c228a-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to get the elements that can be inserted in a sequence diagram with the JONI API." + "\r\n*/";
        generateJavadoc(docText);
        
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassHeader();
        generateInnerClassBody();
        generateInnerClassClosure();
        generateClassClosure();
    }

    @objid ("1b6c228c-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class Uml2InsertableIFVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1b6e84d8-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Represents the elements that can be inserted in a sequence diagram with the JONI API." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private List<Element> insertableElements = new ArrayList<Element>();\r\n");
        
        docText = "/**\r\n* " + "Visitor that will realize a switch on InteractionFragments." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private Uml2InsertableIFVisitorGenerated.InsertableIFSelector selector;\r\n");
    }

    @objid ("1b6e84da-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        String docText = "/**\r\n* " + "Constructor." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public Uml2InsertableIFVisitorGenerated() {");
        this.printer.println("selector = new InsertableIFSelector();");
        this.printer.println("}\r\n");
    }

    @objid ("1b6e84dc-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        generateGetInsertableElementsMethod();
    }

    @objid ("1b6e84de-4583-11e0-b54c-00137279a832")
    private void generateGetInsertableElementsMethod() {
        String docText = "/**\r\n* " + "Returns the list of elements that can be inserted in a sequence diagram with the JONI API." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public List<Element> getInsertableElements(InteractionFragment theIF) {");
        this.printer.println("insertableElements.clear();");
        this.printer.println("selector.doSwitch(theIF);");
        this.printer.println("return insertableElements;");
        this.printer.println("}\r\n");
    }

    @objid ("1b6e84e0-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        String docText = "/**\r\n* " + "Visitor that will realize a switch on InteractionFragments.<BR>\r\n" + "Each method \"caseX\" shall not return null, as it would call the \"super.caseX\" method. As a consequence, all methods return 42, as it is the Ultimate Answer to Life, the Universe, and Everything... (The Hitchhiker's Guide to the Galaxy, Douglas Adams)" + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private class InsertableIFSelector");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1b6e84e2-4583-11e0-b54c-00137279a832")
    private void generateInnerClassBody() {
        generateDoSwitchMethod();
        Resource res = loadEcoreModel();
        if (res != null) {
            for (Class uml2Class : getUml2IF(res)) {
                LOG.info(XMI.PLUGIN_ID, "metaclass = " + uml2Class.getName());
                visitMetaclass(uml2Class);
            }
        }
    }

    @objid ("1b70e732-4583-11e0-b54c-00137279a832")
    private void generateDoSwitchMethod() {
        String docText = "/**\r\n" + "Calls caseXXX for each class of the model until one returns a non null" + " result; it yields that result.\r\n" + "@param inputElement the target of the switch.\r\n" + "@return the first non-null result returned by a caseXXX call.\r\n" + "*/";
        generateJavadoc(docText);
        
        this.printer.println("@Override");
        this.printer.println("public Object doSwitch(");
        this.printer.println("EObject inputElement)");
        this.printer.println("{");
        this.printer.println("if (inputElement != null)");
        this.printer.println("return super.doSwitch(inputElement);");
        this.printer.println("else");
        this.printer.println("return null;");
        this.printer.println("}\r\n");
    }

    @objid ("1b70e734-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        String path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/uml2 metamodel version XMI/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1b70e738-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader() {
        String docText = "/**\r\n* " + "Visits the InteractionFragment \"" + this.metaclassName + "\" and its relations to other elements to store those that can be inserted in a sequence diagram with the JONI API." + ".<BR>\r\n" + "@return a non null value in order not to call the super class (see class definition for explanation of the \"42\" value)." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.println("public Object case" + this.metaclassName + "(" + this.metaclassName + " fragment" + ") {");
    }

    @objid ("1b70e73a-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return 42;");
    }

    @objid ("1b70e73c-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
    }

    @objid ("1b73498c-4583-11e0-b54c-00137279a832")
    private void generateInnerClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b73498e-4583-11e0-b54c-00137279a832")
    private void generateClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b734990-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b734992-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        Uml2InsertableIFVisitorGen gen = new Uml2InsertableIFVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/reverse/Uml2InsertableIFVisitorGenerated.java");
        
        gen.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b734997-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2IF(Resource res) {
        List<Class> uml2IFList = new ArrayList<Class>();
        
        List<Class> uml2Metaclasses = getUml2Metaclasses(res);
        
        Class uml2IFMetaclass = null;
        for (Class uml2Class : uml2Metaclasses) {
            if ("InteractionFragment".equals(((NamedElement) uml2Class).getName())) {
                uml2IFMetaclass = uml2Class;
                break;
            }
        }
        
        for (Class uml2Class : uml2Metaclasses) {
            List<Classifier> generals = getRecursiveGenerals(uml2Class, new ArrayList<Classifier>());
            if (generals.contains(uml2IFMetaclass))
                uml2IFList.add(uml2Class);
        }
        return uml2IFList;
    }

    @objid ("1b75abea-4583-11e0-b54c-00137279a832")
    private List<Classifier> getRecursiveGenerals(Classifier uml2Class, List<Classifier> superClasses) {
        EList<Classifier> generals = uml2Class.getGenerals();
        
        if (generals.size() > 0) {
            superClasses.addAll(generals);
            for (Classifier superClass : generals) {
                getRecursiveGenerals(superClass, superClasses);
            }
        }
        return superClasses;
    }

    @objid ("1b75abf4-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2Metaclasses(Resource res) {
        List<Class> uml2Classes = new ArrayList<Class>();
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                ModelImpl mdl = (ModelImpl) elem;
                for (Object umlElt : mdl.getPackagedElements()) {
                    if (umlElt instanceof Class) {
                        // && !((Class)umlElt).isAbstract()) {
                        uml2Classes.add((Class) umlElt);
                    }
                }
            }
        }
        // Sort classes by name:
        uml2Classes = sortByName(uml2Classes);
        return uml2Classes;
    }

    @objid ("1b780e45-4583-11e0-b54c-00137279a832")
    private List<Class> sortByName(List<Class> uml2Classes) {
        int listSize = uml2Classes.size();
        boolean isDisordered = true;
        
        for (int i = 0; i < listSize && isDisordered; i++) {
            isDisordered = false;
            for (int j = 1; j < listSize - i; j++) {
                if (compareTo(uml2Classes.get(j), uml2Classes.get(j - 1)) < 0) {
                    Class tmpClass = uml2Classes.get(j - 1);
                    uml2Classes.set(j - 1, uml2Classes.get(j));
                    uml2Classes.set(j, tmpClass);
                    isDisordered = true;
                }
            }
        }
        return uml2Classes;
    }

    @objid ("1b780e4e-4583-11e0-b54c-00137279a832")
    private int compareTo(Class class1, Class class2) {
        if (class1 != null && class2 != null)
            return class1.getName().toUpperCase().compareTo(class2.getName().toUpperCase());
        return -1;
    }

    @objid ("1b7a709f-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(Class metaclass) {
        this.metaclassName = metaclass.getName();
        generateMethodHeader();
        
        generateMethodBody();
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1b7a70a2-4583-11e0-b54c-00137279a832")
    private void generateMethodBody() {
        // if ("Interaction".equals(metaclassName))
        // printer.print("//");
        // printer.println("insertableElements = new " + metaclassName
        // + "ImportMapper(fragment);");
    }

}
