/*
 * Copyright 2011 Modeliosoft
 *
 * This file is part of Modelio.
 *
 * Modelio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Modelio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Modelio.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */  
                                    
package com.modeliosoft.modelio.xmi.util.generators;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.modeliosoft.modelio.core.log.LOG;
import com.modeliosoft.modelio.javadesigner.annotations.objid;
import com.modeliosoft.modelio.xmi.plugin.XMI;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.internal.impl.ModelImpl;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.eclipse.uml2.uml.resource.UMLResource;

@objid ("1b7a70a5-4583-11e0-b54c-00137279a832")
public class Uml2InteractionFragmentVisitorGen {
    @objid ("1b7a70a6-4583-11e0-b54c-00137279a832")
    private String metaclassName;

    @objid ("1b7a70a7-4583-11e0-b54c-00137279a832")
    private File file = null;

    @objid ("1b7cd2f4-4583-11e0-b54c-00137279a832")
    private BufferedWriter buffer = null;

    @objid ("1b7cd2f5-4583-11e0-b54c-00137279a832")
    private PrintWriter printer = null;


    @objid ("1b7cd2f6-4583-11e0-b54c-00137279a832")
    public Uml2InteractionFragmentVisitorGen(String path) {
        this.file = createFile(path);
        this.buffer = createBuffer(this.file);
        this.printer = createPrinter(this.buffer);
    }

    @objid ("1b7cd2f9-4583-11e0-b54c-00137279a832")
    public void generate() {
        generateImports();
        generateClassCode();
        closeFile();
    }

    @objid ("1b7cd2fb-4583-11e0-b54c-00137279a832")
    private File createFile(String path) {
        return new File(path);
    }

    @objid ("1b7cd300-4583-11e0-b54c-00137279a832")
    private BufferedWriter createBuffer(File theFile) {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(theFile));
        } catch (IOException e) {
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return bw;
    }

    @objid ("1b7f354e-4583-11e0-b54c-00137279a832")
    private PrintWriter createPrinter(BufferedWriter theBuffer) {
        PrintWriter print = null;
        if (theBuffer != null)
            print = new PrintWriter(theBuffer);
        return print;
    }

    @objid ("1b7f3553-4583-11e0-b54c-00137279a832")
    private void generateJavadoc(String docText) {
        this.printer.println(docText);
    }

    @objid ("1b7f3556-4583-11e0-b54c-00137279a832")
    private void generateImports() {
        this.printer.println("package com.modeliosoft.modelio.xmi.reverse;\r\n");
        
        this.printer.println("import org.eclipse.emf.ecore.EObject;");
        this.printer.println("import org.eclipse.uml2.uml.ActionExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.BehaviorExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.CombinedFragment;");
        this.printer.println("import org.eclipse.uml2.uml.ConsiderIgnoreFragment;");
        this.printer.println("import org.eclipse.uml2.uml.Continuation;");
        this.printer.println("import org.eclipse.uml2.uml.ExecutionOccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.ExecutionSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.Interaction;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionFragment;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionOperand;");
        this.printer.println("import org.eclipse.uml2.uml.InteractionUse;");
        this.printer.println("import org.eclipse.uml2.uml.MessageOccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.OccurrenceSpecification;");
        this.printer.println("import org.eclipse.uml2.uml.PartDecomposition;");
        this.printer.println("import org.eclipse.uml2.uml.StateInvariant;");
        this.printer.println("import org.eclipse.uml2.uml.util.UMLSwitch;\r\n");
        
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.ActionExecutionSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.BehaviorExecutionSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.CombinedFragmentImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.ConsiderIgnoreFragmentImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.ContinuationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.ExecutionOccurrenceSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.ExecutionSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.InteractionFragmentImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.InteractionOperandImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.InteractionUseImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.MessageOccurrenceSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.OccurrenceSpecificationImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.PartDecompositionImportMapper;");
        this.printer.println("import com.modeliosoft.modelio.xmi.model.ecore.sequenceOld.StateInvariantImportMapper;\r\n");
    }

    @objid ("1b7f3558-4583-11e0-b54c-00137279a832")
    private void generateClassCode() {
        String docText = "/**\r\n* " + "This class is used to instanciate a Class that will do the mapping of an InteractionFragment from Emf-Uml2 to Modelio." + "\r\n*/";
        generateJavadoc(docText);
        
        generateClassHeader();
        generateMemberAttributes();
        generateConstructor();
        generatePublicMethods();
        generateInnerClassHeader();
        generateInnerClassBody();
        generateInnerClassClosure();
        generateClassClosure();
    }

    @objid ("1b7f355a-4583-11e0-b54c-00137279a832")
    private void generateClassHeader() {
        this.printer.println("public class Uml2InteractionFragmentVisitorGenerated");
        this.printer.println("{");
    }

    @objid ("1b7f355c-4583-11e0-b54c-00137279a832")
    private void generateMemberAttributes() {
        String docText = "/**\r\n* " + "Represents the composition owner that will be returned." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private InteractionFragmentImportMapper mapper;\r\n");
        
        docText = "/**\r\n* " + "Visitor that will realize a switch on InteractionFragments." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private Uml2InteractionFragmentVisitorGenerated.IFImportMapperSelector selector;\r\n");
    }

    @objid ("1b8197a8-4583-11e0-b54c-00137279a832")
    private void generateConstructor() {
        String docText = "/**\r\n* " + "Constructor." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public Uml2InteractionFragmentVisitorGenerated() {");
        this.printer.println("selector = new IFImportMapperSelector();");
        this.printer.println("}\r\n");
    }

    @objid ("1b8197aa-4583-11e0-b54c-00137279a832")
    private void generatePublicMethods() {
        generateGetInteractionFragmentImportMapperMethod();
    }

    @objid ("1b8197ac-4583-11e0-b54c-00137279a832")
    private void generateGetInteractionFragmentImportMapperMethod() {
        String docText = "/**\r\n* " + "Returns an instance that will realize the mapping of an InteractionFragment from Emf-Uml2 to Modelio." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("public InteractionFragmentImportMapper getIFImportMapper(InteractionFragment theIF) {");
        this.printer.println("mapper = null;");
        this.printer.println("selector.doSwitch(theIF);");
        this.printer.println("return mapper;");
        this.printer.println("}\r\n");
    }

    @objid ("1b8197ae-4583-11e0-b54c-00137279a832")
    private void generateInnerClassHeader() {
        String docText = "/**\r\n* " + "Visitor that will realize a switch on InteractionFragments.<BR>\r\n" + "Each method \"caseX\" shall not return null, as it would call the \"super.caseX\" method. As a consequence, all methods return 42, as it is the Ultimate Answer to Life, the Universe, and Everything... (The Hitchhiker's Guide to the Galaxy, Douglas Adams)" + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("private class IFImportMapperSelector");
        this.printer.println("extends UMLSwitch");
        this.printer.println("{");
    }

    @objid ("1b8197b0-4583-11e0-b54c-00137279a832")
    private void generateInnerClassBody() {
        generateDoSwitchMethod();
        Resource res = loadEcoreModel();
        if (res != null) {
            for (Class uml2Class : getUml2IF(res)) {
                LOG.info(XMI.PLUGIN_ID, "metaclass = " + uml2Class.getName());
                visitMetaclass(uml2Class);
            }
        }
    }

    @objid ("1b8197b2-4583-11e0-b54c-00137279a832")
    private void generateDoSwitchMethod() {
        String docText = "/**\r\n" + "Calls caseXXX for each class of the model until one returns a non null" + " result; it yields that result.\r\n" + "@param inputElement the target of the switch.\r\n" + "@return the first non-null result returned by a caseXXX call.\r\n" + "*/";
        generateJavadoc(docText);
        
        this.printer.println("@Override");
        this.printer.println("public Object doSwitch(");
        this.printer.println("EObject inputElement)");
        this.printer.println("{");
        this.printer.println("if (inputElement != null)");
        this.printer.println("return super.doSwitch(inputElement);");
        this.printer.println("else");
        this.printer.println("return null;");
        this.printer.println("}\r\n");
    }

    @objid ("1b83fa02-4583-11e0-b54c-00137279a832")
    private Resource loadEcoreModel() {
        ResourceSet resourceSet2 = new ResourceSetImpl();
        
        resourceSet2.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
        Map<URI, URI> uriMap = resourceSet2.getURIConverter().getURIMap();
        
        String path;
        path = "C:/eclipse new/workspace/XMI_prototype/lib/org.eclipse.uml2.uml.resources_2.0.0.v200606221411.jar!/";
        URI uri = URI.createURI("jar:file:/" + path);
        
        uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), uri.appendSegment("libraries").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), uri.appendSegment("metamodels").appendSegment(""));
        uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), uri.appendSegment("profiles").appendSegment(""));
        
        resourceSet2.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new UMLResourceFactoryImpl());
        
        // Get the URI of the model file.
        File modelFile = new File("C:/uml2 metamodel version XMI/UML.metamodel.uml");
        
        URI fileURI2 = URI.createFileURI(modelFile.getAbsolutePath());
        
        try {
            return resourceSet2.getResource(fileURI2, true);
        } catch (Exception e) {
            LOG.error(XMI.PLUGIN_ID, "invalidModel");
            LOG.error(XMI.PLUGIN_ID, e);
        }
        return null;
    }

    @objid ("1b83fa06-4583-11e0-b54c-00137279a832")
    private void generateMethodHeader() {
        String docText = "/**\r\n* " + "Instanciates the class that will realize the mapping for the InteractionFragment: " + this.metaclassName + ".<BR>\r\n" + "@return a non null value in order not to call the super class (see class definition for explanation of the \"42\" value)." + "\r\n*/";
        generateJavadoc(docText);
        this.printer.println("@Override");
        this.printer.println("public Object case" + this.metaclassName + "(" + this.metaclassName + " fragment" + ") {");
    }

    @objid ("1b83fa08-4583-11e0-b54c-00137279a832")
    private void generateMethodReturnValue() {
        this.printer.println("return 42;");
    }

    @objid ("1b83fa0a-4583-11e0-b54c-00137279a832")
    private void generateMethodFooter() {
        this.printer.println("}");
    }

    @objid ("1b83fa0c-4583-11e0-b54c-00137279a832")
    private void generateInnerClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b83fa0e-4583-11e0-b54c-00137279a832")
    private void generateClassClosure() {
        this.printer.println("}");
    }

    @objid ("1b865c5c-4583-11e0-b54c-00137279a832")
    private void closeFile() {
        this.printer.flush();
        this.printer.close();
    }

    @objid ("1b865c5e-4583-11e0-b54c-00137279a832")
    public static void main(String[] args) {
        LOG.info(XMI.PLUGIN_ID, "BEGIN");
        Uml2InteractionFragmentVisitorGen gen = new Uml2InteractionFragmentVisitorGen("C:/eclipse new/workspace/XMI_prototype/src/com/softeam/Modelio/xmi/reverse/Uml2InteractionFragmentVisitorGenerated.java");
        
        gen.generate();
        
        LOG.info(XMI.PLUGIN_ID, "END");
    }

    @objid ("1b865c63-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2IF(Resource res) {
        List<Class> uml2IFList = new ArrayList<Class>();
        
        List<Class> uml2Metaclasses = getUml2Metaclasses(res);
        
        Class uml2IFMetaclass = null;
        for (Class uml2Class : uml2Metaclasses) {
            if ("InteractionFragment".equals(((NamedElement) uml2Class).getName())) {
                uml2IFMetaclass = uml2Class;
                break;
            }
        }
        
        for (Class uml2Class : uml2Metaclasses) {
            List<Classifier> generals = getRecursiveGenerals(uml2Class, new ArrayList<Classifier>());
            if (generals.contains(uml2IFMetaclass))
                uml2IFList.add(uml2Class);
        }
        return uml2IFList;
    }

    @objid ("1b88beb6-4583-11e0-b54c-00137279a832")
    private List<Classifier> getRecursiveGenerals(Classifier uml2Class, List<Classifier> superClasses) {
        List<Classifier> generals = uml2Class.getGenerals();
        
        if (generals.size() > 0) {
            superClasses.addAll(generals);
            for (Classifier superClass : generals) {
                getRecursiveGenerals(superClass, superClasses);
            }
        }
        return superClasses;
    }

    @objid ("1b88bec0-4583-11e0-b54c-00137279a832")
    private List<Class> getUml2Metaclasses(Resource res) {
        List<Class> uml2Classes = new ArrayList<Class>();
        for (Object elem : res.getContents()) {
            if (elem instanceof ModelImpl) {
                ModelImpl mdl = (ModelImpl) elem;
                for (Object umlElt : mdl.getPackagedElements()) {
                    if (umlElt instanceof Class) {
                        // && !((Class)umlElt).isAbstract()) {
                        uml2Classes.add((Class) umlElt);
                    }
                }
            }
        }
        // Sort classes by name:
        uml2Classes = sortByName(uml2Classes);
        return uml2Classes;
    }

    @objid ("1b8b2111-4583-11e0-b54c-00137279a832")
    private List<Class> sortByName(List<Class> uml2Classes) {
        int listSize = uml2Classes.size();
        boolean isDisordered = true;
        
        for (int i = 0; i < listSize && isDisordered; i++) {
            isDisordered = false;
            for (int j = 1; j < listSize - i; j++) {
                if (compareTo(uml2Classes.get(j), uml2Classes.get(j - 1)) < 0) {
                    Class tmpClass = uml2Classes.get(j - 1);
                    uml2Classes.set(j - 1, uml2Classes.get(j));
                    uml2Classes.set(j, tmpClass);
                    isDisordered = true;
                }
            }
        }
        return uml2Classes;
    }

    @objid ("1b8b211a-4583-11e0-b54c-00137279a832")
    private int compareTo(Class class1, Class class2) {
        if (class1 != null && class2 != null)
            return class1.getName().toUpperCase().compareTo(class2.getName().toUpperCase());
        return -1;
    }

    @objid ("1b8d836a-4583-11e0-b54c-00137279a832")
    private void visitMetaclass(Class metaclass) {
        this.metaclassName = metaclass.getName();
        generateMethodHeader();
        
        generateMethodBody();
        
        generateMethodReturnValue();
        generateMethodFooter();
    }

    @objid ("1b8d836d-4583-11e0-b54c-00137279a832")
    private void generateMethodBody() {
        if ("Interaction".equals(this.metaclassName))
            this.printer.print("//");
        this.printer.println("mapper = new " + this.metaclassName + "ImportMapper(fragment);");
    }

}
